# $Id: suffix.t,v 1.7 1993/10/21 12:50:02 kon Exp $
か #N2T30*1187 化
てき #N2T17*1187 的
さん #JNSUC*1139 さん
てん #SUC*809 点
せい #SUC*693 性
かた #D2T35*656 方
しゃ #SUC*656 者
ちゅう #SUC*427 中
じょう #SUC*398 上
ない #SUC*384 内
けん #JNSUC*357 研
じ #SUC*357 時
ご #SUC*344 後
かい #SUC*331 会
し #JNSUC*331 氏
すう #SUC*331 数
せん #SUC*331 線
ほう #SUC*331 法
しょ #SUC*318 書
けい #SUC*305 系
めい #SUC*305 名
ぶ #SUC*281 部
しき #SUC*258 式
くん #JNSUC*236 君
し #CNSUC1*225 市
ばん #SUC*214 版
がた #SUC*194 型
やす #D2KY*184 やす
ら #JNSUC*184 ら
さま #JNSUC*166 様
たち #JNSUC*166 たち
だい #SUC*157 代
ちゃん #JNSUC*157 ちゃん
ちょう #SUC*148 長
など #SUC*148 等
ー #SUC*131 ー
そう #SUC*131 層
かん #CNSUC1*101 間
さん #CNSUC1*87 産
けん #SUC*80 権
じゅん #SUC*80 順
やく #SUC*80 役
しだい #D2T35*74 次第
しょ #SUC*74 所
び #SUC*74 日
めん #SUC*74 面
き #SUC*68 機
けい #SUC*68 形
しつ #SUC*68 室
えき #CNSUC1*57 駅
よう #SUC*57 用
がわ #CNSUC1*52 川
じん #SUC*52 人
じん #CNSUC1*52 人
ずみ #SUC*52 済み
たち #JNSUC*52 達
へん #JNSUC*47 編
むら #CNSUC1*47 村
どおり #SUC*37 通り
は #JNSUC*33 派
ぐみ #JNSUC*29 組
ちょ #JNSUC*29 著
そん #CNSUC1*25 村
ふう #CNSUC1*25 風
がき #N2T30*22 書き
がわ #CNSUC1*22 河
せい #CNSUC1*22 製
じゅう #CNSUC1*19 中
たく #JNSUC*19 宅
ちゃく #CNSUC1*19 着
はし #CNSUC1*16 橋
ばし #CNSUC1*16 橋
ちく #CNSUC1*13 地区
ちほう #CNSUC1*13 地方
さん #CNSUC1*10 山
ざん #CNSUC1*10 山
どの #JNSUC*6 殿
りゅう #JNSUC*6 流
きた #CNSUC1*4 北
