あ #PRE*131 亜
あい #PRE*10 相
あお #PRE*4 青
あか #PRE*10 赤
あく #PRE*10 悪
あざ #CNPRE*2 字
あま #PRE*1 アマ #PRE 甘
あら #PRE*4 粗
いく #PRE 幾
いっさく #PRE*1 一昨
う #PRE*2 右
うえ #PRE*2 上
うち #PRE*1 内
えせ #PRE 似非
お #PRE*114 @ #PRE*4 御 #PRE @
おーと #PRE*2 オート
おお #PRE*80 大
おおあざ #CNPRE*3 大字
おん #PRE*3 御
か #PRE 過
かい #PRE 快 怪
かく #PRE*231 各
かた #PRE 片
かみ #CNPRE 上
かり #PRE 仮
き #PRE 既 貴 生
きた #CNPRE*4 北
きゅう #PRE 急 旧
きょう #PRE 強
けい #PRE*1 軽
げん #PRE 原 現
こ #PRE 故 小
こあざ #CNPRE 小字
こう #PRE 高
ご #PRE @ 御 誤
ごーるでん #PRE*1 ゴールデン
さ #PRE*1 左
さい #PRE 再 最
さいばー #PRE*1 サイバー
さぶ #PRE*52 サブ
した #PRE*3 下
しも #CNPRE*1 下
しゅ #PRE 主
しょ #PRE 初 諸
しょう #PRE 小 省
しん #PRE 新
じ #PRE 次
じっ #PRE 実
じゃく #PRE 弱
じゅう #PRE 重
じゅん #PRE 准 準 純
じょ #PRE 助
じょう #PRE 上
す #PRE*62 素
すーぱ #PRE*4 スーパ
すーぱー #PRE*22 スーパー #PRE*1 スーパー
せい #PRE 正
ぜん #PRE 前 全
そう #PRE*42 総
た #PRE 他 多
たい #PRE*1 対
だーい #PRE*1 @
だい #NNPRE 第 #PRE 大
ちゅう #PRE 中 駐
ちょう #PRE 超
てい #PRE 低
とう #PRE 当
なか #CNPRE 中
にし #CNPRE*4 西
にゅー #PRE*6 ニュー
のん #PRE*3 ノン
はーふ #PRE*2 ハーフ
はい #PRE ハイ
はいぱー #PRE*6 ハイパー
はん #PRE 半 反
ばいお #PRE*3 バイオ
ひ #PRE*148 非 #PRE 被
ひがし #CNPRE*1 東
ひと #PRE @
ひら #PRE*16 平
び #PRE*2 微
ふ #PRE*139 不
ふぉあ #PRE*2 フォア
ふく #PRE*1 複 #PRE 副
ふな #PRE*1 舟 船
ふる #PRE*4 フル
ぶ #PRE 不
ぷり #PRE*22 プリ
ほん #PRE 本
ぼう #PRE*52 某
ぼつ #PRE 没
ぽり #PRE*1 ポリ
まい #NNPRE 毎 #PRE マイ
まえ #PRE 前
まるち #PRE*57 マルチ #PRE*1 マルチ
み #PRE*68 未
みなみ #CNPRE*1 南
みに #PRE*13 ミニ
む #PRE*62 無
めい #PRE 名 銘
めた #PRE*22 メタ
もと #PRE*1 元 #PRE 本
やく #NNPRE*101 約
ゆに #PRE*1 ユニ
よくよく #PRE*4 翌々
りょう #PRE 両
ろー #PRE*10 ロー
わか #PRE 若
