# Copyright 1994 NEC Corporation, Tokyo, Japan.
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without
# fee, provided that the above copyright notice appear in all copies
# and that both that copyright notice and this permission notice
# appear in supporting documentation, and that the name of NEC
# Corporation not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.  NEC Corporation makes no representations about the
# suitability of this software for any purpose.  It is provided "as
# is" without express or implied warranty.
#
# NEC CORPORATION DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN 
# NO EVENT SHALL NEC CORPORATION BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR 
# OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
# PERFORMANCE OF THIS SOFTWARE. 
#
# $Id: chimei.t,v 1.5 1994/01/27 11:07:17 kuma Exp $
あーかんそー #CN アーカンソー
あいおい #CN 相生
あいおいし #CN 相生市
あいおわ #CN アイオワ
あいすらんど #CN アイスランド
あいち #CN 愛知
あいちけん #CN 愛知県
あいづわかまつ #CN 会津若松
あいづわかまつし #CN 会津若松市
あいるらんど #CN アイルランド
あおもり #CN 青森
あおもりけん #CN 青森県
あおもりし #CN 青森市
あかいしだけ #CN 赤石岳
あかさか #CN 赤坂
あかし #CN 明石
あかしし #CN 明石市
あかばね #CN 赤羽
あかばねせん #CN 赤羽線
あかびら #CN 赤平
あかびらし #CN 赤平市
あかんこ #CN 阿寒湖
あき #CN 安芸
あきがわ #CN 秋川
あきがわし #CN 秋川市
あきく #CN 安芸区
あきし #CN 安芸市
あきしま #CN 昭島
あきしまし #CN 昭島市
あきた #CN 秋田
あきたけん #CN 秋田県
あきたし #CN 秋田市
あきつ #CN 秋津
あきはばら #CN 秋葉原
あきよしだい #CN 秋吉台
あくね #CN 阿久根
あくねし #CN 阿久根市
あくら #CN アクラ
あけぼのばし #CN 曙橋
あげお #CN 上尾
あげおし #CN 上尾市
あこう #CN 赤穂
あこうし #CN 赤穂市
あさおく #CN 麻生区
あさか #CN 朝霞
あさかし #CN 朝霞市
あさがや #CN 阿佐ケ谷
あさきたく #CN 安佐北区
あさくさ #CN 浅草
あさくさばし #CN 浅草橋
あさひ #CN 旭
あさひかわ #CN 旭川
あさひかわし #CN 旭川市
あさひく #CN 旭区
あさひし #CN 旭市
あさひちょう #CN 旭町
あさみなみく #CN 安佐南区
あざぶ #CN 麻布
あしかが #CN 足利
あしかがし #CN 足利市
あしのこ #CN 芦ノ湖
あしはばーど #CN アシハバード
あしべつ #CN 芦別
あしべつし #CN 芦別市
あしや #CN 芦屋
あしやし #CN 芦屋市
あじあ #CN アジア 亜細亜
あじすあべば #CN アジスアベバ
あすまら #CN アスマラ
あすんしおん #CN アスンシオン
あずまばし #CN 吾妻橋
あぜるばいじゃん #CN アゼルバイジャン
あそざん #CN 阿蘇山
あたご #CN 愛宕
あたみ #CN 熱海
あたみし #CN 熱海市
あだたら #CN 安達太良
あだちく #CN 足立区
あつぎ #CN 厚木
あつぎし #CN 厚木市
あつたく #CN 熱田区
あてね #CN アテネ
あでん #CN アデン
あなん #CN 阿南
あなんし #CN 阿南市
あばしり #CN 網走
あばしりし #CN 網走市
あぱらちあ #CN アパラチア
あびこ #CN 我孫子
あびこし #CN 我孫子市
あびじゃん #CN アビジャン
あぴあ #CN アピア
あふがにすたん #CN アフガニスタン
あふりか #CN アフリカ
あぶくまがわ #CN 阿武隈川
あぶだび #CN アブダビ
あぶらつぼ #CN 油壷
あべのく #CN 阿倍野区
あべのすじ #CN 阿倍野筋
あまがさき #CN 尼崎
あまがさきし #CN 尼崎市
あまぎ #CN 甘木
あまぎし #CN 甘木市
あまみ #CN 奄美
あむすてるだむ #CN アムステルダム
あめりか #CN アメリカ
あやせ #CN 綾瀬
あやせし #CN 綾瀬市
あやべ #CN 綾部
あやべし #CN 綾部市
あらい #CN 新井
あらいし #CN 新井市
あらお #CN 荒尾
あらおし #CN 荒尾市
あらかわく #CN 荒川区
あらしやま #CN 嵐山
あらすか #CN アラスカ
あらびあ #CN アラビア
あらぶ #CN アラブ
ありあけ #CN 有明
ありだ #CN 有田
ありだし #CN 有田市
ありゅーしゃん #CN アリューシャン
あるじぇ #CN アルジェ
あるじぇりあ #CN アルジェリア
あるぜんちん #CN アルゼンチン
あるばにあ #CN アルバニア
あるぷす #CN アルプス
あるまあた #CN アルマアタ
あるめにあ #CN アルメニア
あれきさんどりあ #CN アレキサンドリア
あれくさんどろふすく #CN アレクサンドロフスク
あわ #CN 阿波
あわじ #CN 淡路
あわじしま #CN 淡路島
あんから #CN アンカラ
あんかれじ #CN アンカレジ
あんごら #CN アンゴラ
あんじょう #CN 安城
あんじょうし #CN 安城市
あんたななりば #CN アンタナリボ
あんどら #CN アンドラ
あんどららべら #CN アンドララベラ
あんなか #CN 安中
あんなかし #CN 安中市
あんまん #CN アンマン
いいだ #CN 飯田
いいだし #CN 飯田市
いいだばし #CN 飯田橋
いいづか #CN 飯塚
いいづかし #CN 飯塚市
いいもりやま #CN 飯盛山
いいやま #CN 飯山
いいやまし #CN 飯山市
いえめん #CN イエメン
いき #CN 壱岐
いぎりす #CN イギリス
いくのく #CN 生野区
いけがみ #CN 池上
いけじり #CN 池尻
いけじりおおはし #CN 池尻大橋
いけだ #CN 池田
いけだし #CN 池田市
いけのはた #CN 池ノ端
いけぶくろ #CN 池袋
いけぶくろせん #CN 池袋線
いこま #CN 生駒
いこまし #CN 生駒市
いさはや #CN 諌早
いさはやし #CN 諌早市
いしおか #CN 石岡
いしおかし #CN 石岡市
いしかり #CN 石狩
いしかわ #CN 石川
いしかわけん #CN 石川県
いしかわし #CN 石川市
いしかわちょう #CN 石川町
いしがき #CN 石垣
いしがきし #CN 石垣市
いしがきじま #CN 石垣島
いしのまき #CN 石巻
いしのまきし #CN 石巻市
いすたんぶーる #CN イスタンブール
いすらえる #CN イスラエル
いすらまばーど #CN イスラマバード
いず #CN 伊豆
いずみ #CN 出水 泉 和泉
いずみおおつ #CN 泉大津
いずみおおつし #CN 泉大津市
いずみさの #CN 泉佐野
いずみさのし #CN 泉佐野市
いずみし #CN 出水市 泉市 和泉市
いずも #CN 出雲
いずもし #CN 出雲市
いせ #CN 伊勢
いせき #CN 遺跡
いせさき #CN 伊勢崎
いせさきし #CN 伊勢崎市
いせざき #CN 伊勢佐木 伊勢崎
いせし #CN 伊勢市
いせはら #CN 伊勢原
いせはらし #CN 伊勢原市
いそご #CN 磯子
いそごく #CN 磯子区
いたばしく #CN 板橋区
いたみ #CN 伊丹
いたみし #CN 伊丹市
いたりあ #CN イタリア
いちかわ #CN 市川
いちかわし #CN 市川市
いちがや #CN 市ケ谷
いちのせき #CN 一関
いちのせきし #CN 一関市
いちのみや #CN 一宮
いちのみやし #CN 一宮市
いちはら #CN 市原
いちはらし #CN 市原市
いちばんがい #CN 一番街
いつかいちせん #CN 五日市線
いつくしま #CN 厳島
いといがわ #CN 糸魚川
いといがわし #CN 糸魚川市
いとう #CN 伊東
いとうし #CN 伊東市
いとまん #CN 糸満
いとまんし #CN 糸満市
いな #CN 伊那
いなぎ #CN 稲城
いなぎし #CN 稲城市
いなざわ #CN 稲沢
いなざわし #CN 稲沢市
いなし #CN 伊那市
いなわしろこ #CN 猪苗代湖
いぬやま #CN 犬山
いぬやまし #CN 犬山市
いばら #CN 井原
いばらき #CN 茨城 茨木
いばらきけん #CN 茨城県
いばらきし #CN 茨木市
いばらし #CN 井原市
いぶすき #CN 指宿
いぶすきし #CN 指宿市
いまいち #CN 今市
いまいちし #CN 今市市
いまざと #CN 今里
いまでがわ #CN 今出川
いまばし #CN 今橋
いまばり #CN 今治
いまばりし #CN 今治市
いまり #CN 伊万里
いまりし #CN 伊万里市
いよ #CN 伊予
いよし #CN 伊予市
いよみしま #CN 伊予三島
いよみしまし #CN 伊予三島市
いらく #CN イラク
いらん #CN イラン
いりのい #CN イリノイ
いりふね #CN 入船
いるま #CN 入間
いるまし #CN 入間市
いわい #CN 岩井
いわいし #CN 岩井市
いわきし #CN いわき市
いわくに #CN 岩国
いわくにし #CN 岩国市
いわくら #CN 岩倉
いわくらし #CN 岩倉市
いわた #CN 磐田
いわたし #CN 磐田市
いわつか #CN 岩塚
いわつき #CN 岩槻
いわつきし #CN 岩槻市
いわて #CN 岩手
いわてけん #CN 岩手県
いわぬま #CN 岩沼
いわぬまし #CN 岩沼市
いわみざわ #CN 岩見沢
いわみざわし #CN 岩見沢市
いわもとちょう #CN 岩本町
いんじあな #CN インジアナ
いんど #CN インド 印度
いんどねしあ #CN インドネシア
いんのしま #CN 因島
いんのしまし #CN 因島市
いんば #CN 印旛
うぃーん #CN ウィーン
うぃすこんしん #CN ウィスコンシン
うぃんとふく #CN ウィントフク
うぇりんとん #CN ウェリントン
うえだ #CN 上田
うえだし #CN 上田市
うえの #CN 上野
うえのし #CN 上野市
うぉーるがい #CN ウォール街
うおづ #CN 魚津
うおづし #CN 魚津市
うがんだ #CN ウガンダ
うきょうく #CN 右京区
うくらいな #CN ウクライナ
うぐいすだに #CN 鴬谷
うさ #CN 宇佐
うさし #CN 宇佐市
うしくぬま #CN 牛久沼
うしごめ #CN 牛込
うしぶか #CN 牛深
うしぶかし #CN 牛深市
うじ #CN 宇治
うじし #CN 宇治市
うすき #CN 臼杵
うすきし #CN 臼杵市
うすざん #CN 有珠山
うずべく #CN ウズベク
うずまさ #CN 太秦
うたしない #CN 歌志内
うたしないし #CN 歌志内市
うちさいわい #CN 内幸
うちさいわいちょう #CN 内幸町
うつくしがはら #CN 美ケ原
うつのみや #CN 宇都宮
うつのみやし #CN 宇都宮市
うと #CN 宇土
うとし #CN 宇土市
うべ #CN 宇部
うべし #CN 宇部市
うらが #CN 浦賀
うらじおすとく #CN ウラジオストク
うらそえ #CN 浦添
うらそえし #CN 浦添市
うらやす #CN 浦安
うらやすし #CN 浦安市
うらわ #CN 浦和
うらわし #CN 浦和市
うらんばーとる #CN ウランバートル
うるがあい #CN ウルグアイ
うわじま #CN 宇和島
うわじまし #CN 宇和島市
うんぜんだけ #CN 雲仙岳
えいこく #CN 英国
えいたい #CN 永代
えきまえ #CN 駅前
えくあどる #CN エクアドル
えこだ #CN 江古田
えさし #CN 江刺
えさしし #CN 江刺市
えじぷと #CN エジプト
えじんばら #CN エジンバラ
えすとにあ #CN エストニア
えちおぴあ #CN エチオピア
えとろふ #CN 択捉
えどがわく #CN 江戸川区
えどがわばし #CN 江戸川橋
えどぼり #CN 江戸堀
えな #CN 恵那
えなし #CN 恵那市
えにわ #CN 恵庭
えにわし #CN 恵庭市
えのしま #CN 江ノ島
えひめ #CN 愛媛
えひめけん #CN 愛媛県
えびす #CN 恵比寿 恵美須
えびな #CN 海老名
えびなし #CN 海老名市
えびのし #CN えびの市
えべつ #CN 江別
えべつし #CN 江別市
えべれすと #CN エベレスト
えぼしだけ #CN 烏帽子岳
えりとりあ #CN エリトリア
えるさるばどる #CN エルサルバドル
えるされむ #CN エルサレム
えればん #CN エレバン
えんざん #CN 塩山
えんざんし #CN 塩山市
えんてべ #CN エンテベ
おーくらんど #CN オークランド
おーすとらりあ #CN オーストラリア
おーすとりあ #CN オーストリア
おーとぼるた #CN オートボルタ
おいけ #CN 御池
おうう #CN 奥羽
おうしゅう #CN 欧州
おうめ #CN 青梅
おうめし #CN 青梅市
おおい #CN 大井
おおいずみ #CN 大泉
おおいずみがくえん #CN 大泉学園
おおいそ #CN 大磯
おおいた #CN 大分
おおいたけん #CN 大分県
おおいたし #CN 大分市
おおいまちせん #CN 大井町線
おおおかやま #CN 大岡山
おおかわ #CN 大川
おおかわし #CN 大川市
おおがき #CN 大垣
おおがきし #CN 大垣市
おおぎまち #CN 扇町
おおくち #CN 大口
おおくちし #CN 大口市
おおさか #CN 大阪
おおさかし #CN 大阪市
おおさかふ #CN 大阪府
おおさかわん #CN 大阪湾
おおさき #CN 大崎
おおしま #CN 大島
おおじま #CN 大島
おおず #CN 大洲
おおずし #CN 大洲市
おおた #CN 太田
おおたく #CN 大田区
おおたけ #CN 大竹
おおたけし #CN 大竹市
おおたし #CN 太田市
おおたわら #CN 大田原
おおたわらし #CN 大田原市
おおだ #CN 大田
おおだし #CN 大田市
おおだて #CN 大館
おおだてし #CN 大館市
おおつ #CN 大津
おおつか #CN 大塚
おおつき #CN 大月
おおつきし #CN 大月市
おおつし #CN 大津市
おおてまち #CN 大手町
おおでんまちょう #CN 大伝馬町
おおどおり #CN 大通り
おおの #CN 大野
おおのし #CN 大野市
おおのじょう #CN 大野城
おおのじょうし #CN 大野城市
おおふな #CN 大船
おおふなと #CN 大船渡
おおふなとし #CN 大船渡市
おおぶ #CN 大府
おおぶし #CN 大府市
おおまがり #CN 大曲
おおまがりし #CN 大曲市
おおまち #CN 大町
おおまちし #CN 大町市
おおみはちまん #CN 近江八幡
おおみはちまんし #CN 近江八幡市
おおみや #CN 大宮
おおみやし #CN 大宮市
おおむた #CN 大牟田
おおむたし #CN 大牟田市
おおむら #CN 大村
おおむらし #CN 大村市
おおよどく #CN 大淀区
おかざき #CN 岡崎
おかざきし #CN 岡崎市
おかちまち #CN 御徒町
おかや #CN 岡谷
おかやし #CN 岡谷市
おかやま #CN 岡山
おかやまけん #CN 岡山県
おかやまし #CN 岡山市
おが #CN 男鹿
おがさわら #CN 小笠原
おがし #CN 男鹿市
おき #CN 隠岐
おきなわ #CN 沖縄
おきなわけん #CN 沖縄県
おきなわし #CN 沖縄市
おぎくぼ #CN 荻窪
おくらほましてぃー #CN オクラホマシティー
おけがわ #CN 桶川
おけがわし #CN 桶川市
おけはざま #CN 桶狭間
おごおり #CN 小郡
おごおりし #CN 小郡市
おしあげ #CN 押上
おすろ #CN オスロ
おせあにあ #CN アセナニア
おたる #CN 小樽
おたるし #CN 小樽市
おたわ #CN オタワ
おだきゅう #CN 小田急
おだわら #CN 小田原
おだわらし #CN 小田原市
おちあい #CN 落合
おちゃのみず #CN 御茶ノ水
おぢや #CN 小千谷
おぢやし #CN 小千谷市
おっくすふぉーど #CN オックスフォード
おでっさ #CN オデッサ
おとわ #CN 音羽
おなりもん #CN 御成門
おの #CN 小野
おのし #CN 小野市
おのだ #CN 小野田
おのだし #CN 小野田市
おのみち #CN 尾道
おのみちし #CN 尾道市
おばなざわ #CN 尾花沢
おばなざわし #CN 尾花沢市
おばま #CN 小浜
おばまし #CN 小浜市
おびひろ #CN 帯広
おびひろし #CN 帯広市
おほーつく #CN オホーツク
おまーん #CN オマーン
おもてさんどう #CN 表参道
おやべ #CN 小矢部
おやべし #CN 小矢部市
おやま #CN 小山
おやまし #CN 小山市
おらんだ #CN オランダ
おりんぴあ #CN オリンピア
おわせ #CN 尾鷲
おわせし #CN 尾鷲市
おわりあさひ #CN 尾張旭
おわりあさひし #CN 尾張旭市
おんせんきょう #CN 温泉郷
おんたけさん #CN 御岳山
かいづか #CN 貝塚
かいづかし #CN 貝塚市
かいなん #CN 海南
かいなんし #CN 海南市
かいろ #CN カイロ
かかみがはら #CN 各務原
かかみがはらし #CN 各務原市
かが #CN 加賀
かがし #CN 加賀市
かがわ #CN 香川
かがわけん #CN 香川県
かくだ #CN 角田
かくだし #CN 角田市
かぐらざか #CN 神楽坂
かけがわ #CN 掛川
かけがわし #CN 掛川市
かこがわ #CN 加古川
かこがわし #CN 加古川市
かごしま #CN 鹿児島
かごしまけん #CN 鹿児島県
かごしまし #CN 鹿児島市
かさい #CN 加西
かさいし #CN 加西市
かさおか #CN 笠岡
かさおかし #CN 笠岡市
かさま #CN 笠間
かさまし #CN 笠間市
かざふ #CN カザフ
かしはら #CN 橿原
かしはらし #CN 橿原市
かしま #CN 鹿島
かしまし #CN 鹿島市
かしわ #CN 柏
かしわざき #CN 柏崎
かしわざきし #CN 柏崎市
かしわし #CN 柏市
かしわら #CN 柏原
かしわらし #CN 柏原市
かすかべ #CN 春日部
かすかべし #CN 春日部市
かすが #CN 春日
かすがい #CN 春日井
かすがいし #CN 春日井市
かすがし #CN 春日市
かすがで #CN 春日出
かすとりーず #CN カストリーズ
かすぴかい #CN カスピ海
かすみがせき #CN 霞ケ関
かせだ #CN 加世田
かせだし #CN 加世田市
かぞ #CN 加須
かぞし #CN 加須市
かたーる #CN カタール
かたの #CN 交野
かたのし #CN 交野市
かつうら #CN 勝浦
かつうらし #CN 勝浦市
かつしかく #CN 葛飾区
かつた #CN 勝田
かつたし #CN 勝田市
かつやま #CN 勝山
かつやまし #CN 勝山市
かづの #CN 鹿角
かづのし #CN 鹿角市
かとまんず #CN カトマンズ
かどま #CN 門真
かどまし #CN 門真市
かながわ #CN 神奈川
かながわく #CN 神奈川区
かながわけん #CN 神奈川県
かなざわ #CN 金沢
かなざわく #CN 金沢区
かなざわし #CN 金沢市
かなざわぶんこ #CN 金沢文庫
かなだ #CN カナダ
かなまち #CN 金町
かなりあ #CN カナリア
かに #CN 可児
かにし #CN 可児市
かぬま #CN 鹿沼
かぬまし #CN 鹿沼市
かのや #CN 鹿屋
かのやし #CN 鹿屋市
かぶき #CN 歌舞伎
かぶきちょう #CN 歌舞伎町
かぶとまち #CN 兜町
かぷーる #CN カプール
かぼべるで #CN カボベルデ
かまいし #CN 釜石
かまいしし #CN 釜石市
かまがや #CN 鎌ケ谷
かまがやし #CN 鎌ケ谷市
かまくら #CN 鎌倉
かまくらし #CN 鎌倉市
かまた #CN 蒲田
かみおおおか #CN 上大岡
かみがた #CN 上方
かみぎょうく #CN 上京区
かみたかいど #CN 上高井戸
かみなりもん #CN 雷門
かみのげ #CN 上野毛
かみのやま #CN 上山
かみのやまし #CN 上山市
かみふくおか #CN 上福岡
かみふくおかし #CN 上福岡市
かみやちょう #CN 神谷町
かむちゃっか #CN カムチャッカ
かめあり #CN 亀有
かめいど #CN 亀戸
かめおか #CN 亀岡
かめおかし #CN 亀岡市
かめやま #CN 亀山
かめやまし #CN 亀山市
かめるーん #CN カメルーン
かも #CN 加茂
かもがわ #CN 鴨川
かもがわし #CN 鴨川市
かもし #CN 加茂市
かやば #CN 茅場
かやばちょう #CN 茅場町
からかす #CN カラカス
からすま #CN 烏丸
からすもり #CN 烏森
からすやま #CN 烏山
からつ #CN 唐津
からつし #CN 唐津市
からふと #CN 樺太
からほりちょう #CN 空堀町
かりふぉるにあ #CN カリフォルニア
かりぶ #CN カリブ
かりぶかい #CN カリブ海
かりや #CN 刈谷
かりやし #CN 刈谷市
かるいざわ #CN 軽井沢
かるかった #CN カルカッタ
かるがり #CN カルガリ
かわぐち #CN 川口
かわぐちこ #CN 河口湖
かわぐちし #CN 川口市
かわごえ #CN 川越
かわごえし #CN 川越市
かわさき #CN 川崎
かわさきく #CN 川崎区
かわさきし #CN 川崎市
かわちながの #CN 河内長野
かわちながのし #CN 河内長野市
かわにし #CN 川西
かわにしし #CN 川西市
かわのえ #CN 川之江
かわのえし #CN 川之江市
かわらまち #CN 河原町 瓦町
かんおんじ #CN 観音寺
かんおんじし #CN 観音寺市
かんこく #CN 韓国 大韓民国
かんさい #CN 関西
かんざすしてぃー #CN カンザスシティー
かんじょうせん #CN 環状線
かんだ #CN 神田
かんとう #CN 関東
かんとうちほう #CN 関東地方
かんない #CN 関内
かんはっしゅう #CN 関八州
かんぱら #CN カンパラ
かんぼじあ #CN カンボジア
がーな #CN ガーナ
がいあな #CN ガイアナ
がくげいだいがく #CN 学芸大学
がぼろーね #CN ガボローネ
がぼん #CN ガボン
がまごおり #CN 蒲郡
がまごおりし #CN 蒲郡市
がんびあ #CN ガンビア
きい #CN 紀伊
きえふ #CN キエフ
きがり #CN キガリ
きくち #CN 菊池
きくちし #CN 菊池市
きさらづ #CN 木更津
きさらづし #CN 木更津市
きざきこ #CN 木崎湖
きしにょふ #CN キシニョフ
きしゅう #CN 紀州
きしわだ #CN 岸和田
きしわだし #CN 岸和田市
きそ #CN 木曽
きそがわ #CN 木曽川
きたいばらき #CN 北茨城
きたいばらきし #CN 北茨城市
きたかた #CN 喜多方
きたかたし #CN 喜多方市
きたかみ #CN 北上
きたかみし #CN 北上市
きたかんとう #CN 北関東
きたきゅうしゅう #CN 北九州
きたきゅうしゅうし #CN 北九州市
きたく #CN 北区
きたせんじゅ #CN 北千住
きたたいせいよう #CN 北大西洋
きたにほん #CN 北日本
きたみ #CN 北見
きたみし #CN 北見市
きたもと #CN 北本
きたもとし #CN 北本市
きちじょうじ #CN 吉祥寺
きつき #CN 杵築
きつきし #CN 杵築市
きと #CN キト
きぬた #CN 砧
きぷろす #CN キプロス
きぼうほう #CN 喜望峰
きみつ #CN 君津
きみつし #CN 君津市
きゃんべら #CN キャンベラ
きゅーば #CN キューバ
きゅうしゅう #CN 九州
きゅうしゅうちほう #CN 九州地方
きょうと #CN 京都
きょうとし #CN 京都市
きょうとふ #CN 京都府
きょうばし #CN 京橋
きょうまちぼり #CN 京町堀
きよせ #CN 清瀬
きよせし #CN 清瀬市
きりしまやま #CN 霧島山
きりばす #CN キリバス
きりゅう #CN 桐生
きりゅうし #CN 桐生市
きるぎす #CN キルギス
きんき #CN 近畿
きんきちほう #CN 近畿地方
きんぐすとん #CN キングストン
きんしちょう #CN 錦糸町
きんしゃさ #CN キンシャサ
ぎおん #CN 祇園
ぎにあ #CN ギニア
ぎにあびさう #CN ギニアビサウ
ぎのわん #CN 宜野湾
ぎのわんし #CN 宜野湾市
ぎふ #CN 岐阜
ぎふけん #CN 岐阜県
ぎふし #CN 岐阜市
ぎょうだ #CN 行田
ぎょうだし #CN 行田市
ぎょうとく #CN 行徳
ぎりしゃ #CN ギリシャ
ぎんざ #CN 銀座
ぎんざせん #CN 銀座線
くあらるんぷーる #CN クアラルンプール
くうぇーと #CN クウェート
くうこうせん #CN 空港線
くがやま #CN 久我山
くき #CN 久喜
くきし #CN 久喜市
くさつ #CN 草津
くさつし #CN 草津市
くしきの #CN 串木野
くしきのし #CN 串木野市
くしま #CN 串間
くしまし #CN 串間市
くしろ #CN 釧路
くしろし #CN 釧路市
くじ #CN 久慈
くじし #CN 久慈市
くじゅうさん #CN 久住山
くじょう #CN 九条
くずりゅうがわ #CN 九頭竜川
くだまつ #CN 下松
くだまつし #CN 下松市
くだん #CN 九段
くっしゃろこ #CN 屈斜路湖
くなしり #CN 国後
くにたち #CN 国立
くにたちし #CN 国立市
くまがや #CN 熊谷
くまがやし #CN 熊谷市
くまがわ #CN 球磨川
くまの #CN 熊野
くまのし #CN 熊野市
くまもと #CN 熊本
くまもとけん #CN 熊本県
くまもとし #CN 熊本市
くらいすとちゃーち #CN クライストチャーチ
くらしき #CN 倉敷
くらしきし #CN 倉敷市
くらま #CN 鞍馬
くらまえ #CN 蔵前
くらよし #CN 倉吉
くらよしし #CN 倉吉市
くるめ #CN 久留米
くるめし #CN 久留米市
くれ #CN 呉
くれし #CN 呉市
くろいし #CN 黒石
くろいしし #CN 黒石市
くろいそ #CN 黒磯
くろいそし #CN 黒磯市
くろさきまち #CN 黒埼町
くろべ #CN 黒部
くろべし #CN 黒部市
くわな #CN 桑名
くわなし #CN 桑名市
ぐあてまら #CN グアテマラ
ぐあどるーぷ #CN グアドループ
ぐあむ #CN グアム
ぐあむとう #CN グアム島
ぐしかわ #CN 具志川
ぐしかわし #CN 具志川市
ぐりーんらんど #CN グリーンランド
ぐるじや #CN グルジヤ
ぐれなだ #CN グレナダ
ぐんま #CN 群馬
ぐんまけん #CN 群馬県
けーぷたうん #CN ケープタウン
けいおう #CN 京王
けいせい #CN 京成
けいはん #CN 京阪
けいはんしん #CN 京阪神
けいひん #CN 京浜
けいひんきゅうこう #CN 京浜急行
けいひんとうほく #CN 京浜東北
けいよう #CN 京葉
けごんのたき #CN 華厳滝
けせんぬま #CN 気仙沼
けせんぬまし #CN 気仙沼市
けにあ #CN ケニア
けんたっきー #CN ケンタッキー
けんぶりっじ #CN ケンブリッジ
げんかいなだ #CN 玄界灘
こーとじぼあーる #CN コートジボアール
こいしかわ #CN 小石川
こいわ #CN 小岩
こうえんじ #CN 高円寺
こうかい #CN 黄海 紅海
こうしょく #CN 更埴
こうしょくし #CN 更埴市
こうしんえつ #CN 甲信越
こうじまち #CN 麹町
こうじや #CN 糀谷
こうそくどうろ #CN 高速道路
こうち #CN 高知
こうちけん #CN 高知県
こうちし #CN 高知市
こうとう #CN 江東
こうとうく #CN 江東区
こうなん #CN 江南 港南
こうなんく #CN 港南区
こうなんし #CN 江南市
こうのす #CN 鴻巣
こうのすし #CN 鴻巣市
こうばいちょう #CN 紅梅町
こうふ #CN 甲府
こうふし #CN 甲府市
こうべ #CN 神戸
こうべし #CN 神戸市
こうほくく #CN 港北区
こうらいばし #CN 高麗橋
こおりやま #CN 郡山
こおりやまし #CN 郡山市
こが #CN 古河
こがし #CN 古河市
こがねい #CN 小金井
こがねいし #CN 小金井市
こくさいくうこう #CN 国際空港
こくぶ #CN 国分
こくぶし #CN 国分市
こくぶんじ #CN 国分寺
こくぶんじし #CN 国分寺市
こくらきたく #CN 小倉北区
こくらみなみく #CN 小倉南区
こさい #CN 湖西
こさいし #CN 湖西市
こしがや #CN 越谷
こしがやし #CN 越谷市
こすたりか #CN コスタリカ
こたばる #CN コタバル
こだいら #CN 小平
こだいらし #CN 小平市
こっかい #CN 黒海
こでんまちょう #CN 小伝馬町
こなくり #CN コナクリ
こねちかっと #CN コネチカット
このはなく #CN 此花区
こばやし #CN 小林
こばやしし #CN 小林市
こぺんはーげん #CN コペンハーゲン
こまえ #CN 狛江
こまえし #CN 狛江市
こまがたけ #CN 駒ケ岳
こまがね #CN 駒ケ根
こまがねし #CN 駒ケ根市
こまき #CN 小牧
こまきし #CN 小牧市
こまごめ #CN 駒込
こまつ #CN 小松
こまつし #CN 小松市
こまつしま #CN 小松島
こまつしまし #CN 小松島市
こみやちょう #CN 小宮町
こもろ #CN コモロ 小諸
こもろし #CN 小諸市
こやま #CN 小山
ころんびあ #CN コロンビア
ころんぼ #CN コロンボ
こんご #CN コンゴ
こんごうさん #CN 金剛山
ごうしゅう #CN 豪州
ごうつ #CN 江津
ごうつし #CN 江津市
ごこくじ #CN 護国寺
ごしきぬま #CN 五色沼
ごしきやま #CN 五色山
ごしょがわら #CN 五所川原
ごしょがわらし #CN 五所川原市
ごじょう #CN 五條
ごじょうし #CN 五條市
ごせ #CN 御所
ごせし #CN 御所市
ごせん #CN 五泉
ごせんし #CN 五泉市
ごたんだ #CN 五反田
ごっとほーぷ #CN ゴットホープ
ごてんば #CN 御殿場
ごてんばし #CN 御殿場市
ごぼう #CN 御坊
ごぼうし #CN 御坊市
さいき #CN 佐伯
さいきし #CN 佐伯市
さいこ #CN 西湖
さいじょう #CN 西条
さいじょうし #CN 西条市
さいたま #CN 埼玉
さいたまけん #CN 埼玉県
さいと #CN 西都
さいとし #CN 西都市
さいわいく #CN 幸区
さうじあらびあ #CN サウジアラビア
さかい #CN 堺
さかいし #CN 堺市
さかいで #CN 坂出
さかいでし #CN 坂出市
さかいみなと #CN 境港
さかいみなとし #CN 境港市
さかた #CN 酒田
さかたし #CN 酒田市
さかど #CN 坂戸
さかどし #CN 坂戸市
さが #CN 佐賀
さがえ #CN 寒河江
さがえし #CN 寒河江市
さがけん #CN 佐賀県
さがし #CN 佐賀市
さがの #CN 嵯峨野
さがみこ #CN 相模湖
さがみはら #CN 相模原
さがみはらし #CN 相模原市
さきょうく #CN 左京区
さく #CN 佐久
さくし #CN 佐久市
さくら #CN 佐倉
さくらい #CN 桜井
さくらいし #CN 桜井市
さくらがおか #CN 桜ケ丘
さくらぎちょう #CN 桜木町
さくらし #CN 佐倉市
さくらしんまち #CN 桜新町
さくらじま #CN 桜島
さくらだ #CN 桜田
さくらだい #CN 桜台
さくらだもん #CN 桜田門
さくらのみや #CN 桜ノ宮
ささづか #CN 笹塚
させぼ #CN 佐世保
させぼし #CN 佐世保市
さっぽろ #CN 札幌
さっぽろし #CN 札幌市
さど #CN 佐渡
さどがしま #CN 佐渡島
さないちょう #CN 左内町
さなだやまちょう #CN 真田山町
さぬあ #CN サヌア
さの #CN 佐野
さのし #CN 佐野市
さはら #CN サハラ
さばえ #CN 鯖江
さばえし #CN 鯖江市
さばな #CN サバナ
さやま #CN 狭山
さやまし #CN 狭山市
さらえぼ #CN サラエボ
さわら #CN 佐原
さわらく #CN 早良区
さわらし #CN 佐原市
さんいん #CN 山陰
さんげんぢゃや #CN 三軒茶屋
さんさるばどる #CN サンサルバドル
さんじえご #CN サンジエゴ
さんじょう #CN 三条
さんじょうし #CN 三条市
さんだ #CN 三田
さんだし #CN 三田市
さんちあご #CN サンチアゴ
さんとどみんご #CN サントドミンゴ
さんとめ #CN サントメ
さんとめぷりんしぺ #CN サントメプリンシペ
さんどに #CN サンドニ
さんのう #CN 山王
さんのみや #CN 三の宮
さんぱうろ #CN サンパウロ
さんふあん #CN サンフアン
さんふらんしすこ #CN サンフランシスコ
さんほせ #CN サンホセ
さんぼんまつ #CN 三本松
さんまりの #CN サンマリノ
さんよう #CN 山陽
さんりく #CN 三陸
ざいーる #CN ザイール
ざいけ #CN 在家
ざいもくちょう #CN 材木町
ざおうさん #CN 蔵王山
ざま #CN 座間
ざまし #CN 座間市
ざるつぶるく #CN ザルツビルク
ざんびあ #CN ザンビア
しあとる #CN シアトル
しえられおね #CN シエラレオネ
しおがま #CN 塩竃 塩釜
しおがまし #CN 塩竃市
しおじり #CN 塩尻
しおじりし #CN 塩尻市
しおみばし #CN 汐見橋
しかい #CN 死海
しかご #CN シカゴ
しが #CN 滋賀
しがけん #CN 滋賀県
しき #CN 志木
しきし #CN 志木市
しぎさん #CN 信貴山
しこく #CN 四国
しこくちほう #CN 四国地方
しこたん #CN 色丹
しじょうなわて #CN 四条畷
しじょうなわてし #CN 四条畷市
しずおか #CN 静岡
しずおかけん #CN 静岡県
しずおかし #CN 静岡市
したまち #CN 下町
しどにー #CN シドニー
しながわ #CN 品川
しながわく #CN 品川区
しなの #CN 信濃
しなのまち #CN 信濃町
しののめ #CN 東雲
しばこうえん #CN 芝公園
しばた #CN 新発田
しばたし #CN 新発田市
しばだいもん #CN 芝大門
しぶかわ #CN 渋川
しぶかわし #CN 渋川市
しぶや #CN 渋谷
しぶやく #CN 渋谷区
しべつ #CN 士別
しべつし #CN 士別市
しべりあ #CN シベリア
しまだ #CN 島田
しまだし #CN 島田市
しまね #CN 島根
しまねけん #CN 島根県
しまばら #CN 島原
しまばらし #CN 島原市
しみず #CN 清水
しみずし #CN 清水市
しみずだにちょう #CN 清水谷町
しもきたざわ #CN 下北沢
しもぎょうく #CN 下京区
しもたかいど #CN 下高井戸
しもだ #CN 下田
しもだし #CN 下田市
しもだて #CN 下館
しもだてし #CN 下館市
しもつま #CN 下妻
しもつまし #CN 下妻市
しものせき #CN 下関
しものせきし #CN 下関市
しもまるこ #CN 下丸子
しゃんぜりぜ #CN シャンゼリゼ
しょうじこ #CN 精進湖
しょうどしま #CN 小豆島
しょうない #CN 庄内
しょうなん #CN 湘南
しょうにゅうどう #CN 鍾乳洞
しょうばら #CN 庄原
しょうばらし #CN 庄原市
しょうわく #CN 昭和区
しょとう #CN 諸島
しらうめ #CN 白梅
しらかわ #CN 白河
しらかわし #CN 白河市
しらねさん #CN 白根山
しりあ #CN シリア
しれとこ #CN 知床
しろいし #CN 白石
しろいしく #CN 白石区
しろいしし #CN 白石市
しろかね #CN 白金
しろね #CN 白根
しろねし #CN 白根市
しろろしあ #CN 白ロシア
しんえつ #CN 信越
しんおおくぼ #CN 新大久保
しんおおさか #CN 新大阪
しんかんせん #CN 新幹線
しんがぽーる #CN シンガポール
しんぐう #CN 新宮
しんぐうし #CN 新宮市
しんこだいら #CN 新小平
しんさいばしすじ #CN 心斎橋筋
しんさくらだい #CN 新桜台
しんしゅう #CN 信州
しんしろ #CN 新城
しんしろし #CN 新城市
しんじゅく #CN 新宿
しんじゅくぎょえん #CN 新宿御苑
しんじゅくく #CN 新宿区
しんじゅわん #CN 真珠湾
しんじょう #CN 新庄
しんじょうし #CN 新庄市
しんたまがわせん #CN 新玉川線
しんとうきょうこくさい #CN 新東京国際
しんなんよう #CN 新南陽
しんなんようし #CN 新南陽市
しんばし #CN 新橋
しんまち #CN 新町
しんみなと #CN 新湊
しんみなとし #CN 新湊市
じぇのば #CN ジェノバ
じぶち #CN ジブチ
じゃかるた #CN ジャカルタ
じゃまいか #CN ジャマイカ
じゃわ #CN ジャワ
じゅうじょう #CN 十条
じゅしゃんべ #CN ジュシャンベ
じゅねーぶ #CN ジュネーブ
じょーじあ #CN ジョージア
じょーじたうん #CN ジョージタウン
じょうえつ #CN 上越
じょうえつし #CN 上越市
じょうがしま #CN 城ケ島
じょうしゅう #CN 上州
じょうしんえつ #CN 上信越
じょうせき #CN 城跡
じょうとうく #CN 城東区
じょうなんく #CN 城南区
じょうばんせん #CN 常盤線
じょうよう #CN 城陽
じょうようし #CN 城陽市
じょほーる #CN ジョホール
じんぐうまえ #CN 神宮前
じんづうがわ #CN 神通川
じんなん #CN 神南
じんばぶえ #CN ジンバブエ
じんぼうちょう #CN 神保町
すーだん #CN スーダン
すいす #CN スイス
すいた #CN 吹田
すいたし #CN 吹田市
すいどうばし #CN 水道橋
すうぇーでん #CN スウェデーン
すえひろちょう #CN 末広町
すかがわ #CN 須賀川
すかがわし #CN 須賀川市
すかんじなびあ #CN スカンジナビア
すがも #CN 巣鴨
すぎなみく #CN 杉並区
すくも #CN 宿毛
すくもし #CN 宿毛市
すさき #CN 須崎
すさきし #CN 須崎市
すざか #CN 須坂
すざかし #CN 須坂市
すず #CN 珠洲
すずか #CN 鈴鹿
すずかし #CN 鈴鹿市
すずし #CN 珠洲市
すその #CN 裾野
すそのし #CN 裾野市
すとっくほるむ #CN ストックホルム
すながわ #CN 砂川
すながわし #CN 砂川市
すば #CN スバ
すぺいん #CN スペイン
すぺりおる #CN スペリオル
すまく #CN 須磨区
すみだ #CN 墨田
すみだく #CN 墨田区
すみのえ #CN 住之江
すみのえく #CN 住之江区
すみよし #CN 住吉
すみよしく #CN 住吉区
すみよしちょう #CN 住吉町
すもと #CN 洲本
すもとし #CN 洲本市
すりらんか #CN スリランカ
するが #CN 駿河
すわ #CN 諏訪
すわこ #CN 諏訪湖
すわし #CN 諏訪市
すわじらんど #CN スワジランド
ずし #CN 逗子
ずしし #CN 逗子市
せーしぇる #CN セーシェル
せいおう #CN 西欧
せいじょう #CN 成城
せいせき #CN 聖蹟
せいどく #CN 西独
せいよう #CN 西洋
せき #CN 関
せきし #CN 関市
せきどうぎにあ #CN 赤道ギニア
せきや #CN 関屋
せたがや #CN 世田谷
せたがやく #CN 世田谷区
せっつ #CN 摂津
せっつし #CN 摂津市
せと #CN 瀬戸
せとうち #CN 瀬戸内
せとし #CN 瀬戸市
せとないかい #CN 瀬戸内海
せねがる #CN セネガル
せやく #CN 瀬谷区
せんかく #CN 尖閣
せんがくじ #CN 泉岳寺
せんごく #CN 千石
せんじゅ #CN 千住
せんじゅおおはし #CN 千住大橋
せんだい #CN 仙台 川内
せんだいし #CN 仙台市 川内市
せんだぎ #CN 千駄木
せんとじょーじず #CN セントジョージズ
せんとびんせんと #CN セントビンセント
せんとるしあ #CN セントルシア
せんなん #CN 泉南
せんなんし #CN 泉南市
せんにちまえ #CN 千日前
せんば #CN 船場
せんばちゅうおう #CN 船場中央
ぜんつうじ #CN 善通寺
ぜんつうじし #CN 善通寺市
そーるずべり #CN ソールズベリ
そうか #CN 草加
そうかし #CN 草加市
そうじゃ #CN 総社
そうじゃし #CN 総社市
そうぶせん #CN 総武線
そうぶほんせん #CN 総武本線
そうま #CN 相馬
そうまし #CN 相馬市
そうや #CN 宗谷
そうる #CN ソウル
そねざき #CN 曽根崎
そびえと #CN ソビエト
そふぃあ #CN ソフィア
そまりあ #CN ソマリア
それん #CN ソビエト連邦 ソ連
そろもん #CN ソロモン
ぞうしがや #CN 雑司が谷
たい #CN タイ
たいしょうく #CN 大正区
たいせいよう #CN 大西洋
たいとうく #CN 台東区
たいへい #CN 太平
たいへいよう #CN 太平洋
たいわん #CN 台湾
たかいし #CN 高石
たかいしし #CN 高石市
たかお #CN 高尾
たかおか #CN 高岡
たかおかし #CN 高岡市
たかさき #CN 高崎
たかさきし #CN 高崎市
たかさご #CN 高砂
たかさごし #CN 高砂市
たかしまだいら #CN 高島平
たかしまや #CN 高島屋
たかだのばば #CN 高田馬場
たかちほのみね #CN 高千穂峰
たかつき #CN 高槻
たかつきし #CN 高槻市
たかつく #CN 高津区
たかなわだい #CN 高輪台
たかはぎ #CN 高萩
たかはぎし #CN 高萩市
たかはし #CN 高梁
たかはしし #CN 高梁市
たかはま #CN 高浜
たかはまし #CN 高浜市
たかまがはら #CN 高天原
たかまつ #CN 高松
たかまつし #CN 高松市
たかやま #CN 高山
たかやまし #CN 高山市
たからづか #CN 宝塚
たからづかし #CN 宝塚市
たがじょう #CN 多賀城
たがじょうし #CN 多賀城市
たがわ #CN 田川
たがわし #CN 田川市
たきかわ #CN 滝川
たきかわし #CN 滝川市
たきのがわ #CN 滝野川
たく #CN 多久
たくし #CN 多久市
たけお #CN 武雄
たけおし #CN 武雄市
たけた #CN 竹田
たけたし #CN 竹田市
たけのつか #CN 竹の塚
たけはら #CN 竹原
たけはらし #CN 竹原市
たけばし #CN 竹橋
たけふ #CN 武生
たけふし #CN 武生市
たざわこ #CN 田沢湖
たしけんと #CN タシケント
たじく #CN タジク
たじみ #CN 多治見
たじみし #CN 多治見市
たたらがわ #CN 多多良川
たちかわ #CN 立川
たちかわし #CN 立川市
たつの #CN 龍野
たつのし #CN 龍野市
たてばやし #CN 館林
たてばやしし #CN 館林市
たてやま #CN 館山
たてやまし #CN 館山市
たなし #CN 田無
たなしし #CN 田無市
たなべ #CN 田辺
たなべし #CN 田辺市
たにまち #CN 谷町
たばた #CN 田端
たひち #CN タヒチ
たま #CN 多摩
たまがわ #CN 多摩川
たまがわじょうすい #CN 玉川上水
たまく #CN 多摩区
たまこせん #CN 多摩湖線
たまし #CN 多摩市
たまな #CN 玉名
たまなし #CN 玉名市
たまの #CN 玉野
たまのし #CN 玉野市
たまれいえん #CN 多摩霊園
たらわ #CN タラワ
たりん #CN タリン
たるみ #CN 垂水
たるみく #CN 垂水区
たるみず #CN 垂水
たるみずし #CN 垂水市
たわらちょう #CN 田原町
たんざにあ #CN タンザニア
だーうぃん #CN ダーウィン
だーじりん #CN ダージリン
だいかんみんこく #CN 大韓民国
だいかんやま #CN 代官山
だいざわ #CN 代沢
だいしまえ #CN 大師前
だいとう #CN 大東
だいとうし #CN 大東市
だいもん #CN 大門
だいもんじやま #CN 大文字山
だかーる #CN ダカール
だざいふ #CN 太宰府
だざいふし #CN 太宰府市
だっか #CN ダッカ
だて #CN 伊達
だてし #CN 伊達市
だぶりん #CN ダブリン
だますかす #CN ダマスカス
だるえすさらーむ #CN ダルエスサラーム
ちぇこするばきあ #CN チェコスロバキア
ちぇりゃびんすく #CN チェリャビンスク
ちがさき #CN 茅ケ崎
ちがさきし #CN 茅ケ崎市
ちくご #CN 筑後
ちくごがわ #CN 筑後川
ちくごし #CN 筑後市
ちくさく #CN 千種区
ちくしの #CN 筑紫野
ちくしのし #CN 筑紫野市
ちしま #CN 千島
ちた #CN 知多
ちたし #CN 知多市
ちちぶ #CN 秩父
ちちぶし #CN 秩父市
ちちゅうかい #CN 地中海
ちっこう #CN 築港
ちとせ #CN 千歳
ちとせからすやま #CN 千歳烏山
ちとせし #CN 千歳市
ちどりちょう #CN 千鳥町
ちの #CN 茅野
ちのし #CN 茅野市
ちば #CN 千葉
ちばけん #CN 千葉県
ちばし #CN 千葉市
ちべっと #CN チベット
ちゃうすやま #CN 茶臼山
ちゃど #CN チャド
ちゃやまち #CN 茶屋町
ちゅーりっひ #CN チューリッヒ
ちゅうおうあふりか #CN 中央アフリカ
ちゅうおうく #CN 中央区
ちゅうおうせん #CN 中央線
ちゅうか #CN 中華
ちゅうかがい #CN 中華街
ちゅうかみんこく #CN 中華民国
ちゅうきんとう #CN 中近東
ちゅうごく #CN 中華人民共和国 中国
ちゅうごくちほう #CN 中国地方
ちゅうぜんじこ #CN 中禅寺湖
ちゅうとう #CN 中東
ちゅうなんべい #CN 中南米
ちゅうにち #CN 中日
ちゅうぶちほう #CN 中部地方
ちゅうべい #CN 中米
ちゅにじあ #CN チュニジア
ちゅにす #CN チュニス
ちょうかいさん #CN 鳥海山
ちょうし #CN 銚子
ちょうしし #CN 銚子市
ちょうじゃまち #CN 長者町
ちょうせん #CN 朝鮮
ちょうふ #CN 調布
ちょうふし #CN 調布市
ちよざき #CN 千代崎
ちよだく #CN 千代田区
ちよだせん #CN 千代田線
ちらな #CN チラナ
ちり #CN チリ
ちりゅう #CN 知立
ちりゅうし #CN 知立市
ちんぷー #CN チンプー
つ #CN 津
つくし #CN 筑紫
つくだ #CN 佃
つくばさん #CN 筑波山
つくみ #CN 津久見
つくみし #CN 津久見市
つし #CN 津市
つしま #CN 対馬 津島
つしまし #CN 津島市
つじどう #CN 辻堂
つだぬま #CN 津田沼
つちうら #CN 土浦
つちうらし #CN 土浦市
つばめ #CN 燕
つばめし #CN 燕市
つばる #CN ツバル
つまご #CN 妻篭
つやま #CN 津山
つやまし #CN 津山市
つりがねちょう #CN 釣鐘町
つる #CN 都留
つるおか #CN 鶴岡
つるおかし #CN 鶴岡市
つるが #CN 敦賀
つるがし #CN 敦賀市
つるぎさん #CN 剣山
つるし #CN 都留市
つるはし #CN 鶴橋
つるまき #CN 弦巻
つるみ #CN 鶴見
つるみく #CN 鶴見区
てきさす #CN テキサス
てぐしがるぱ #CN テグシガルパ
てへらん #CN テヘラン
てむず #CN テムズ
てんじん #CN 天神
てんじんばし #CN 天神橋
てんどう #CN 天童
てんどうし #CN 天童市
てんのうじく #CN 天王寺区
てんぱくく #CN 天白区
てんま #CN 天満
てんり #CN 天理
てんりし #CN 天理市
てんりゅう #CN 天竜
てんりゅうし #CN 天竜市
でんえんちょうふ #CN 田園調布
でんまーく #CN デンマーク
とーご #CN トーゴ
とうおう #CN 東欧
とうかい #CN 東海
とうかいし #CN 東海市
とうがね #CN 東金
とうがねし #CN 東金市
とうきょう #CN 東京
とうきょうこくさい #CN 東京国際
とうきょうと #CN 東京都
とうきょうわん #CN 東京湾
とうざいせん #CN 東西線
とうじょうせん #CN 東上線
とうどく #CN 東独
とうほく #CN 東北
とうほくちほう #CN 東北地方
とうほくほんせん #CN 東北本線
とうやこ #CN 洞爺湖
とうよ #CN 東予
とうよう #CN 東洋
とうようちょう #CN 東陽町
とうよし #CN 東予市
とおかまち #CN 十日町
とおかまちし #CN 十日町市
とおの #CN 遠野
とおのし #CN 遠野市
とかち #CN 十勝
とき #CN 土岐
ときし #CN 土岐市
とくしま #CN 徳島
とくしまけん #CN 徳島県
とくしまし #CN 徳島市
とくのしま #CN 徳之島
とくやま #CN 徳山
とくやまし #CN 徳山市
とこなめ #CN 常滑
とこなめし #CN 常滑市
ところざわ #CN 所沢
ところざわし #CN 所沢市
とごし #CN 戸越
とさ #CN 土佐
とさし #CN 土佐市
とさしみず #CN 土佐清水
とさしみずし #CN 土佐清水市
とさぼり #CN 土佐堀
としまえん #CN 豊島園
としまく #CN 豊島区
とす #CN 鳥栖
とすし #CN 鳥栖市
とだ #CN 戸田
とだし #CN 戸田市
とちお #CN 栃尾
とちおし #CN 栃尾市
とちぎ #CN 栃木
とちぎけん #CN 栃木県
とちぎし #CN 栃木市
とっとり #CN 鳥取
とっとりけん #CN 鳥取県
とっとりし #CN 鳥取市
とつかく #CN 戸塚区
となみ #CN 礪波
となみし #CN 礪波市
とば #CN 鳥羽
とばし #CN 鳥羽市
とばたく #CN 戸畑区
とびりし #CN トビリシ
とまこまい #CN 苫小牧
とまこまいし #CN 苫小牧市
とみおか #CN 富岡
とみおかし #CN 富岡市
とやま #CN 富山
とやまけん #CN 富山県
とやまし #CN 富山市
とよあけ #CN 豊明
とよあけし #CN 豊明市
とよおか #CN 豊岡
とよおかし #CN 豊岡市
とよかわ #CN 豊川
とよかわし #CN 豊川市
とよさか #CN 豊栄
とよさかし #CN 豊栄市
とよす #CN 豊洲
とよた #CN 豊田
とよたし #CN 豊田市
とよたま #CN 豊玉
とよなか #CN 豊中
とよなかし #CN 豊中市
とよはし #CN 豊橋
とよはしし #CN 豊橋市
とよひら #CN 豊平
とよひらく #CN 豊平区
とらのもん #CN 虎ノ門
とりで #CN 取手
とりでし #CN 取手市
とりにだーどとばご #CN トリニダードトバゴ
とりぽり #CN トリポリ
とるくめん #CN トルクメン
とるこ #CN トルコ
とわだ #CN 十和田
とわだこ #CN 十和田湖
とわだし #CN 十和田市
とんが #CN トンガ
とんがおうこく #CN トンガ王国
とんだばやし #CN 富田林
とんだばやしし #CN 富田林市
どーば #CN ドーバ
どーばかいきょう #CN ドーバ海峡
どうげんざか #CN 道玄坂
どうご #CN 道後
どうじま #CN 堂島
どうとんぼり #CN 道頓堀
どぶいけすじ #CN 丼池筋
どみにか #CN ドミニカ
ないじぇりあ #CN ナイジェリア
ないろび #CN ナイロビ
なうる #CN ナウル
なかがみ #CN 中神
なかがわく #CN 中川区
なかぎょうく #CN 中京区
なかく #CN 中区
なかじま #CN 中島
なかす #CN 中洲
なかつ #CN 中津
なかつがわ #CN 中津川
なかつがわし #CN 中津川市
なかつし #CN 中津市
なかの #CN 中野
なかのく #CN 中野区
なかのし #CN 中野市
なかのしま #CN 中ノ島 中之島
なかのぶ #CN 中延
なかはらく #CN 中原区
なかま #CN 中間
なかまし #CN 中間市
なかみなと #CN 那珂湊
なかみなとし #CN 那珂湊市
なかむら #CN 中村
なかむらく #CN 中村区
なかむらし #CN 中村市
なかめぐろ #CN 中目黒
なかもず #CN 中百舌鳥
ながい #CN 長井
ながいし #CN 長井市
ながおか #CN 長岡
ながおかきょう #CN 長岡京
ながおかきょうし #CN 長岡京市
ながおかし #CN 長岡市
ながおかてんじん #CN 長岡天神
ながさき #CN 長崎
ながさきけん #CN 長崎県
ながさきし #CN 長崎市
ながたく #CN 長田区
ながたちょう #CN 永田町
ながつだ #CN 長津田
ながと #CN 長門
ながとし #CN 長門市
ながの #CN 長野
ながのけん #CN 長野県
ながのし #CN 長野市
ながはま #CN 長浜
ながはまし #CN 長浜市
ながれやま #CN 流山
ながれやまし #CN 流山市
なぎさん #CN 那岐山
なご #CN 名護
なごし #CN 名護市
なごや #CN 名古屋
なごやし #CN 名古屋市
なぜ #CN 名瀬
なぜし #CN 名瀬市
なだく #CN 灘区
なっそー #CN ナッソー
なとり #CN 名取
なとりし #CN 名取市
ななお #CN 七尾
ななおし #CN 七尾市
なにわく #CN 浪速区
なは #CN 那覇
なはし #CN 那覇市
なばり #CN 名張
なばりし #CN 名張市
なほとか #CN ナホトカ
なぽり #CN ナポリ
なみびあ #CN ナミビア
なみよけ #CN 波除
なめりかわ #CN 滑川
なめりかわし #CN 滑川市
なよろ #CN 名寄
なよろし #CN 名寄市
なら #CN 奈良
ならけん #CN 奈良県
ならし #CN 奈良市
ならしの #CN 習志野
ならしのし #CN 習志野市
なりた #CN 成田
なりたし #CN 成田市
なります #CN 成増
なると #CN 鳴門
なるとし #CN 鳴門市
なるみ #CN 鳴海
なんき #CN 南紀
なんこく #CN 南国
なんこくし #CN 南国市
なんたいざん #CN 男体山
なんばしんち #CN 難波新地
なんぶせん #CN 南武線
なんべい #CN 南米
なんぺいだい #CN 南平台
なんよう #CN 南陽
なんようし #CN 南陽市
にあめー #CN ニアメー
にいがた #CN 新潟
にいがたけん #CN 新潟県
にいがたし #CN 新潟市
にいざ #CN 新座
にいざし #CN 新座市
にいじゅく #CN 新宿
にいつ #CN 新津
にいつし #CN 新津市
にいはま #CN 新居浜
にいはまし #CN 新居浜市
にいみ #CN 新見
にいみし #CN 新見市
にからぐあ #CN ニカラグア
にこしあ #CN ニコシア
にしあきる #CN 西秋留
にしお #CN 西尾
にしおおじ #CN 西大路
にしおし #CN 西尾市
にしきょうく #CN 西京区
にしく #CN 西区
にしさもあ #CN 西サモア
にしだい #CN 西台
にしどいつ #CN 西ドイツ
にしなりく #CN 西成区
にしにほん #CN 西日本
にしのおもて #CN 西之表
にしのおもてし #CN 西之表市
にしのみや #CN 西宮
にしのみやし #CN 西宮市
にしまごめ #CN 西馬込
にしよどがわく #CN 西淀川区
にしわき #CN 西脇
にしわきし #CN 西脇市
にじぇーる #CN ニジェール
にじゅうばしまえ #CN 二重橋前
にちえい #CN 日英
にちどく #CN 日独
にちなん #CN 日南
にちなんし #CN 日南市
にちふつ #CN 日仏
にちべい #CN 日米
にっかん #CN 日韓
にっこう #CN 日光
にっこうし #CN 日光市
にっそ #CN 日ソ
にっちゅう #CN 日中
にっぽり #CN 日暮里
にのへ #CN 二戸
にのへし #CN 二戸市
にほんかい #CN 日本海
にほんかいこう #CN 日本海溝
にほんばし #CN 日本橋
にほんまつ #CN 二本松
にほんまつし #CN 二本松市
にゅーおーりんす #CN ニューオーリンス
にゅーかれどにあ #CN ニューカレドニア
にゅーぎにあ #CN ニューギニア
にゅーじーらんど #CN ニュージーランド
にゅーじゃーじー #CN ニュージャージー
にゅーでりー #CN ニューデリー
にゅーはんぷしゃー #CN ニューハンプシャー
にゅーよーく #CN ニューヨーク
にらさき #CN 韮崎
にらさきし #CN 韮崎市
にんぎょうちょう #CN 人形町
ぬーめあ #CN ヌーメア
ぬあくしょっと #CN ヌアクショット
ぬくあろふぁ #CN ヌクアロファ
ぬじぇめな #CN ヌジェメナ
ぬのびき #CN 布引
ぬまた #CN 沼田
ぬまたし #CN 沼田市
ぬまづ #CN 沼津
ぬまづし #CN 沼津市
ねばだ #CN ネバダ
ねぱーる #CN ネパール
ねむろ #CN 根室
ねむろし #CN 根室市
ねやがわ #CN 寝屋川
ねやがわし #CN 寝屋川市
ねりま #CN 練馬
ねりまく #CN 練馬区
のうび #CN 濃尾
のおがた #CN 直方
のおがたし #CN 直方市
のぎざか #CN 乃木坂
のげ #CN 野毛
のしろ #CN 能代
のしろし #CN 能代市
のじりこ #CN 野尻湖
のだ #CN 野田
のだし #CN 野田市
のと #CN 能登
のべおか #CN 延岡
のべおかし #CN 延岡市
のぼりべつ #CN 登別
のぼりべつし #CN 登別市
のりくらだけ #CN 乗鞍岳
のるうぇー #CN ノルウェー
はいじま #CN 拝島
はいち #CN ハイチ
はかたく #CN 博多区
はぎ #CN 萩
はぎし #CN 萩市
はくい #CN 羽咋
はくいし #CN 羽咋市
はこざき #CN 箱崎
はこだて #CN 函館
はこだてし #CN 函館市
はしづめまち #CN 橋詰町
はしま #CN 羽島
はしまし #CN 羽島市
はしもと #CN 橋本
はしもとし #CN 橋本市
はすだ #CN 蓮田
はすだし #CN 蓮田市
はすぬま #CN 蓮沼
はすね #CN 蓮根
はだの #CN 秦野
はだのし #CN 秦野市
はちおうじ #CN 八王子
はちおうじし #CN 八王子市
はちこうせん #CN 八高線
はちじょうじま #CN 八丈島
はちのへ #CN 八戸
はちのへし #CN 八戸市
はちまん #CN 八幡
はちまんちょう #CN 八幡町
はっこうださん #CN 八甲田山
はっちょうぼり #CN 八丁堀
はとがや #CN 鳩ケ谷
はとがやし #CN 鳩ケ谷市
はなぞの #CN 花園
はなてん #CN 放出
はなはた #CN 花畑
はなまき #CN 花巻
はなまきし #CN 花巻市
はにゅう #CN 羽生
はにゅうし #CN 羽生市
はねだ #CN 羽田
はねだくうこう #CN 羽田空港
はのい #CN ハノイ
はばな #CN ハバナ
はびきの #CN 羽曳野
はびきのし #CN 羽曳野市
はぼまい #CN 歯舞
はまきた #CN 浜北
はまきたし #CN 浜北市
はまだ #CN 浜田
はまだし #CN 浜田市
はままち #CN 浜町
はままつ #CN 浜松
はままつし #CN 浜松市
はままつちょう #CN 浜松町
はまりきゅう #CN 浜離宮
はらじゅく #CN 原宿
はらまち #CN 原町
はらまちし #CN 原町市
はるつーむ #CN ハルツーム
はるみ #CN 晴海
はわい #CN ハワイ
はんがりー #CN ハンガリー
はんしん #CN 阪神
はんぞうもん #CN 半蔵門
はんだ #CN 半田
はんだし #CN 半田市
はんのう #CN 飯能
はんのうし #CN 飯能市
ばーみんがむ #CN バーミンガム
ばーれーん #CN バーレーン
ばくー #CN バクー
ばくろ #CN 馬喰
ばくろうまち #CN 博労町
ばくろよこやま #CN 馬喰横山
ばぐだっど #CN バグダッド
ばちかん #CN バチカン
ばぬあつ #CN バヌアツ
ばはま #CN バハマ
ばまこ #CN バマコ
ばみゅーだ #CN バミューダ
ばらきなかやま #CN 原木中山
ばり #CN バリ
ばるばどす #CN バルバドス
ばれった #CN バレッタ
ばんぎ #CN バンギ
ばんくーばー #CN バンクーバー
ばんぐらでしゅ #CN バングラデシュ
ばんこく #CN バンコク
ばんじゅる #CN バンジュル
ばんりのちょうじょう #CN 万里長城
ぱきすたん #CN パキスタン
ぱなま #CN パナマ
ぱぷあにゅーぎにあ #CN パプアニューギニア
ぱらぐあい #CN パラグアイ
ぱらまりぼ #CN パラマリボ
ぱり #CN パリ
ひえいざん #CN 比叡山
ひかり #CN 光
ひかりし #CN 光市
ひがしおおさか #CN 東大阪
ひがしおおさかし #CN 東大阪市
ひがしく #CN 東区
ひがしくるめ #CN 東久留米
ひがしくるめし #CN 東久留米市
ひがししなかい #CN 東シナ海
ひがしすな #CN 東砂
ひがしすみよしく #CN 東住吉区
ひがしどいつ #CN 東ドイツ
ひがしなだく #CN 東灘区
ひがしなりく #CN 東成区
ひがしにほん #CN 東日本
ひがしね #CN 東根
ひがしねし #CN 東根市
ひがしひろしま #CN 東広島
ひがしひろしまし #CN 東広島市
ひがしまつやま #CN 東松山
ひがしまつやまし #CN 東松山市
ひがしむらやま #CN 東村山
ひがしむらやまし #CN 東村山市
ひがしやまく #CN 東山区
ひがしやまと #CN 東大和
ひがしやまとし #CN 東大和市
ひがしよどがわく #CN 東淀川区
ひこうじょう #CN 飛行場
ひこね #CN 彦根
ひこねし #CN 彦根市
ひさい #CN 久居
ひさいし #CN 久居市
ひた #CN 日田
ひたし #CN 日田市
ひたち #CN 常陸 日立
ひたちおおた #CN 常陸太田
ひたちおおたし #CN 常陸太田市
ひたちし #CN 日立市
ひだ #CN 飛騨
ひとつばし #CN 一ツ橋
ひとよし #CN 人吉
ひとよしし #CN 人吉市
ひの #CN 日野
ひのし #CN 日野市
ひびや #CN 日比谷
ひみ #CN 氷見
ひみし #CN 氷見市
ひめじ #CN 姫路
ひめじし #CN 姫路市
ひゃくにんちょう #CN 百人町
ひゅーろんこ #CN ヒューロン湖
ひゅうが #CN 日向
ひゅうがし #CN 日向市
ひょうご #CN 兵庫
ひょうごく #CN 兵庫区
ひょうごけん #CN 兵庫県
ひょうのせん #CN 氷ノ山
ひらかた #CN 枚方
ひらかたし #CN 枚方市
ひらた #CN 平田
ひらたし #CN 平田市
ひらつか #CN 平塚
ひらつかし #CN 平塚市
ひらど #CN 平戸
ひらどし #CN 平戸市
ひらの #CN 平野
ひらのく #CN 平野区
ひらら #CN 平良
ひららし #CN 平良市
ひろお #CN 広尾
ひろこうじ #CN 広小路
ひろさき #CN 弘前
ひろさきし #CN 弘前市
ひろしま #CN 広島
ひろしまけん #CN 広島県
ひろしまし #CN 広島市
びえんちゃん #CN ビエンチャン
びきに #CN ビキニ
びきにかんしょう #CN ビキニ環礁
びさい #CN 尾西
びさいし #CN 尾西市
びさう #CN ビサウ
びぜん #CN 備前
びぜんし #CN 備前市
びばい #CN 美唄
びばいし #CN 美唄市
びら #CN ビラ
びらしすねろす #CN ビラシスネロス
びりにゅす #CN ビリニュス
びるま #CN ビルマ
びわこ #CN 琵琶湖
びわしま #CN 枇杷島
びんご #CN 備後
ふぁどーつ #CN ファドーツ
ふぃじー #CN フィジー
ふぃらでるふぃあ #CN フィラデルフィア
ふぃりぴん #CN フィリピン
ふぃれんつぇ #CN フィレンツェ
ふぃんらんど #CN フィンランド
ふかや #CN 深谷
ふかやし #CN 深谷市
ふくい #CN 福井
ふくいけん #CN 福井県
ふくいし #CN 福井市
ふくえ #CN 福江
ふくえし #CN 福江市
ふくおか #CN 福岡
ふくおかけん #CN 福岡県
ふくおかし #CN 福岡市
ふくしま #CN 福島
ふくしまく #CN 福島区
ふくしまけん #CN 福島県
ふくしまし #CN 福島市
ふくちやま #CN 福知山
ふくちやまし #CN 福知山市
ふくやま #CN 福山
ふくやまし #CN 福山市
ふくろい #CN 袋井
ふくろいし #CN 袋井市
ふしみ #CN 伏見
ふしみく #CN 伏見区
ふじ #CN 富士
ふじいでら #CN 藤井寺
ふじいでらし #CN 藤井寺市
ふじえだ #CN 藤枝
ふじえだし #CN 藤枝市
ふじおか #CN 藤岡
ふじおかし #CN 藤岡市
ふじさわ #CN 藤沢
ふじさわし #CN 藤沢市
ふじさん #CN 富士山
ふじし #CN 富士市
ふじのみや #CN 富士宮
ふじのみやし #CN 富士宮市
ふじみ #CN 富士見
ふじみし #CN 富士見市
ふじよしだ #CN 富士吉田
ふじよしだし #CN 富士吉田市
ふたばちょう #CN 双葉町
ふたまたがわ #CN 二俣川
ふちゅう #CN 府中
ふちゅうし #CN 府中市
ふっこく #CN 仏国
ふっさ #CN 福生
ふっさし #CN 福生市
ふっつ #CN 富津
ふっつし #CN 富津市
ふなばし #CN 船橋
ふなばしし #CN 船橋市
ふなふち #CN フナフチ
ふらの #CN 富良野
ふらのし #CN 富良野市
ふらんす #CN フランス
ふりーたうん #CN フリータウン
ふるかわ #CN 古川
ふるかわし #CN 古川市
ふるんぜ #CN フルンゼ
ぶーたん #CN ブータン
ぶえのすあいれす #CN ブエノスアイレス
ぶかれすと #CN ブカレスト
ぶじゅんぶら #CN ブジュンブラ
ぶぜん #CN 豊前
ぶぜんし #CN 豊前市
ぶだぺすと #CN ブダペスト
ぶばいがわら #CN 分倍河原
ぶらざびる #CN ブラザビル
ぶらじりあ #CN ブラジリア
ぶらじる #CN ブラジル
ぶりすべーん #CN ブリスベーン
ぶりっじたうん #CN ブリッジタウン
ぶりゅっせる #CN ブリュッセル
ぶるがりあ #CN ブルガリア
ぶるねい #CN ブルネイ
ぶるんじ #CN ブルンジ
ぶんきょうく #CN 文京区
ぶんごたかだ #CN 豊後高田
ぶんごたかだし #CN 豊後高田市
ぷえるとりこ #CN プエルトリコ
ぷのんぺん #CN プノンペン
ぷらいあ #CN プライア
ぷらは #CN プラハ
ぷれとりあ #CN プレトリア
へいわじま #CN 平和島
へいわだい #CN 平和台
へきなん #CN 碧南
へきなんし #CN 碧南市
へるしんき #CN ヘルシンキ
べいこく #CN アメリカ合衆国 米国
べいるーと #CN ベイルート
べおぐらーど #CN ベオグラード
べっぷ #CN 別府
べっぷし #CN 別府市
べついん #CN 別院
べとなむ #CN ベトナム
べにん #CN ベニン
べねずえら #CN ベネズエラ
べねちあ #CN ベネチア
べらう #CN ベラウ
べるぎー #CN ベルギー
べるさいゆ #CN ベルサイユ
べるりん #CN ベルリン
べるん #CN ベルン
べんてん #CN 弁天
ぺきん #CN 北京
ぺるー #CN ペルー
ぺるしゃ #CN ペルシャ
ぺるしゃわん #CN ペルシャ湾
ぺんしるべにあ #CN ペンシルベニア
ほうじょう #CN 北条
ほうじょうし #CN 北条市
ほうふ #CN 防府
ほうふし #CN 防府市
ほうや #CN 保谷
ほうやし #CN 保谷市
ほくべい #CN 北米
ほくりく #CN 北陸
ほだかだけ #CN 穂高岳
ほっかい #CN 北海
ほっかいどう #CN 北海道
ほっかいどうちほう #CN 北海道地方
ほった #CN 堀田
ほどがやく #CN 保土ケ谷区
ほにあら #CN ホニアラ
ほのるる #CN ホノルル
ほりえ #CN 堀江
ほりきり #CN 堀切
ほりどめ #CN 堀留
ほろひらはし #CN 幌平橋
ほんがんじ #CN 本願寺
ほんこん #CN 香港
ほんごく #CN 本石
ほんざん #CN 本山
ほんしゅう #CN 本州
ほんじゅらす #CN ホンジュラス
ほんじょ #CN 本所
ほんじょう #CN 本庄 本荘
ほんじょうし #CN 本庄市 本荘市
ほんじん #CN 本陣
ほんど #CN 本渡
ほんどおり #CN 本通り
ほんどし #CN 本渡市
ほんまち #CN 本町
ぼうしゅう #CN 房州
ぼうそう #CN 房総
ぼごた #CN ボゴタ
ぼつわな #CN ボツワナ
ぼりびあ #CN ボリビア
ぼん #CN ボン
ぽーつます #CN ポーツマス
ぽーとおぶすぺいん #CN ポートオブスペイン
ぽーともれすびー #CN ポートモレスビー
ぽーとるいす #CN ポートルイス
ぽーらんど #CN ポーランド
ぽつだむ #CN ポツダム
ぽるとーぷらんす #CN ポルトーブランス
ぽるとがる #CN ポルトガル
ぽるとのぼ #CN ポルトノボ
まいづる #CN 舞鶴
まいづるし #CN 舞鶴市
まうい #CN マウイ
まういとう #CN マウイ島
まえばし #CN 前橋
まえばしし #CN 前橋市
まかお #CN マカオ
まかっさる #CN マカッサル
まかっさるかいきょう #CN マカッサル海峡
まくらざき #CN 枕崎
まくらざきし #CN 枕崎市
まごめ #CN 馬込
まさちゅーせっつ #CN マサチューセッツ
ましゅうこ #CN 摩周湖
ますかっと #CN マスカット
ますだ #CN 益田
ますだし #CN 益田市
ませる #CN マセル
まぜらんかいきょう #CN マゼラン海峡
まだがすかる #CN マダガスカル
まちだ #CN 町田
まちだし #CN 町田市
まちや #CN 町屋
まっきんりー #CN マッキンリー
まったーほーん #CN マッターホーン
まつうら #CN 松浦
まつうらし #CN 松浦市
まつえ #CN 松江
まつえし #CN 松江市
まつさか #CN 松阪
まつさかし #CN 松阪市
まつとう #CN 松任
まつとうし #CN 松任市
まつど #CN 松戸
まつどし #CN 松戸市
まつばら #CN 松原
まつばらし #CN 松原市
まつもと #CN 松本
まつもとし #CN 松本市
まつやま #CN 松山
まつやまし #CN 松山市
まどりーど #CN マドリード
まなぐあ #CN マナグア
まなま #CN マナマ
まにら #CN マニラ
まぷと #CN マプト
まみや #CN 間宮
まらうぃ #CN マラウィ
まらぼ #CN マラボ
まり #CN マリ
まるがめ #CN 丸亀
まるがめし #CN 丸亀市
まるこ #CN 丸子
まるた #CN マルタ
まるたまち #CN 丸太町
まるのうち #CN 丸ノ内
まるのうちせん #CN 丸ノ内線
まれーしあ #CN マレーシア
みうら #CN 三浦
みうらし #CN 三浦市
みえ #CN 三重
みえけん #CN 三重県
みかさ #CN 三笠
みかさし #CN 三笠市
みかわ #CN 三河
みかわしま #CN 三河島
みき #CN 三木
みきし #CN 三木市
みくに #CN 三国
みくにとうげ #CN 三国峠
みさと #CN 三郷
みさとし #CN 三郷市
みさわ #CN 三沢
みさわし #CN 三沢市
みしがん #CN ミシガン
みししっぴ #CN ミシシッピ
みしま #CN 三島
みしまし #CN 三島市
みずーり #CN ミズーリ
みずさわ #CN 水沢
みずさわし #CN 水沢市
みずなみ #CN 瑞浪
みずなみし #CN 瑞浪市
みずほく #CN 瑞穂区
みぞのくち #CN 溝ノ口
みた #CN 三田
みたか #CN 三鷹
みたかし #CN 三鷹市
みっどうぇー #CN ミッドウェー
みつかいどう #CN 水海道
みつかいどうし #CN 水海道市
みつけ #CN 見附
みつけし #CN 見附市
みと #CN 水戸
みとし #CN 水戸市
みとしろ #CN 美土代
みどうすじ #CN 御堂筋
みどりがおか #CN 緑が丘
みどりく #CN 緑区
みなとく #CN 港区
みなまた #CN 水俣
みなまたし #CN 水俣市
みなみあしがら #CN 南足柄
みなみあしがらし #CN 南足柄市
みなみかんとう #CN 南関東
みなみきゅうしゅう #CN 南九州
みなみく #CN 南区
みなみにほん #CN 南日本
みね #CN 美禰
みねし #CN 美禰市
みの #CN 美濃
みのお #CN 箕面
みのおし #CN 箕面市
みのかも #CN 美濃加茂
みのかもし #CN 美濃加茂市
みのし #CN 美濃市
みのしま #CN 蓑島
みはら #CN 三原
みはらし #CN 三原市
みやぎ #CN 宮城
みやぎけん #CN 宮城県
みやこ #CN 宮古
みやこし #CN 宮古市
みやこじま #CN 都島
みやこじまく #CN 都島区
みやこのじょう #CN 都城
みやこのじょうし #CN 都城市
みやざき #CN 宮崎
みやざきけん #CN 宮崎県
みやざきし #CN 宮崎市
みやづ #CN 宮津
みやづし #CN 宮津市
みやまえく #CN 宮前区
みゅんへん #CN ミュンヘン
みょうがだに #CN 茗荷谷
みょうぎさん #CN 妙義山
みよし #CN 三次
みよしし #CN 三次市
みらの #CN ミラノ
みるうぉーきー #CN ミルウォーキー
みんすく #CN ミンスク
みんだなお #CN ミンダナオ
むこう #CN 向日
むこうし #CN 向日市
むこがわ #CN 武庫川
むさし #CN 武蔵
むさしこすぎ #CN 武蔵小杉
むさしこやま #CN 武蔵小山
むさしさかい #CN 武蔵境
むさしの #CN 武蔵野
むさしのし #CN 武蔵野市
むさしむらやま #CN 武蔵村山
むさしむらやまし #CN 武蔵村山市
むつし #CN むつ市
むなかた #CN 宗像
むなかたし #CN 宗像市
むばばね #CN ムババネ
むらかみ #CN 村上
むらかみし #CN 村上市
むらやま #CN 村山
むらやまし #CN 村山市
むろと #CN 室戸
むろとし #CN 室戸市
むろらん #CN 室蘭
むろらんし #CN 室蘭市
めーん #CN メーン
めいじじんぐう #CN 明治神宮
めいとうく #CN 名東区
めきしこ #CN メキシコ
めぐろ #CN 目黒
めぐろく #CN 目黒区
めじろ #CN 目白
めじろだい #CN 目白台
めっか #CN メッカ
めるぼるん #CN メルボルン
めんどさ #CN メンドサ
もーりしゃす #CN モーリシャス
もーりたらにあ #CN モーリタニア
もうこ #CN 蒙古
もおか #CN 真岡
もおかし #CN 真岡市
もか #CN モカ
もがじしお #CN モガジシオ
もざんびーく #CN モザンビーク
もじく #CN 門司区
もすくわ #CN モスクワ
もとすこ #CN 本栖湖
もとまち #CN 元町
もなこ #CN モナコ
もばら #CN 茂原
もばらし #CN 茂原市
もりおか #CN 盛岡
もりおかし #CN 盛岡市
もりぐち #CN 守口
もりぐちし #CN 守口市
もりのみや #CN 森之宮
もりやま #CN 守山
もりやまく #CN 守山区
もりやまし #CN 守山市
もるじぶ #CN モルジブ
もるだびあ #CN モルダビア
もろっこ #CN モロッコ
もんごる #CN モンゴル
もんぜんなかちょう #CN 門前仲町
もんてびでお #CN モンテビデオ
もんとりおーる #CN モントリオール
もんばさ #CN モンバサ
もんべつ #CN 紋別
もんべつし #CN 紋別市
もんろびあ #CN モンロビア
やいた #CN 矢板
やいたし #CN 矢板市
やいづ #CN 焼津
やいづし #CN 焼津市
やうんで #CN ヤウンデ
やえす #CN 八重州
やえすぐち #CN 八重州口
やお #CN 八尾
やおし #CN 八尾市
やくし #CN 薬師
やしお #CN 八潮
やしおし #CN 八潮市
やすぎ #CN 安来
やすぎし #CN 安来市
やすくに #CN 靖国
やちよ #CN 八千代
やちよし #CN 八千代市
やつがたけ #CN 八ケ岳
やつしろ #CN 八代
やつしろし #CN 八代市
やない #CN 柳井
やないし #CN 柳井市
やながわ #CN 柳川
やながわし #CN 柳川市
やはたにしく #CN 八幡西区
やはたひがしく #CN 八幡東区
やまが #CN 山鹿
やまがし #CN 山鹿市
やまがた #CN 山形
やまがたけん #CN 山形県
やまがたし #CN 山形市
やまぐち #CN 山口
やまぐちけん #CN 山口県
やまぐちし #CN 山口市
やましなく #CN 山科区
やまだ #CN 山田
やまだし #CN 山田市
やまて #CN 山手
やまと #CN 大和
やまとこおりやま #CN 大和郡山
やまとこおりやまし #CN 大和郡山市
やまとし #CN 大和市
やまとたかだ #CN 大和高田
やまとたかだし #CN 大和高田市
やまなかこ #CN 山中湖
やまなし #CN 山梨
やまなしけん #CN 山梨県
やまなしし #CN 山梨市
やまのて #CN 山手
やめ #CN 八女
やめし #CN 八女市
やよい #CN 弥生
やりがたけ #CN 槍ケ岳
やわた #CN 八幡
やわたし #CN 八幡市
やわたはま #CN 八幡浜
やわたはまし #CN 八幡浜市
ゆーごすらびあ #CN ユーゴスラビア
ゆーふらてす #CN ユーフラテス
ゆうき #CN 結城
ゆうきし #CN 結城市
ゆうばり #CN 夕張
ゆうばりし #CN 夕張市
ゆうべつがわ #CN 湧別川
ゆうらくちょう #CN 有楽町
ゆうりょうどうろ #CN 有料道路
ゆがわら #CN 湯河原
ゆきがや #CN 雪ケ谷 雪谷
ゆくはし #CN 行橋
ゆくはしし #CN 行橋市
ゆざわ #CN 湯沢
ゆざわし #CN 湯沢市
ゆしま #CN 湯島
ゆた #CN ユタ
ゆめのしま #CN 夢の島
よーくしゃー #CN ヨークシャー
よーろっぱ #CN ヨーロッパ
ようかいち #CN 八日市
ようかいちし #CN 八日市市
ようかいちば #CN 八日市場
ようかいちばし #CN 八日市場市
ようこうだい #CN 洋光台
ようていざん #CN 羊蹄山
よこすか #CN 横須賀
よこすかし #CN 横須賀市
よこて #CN 横手
よこてし #CN 横手市
よこはま #CN 横浜
よこはまし #CN 横浜市
よこやま #CN 横山
よっかいち #CN 四日市
よっかいちし #CN 四日市市
よつぎ #CN 四ツ木
よつばし #CN 四ツ橋
よどがわく #CN 淀川区
よどやばし #CN 淀屋橋
よなご #CN 米子
よなごし #CN 米子市
よなしろがわ #CN 米代川
よねざわ #CN 米沢
よねざわし #CN 米沢市
よの #CN 与野
よのし #CN 与野市
よよぎ #CN 代々木
よるだん #CN ヨルダン
らおす #CN ラオス
らごす #CN ラゴス
らさ #CN ラサ
らすべがす #CN ラスベガス
らとびあ #CN ラトビア
らばと #CN ラバト
らぱす #CN ラパス
らんかしゃー #CN ランカシャー
らんぐーん #CN ラングーン
りーぶるびる #CN リーブルビル
りが #CN リガ
りくう #CN 陸羽
りくぜんたかた #CN 陸前高田
りくぜんたかたし #CN 陸前高田市
りすぼん #CN リスボン
りとあにあ #CN リトアニア
りひてんしゅたいん #CN リヒテンシュタイン
りびあ #CN リビア
りべりあ #CN リベリア
りま #CN リマ
りやど #CN リヤド
りゅうがさき #CN 竜ケ崎
りゅうがさきし #CN 竜ケ崎市
りゅうきゅう #CN 琉球
りょうがえまち #CN 両替町
りょうごく #CN 両国
りょうつ #CN 両津
りょうつし #CN 両津市
りろんぐえ #CN リロングエ
るーまにあ #CN ルーマニア
るあんだ #CN ルアンダ
るくせんぶるく #CN ルクセンブルク
るさか #CN ルサカ
るもい #CN 留萌
るもいし #CN 留萌市
るわんだ #CN ルワンダ
れいきゃびく #CN レイキャビク
れそと #CN レソト
ればのん #CN レバノン
れゆにおん #CN レユニオン
ろーま #CN ローマ
ろさんぜるす #CN ロサンゼルス
ろしあ #CN ロシア
ろぞー #CN ロゾー
ろめ #CN ロメ
ろんどん #CN ロンドン
わいおみんぐ #CN ワイオミング
わかまつく #CN 若松区
わかやま #CN 和歌山
わかやまけん #CN 和歌山県
わかやまし #CN 和歌山市
わがどぐ #CN ワガドグ
わこう #CN 和光
わこうし #CN 和光市
わしんとん #CN ワシントン
わじま #CN 輪島
わじまし #CN 輪島市
わせだ #CN 早稲田
わたらせがわ #CN 渡良瀬川
わっかない #CN 稚内
わっかないし #CN 稚内市
わらび #CN 蕨
わらびし #CN 蕨市
わるしゃわ #CN ワルシャワ
わんがん #CN 湾岸
