# Copyright 1994 NEC Corporation, Tokyo, Japan.
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without
# fee, provided that the above copyright notice appear in all copies
# and that both that copyright notice and this permission notice
# appear in supporting documentation, and that the name of NEC
# Corporation not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.  NEC Corporation makes no representations about the
# suitability of this software for any purpose.  It is provided "as
# is" without express or implied warranty.
#
# NEC CORPORATION DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN 
# NO EVENT SHALL NEC CORPORATION BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR 
# OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
# PERFORMANCE OF THIS SOFTWARE. 
#
# $Id: hojoswd.t,v 1.35 1994/06/01 06:45:14 kuma Exp $
い #Zi @ #ktok5iO @
か #Zka @
か #ktec5a @ #ktok5a @
かい #Zkai @
かしら #Zkasira @
かな #Zkana @
がた #yasui @
き #ktok5i @
きゃ #ktec5ya @ #ktok5ya @ #kykya @ #k5ya @ #c5rya @
きゃあ #ktec5ya @ #ktok5ya @ #kykya @ #k5ya @ #c5rya @
ぎゃ #g5ya @
ぎゃあ #g5ya @
く #ktec5u @ #ktok5u @
くらい #FKN @
くりゃ #kxya @
くん #JNSUC 君 くん
ぐらい #FKN @
け #ktec5e @ #ktok5e @
けど #Skeredo けど
けども #Skeredo @
けりゃ #kykya @
こ #ktec5o @ #ktok5o @
こそ #FKN @
さ #Zsa @
さし #sasiru さし
さま #JNSUC 様 さま
さん #JNSUC さん
し #siru @ #JNSUC 氏 #k5siru @
しゃ #s5ya @
しゃあ #s5ya @
すりゃ #sxya @
すりゃあ #sxya @
ずりゃ #zxya @
ずりゃあ #zxya @
ぜ #Zze @
ぞ #Zzo @
たって #ktemo @
だけ #FKN @
ちゃ #t5ya @
ちゃあ #t5ya @
ちゃん #JNSUC ちゃん
っ #ktec5tO @
っけ #kke @
ったって #Fttatte @
ったら #Fttara @
ってば #Fttara @
っぱなし #D2T35 っぱなし
づら #yasui @
と #kto @
とも #Ztomo @
ど #kdo @
どの #JNSUC 殿 どの
な #Zna @
なあ #Znaa @
など #FKN @
にゃ #n5ya @
にゃ #naiya @
にゃあ #n5ya @
にゃあ #naiya @
にく #yasui @
ね #Zne @
ねえ #Znee @
ねえ #nainee @
の #Zno @
びゃ #b5ya @
びゃあ #b5ya @
みゃ #m5ya @
みゃあ #m5ya @
もの #Zmono @
もん #Zmono @
や #Zya @
やす #yasui @
ゆえ #FKN @
よ #Zyo @ #yasui @
よな #Zyona @
よなあ #Zyona @
りゃ #r5ya @ #ksya @
りゃあ #r5ya @ #ksya @
れ #rareru れ
わ #Zwa @
ん #nn @
ん #r5nO @
ん #JTNN @
