# $Id: yuubin.t,v 1.3 1993/08/10 05:18:09 kon Exp $
001 #CN 北海道札幌市北区
002 #CN 北海道札幌市北区 北海道石狩郡
003 #CN 北海道札幌市白石区
004 #CN 北海道札幌市白石区 北海道札幌市豊平区
005 #CN 北海道札幌市南区
006 #CN 北海道札幌市西区
010 #CN 秋田県河辺郡 秋田県秋田市 秋田県男鹿市 秋田県南秋田郡
011 #CN 秋田県秋田市
012 #CN 秋田県湯沢市 秋田県由利郡 秋田県雄勝郡
013 #CN 秋田県横手市 秋田県平鹿郡 秋田県雄勝郡
014 #CN 秋田県仙北郡 秋田県大曲市 秋田県平鹿郡
015 #CN 秋田県本荘市 秋田県由利郡
016 #CN 秋田県能代市
017 #CN 秋田県鹿角郡 秋田県大館市
018 #CN 秋田県山本郡 秋田県鹿角郡 秋田県鹿角市 秋田県大館市 秋田県南秋田郡 秋田県能代市 秋田県北秋田郡 秋田県本荘市 秋田県由利郡 青森県上北郡
019 #CN 秋田県横手市 秋田県河辺郡 秋田県仙北郡 秋田県湯沢市 秋田県平鹿郡 秋田県雄勝郡
020 #CN 岩手県岩手郡 岩手県紫波郡 岩手県盛岡市
021 #CN 岩手県一関市
022 #CN 岩手県気仙郡 岩手県大船渡市
023 #CN 岩手県江刺市 岩手県水沢市 岩手県胆沢郡
024 #CN 岩手県北上市 岩手県和賀郡
025 #CN 岩手県花巻市
026 #CN 岩手県釜石市
027 #CN 岩手県下閉伊郡 岩手県宮古市
028 #CN 岩手県遠野市 岩手県下閉伊郡 岩手県岩手郡 岩手県九戸郡 岩手県紫波郡 岩手県上閉伊郡 岩手県二戸郡 岩手県二戸市 岩手県稗貫郡 岩手県和賀郡
029 #CN 岩手県一関市 岩手県気仙郡 岩手県西磐井郡 岩手県胆沢郡 岩手県東磐井郡 岩手県陸前高田市 岩手県和賀郡
030 #CN 青森県青森市 青森県東津軽郡 青森県北津軽郡
031 #CN 青森県三戸郡 青森県八戸市
032 #CN 岩手県下閉伊郡 岩手県久慈市 岩手県九戸郡
033 #CN 青森県三沢市 青森県上北郡
034 #CN 青森県十和田市 青森県上北郡
035 #CN 青森県むつ市 青森県下北郡
036 #CN 青森県弘前市 青森県黒石市 青森県中津軽郡 青森県南津軽郡
037 #CN 青森県五所川原市 青森県西津軽郡 青森県北津軽郡
038 #CN 青森県弘前市 青森県西津軽郡 青森県南津軽郡 青森県北津軽郡
039 #CN 岩手県久慈市 岩手県九戸郡 青森県むつ市 青森県下北郡 青森県三戸郡 青森県上北郡 青森県青森市 青森県東津軽郡 青森県八戸市
040 #CN 北海道函館市
041 #CN 北海道茅部郡 北海道亀田郡 北海道函館市
042 #CN 北海道函館市
043 #CN 北海道奥尻郡 北海道久遠郡 北海道爾志郡 北海道桧山郡
044 #CN 北海道虻田郡 北海道有珠郡
045 #CN 北海道岩内郡 北海道古宇郡
046 #CN 北海道古平郡 北海道積丹郡 北海道余市郡
047 #CN 北海道小樽市 北海道余市郡
048 #CN 北海道虻田郡 北海道磯谷郡 北海道岩内郡 北海道寿都郡 北海道小樽市 北海道島牧郡 北海道余市郡
049 #CN 北海道虻田郡 北海道磯谷郡 北海道茅部郡 北海道亀田郡 北海道山越郡 北海道松前郡 北海道上磯郡 北海道瀬棚郡 北海道島牧郡 北海道桧山郡 北海道有珠郡
050 #CN 北海道室蘭市
051 #CN 北海道室蘭市
052 #CN 北海道伊達市 北海道有珠郡
053 #CN 北海道苫小牧市
054 #CN 北海道勇払郡
055 #CN 北海道沙流郡
056 #CN 北海道静内郡
057 #CN 北海道浦河郡
058 #CN 北海道浦河郡 北海道幌泉郡 北海道様似郡
059 #CN 北海道伊達市 北海道浦河郡 北海道沙流郡 北海道三石郡 北海道新冠郡 北海道静内郡 北海道登別市 北海道苫小牧市 北海道白老郡 北海道勇払郡 北海道夕張郡
060 #CN 北海道札幌市中央区 北海道札幌市東区 北海道札幌市北区
061 #CN 北海道樺戸郡 北海道恵庭市 北海道厚田郡 北海道札幌郡 北海道札幌市南区 北海道小樽市 北海道石狩郡
062 #CN 北海道札幌市豊平区
063 #CN 北海道札幌市西区
064 #CN 北海道札幌市中央区
065 #CN 北海道札幌市東区
066 #CN 北海道千歳市 北海道苫小牧市
067 #CN 北海道江別市
068 #CN 北海道樺戸郡 北海道岩見沢市 北海道空知郡 北海道三笠市 北海道石狩郡 北海道美唄市 北海道夕張郡 北海道夕張市
069 #CN 北海道岩見沢市 北海道空知郡 北海道江別市 北海道千歳市 北海道夕張郡
070 #CN 北海道旭川市
071 #CN 北海道旭川市 北海道空知郡 北海道上川郡
072 #CN 北海道美唄市
073 #CN 北海道歌志内市 北海道樺戸郡 北海道空知郡 北海道厚田郡 北海道砂川市 北海道滝川市 北海道浜益郡
074 #CN 北海道旭川市 北海道雨竜郡 北海道深川市
075 #CN 北海道芦別市
076 #CN 北海道芦別市 北海道富良野市
077 #CN 北海道増毛郡 北海道浜益郡 北海道留萌郡 北海道留萌市
078 #CN 北海道旭川市 北海道雨竜郡 北海道樺戸郡 北海道上川郡 北海道深川市 北海道苫前郡 北海道留萌郡 北海道留萌市
079 #CN 北海道旭川市 北海道芦別市 北海道雨竜郡 北海道岩見沢市 北海道空知郡 北海道沙流郡 北海道砂川市 北海道赤平市 北海道滝川市 北海道美唄市 北海道富良野市 北海道勇払郡
080 #CN 北海道河東郡 北海道帯広市
081 #CN 北海道河東郡 北海道上川郡
082 #CN 北海道河西郡
083 #CN 北海道中川郡
084 #CN 北海道釧路市
085 #CN 北海道阿寒郡 北海道釧路郡 北海道釧路市 北海道川上郡
086 #CN 北海道厚岸郡 北海道根室市 北海道標津郡 北海道目梨郡 北海道野付郡
087 #CN 北海道根室市
088 #CN 北海道釧路郡 北海道釧路市 北海道厚岸郡 北海道根室市 北海道十勝郡 北海道川上郡 北海道白糠郡 北海道標津郡 北海道野付郡
089 #CN 北海道河西郡 北海道広尾郡 北海道十勝郡 北海道上川郡 北海道足寄郡 北海道帯広市 北海道中川郡
090 #CN 北海道常呂郡 北海道北見市
091 #CN 北海道常呂郡
092 #CN 北海道網走郡
093 #CN 北海道常呂郡 北海道網走市 北海道紋別郡
094 #CN 北海道紋別市
095 #CN 北海道士別市 北海道上川郡
096 #CN 北海道名寄市
097 #CN 北海道稚内市 北海道利尻郡 北海道礼文郡
098 #CN 北海道士別市 北海道枝幸郡 北海道宗谷郡 北海道上川郡 北海道稚内市 北海道中川郡 北海道天塩郡 北海道苫前郡 北海道名寄市 北海道紋別郡
099 #CN 北海道斜里郡 北海道常呂郡 北海道北見市 北海道網走郡 北海道網走市 北海道紋別郡 北海道紋別市
100 #CN 東京都御蔵島村 東京都三宅村 東京都小笠原村 東京都新島本村 東京都神津島村 東京都青ケ島村 東京都千代田区 東京都大島町 東京都八丈町 東京都利島村
101 #CN 東京都千代田区
102 #CN 東京都千代田区
103 #CN 東京都中央区
104 #CN 東京都中央区
105 #CN 東京都港区
106 #CN 東京都港区
107 #CN 東京都港区
108 #CN 東京都港区
109 #CN 東京都港区
110 #CN 東京都台東区
111 #CN 東京都台東区
112 #CN 東京都文京区
113 #CN 東京都文京区
114 #CN 東京都北区
115 #CN 東京都北区
116 #CN 東京都荒川区
120 #CN 東京都足立区
121 #CN 東京都足立区
123 #CN 東京都足立区
124 #CN 東京都葛飾区
125 #CN 東京都葛飾区
130 #CN 東京都墨田区
131 #CN 東京都墨田区
132 #CN 東京都江戸川区
133 #CN 東京都江戸川区
134 #CN 東京都江戸川区
135 #CN 東京都江東区 東京都港区 東京都品川区
136 #CN 東京都江東区
140 #CN 東京都品川区
141 #CN 東京都品川区
142 #CN 東京都品川区
143 #CN 東京都大田区
144 #CN 東京都大田区
145 #CN 東京都大田区
146 #CN 東京都大田区
149 #CN 東京都大田区
150 #CN 東京都渋谷区
151 #CN 東京都渋谷区
152 #CN 東京都目黒区
153 #CN 東京都目黒区
154 #CN 東京都世田谷区
155 #CN 東京都世田谷区
156 #CN 東京都世田谷区
157 #CN 東京都世田谷区
158 #CN 東京都世田谷区
160 #CN 東京都新宿区
161 #CN 東京都新宿区
162 #CN 東京都新宿区
163 #CN 東京都新宿区
164 #CN 東京都中野区
165 #CN 東京都中野区
166 #CN 東京都杉並区
167 #CN 東京都杉並区
168 #CN 東京都杉並区
169 #CN 東京都新宿区
170 #CN 東京都豊島区
171 #CN 東京都豊島区
173 #CN 東京都板橋区
174 #CN 東京都板橋区
175 #CN 東京都板橋区
176 #CN 東京都練馬区
177 #CN 東京都練馬区
180 #CN 東京都武蔵野市
181 #CN 東京都三鷹市
182 #CN 東京都調布市
183 #CN 東京都府中市
184 #CN 東京都小金井市
185 #CN 東京都国分寺市
186 #CN 東京都国立市
187 #CN 東京都小平市
188 #CN 東京都田無市
189 #CN 東京都東村山市 東京都東大和市
190 #CN 東京都西多摩郡 東京都武蔵村山市 東京都立川市
191 #CN 東京都日野市
192 #CN 東京都八王子市
193 #CN 東京都八王子市
194 #CN 東京都町田市
196 #CN 東京都昭島市
197 #CN 東京都秋川市 東京都福生市
198 #CN 東京都西多摩郡 東京都青梅市
199 #CN 神奈川県津久井郡
201 #CN 東京都狛江市
202 #CN 東京都保谷市
203 #CN 東京都東久留米市
204 #CN 東京都清瀬市
206 #CN 東京都稲城市 東京都多摩市
210 #CN 神奈川県川崎市幸区 神奈川県川崎市川崎区
211 #CN 神奈川県川崎市幸区 神奈川県川崎市中原区
213 #CN 神奈川県川崎市宮前区 神奈川県川崎市高津区
214 #CN 神奈川県川崎市多摩区
215 #CN 神奈川県川崎市麻生区
220 #CN 神奈川県横浜市西区 神奈川県津久井郡
221 #CN 神奈川県横浜市神奈川区
222 #CN 神奈川県横浜市港北区
223 #CN 神奈川県横浜市港北区
226 #CN 神奈川県横浜市緑区
227 #CN 神奈川県横浜市緑区
228 #CN 神奈川県座間市 神奈川県相模原市
229 #CN 神奈川県相模原市
230 #CN 神奈川県横浜市鶴見区
231 #CN 神奈川県横浜市中区
232 #CN 神奈川県横浜市南区
233 #CN 神奈川県横浜市港南区
234 #CN 神奈川県横浜市神奈川区
235 #CN 神奈川県横浜市磯子区
236 #CN 神奈川県横浜市金沢区
237 #CN 神奈川県横須賀市
238 #CN 神奈川県横須賀市 神奈川県三浦市
239 #CN 神奈川県横須賀市
240 #CN 神奈川県横須賀市 神奈川県横浜市保土ケ谷区 神奈川県三浦郡
241 #CN 神奈川県横浜市旭区
242 #CN 神奈川県大和市
243 #CN 神奈川県愛甲郡 神奈川県海老名市 神奈川県厚木市
244 #CN 神奈川県横浜市栄区 神奈川県横浜市戸塚区
245 #CN 神奈川県横浜市戸塚区 神奈川県横浜市泉区
246 #CN 神奈川県横浜市瀬谷区
247 #CN 神奈川県横浜市栄区 神奈川県鎌倉市
248 #CN 神奈川県鎌倉市
249 #CN 神奈川県逗子市
250 #CN 神奈川県小田原市 神奈川県足柄下郡 神奈川県南足柄市
251 #CN 神奈川県藤沢市
252 #CN 神奈川県綾瀬市 神奈川県藤沢市
253 #CN 神奈川県茅ケ崎市 神奈川県高座郡
254 #CN 神奈川県平塚市
255 #CN 神奈川県中郡
256 #CN 神奈川県小田原市
257 #CN 神奈川県愛甲郡 神奈川県秦野市
258 #CN 神奈川県足柄上郡
259 #CN 神奈川県伊勢原市 神奈川県秦野市 神奈川県足柄下郡 神奈川県足柄上郡 神奈川県中郡 神奈川県平塚市
260 #CN 千葉県千葉市
270 #CN 茨城県北相馬郡 千葉県印旛郡 千葉県我孫子市 千葉県松戸市 千葉県船橋市 千葉県東葛飾郡 千葉県野田市 千葉県流山市
271 #CN 千葉県松戸市
272 #CN 千葉県市川市
273 #CN 千葉県鎌ケ谷市 千葉県船橋市
274 #CN 千葉県船橋市
275 #CN 千葉県習志野市
276 #CN 千葉県八千代市
277 #CN 千葉県東葛飾郡 千葉県柏市
278 #CN 千葉県野田市
279 #CN 千葉県浦安市
280 #CN 千葉県千葉市
281 #CN 千葉県千葉市
282 #CN 千葉県成田市
283 #CN 千葉県山武郡 千葉県東金市
284 #CN 千葉県四街道市
285 #CN 千葉県印旛郡 千葉県佐倉市
286 #CN 千葉県印旛郡 千葉県成田市
287 #CN 千葉県香取郡 千葉県佐原市
288 #CN 千葉県銚子市
289 #CN 千葉県旭市 千葉県印旛郡 千葉県海上郡 千葉県香取郡 千葉県山武郡 千葉県匝瑳郡 千葉県八日市場市
290 #CN 千葉県市原市
292 #CN 千葉県君津市 千葉県木更津市
294 #CN 千葉県安房郡 千葉県館山市
295 #CN 千葉県安房郡
296 #CN 千葉県鴨川市
297 #CN 千葉県長生郡 千葉県茂原市
298 #CN 千葉県夷隅郡
299 #CN 千葉県安房郡 千葉県夷隅郡 千葉県鴨川市 千葉県君津郡 千葉県君津市 千葉県山武郡 千葉県市原市 千葉県勝浦市 千葉県千葉市 千葉県長生郡 千葉県富津市 千葉県茂原市
300 #CN 茨城県つくば市 茨城県稲敷郡 茨城県牛久市 茨城県結城郡 茨城県新治郡 茨城県真壁郡 茨城県水海道市 茨城県筑波郡 茨城県土浦市 茨城県北相馬郡
301 #CN 茨城県稲敷郡 茨城県竜ケ崎市
302 #CN 茨城県取手市 茨城県水海道市 茨城県筑波郡 茨城県北相馬郡
303 #CN 茨城県水海道市
304 #CN 茨城県下妻市
305 #CN 茨城県つくば市 茨城県稲敷郡
306 #CN 茨城県猿島郡 茨城県岩井市 茨城県古河市
307 #CN 茨城県結城市 栃木県河内郡 栃木県小山市
308 #CN 茨城県下館市 茨城県真壁郡
309 #CN 茨城県笠間市 茨城県新治郡 茨城県真壁郡 茨城県西茨城郡
310 #CN 茨城県水戸市
311 #CN 茨城県久慈郡 茨城県行方郡 茨城県鹿島郡 茨城県常陸太田市 茨城県新治郡 茨城県水戸市 茨城県西茨城郡 茨城県東茨城郡 茨城県那珂郡 茨城県那珂湊市 茨城県日立市
312 #CN 茨城県勝田市
313 #CN 茨城県久慈郡 茨城県常陸太田市
314 #CN 茨城県鹿島郡
315 #CN 茨城県新治郡 茨城県石岡市
316 #CN 茨城県日立市
317 #CN 茨城県日立市
318 #CN 茨城県高萩市
319 #CN 茨城県久慈郡 茨城県新治郡 茨城県西茨城郡 茨城県多賀郡 茨城県東茨城郡 茨城県那珂郡 茨城県日立市 茨城県北茨城市
320 #CN 栃木県宇都宮市
321 #CN 栃木県宇都宮市 栃木県塩谷郡 栃木県下都賀郡 栃木県河内郡 栃木県今市市 栃木県鹿沼市 栃木県真岡市 栃木県那須郡 栃木県日光市 栃木県芳賀郡
322 #CN 栃木県鹿沼市 栃木県上都賀郡
323 #CN 栃木県小山市
324 #CN 栃木県大田原市 栃木県那須郡
325 #CN 栃木県黒磯市 栃木県那須郡
326 #CN 栃木県足利市
327 #CN 栃木県安蘇郡 栃木県佐野市
328 #CN 栃木県安蘇郡 栃木県下都賀郡 栃木県上都賀郡 栃木県栃木市
329 #CN 栃木県塩谷郡 栃木県下都賀郡 栃木県河内郡 栃木県黒磯市 栃木県小山市 栃木県足利市 栃木県大田原市 栃木県那須郡 栃木県矢板市
330 #CN 埼玉県大宮市
331 #CN 埼玉県大宮市
332 #CN 埼玉県川口市
333 #CN 埼玉県川口市
334 #CN 埼玉県川口市 埼玉県鳩ケ谷市
335 #CN 埼玉県戸田市 埼玉県蕨市
336 #CN 埼玉県浦和市
337 #CN 埼玉県浦和市
338 #CN 埼玉県浦和市 埼玉県与野市
339 #CN 埼玉県岩槻市
340 #CN 埼玉県草加市 埼玉県八潮市 埼玉県北葛飾郡
341 #CN 埼玉県三郷市
342 #CN 埼玉県北葛飾郡
343 #CN 埼玉県越谷市 埼玉県北葛飾郡
344 #CN 埼玉県春日部市 埼玉県北葛飾郡
345 #CN 埼玉県南埼玉郡 埼玉県北葛飾郡
346 #CN 埼玉県久喜市 埼玉県南埼玉郡
347 #CN 埼玉県加須市 埼玉県北埼玉郡
348 #CN 埼玉県羽生市
349 #CN 埼玉県南埼玉郡 埼玉県北葛飾郡 埼玉県北埼玉郡 埼玉県蓮田市 栃木県下都賀郡
350 #CN 埼玉県狭山市 埼玉県坂戸市 埼玉県川越市 埼玉県入間郡 埼玉県比企郡
351 #CN 埼玉県朝霞市 埼玉県和光市
352 #CN 埼玉県新座市
353 #CN 埼玉県志木市
354 #CN 埼玉県入間郡 埼玉県富士見市
355 #CN 埼玉県秩父郡 埼玉県東松山市 埼玉県比企郡
356 #CN 埼玉県上福岡市 埼玉県川越市
357 #CN 埼玉県入間郡 埼玉県飯能市
358 #CN 埼玉県入間市
359 #CN 埼玉県所沢市
360 #CN 埼玉県熊谷市 埼玉県行田市 埼玉県大里郡 埼玉県北埼玉郡
361 #CN 埼玉県行田市
362 #CN 埼玉県上尾市 埼玉県北足立郡
363 #CN 埼玉県桶川市
364 #CN 埼玉県北本市
365 #CN 埼玉県鴻巣市 埼玉県北埼玉郡
366 #CN 埼玉県深谷市
367 #CN 埼玉県児玉郡 埼玉県本庄市
368 #CN 埼玉県秩父郡 埼玉県秩父市
369 #CN 埼玉県児玉郡 埼玉県大里郡 埼玉県秩父郡 埼玉県秩父市 埼玉県北足立郡
370 #CN 群馬県甘楽郡 群馬県館林市 群馬県群馬郡 群馬県吾妻郡 群馬県高崎市 群馬県佐波郡 群馬県新田郡 群馬県前橋市 群馬県多野郡 群馬県富岡市 群馬県北群馬郡 群馬県邑楽郡
371 #CN 群馬県勢多郡 群馬県前橋市
372 #CN 群馬県伊勢崎市
373 #CN 群馬県太田市
374 #CN 群馬県館林市 群馬県邑楽郡
375 #CN 群馬県藤岡市
376 #CN 群馬県桐生市 群馬県山田郡 群馬県勢多郡 栃木県上都賀郡
377 #CN 群馬県吾妻郡 群馬県渋川市 群馬県勢多郡 群馬県北群馬郡
378 #CN 群馬県沼田市 群馬県利根郡
379 #CN 群馬県安中市 群馬県碓氷郡 群馬県甘楽郡 群馬県佐波郡 群馬県新田郡 群馬県勢多郡 群馬県前橋市 群馬県利根郡
380 #CN 長野県長野市
381 #CN 長野県下高井郡 長野県更級郡 長野県上高井郡 長野県上水内郡 長野県長野市
382 #CN 長野県上高井郡 長野県須坂市
383 #CN 長野県中野市
384 #CN 群馬県吾妻郡 長野県佐久市 長野県小諸市 長野県南佐久郡 長野県北佐久郡
385 #CN 長野県佐久市
386 #CN 長野県小県郡 長野県上田市 長野県須坂市
387 #CN 長野県更埴市
388 #CN 長野県長野市
389 #CN 群馬県碓氷郡 新潟県中頚城郡 長野県下高井郡 長野県下水内郡 長野県更級郡 長野県佐久市 長野県小県郡 長野県小諸市 長野県上水内郡 長野県埴科郡 長野県中野市 長野県飯山市 長野県北佐久郡
390 #CN 長野県松本市 長野県東筑摩郡 長野県南安曇郡
391 #CN 長野県茅野市 長野県諏訪郡 長野県北佐久郡
392 #CN 長野県諏訪市
393 #CN 長野県諏訪郡
394 #CN 長野県塩尻市 長野県岡谷市
395 #CN 長野県下伊那郡 長野県飯田市
396 #CN 長野県伊那市 長野県上伊那郡
397 #CN 長野県木曽郡
398 #CN 長野県大町市
399 #CN 長野県伊那市 長野県塩尻市 長野県下伊那郡 長野県駒ケ根市 長野県松本市 長野県上伊那郡 長野県諏訪郡 長野県東筑摩郡 長野県南安曇郡 長野県飯田市 長野県北安曇郡 長野県木曽郡
400 #CN 山梨県甲府市 山梨県中巨摩郡 山梨県東八代郡 山梨県南巨摩郡
401 #CN 山梨県大月市 山梨県南都留郡
402 #CN 山梨県都留市 山梨県南都留郡
403 #CN 山梨県南都留郡 山梨県富士吉田市
404 #CN 山梨県塩山市 山梨県東山梨郡
405 #CN 山梨県山梨市
406 #CN 山梨県甲府市 山梨県東山梨郡 山梨県東八代郡
407 #CN 山梨県韮崎市 山梨県北巨摩郡
408 #CN 山梨県北巨摩郡
409 #CN 山梨県塩山市 山梨県西八代郡 山梨県大月市 山梨県中巨摩郡 山梨県東山梨郡 山梨県東八代郡 山梨県南巨摩郡 山梨県北巨摩郡 山梨県北都留郡
410 #CN 静岡県賀茂郡 静岡県駿東郡 静岡県沼津市 静岡県裾野市 静岡県田方郡
411 #CN 静岡県三島市 静岡県駿東郡 静岡県沼津市 静岡県田方郡
412 #CN 静岡県御殿場市
413 #CN 静岡県伊東市 静岡県下田市 静岡県賀茂郡 静岡県熱海市
414 #CN 静岡県伊東市
415 #CN 静岡県下田市 静岡県賀茂郡
416 #CN 静岡県富士市
417 #CN 静岡県富士市
418 #CN 静岡県富士宮市 静岡県富士郡
419 #CN 静岡県田方郡 静岡県富士郡 静岡県富士市
420 #CN 静岡県静岡市
421 #CN 静岡県庵原郡 静岡県志太郡 静岡県榛原郡 静岡県静岡市
422 #CN 静岡県静岡市
424 #CN 静岡県清水市
425 #CN 静岡県焼津市
426 #CN 静岡県藤枝市
427 #CN 静岡県島田市
428 #CN 静岡県榛原郡 静岡県静岡市
430 #CN 静岡県浜松市
431 #CN 静岡県引佐郡 静岡県湖西市 静岡県天竜市 静岡県磐田郡 静岡県浜松市 静岡県浜名郡
432 #CN 静岡県浜松市 静岡県浜名郡
433 #CN 静岡県浜松市
434 #CN 静岡県浜北市
435 #CN 静岡県浜松市
436 #CN 静岡県掛川市
437 #CN 静岡県周智郡 静岡県小笠郡 静岡県袋井市 静岡県磐田郡
438 #CN 静岡県磐田郡 静岡県磐田市
439 #CN 静岡県小笠郡
440 #CN 愛知県豊橋市
441 #CN 愛知県渥美郡 愛知県新城市 愛知県南設楽郡 愛知県宝飯郡 愛知県豊橋市 愛知県豊川市 愛知県北設楽郡
442 #CN 愛知県豊川市
443 #CN 愛知県蒲郡市
444 #CN 愛知県安城市 愛知県岡崎市 愛知県額田郡 愛知県高浜市 愛知県西尾市 愛知県東加茂郡 愛知県幡豆郡 愛知県豊田市
445 #CN 愛知県西尾市
446 #CN 愛知県安城市
447 #CN 愛知県碧南市
448 #CN 愛知県刈谷市
449 #CN 愛知県北設楽郡
450 #CN 愛知県名古屋市中区 愛知県名古屋市中村区
451 #CN 愛知県名古屋市西区
452 #CN 愛知県西春日井郡 愛知県名古屋市西区
453 #CN 愛知県名古屋市中村区
454 #CN 愛知県名古屋市中川区
455 #CN 愛知県名古屋市港区
456 #CN 愛知県名古屋市熱田区
457 #CN 愛知県名古屋市南区
458 #CN 愛知県名古屋市緑区
459 #CN 愛知県名古屋市緑区
460 #CN 愛知県名古屋市千種区 愛知県名古屋市中区
461 #CN 愛知県名古屋市東区
462 #CN 愛知県名古屋市北区
463 #CN 愛知県名古屋市守山区
464 #CN 愛知県名古屋市千種区
465 #CN 愛知県名古屋市名東区
466 #CN 愛知県名古屋市昭和区
467 #CN 愛知県名古屋市瑞穂区
468 #CN 愛知県名古屋市千種区 愛知県名古屋市天白区
469 #CN 愛知県名古屋市中区 愛知県名古屋市熱田区
470 #CN 愛知県愛知郡 愛知県西加茂郡 愛知県知多郡 愛知県豊田市 愛知県豊明市
471 #CN 愛知県豊田市
472 #CN 愛知県知立市
473 #CN 愛知県豊田市
474 #CN 愛知県大府市
475 #CN 愛知県半田市
476 #CN 愛知県東海市
477 #CN 愛知県東海市
478 #CN 愛知県知多市
479 #CN 愛知県常滑市
480 #CN 愛知県愛知郡 愛知県春日井市 愛知県瀬戸市 愛知県西春日井郡 愛知県丹羽郡
481 #CN 愛知県西春日井郡
482 #CN 愛知県岩倉市
483 #CN 愛知県江南市 岐阜県羽島郡
484 #CN 愛知県犬山市
485 #CN 愛知県小牧市
486 #CN 愛知県春日井市
487 #CN 愛知県春日井市
488 #CN 愛知県尾張旭市
489 #CN 愛知県瀬戸市
490 #CN 愛知県海部郡 愛知県中島郡
491 #CN 愛知県一宮市
492 #CN 愛知県稲沢市
493 #CN 愛知県一宮市 愛知県葉栗郡
494 #CN 愛知県尾西市
495 #CN 愛知県中島郡
496 #CN 愛知県海部郡 愛知県津島市
497 #CN 愛知県海部郡
498 #CN 愛知県海部郡 三重県桑名郡
500 #CN 岐阜県岐阜市
501 #CN 岐阜県安八郡 岐阜県羽島郡 岐阜県羽島市 岐阜県加茂郡 岐阜県関市 岐阜県岐阜市 岐阜県郡上郡 岐阜県山県郡 岐阜県大野郡 岐阜県美濃市 岐阜県武儀郡 岐阜県本巣郡 岐阜県揖斐郡
502 #CN 岐阜県岐阜市
503 #CN 岐阜県安八郡 岐阜県海津郡 岐阜県大垣市 岐阜県不破郡 岐阜県揖斐郡 岐阜県養老郡
504 #CN 岐阜県各務原市
505 #CN 岐阜県加茂郡 岐阜県可児郡 岐阜県美濃加茂市
506 #CN 岐阜県吉城郡 岐阜県高山市 岐阜県大野郡 富山県上新川郡
507 #CN 岐阜県多治見市 岐阜県土岐郡
508 #CN 岐阜県恵那郡 岐阜県中津川市
509 #CN 岐阜県益田郡 岐阜県加茂郡 岐阜県可児市 岐阜県各務原市 岐阜県吉城郡 岐阜県恵那郡 岐阜県恵那市 岐阜県瑞浪市 岐阜県大野郡 岐阜県中津川市 岐阜県土岐市
510 #CN 三重県安芸郡 三重県三重郡 三重県四日市市 三重県鈴鹿市
511 #CN 三重県員弁郡 三重県桑名郡 三重県桑名市
512 #CN 三重県三重郡 三重県四日市市
513 #CN 三重県鈴鹿市
514 #CN 三重県安芸郡 三重県一志郡 三重県久居市 三重県津市
515 #CN 三重県伊勢市 三重県一志郡 三重県松阪市 三重県多気郡 三重県度会郡 三重県飯南郡
516 #CN 三重県伊勢市 三重県度会郡
517 #CN 三重県伊勢市 三重県志摩郡 三重県鳥羽市
518 #CN 三重県阿山郡 三重県上野市 三重県名賀郡 三重県名張市 奈良県山辺郡 奈良県添上郡
519 #CN 三重県阿山郡 三重県亀山市 三重県熊野市 三重県四日市市 三重県松阪市 三重県上野市 三重県多気郡 三重県度会郡 三重県南牟婁郡 三重県尾鷲市 三重県北牟婁郡 三重県鈴鹿郡 三重県鈴鹿市 和歌山県東牟婁郡
520 #CN 京都府京都市左京区 滋賀県蒲生郡 滋賀県栗太郡 滋賀県甲賀郡 滋賀県高島郡 滋賀県滋賀郡 滋賀県大津市 滋賀県野洲郡
521 #CN 滋賀県蒲生郡 滋賀県坂田郡 滋賀県神崎郡 滋賀県彦根市
522 #CN 滋賀県犬上郡 滋賀県彦根市
523 #CN 滋賀県近江八幡市
524 #CN 滋賀県守山市 滋賀県野洲郡
525 #CN 滋賀県草津市
526 #CN 滋賀県長浜市 滋賀県東浅井郡
527 #CN 滋賀県愛知郡 滋賀県神崎郡 滋賀県八日市市
528 #CN 滋賀県甲賀郡
529 #CN 滋賀県愛知郡 滋賀県伊香郡 滋賀県蒲生郡 滋賀県犬上郡 滋賀県甲賀郡 滋賀県神崎郡 滋賀県東浅井郡 滋賀県彦根市
530 #CN 大阪府大阪市東区 大阪府大阪市北区
531 #CN 大阪府大阪市大淀区
532 #CN 大阪府大阪市淀川区
533 #CN 大阪府大阪市東淀川区
534 #CN 大阪府大阪市都島区
535 #CN 大阪府大阪市旭区
536 #CN 大阪府大阪市城東区
537 #CN 大阪府大阪市東成区
538 #CN 大阪府大阪市鶴見区
540 #CN 大阪府大阪市東区
541 #CN 大阪府大阪市東区
542 #CN 大阪府大阪市南区
543 #CN 大阪府大阪市天王寺区
544 #CN 大阪府大阪市生野区
545 #CN 大阪府大阪市阿倍野区
546 #CN 大阪府大阪市東住吉区
547 #CN 大阪府大阪市平野区
550 #CN 大阪府大阪市西区
551 #CN 大阪府大阪市大正区
552 #CN 大阪府大阪市港区
553 #CN 大阪府大阪市福島区
554 #CN 大阪府大阪市此花区
555 #CN 大阪府大阪市西淀川区
556 #CN 大阪府大阪市浪速区
557 #CN 大阪府大阪市西成区
558 #CN 大阪府大阪市住吉区
559 #CN 大阪府大阪市住之江区
560 #CN 大阪府豊中市 兵庫県伊丹市
561 #CN 大阪府豊中市
562 #CN 大阪府箕面市
563 #CN 大阪府池田市 大阪府豊能郡 大阪府箕面市
564 #CN 大阪府吹田市
565 #CN 大阪府吹田市 大阪府豊中市
566 #CN 大阪府摂津市
567 #CN 大阪府茨木市
568 #CN 大阪府茨木市
569 #CN 大阪府高槻市
570 #CN 大阪府守口市
571 #CN 大阪府門真市
572 #CN 大阪府寝屋川市
573 #CN 大阪府枚方市
574 #CN 大阪府大東市
575 #CN 大阪府四条畷市
576 #CN 大阪府交野市
577 #CN 大阪府東大阪市
578 #CN 大阪府東大阪市
579 #CN 大阪府東大阪市 奈良県生駒市
580 #CN 大阪府松原市
581 #CN 大阪府八尾市
582 #CN 大阪府柏原市
583 #CN 大阪府羽曳野市 大阪府藤井寺市 大阪府南河内郡
584 #CN 大阪府富田林市
585 #CN 大阪府南河内郡
586 #CN 大阪府河内長野市
587 #CN 大阪府南河内郡
588 #CN 大阪府堺市
589 #CN 大阪府南河内郡
590 #CN 大阪府堺市 大阪府泉佐野市 大阪府泉南郡 大阪府泉南市 大阪府和泉市
591 #CN 大阪府堺市
592 #CN 大阪府高石市 大阪府堺市
593 #CN 大阪府堺市
594 #CN 大阪府和泉市
595 #CN 大阪府泉大津市 大阪府泉北郡
596 #CN 大阪府岸和田市
597 #CN 大阪府貝塚市
598 #CN 大阪府泉佐野市 大阪府泉南郡
599 #CN 大阪府泉南郡
600 #CN 京都府京都市下京区
601 #CN 京都府宇治市 京都府京都市左京区 京都府京都市南区 京都府京都市伏見区 京都府京都市北区 京都府北桑田郡
602 #CN 京都府京都市上京区
603 #CN 京都府京都市北区
604 #CN 京都府京都市中京区
605 #CN 京都府京都市東山区
606 #CN 京都府京都市左京区
607 #CN 京都府京都市山科区
610 #CN 京都府京都市西京区 京都府城陽市 京都府綴喜郡
611 #CN 京都府宇治市
612 #CN 京都府京都市伏見区
613 #CN 京都府久世郡 京都府京都市伏見区 京都府八幡市
614 #CN 京都府八幡市
615 #CN 京都府京都市右京区 京都府京都市西京区
616 #CN 京都府京都市右京区 京都府京都市西京区
617 #CN 京都府向日市 京都府長岡京市
618 #CN 京都府乙訓郡 大阪府三島郡
619 #CN 京都府相楽郡
620 #CN 京都府加佐郡 京都府天田郡 京都府福知山市
621 #CN 京都府亀岡市 大阪府高槻市
622 #CN 京都府船井郡
623 #CN 京都府綾部市 京都府福知山市
624 #CN 京都府舞鶴市
625 #CN 京都府舞鶴市
626 #CN 京都府宮津市 京都府与謝郡
627 #CN 京都府竹野郡 京都府中郡
629 #CN 京都府綾部市 京都府宮津市 京都府熊野郡 京都府船井郡 京都府竹野郡 京都府中郡 京都府天田郡 京都府舞鶴市 京都府福知山市 京都府与謝郡
630 #CN 大阪府東大阪市 奈良県山辺郡 奈良県生駒市 奈良県奈良市
631 #CN 奈良県奈良市
632 #CN 奈良県宇陀郡 奈良県山辺郡 奈良県天理市
633 #CN 奈良県宇陀郡 奈良県吉野郡 奈良県桜井市
634 #CN 奈良県橿原市 奈良県高市郡
635 #CN 奈良県高市郡 奈良県大和高田市 奈良県北葛城郡
636 #CN 奈良県磯城郡 奈良県生駒郡 奈良県北葛城郡
637 #CN 奈良県吉野郡 奈良県五條市 和歌山県伊都郡
638 #CN 奈良県吉野郡
639 #CN 奈良県吉野郡 奈良県御所市 奈良県大和郡山市 奈良県北葛城郡
640 #CN 和歌山県伊都郡 和歌山県海草郡 和歌山県海南市 和歌山県那賀郡 和歌山県和歌山市
641 #CN 和歌山県和歌山市
642 #CN 和歌山県海南市
643 #CN 和歌山県伊都郡 和歌山県有田郡
644 #CN 和歌山県御坊市 和歌山県日高郡
645 #CN 和歌山県日高郡
646 #CN 和歌山県西牟婁郡 和歌山県田辺市
647 #CN 三重県南牟婁郡 奈良県吉野郡 和歌山県新宮市 和歌山県東牟婁郡
648 #CN 奈良県吉野郡 和歌山県伊都郡 和歌山県橋本市
649 #CN 和歌山県伊都郡 和歌山県海草郡 和歌山県橋本市 和歌山県御坊市 和歌山県西牟婁郡 和歌山県東牟婁郡 和歌山県那賀郡 和歌山県日高郡 和歌山県有田郡 和歌山県有田市 和歌山県和歌山市
650 #CN 兵庫県神戸市中央区
651 #CN 兵庫県神戸市中央区 兵庫県神戸市北区 兵庫県西宮市
652 #CN 兵庫県神戸市兵庫区 兵庫県神戸市北区
653 #CN 兵庫県神戸市長田区
654 #CN 兵庫県神戸市須磨区
655 #CN 兵庫県神戸市垂水区
656 #CN 兵庫県三原郡 兵庫県洲本市 兵庫県津名郡
657 #CN 兵庫県神戸市灘区 兵庫県神戸市北区
658 #CN 兵庫県神戸市東灘区
659 #CN 兵庫県芦屋市
660 #CN 兵庫県尼崎市
661 #CN 兵庫県尼崎市
662 #CN 兵庫県西宮市
663 #CN 兵庫県西宮市
664 #CN 兵庫県伊丹市
665 #CN 兵庫県川西市 兵庫県宝塚市
666 #CN 兵庫県川西市 兵庫県川辺郡 兵庫県宝塚市
667 #CN 兵庫県美方郡 兵庫県養父郡
668 #CN 兵庫県出石郡 兵庫県豊岡市
669 #CN 兵庫県三田市 兵庫県城崎郡 兵庫県神戸市北区 兵庫県西宮市 兵庫県多紀郡 兵庫県朝来郡 兵庫県美方郡 兵庫県氷上郡 兵庫県宝塚市 兵庫県豊岡市
670 #CN 兵庫県姫路市
671 #CN 兵庫県高砂市 兵庫県佐用郡 兵庫県宍粟郡 兵庫県飾磨郡 兵庫県姫路市 兵庫県揖保郡
672 #CN 兵庫県飾磨郡 兵庫県姫路市
673 #CN 兵庫県加東郡 兵庫県三木市 兵庫県神戸市西区 兵庫県神戸市北区 兵庫県美嚢郡 兵庫県明石市
674 #CN 兵庫県神戸市西区 兵庫県明石市
675 #CN 兵庫県加古郡 兵庫県加古川市 兵庫県加西市 兵庫県小野市
676 #CN 兵庫県高砂市
677 #CN 兵庫県西脇市 兵庫県多可郡
678 #CN 兵庫県赤穂郡 兵庫県赤穂市 兵庫県相生市
679 #CN 兵庫県加西市 兵庫県加東郡 兵庫県佐用郡 兵庫県神崎郡 兵庫県多可郡 兵庫県朝来郡 兵庫県姫路市 兵庫県揖保郡 兵庫県竜野市
680 #CN 鳥取県岩美郡 鳥取県鳥取市 鳥取県八頭郡
681 #CN 鳥取県岩美郡
682 #CN 鳥取県倉吉市 鳥取県東伯郡
683 #CN 鳥取県西伯郡 鳥取県米子市
684 #CN 鳥取県境港市 島根県隠岐郡
685 #CN 島根県隠岐郡
689 #CN 鳥取県岩美郡 鳥取県気高郡 鳥取県西伯郡 鳥取県鳥取市 鳥取県東伯郡 鳥取県日野郡 鳥取県八頭郡 鳥取県米子市
690 #CN 島根県松江市 島根県八束郡 島根県飯石郡
691 #CN 島根県平田市
692 #CN 島根県安来市 島根県能義郡
693 #CN 島根県出雲市 島根県簸川郡
694 #CN 島根県大田市
695 #CN 島根県江津市 島根県浜田市
696 #CN 島根県邑智郡
697 #CN 島根県那賀郡 島根県浜田市 島根県邑智郡
698 #CN 島根県益田市 島根県美濃郡
699 #CN 島根県益田市 島根県江津市 島根県鹿足郡 島根県出雲市 島根県仁多郡 島根県大原郡 島根県大田市 島根県那賀郡 島根県迩摩郡 島根県八束郡 島根県簸川郡 島根県邑智郡
700 #CN 岡山県岡山市
701 #CN 岡山県英田郡 岡山県岡山市 岡山県赤磐郡 岡山県倉敷市 岡山県都窪郡 岡山県邑久郡 岡山県和気郡
702 #CN 岡山県岡山市
703 #CN 岡山県岡山市
704 #CN 岡山県岡山市
705 #CN 岡山県備前市
706 #CN 岡山県岡山市 岡山県玉野市
707 #CN 岡山県英田郡 岡山県勝田郡
708 #CN 岡山県久米郡 岡山県勝田郡 岡山県津山市 岡山県苫田郡
709 #CN 岡山県英田郡 岡山県岡山市 岡山県久米郡 岡山県御津郡 岡山県児島郡 岡山県勝田郡 岡山県赤磐郡 岡山県倉敷市 岡山県苫田郡 岡山県備前市 岡山県和気郡
710 #CN 岡山県吉備郡 岡山県児島郡 岡山県浅口郡 岡山県倉敷市 岡山県総社市
711 #CN 岡山県倉敷市
712 #CN 岡山県倉敷市
713 #CN 岡山県倉敷市
714 #CN 岡山県笠岡市 岡山県後月郡 岡山県小田郡 岡山県川上郡 岡山県浅口郡
715 #CN 岡山県井原市
716 #CN 岡山県高梁市 岡山県上房郡 岡山県川上郡
717 #CN 岡山県真庭郡
718 #CN 岡山県阿哲郡 岡山県新見市
719 #CN 岡山県阿哲郡 岡山県高梁市 岡山県新見市 岡山県真庭郡 岡山県川上郡 岡山県浅口郡 岡山県総社市 岡山県都窪郡 岡山県苫田郡
720 #CN 広島県沼隈郡 広島県深安郡 広島県神石郡 広島県比婆郡 広島県尾道市 広島県福山市
721 #CN 広島県福山市
722 #CN 広島県因島市 広島県御調郡 広島県三原市 広島県沼隈郡 広島県世羅郡 広島県尾道市 広島県府中市 広島県豊田郡
723 #CN 広島県三原市
724 #CN 広島県賀茂郡 広島県呉市 広島県高田郡 広島県東広島市
725 #CN 広島県竹原市 広島県豊田郡
726 #CN 広島県府中市
727 #CN 広島県庄原市 広島県比婆郡
728 #CN 広島県三次市 広島県双三郡 広島県比婆郡
729 #CN 広島県芦品郡 広島県賀茂郡 広島県呉市 広島県甲奴郡 広島県高田郡 広島県三原市 広島県三次市 広島県庄原市 広島県神石郡 広島県世羅郡 広島県双三郡 広島県竹原市 広島県東広島市 広島県比婆郡 広島県尾道市 広島県府中市 広島県福山市 広島県豊田郡
730 #CN 広島県広島市中区 広島県広島市東区
731 #CN 広島県安芸郡 広島県広島市安芸区 広島県広島市安佐南区 広島県広島市安佐北区 広島県高田郡 広島県佐伯郡 広島県山県郡
732 #CN 広島県広島市中区 広島県広島市東区 広島県広島市南区
733 #CN 広島県広島市西区
734 #CN 広島県広島市南区 広島県豊田郡
735 #CN 広島県安芸郡 広島県広島市東区
736 #CN 広島県安芸郡 広島県広島市安芸区
737 #CN 広島県安芸郡 広島県呉市 広島県佐伯郡
738 #CN 広島県佐伯郡
739 #CN 広島県広島市安芸区 広島県広島市安佐北区 広島県高田郡 広島県佐伯郡 広島県大竹市 広島県東広島市
740 #CN 山口県岩国市 山口県玖珂郡
741 #CN 山口県岩国市 山口県玖珂郡
742 #CN 山口県玖珂郡 山口県熊毛郡 山口県大島郡 山口県柳井市
743 #CN 山口県熊毛郡 山口県光市
744 #CN 山口県下松市
745 #CN 山口県下松市 山口県熊毛郡 山口県都濃郡 山口県徳山市
746 #CN 山口県新南陽市 山口県徳山市
747 #CN 山口県佐波郡 山口県山口市 山口県都濃郡 山口県防府市
749 #CN 山口県玖珂郡
750 #CN 山口県下関市 山口県美祢市 山口県豊浦郡
751 #CN 山口県下関市
752 #CN 山口県下関市
753 #CN 山口県阿武郡 山口県吉敷郡 山口県山口市
754 #CN 山口県阿武郡 山口県宇部市 山口県吉敷郡 山口県山口市 山口県美祢郡
755 #CN 山口県宇部市
756 #CN 山口県小野田市
757 #CN 山口県厚狭郡
758 #CN 山口県阿武郡 山口県萩市
759 #CN 山口県阿武郡 山口県宇部市 山口県下関市 山口県山口市 山口県大津郡 山口県長門市 山口県萩市 山口県美祢市 山口県豊浦郡
760 #CN 香川県高松市
761 #CN 香川県綾歌郡 香川県香川郡 香川県高松市 香川県小豆郡 香川県大川郡 香川県木田郡
762 #CN 香川県綾歌郡 香川県坂出市
763 #CN 香川県丸亀市
764 #CN 香川県仲多度郡
765 #CN 香川県善通寺市
766 #CN 香川県仲多度郡
767 #CN 香川県三豊郡
768 #CN 香川県観音寺市 香川県三豊郡
769 #CN 香川県綾歌郡 香川県三豊郡 香川県大川郡 香川県仲多度郡 香川県木田郡
770 #CN 徳島県徳島市
771 #CN 徳島県阿南市 徳島県阿波郡 徳島県三好郡 徳島県勝浦郡 徳島県徳島市 徳島県那賀郡 徳島県板野郡 徳島県美馬郡 徳島県名西郡 徳島県名東郡 徳島県鳴門市
772 #CN 徳島県鳴門市
773 #CN 徳島県小松島市 徳島県徳島市
774 #CN 徳島県阿南市
775 #CN 徳島県海部郡
776 #CN 徳島県麻植郡
777 #CN 徳島県美馬郡
778 #CN 徳島県三好郡
779 #CN 徳島県阿南市 徳島県海部郡 徳島県三好郡 徳島県徳島市 徳島県那賀郡 徳島県板野郡 徳島県美馬郡 徳島県麻植郡 徳島県名西郡 徳島県鳴門市
780 #CN 高知県高知市
781 #CN 高知県安芸郡 高知県吾川郡 高知県香美郡 高知県高岡郡 高知県高知市 高知県室戸市 高知県長岡郡 高知県土佐郡 高知県土佐市 高知県南国市
782 #CN 高知県香美郡
783 #CN 高知県高知市 高知県南国市
784 #CN 高知県安芸郡 高知県安芸市 高知県香美郡
785 #CN 高知県高岡郡 高知県須崎市
786 #CN 高知県高岡郡 高知県幡多郡
787 #CN 高知県宿毛市 高知県中村市 高知県土佐清水市 高知県幡多郡
788 #CN 高知県宿毛市 高知県幡多郡
789 #CN 高知県香美郡 高知県高岡郡 高知県長岡郡 高知県幡多郡
790 #CN 愛媛県松山市
791 #CN 愛媛県伊予郡 愛媛県温泉郡 愛媛県喜多郡 愛媛県周桑郡 愛媛県松山市 愛媛県上浮穴郡 愛媛県北条市
792 #CN 愛媛県越智郡 愛媛県新居浜市
793 #CN 愛媛県西条市
794 #CN 愛媛県越智郡 愛媛県今治市
795 #CN 愛媛県喜多郡 愛媛県大洲市
796 #CN 愛媛県西宇和郡 愛媛県八幡浜市
797 #CN 愛媛県喜多郡 愛媛県東宇和郡
798 #CN 愛媛県宇和島市 愛媛県南宇和郡 愛媛県北宇和郡
799 #CN 愛媛県伊予郡 愛媛県伊予三島市 愛媛県伊予市 愛媛県宇摩郡 愛媛県越智郡 愛媛県喜多郡 愛媛県今治市 愛媛県周桑郡 愛媛県松山市 愛媛県西条市 愛媛県川之江市 愛媛県東予市 愛媛県北宇和郡 愛媛県北条市
800 #CN 福岡県京都郡 福岡県北九州市小倉南区 福岡県北九州市門司区
801 #CN 福岡県北九州市門司区
802 #CN 福岡県北九州市小倉南区 福岡県北九州市小倉北区
803 #CN 福岡県北九州市小倉南区 福岡県北九州市小倉北区
804 #CN 福岡県北九州市戸畑区
805 #CN 福岡県北九州市八幡東区
806 #CN 福岡県北九州市八幡西区
807 #CN 福岡県鞍手郡 福岡県遠賀郡 福岡県中間市 福岡県北九州市八幡西区
808 #CN 福岡県北九州市若松区
809 #CN 福岡県中間市 福岡県北九州市八幡西区
810 #CN 福岡県福岡市中央区 福岡県福岡市南区 福岡県福岡市博多区
811 #CN 長崎県壱岐郡 福岡県遠賀郡 福岡県宗像郡 福岡県宗像市 福岡県筑紫郡 福岡県粕屋郡 福岡県福岡市西区 福岡県福岡市早良区 福岡県福岡市東区
812 #CN 福岡県福岡市中央区 福岡県福岡市東区 福岡県福岡市博多区
813 #CN 福岡県粕屋郡 福岡県福岡市東区
814 #CN 福岡県福岡市城南区 福岡県福岡市西区 福岡県福岡市早良区 福岡県福岡市中央区
815 #CN 福岡県福岡市城南区 福岡県福岡市中央区 福岡県福岡市南区
816 #CN 福岡県春日市 福岡県大野城市 福岡県福岡市南区 福岡県福岡市博多区
817 #CN 長崎県下県郡 長崎県上県郡
818 #CN 福岡県太宰府市 福岡県筑紫野市
819 #CN 福岡県糸島郡 福岡県福岡市西区
820 #CN 福岡県鞍手郡 福岡県嘉穂郡 福岡県飯塚市
821 #CN 福岡県嘉穂郡 福岡県山田市
822 #CN 福岡県鞍手郡 福岡県嘉穂郡 福岡県直方市 福岡県田川郡
823 #CN 福岡県鞍手郡
824 #CN 福岡県京都郡 福岡県行橋市 福岡県田川郡
825 #CN 福岡県田川市
826 #CN 福岡県田川郡 福岡県田川市
827 #CN 福岡県田川郡
828 #CN 福岡県豊前市
829 #CN 福岡県築上郡
830 #CN 福岡県久留米市 福岡県三井郡 福岡県三潴郡
831 #CN 福岡県大川市
832 #CN 福岡県山門郡 福岡県柳川市
833 #CN 福岡県筑後市
834 #CN 福岡県八女郡 福岡県八女市
835 #CN 福岡県三池郡 福岡県山門郡
836 #CN 福岡県大牟田市
837 #CN 福岡県大牟田市
838 #CN 福岡県甘木市 福岡県小郡市 福岡県朝倉郡
839 #CN 大分県日田郡 福岡県久留米市 福岡県三池郡 福岡県山門郡 福岡県浮羽郡
840 #CN 佐賀県佐賀郡 佐賀県佐賀市 佐賀県三養基郡
841 #CN 佐賀県三養基郡 佐賀県鳥栖市
842 #CN 佐賀県神埼郡
843 #CN 佐賀県藤津郡 佐賀県武雄市
844 #CN 佐賀県西松浦郡
845 #CN 佐賀県小城郡
846 #CN 佐賀県多久市
847 #CN 佐賀県唐津市 佐賀県東松浦郡
848 #CN 佐賀県伊万里市
849 #CN 佐賀県伊万里市 佐賀県杵島郡 佐賀県佐賀郡 佐賀県佐賀市 佐賀県三養基郡 佐賀県鹿島市 佐賀県小城郡 佐賀県西松浦郡 佐賀県東松浦郡 佐賀県藤津郡 佐賀県武雄市
850 #CN 長崎県長崎市
851 #CN 長崎県西彼杵郡 長崎県長崎市
852 #CN 長崎県長崎市
853 #CN 長崎県南松浦郡 長崎県福江市
854 #CN 長崎県諌早市 長崎県南高来郡 長崎県北高来郡
855 #CN 長崎県島原市
856 #CN 長崎県大村市
857 #CN 長崎県佐世保市 長崎県西彼杵郡 長崎県南松浦郡 長崎県北松浦郡
858 #CN 長崎県佐世保市
859 #CN 長崎県諌早市 長崎県佐世保市 長崎県松浦市 長崎県西彼杵郡 長崎県東彼杵郡 長崎県南高来郡 長崎県平戸市 長崎県北高来郡 長崎県北松浦郡
860 #CN 熊本県熊本市
861 #CN 熊本県阿蘇郡 熊本県下益城郡 熊本県菊池郡 熊本県菊池市 熊本県玉名郡 熊本県熊本市 熊本県山鹿市 熊本県鹿本郡 熊本県上益城郡 熊本県天草郡 熊本県飽託郡 熊本県本渡市
862 #CN 熊本県熊本市
863 #CN 熊本県牛深市 熊本県天草郡 熊本県本渡市
864 #CN 熊本県荒尾市
865 #CN 熊本県玉名郡 熊本県玉名市
866 #CN 熊本県天草郡 熊本県八代市
867 #CN 熊本県水俣市
868 #CN 熊本県球磨郡 熊本県人吉市
869 #CN 熊本県阿蘇郡 熊本県葦北郡 熊本県宇土郡 熊本県宇土市 熊本県下益城郡 熊本県菊池郡 熊本県球磨郡 熊本県玉名郡 熊本県人吉市 熊本県天草郡 熊本県八代郡 熊本県八代市
870 #CN 大分県大分郡 大分県大分市
871 #CN 大分県下毛郡 大分県中津市 福岡県築上郡
872 #CN 大分県宇佐郡 大分県宇佐市 大分県西国東郡 大分県東国東郡
873 #CN 大分県杵築市 大分県西国東郡 大分県東国東郡
874 #CN 大分県別府市
875 #CN 大分県臼杵市 大分県大野郡
876 #CN 大分県佐伯市 大分県南海部郡
877 #CN 大分県日田郡 大分県日田市
878 #CN 大分県竹田市 大分県直入郡
879 #CN 大分県宇佐市 大分県臼杵市 大分県下毛郡 大分県玖珠郡 大分県西国東郡 大分県速見郡 大分県大分郡 大分県大分市 大分県大野郡 大分県竹田市 大分県中津市 大分県直入郡 大分県津久見市 大分県南海部郡 大分県日田郡 大分県日田市 大分県別府市 大分県豊後高田市 大分県北海部郡
880 #CN 宮崎県宮崎郡 宮崎県宮崎市 宮崎県児湯郡 宮崎県西都市 宮崎県東諸県郡 宮崎県北諸県郡
881 #CN 宮崎県児湯郡 宮崎県西都市
882 #CN 宮崎県延岡市 宮崎県西臼杵郡 宮崎県東臼杵郡
883 #CN 宮崎県東臼杵郡 宮崎県日向市
884 #CN 宮崎県児湯郡
885 #CN 宮崎県都城市 宮崎県北諸県郡
886 #CN 宮崎県小林市 宮崎県西諸県郡 宮崎県東諸県郡
887 #CN 宮崎県日南市
888 #CN 宮崎県串間市
889 #CN 宮崎県えびの市 宮崎県延岡市 宮崎県宮崎郡 宮崎県宮崎市 宮崎県串間市 宮崎県児湯郡 宮崎県小林市 宮崎県西諸県郡 宮崎県東臼杵郡 宮崎県東諸県郡 宮崎県南那珂郡 宮崎県日向市 宮崎県日南市 宮崎県北諸県郡
890 #CN 鹿児島県鹿児島郡 鹿児島県鹿児島市
891 #CN 鹿児島県熊毛郡 鹿児島県指宿市 鹿児島県鹿屋市 鹿児島県鹿児島郡 鹿児島県鹿児島市 鹿児島県垂水市 鹿児島県西之表市 鹿児島県川辺郡 鹿児島県大島郡 鹿児島県日置郡 鹿児島県揖宿郡
892 #CN 鹿児島県鹿児島市
893 #CN 鹿児島県肝属郡 鹿児島県鹿屋市 鹿児島県曾於郡
894 #CN 鹿児島県大島郡 鹿児島県名瀬市
895 #CN 鹿児島県伊佐郡 鹿児島県薩摩郡 鹿児島県川内市 鹿児島県大口市
896 #CN 鹿児島県串木野市 鹿児島県薩摩郡
897 #CN 鹿児島県加世田市 鹿児島県川辺郡
898 #CN 鹿児島県川辺郡 鹿児島県枕崎市
899 #CN 鹿児島県阿久根市 鹿児島県姶良郡 鹿児島県加世田市 鹿児島県国分市 鹿児島県鹿児島市 鹿児島県出水郡 鹿児島県出水市 鹿児島県垂水市 鹿児島県川内市 鹿児島県曾於郡 鹿児島県日置郡
900 #CN 沖縄県那覇市
901 #CN 沖縄県浦添市 沖縄県宜野湾市 沖縄県糸満市 沖縄県中頭郡 沖縄県島尻郡 沖縄県那覇市
902 #CN 沖縄県那覇市
903 #CN 沖縄県中頭郡 沖縄県那覇市
904 #CN 沖縄県沖縄市 沖縄県具志川市 沖縄県国頭郡 沖縄県石川市 沖縄県中頭郡
905 #CN 沖縄県国頭郡 沖縄県島尻郡 沖縄県名護市
906 #CN 沖縄県宮古郡 沖縄県平良市
907 #CN 沖縄県石垣市 沖縄県八重山郡
910 #CN 福井県吉田郡 福井県今立郡 福井県坂井郡 福井県足羽郡 福井県丹生郡 福井県福井市
911 #CN 福井県勝山市
912 #CN 福井県大野郡 福井県大野市
913 #CN 福井県坂井郡
914 #CN 福井県敦賀市
915 #CN 福井県今立郡 福井県南条郡 福井県武生市
916 #CN 福井県鯖江市 福井県丹生郡
917 #CN 福井県遠敷郡 福井県小浜市
919 #CN 福井県遠敷郡 福井県坂井郡 福井県三方郡 福井県大飯郡 福井県敦賀市 福井県南条郡 福井県福井市
920 #CN 石川県河北郡 石川県金沢市 石川県石川郡
921 #CN 石川県金沢市 石川県石川郡
922 #CN 石川県加賀市 石川県江沼郡 福井県坂井郡
923 #CN 石川県小松市 石川県能美郡
924 #CN 石川県松任市
925 #CN 石川県羽咋郡 石川県羽咋市
926 #CN 石川県鹿島郡 石川県七尾市
927 #CN 石川県珠洲郡 石川県珠洲市 石川県鳳至郡
928 #CN 石川県鳳至郡 石川県輪島市
929 #CN 石川県羽咋郡 石川県羽咋市 石川県河北郡 石川県鹿島郡 石川県石川郡 石川県能美郡 石川県輪島市
930 #CN 富山県上新川郡 富山県中新川郡 富山県婦負郡 富山県富山市
931 #CN 富山県富山市
932 #CN 富山県小矢部市 富山県東砺波郡
933 #CN 富山県高岡市 富山県射水郡 富山県新湊市
934 #CN 富山県高岡市 富山県新湊市
935 #CN 富山県氷見市
936 #CN 富山県滑川市 富山県中新川郡
937 #CN 富山県魚津市
938 #CN 富山県下新川郡 富山県黒部市
939 #CN 石川県羽咋郡 富山県下新川郡 富山県高岡市 富山県射水郡 富山県上新川郡 富山県西砺波郡 富山県東砺波郡 富山県婦負郡 富山県富山市 富山県礪波市
940 #CN 新潟県三島郡 新潟県長岡市 新潟県栃尾市
941 #CN 新潟県糸魚川市
942 #CN 新潟県上越市 新潟県中頚城郡 新潟県東頚城郡
943 #CN 新潟県上越市 新潟県中頚城郡 新潟県東頚城郡
944 #CN 新潟県新井市 新潟県中頚城郡
945 #CN 新潟県刈羽郡 新潟県柏崎市
946 #CN 新潟県北魚沼郡
947 #CN 新潟県古志郡 新潟県三島郡 新潟県小千谷市
948 #CN 新潟県十日町市 新潟県中魚沼郡
949 #CN 新潟県刈羽郡 新潟県三島郡 新潟県糸魚川市 新潟県十日町市 新潟県小千谷市 新潟県上越市 新潟県西頚城郡 新潟県中魚沼郡 新潟県中頚城郡 新潟県南魚沼郡 新潟県柏崎市 新潟県北魚沼郡 長野県下水内郡
950 #CN 新潟県加茂市 新潟県新潟市 新潟県西蒲原郡 新潟県中蒲原郡 新潟県白根市 新潟県豊栄市
951 #CN 新潟県新潟市
952 #CN 新潟県佐渡郡 新潟県両津市
953 #CN 新潟県西蒲原郡
954 #CN 新潟県見附市 新潟県長岡市 新潟県南蒲原郡
955 #CN 新潟県三条市 新潟県南蒲原郡
956 #CN 新潟県新津市 新潟県中蒲原郡
957 #CN 新潟県新発田市 新潟県北蒲原郡
958 #CN 新潟県岩船郡 新潟県村上市
959 #CN 新潟県燕市 新潟県加茂市 新潟県岩船郡 新潟県五泉市 新潟県三島郡 新潟県新発田市 新潟県西蒲原郡 新潟県中蒲原郡 新潟県東蒲原郡 新潟県南蒲原郡 新潟県北蒲原郡
960 #CN 福島県伊達郡 福島県相馬郡 福島県福島市
961 #CN 福島県西白河郡 福島県白河市
962 #CN 福島県岩瀬郡 福島県須賀川市
963 #CN 福島県郡山市 福島県田村郡
964 #CN 福島県安達郡 福島県二本松市
965 #CN 福島県会津若松市 福島県北会津郡
966 #CN 福島県喜多方市 福島県耶麻郡
967 #CN 福島県南会津郡
968 #CN 福島県大沼郡 福島県南会津郡
969 #CN 福島県安達郡 福島県伊達郡 福島県河沼郡 福島県会津若松市 福島県岩瀬郡 福島県郡山市 福島県須賀川市 福島県西白河郡 福島県大沼郡 福島県南会津郡 福島県福島市 福島県北会津郡 福島県耶麻郡
970 #CN 福島県いわき市
971 #CN 福島県いわき市
972 #CN 福島県いわき市
973 #CN 福島県いわき市
974 #CN 福島県いわき市
975 #CN 福島県原町市
976 #CN 福島県相馬市
977 #CN 福島県田村郡
978 #CN 福島県石川郡 福島県東白川郡
979 #CN 福島県いわき市 福島県原町市 福島県石川郡 福島県双葉郡 福島県相馬郡 福島県相馬市 福島県田村郡 福島県東白川郡
980 #CN 宮城県仙台市
981 #CN 宮城県伊具郡 宮城県加美郡 宮城県角田市 宮城県宮城郡 宮城県黒川郡 宮城県仙台市 宮城県桃生郡 宮城県名取市
982 #CN 宮城県仙台市
983 #CN 宮城県仙台市
985 #CN 宮城県塩竃市 宮城県宮城郡 宮城県仙台市 宮城県多賀城市
986 #CN 宮城県牡鹿郡 宮城県石巻市 宮城県桃生郡 宮城県本吉郡
987 #CN 宮城県遠田郡 宮城県玉造郡 宮城県栗原郡 宮城県古川市 宮城県志田郡 宮城県登米郡 宮城県桃生郡
988 #CN 宮城県気仙沼市 宮城県本吉郡
989 #CN 宮城県遠田郡 宮城県刈田郡 宮城県岩沼市 宮城県宮城郡 宮城県玉造郡 宮城県栗原郡 宮城県古川市 宮城県志田郡 宮城県柴田郡 宮城県登米郡 宮城県白石市 宮城県名取郡 宮城県名取市 宮城県亘理郡
990 #CN 山形県寒河江市 山形県山形市 山形県上山市 山形県西村山郡 山形県東村山郡
991 #CN 山形県寒河江市 山形県西村山郡
992 #CN 山形県西置賜郡 山形県長井市 山形県東置賜郡 山形県南陽市 山形県米沢市
993 #CN 山形県長井市
994 #CN 山形県天童市
995 #CN 山形県村山市 山形県尾花沢市
996 #CN 山形県最上郡 山形県新庄市
997 #CN 山形県鶴岡市 山形県東田川郡
998 #CN 山形県酒田市
999 #CN 山形県最上郡 山形県山形市 山形県酒田市 山形県上山市 山形県新庄市 山形県西村山郡 山形県西置賜郡 山形県西田川郡 山形県長井市 山形県鶴岡市 山形県天童市 山形県東根市 山形県東置賜郡 山形県東田川郡 山形県南陽市 山形県尾花沢市 山形県米沢市 山形県飽海郡 山形県北村山郡
TOKYO #CN 東京
