# Copyright 1994 NEC Corporation, Tokyo, Japan.
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without
# fee, provided that the above copyright notice appear in all copies
# and that both that copyright notice and this permission notice
# appear in supporting documentation, and that the name of NEC
# Corporation not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.  NEC Corporation makes no representations about the
# suitability of this software for any purpose.  It is provided "as
# is" without express or implied warranty.
#
# NEC CORPORATION DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN 
# NO EVENT SHALL NEC CORPORATION BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR 
# OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
# PERFORMANCE OF THIS SOFTWARE. 
#
# $Id: software.t,v 1.4 1994/01/27 11:07:29 kuma Exp $
#
あるふぁ #T35 α
いしょくせい #T35 移植性
うへん #T35 右辺
えーあい #T35 ＡＩ
えてぃてぃ #T35 ＡＴ＆Ｔ
えんざんし #T35 演算子
おうようそう #T35 応用層
かい #T35 下位
かいしきごう #T35 開始記号
かいせん #T35 回線
かいせんそくど #T35 回線速度
かいはつかんきょう #T35 開発環境
かいはつほんぶ #T35 開発本部
かし #T35 可視
かしせい #T35 可視性
かそう #T30 仮想
かそうか #T30 仮想化
かためい #T35 型名
かへんぶ #T35 可変部
かりひきすう #T35 仮引数
かんすうがた #T35 関数型
かんすうてきよう #T30 関数適用
かんねつ #T35 感熱
かんねつしき #T35 感熱式
かんやく #T30 簡約
かんりほう #T35 管理法
きこうぞう #T35 木構造
きごうてき #T35 記号的
きしゅいぞん #T35 機種依存
きしゅどくりつ #T35 機種独立
きじゅつし #T35 記述子
きどう #T30 起動
きほう #T35 記法
きほんほうしき #T35 基本方式
きょくしょ #T35 局所
きょくしょてき #T15 局所的
きろくし #T35 記録紙
きんぼう #T35 近傍
ぎじゅつほんぶ #T35 技術本部
ぎほん #T35 技本
くみこみ #T30 組込み
ぐげんか #T30 具現化
けっていせい #T35 決定性
けんきゅうかいはつ #T30 研究開発
けんきゅうしょ #T35 研究所
げんていがた #T35 限定型
こうかい #T35 高階
こうさ #T30 交差
こうぞうたい #T35 構造体
こうぶんかいせき #T30 構文解析
こうぶんき #T35 構文木
こうぶんきごう #T35 構文記号
こうぶんきそく #T35 構文規則
こうぶんぎ #T35 構文木
ごうき #T35 号機
ごうせい #T30 合成
さいき #T35 再帰
さいきてき #T35 再帰的
さいてきか #T30 最適化
さへん #T35 左辺
さよう #T30 作用
さようがた #T35 作用型
さんじゅつ #T35 算術
さんたい #T35 算体
さんぷ #T35 算譜
しーあんどしー #T35 Ｃ＆Ｃ
しきべつし #T35 識別子
しこう #T30 指向
しゅうたん #T35 終端
しょうりゃくじ #T35 省略時
しょうりゃくち #T35 省略値
しょきか #T30 初期化
しょきじょうたい #T35 初期状態
しょきせってい #T35 初期設定
しょきち #T35 初期値
しょりけい #T35 処理系
しようしょ #T35 仕様書
しんらいせい #T35 信頼性
じか #T30 磁化
じけいれつ #T35 時系列
じっすうがた #T35 実数型
じったい #T35 実体
じつじかん #T35 実時間
じつじかんせい #T35 実時間性
じつひきすう #T35 実引数
じょうい #T35 上位
じょうしょ #T35 情処
じょうちょう #T35 冗長
じんこうちのう #T35 人工知能
せいすうがた #T35 整数型
せいせいきそく #T35 生成規則
せいてき #T35 静的
せっていしき #T35 設定式
せんい #T30 遷移
せんたくし #T35 選択肢
そうさ #T30 走査
そうしゅつ #T30 送出
そえじ #T35 添字
ぞくせい #T35 属性
たいいき #T35 大域
たいいきてき #T15 大域的
たこう #T35 多項
たそう #T35 多相
たち #T35 多値
たんこう #T35 単項
だいにゅう #T30 代入
ちゃっこ #T30 着呼
ちゅうしょう #T35 抽象
ちゅうしょうてき #T15 抽象的
ちょうでんどう #T35 超伝導
つうけん #T35 通研
ていきゅう #T35 低級
てんしゃ #T30 転写
てんそう #T30 転送
でんそう #T30 伝送
でんどう #T30 伝導
とうか #T15 等価
とうごう #T35 等号
どうしゅつ #T30 導出
どうてき #T15 動的
なふだ #T35 名札
にこう #T35 二項
にゅうりょく #T30 入力
ねつてんしゃ #T30 熱転写
はいれつ #T30 配列
はいれつがた #T35 配列型
はせいがた #T35 派生型
はっこ #T30 発呼
はんべつし #T35 判別子
ひきすう #T35 引数
ひけってい #T35 非決定
ひけっていせい #T35 非決定性
ひしゅうたん #T35 非終端
ぶそう #T30 武装
ぶそうか #T30 武装化
ぶぶんがた #T35 部分型
ぶぶんしき #T35 部分式
ぶんかかい #T35 分科会
べーた #T35 β
べっかん #T35 別館
ほうほうろん #T35 方法論
みっぺいがた #T35 密閉型
むせいやく #T35 無制約
めいひょう #T35 名標
もじがた #T35 文字型
もじれつ #T35 文字列
もじれつがた #T35 文字列型
よびだし #T30 呼出し
りれきいぞん #T35 履歴依存
れっきょがた #T35 列挙型
わりこみ #T30 割込み
わりだし #T35 割出し
わりつけ #T35 割付け
