# Copyright 1994 NEC Corporation, Tokyo, Japan.
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without
# fee, provided that the above copyright notice appear in all copies
# and that both that copyright notice and this permission notice
# appear in supporting documentation, and that the name of NEC
# Corporation not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.  NEC Corporation makes no representations about the
# suitability of this software for any purpose.  It is provided "as
# is" without express or implied warranty.
#
# NEC CORPORATION DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN 
# NO EVENT SHALL NEC CORPORATION BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR 
# OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
# PERFORMANCE OF THIS SOFTWARE. 
#
# $Id: katakana.t,v 1.3 1994/01/27 11:07:24 kuma Exp $
#
あーかいぶ #T30 アーカイブ
あーきてくちゃ #T35 アーキテクチャ
あーぎゅめんと #T30 アーギュメント
あーちすと #T35 アーチスト
あーと #T30 アート
あーびた #T35 アービタ
あーびとれーしょん #T35 アービトレーション
あーむ #T35 アーム
あーる #T35 アール
あいこん #T35 アイコン
あいてむ #T35 アイテム
あいであ #T30 アイデア
あいどりんぐ #T35 アイドリング
あいどる #T35 アイドル
あうと #T15 アウト
あうとぷっと #T30 アウトプット
あうとらいん #T30 アウトライン
あかうんと #T35 アカウント
あかでみー #T35 アカデミー
あきゅむれーた #T35 アキュムレータ
あくいじしょん #T35 アクイジション
あくせさり #T35 アクセサリ
あくせす #T30 アクセス
あくせられーた #T35 アクセラレータ
あくせんと #T35 アクセント
あくてぃぶ #T15 アクティブ
あさーしょん #T35 アサーション
あすきー #T35 アスキー
あせんぶら #T35 アセンブラ
あせんぶる #T30 アセンブル
あだぷた #T35 アダプタ
あっぱー #T35 アッパー
あっぷ #T30 アップ
あっぷでーと #T30 アップデート
あとりびゅーと #T30 アトリビュート
あどおん #T35 アドオン
あどばいざ #T35 アドバイザ
あどばいす #T30 アドバイス
あどれす #T30 アドレス
あならいざ #T35 アナライザ
あなりしす #T35 アナリシス
あなろぐ #T35 アナログ
あなろじー #T35 アナロジー
あにめ #T35 アニメ
あにめーしょん #T30 アニメーション
あのーど #T35 アノード
あぱちゃ #T35 アパチャ
あふたー #T30 アフター
あぷりけーしょん #T35 アプリケーション
あぷろーち #T30 アプローチ
あぺんど #T30 アペンド
あぼーと #T30 アボート
あま #T35 アマ
あまちゅあ #T15 アマチュア
あらーむ #T30 アラーム
あるごりずむ #T35 アルゴリズム
あるごる #T35 アルゴル
あるふぁべっと #T35 アルファベット
あるみ #T35 アルミ
あれい #T35 アレイ
あろー #T35 アロー
あろーん #T35 アローン
あろけーと #T30 アロケート
あんけーと #T30 アンケート
あんどう #T30 アンドウ
あんぱっく #T35 アンパック
あんふぇのーる #T35 アンフェノール
いーさ #T35 イーサ
いーさねっと #T35 イーサネット
いーまっくす #T35 イーマックス
いたりっく #T35 イタリック
いにしゃらいず #T30 イニシャライズ
いにしゃる #T35 イニシャル
いべんと #T30 イベント
いめーじ #T30 イメージ
いん #T35 イン
いんく #T35 インク
いんくりめんたる #T05 インクリメンタル
いんくりめんと #T30 インクリメント
いんくるーど #T30 インクルード
いんさーと #T30 インサート
いんすとーる #T30 インストール
いんた #T35 インタ
いんたーふぇーす #T35 インターフェース
いんたーぷりた #T35 インタープリタ
いんたねっと #T35 インタネット
いんたびゅー #T30 インタビュー
いんたふぇーす #T30 インタフェース
いんたぷりた #T30 インタプリタ
いんたぷりてぃぶ #T10 インタプリティブ
いんたらくてぃぶ #T10 インタラクティブ
いんたらぷと #T30 インタラプト
いんたりーぶ #T35 インタリーブ
いんてぐれーしょん #T35 インテグレーション
いんてりじぇんと #T30 インテリジェント
いんでぃけーた #T35 インディケータ
いんでっくす #T30 インデックス
いんでんと #T30 インデント
いんぱくと #T35 インパクト
いんふぃっくす #T35 インフィックス
いんぷっと #T30 インプット
いんぷりめんと #T30 インプリメント
いんぽーず #T30 インポーズ
うぃんどう #T35 ウィンドウ
ういんどう #T35 ウインドウ
うぇいと #T30 ウェイト
うえいと #T30 ウエイト
うぉーた #T35 ウォータ
えきすぱーと #T15 エキスパート
えこー #T30 エコー
えっじ #T35 エッジ
えでぃた #T35 エディタ
えでぃっと #T30 エディット
えねるぎー #T35 エネルギー
えぴそーど #T35 エピソード
えみった #T35 エミッタ
えみゅれーしょん #T30 エミュレーション
えみゅれーた #T35 エミュレータ
えみゅれーと #T30 エミュレート
えらー #T30 エラー
えりあ #T35 エリア
えれがんと #T05 エレガント
えれくとろにくす #T35 エレクトロニクス
えれめんと #T35 エレメント
えんこーだ #T35 エンコーダ
えんじにあ #T35 エンジニア
えんじにありんぐ #T35 エンジニアリング
えんじん #T35 エンジン
えんてぃてぃ #T35 エンティティ
えんとり #T30 エントリ
えんとりー #T30 エントリー
えんど #T30 エンド
おーだ #T30 オーダ
おーでぃお #T35 オーディオ
おーと #T35 オート
おーとまちっく #T15 オートマチック
おーとまてぃずむ #T35 オートマティズム
おーとまとん #T35 オートマトン
おーとめーしょん #T35 オートメーション
おーば #T10 オーバ
おーばー #T10 オーバー
おーばぱんち #T30 オーバパンチ
おーばへっど #T30 オーバヘッド
おーばれい #T30 オーバレイ
おーぷん #T10 オープン
おくたる #T35 オクタル
おす #T30 オス
おふ #T35 オフ
おふぃす #T35 オフィス
おふこん #T35 オフコン
おふせっと #T30 オフセット
おぶじぇくと #T35 オブジェクト
おぷしょん #T35 オプション
おぺらんど #T35 オペランド
おぺれーしょん #T30 オペレーション
おぺれーしょんず #T35 オペレーションズ
おぺれーてぃんぐ #T35 オペレーティング
おりじなる #T05 オリジナル
おりんぴっく #T35 オリンピック
おん #T30 オン
おんざふらい #T35 オンザフライ
おんぼーど #T35 オンボード
おんらいん #T35 オンライン
かーそる #T35 カーソル
かーとりっじ #T35 カートリッジ
かーど #T35 カード
かーねる #T35 カーネル
かーぶ #T30 カーブ
かーぼん #T35 カーボン
かうんた #T35 カウンタ
かうんと #T30 カウント
かすけーど #T35 カスケード
かすたまいず #T30 カスタマイズ
かすたむ #T25 カスタム
かせっと #T35 カセット
かそーど #T35 カソード
かたかな #T35 カタカナ
かたろぐ #T35 カタログ
かっと #T30 カット
かっぷ #T35 カップ
かな #T35 カナ
かばー #T30 カバー
かぷら #T35 カプラ
からー #T35 カラー
からむ #T35 カラム
かりきゅらむ #T30 カリキュラム
かるちゃー #T35 カルチャー
かるのー #T35 カルノー
かるろ #T35 カルロ
かれんと #T35 カレント
かろりー #T35 カロリー
がーど #T30 ガード
がーべっじ #T35 ガーベッジ
がいだんす #T35 ガイダンス
がいど #T35 ガイド
がいどらいん #T35 ガイドライン
きー #T35 キー
きーぼーど #T35 キーボード
きゃっしゅ #T30 キャッシュ
きゃぱしてぃ #T35 キャパシティ
きゃぴたる #T35 キャピタル
きゃぷてん #T35 キャプテン
きゃらくた #T35 キャラクタ
きゃりっじ #T35 キャリッジ
きゃんせる #T30 キャンセル
きゅー #T35 キュー
きゅーいんぐ #T30 キューイング
きゅーぶ #T35 キューブ
ぎが #T35 ギガ
くぉーと #T30 クォート
くらいあんと #T35 クライアント
くらす #T35 クラス
くらっしゅ #T30 クラッシュ
くりあ #T30 クリア
くりえーと #T30 クリエート
くりえいてぃぶ #T15 クリエイティブ
くりっく #T30 クリック
くりっぷ #T30 クリップ
くろーず #T30 クローズ
くろーずど #T30 クローズド
くろす #T30 クロス
くろすふっと #T35 クロスフット
くろっく #T35 クロック
くろむ #T35 クロム
ぐらうんど #T35 グラウンド
ぐらふ #T35 グラフ
ぐらふぃっく #T35 グラフィック
ぐらふぃっくす #T35 グラフィックス
ぐるーぷ #T35 グループ
ぐれーど #T35 グレード
ぐれい #T05 グレイ
ぐろーばる #T35 グローバル
けーす #T35 ケース
けーぶる #T35 ケーブル
けあ #T35 ケア
げーと #T35 ゲート
げーとうぇい #T35 ゲートウェイ
げーむ #T30 ゲーム
こーす #T35 コース
こーち #T30 コーチ
こーでぃんぐ #T30 コーディング
こーど #T35 コード
こーる #T30 コール
こあ #T35 コア
こすと #T30 コスト
こねくしょん #T35 コネクション
こねくた #T35 コネクタ
こぴー #T30 コピー
こぷろせっさ #T35 コプロセッサ
こぼる #T35 コボル
こまんだー #T35 コマンダー
こまんど #T35 コマンド
こみゅにけーしょん #T30 コミュニケーション
こみゅにてぃ #T35 コミュニティ
こめんと #T35 コメント
こもん #T35 コモン
こらむ #T35 コラム
これくしょん #T30 コレクション
これくた #T35 コレクタ
これくと #T35 コレクト
こんかれんし #T35 コンカレンシ
こんかれんと #T35 コンカレント
こんせぷと #T30 コンセプト
こんそーる #T35 コンソール
こんだくた #T35 コンダクタ
こんてくすと #T35 コンテクスト
こんてすと #T35 コンテスト
こんてんしょん #T35 コンテンション
こんてんつ #T35 コンテンツ
こんとろーら #T35 コントローラ
こんとろーる #T30 コントロール
こんばーた #T35 コンバータ
こんぱいら #T35 コンパイラ
こんぱいる #T30 コンパイル
こんぱくと #T05 コンパクト
こんぱち #T35 コンパチ
こんぱちぶる #T35 コンパチブル
こんびねーた #T35 コンビネータ
こんぴゅーた #T35 コンピュータ
こんぴゅーたー #T35 コンピューター
こんぴゅーてぃんぐ #T35 コンピューティング
こんふぃぎゅれーしょん #T35 コンフィギュレーション
ごーる #T30 ゴール
さーきっと #T35 サーキット
さーち #T30 サーチ
さーど #T35 サード
さーば #T35 サーバ
さーびす #T30 サービス
さいえんす #T35 サイエンス
さいくる #T35 サイクル
さいず #T35 サイズ
さいと #T35 サイト
さいど #T35 サイド
さいばねてぃっくす #T35 サイバネティックス
さうんど #T35 サウンド
さぶ #T35 サブ
さぶるーちん #T35 サブルーチン
さぽーと #T30 サポート
さむ #T35 サム
さんぷりんぐ #T30 サンプリング
さんぷる #T30 サンプル
ざぐ #T35 ザグ
しーく #T30 シーク
しーけんす #T30 シーケンス
しーと #T30 シート
しーん #T35 シーン
しぇーでぃんぐ #T35 シェーディング
しぇーど #T35 シェード
しぇあ #T30 シェア
しぇあど #T35 シェアド
しぇありんぐ #T30 シェアリング
しぇる #T35 シェル
しぐなる #T35 シグナル
しぐま #T35 シグマ Σ
しすてむ #T35 システム
しそーらす #T35 シソーラス
しっぷ #T35 シップ
しなりお #T35 シナリオ
しふた #T35 シフタ
しふと #T30 シフト
しみゅれーしょん #T30 シミュレーション
しみゅれーた #T35 シミュレータ
しみゅれーと #T30 シミュレート
しゃーし #T35 シャーシ
しゃーぷ #T05 シャープ
しゃわー #T35 シャワー
しょー #T35 ショー
しょーと #T30 ショート
しょう #T35 ショウ
しょっぷ #T35 ショップ
しりーず #T35 シリーズ
しりある #T35 シリアル
しりこん #T35 シリコン
しりんだ #T35 シリンダ
しんく #T35 シンク
しんたっくす #T35 シンタックス
しんぷる #T05 シンプル
しんぼりっく #T35 シンボリック
しんぼる #T35 シンボル
じぇっと #T35 ジェット
じぇねれーしょん #T35 ジェネレーション
じぇねれーた #T35 ジェネレータ
じぐ #T35 ジグ
じゃーなる #T35 ジャーナル
じゃぱん #T35 ジャパン
じゃんぷ #T30 ジャンプ
じょーく #T35 ジョーク
じょい #T35 ジョイ
じょいすてぃっく #T35 ジョイスティック
じょぶ #T35 ジョブ
すーぱ #T35 スーパ
すーぱー #T35 スーパー
すーぱばいざ #T35 スーパバイザ
すいっち #T30 スイッチ
すいっちんぐ #T30 スイッチング
すから #T30 スカラ
すきー #T30 スキー
すきゃな #T35 スキャナ
すきゃん #T30 スキャン
すきゅー #T30 スキュー
すくーぷ #T30 スクープ
すくらっぷ #T30 スクラップ
すくりーん #T35 スクリーン
すくりぷと #T30 スクリプト
すけじゅーら #T35 スケジューラ
すけじゅーる #T30 スケジュール
すけるとん #T35 スケルトン
すこーぷ #T35 スコープ
すたーと #T30 スタート
すたいる #T35 スタイル
すたっか #T35 スタッカ
すたっく #T30 スタック
すたっふ #T35 スタッフ
すたてぃさいざ #T30 スタティサイザ
すたてぃっく #T35 スタティック
すたんだーど #T35 スタンダード
すたんど #T35 スタンド
すたんばい #T30 スタンバイ
すてーしょん #T35 ステーション
すてーじ #T35 ステージ
すてーたす #T35 ステータス
すてーと #T35 ステート
すてぃっく #T30 スティック
すてっぴんぐ #T35 ステッピング
すてっぷ #T30 ステップ
すとーる #T30 ストール
すとっぷ #T30 ストップ
すとらいぷ #T35 ストライプ
すとらくと #T30 ストラクト
すとりーま #T35 ストリーマ
すとりーむ #T35 ストリーム
すとりんぐ #T35 ストリング
すなっぷしょっと #T30 スナップショット
すぴーかー #T35 スピーカー
すぴーど #T35 スピード
すぷーりんぐ #T30 スプーリング
すぷーる #T30 スプール
すぷらいと #T35 スプライト
すぷらいん #T35 スプライン
すぷれー #T30 スプレー
すぷれっど #T35 スプレッド
すぺーさ #T35 スペーサ
すぺーしんぐ #T35 スペーシング
すぺーす #T30 スペース
すぺくとる #T35 スペクトル
すぺしゃる #T35 スペシャル
すぺっく #T35 スペック
すぽーつ #T30 スポーツ
すぽっと #T35 スポット
すぽんさー #T35 スポンサー
すむーじんぐ #T35 スムージング
すむーず #T05 スムーズ
すもーる #T05 スモール
すらいしんぐ #T35 スライシング
するー #T30 スルー
するーぷっと #T30 スループット
すれーぶ #T30 スレーブ
すれっど #T35 スレッド
すろっと #T35 スロット
すわっぴんぐ #T30 スワッピング
すわっぷ #T30 スワップ
せーぶ #T30 セーブ
せきゅりてぃ #T35 セキュリティ
せくた #T35 セクタ
せくたー #T35 セクター
せぐめんと #T30 セグメント
せっしょん #T30 セッション
せっと #T30 セット
せまふぉ #T35 セマフォ
せまんてぃく #T35 セマンティク
せまんてぃくす #T35 セマンティクス
せみなー #T30 セミナー
せる #T35 セル
せるふ #T15 セルフ
せれくしょん #T30 セレクション
せれくた #T35 セレクタ
せれくてぃんぐ #T30 セレクティング
せんす #T30 センス
せんた #T35 センタ
せんたー #T35 センター
せんたりんぐ #T30 センタリング
せんとろ #T35 セントロ
せんとろにくす #T35 セントロニクス
ぜねれーた #T35 ゼネレータ
ぜろ #T35 ゼロ
そーす #T35 ソース
そーてぃんぐ #T30 ソーティング
そーと #T30 ソート
そけっと #T35 ソケット
そふと #T35 ソフト
そふとうぇあ #T35 ソフトウェア
そふとうえあ #T35 ソフトウエア
ぞーん #T35 ゾーン
たーげっと #T35 ターゲット
たーみなる #T30 ターミナル
たーんあらうんど #T30 ターンアラウンド
たいとる #T35 タイトル
たいぴんぐ #T30 タイピング
たいぷ #T30 タイプ
たいま #T35 タイマ
たいみんぐ #T35 タイミング
たいむ #T30 タイム
たすきんぐ #T30 タスキング
たすく #T35 タスク
たっち #T30 タッチ
たぶ #T35 タブ
たぶれっと #T30 タブレット
たんく #T35 タンク
だいある #T30 ダイアル
だいあるいん #T30 ダイアルイン
だいあろぐ #T35 ダイアログ
だいなみさいざ #T35 ダイナミサイザ
だいなみっく #T15 ダイナミック
だうん #T30 ダウン
だぶる #T30 ダブル
だみー #T30 ダミー
だめーじ #T30 ダメージ
だらす #T30 ダラス
だんぷ #T30 ダンプ
ちーむ #T35 チーム
ちぇーん #T35 チェーン
ちぇいん #T30 チェイン
ちぇっく #T30 チェック
ちっぷ #T30 チップ
ちゃど #T35 チャド
ちゃどれす #T35 チャドレス
ちゃねる #T35 チャネル
ちゃんす #T15 チャンス
ちゃんねる #T35 チャンネル
ちゅーりんぐ #T30 チューリング
つーる #T35 ツール
つあー #T35 ツアー
ついん #T35 ツイン
つりー #T35 ツリー
てーぶる #T35 テーブル
てーぷ #T30 テープ
てーま #T35 テーマ
てきさす #T30 テキサス
てきすと #T30 テキスト
てく #T30 テク
てくにっく #T35 テクニック
てくの #T35 テクノ
てくのろじー #T35 テクノロジー
てすと #T30 テスト
てら #T30 テラ
てるねっと #T35 テルネット
てれたいぷ #T35 テレタイプ
てれっくす #T30 テレックス
てれび #T30 テレビ
てん #T35 テン
でーた #T35 データ
でーもん #T35 デーモン
でぃじたる #T35 ディジタル
でぃすく #T35 ディスク
でぃすくりぷた #T35 ディスクリプタ
でぃすけっと #T30 ディスケット
でぃすとりびゅーしょん #T35 ディストリビューション
でぃすぱっち #T30 ディスパッチ
でぃすぱっちゃ #T35 ディスパッチャ
でぃすぷれい #T35 ディスプレイ
でぃっぷ #T30 ディップ
でぃれい #T35 ディレイ
でぃれくとり #T35 ディレクトリ
でいじー #T35 デイジー
でこーだ #T35 デコーダ
でざいん #T30 デザイン
でしまる #T35 デシマル
でじたる #T35 デジタル
ですく #T35 デスク
でっく #T30 デック
でばいす #T35 デバイス
でばっが #T35 デバッガ
でばっぐ #T30 デバッグ
でふぉると #T35 デフォルト
でめりっと #T05 デメリット
でも #T30 デモ
でもんすとれーしょん #T35 デモンストレーション
でゅある #T35 デュアル
でりけーと #T15 デリケート
とーく #T30 トーク
とーたる #T30 トータル
とーん #T35 トーン
とっぷ #T35 トップ
とぴっくす #T35 トピックス
とらくた #T35 トラクタ
とらっきんぐ #T35 トラッキング
とらっく #T30 トラック
とらっぷ #T30 トラップ
とらふぃっく #T30 トラフィック
とらぶる #T30 トラブル
とらんざくしょん #T30 トランザクション
とらんしーば #T35 トランシーバ
とらんじすた #T35 トランジスタ
とらんす #T30 トランス
とらんすぺあれんし #T30 トランスペアレンシ
とらんすぺあれんと #T30 トランスペアレント
とらんすぽーた #T35 トランスポータ
とらんすぽーと #T30 トランスポート
とらんすれーた #T35 トランスレータ
とりが #T30 トリガ
とりがー #T30 トリガー
とれーさ #T35 トレーサ
とれーしんぐ #T35 トレーシング
とれーす #T30 トレース
とれーな #T35 トレーナ
とろん #T35 TRON トロン
どきゅめんてーしょん #T35 ドキュメンテーション
どきゅめんと #T30 ドキュメント
どっと #T35 ドット
どめいん #T30 ドメイン
どらいば #T35 ドライバ
どらいぶ #T30 ドライブ
どらっきんぐ #T30 ドラッキング
どらっぐ #T30 ドラッグ
どらふと #T30 ドラフト
どらむ #T35 ドラム
どろっぷ #T30 ドロップ
ないふ #T35 ナイフ
なんばー #T30 ナンバー
にーず #T35 ニーズ
にっかど #T35 ニッカド
にゅー #T35 ニュー
にゅーす #T30 ニュース
にゅーらる #T35 ニューラル
にゅーろ #T35 ニューロ
にゅあんす #T30 ニュアンス
ねーみんぐ #T30 ネーミング
ねーむ #T30 ネーム
ねがてぃぶ #T30 ネガティブ
ねっく #T35 ネック
ねっと #T30 ネット
ねっとわーきんぐ #T30 ネットワーキング
ねっとわーく #T30 ネットワーク
のー #T35 ノー
のーど #T35 ノード
のーまる #T05 ノーマル
のうはう #T35 ノウハウ
のんいんぱくと #T30 ノンインパクト
のんろっきんぐ #T30 ノンロッキング
はーと #T35 ハート
はーど #T05 ハード
はーどうぇあ #T35 ハードウェア
はーどうえあ #T35 ハードウエア
はい #T15 ハイ
はいぐれーど #T15 ハイグレード
はいぶりっど #T35 ハイブリッド
はいらいと #T35 ハイライト
はうす #T35 ハウス
はうすきーぴんぐ #T30 ハウスキーピング
はっかー #T35 ハッカー
はっしゅ #T30 ハッシュ
はみんぐ #T30 ハミング
はんだ #T35 ハンダ
はんど #T35 ハンド
はんどしぇーく #T30 ハンドシェーク
はんどら #T35 ハンドラ
はんどりんぐ #T30 ハンドリング
はんどる #T35 ハンドル
はんま #T35 ハンマ
ばー #T35 バー
ばーじょん #T35 バージョン
ばーすと #T30 バースト
ばーちゃる #T30 バーチャル
ばいち #T35 バイチ
ばいと #T30 バイト
ばいなり #T35 バイナリ
ばいんど #T30 バインド
ばうんど #T30 バウンド
ばぐ #T35 バグ
ばす #T35 バス
ばっく #T30 バック
ばっくとらっく #T30 バックトラック
ばっち #T35 バッチ
ばってり #T35 バッテリ
ばっふぁ #T30 バッファ
ばっふぁりんぐ #T30 バッファリング
ばらっく #T35 バラック
ばんく #T30 バンク
ばんど #T35 バンド
ぱーせんと #T35 パーセント
ぱーそなる #T35 パーソナル
ぱーてぃ #T30 パーティ
ぱーふぉまんす #T30 パーフォマンス
ぱーみっしょん #T35 パーミッション
ぱいぷ #T30 パイプ
ぱうだー #T35 パウダー
ぱけっと #T30 パケット
ぱす #T30 パス
ぱそこん #T35 パソコン
ぱたーん #T35 パターン
ぱっく #T30 パック
ぱっけーじ #T30 パッケージ
ぱっち #T35 パッチ
ぱねる #T30 パネル
ぱふぉーまんす #T30 パフォーマンス
ぱぶりっく #T35 パブリック
ぱぶりっしんぐ #T35 パブリッシング
ぱら #T35 パラ
ぱらぐらふ #T35 パラグラフ
ぱらだいむ #T35 パラダイム
ぱらめーた #T35 パラメータ
ぱらめとろん #T35 パラメトロン
ぱられる #T15 パラレル
ぱりてぃ #T35 パリティ
ぱるす #T35 パルス
ぱれっと #T35 パレット
ぱわー #T35 パワー
ぱんち #T30 パンチ
ひっと #T30 ヒット
ひゅーじ #T30 ヒュージ
ひゅーまん #T30 ヒューマン
ひゅーれっと #T30 ヒューレット
びーむ #T30 ビーム
びじねす #T30 ビジネス
びじゅある #T35 ビジュアル
びじょん #T30 ビジョン
びたみん #T35 ビタミン
びっと #T35 ビット
びでお #T30 ビデオ
びる #T35 ビル
ぴくせる #T30 ピクセル
ぴくちゃ #T35 ピクチャ
ぴこ #T30 ピコ
ぴっく #T30 ピック
ぴったり #T05 ピッタリ
ぴっち #T30 ピッチ
ぴらみっど #T35 ピラミッド
ぴりおど #T35 ピリオド
ぴん #T30 ピン
ぴんぼーど #T35 ピンボード
ふーど #T30 フード
ふぁーすと #T30 ファースト
ふぁいなる #T35 ファイナル
ふぁいる #T30 ファイル
ふぁくしみり #T30 ファクシミリ
ふぁくた #T35 ファクタ
ふぁくたー #T30 ファクター
ふぁくと #T30 ファクト
ふぁくとり #T35 ファクトリ
ふぁしりてぃ #T30 ファシリティ
ふぁすと #T30 ファスト
ふぁっくす #T35 ファックス
ふぁっしょん #T35 ファッション
ふぁみこん #T30 ファミコン
ふぁん #T30 ファン
ふぁんくしょん #T30 ファンクション
ふぃーど #T30 フィード
ふぃーるど #T30 フィールド
ふぃくしょん #T30 フィクション
ふぃぼなっち #T30 フィボナッチ
ふぃるむ #T35 フィルム
ふぇーす #T35 フェース
ふぇーず #T30 フェーズ
ふぉーく #T30 フォーク
ふぉーとらん #T30 フォートラン
ふぉーまっと #T30 フォーマット
ふぉーまる #T30 フォーマル
ふぉーむ #T30 フォーム
ふぉーる #T30 フォール
ふぉーると #T30 フォールト
ふぉと #T35 フォト
ふぉるだ #T35 フォルダ
ふぉるど #T35 フォルド
ふぉるむ #T35 フォルム
ふぉろー #T30 フォロー
ふぉろあ #T30 フォロア
ふぉん #T30 フォン
ふぉんと #T35 フォント
ふっきんぐ #T30 フッキング
ふっく #T30 フック
ふらぐ #T35 フラグ
ふらっしゅ #T30 フラッシュ
ふりくしょん #T30 フリクション
ふりっぷふろっぷ #T35 フリップフロップ
ふるーつ #T35 フルーツ
ふれーず #T35 フレーズ
ふれーむ #T30 フレーム
ふれきしぶる #T35 フレキシブル
ふれんどりー #T15 フレンドリー
ふろー #T30 フロー
ふろーてぃんぐ #T30 フローティング
ふろーと #T30 フロート
ふろあ #T35 フロア
ふろっぴー #T30 フロッピー
ふろんと #T35 フロント
ぶーと #T30 ブート
ぶーとすとらっぷ #T30 ブートストラップ
ぶーりあん #T30 ブーリアン
ぶーる #T35 ブール
ぶっく #T30 ブック
ぶらうん #T05 ブラウン
ぶらんきんぐ #T30 ブランキング
ぶらんち #T35 ブランチ
ぶりんきんぐ #T30 ブリンキング
ぶれーき #T30 ブレーキ
ぶれーく #T30 ブレーク
ぶれーん #T35 ブレーン
ぶろっく #T30 ブロック
ぷっしゅ #T30 プッシュ
ぷらいおりてぃ #T30 プライオリティ
ぷらいばしー #T35 プライバシー
ぷらいべーと #T35 プライベート
ぷらくてぃす #T30 プラクティス
ぷらぐまてぃくす #T35 プラグマティクス
ぷらん #T30 プラン
ぷりあんぶる #T35 プリアンブル
ぷりぷろせっさ #T35 プリプロセッサ
ぷりみてぃぶ #T10 プリミティブ
ぷりんすとん #T30 プリンストン
ぷりんた #T35 プリンタ
ぷりんてぃんぐ #T35 プリンティング
ぷりんと #T30 プリント
ぷるだうん #T30 プルダウン
ぷれーと #T35 プレート
ぷれーん #T30 プレーン
ぷれいや #T30 プレイヤ
ぷれす #T30 プレス
ぷれぜんてーしょん #T35 プレゼンテーション
ぷれぜんてーと #T30 プレゼンテート
ぷれぜんと #T30 プレゼント
ぷれてっど #T35 プレテッド
ぷれふぃっくす #T35 プレフィックス
ぷろ #T35 プロ
ぷろーぶ #T35 プローブ
ぷろぐらま #T35 プログラマ
ぷろぐらみんぐ #T30 プログラミング
ぷろぐらむ #T30 プログラム
ぷろしじゃ #T35 プロシジャ
ぷろじぇくと #T35 プロジェクト
ぷろせさ #T35 プロセサ
ぷろせす #T35 プロセス
ぷろせっさ #T35 プロセッサ
ぷろせっしんぐ #T30 プロセッシング
ぷろだくしょん #T35 プロダクション
ぷろだくと #T30 プロダクト
ぷろった #T30 プロッタ
ぷろていん #T35 プロテイン
ぷろてくと #T30 プロテクト
ぷろと #T35 プロト
ぷろとこる #T35 プロトコル
ぷろとたいぴんぐ #T35 プロトタイピング
ぷろとたいぷ #T35 プロトタイプ
ぷろふぇっしょなる #T10 プロフェッショナル
ぷろんぷと #T35 プロンプト
へきさ #T35 ヘキサ
へくたーる #T35 ヘクタール
へくと #T35 ヘクト
へっだ #T35 ヘッダ
へっでぃんぐ #T30 ヘッディング
へっど #T35 ヘッド
へっどほん #T35 ヘッドホン
べーす #T35 ベース
べくた #T35 ベクタ
べくとる #T35 ベクトル
べすと #T15 ベスト
べる #T35 ベル
べん #T35 ベン
べんだ #T35 ベンダ
べんち #T35 ベンチ
べんちまーく #T30 ベンチマーク
べんちゃー #T35 ベンチャー
ぺーじ #T35 ページ
ぺーじんぐ #T30 ページング
ぺーすと #T30 ペースト
ぺーぱ #T35 ペーパ
ぺりふぇらる #T35 ペリフェラル
ぺん #T35 ペン
ぺんしょん #T35 ペンション
ほーむ #T35 ホーム
ほーる #T35 ホール
ほーん #T35 ホーン
ほいーる #T35 ホイール
ほこり #T35 ホコリ
ほすと #T30 ホスト
ほっぱ #T35 ホッパ
ほるまんと #T35 ホルマント
ほろぐらむ #T35 ホログラム
ぼー #T35 ボー
ぼーと #T35 ボート
ぼーど #T35 ボード
ぼーる #T35 ボール
ぼーるど #T15 ボールド
ぼたん #T35 ボタン
ぼっくす #T35 ボックス
ぼでぃ #T35 ボディ
ぼとむ #T35 ボトム
ぼとる #T35 ボトル
ぼりゅーむ #T35 ボリューム
ぽーたびりぃてぃ #T35 ポータビリィティ
ぽーたぶる #T15 ポータブル
ぽーと #T30 ポート
ぽーらんど #CN ポーランド
ぽーりんぐ #T30 ポーリング
ぽいてぃんぐ #T35 ポイティング
ぽいんた #T35 ポインタ
ぽいんと #T30 ポイント
ぽけっと #T35 ポケット
ぽじてぃぶ #T35 ポジティブ
ぽすと #T30 ポスト
ぽすとあんぶる #T35 ポストアンブル
ぽすとふぃっくす #T35 ポストフィックス
ぽすとぷろせっさ #T35 ポストプロセッサ
ぽっぷ #T30 ポップ
ぽぴゅらー #T05 ポピュラー
ぽり #T35 ポリ
ぽりごん #T35 ポリゴン
ぽんぷ #T35 ポンプ
まーか #T35 マーカ
まーく #T30 マーク
まーじ #T30 マージ
まいくろ #T35 マイクロ
まいこん #T35 マイコン
まいな #T05 マイナ
まいなー #T05 マイナー
まうす #T35 マウス
まうんと #T30 マウント
まくろ #T35 マクロ
まくろぷろせっさ #T30 マクロプロセッサ
まぐかっぷ #T35 マグカップ
まぐねてぃっく #T35 マグネティック
まざーぼーど #T35 マザーボード
ましん #T35 マシン
まじっく #T35 マジック
ますく #T30 マスク
ますた #T30 マスタ
ますたー #T30 マスター
まっきんとっしゅ #T35 マッキントッシュ
まっく #T35 マック
まっち #T30 マッチ
まっぴんぐ #T30 マッピング
まっぷ #T35 マップ
まとりくす #T35 マトリクス
まとりっくす #T35 マトリックス
まにあ #T15 マニア
まにゅある #T35 マニュアル
まねーじゃ #T35 マネージャ
まねじめんと #T30 マネジメント
まるち #T35 マルチ
まるちたすきんぐ #T30 マルチタスキング
まるちぶれくさ #T35 マルチブレクサ
まん #T35 マン
みーてぃんぐ #T30 ミーティング
みくろ #T35 ミクロ
みす #T30 ミス
みっくす #T30 ミックス
みに #T15 ミニ
みにこん #T35 ミニコン
みねらる #T35 ミネラル
みらー #T35 ミラー
みるく #T35 ミルク
むーど #T30 ムード
めーか #T35 メーカ
めーりんぐ #T35 メーリング
めーる #T30 メール
めいく #T30 メイク
めいん #T35 メイン
めいんてなんす #T30 メインテナンス
めかにずむ #T35 メカニズム
めじゃー #T15 メジャー
めす #T35 メス
めた #T35 メタ
めっせーじ #T30 メッセージ
めでぃあ #T35 メディア
めにゅー #T35 メニュー
めも #T30 メモ
めもり #T35 メモリ
めりっと #T35 メリット
めんてなんす #T30 メンテナンス
めんば #T35 メンバ
めんばー #T35 メンバー
もーだる #T35 モーダル
もーてむ #T35 モーテム
もーど #T35 モード
もじゅーる #T35 モジュール
もじゅらー #T10 モジュラー
もじゅろ #T35 モジュロ
もでむ #T35 モデム
もでる #T35 モデル
もとろーら #T35 モトローラ
もにた #T35 モニタ
もの #T35 モノ
もんて #T35 モンテ
ゆーざ #T35 ユーザ
ゆーてぃりてぃ #T35 ユーティリティ
ゆにーく #T05 ユニーク
ゆにっくす #T35 ユニックス
ゆにっと #T35 ユニット
らーじ #T35 ラージ
らいく #T05 ライク
らいた #T35 ライタ
らいてぃんぐ #T30 ライティング
らいと #T35 ライト
らいふ #T35 ライフ
らいぶらり #T35 ライブラリ
らいん #T35 ライン
らすた #T30 ラスタ
らっく #T35 ラック
らっち #T30 ラッチ
らっぴんぐ #T30 ラッピング
らっぷ #T30 ラップ
らぴっど #T35 ラピッド
らべる #T35 ラベル
らむだ #T35 ラムダ
らんく #T30 ランク
らんだむ #T15 ランダム
らんにんぐ #T35 ランニング
りーだ #T35 リーダ
りーど #T30 リード
りあ #T30 リア
りある #T05 リアル
りくえすと #T30 リクエスト
りさーち #T30 リサーチ
りすと #T35 リスト
りすとあ #T30 リストア
りすぷ #T35 リスプ
りせっと #T30 リセット
りそーす #T35 リソース
りたーげったぶる #T30 リターゲッタブル
りたーん #T30 リターン
りたらしー #T35 リタラシー
りだくしょん #T35 リダクション
りてらる #T30 リテラル
りとらい #T30 リトライ
りにあ #T30 リニア
りふぁれんす #T35 リファレンス
りふれっしゅ #T30 リフレッシュ
りぷれーす #T30 リプレース
りぼん #T35 リボン
りぽーと #T30 リポート
りみっと #T35 リミット
りもーと #T30 リモート
りりーす #T30 リリース
りれー #T30 リレー
りれーしょなる #T35 リレーショナル
りれーしょん #T30 リレーション
りれーしょんしっぷ #T35 リレーションシップ
りんか #T35 リンカ
りんかー #T35 リンカー
りんく #T30 リンク
りんぐ #T30 リング
りんけーじ #T30 リンケージ
るーちん #T35 ルーチン
るーと #T30 ルート
るーぷ #T30 ループ
るーる #T30 ルール
るかーしゅびっち #T35 ルカーシュビッチ
れーざ #T35 レーザ
れーと #T35 レート
れいあうと #T30 レイアウト
れいや #T35 レイヤ
れこーど #T30 レコード
れしーば #T35 レシーバ
れしーぶ #T30 レシーブ
れじすた #T35 レジスタ
れじでんと #T35 レジデント
れす #T35 レス
れぞりゅーしょん #T35 レゾリューション
れたー #T35 レター
れびゅー #T30 レビュー
れべる #T35 レベル
れぽーと #T30 レポート
れんじ #T35 レンジ
ろー #T35 ロー
ろーあ #T35 ローア
ろーかる #T15 ローカル
ろーぜっと #T35 ローゼット
ろーだ #T35 ローダ
ろーど #T30 ロード
ろーま #CN ローマ
ろーる #T30 ロール
ろあ #T35 ロア
ろがー #T35 ロガー
ろぐ #T35 ログ
ろぐあうと #T30 ログアウト
ろぐいん #T30 ログイン
ろじかる #T15 ロジカル
ろじっく #T35 ロジック
ろっきんぐ #T30 ロッキング
ろっく #T30 ロック
ろぼっと #T35 ロボット
ろんぐ #T05 ロング
わーきんぐ #T35 ワーキング
わーく #T30 ワーク
わーすと #T35 ワースト
わーど #T35 ワード
わーぷろ #T35 ワープロ
わーるど #T35 ワールド
わいあ #T35 ワイア
わいど #T05 ワイド
わいぱー #T35 ワイパー
わいや #T35 ワイヤ
わん #T35 ワン
わんたっち #T15 ワンタッチ
