# Copyright 1994 NEC Corporation, Tokyo, Japan.
#
# Permission to use, copy, modify, distribute and sell this software
# and its documentation for any purpose is hereby granted without
# fee, provided that the above copyright notice appear in all copies
# and that both that copyright notice and this permission notice
# appear in supporting documentation, and that the name of NEC
# Corporation not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.  NEC Corporation makes no representations about the
# suitability of this software for any purpose.  It is provided "as
# is" without express or implied warranty.
#
# NEC CORPORATION DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN 
# NO EVENT SHALL NEC CORPORATION BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR 
# OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
# PERFORMANCE OF THIS SOFTWARE. 
#
# $Id: number.t,v 1.2 1994/06/01 08:18:12 kon Exp $
#
い #JS*68 位
いく #NN*3 幾
いち #NN*255 一
いちおく #N00 一億
いちまん #N00 一万
いっ #NN*59 一
いっせん #N01 一千
いっちょう #N00 一兆
いんち #JS*10 インチ #JS*0 吋
えん #JS*131 円
えんだか #JS*2 円高
おく #N00 億
か #JS*68 日 #JS*6 課 #JS*1 価
かい #JS*184 回 #JS*87 階
かいき #JS*1 回忌
かいきゅう #JS*2 階級
かいせい #JS*6 回生
かいせん #JS*80 回線 #JS*1 回戦
かいそう #JS*37 階層
かいてん #JS*3 回転
かいり #JS*0 カイリ 海里 浬
かく #JS*8 角 #JS*1 画
かげつ #JS*16 カ月 #JS*10 ヶ月 #JS*3 か月 #JS*1 ヵ月
かこく #JS*2 カ国 #JS*0 ヶ国
かしょ #JS*25 箇所 #JS*4 ヶ所
かじょう #JS*1 ヶ条 箇条 #JS*0 か条
かづけ #JS*0 日付
かねん #JS*0 か年 ヶ年
かぶ #JS*62 株
かもく #JS*2 科目
かろりー #JS*1 カロリー
かん #JS*8 巻 #JS*2 缶 #JSSUC 間 #JS*0 貫
かんく #JS*0 管区
がい #JS*4 階
がお #JS*1 顔
がくしょう #JS*1 楽章
がくねん #JS*2 学年
がっき #JS*4 学期
がっきゅう #JS*0 学級
がっく #JS*0 学区
がつ #JS*472 月
がつごう #JS*0 月号
がつまつ #JS*1 月末
き #JS*25 期 #JS*2 基 機 #JS*0 騎
きせい #JS*1 期生
きゃく #JS*2 脚 #JS*0 客
きゅう #NN*14 九 #JS*1 球 #JS*0 級
きゅうおく #N00 九億
きゅうじゅう #N03 九十
きゅうせん #N01 九千
きゅうちょう #N00 九兆
きゅうひゃく #N02 九百
きゅうまん #N00 九万
きょう #JSSUC 強
きょく #JS*19 曲
きろ #JS*6 キロ
きろめーとる #JS*0 キロメートル 粁
きん #JS*0 斤
ぎょう #JS*269 行
く #JS*247 区 #JS*19 句 #NN*6 九
くち #JS*16 口
くみ #JS*47 組
ぐらむ #JS*29 グラム
けた #JS*13 桁
けん #JS*148 件 #JS*10 県 #JS*4 軒 #JS*1 間
げん #JS*19 元 #JS*2 弦
こ #JS*236 個 #JS*1 戸
こう #JS*87 項 #JS*4 校
こうけい #JS*0 口径
こうねん #JS*0 光年
ご #NN*190 五 #JS*6 語
ごう #JS*68 号 #JS*8 合
ごうかん #JS*10 号館
ごうしつ #JS*4 号室
ごうしょう #JS*0 号証
ごうせん #JS*2 号線
ごうとう #JS*0 号棟
ごおく #N00 五億
ごじゅう #N03 五十
ごせん #N01 五千
ごちょう #N00 五兆
ごと #JSSUC 毎
ごひゃく #N02 五百
ごまん #N00 五万
さい #JS*25 歳 #JS*4 才
さお #JS*1 棹 #JS*0 竿
さつ #JS*29 冊 #JS*1 刷
さら #JS*16 皿
さん #NN*153 三
さんおく #N00 三億
さんか #JS*2 酸化
さんじゅう #N03 三十
さんぜん #N01 三千
さんちょう #N00 三兆
さんびゃく #N02 三百
さんまん #N00 三万
し #JS*33 誌 #NN*10 四 #JS*1 紙
しき #JS*258 式
しじゅう #N03 四十
しち #NN*4 七
しちおく #N00 七億
しちまん #N00 七万
しな #JS*13 品
しゃく #JS*2 尺 #JS*0 勺
しゅ #JS*131 種 #JS*2 首
しゅう #JS*94 週 #JS*8 周 #JS*6 州
しゅうかん #JS*29 週間
しゅうき #JS*4 周期 #JS*0 周忌
しゅうねん #JS*1 周年
しゅるい #JS*62 種類
しょう #JS*175 章 #JS*4 升 #JS*2 勝
しょうせつ #JS*0 小節
しょく #JS*22 色 #JS*0 食
しん #JS*2 進
しんすう #JS*1 進数
しんとう #JS*0 親等
じ #JS*357 時 #JS*123 字 #JS*68 次 #JS*16 児
じかん #JS*331 時間
じげん #JS*29 次元 #JS*0 時限
じすぎ #JS*1 時すぎ
じっ #NN*5 十
じゅう #JS*2 重 #N03 十
じょ #JS*2 女
じょう #JS*52 条 #JS*2 畳 #JS*1 錠 帖
じん #JS*0 陣
すうおく #N00 数億
すうじゅう #N03 数十
すうせん #N01 数千
すうちょう #N00 数兆
すうひゃく #N02 数百
すうまん #N00 数万
すん #JS*2 寸
せ #JS*25 畝
せい #JS*1 世
せいき #JS*2 世紀
せき #JS*0 隻
せたい #JS*1 世帯
せだい #JS*10 世代
せつ #JS*115 節
せん #JS*1 選 銭 #N01 千
せんち #JS*1 センチ
せんちめーとる #JS*0 センチメートル 糎
ぜん #JS*2 膳
そく #JS*2 束 #JS*1 足
ぞく #JS*1 足 #JS*0 束
たく #JS*8 卓
たば #JS*1 束
たん #JS*1 反
だ #JS*62 打
だーす #JS*2 ダース
だい #JS*157 代 #JS*16 題 #JSSUC 台 #NNPRE 第
だせき #JS*0 打席
だてん #JS*0 打点
だん #JS*74 段
だんかい #JS*29 段階
ちゃく #JS*19 着
ちょい #JSSUC @
ちょう #JS*33 丁 #JS*1 帖 #N00 兆
ちょうめ #JS*13 丁目
つ #JS*293 @
つい #JS*42 対
つう #JS*68 通
つき #JS*62 月
つぶ #JS*3 粒
つぼ #JS*3 坪
づけ #JS*1 付
て #JS*10 手
てき #JS*0 滴
てん #JS*809 点
と #JS*2 都 #JS*1 斗
とう #JS*281 等 #JS*74 棟 #JS*4 頭
とうきゅう #JS*2 等級
とうしん #JS*1 頭身 #JS*0 等親
とうへん #JS*0 等辺
とおり #JS*1 通り
とん #JS*16 トン #JS*0 噸
ど #JS*101 度
どる #JS*4 ドル
なな #NN*17 七
ななおく #N00 七億
ななじゅう #N03 七十
ななせん #N01 七千
ななちょう #N00 七兆
ななひゃく #N02 七百
ななまん #N00 七万
なり #JS*2 也
なん #NN*193 何
なんおく #N00 何億
なんじゅう #N03 何十
なんぜん #N01 何千
なんちょう #N00 何兆
なんびゃく #N02 何百
なんまん #N00 何万
に #NN*255 二
におく #N00 二億
にじゅう #N03 二十
にせん #N01 二千
にち #JS*569 日
にちごう #JS*0 日号
にちづけ #JS*2 日付
にちぶり #JS*0 日ぶり
にちょう #N00 二兆
にひゃく #N02 二百
にまん #N00 二万
にん #JS*52 人
にんぐみ #JS*0 人組
にんげつ #JS*1 人月
ねん #JS*225 年
ねんかん #JS*10 年間
ねんご #JS*1 年後
ねんせい #JS*8 年生
ねんだい #JS*2 年代
ねんど #JS*22 年度
ねんぶり #JS*2 年ぶり
はい #JS*4 杯 #JS*1 敗
はく #JS*13 泊 #JS*3 拍
はち #NN*30 八
はちおく #N00 八億
はちじゅう #N03 八十
はちまん #N00 八万
はっ #NN*9 八
はっせん #N01 八千
はっちょう #N00 八兆
はっぴゃく #N02 八百
はつ #JS*19 発
はん #JS*204 版
ば #JS*4 羽
ばい #JS*57 倍
ばいと #JS*148 バイト
ばりき #JS*1 馬力
ばん #JS*115 番
ばんせん #JS*0 番線
ばんだい #JS*0 番台
ばんち #JS*4 番地
ばんて #JS*0 番手
ぱ #JS*37 羽 #JS*1 波
ぱーせんと #JS*2 パーセント
ぱい #JS*4 杯 #JS*1 敗
ぱく #JS*3 泊 #JS*0 拍
ぱこ #JS*1 箱
ぱつ #JS*2 発
ぱん #JS*13 版
ひき #JS*1 匹
ひと #NN*60 一
ひゃく #N02 百
ひょう #JS*0 俵 票
ひん #JS*1 品
ひんもく #JS*1 品目
びき #JS*1 匹
びょう #JS*29 秒 #JS*0 票
びょうし #JS*2 拍子
ぴーす #JS*6 ピース
ぴき #JS*2 匹
ぴくせる #JS*10 ピクセル
ぴょう #JS*2 票
ぴん #JS*13 ピン #JS*1 品
ふぃーと #JS*1 フィート
ふた #NN*6 二
ふらん #JS*1 フラン
ふん #JS*29 分
ぶ #JS*1 部 分 歩
ぶせい #JS*0 部制
ぶん #JS*258 分
ぶんかつ #JS*1 分割
ぶんぎ #JS*6 分木
ぷん #JS*47 分
へいべい #JS*0 平米
へいほう #JS*2 平方
へるつ #JS*1 ヘルツ
へん #JS*157 辺
ぺーじ #JS*87 ページ #JS*3 頁
ぺいじ #JS*2 ペイジ
ぺん #JS*3 辺
ほ #JS*42 歩
ほう #JS*10 報
ほん #JS*214 本
ぼん #JS*6 本
ぽ #JS*4 歩
ぽう #JS*2 包 報
ぽん #JS*6 本
ぽんど #JS*1 ポンド
まい #JS*57 枚 #NNPRE 毎
まいる #JS*3 マイル #JS*0 哩
まえ #JSSUC 前
まん #N00 万
みり #JS*10 ミリ
みりめーとる #JS*0 ミリメートル 粍
むね #JS*0 棟
め #JSSUC 目
めーとる #JS*2 メートル
めい #JS*305 名
めいさま #JS*0 名様
めが #JS*3 メガ
めん #JS*52 面
もう #JS*1 毛
もん #JS*2 問
もんめ #JS*1 匁
やーど #JS*1 ヤード
やく #NNPRE*101 約
よ #NN*19 四
よん #NN*37 四
よんおく #N00 四億
よんじゅう #N03 四十
よんせん #N01 四千
よんちょう #N00 四兆
よんひゃく #N02 四百
よんまん #N00 四万
り #JS*3 里
りっとる #JS*2 リットル
りっぽう #JS*1 立方
りょう #JS*1 両
りん #JS*19 輪 #JS*0 厘
るい #JS*16 類 #JS*0 塁
れつ #JS*101 列
ろく #NN*25 六
ろくおく #N00 六億
ろくじゅう #N03 六十
ろくせん #N01 六千
ろくちょう #N00 六兆
ろくまん #N00 六万
ろっ #NN*3 六
ろっぴゃく #N02 六百
わ #JS*4 羽 #JS*0 把
わーど #JS*16 ワード
わっと #JS*1 ワット
わり #JS*62 割
