# $Id: iroha.t,v 1.83 1994/07/04 09:17:40 kon Exp $
ー #SUC*179 ー
あ #R5*255 あ #W5*255 会 #CJ*218 あ #K5r*199 空 #PRE*131 亜 #W5*127 合 #R5*59 有 #M5*32 編 #W5*25 あ #KJ*13 阿 #W5*11 逢 #R5*11 在 #KJ*10 唖 #W5*5 遭 #KJ*3 亜 娃 #K5r*3 開 #KJ*3 吾 #K5r*1 あ 飽 明
あーかいば #T35*1 アーカイバ
あーかいぶ #T30*19 アーカイブ
あーきてくちゃ #T35*26 アーキテクチャ
あーぎゅめんと #T35*6 アーギュメント
あーく #T35*1 アーク
あーす #T30*1 アース
あーち #T35*4 アーチ
あーちすと #T35*3 アーチスト
あーてぃくる #T35*1 アーティクル
あーてぃすと #T35*3 アーティスト
あーでんと #KK*2 アーデント
あーびとれーしょん #T30*2 アービトレーション
ああ #T30*62 ああ #CJ 嗚呼
ああいう #RT*3 ああいう
あい #T30*120 愛 #KJ*13 相 #PRE*10 相 #KJ*7 合 #T35*6 藍 #KJ*3 哀 #KJ 娃 #S5 愛 #KJ 挨 姶 逢
あいあん #T35*1 アイアン
あいいれな #KY*1 相容れな
あいお #CN 秋穂
あいおい #JCN 相生 #T35 相生
あいか #T35 哀歌
あいかた #T35 相方
あいかわ #JNS 愛川 #JCN 合川 相川
あいかわらず #T06*19 相変わらず
あいかん #T35 哀歓
あいがん #T30*1 愛玩 #T30 哀願
あいきょう #T35*2 愛敬 #T35*1 愛嬌
あいくち #T35 匕首
あいけん #T35 愛犬
あいこ #T30*4 愛顧 #T15 あいこ #JNM アイ子 愛子
あいこう #T30*2 愛好 #JCN 愛甲
あいこく #T35*1 愛国
あいことな #R5 相異な
あいこん #T35*44 アイコン
あいご #T30 愛護
あいさい #T35*1 愛妻
あいさつ #T30*6 挨拶
あいさわ #JNS 相沢
あいざっく #JN アイザック
あいざわ #JNS 相沢 相澤
あいしゅう #T35*1 哀愁
あいしょう #T35*8 相性 #T35*2 愛称 #T30*1 愛唱 愛誦
あいじ #T35 愛児
あいじょう #T35*5 愛情
あいじん #T35*1 愛人
あいす #T35*3 アイス
あいず #T30*1 合図
あいずみ #CN 藍住
あいせき #T30 哀惜
あいせつ #T05 哀切
あいそ #T35*4 愛想 #JNS 相磯
あいそれーしょん #T30*1 アイソレーション
あいそれーた #T35*1 アイソレータ
あいぞう #T35 愛憎
あいたい #T30*1 相対
あいだ #T35*182 間 #JCN 英田 #JNS 会田 間 相田
あいだがら #T35*1 間柄
あいち #JCN 愛知
あいちゃく #T30 愛着
あいちょう #T35 愛鳥
あいつ #T35*3 あいつ #G5*1 相次 #G5 相継
あいつう #ZX 相通
あいづ #CN 会津
あいづたかだ #CN 会津高田
あいづばんげ #CN 会津坂下
あいづわかまつ #CN 会津若松
あいて #T35*88 相手
あいてさき #T35*2 相手先
あいてむ #T35*14 アイテム
あいであ #T35*19 アイデア
あいでぃあ #T35*8 アイディア
あいでんてぃてぃ #T35 アイデンティティ
あいでんてぃてぃー #T35*1 アイデンティティー
あいでんてぃふぁい #T30 アイデンティファイ
あいとう #T30 哀悼 #CN 愛東
あいどく #T30*1 愛読
あいどりんぐ #T30*3 アイドリング
あいどる #T35*8 アイドル
あいにく #T06*4 あいにく #T06*3 生憎 #F06 あいにく 生憎
あいの #JNS 愛野
あいはら #JCN 相原
あいば #JNS 饗場 相場
あいびーえむ #KK*1 アイビーエム
あいびき #T30 逢引
あいぶ #T30 愛撫
あいべつ #T35 哀別 #CN 愛別
あいま #T35*2 合間
あいまい #T05*5 曖昧 #T05*3 あいまい
あいみ #JCN 会見
あいよう #T30*3 愛用
あいら #JCN 姶良 #CN 吾平
あいらく #T35 哀楽
あいらし #KY 愛らし
あいらしゅう #KYU 愛らしゅう
あいろ #T35*2 隘路
あいろん #T35*3 アイロン
あうと #T30*72 アウト
あうとさいだー #T35*1 アウトサイダー
あうん #T35 阿吽
あえ #G5r*1 あえ #KSr 和え #G5r 喘
あえて #F14*3 敢えて #F14*1 あえて
あえな #KY 敢な
あえのう #KYU 敢のう
あえろふろーと #KK*1 アエロフロート
あえん #T35*1 亜鉛
あお #KYmi*27 青 #T35*25 青 #G5*5 仰 #PRE*4 青 #KYmi*4 蒼 #R5r*2 煽 #G5 扇 #T35 蒼 碧
あおあお #F00*1 青々
あおい #T35 葵 #JCN 青井
あおう #KYU*2 青う #KYU*1 蒼う
あおおに #T35 青鬼
あおがえる #T35 青蛙
あおがき #JCN 青垣
あおがしま #CN 青ヶ島
あおき #JNS 青木
あおざめ #KS*1 青ざめ
あおしま #JCN 青島
あおじろ #KYmi 青白
あおじろう #KYU 青白う
あおぞら #T35*1 青空
あおた #T35 青田
あおぬま #JCN 青沼
あおの #JCN 青野
あおば #T35*1 青葉
あおむし #T35 青虫
あおもの #T35*2 青物
あおもり #CN 青森
あおや #JCN 青谷
あおやぎ #T35*2 青柳 #JCN 青柳
あおやま #JCN 青山
あか #T35*46 赤 #KYmi*31 赤 #PRE*10 赤 #T35*5 垢 #S5*3 明か #S5 飽か
あかい #JCN 赤井
あかいがわ #CN 赤井川
あかいけ #JCN 赤池
あかいわ #CN 赤磐
あかうんと #T35*53 アカウント
あかえだ #JNS 赤枝
あかおか #JCN 赤岡
あかおに #T35 赤鬼
あかがい #T35 赤貝
あかがわ #JCN 赤川
あかぎ #JCN 赤城 #JNS 赤木
あかげ #T35 赤毛
あかげら #T35 アカゲラ 赤啄木鳥
あかご #T35*1 赤子
あかさか #JCN 赤坂
あかさき #CN 赤碕
あかざわ #JCN 赤沢
あかし #T35 証 灯 #JCN 明石
あかしお #T35 赤潮
あかしな #CN 明科
あかじ #T35*8 赤字
あかちん #T35 赤チン
あかっぱじ #T35 赤っ恥
あかつか #JCN 赤塚
あかつき #T35*2 暁 #T35*1 あかつき
あかつち #T35 赤土
あかつつみ #CN 赤堤
あかどまり #JCN 赤泊
あかぬけ #KS 垢抜け
あかね #T35*1 茜 #JN 茜
あかはに #JNS 赤埴
あかはね #JNS 赤羽
あかばね #JCN 赤羽 赤羽根
あかびら #CN 赤平
あかぼう #T35*1 赤帽
あかぼり #JCN 赤堀
あかま #JCN 赤間
あかまつ #JCN 赤松 #T35 赤松
あかまる #T35 赤丸
あから #M5r 赤ら
あからがお #T35 赤ら顔
あからさま #T05 あからさま
あかり #T35*1 灯 灯り 明り
あかる #KYmi*17 明る
あかるう #KYU*2 明るう
あかるみ #T35*1 明るみ
あかん #T35*2 あかん #CN 阿寒
あかんぼう #T35*1 赤ん坊
あが #R5r*69 上が #R5r*11 あが #R5*7 挙が #K5r あが 足掻 #R5 騰が 揚が
あがつま #JNS 我妻 #JCN 吾妻
あがる #CNSUC1*1 上ル
あがわ #CN 吾川
あき #T35*75 秋 #KSr*14 飽き #KJ*3 明 #KJ*1 穐 #JN あき #JNM 亜紀 #JCN 安岐 安芸 #KSr 厭き #KJ 龝
あきう #CN 秋保
あきお #JNM 秋男 秋夫 秋雄 昭男 昭夫 昭雄 章夫 明男 明夫 明雄
あきかわ #JCN 秋川
あきがわ #CN 秋川
あきこ #JNM 顕子 晃子 昭子 晶子 章子 明子
あきさめ #T35 秋雨
あきさわ #JNS 秋澤
あきしま #JCN 昭島
あきす #T35 空巣
あきすえ #JNS 秋末
あきた #JCN 秋田 飽田
あきち #T35 空き地
あきつ #CN 安芸津
あきとし #JNM 晃年
あきとも #JN 秋友
あきな #W5r 商 #JNM 明菜
あきない #T30 商い
あきのり #JNM 明紀 明憲
あきはばら #CN 秋葉原
あきば #JCN 秋葉
あきばれ #T35*1 秋晴れ
あきひこ #JNM 昭彦 明彦
あきひで #JNM 彰英
あきひと #JNM 明仁
あきひろ #JNM 明宏 明弘
あきみ #JNM 秋生
あきもと #JCN 秋元 #JNS 秋本
あきや #T35 空き家
あきやま #JCN 秋山
あきゅーむれーた #T35*1 アキュームレータ
あきゅむれーた #T35*1 アキュムレータ
あきら #JNM 旭 鑑 光 晃 彰 昭 晶 章 明 亮 朗 晁
あきらか #T05*33 明らか
あきらめ #KSr*19 あきらめ #KSr*4 諦め
あきれ #KS*3 呆れ
あきれす #JN アキレス
あきれすけん #T35 アキレス腱
あく #PRE*10 悪 #T35*6 悪 #KJ 握 渥
あくえき #T35 悪疫
あくしつ #T05*2 悪質
あくしでんと #T35*1 アクシデント
あくしゅ #T30*1 握手 #T35 悪手
あくしゅう #T35 悪臭
あくしゅみ #T05 悪趣味
あくしょん #T35*4 アクション
あくじ #T35*1 悪事
あくじき #T05 悪食
あくせい #T35 悪性
あくせさり #T35*1 アクセサリ
あくせす #T30*4 アクセス
あくせっしんぐ #T35*1 アクセッシング
あくせぷと #T30*2 アクセプト
あくせる #T35*3 アクセル
あくせん #T30 悪戦 #T35 悪銭
あくせんと #T35*1 アクセント
あくた #T35*16 芥
あくたー #T35*1 アクター
あくたい #T35*1 悪態
あくたがわ #JCN 芥川
あくだま #T35 悪玉
あくつ #JCN 阿久津
あくてぃびてぃ #T35*11 アクティビティ
あくてぃぶ #T05*8 アクティブ
あくてぃべーしょん #T35 アクティベーション
あくとく #T05*1 悪徳
あくど #KY あくど
あくにん #T35*2 悪人
あくね #JCN 阿久根
あくば #T35 悪罵
あくび #T30*1 あくび
あくびょう #T35*1 悪病
あくびょうどう #T15 悪平等
あくへい #T35 悪弊
あくへき #T35 悪癖
あくま #T35*6 悪魔
あくまで #F14*14 あくまで
あくまでも #F14*1 あくまでも
あくみ #JCN 飽海
あくみょう #T35*1 悪名
あくむ #T35*1 悪夢
あくめい #T35 悪名
あくやく #T35 悪役
あくゆう #T35 悪友
あくよう #T30*1 悪用
あくりょう #T35 悪霊
あくりょく #T35 握力
あくりる #T35 アクリル
あぐ #M5 倦
あぐい #JCN 阿久比 #JNS 安食
あぐさ #JNS 阿草
あぐに #JCN 粟国
あぐね #KS あぐね
あぐねす #JN アグネス
あけ #KS*39 開け #KSr*19 明け #KS*3 空け #T35 朱
あけがた #T35*1 明け方
あけち #JCN 明智
あけっぴろげ #T06 開けっ広げ
あけの #JCN 明野
あけはな #S5r 開け放
あけはま #CN 明浜
あけぼの #T35*1 曙
あけわた #S5r 明け渡
あげ #KS*149 あげ #KSr*138 上げ #KS*57 挙げ #KSr*6 揚げ #CN 安芸
あげあし #T35*6 挙げ足 #T35*5 揚げ足
あげお #JCN 上尾
あげく #T35*1 挙げ句
あげぞこ #T35 上げ底
あげつら #W5*1 論
あげまつ #JCN 上松
あこーすてぃっく #T05*1 アコースティック
あこう #KYU*2 赤う #JCN 赤穂
あこお #CN 赤穂
あこがれ #KSr*8 憧れ #KSr*1 あこがれ
あご #CN 阿児 #T35 顎
あさ #T35*112 朝 #KY*42 浅 #T35*6 麻 #R5r*4 漁 #KJ*3 浅 #JCN 安佐 厚狭
あさーと #T30*6 アサート
あさい #JCN 浅井
あさいち #T35*2 朝市
あさいん #T30 アサイン
あさお #CN 麻生
あさか #JCN 浅香 朝霞
あさかわ #JCN 浅川
あさが #JNS 浅賀
あさがた #T35 朝方
あさがや #CN 阿佐谷
あさきた #CN 安佐北
あさく #T35 浅く
あさくさ #CN 浅草
あさくち #CN 浅口
あさくら #JCN 朝倉
あさぐろ #KY 浅黒
あさぐろう #KYU 浅黒う
あさこ #JNM あさ子 亜抄子 #JCN 朝来
あさご #CN 朝来
あさしな #CN 浅科
あさじ #CN 朝地
あさせ #T35 浅瀬
あさだ #JNS 浅田 麻田
あさって #T35*15 明後日 #T35*4 あさって
あさぬま #JNS 浅沼
あさの #JCN 浅野
あさはら #JNS 浅原
あさば #JNS 浅羽
あさばん #T35*1 朝晩
あさひ #T35*21 朝日 #T35*1 旭 #JCN 旭 朝日
あさひかせい #KK*4 旭化成
あさひかわ #CN 旭川
あさひがわ #CN 旭川
あさふく #JNS 浅福
あさまし #KY 浅まし
あさましゅう #KYU 浅ましゅう
あさみ #JNS 浅見
あさみなみ #CN 安佐南
あさめし #T35*1 朝飯
あさやけ #T35*1 朝焼け
あさゆう #T35 朝夕
あさり #T35*1 鯏
あざ #CNPRE*2 字 #T35 字
あざい #JCN 浅井
あざけ #R5r 嘲
あざと #KY あざと
あざな #W5 糾
あざぶ #CN 麻布
あざむ #K5 欺
あざやか #T05*1 鮮やか
あざらし #T35 アザラシ 海豹
あし #T35*53 足 #KY*4 悪し #T35*1 脚 #T35 葦 芦
あしあと #T35*1 足跡
あしお #JCN 足尾
あしおと #T35*1 足音
あしかが #JCN 足利
あしかけ #T35 足掛け
あしからず #F14*13 あしからず
あしかり #JCN 芦刈
あしがかり #T35*1 足掛かり 足掛り
あしがため #T30 足固め
あしがら #JCN 足柄
あしがらかみ #CN 足柄上
あしがらしも #CN 足柄下
あしがわ #CN 芦川
あしきしゅうきゅう #T35 ア式蹴球
あしきた #CN 葦北 芦北
あしくび #T35 足首
あしこし #T35 足腰
あしざわ #JNS 芦沢
あしすたんと #T35*2 アシスタント
あしずり #CN 足摺
あした #T35*183 明日
あしたか #CN 愛鷹
あしどり #T35*1 足取り
あしな #JNS 芦品
あしなみ #T35 足並み
あしのこ #CN 芦ノ湖
あしび #T35*1 馬酔木
あしべ #JCN 芦辺
あしべつ #CN 芦別
あしもと #T35 足元
あしもふ #JN アシモフ
あしや #JCN 芦屋
あしやす #CN 芦安
あしゅう #KYU*1 悪しゅう
あしゅら #JNM 阿修羅
あしょろ #CN 足寄
あしよろ #CN 足寄
あしろ #CN 安代
あしわだ #CN 足和田
あじ #T35*16 味 #T35*1 鰺 #T35 鯵 #JCN 庵治
あじあ #CN アジア
あじがさわ #CN 鯵ヶ沢
あじけな #KY*4 味気な
あじけのう #KYU*1 味気のう
あじさい #T35*1 紫陽花
あじさか #JNS 鯵坂
あじす #CN 阿知須
あじつけ #T30 味付け
あじとみ #JNS 安次富
あじな #CN 芦品
あじみ #T30*1 味見
あじむ #CN 安心院
あじわ #W5r*4 味わ
あす #T35*161 明日
あすか #JCN 飛鳥 明日香
あすかい #JNS 飛鳥井
あすきー #KK*24 アスキー
あすけ #JCN 足助
あすたりすく #T35*1 アスタリスク
あすてっく #KK*14 アステック
あすぱらがす #T35*1 アスパラガス
あすぴりん #T35 アスピリン
あすぺくと #T35*3 アスペクト
あすわ #JCN 足羽
あずか #R5r*17 預 #R5r 与 預か
あずき #T35 あずき 小豆
あずけ #KS*8 預け
あずけいれ #KSr*1 預け入れ
あずさ #T35 梓
あずさがわ #CN 梓川
あずま #T35*5 東 #JCN 東
あずみ #JCN 安曇
あせ #R5r*18 焦 #KS*8 褪せ #T30*5 汗
あせかき #T15 汗かき
あせんぶら #T35*11 アセンブラ
あせんぶり #T35*1 アセンブリ
あせんぶる #T30*1 アセンブル
あぜ #T35 畦 畔
あぜくら #T35 校倉
あそ #B5r*93 遊 #JCN 阿曽 阿蘇 安蘇
あそう #KYU*2 浅う #JCN 麻生
あそこ #T35*4 あそこ
あそしぇいつ #T35*1 アソシェイツ
あそしえーしょん #T35*4 アソシエーション
あそびごころ #T35*2 遊び心
あそびまわ #R5 遊び回
あた #R5r*190 当た #R5r*26 あた #R5r*2 当
あたい #T30*255 値 #T30*3 値い #T35 価
あたえ #KS*162 与え
あたかも #F14*4 あたかも
あたし #T35*2 あたし
あたたか #KYmi*7 暖か #T05*3 暖か #KYmi*1 温か #T05*1 温か
あたたこう #KYU*1 温こう 暖こう
あたたま #R5*3 温ま #R5 暖ま
あたため #KSr*1 温め #KSr 暖め
あたっく #T30*2 アタック
あたっち #T30*1 アタッチ
あたっちめんと #T35*3 アタッチメント
あたま #T35*60 頭
あたまうち #T35 頭打ち
あたまかず #T35 頭数
あたまきん #T35 頭金
あたまごなし #T05 頭ごなし
あたみ #JCN 熱海
あたらし #KYmi*255 新し
あたらしゅう #KYU*3 新しゅう
あたり #SUC*52 あたり #T35*30 辺り #KK*1 アタリ
あたりまえ #T05*8 当り前 #T05*6 当たり前 #T05*3 当たりまえ
あたる #JNM 中
あだ #T35 仇 徒
あだうち #T30 仇討ち
あだち #JCN 安達 #JNS 足達 #JCN 足立
あだばな #T35 仇花
あだぷた #T35*1 アダプタ
あだむ #JN アダム
あち #CN 阿智
あちこち #T35*3 あちこち
あちら #T35*6 あちら
あっ #KJ*6 斡 #CJ*4 あっ #S5 圧 #SX 圧
あっか #T30*1 悪化
あっかん #T35*2 圧巻 #T35 悪漢
あっけ #T35 呆気
あっけし #CN 厚岸
あっけらかん #F06*1 アッケラカン
あっこう #T30 悪口
あっさぶ #CN 厚沢部
あっさり #F04*2 あっさり
あっしゅく #T30*40 圧縮
あっしょう #T30*1 圧勝
あっせん #T30 斡旋
あっち #T35*2 あっち
あっちゃく #T30*1 圧着
あっと #F14*9 あっと
あっとう #T30*7 圧倒
あっぱく #T30*2 圧迫
あっぴ #CN 安比
あっぷ #T30*141 アップ
あっぷる #KK*9 アップル #T35*2 アップル
あつ #KYmi*47 厚 #KY*44 暑 #KY*21 熱 #KJ*19 圧 #KY*1 篤
あつう #KYU*2 厚う 暑う 熱う #KYU*1 篤う
あつえん #T30*2 圧延
あつか #W5r*194 扱
あつかいだか #T35 扱い高
あつかまし #KY*3 厚かまし
あつかましゅう #KYU*1 厚かましゅう
あつかん #T35*1 熱燗
あつがみ #T35 厚紙
あつぎ #T30*13 厚着 #JCN 厚木
あつくるし #KY 暑苦し
あつくるしゅう #KYU 暑苦しゅう
あつけし #CN 厚岸
あつこ #JNM 敦子
あつさ #T35 暑さ
あつし #JNM 厚 淳 篤 敦 敦司
あつしおかのう #CN 熱塩加納
あつじ #T35 厚地
あつた #JCN 厚田 熱田
あつで #T35*1 厚手
あつひこ #JNM 篤彦 敦彦
あつま #R5r*32 集ま #CN 厚真
あつみ #JCN 渥美 温海
あつめ #KSr*39 集め #T35 厚め
あつもり #JN 敦盛
あつりょく #T35*5 圧力
あつれき #T35 圧轢
あづち #JCN 安土
あづま #JCN 吾妻
あて #T35*93 宛 #KSr*75 当て #KSr*8 あて #KSr*6 宛て #KS*3 充て
あてが #W5*1 あてが
あてさき #T35*24 宛先
あてじ #T30 宛て字 当て字
あてずっぽう #T15 当てずっぽう
あてつ #CN 阿哲
あてつけ #KSr*4 当てつけ
あてな #T35*14 宛名
あてね #CN アテネ
あてはま #R5*7 当てはま
あてはめ #KS*2 当てはめ
あと #T35*255 後 #T35*105 あと #T35*2 跡 #T35*1 痕 #T35 蹟
あとあと #T35*1 後々
あとう #CN 阿東
あとおし #T30 後押し
あとかたづけ #T30*1 後かたづけ
あとがま #T35 後釜
あとさき #T35*1 後先
あとざん #T35*1 後産
あとしまつ #T30*4 後始末
あとだし #T30 後出し
あとち #T35*1 跡地
あとつぎ #T35 跡継ぎ
あとばらい #T30 後払い
あとみっく #T35*1 アトミック
あとむ #T35*4 アトム
あともどり #T30*2 後戻り
あとりえ #T35*1 アトリエ
あとりびゅーと #T35*3 アトリビュート
あどがわ #JCN 安曇川
あどけな #KY*1 あどけな
あどけのう #KYU*1 あどけのう
あどばいす #T30*7 アドバイス
あどほっく #T05*1 アドホック
あどれす #T35*204 アドレス
あどれっしんぐ #T30*17 アドレッシング
あどれなりん #T35*1 アドレナリン
あな #T35*15 穴 #T35 坑 孔
あなーきー #T05 アナーキー
あなーきすと #T35 アナーキスト
あなーきずむ #T35 アナーキズム
あなうめ #T30*1 穴埋め
あなうんさ #T35*1 アナウンサ
あなうんさー #T35*2 アナウンサー
あなうんす #T30*22 アナウンス
あながち #F14*1 あながち
あなぐらむ #T30 アナグラム
あなた #T35*67 あなた #T35*13 貴方
あなたたち #T35 あなたたち あなた達 貴方達
あなど #R5r*1 侮
あなぶき #JCN 穴吹
あなみず #JCN 穴水
あならいざ #T35*1 アナライザ
あなりすと #T35*1 アナリスト
あなろぐ #T05*14 アナログ
あなろじー #T35*2 アナロジー
あなん #JCN 阿南
あに #T35*3 兄 #JCN 阿仁
あにき #T35*1 兄貴
あにみずむ #T35 アニミズム
あにめ #T35*6 アニメ
あにめーしょん #T35*4 アニメーション
あね #T35*5 姉 #T35*1 姐
あねったい #T35 亜熱帯
あの #RT*79 あの
あのう #CN 安濃
あのこと #T35 あのこと
あのにます #T05 アノニマス
あのよう #T05*1 あの様 #T05 あのよう
あば #K5*1 暴 #CN 阿波
あばきだ #S5 暴き出
あばしり #CN 網走
あばらぼね #T35 肋骨
あばれ #KS*4 暴れ
あばれんぼう #T35 暴れん坊
あぱーちゃ #T35*13 アパーチャ
あぱーと #T35*7 アパート
あぱれる #T35*2 アパレル
あび #KS*1 浴び
あびーむ #T35*1 アビーム
あびきょうかん #T35*2 阿鼻叫喚
あびこ #JNS 安彦 #JCN 我孫子
あびせ #KS*1 浴びせ
あぴーる #T30*3 アピール
あふぃん #T35*1 アフィン
あふがにすたん #CN アフガニスタン
あふりか #CN アフリカ
あふれ #KS*13 溢れ #KS*3 あふれ
あぶ #R5*1 あぶ #T35*1 虻 #JCN 阿武
あぶすとらくしょん #T35*1 アブストラクション
あぶすとらくと #T35*2 アブストラクト
あぶた #CN 虻田
あぶな #KY*23 危な #KY あぶな
あぶなっかし #KY 危なっかし
あぶのう #KYU*2 危のう #KYU あぶのう
あぶみ #T35 鐙
あぶら #T35*6 油 #T35 脂
あぶらえ #T35 油絵
あぶらぎ #R5 脂ぎ
あぶらっこ #KY 油っこ
あぷりけーしょん #T35*255 アプリケーション
あぷろーち #T30*12 アプローチ
あへん #T35 阿片
あべ #JNS 阿部 安部
あべっく #T35*1 アベック
あべの #JCN 阿倍野
あべれーじ #T35*3 アベレージ
あほ #T05*51 阿呆
あほう #T05*3 阿呆
あほうどり #T35*1 信天翁
あぼ #JNS 安保
あぼーと #T30*5 アボート
あま #R5r*46 余 #S5*33 余 #KYmi*17 甘 #PRE*1 アマ #T35*1 海女 #CN 海士 #JNS 海部 #PRE 甘 #R5r 剰 #KJ 天 #T35 尼
あまあし #T35 雨足
あまえ #KSr*4 甘え
あまから #KY 甘辛
あまがい #JNS 天海
あまがえる #T35 雨蛙
あまがき #T35 甘柿
あまがさ #T35*1 雨がさ 雨ガサ 雨傘
あまがさき #JCN 尼崎
あまがせ #CN 天瀬
あまぎ #CN 甘木 #JCN 天城
あまぎゆがしま #CN 天城湯ヶ島
あまくさ #JCN 天草
あまくち #T05 甘口
あまぐも #T35 雨雲
あまごい #T30 雨乞い
あまた #T35 あまた 数多 #JCN 天田
あまだれ #T35*1 雨だれ
あまちゅあ #T35*4 アマチュア
あまったる #KY 甘ったる
あまつこみなと #CN 天津小湊
あまでら #T35 尼寺
あまど #T35*1 雨戸
あまぬま #JCN 天沼
あまねく #F14 あまねく
あまの #JCN 天野
あまのじゃく #T05*1 天の邪鬼 #T35 天邪鬼
あまべ #JNS 海部
あまみ #JCN 奄美 #T35 甘味
あまみず #T35 雨水
あまみや #JNS 雨宮 天宮
あまもり #T30*1 雨漏り
あまやか #S5r*2 甘やか #T05 甘やか
あまり #T35*149 あまり #F14*142 あまり
あまるめ #CN 余目
あまん #ZX 甘ん
あみ #T35*16 網 #JCN 阿見
あみき #T35 編み機
あみだ #JN 阿弥陀
あみだな #T35*1 網棚
あみど #T35*1 アミド
あみの #JCN 網野
あみのさん #T35*1 アミノ酸
あみめ #T35 編目 網目
あむねすてぃ #KK*1 アムネスティ
あめ #T35*62 雨 #T35*7 飴 #T35*2 あめ
あめあがり #T35 雨上がり
あめふり #T35*1 雨ふり #T35 雨降り
あめみや #JNS 雨宮
あめよこ #CN アメ横
あめりか #CN アメリカ
あめりかん #T35*1 アメリカン
あもう #KYU*2 甘う
あや #T35*34 文 #T35*1 綾 #T35 絢 #JCN 綾 #JNM 彩 #T35 彩 彪
あやう #KY*2 危う #KY*1 あやう
あやうた #CN 綾歌
あやかみ #JCN 綾上
あやぎぬ #T35*1 綺
あやこ #JNM 綾子 章子 文子
あやし #KYmi*8 怪し #M5*6 怪し #KY 妖し
あやしゅう #KYU*1 怪しゅう #KYU 妖しゅう
あやせ #JCN 綾瀬
あやつ #R5 操
あやふや #T05*1 あやふや
あやぶ #M5*1 危ぶ
あやべ #JCN 綾部
あやま #R5r*101 誤 #R5*29 謝 #JCN 阿山 #T5r 過
あやめ #KS 殺め #T35 菖蒲
あゆ #M5r*3 歩 #T35*1 鮎 #T30 阿諛
あゆみ #JNM あゆみ 亜弓
あゆみよ #R5r*1 歩み寄
あら #W5r*54 洗 #KY*9 荒 #S5r*8 荒 #PRE*4 粗 #KY*2 粗 #JNS 荒 #S5r 荒ら
あらー #JN アラー
あらあらし #KY 荒々し
あらあらしゅう #KYU 荒々しゅう
あらい #JNS 荒井 #JCN 新井 #JNS 新居
あらいあんと #KK*4 アライアント
あらいおと #S5 洗い落と
あらいざらい #F14 洗いざらい
あらいそ #T35 荒磯
あらいだ #S5r 洗い出
あらいなが #S5 洗い流
あらいんめんと #T35*1 アラインメント
あらうんど #T35*1 アラウンド
あらお #JCN 荒尾
あらかじめ #F14*27 あらかじめ #F14*6 予め
あらかた #F14 あらかた 粗方
あらかると #T35*1 アラカルト
あらかわ #JCN 荒川
あらき #JN*1 荒木 #JCN 荒木
あらぎょう #T35 荒行
あらくれ #T35 荒くれ
あらけずり #T05 荒削り
あらげ #KS 荒げ
あらごなし #T30*1 荒ごなし
あらし #T35*9 嵐
あらしまわ #R5 荒し回
あらしやま #CN 嵐山
あらすか #CN アラスカ
あらすじ #T35*1 粗筋
あらそ #W5r*4 争
あらた #T05*111 新た
あらたま #R5r*1 改ま
あらため #KS*17 改め
あらためて #F14*1 改めて
あらっぽ #KY*1 荒っぽ
あらて #T35*2 新手
あらなみ #T35*1 荒波
あらなわ #T35 荒縄
あらびあ #CN アラビア
あらまき #JNS 新巻
あらゆる #RT*5 あらゆる
あらりょうじ #T30 荒療治
あらわ #S5*181 表 #S5*2 あらわ 現 #T15 あらわ #S5 著 #T15 露わ
あらわれ #KSr*49 現れ #KSr*8 現われ #KSr*5 表れ
あり #T35*5 蟻 #T36 あり 有り
ありあけ #JCN 有明
ありあけかい #CN 有明海
ありあり #F06 ありあり
ありあわせ #T35 有り合わせ
ありえ #JCN 有家
ありか #T35*2 在処
ありかわ #JCN 有川
ありが #JNS 有賀
ありがた #KYmi*6 ありがた 有難
ありがたみ #T35*1 ありがた味 有難味
ありがとう #CJ*168 ありがとう #CJ*42 有難う #KYU*1 ありがとう 有難う
ありさま #T35*2 有り様
ありさわ #JNS 有沢
ありざ #JNS 有座
ありすとてれす #JN アリストテレス
ありぞな #CN アリゾナ
ありた #JCN 有田
ありだ #CN 有田
ありったけ #T15 ありったけ 有りったけ
ありてぃ #T35*1 アリティ
ありてい #T05*1 有り体
ありとあらゆる #RT ありとあらゆる 有りとあらゆる
ありとある #RT ありとある 有りとある
ありふれ #KS*1 ありふれ
ありま #JCN 有馬
ありまさ #JNM 有正
ありゃ #CJ ありゃ
ありや #JNS 有家
ありゅう #T35*1 亜流
ありゅうさん #T35 亜硫酸
ありよし #JN 有吉
ある #K5r*34 歩 #T35*1 ∃ #RT*1 ある 或 #KJ 或
あるいは #F14*69 あるいは #F14*3 或は
あるかり #T35 アルカリ
あるがまま #T15 あるがまま 有るがまま
あるきめです #JN アルキメデス
あるこーる #T35*3 アルコール
あるごりずみっく #T05*4 アルゴリズミック
あるごりずむ #T35*57 アルゴリズム
あるごん #T35*1 アルゴン
あるじ #T35*1 主
あるぜんちん #CN アルゼンチン
あるちゅう #T35 アル中
あるばいと #T30*4 アルバイト
あるばむ #T35*11 アルバム
あるびの #T35*1 アルビノ
あるふぁ #T35*45 α #T35*2 Α
あるふぁべっと #T35*75 アルファベット
あるぶみん #T35*1 アルブミン
あるぷす #CN アルプス
あるぺじお #T30 アルペジオ
あるぺん #T35*1 アルペン
あるまいと #T35*3 アルマイト
あるみ #T35 アルミ
あるみにうむ #T35 アルミニウム
あるめん #T35*1 アル綿
あれ #T35*15 あれ #KSr*3 荒れ #KSr 粗れ
あれい #T35*2 アレイ
あれよあれよ #F06 あれよあれよ
あれらの #RT あれらの
あれるぎ #T35 アレルギ
あれんじ #T30*3 アレンジ
あろー #JN アロー
あろう #KYU*1 荒ろう 粗ろう
あろけーしょん #T30*1 アロケーション
あろけーと #T30*16 アロケート
あろっく #T30*4 アロック
あわ #T35*5 粟 #S5*4 会わ #KYmi*3 淡 #T35*2 泡 #S5*1 合わ #JCN 阿波 安房
あわさ #R5r*1 合わさ
あわじ #JCN 淡路
あわじしま #CN 淡路島
あわじちょう #CN 淡路町
あわせ #T35*9 合わせ #T35 袷 #KS 併せ
あわせて #CJ*6 併せて
あわだ #T5r 泡立
あわて #KS*24 慌て #KS*1 あわて
あわの #JCN 粟野
あわび #T35*1 鮑
あわや #F14 あわや #JNS 粟谷
あわゆき #T35 淡雪
あわら #CN 芦原
あわれ #M5r*1 哀れ #T05*1 哀れ #M5r あわれ 憐れ
あん #SUC*118 案 #KJ*16 安 #ZX*3 案 #ZX*2 按 #KJ*1 庵 暗 #T35*1 案 #KJ*1 杏 #KJ 按 闇 鞍 #T35 餡
あんい #T05*14 安易
あんいつ #T05 安逸
あんうん #T35 暗雲
あんか #T05*4 安価
あんかん #F02 安閑
あんがい #T05*7 案外
あんき #T35*1 暗鬼 #T30 暗記
あんきょ #T30 安居
あんけーと #T30*20 アンケート
あんこ #T35 アンコ
あんこーる #T30*1 アンコール
あんこう #T35*1 鮟 鮟鱇 鱇
あんこがた #T15 アンコ形
あんこく #T05 暗黒
あんごう #T35*25 暗号
あんさつ #T30*1 暗殺
あんざい #JNS 安斎 安西 安斉
あんざん #T30 暗算
あんしつ #T35 暗室
あんしょ #T35 暗所
あんしょう #T30*16 暗証 #T30 暗唱 #T35 暗礁 #T30 暗誦
あんしん #T00*28 安心
あんじ #T30*3 暗示 #KS*1 案じ
あんじゅう #T30*1 安住
あんじょう #JCN 安城
あんず #T35*1 杏 #T35 あんず
あんせい #T05 安静
あんぜん #T05*29 安全
あんそく #T35 安息
あんたい #T05 安泰
あんたん #T02 暗澹
あんだー #T35*3 アンダー
あんだーそん #JN アンダーソン
あんだーらいん #T35*8 ＿
あんち #T30*1 安置
あんちゅう #T35*1 暗中
あんちゅうもさく #T30*1 暗中模索
あんちょく #T05*4 安直
あんてい #T00*16 安定
あんてな #T35*2 アンテナ
あんとく #JNS 安徳
あんど #T30*19 安堵 #JNS 安堵
あんどう #JNS 安東 安藤
あんな #T05*5 あんな
あんない #T30*28 案内
あんなか #JCN 安中
あんに #F14*2 暗に
あんのじょう #F14*3 案の定 #T36 案の定
あんばい #T35 塩梅
あんばらんす #T05*2 アンバランス
あんぱさんど #T35*1 ＆
あんぱち #CN 安八
あんぴ #T35 安否
あんぶん #T30*2 按分
あんぷ #T30 暗譜
あんぺあ #T35 アンペア
あんぽ #T35 安保
あんま #T30 按摩
あんまり #T17*25 あんまり
あんみん #T30*1 安眠
あんもく #T35*5 暗黙
あんもにあ #T35 アンモニア
あんゆ #T35*1 暗喩
あんらく #T05*1 安楽
い #KS*255 い #KY*255 い #W5*255 い 言 #C5r*255 行 #C5r*236 い #KY*117 良 #JS*98 位 #R5*90 い #KS*36 居 #M5*33 忌 #R5r*27 入 #T35*26 意 #T35*24 井 #R5*21 要 #T35*10 胃 #T05*8 異 #C5r*6 往 #KJ*5 伊 #KJ*4 移 #T35*4 医 #W5*3 云 #KJ*2 慰 #W5*2 謂 #KJ*2 遺 #N5*2 往 #KJ*1 以 位 偉 夷 威 尉 惟 易 畏 異 #T35*1 衣 #KJ*1 堰 #KS*1 射 #C5r*1 逝 #KJ*1 詑 #SUC 位 #KJ 依 囲 委 #SUC 委 #KJ 椅 為 維 緯 萎 謂 違 #T35 亥 #KJ 蔚 莞 居 #R5 煎 #KJ 胆 #KS 鋳 #KJ 猪 斐 鮪 揖
いーさ #T35*4 イーサ
いーさねっと #KK*74 イーサネット
いーた #T35*2 Η η
いあつ #T30*1 威圧
いあん #T30*1 慰安
いい #KJ*15 飯
いいあ #W5r*1 言い合
いいあらそ #W5r 言い争
いいあらわ #S5*2 言い表
いいえ #CJ*4 いいえ #T35*2 いいえ
いいおか #JCN 飯岡
いいかえ #KSr*16 言い替え #KSr*2 言い換え #S5r 言い返
いいかげん #T05*17 いい加減
いいかね #KS 言い兼ね
いいかわ #S5 言い交わ
いいき #R5r*5 言い切
いいくるめ #KSr 言いくるめ
いいし #JCN 飯石
いいじま #JCN 飯島 #JNS 飯嶋
いいすて #KSr 言い捨て
いいそこな #W5r 言い損な
いいたか #JN 飯高
いいたがわ #CN 飯田川
いいたて #JNS 飯館
いいだ #S5*4 言い出 #JCN 飯田
いいだくだく #F00 唯々諾々
いいだしっぺ #T35 言い出しっぺ
いいつけ #KS 言いつけ
いいづか #JCN 飯塚
いいで #JCN 飯豊
いいなお #S5r*1 言い直
いいなずけ #T35 許嫁
いいなん #CN 飯南
いいぬま #JNS 飯沼
いいの #JCN 飯野
いいのがれ #KSr*1 言い逃れ
いいは #R5 言い張
いいふくめ #KSr 言い含め
いいふら #S5 言い触ら
いいぶん #T35*1 言い分
いいまわし #T35 言い回し
いいむら #JNS 飯村
いいもり #JCN 飯森 飯盛
いいやま #JCN 飯山
いいわけ #T30*24 言い訳
いいわた #S5 言い渡
いいん #T35*15 委員 #T35 医院
いいんかい #T35*31 委員会
いえ #T35*150 家 #CJ*1 いえ #KS*1 癒え #JCN 伊江
いえがら #T35 家柄
いえしま #JCN 家島
いえじ #T35*1 家路
いえす #JN イエス
いえで #T30 家出
いえもと #T35*2 家元
いえやす #JNM 家康
いえろー #T05*1 イエロー
いえん #T35 胃炎
いおう #T35 硫黄
いおうじま #CN 伊王島
いおた #T35*1 Ι ι
いおり #T35 庵
いおん #T35 イオン
いか #T35*255 以下 #S5*12 生か #R5r*5 怒 #T35*5 如何 #S5*3 活か #T35*1 ≦ #T30*1 異化 #T35*1 医科 烏賊 #JCN 伊香
いかい #JNS 猪飼
いかいよう #T35 胃潰瘍
いかが #T11*45 いかが #T11*11 如何
いかがわし #KY いかがわし
いかがわしゅう #KYU いかがわしゅう
いかく #T30 威嚇
いかさま #T30*1 いかさま
いかざき #JCN 五十崎
いかすい #T35 胃下垂
いかた #CN 伊方
いかだ #T35*1 筏
いかだい #JNS 筏井
いかなる #RT*4 如何なる #RT*2 いかなる
いかに #F14*18 いかに
いかにも #F14*1 いかにも
いかほ #JCN 伊香保
いかほど #T15 いかほど
いかり #T35 碇 錨
いかりがせき #CN 碇ヶ関
いかわ #JCN 井川 #JNS 居川
いかん #T30*34 移管 #T05*1 遺憾 #T35*1 如何 #T35 尉官 衣冠
いかんせん #CJ いかんせん
いが #JCN 伊賀
いがい #T35*255 以外 #T05*9 意外 #T35 遺骸
いがく #T35*3 医学
いがた #T35 鋳型
いがみあ #W5r いがみ合
いがや #JNS 井ヶ谷
いがらし #JNS 五十嵐
いがり #JNS 猪狩
いがわ #JNS 井川 居川
いがん #T35 依願 胃癌
いき #KSr*62 生き #T35*7 域 #T30*6 息 #CNSUC1*3 行 #KS*2 活き #T35*1 意気 #T05*1 粋 #T30 遺棄 #CN 壱岐
いきいき #F04*2 イキイキ #F04*1 生き生き #F04 いきいき
いきうめ #T35 生き埋め
いきおい #T35*3 勢い
いきおいこ #M5 勢い込
いきおいづ #K5 勢い付
いきか #T35 閾下
いきかえ #R5r*1 生き返
いきがい #T35*2 生きがい
いきき #T30*2 行き来
いきぎれ #T30*1 息切れ
いきぐるし #KY 息苦し
いきぐるしゅう #KYU 息苦しゅう
いきご #M5r*2 意気込
いきさき #T35*17 行き先
いきさつ #T35*1 いきさつ #T35 経緯
いきしゅ #T35*14 異機種
いきすぎ #KSr*2 行き過ぎ
いきち #T35 生き血 閾値
いきちが #W5r 行き違
いきつき #JCN 生月
いきづま #R5r*1 息詰ま
いきとしいける #RT 生きとし生ける
いきとど #K5r*2 行き届 #K5r いきとど
いきどお #R5r 憤
いきな #JCN 生名
いきなり #F14*3 いきなり
いきぬき #T30*1 息抜き
いきのこ #R5r*5 生き残
いきもの #T35*3 生き物
いきょう #T35*3 異境 #T35 異郷
いきょうと #T35 異教徒
いきょく #T35 委曲 医局
いきわた #R5*1 行き渡
いぎ #T35*7 異義 #T35*4 意義 #T35*1 異議
いぎょう #T35*1 医業 #T35 偉業 遺業
いぎりす #CN イギリス
いく #NN*3 幾 #KJ*1 郁 #KJ 育 粥 #PRE 幾
いくいっぷめんと #T35*1 イクイップメント
いくいな #JNS 生稲
いくえ #T35 幾重
いくえい #T30*2 育英
いくお #JNM 育男 郁夫
いくこ #JNM 郁子
いくさ #T35 戦
いくさか #JCN 生坂
いくしゅ #T30*2 育種
いくじ #T30*1 育児
いくせい #T30*4 育成
いくた #JCN 生田
いくたはら #JCN 生田原
いくつ #T35*136 いくつ #T35*52 幾つ
いくつもの #RT*20 いくつもの
いくど #T35*1 幾度
いくどうおん #T35 異口同音
いくにち #T35 幾日
いくの #JCN 生野
いくばく #T35 いくばく
いくぶん #T35*2 幾分
いくま #JN 伊玖磨
いくよ #JNM 郁代
いくら #T35*20 いくら #T35*1 幾ら
いくらか #F14*1 幾らか
いぐ #JCN 伊具
いぐさ #JCN 井草
いぐち #JCN 井口
いけ #KS*31 いけ #T35*21 池 #KS 生け
いけい #T30 畏敬
いけがき #T35 生け垣
いけがみ #JCN 池上
いけがわ #JCN 池川
いけだ #JCN 池田
いけど #R5r 生け捕
いけばな #T35 生け花
いけぶくろ #CN 池袋
いけもと #JNS 池本
いけん #T30*125 意見 #T05 違憲
いげた #T35 井桁
いげん #T35*1 威厳
いこ #U5r 憩
いこーる #T35*1 ＝
いこい #T35*1 憩 #T35 憩い
いこう #T35*137 以降 #T30*137 移行 #T35*16 意向 #T30 移項
いこく #T35*2 異国
いこじ #T05 依怙地 意固地
いこつ #T35 遺骨
いこま #JCN 生駒
いこん #T35 遺恨
いご #T35*31 以後 #T35*1 囲碁
いごこち #T35*1 居心地
いごん #T30 遺言
いさ #M5*1 勇 #JCN 伊佐 #KJ 諌
いさい #T35 委細 #T05 異彩
いさお #JN 勲 功 #JNM 勇夫 勇雄
いさか #W5r いさか #JNS 伊坂 #W5r 諍
いさかい #T30*3 諍い
いさぎよ #KY*1 潔
いさぎよう #KYU*1 潔う
いさく #T35 遺作
いさこ #JNM いさ子
いささか #T36*2 いささか
いさはや #JCN 諌早
いさはら #JNS 井佐原
いさまし #KY*1 勇まし
いさましゅう #KYU*1 勇ましゅう
いさみ #T35*1 勇 #JN 勇
いさみあし #T15 勇み足
いさむ #JN 勲 勇
いさめ #KSr 諌め
いさわ #JNS 伊沢 井沢 #JCN 石和 胆沢
いさん #T35*4 遺産
いざ #F06*2 いざ
いざかや #T35*2 居酒屋
いざき #JNS 井崎
いざな #W5r 誘
いざわ #JNS 伊沢 井沢
いし #T35*21 石 #T30*13 意志 #T30*6 意思 #T35*1 医師 #T35 遺志
いしい #JCN 石井
いしおか #JCN 石岡
いしかり #CN 石狩
いしかわ #JNS 石河 #JCN 石川
いしがき #T35*3 石垣 #JCN 石垣
いしき #T30*59 意識
いしくら #JCN 石倉
いしぐろ #JCN 石黒
いしげ #JCN 石下 #JNS 石毛
いしこし #JCN 石越
いしざか #JNS 石坂
いしざき #JNS 石崎
いしざわ #JNS 石沢
いしじま #JCN 石島 #JNS 石嶋
いしずえ #T35 礎
いしそね #JNS 石曽根
いしだ #JCN 石田
いしつ #T05 異質 #T30 遺失
いしづか #JCN 石塚
いしどう #JCN 石堂
いしどりや #JCN 石鳥谷
いしなべ #JNS 石鍋
いしの #JNS 石野
いしのまき #CN 石巻
いしはた #JNS 石畑
いしはら #JNS 石原
いしばし #T35*13 石橋 #JCN 石橋
いしぶみ #T35 碑
いしべ #JCN 石部
いしまる #JCN 石丸
いしみね #JN 石嶺
いしゃ #T35*19 医者 #T30*1 慰謝
いしやま #JCN 石山
いしゅ #T35*2 異種
いしゅう #T35*2 異臭 #T30 謂集
いしゅく #T30 萎縮
いしゅつ #T30 移出
いしゆみ #T35 弩
いしょ #T35*8 遺書 #T35 医書
いしょう #T35*1 衣装 #T35 意匠
いしょく #T30*52 移植 #T05*2 異色 #T30 委嘱 衣食
いしょくじゅう #T35 衣食住
いしわたり #JNS 石渡
いしん #T35*1 維新 #T35 威信
いしんでんしん #T35 以心伝心
いじ #R5r*40 いじ #T30*27 維持 #T35*10 意地 #T35 遺児
いじく #R5*2 いじく
いじめ #KSr*3 いじめ #KSr*2 苛め
いじゅう #T30 移住
いじゅういん #JCN 伊集院
いじょう #T35*255 以上 #T05*54 異常 #T30*43 委譲 #T35*1 ≧ #T30 移譲
いじら #JCN 伊自良
いじわる #T05*2 意地悪
いじん #T35*2 偉人 異人
いす #T35*9 椅子
いすみ #JCN 夷隅
いすらえる #CN イスラエル
いすらむ #KK*1 イスラム
いすわ #R5r*1 居座
いず #JCN 伊豆
いずくんぞ #CJ いずくんぞ
いずし #JCN 出石
いずながおか #CN 伊豆長岡
いずみ #T35*8 泉 #JCN 出水 泉 和泉
いずみおおつ #CN 泉大津
いずみさき #JCN 泉崎
いずみさの #JCN 泉佐野
いずみや #JNS 泉谷
いずも #JCN 出雲
いずもざき #CN 出雲崎
いずれ #T35*41 いずれ
いせ #JCN 伊勢
いせい #T35*2 異性 #T35 威勢
いせいしゃ #T35*2 為政者
いせき #T30*1 移籍 #T35*1 遺跡
いせさき #JCN 伊勢崎
いせたん #KK*1 伊勢丹
いせつ #T35 異説 #T30 移設
いせはら #JCN 伊勢原
いせり #JNS 井芹
いせん #CN 伊仙
いぜな #JCN 伊是名
いぜん #T35*170 以前 #F02*15 依然 #T35 已然
いぜんけい #T35 已然形
いそ #G5r*42 急 #T35*2 磯 #JNS 磯
いそう #T35*8 位相 #T30 移送
いそうろう #T30*1 居候
いそがい #JNS 磯貝
いそがし #KY*136 忙し
いそがしゅう #KYU*3 忙しゅう
いそご #JCN 磯子
いそざき #JCN 磯崎
いそし #M5*1 勤し
いそじ #T35 五十路
いそだ #JNS 磯田
いその #JNS 磯野
いそべ #JNS 磯部
いそや #JCN 磯谷
いそん #T30*6 依存
いぞう #T30*1 遺贈
いぞく #T35 遺族
いぞん #T30*121 依存 #T35*1 異存
いた #S5*203 致 #KYmi*35 痛 #M5r*24 痛 #R5*21 至 #S5*10 いた #T35*7 板 #M5*1 悼 #M5 傷 #R5 到
いたい #T35*1 異体 遺体
いたいけ #T05 いたいけ
いたいじ #T35 異体字
いたいたし #KY 痛々し
いたいたしゅう #KYU 痛々しゅう
いたがき #JNS 板垣
いたく #T30*3 委託 #F14 いたく #T30 依託
いたくら #JCN 板倉
いたこ #T35 イタコ #CN 潮来
いたしかたな #KY 致し方な
いたしかたのう #KYU 致し方のう
いたずら #T00*6 悪戯 #T05*2 徒
いたずらに #F14*1 徒に
いたたまらな #KY*1 居たたまらな 居た堪らな
いたたまらのう #KYU*1 居たたまらのう 居た堪らのう
いたたまれな #KY いたたまれな 居たたまれな
いただ #K5*255 頂 #K5*192 いただ #K5*11 戴
いただき #T35*1 頂
いたって #F14 いたって
いたで #T15*1 痛手
いたどり #JCN 板取
いたの #JCN 板野
いたばし #JCN 板橋
いたまえ #T35 板前
いたまし #KY 痛まし #M5 痛まし
いたましゅう #KYU 痛ましゅう
いたみ #JCN 伊丹
いため #KS*1 痛め #KSr*1 炒め #KS 傷め
いためつけ #KS 痛めつけ
いたやなぎ #JCN 板柳
いたりあ #CN イタリア
いたわ #R5r*3 いたわ
いたん #T35 異端
いだ #K5*5 抱 #JCN 井田
いだい #T05*6 偉大 #T35*1 医大
いち #T30*255 位置 #NN*255 一 #T35*11 市 #T35*1 壱
いちい #T05*17 一意 #JNS 一井
いちいち #F14*16 いちいち #F14*2 一々
いちいてき #T05*2 一意的
いちいん #T35*2 一員 #T35*1 一因
いちう #JN 一宇
いちえん #T35 一円
いちおう #F14*155 一応 #T36*154 一応
いちかい #JNS 市貝
いちかごん #T35 一家言
いちかわ #JCN 市川
いちかわだいもん #CN 市川大門
いちがいに #F14*3 一概に
いちがお #CN 市ヶ尾
いちがや #CN 市ヶ谷
いちがん #T35*1 一丸 #T15 一眼
いちき #JCN 市来
いちげき #T30*1 一撃
いちげん #T35*4 一元 #T35 一見 #T30 一言
いちこ #JN 一子
いちご #T35*4 苺
いちごん #T30 一言
いちし #JCN 一志
いちじ #T35*22 一時 #T35*2 一次 #T35*1 一字 #T35 一事
いちじま #JCN 市島
いちじょ #T35*1 一助
いちじょう #JCN 一条
いちじょうじ #CN 一乗寺
いちじるし #KY*4 著し
いちじるしゅう #KYU*1 著しゅう
いちぞん #T35*1 一存
いちたいた #T35*2 一対多
いちたろう #JNM 一太郎 市太郎
いちだ #JCN 市田
いちだん #T35*7 一段 #T35*1 一団
いちだんらく #T30*2 一段落
いちづけ #KS*4 位置付け #T30 位置づけ
いちど #T35*213 一度
いちどう #T35*3 一同 #T35*1 一堂
いちどき #T15 いちどき 一時
いちどく #T30*3 一読
いちにん #T30*1 一任
いちにんしょう #T35 一人称
いちにんまえ #T15*1 一人前
いちねん #T35*11 一年 #T35*1 一念
いちのせ #JCN 一の瀬
いちのせき #JCN 一関
いちのへ #JCN 一戸
いちのみや #JCN 一の宮 一宮
いちのもと #JCN 櫟本
いちはさま #JCN 一迫
いちはや #KY 逸早
いちはやく #F14*1 いち早く
いちはら #JCN 市原
いちば #T35*4 市場 #JNS 市場
いちばん #T35*136 一番 #F14*6 いちばん
いちぶ #T35*93 一部
いちぶぶん #T35*3 一部分
いちぶん #T35 一文
いちまつ #T35 一抹 市松
いちむら #JNS 市村
いちめい #T35 一命
いちめん #T35*2 一面
いちもく #T30 一目
いちもくりょうぜん #T05*2 一目瞭然
いちもん #T35*2 一門
いちゃもん #T35 いちゃもん
いちや #T35*2 一夜
いちやく #T30*1 一躍
いちゅう #T35*2 意中
いちょ #T35*1 遺著
いちょう #T35*5 胃腸 #T35*1 銀杏
いちよう #T05*4 一様
いちらん #T30*255 一覧
いちり #T35*2 一理
いちりつ #T05*2 一律 #T35 市立
いちりゅう #T35*1 一流
いちる #T35 一縷
いちれい #T35*2 一例
いちれん #T35*9 一連
いちろ #T35 一路
いちろう #JNM 一朗 一郎 市郎
いっ #NN*59 一 #SX*4 逸 #S5*1 逸
いっか #T35*11 一家 #T35 一下
いっかい #T35*73 一回 #T35*62 一階 #T35*2 一介
いっかく #T35*1 一角
いっかせい #T35*1 一過性
いっかつ #T30*11 一括 #T30 一喝
いっかん #T30*20 一貫 #T35*2 一環 #T35*1 一巻
いっき #T35*3 一期 #T35*1 一揆
いっきとうせん #T15 一騎当千
いっきに #F14*12 一気に
いっきょ #T35*1 一挙
いっきょう #T35*1 一興
いっけい #T35 一計
いっけん #T30*11 一見 #T35*1 一軒
いっこう #F14*5 一向 #T35*4 一行
いっこうに #F14 いっこうに
いっこく #T35*2 一刻 #T35 一国
いっさい #T35*11 一切
いっさいがっさい #T35 一切合財
いっさく #PRE*1 一昨
いっさくじつ #T35 一昨日
いっさくねん #T35*1 一昨年
いっさんかたんそ #T35 一酸化炭素
いっし #JCN 一志
いっしき #T35*7 一式 #JN 一色
いっしつ #T35*2 一室
いっしゅ #T35*5 一種 #T35*2 一首
いっしゅう #T30*5 一周 #T30 一蹴
いっしゅうかん #T35*13 一週間
いっしゅん #T35*10 一瞬
いっしょ #T10*136 一緒 #T10*7 いっしょ
いっしょう #T35*22 一生 #T35*10 一章
いっしょうけんめい #T05*9 一生懸命 #T05*6 一所懸命
いっしょくた #T06 いっしょくた 一緒くた
いっしょけんめい #T05*1 一所懸命
いっしん #T35*2 一審 #T35*1 一心 #T30 一新
いっしんに #F14 一心に
いっすい #T30*2 一睡
いっせい #F14*2 一斉 #T35 一世
いっせいに #F14*3 一斉に #F14*1 いっせいに
いっせきにちょう #T35*1 一石二鳥
いっせつ #T35*1 一説
いっせん #T35*1 一千 一線
いっそ #F14*3 いっそ
いっそう #F14*4 いっそう #T35*4 一層 #T30*1 一掃
いっそく #T35 一足
いっその #RT いっその
いったい #T05*27 一体 #F14*6 いったい #T35*1 一帯
いったん #F14*41 一旦 #F14*2 いったん #T35*2 一端 #F14*1 一たん
いっち #T30*59 一致
いっちゅうや #T35 一昼夜
いっちょういっせき #T35*2 一朝一夕
いっちょういったん #T35*2 一長一短
いっちょうら #T35 一張羅
いって #T35*11 一手
いってい #T00*14 一定
いってん #T35*2 一点 #T30*1 一転
いっと #T35*1 一途
いっとう #T35*2 一等
いっとき #T35 いっとき 一刻 一時
いっぱ #T35*2 一派
いっぱい #T05*120 いっぱい #T35*35 一杯
いっぱく #T30*6 一泊
いっぱつ #T35*7 一発
いっぱん #T35*167 一般
いっぱんてき #T15 一般的
いっぴつ #T35*1 一筆
いっぴん #T35 逸品
いっぷく #T30 一服
いっぺい #JN 一平
いっぺん #T35*7 一辺 #T30*2 一変
いっぺんとう #T15*1 一辺倒
いっぽ #T35*4 一歩
いっぽう #T35*49 一方 #CJ*4 いっぽう #T30*1 一報
いっぽん #T35*8 一本
いっぽんまつ #JCN 一本松
いつ #T35*163 いつ #T35*22 何時 #KJ*1 一 逸 #KJ 壱 溢 乙 伍
いつう #T35*1 胃痛
いつか #T35*2 五日 #T35 5日 ５日
いつかいち #JCN 五日市
いつき #JCN 五木
いつくし #M5r 慈し
いつこ #JNM 伊都子
いつざい #T35 逸材
いつぞや #T36 いつぞや
いつだつ #T30*1 逸脱
いつつ #T35*3 五つ
いつはし #JNS 五橋
いつぶ #S5r 鋳潰
いつまで #F14*8 いつまで
いつみ #JNM いつみ
いつも #F14*27 いつも #T36*26 いつも
いつわ #R5r*2 偽 #T35*1 逸話 #JN 五輪 #JCN 五和
いづはら #JCN 厳原
いて #T35 射手
いてき #T35 夷狄
いてざ #KK*1 射手座
いてつ #K5*2 凍てつ
いてん #T30*3 移転
いで #CN 井手 #JCN 井出
いでぃおむ #T35 イディオム
いでおろぎー #T35*1 イデオロギー
いでみつ #KK*2 出光
いでむら #JNS 出村
いでん #T30*1 遺伝
いでんし #T35*1 遺伝子
いと #T30*31 意図 #U5*10 厭 #T35*2 糸 #JCN 伊都
いといがわ #CN 糸魚川
いとう #KYU*2 痛う #JCN 伊東 #JNS 伊藤
いとうちゅう #KK*4 伊藤忠
いとおし #KYmi*2 いとおし
いとおしゅう #KYU*1 いとおしゅう
いとぐち #T35*1 糸口
いとこ #T35*4 いとこ
いとし #KYmi*3 愛し
いとしの #RT 愛しの
いとしま #JCN 糸島
いとたに #JNS 糸谷
いとだ #JCN 糸田
いとな #M5r*1 営
いとぬき #JCN 糸貫
いとま #T35*4 暇
いとまん #JCN 糸満
いとめ #KS 射止め
いとも #F14 いとも
いど #T35*3 井戸 #T35 緯度 #M5 挑
いどう #T30*255 移動 #T35*1 異同
いどころ #T35*1 居所
いどばた #T35*3 井戸端
いな #T35*85 否 #JCN 伊奈 伊那 伊南 #KJ 稲 #M5 否
いない #T35*33 以内 #JCN 稲井
いなお #R5r 居直
いなおか #JNS 稲岡
いなか #T35*5 田舎
いなかだて #JCN 田舎館
いなかっぺ #T35 田舎っぺ
いなかわ #JCN 稲川
いながき #JCN 稲垣
いながらにして #F14 いながらにして
いながわ #JCN 猪名川
いなぎ #JCN 稲城
いなさ #JCN 引佐
いなさぐん #CN 引佐郡
いなざわ #JCN 稲沢
いなしき #JCN 稲敷
いなずま #T35*1 稲妻
いなせ #T05 いなせ
いなだ #JCN 稲田
いなつき #JCN 稲築
いなにわ #JCN 稲庭
いなば #JNS 稲葉 #JCN 因幡
いなびかり #T35*1 稲光
いなぶ #JCN 稲武
いなべ #JCN 員弁
いなみ #JN 井波 #JCN 稲美 印南
いなむら #JCN 稲村
いなら #B5 居並
いなり #T35*1 稲荷
いなわしろ #CN 猪苗代
いなん #CNSUC1*2 以南
いにしえ #T35*1 いにしえ
いにしえーた #T35*2 イニシエータ
いにしゃらいず #T30*4 イニシャライズ
いにしゃる #T35*3 イニシャル
いにゅー #KK イニュー
いにゅう #T30 移入
いにん #T30*2 委任
いぬ #T35*5 犬 #T35 狗 #K5 射抜
いぬい #T35*1 乾
いぬかい #JCN 犬飼
いぬかみ #JCN 犬上
いぬころ #T35 犬ころ
いぬっころ #T35 犬っころ
いぬやま #JCN 犬山
いね #T35*5 稲 #JCN 伊根 #T35 禾
いねかり #T30 稲刈り
いねむり #T30*1 居眠り
いの #R5r*10 祈 #JCN 伊野 #R5 祷
いのうえ #JNS 井上
いのき #JNS 猪木
いのくち #JCN 井口
いのこ #R5r*1 居残
いのしし #T35*1 猪
いのせ #JNS 猪瀬
いのち #T35*10 命
いのちごい #T30 命乞い
いのまた #JNS 猪股
いはい #T30*2 違背
いはく #T35*1 曰く
いはら #JCN 庵原
いはん #T10*8 違反
いば #R5*4 威張
いばしょ #T35*3 居場所
いばら #T35*2 茨 #JCN 井原
いばらき #JNS 茨城 #JCN 茨木
いひょう #T35 意表
いひん #T35 遺品
いび #JCN 揖斐
いびがわ #CN 揖斐川
いびつ #T05*1 いびつ
いふ #T30*1 畏怖
いふう #T35 威風
いふく #T35 衣服
いぶき #JCN 伊吹 #T35 息吹
いぶすき #CN 指宿 揖宿
いぶつ #T35*2 遺物 #T35 異物
いぶり #CN 胆振
いぷしろん #T35*2 Ε ε
いへや #CN 伊平屋
いへん #T35 異変
いべんと #T35*74 イベント
いほう #T05*2 違法 #T35 異邦
いぼ #JCN 揖保
いぼがわ #JCN 揖保川
いま #T35*255 今 #T35*29 いま #T35*3 居間
いまい #JCN 今井
いまいずみ #JCN 今泉
いまいち #T15*55 いまいち #JCN 今市
いまいまし #KY 忌々し
いまいましゅう #KYU 忌々しゅう
いまがね #JCN 今金
いまがわ #JCN 今川
いまごろ #F14*8 今ごろ
いまさら #F14*13 いまさら #F14*3 今更 #F14*1 今さら #T36 いまさら 今さら 今更
いましがた #F14 今しがた
いましめ #KSr 戒め
いまじょう #JCN 今庄
いまぜき #JNS 今関
いまだ #F14*16 いまだ #F14*11 未だ #JNS 今田
いまだて #JCN 今立
いまだに #F14*16 いまだに #F14*11 未だに
いまづ #JCN 今津
いまでがわ #JCN 今出川
いまばり #JCN 今治
いまべつ #JCN 今別
いままで #F14*47 今まで #F14 いままで
いまむら #JNS 今村
いまや #F14*1 いまや
いまり #CN 伊万里
いまわし #KYmi*1 忌まわし
いまわしゅう #KYU*1 忌まわしゅう
いみ #T30*255 意味
いみず #JCN 射水
いみでぃえいと #T05*1 イミディエイト
いみん #T30*1 移民
いむ #T35*1 医務
いむら #JNS 井村
いめーじ #T30*98 イメージ
いめい #T35 違名
いめちぇん #T30 イメチェン
いも #T35*17 芋 #T35*5 いも
いもうと #T35*6 妹
いもの #T35 鋳物
いもむし #T35 芋虫
いもん #T30 慰問
いや #T05*35 嫌 #T05*17 いや #KJ*1 彌 #T05 厭 #JCN 祖谷 #S5 癒
いやおう #T35*1 否応
いやおうなしに #F14*1 否応なしに
いやがおうでも #CJ いやが応でも
いやがさこ #JCN 伊屋ヶ迫
いやく #T30*1 意訳 #T35*1 医薬 #T30 違約
いやくきん #T35 違約金
いやけ #T35*1 嫌気
いやし #KYmi 卑し #M5 卑し
いやしめ #KS 卑しめ
いやしゅう #KYU 卑しゅう
いやに #F14 いやに
いやま #S5r いや増
いやみ #T05*2 厭味 #T05*1 嫌味
いやらし #KY*1 いやらし
いやらしゅう #KYU*1 いやらしゅう
いやりんぐ #T35 イヤリング
いゆう #T35 畏友
いよ #JN 伊代 #JCN 伊予
いよいよ #F14*5 いよいよ
いよう #T05*5 異様 #T35 威容
いよく #T35*2 意欲
いよじ #JN 五代嗣
いよみしま #CN 伊予三島
いら #KJ*6 苛
いらい #T30*194 依頼 #T36*21 以来 #F14*20 以来
いらいら #F04*26 イライラ #F04 苛々
いらか #T35 甍
いらすと #T35 イラスト
いらだ #T5r*2 苛立
いらっしゃ #L5*40 いらっしゃ
いらぶ #JCN 伊良部
いらんくう #T35*1 囲卵腔
いりえ #T35*10 入江 #T35*1 入り江 #JCN 入江
いりおもて #CN 西表
いりがい #JNS 入貝
いりき #JCN 入来
いりく #M5 入り組
いりぐち #T35*15 入口
いりこ #M5 入り込
いりの #JCN 入野
いりひろせ #CN 入広瀬
いりゅう #T30*1 慰留 #T30 遺留
いりょう #T35*2 医療 #T35*1 衣料
いりょく #T35*2 威力
いる #CNSUC1*13 入ル
いるーじょん #T35*1 イルージョン
いるい #T35*2 衣類
いるか #T35*2 海豚 #T35*1 鯆
いるま #JCN 入間
いれ #KS*255 入れ #KS*1 容れ #T35 入れ
いれい #T05*1 異例
いれかえ #KSr*27 入れ換え 入れ替え
いれかわ #R5r*9 入れ替わ
いれこ #T35*1 入れ子 #M5r 入れ込
いれぢえ #T30 入れ知恵
いれなお #S5r*3 入れ直
いろ #T35*69 色
いろあい #T35 色合い
いろあせ #KSr*1 色あせ 色褪せ #T30 色褪せ
いろいろ #T05*160 いろいろ #F14*157 いろいろ #T05*81 色々 #F14*40 色々 #F14*3 色色 #T05*1 色色
いろう #T30*1 慰労 #T35 遺漏
いろけ #T35*5 色気
いろこ #KY*1 色濃 #T35 色子
いろこう #KYU*1 色濃う
いろずり #T30 色刷
いろっぽ #KY*1 色っぽ
いろづ #K5r 色づ
いろど #R5r*1 彩
いろは #T35*20 いろは
いろめ #K5 色め
いろわけ #T30 色分け
いろん #T35*2 異論
いろんな #RT*6 いろんな #RT*2 色んな
いわ #W5r*9 祝 #T35*8 岩 #T35 磐
いわい #JCN 岩井 磐井
いわいずみ #JCN 岩泉
いわお #JN 巌 #T35 巌 #JN 岩生 岩男 岩夫 岩雄
いわかん #T35*4 違和感
いわき #CN いわき #JCN 岩城 岩木 磐城
いわぎ #JCN 岩城
いわく #T35*14 曰く
いわくに #JCN 岩国
いわくら #JCN 岩倉
いわさ #JNS 岩佐
いわさき #JCN 岩崎
いわさわ #JNS 岩沢
いわし #T35*13 鰯 #T35*1 鰮
いわした #JNS 岩下
いわしろ #JCN 岩代
いわせ #JCN 岩瀬
いわた #JCN 岩田 磐田
いわたき #JCN 岩滝
いわつき #CN 岩槻
いわて #JCN 岩手
いわで #JCN 岩出
いわでやま #JCN 岩出山
いわない #JCN 岩内
いわなみ #KK*3 岩波
いわぬま #JCN 岩沼
いわはだ #T35 岩肌
いわば #F14*2 いわば #F14*1 謂わば 言わば #T35 岩場
いわふね #JNS 岩舟 #JCN 岩船
いわぶち #JCN 岩淵
いわま #JCN 岩間
いわみ #JCN 岩美 石見
いわみざわ #CN 岩見沢
いわむら #JCN 岩村
いわむろ #JCN 岩室
いわもと #JCN 岩本
いわゆる #RT*16 いわゆる
いん #SUC*35 員 #SUC*28 院 #T30*23 印 #T35*20 イン #T35*3 陰 #KJ*3 隠 #KJ*1 員 #T30*1 因 #KJ*1 引 #T35*1 韻 #KJ*1 音 #KJ 允 咽 姻 #T30 飲 #KJ 淫 胤 蔭 院
いんいおん #T35 陰イオン
いんえい #T35*1 陰影
いんか #T30*1 印加 引火 #T35*1 陰窩
いんかく #T35*2 陰核
いんかん #T35*6 印鑑
いんが #T35*3 因果
いんき #T05*3 陰気
いんきゅべーしょん #T30*1 インキュベーション
いんきゅべーた #T35*1 インキュベータ
いんきゅべーと #T30*1 インキュベート
いんきょ #T30 隠居
いんきょく #T35*1 陰極
いんきん #T35 陰金
いんぎん #T05 慇懃
いんく #T35*2 インク
いんくりめんたる #T05*4 インクリメンタル
いんくりめんと #T30*11 インクリメント
いんくるーど #T30*45 インクルード
いんけい #T35*1 陰茎
いんけん #T05*1 陰険
いんこう #T30*1 淫行 #T35 咽喉
いんごう #T05 因業
いんさーと #T30*1 インサート
いんさつ #T30*43 印刷
いんさん #T05 陰惨
いんざい #JCN 印西
いんし #T35*4 因子 #T35*3 院試 #T35*1 印紙
いんしつ #T05*1 陰湿
いんしゅ #T30*2 飲酒
いんしゅう #T35*1 因習
いんしゅりん #T35*1 インシュリン
いんしょう #T35*23 印象 #T35 印章
いんしょうづけ #KSr 印象づけ #T30 印象付け
いんしょく #T30*2 飲食
いんじ #T30*20 印字 #T35 印璽
いんじぇくしょん #T30*1 インジェクション
いんじけーた #T35*19 インジケータ
いんすう #T35 員数
いんすたんす #T35*25 インスタンス
いんすたんと #T35*1 インスタント
いんすとーる #T30*255 インストール
いんすとらくしょん #T35*19 インストラクション
いんすぺくしょん #T30*3 インスペクション
いんせい #T35 院政 院生 陰性 #T30 隠棲
いんせき #T30 引責
いんせん #T35*1 隠線
いんぜい #T35*1 印税
いんぜみ #T35*1 院ゼミ
いんそつ #T30 引率
いんたーせぷと #T30*1 インターセプト
いんたーちぇんじ #T35 インターチェンジ
いんたーなる #T35 インターナル
いんたーねっと #KK*232 インターネット
いんたーばる #T35*2 インターバル
いんたーふぇーす #T35*68 インターフェース
いんたーふぇいす #T35*45 インターフェイス
いんたーぷりた #T35*2 インタープリタ
いんたーりーぶ #T30*1 インターリーブ
いんたい #T30*3 引退
いんたせぷと #T35*1 インタセプト
いんたびゅー #T30*4 インタビュー
いんたふぇーす #T35*238 インタフェース
いんたふぇいす #T35*16 インタフェイス
いんたふぇす #T35*3 インタフェス
いんたぷりた #T35*10 インタプリタ
いんたぷりと #T30*1 インタプリト
いんたらくしょん #T35*7 インタラクション
いんたらくてぃぶ #T05*13 インタラクティブ
いんだいれくと #T05*1 インダイレクト
いんだくしょん #T30*1 インダクション
いんだすとり #T35*2 インダストリ
いんだすとりある #T05*3 インダストリアル
いんち #T35*16 インチ #JS インチ 吋
いんちょう #T35*1 院長
いんてぐれーしょん #T35*4 インテグレーション
いんてらくてぃぶ #T35*22 インテラクティブ
いんてり #T15 インテリ
いんてりあ #T35 インテリア
いんてりげんちゃ #T35 インテリゲンチャ
いんてる #KK*2 インテル
いんでぃあん #T35*1 インディアン
いんできしんぐ #T30*1 インデキシング
いんでっくす #T35*8 インデックス
いんでんと #T30*8 インデント
いんとう #T05 淫蕩
いんとく #T30 隠匿
いんとねーしょん #T35*1 イントネーション
いんど #CN インド 印度
いんどよう #CN インド洋
いんない #JCN 院内
いんなみ #JCN 印南
いんねん #T35*1 因縁
いんのう #T35*1 陰嚢
いんのしま #CN 因島
いんば #JCN 印旛
いんばい #T35 淫売
いんぱくと #T35*5 インパクト
いんぱるす #T35*1 インパルス
いんび #T05 淫靡
いんぴーだんす #T35*2 インピーダンス
いんふぉめーしょん #T35*3 インフォメーション
いんふるえんざ #T35*1 インフルエンザ
いんふれ #T35 インフレ
いんぷっと #T30*1 インプット
いんぷりめんた #T35*10 インプリメンタ
いんぷりめんてーしょん #T35*3 インプリメンテーション
いんぷりめんと #T30*52 インプリメント
いんへりたんす #T35*2 インヘリタンス
いんぺい #T30*3 隠蔽
いんぺいし #T35*1 隠蔽子
いんぼう #T35*1 陰謀
いんめん #T35*13 隠面
いんゆ #T35 隠喩
いんよう #T30*32 引用 #T35*2 陰陽 #T30 飲用
いんりょう #T35 飲料
いんりょうすい #T35 飲料水
いんりょく #T35*1 引力
いんれき #T35 陰暦
いんろう #T35*1 印籠
う #T35*222 鵜 #T5*176 打 #R5r*127 売 #R5*44 得 #K5r*17 浮 #KJ*12 雨 #R5*5 う #T5*5 う #KJ*5 宇 #KY*5 憂 #M5r*3 生 #PRE*2 右 #KJ*2 卯 #KJ*1 右 羽 #T5*1 撃 #M5r*1 産 #T5*1 討 #KJ*1 有 #KJ 芋 烏 迂 #SUC 雨 #M5 倦 #KJ 胡 兎 菟 得 #M5 膿 #T5 伐
う゛ぁいおりん #T35 ヴァイオリン
う゛ぃーなす #JN ヴィーナス
うーはん #CN 武漢
うーふぁ #T35*1 ウーファ
うーまん #T35*1 ウーマン
うーむ #CJ*8 うーむ
うーろんちゃ #T35*1 烏龍茶
うーん #CJ*21 うーん
うぃーく #T35*3 ウィーク
うぃじっと #T35 ウィジット
うぃりあむ #JN ウィリアム
うぃるす #T35*2 ウィルス
うぃんど #T35*4 ウィンド
うぃんどう #T35*69 ウィンドウ
うい #KJ*8 初 #T35 有為
ういういし #KY 初々し
ういういしゅう #KYU 初々しゅう
ういざん #T35*1 初産
ういすきー #T35*1 ウイスキー
ういるす #T35*1 ウイルス
ういんく #T30*5 ウインク
ういんどう #T35*75 ウインドウ
うぇい #T35*3 ウェイ
うぇいと #T30*1 ウェイト
うえ #T35*255 上 #KS*42 植え #KSr*13 飢え #T35*8 うえ #PRE*2 上 #JNS 上
うえーぶふろんと #KK*2 ウエーブフロント
うえいと #T30*3 ウエイト
うえおか #JNS 植岡
うえき #JCN 植木 #T35 植木
うえこ #M5r 植え込
うえした #T30 上下
うえしば #JNS 植芝
うえじに #T30*1 餓死
うえすぎ #JNS 上杉
うえだ #JCN 上田 #JNS 植田
うえつきぼう #T35*1 ￣
うえつけ #KSr*1 植え付け
うえの #JCN 上野
うえのはら #JCN 上野原
うえは #T35*1 ウエハ
うえはら #JCN 上原 #JNS 植原
うえほんまち #CN 上本町
うえまつ #JNS 植松
うえむら #JCN 上村 #JNS 植村
うえもと #JNS 植本
うえや #T35*3 ↑
うえん #T05 迂遠
うぉーくするー #T35*4 ウォークスルー
うぉーくまん #KK*1 ウォークマン
うぉーたーふぉーる #T35*1 ウォーターフォール
うぉーみんぐ #T30*1 ウォーミング
うぉーむ #T35*1 ウォーム
うぉずにあく #JN ウォズニアク
うお #T35*9 魚
うおうさおう #T30*1 右往左往
うおざ #KK*1 魚座
うおしま #JCN 魚島
うおせ #JNS 魚瀬
うおっちんぐ #T35*2 ウオッチング
うおづ #JCN 魚津
うおぬま #JCN 魚沼
うか #B5*11 浮か #B5*1 うか #R5*1 受か #T30 羽化
うかい #T30*3 迂回 #T35 鵜飼い
うかうか #T30*1 うかうか
うかが #W5r*40 伺 #W5r*1 うかが #W5 窺
うかがい #KJ 窺
うかつ #T05 うかつ 迂闊
うかべ #KS*1 浮かべ
うかれ #KS 浮かれ
うかん #JN 有漢
うがん #T35 右岸
うき #T35 右記 雨期 雨季
うきがや #JNS 浮ヶ谷
うきくさ #T35 浮草
うきしずみ #T30*1 浮き沈み
うきな #T35 浮名
うきは #JCN 浮羽
うきぶくろ #T35 浮袋
うきぼり #T30*1 浮き彫り
うきょう #JCN 右京
うきよえ #T35 浮世絵
うく #JCN 宇久
うぐいす #T35*1 鴬
うぐいすざわ #JCN 鴬沢
うぐいすだに #CN 鴬谷
うけ #KSr*101 受け #KS*6 うけ #KS 承け #KSr 請け
うけあ #W5r*10 請け合
うけい #T30*1 右傾
うけいれ #KSr*7 受け入れ
うけうり #T30 受け売り
うけお #W5r*1 請け負
うけおい #T30*1 請負
うけこたえ #T30*2 受け答え
うけざら #T35 受け皿
うけしょ #T35*1 受書 請書
うけたまわ #R5*2 承
うけつ #G5r*2 受け継
うけつけ #KS*48 受け付け #T30*25 受付
うけて #T35*1 受け手
うけと #R5r*77 受け取
うけとめ #KSr 受け止め
うけとり #T35*1 受取
うけな #JN 浮穴
うけなが #S5r 受け流
うけにん #T35 請人
うけはらい #T30 受払い
うけみ #T35*1 受身
うけも #T5r*2 受け持
うけわたし #T30*6 受渡し
うけん #JCN 宇検
うこん #T35 鬱金
うご #K5r*255 動 #K5*1 うご #CN 羽後
うごか #S5*92 動か #S5 うごか
うごきだ #S5*4 動き出
うごめ #K5*1 蠕
うさ #JCN 宇佐
うさぎ #T35*6 兎 #T35 菟
うさみ #JCN 宇佐見 宇佐美
うざわ #JNS 鵜沢
うし #T35*23 牛 #T35*1 丑
うしお #T35*1 潮
うしがえる #T35 牛蛙
うしく #JCN 牛久
うしじま #JCN 牛島
うしづ #JCN 牛津
うしな #W5*24 失 #W5 喪
うしぶか #JCN 牛深
うしぼり #JCN 牛堀
うしまど #JCN 牛窓
うしみつ #T35 丑三つ
うしみつどき #T35 丑三つ刻
うしろ #T35*91 後ろ
うしろあし #T35 後ろ足
うしろすがた #T35*1 後ろ姿
うしろだて #T35 後ろだて
うしろむき #T35*1 後向き
うしろめた #KY*1 後ろめた
うしろめとう #KYU*1 後ろめとう
うしわた #JNS 牛渡
うじ #T35*2 氏 #T35*1 蛆 #JCN 宇治
うじいえ #JCN 氏家
うじがみ #T35 氏神
うじたわら #CN 宇治田原
うじむし #T35 蛆虫
うす #KYmi*10 薄 #T35*4 碓 #T35 臼 #JCN 有珠
うすい #T35*1 雨水 #JNS 碓井 #JCN 碓氷 臼井 #JNS 薄井
うすう #KYU*2 薄う
うすうす #F14 薄々
うすがた #T15*1 薄型
うすき #JCN 臼杵
うすぎ #T10*1 薄着
うすぎり #T30 薄切り
うすくち #T05 薄口
うすぐら #KY*1 薄暗
うすぐろう #KYU*1 薄暗う
うすじ #T15 薄地
うすじお #T15 薄塩
うすだ #JCN 臼田
うすちゃ #T35 薄茶
うすで #T05*1 薄手
うすび #T35 薄日
うすま #R5r 薄ま
うすまく #T35 薄膜
うすめ #KSr*1 薄め
うすら #G5r 薄ら
うすれ #KS*2 薄れ
うず #T35*3 渦
うずうず #F04 うずうず
うずくま #R5 蹲
うずま #K5 渦巻 #R5 埋ま
うずまき #T35*1 渦巻
うずめ #KS*1 うずめ #KS 埋め
うずもれ #KS 埋もれ
うせ #KS*3 失せ
うせつ #T30*1 右折
うそ #T35*13 嘘 #T35*7 ウソ
うそく #T35 右側
うそっぱち #T05 嘘っ八
うそぶ #K5 うそぶ
うぞうむぞう #T35 有象無象
うた #W5*32 歌 #T35*27 歌 #W5*2 唱 #T35*1 唄 #W5*1 唄 詠 #T35*1 詩 #W5 謡
うたい #T35 謡
うたが #W5r*8 疑
うたがわし #KY*1 疑わし
うたがわしゅう #KYU*1 疑わしゅう
うたぐ #R5r*1 疑
うたぐりぶか #KY 疑り深
うたぐりぶこう #KYU 疑り深う
うたげ #T35*2 宴
うたごえ #T35 歌声
うたしない #CN 歌志内
うたしろ #JN 歌代
うたた #F14 うたた
うたたね #T30 うたた寝
うたつ #JCN 歌津
うたづ #JCN 宇多津
うたの #JCN 菟田野
うたのぼり #JCN 歌登
うたまろ #JNM 歌麿
うたん #T35*5 右端
うだ #JCN 宇陀
うだがわ #JCN 宇田川
うち #T35*83 内 #T35*55 うち 家 #PRE*1 内 #T35 中 裡
うちあ #W5r*1 撃ち合
うちあけ #KS*3 打ち明け
うちあけばなし #T35 打ち明け話
うちあげ #KSr*3 打ち上げ
うちあわせ #T30*56 打合せ #KSr*26 打ち合せ
うちいわい #T30 内祝
うちうち #T35 内々
うちうみ #JCN 内海
うちうら #JCN 内浦
うちおと #S5 撃ち落と
うちか #T5 打ち勝
うちかけ #T35 打掛け
うちかんだ #CN 内神田
うちがわ #T35*2 内側
うちき #R5r*1 打ち切 #T05*1 内気 #JNS 内木
うちきず #T35 打傷
うちきん #T35 内金
うちくだ #K5*1 打ち砕
うちけ #S5r*1 打ち消
うちこ #M5r*4 打ち込 #JCN 内子
うちさいわい #CN 内幸
うちた #JCN 打田
うちだ #S5r*4 打ち出 #JNS 内田
うちづま #JNS 内妻
うちでし #T35 内弟子
うちとけ #KS*2 打ち解け
うちなだ #JCN 内灘
うちの #JNS 内野
うちのうら #CN 内之浦
うちのみ #JCN 内海
うちはら #JCN 内原
うちひし #G5 打ちひし
うちべんけい #T15 内弁慶
うちぼり #T35*1 内堀
うちまく #T35 内幕
うちみ #T35 打ち身
うちみず #T30 打ち水
うちやぶ #R5 打ち破
うちやま #JCN 内山
うちゅう #T35*31 宇宙
うちょうてん #T05 有頂天
うちわ #T15*5 内輪
うちわけ #T35*3 内訳
うっかり #T30*4 うっかり
うっくつ #T30 鬱屈
うっせき #T30*1 うっ積 鬱積
うったえ #KSr*2 訴え
うってつけ #T05*1 うってつけ
うっとうし #KY*2 うっとうし
うっとうしゅう #KYU*1 うっとうしゅう
うつ #R5*77 移 #S5*61 移 #S5r*33 写 #R5r*15 写 #R5r*6 映 #S5*4 映 #R5r*2 うつ #S5*2 うつ #T35*1 欝 #KJ*1 鬱 #KJ 蔚
うつき #JN 宇津喜
うつくし #KY*15 美し
うつくしゅう #KYU*2 美しゅう
うつしだ #S5 映し出
うつしと #R5*1 写し取
うつのみや #JCN 宇都宮
うつびょう #T35 鬱病
うつぼ #T35*1 靭
うつみ #JCN 内海
うつむ #K5*4 うつむ #K5r 俯
うつりぎ #T05 移り気
うつろ #W5r 移ろ #T05 虚ろ
うつわ #T35*2 器
うづき #T35 卯月
うてん #T35*5 雨天
うで #T35*5 腕
うできき #T35 腕利き
うでどけい #T35*1 腕時計
うでまえ #T35 腕前
うと #KYmi*6 疎 #M5*1 疎 #JCN 宇土
うとう #KYU*1 疎う
うとまし #KY 疎まし
うとましゅう #KYU 疎ましゅう
うとん #ZX 疎ん
うどの #JCN 鵜殿
うどん #T35*4 うどん #T35 饂飩
うどんこ #T35 うどん粉 饂飩粉
うな #R5*16 うな
うなかみ #JCN 海上
うなが #S5*5 促
うなぎ #T35*3 鰻
うなじ #T35*1 項
うなず #K5r*1 うなず #K5r 頷
うなづき #JCN 宇奈月
うなばら #T35*1 海原
うぬぼれ #KSr うぬぼれ 自惚れ
うね #T35 畦 畝
うねび #JCN 畝傍
うの #JCN 宇野
うのけ #CN 宇ノ気
うのみ #T35 鵜呑み
うは #T35 右派
うば #W5*2 奪 #T35*1 姥 乳母
うばいと #R5*1 奪い取
うふふ #CJ*1 うふふ
うぶ #T35*3 産 #T05 初
うぶぎ #T35 産着
うぶげ #T35 産毛
うぶごえ #T35 産声
うぶすな #T35 産土
うぶやま #JCN 産山
うぶゆ #T35 産湯
うへん #T35*4 右辺
うべ #JCN 宇部
うま #KYmi*245 うま #T35*31 馬 #R5*2 埋ま #KY*1 旨 #JCN 宇摩 #T35 午
うまじ #JCN 馬路
うまづら #T15 馬面
うまに #T35 甘煮
うまの #JNS 馬野
うまみ #T35*2 旨味
うまや #T35 厩
うまれ #KSr*23 生まれ #KS 産まれ
うまれかわ #R5r 生まれ変わ
うまれつき #T35*4 生まれつき
うみ #T35*33 海 #JCN 宇美 #T35 膿
うみおと #S5 産み落と
うみせんやません #T15*1 海千山千
うみぞい #T35 海沿い
うみだ #S5*5 生み出
うみどり #T35 海鳥
うみべ #T35*1 海辺
うむ #T35*22 有無
うむらうと #T30*1 ウムラウト
うめ #KS*66 埋め #T35*16 梅 #CN 宇目 #K5r 呻
うめあわせ #T30*1 埋め合わせ #T30 埋め合せ
うめきち #JN 梅吉
うめこ #M5r*3 埋め込
うめさわ #JNS 梅沢
うめざわ #JNS 梅沢
うめしゅ #T35 梅酒
うめぞの #JNS 梅園
うめたて #KSr*2 埋め立て #T30 埋立
うめたてち #T35*1 埋立地
うめだ #JCN 梅田
うめづ #JCN 梅津
うめはら #JNS 梅原
うめぼし #T35*4 梅干し
うめもり #JNS 梅森
うもう #KYU*3 うもう #KYU*1 旨う #T35 羽毛
うもれ #KS*4 埋もれ
うやうやし #KY 恭し
うやうやしゅう #KYU 恭しゅう
うやま #W5r*1 敬 #JNS 宇山
うやむや #T05*1 うやむや
うよ #T35*1 紆余
うよく #T35*1 右翼
うら #T35*30 裏 #T35*26 浦 #M5r*1 怨 恨 #T35 裡
うらうす #CN 浦臼
うらおもて #T35 裏表
うらかた #T35*1 裏方
うらかわ #JCN 浦河
うらがえ #S5r*4 裏返 #R5 裏返
うらがき #T30 裏書
うらがなし #KY うら悲し
うらがなしゅう #KYU うら悲しゅう
うらがね #T35 裏金
うらがわ #T35*2 裏側 #JNS 浦川
うらぎ #R5r*5 裏切
うらぐち #T35*1 裏口
うらさく #T35 裏作
うらさびし #KY うら寂し
うらさびしゅう #KYU うら寂しゅう
うらしの #JNS 嬉野
うらじ #T35 裏地
うらそえ #JCN 浦添
うらづけ #KSr*1 裏付け
うらて #T35*1 裏手
うらな #W5r*2 占
うらない #T35 卦
うらにわ #T35*2 裏庭
うらの #JNS 浦野
うらはら #T05*1 裏腹
うらべ #JNS 浦部
うらほろ #JCN 浦幌
うらまち #T35*1 裏町
うらみち #T35 裏道
うらめ #T35*1 裏目
うらめし #KY 恨めし
うらめしゅう #KYU 恨めしゅう
うらや #M5r 羨
うらやす #JCN 浦安
うらやま #T35 裏山
うらやまし #KY*14 羨まし
うらやましゅう #KYU*2 羨ましゅう
うららか #T05 うららか 麗らか
うらわ #JCN 浦和
うらん #T35*1 ウラン
うり #T35*4 瓜
うりあげ #T35*4 売上 #KS*2 売り上げ
うりあげきん #T35 売上金
うりあげだか #T35 売上高
うりかい #T30*1 売り買い
うりかけ #T30*1 売り掛け
うりかけきん #T35 売掛金
うりきれ #KSr*2 売り切れ
うりこ #M5r*1 売り込 #T35 売子
うりだ #S5r*2 売り出
うりだし #T35 売出
うりづら #JCN 瓜連
うりて #T35*1 売り手
うりぬけ #KSr 売り抜け
うりぬし #T35 売り主
うりね #T35 売値
うりはら #W5 売り払
うりば #T35*2 売場
うりゅう #JCN 雨竜 #JNS 雨龍
うりょう #T35*2 雨量
うりわた #S5r 売り渡
うるう #T35*3 閏
うるうどし #T35*2 閏年
うるお #W5r 潤
うるき #JNS 売木
うるさ #KY*21 うるさ #KY*1 五月蝿 煩
うるし #T35 漆
うるそう #KYU*2 うるそう #KYU*1 煩う
うるち #T35 粳
うるちまい #T35 粳米
うるわし #KY*1 麗し
うるわしの #RT 麗しの
うるわしゅう #KYU*1 麗しゅう
うれ #KS*2 熟れ
うれい #T35*2 愁い 憂い
うれえ #KSr 愁え 憂え
うれくち #T35 売れ口
うれし #KYmi*97 嬉し #KYmi*23 うれし
うれしの #JCN 嬉野
うれしゅう #KYU*2 うれしゅう 嬉しゅう
うれたん #T35 ウレタン
うれっこ #T05*2 売れっ子
うれのこ #R5r 売れ残
うれゆき #T35 売行き
うろ #T35 迂路
うろうろ #T30*3 うろうろ
うろおぼえ #T10*1 うろ覚え
うろこ #T35*1 鱗
うろたえ #KSr うろたえ
うろん #T35 胡乱
うわ #T35*3 上 #JCN 宇和
うわがき #T30*74 上書き
うわがわ #T35 上側
うわき #T05*2 浮気
うわぎ #T35*2 上着
うわぐすり #T35 釉
うわぐつ #T35 上靴
うわさ #T30*31 噂
うわじま #JCN 宇和島
うわすべり #T10 上滑り
うわづみ #T30 上積み
うわぬり #T30 上塗り
うわのせ #T30*1 上乗せ
うわばき #T35 上履き
うわべ #T35 上辺
うわまえ #T35 上前
うわまわ #R5*1 上回
うわむ #K5 上向
うわむき #T05 上向き
うわめ #T35 上目
うわやく #T35 上役
うん #T35*33 云 #T35*29 運 #T35*2 雲
うんえい #T30*15 運営
うんかい #T35 雲海
うんが #T35 運河
うんきゅう #T30 運休
うんこう #T30*1 運航 #T30 運行
うんさん #T30 雲散
うんざり #F04*1 うんざり
うんざん #T30 運算
うんしゅう #CN 温州
うんしん #T30 運針
うんせい #T35 運勢
うんぜん #CN 雲仙
うんそう #T30*1 運送
うんそうや #T35 運送屋
うんちく #T35*1 蘊蓄
うんちん #T35*10 運賃
うんてん #T30*22 運転
うんてんしゅ #T35*2 運転手
うんでい #T35*1 雲泥
うんと #F14*1 うんと
うんどう #T30*6 運動
うんどうかい #T35*6 運動会
うんどうじょう #T35 運動場
うんどんこん #T35 運鈍根
うんぬん #T30*9 云々
うんの #JNS 海野
うんぱん #T30*6 運搬
うんぴつ #T35 運筆
うんめい #T35*2 運命
うんゆ #T35*1 運輸
うんゆしょう #KK*1 運輸省
うんよう #T30*78 運用
うんりょう #T35 雲量
え #KS*255 得 #T35*66 絵 #KJ*12 江 #KS*10 選 #KS*2 え #KJ*2 衛 恵 #T35*1 画 #KJ*1 枝 #T35*1 柄 #KJ 荏 #T35 餌 #KJ 会 #KS 獲 #KJ 慧 重 #M5r 笑 #KJ 杖
えーじぇんと #T35*115 エージェント
えーす #T35*1 エース
えーてる #T35*2 エーテル
えあ #T35*14 エア
えあろぞる #KK*1 エアロゾル
えい #KJ*6 泳 #KJ*4 映 #T35*3 栄 #ZX*2 映 #KJ*1 叡 営 永 英 衛 詠 裔 #KJ 嬰 影 曳 洩 瑛 盈 穎 頴 #JCN 頴娃 #JN 英 #ZX 詠 #KJ 鋭
えいい #F14*2 鋭意
えいいち #JNM 栄一 英一 詠一
えいいちろう #JNM 栄一郎
えいえん #T05*4 永遠
えいかく #T35*1 鋭角
えいかん #T35 栄冠
えいが #T35*50 映画
えいがかん #T35*2 映画館
えいき #T35*1 鋭気
えいきち #JNM 栄吉
えいきゅう #T05*6 永久
えいきゅうし #T35 永久歯
えいきょう #T30*108 影響
えいぎょう #T30*31 営業
えいぎょうか #T35 営業課
えいぎょうちゅう #T35 営業中
えいぎょうぶ #T35*5 営業部
えいげんじ #CN 永源寺
えいこ #JNM 栄子 英子
えいこう #T30*1 曳航 #T35*1 栄光
えいこく #KK 英国
えいこせいすい #T30 栄枯盛衰
えいご #KK*64 英語
えいごう #T35*2 永劫
えいさい #T35 英才
えいさく #JNM 栄作
えいざぶろう #JNM 栄三郎
えいしゃ #T30 映写
えいしゃき #T35 映写機
えいしん #T30 栄進
えいじ #T35*9 英字 #T35 嬰児 #KS 映じ #JNM 栄次 栄治 栄二 永二 英司 英次 英治 英二
えいじゅう #T30 永住
えいじろう #JNM 栄次郎
えいすう #T35*19 英数
えいず #T35*2 エイズ
えいせい #T35*13 衛星 #T35*2 衛生 #T35 永世 #T30 永逝
えいせいし #T35*1 衛生士
えいぜん #T30 営繕
えいぞう #T35*3 映像 #T30 営造 #JNM 栄三 栄蔵 英三
えいぞく #T30*1 永続
えいたつ #T30 栄達
えいたろう #JNM 栄太郎
えいたん #T30*1 詠嘆
えいだん #T35*1 営団 #T30*1 英断
えいち #T35*4 英知
えいてん #T30*1 栄転
えいどう #T30*1 泳動
えいねん #T35 永年
えいびん #T05 鋭敏
えいふく #JCN 永福
えいふくちょう #CN 永福町
えいぶん #T35*23 英文
えいへいじ #CN 永平寺
えいみん #T30 永眠
えいめい #T05 英明
えいやく #T30*7 英訳
えいゆう #T35*2 英雄
えいよう #T35*1 栄養
えいようか #T35 栄養価
えいようし #T35 栄養士
えいようそ #T35 栄養素
えいり #T35*2 営利
えいりあす #T30 エイリアス
えいりあん #T35*2 エイリアン
えいりん #T30 営林 #T35 映倫
えいりんしょ #T35 営林署
えいわ #T35*2 英和
ええ #CJ*16 ええ #T35*16 ええ
えおじん #T35*1 エオジン
えが #K5*31 描
えがお #T35*3 笑顔
えがら #T35*1 絵柄
えがわ #JCN 江川
えき #CNSUC1*128 駅 #T35*17 液 #T30*6 益 #T30*2 役 #T35*1 易 #KJ*1 疫 #T35*1 駅 #KJ 亦
えきいん #T35 駅員
えきか #T30 液化
えきがく #T35*1 疫学
えきしゃ #T35*2 易者
えきしょう #T35*2 液晶
えきじょう #T35 液状
えきすぱーと #T35*4 エキスパート
えきせんとりっく #T05*1 エキセントリック
えきたい #T35*13 液体
えきちょう #T35 駅長
えきでん #T35 駅伝
えきびょう #T35 疫病
えきまえ #T35*10 駅前
えきまえどおり #T35 駅前通り
えきむ #T30 役務
えきめい #T35*1 駅名
えきや #CN 駅家
えきり #T35 疫痢
えくあどる #CN エクアドル
えくすてんしょん #T35*1 エクステンション
えくぼ #T35*1 エクボ
えぐち #JCN 江口
えこー #T30*16 エコー
えこーばっく #T30*2 エコーバック
えこだ #JCN 江古田
えこのみすと #T35*1 エコノミスト
えころじ #T35*2 エコロジ
えごいすと #T35 エゴイスト
えごいずむ #T35*1 エゴイズム
えさ #T35 餌
えさき #JNS 江崎
えさし #JCN 江差 江刺 #JNS 江指 #JCN 枝幸
えさん #JCN 恵山
えざわ #JNS 江沢
えし #T30*4 壊死
えしゃく #T30 会釈
えじき #T35*1 餌食
えじそん #JN エジソン
えじぷと #CN エジプト
えすかれーと #T30 エスカレート
えすけーぷ #T30*19 エスケープ
えすててぃっく #T15 エステティック
えすぺらんと #T35*1 エスペラント
えせ #PRE 似非
えそ #T30*1 壊疽
えぞえ #JNS 江副
えたい #T35*1 得体
えたじま #JNS 江田島
えだ #T35*7 枝
えだがり #T30 枝刈り
えだは #T35 枝葉
えだぶり #T35 枝振り
えだまめ #T35*1 枝豆
えだみち #T35 枝道
えち #CN 愛知
えちがわ #CN 愛知川
えちけっと #T35 エチケット
えちご #JCN 越後
えちぜん #JCN 越前
えっ #CJ*1 えっ
えっきょう #T30 越境
えっけん #T30 謁見 越権
えっしゃー #JN エッシャー
えっじ #T35 エッジ
えっせい #T35*1 エッセイ
えっせいすと #T35*1 エッセイスト
えっちゅう #CN 越中
えっとう #T30 越冬
えつ #KJ*10 閲 #KJ*4 越 #KJ*1 悦 #KJ 謁
えつお #JNM 悦夫
えつこ #JNM 悦子
えつたろう #JNM 悦太郎
えつねん #T30 越年
えつや #JNM 悦哉
えつよ #JNM 悦代
えつらく #T35 悦楽
えつらん #T30*1 閲覧
えて #T15*1 得手
えてして #F14 えてして 得てして
えでぃた #T35*120 エディタ
えでぃっと #T30*25 エディット
えでぃぷす #JN エディプス
えと #CJ*2 えと #T35 干支
えとう #JNS 江藤
えとく #T30*1 会得
えとろふ #CN 択捉
えど #JCN 江戸
えどがわ #JCN 江戸川
えどさき #JCN 江戸崎
えどま #T35 江戸間
えな #JCN 恵那
えなじー #T35*2 エナジー
えなり #JN 江成
えにし #T35 縁
えにわ #CN 恵庭
えぬま #JCN 江沼
えねるぎ #T35*1 エネルギ
えねるぎー #T35*16 エネルギー
えのき #T35*2 榎
えのきだ #JNS 榎田
えのぐ #T35*1 絵の具
えのしま #CN 江ノ島
えのもと #JNS 榎本
えはがき #T35*1 絵はがき
えはた #JNS 江畑
えはら #JNS 江原
えばら #JCN 江原
えひめ #CN 愛媛
えび #T35*3 蝦 #T35*1 鰕 #T35 海老
えびさわ #JNS 海老沢
えびす #T35*4 恵比寿 #T35*1 恵比須 #T35 夷 胡
えびちゃ #T35 葡萄茶
えびな #JCN 海老名
えびね #JNS 恵比根
えびの #CN えびの
えびはら #JNS 海老原
えぴそーど #T35*1 エピソード
えふだ #T35 絵札
えべつ #CN 江別
えほん #T35*2 絵本
えま #T35*1 絵馬
えまき #T35 絵巻
えみ #JNM 栄美 #JNS 江民
えみこ #JNM 恵美子
えみゅれーた #T35*13 エミュレータ
えみゅれーと #T30*2 エミュレート
えむかえ #JCN 江迎
えめらるど #T35*1 エメラルド
えもの #T35*1 獲物
えら #B5r*134 選 #KY*54 偉 #KY*6 えら #KJ*6 鰓 #T35*1 えら 鰓 #B5r 撰
えらー #T30*215 エラー
えらびだ #S5r*1 選び出
えらぶ #R5 偉ぶ
えり #T35*4 襟 #T35*2 衿 #JNM 恵利 恵理
えりーと #T35*1 エリート
えりあ #T35*24 エリア
えりくび #T35 襟首
えりこ #JNM 恵理子
えりっく #JN エリック
えりつぃん #JN エリツィン
えりぬ #K5r*1 えり抜
えりぬき #T18 選り抜き
えりもと #T35 襟元
えるさるばどる #CN エルサルバドル
えれがんと #T05*2 エレガント
えれくとりっく #T05*1 エレクトリック
えれくとろにくす #T35*2 エレクトロニクス
えれじー #T35 エレジー
えれべーた #T35*2 エレベータ
えろう #KYU*2 偉う #KYU*1 えろう
えん #JS*255 円 #T35*24 宴 #T35*22 縁 #KJ*6 遠 #ZX*5 演 #T35*4 苑 #KJ*2 延 #T35*1 ￥ 円 園 #KJ*1 炎 煙 鴛 #T35*1 塩 #KJ 宛 厭 #SUC 園 #KJ 堰 奄 怨 掩 援 沿 演 #SUC 炎 #KJ 焔 燕 猿 艶 薗 鉛 垣 鳶 媛 淵
えんえい #T30 遠泳
えんえき #T30*2 演繹
えんえん #F02*4 延々
えんか #T30*1 塩化 #T35 演歌
えんかい #T35*46 宴会 #T35 沿海 遠海
えんかく #T30*2 遠隔 #T35*1 沿革
えんかくち #T35*2 遠隔地
えんかつ #T05*3 円滑
えんかん #T35 円環
えんがる #CN 遠軽
えんがわ #T35 縁側
えんがん #T35*2 沿岸
えんき #T30*20 延期 #T35*1 塩基
えんきょり #T35*1 遠距離
えんきん #T35 遠近
えんぎ #T30*4 演技 #T35*2 縁起
えんけい #T35*1 円形 #T35 遠景
えんげ #T30*1 嚥下
えんげい #T35*1 演芸 #T35 園芸
えんげき #T35*3 演劇
えんこ #T35*8 円弧 #T35 縁故
えんこーだ #T35*2 エンコーダ
えんこーでぃんぐ #T35*2 エンコーディング
えんこーど #T30*6 エンコード
えんこん #T35 怨恨
えんご #T30 援護
えんさ #T35 怨嗟
えんさん #T35*1 塩酸
えんざん #T30*60 演算 #CN 塩山
えんざんし #T35*33 演算子
えんし #T35*1 遠視
えんしゅう #T30*18 演習 #T35 円周
えんしゅつ #T30*4 演出
えんしょう #T30 延焼 #T35 炎症
えんしんりょく #T35 遠心力
えんじ #T35*1 園児 #KS 演じ
えんじにあ #T35*4 エンジニア
えんじにありんぐ #T35*1 エンジニアリング
えんじゃ #T35*1 縁者
えんじゃく #T35 燕雀
えんじゅく #T30 円熟
えんじゅくみ #T35*1 円熟味
えんじょ #T30*6 援助
えんじょい #T30 エンジョイ
えんじょう #T30 炎上
えんじん #T35*28 エンジン #T35 円陣
えんすい #T35*1 円錐
えんすいだい #T35*2 円錐台
えんずい #T35 延随
えんせい #T30*1 遠征 #T30 厭世
えんせき #T35 宴席
えんせん #T35*2 沿線
えんぜつ #T30*1 演説
えんそう #T30*17 演奏
えんそく #T35*2 遠足
えんたー #T35*1 エンター
えんたーていめんと #T35*2 エンターテイメント
えんたーぷらいず #KK*1 エンタープライズ
えんたい #T30 延滞
えんたく #T35 円卓
えんだか #JS*2 円高 #T15*1 円高
えんだん #T35 演壇 縁談
えんちゃく #T30 延着
えんちゅう #T35*19 円柱
えんちょう #T30*5 延長 #T35 園長
えんちょく #T05*1 鉛直
えんちん #T30*1 遠沈
えんてぃてぃ #T35*4 エンティティ
えんてんか #T35*1 炎天下
えんでぃんぐ #T35*2 エンディング
えんとう #T35 円筒
えんとつ #T35 煙突
えんとり #T35*59 エントリ
えんとりー #T30*12 エントリー
えんとろぴ #T35*2 エントロピ
えんど #T35*29 エンド
えんどう #T35*3 沿道 #JNS 遠藤
えんどお #KY 縁遠
えんどゆーざ #T35*4 エンドユーザ
えんにち #T35*1 縁日
えんねつ #T35 炎熱
えんのした #T35 縁の下
えんはんすめんと #T35*1 エンハンスメント
えんばぐ #T30 エンバグ
えんばん #T35*2 円盤
えんぴつ #T35*2 鉛筆
えんぶん #T35*1 塩分 #T35 艶聞
えんべつ #CN 遠別
えんべろーぷ #T35*1 エンベロープ
えんぽう #T35*2 遠方
えんま #JN 閻魔
えんまく #T35*2 煙幕
えんまちょう #T35 閻魔帳
えんまん #T05*1 円満
えんむすび #T35 縁結び
えんめい #T30 延命
えんもく #T35*1 演目
えんやす #T15 円安
えんよう #T30*4 援用 #T35*1 遠洋
えんりょ #T30*18 遠慮
えんろ #T35 遠路
お #K5*255 お 置 #T35*255 尾 #S5*241 押 #W5*177 追 #PRE*114 お #R5*102 お #KJ*69 御 #R5r*20 折 #U5*18 負 #W5*18 負 #KJ*16 小 #R5*12 織 #S5*10 推 #T35*5 緒 #PRE*4 御 #KJ*4 男 雄 #KJ*2 汚 牡 #R5*2 居 #K5*1 於 #KJ*1 於 将 苧 #KJ 悪 #S5r 圧 #KJ 乎 #W5 逐 #KJ 麻
おーけすとら #T35*1 オーケストラ
おーすちん #JN オースチン
おーすとらりあ #CN オーストラリア
おーすとりあ #CN オーストリア
おーだ #T35*6 オーダ
おーだー #T30*10 オーダー
おーでぃお #T35*2 オーディオ
おーと #PRE*2 オート
おーとくちゅーる #T35 オートクチュール
おーとばい #T35 オートバイ
おーとまとん #T35*6 オートマトン
おーとみーる #T35 オートミール
おーとめーしょん #T35*1 オートメーション
おーなー #T35*9 オーナー
おーばー #T30*14 オーバー
おーばーふろー #T30*4 オーバーフロー
おーばーらいん #T35*1 ￣
おーばーらっぷ #T30*1 オーバーラップ
おーばーれい #T30*1 オーバーレイ
おーばふろー #T10 オーバフロー
おーばへっど #T35*18 オーバヘッド
おーぷん #T00*54 オープン
おーむ #JN オーム
おーる #T35*3 オール
おーろら #T35 オーロラ
おあい #OKX*255 お会い #OKX*11 お逢い
おあいて #T30*2 お相手 #T30 御相手
おあかし #OKX*3 お明かし
おあがり #OKX*69 お上がり #OKX*11 おあがり
おあき #OKX*199 お空き #OKX*14 お飽き
おあきらめ #OKX*19 おあきらめ #OKX*4 お諦め
おあけ #OKX*39 お開け #OKX*3 お空け
おあげ #OKX*149 おあげ #OKX*138 お上げ #OKX*57 お挙げ #OKX*6 お揚げ
おあしす #T35*4 オアシス
おあずかり #OKX*17 お預り #OKX お預かり
おあずけ #OKX*8 お預け
おあずけいれ #OKX*1 お預け入れ
おあせり #OKX*18 お焦り
おあそび #OKX*93 お遊び
おあたえ #OKX*162 お与え
おあたたまり #OKX*3 お温まり #OKX お暖まり
おあたため #OKX*1 お温め #OKX お暖め
おあたらしゅう #KYU*3 お新しゅう
おあつう #KYU*2 お暑う お熱う
おあつかい #OKX*194 お扱い
おあつまり #OKX*32 お集まり
おあつめ #OKX*39 お集め
おあて #OKX*75 お当て #OKX*3 お充て
おあてがい #OKX*1 おあてがい
おあばれ #OKX*4 お暴れ
おあび #OKX*1 お浴び
おあまえ #OKX*4 お甘え
おあまり #OKX*46 お余り
おあみ #OKX*32 お編み
おあやまり #OKX*101 お誤り #OKX*29 お謝り
おあらい #OKX*54 お洗い
おあらため #OKX*17 お改め
おあり #OKX*255 おあり #OKX*59 お有り #OKX*11 お在り
おあるき #OKX*34 お歩き
おあれ #OKX*3 お荒れ
おあわせ #OKX お併せ
おあわて #OKX*24 お慌て #OKX*1 おあわて
おあわれみ #OKX*1 お哀れみ #OKX お憐れみ
おい #CJ*15 おい #KSr*10 老い #KJ*1 笈 #T35 甥
おいい #OKX*3 お言い #OKX お云い
おいいだし #OKX*4 お言い出し
おいいつけ #OKX お言いつけ
おいえ #OKX*1 お癒え #JNS 尾家
おいおい #F05 おいおい
おいかけ #KS*2 追いかけ #KS*1 おいかけ
おいかし #OKX*12 お生かし #OKX*3 お活かし
おいかり #OKX*5 お怒り
おいかわ #JNS 及川 生川 追川
おいき #OKX*255 お行き #OKX*236 おいき #OKX*62 お生き #OKX*6 お往き #OKX*2 お活き
おいきかえり #OKX*1 お生き返り
おいきどおり #OKX お憤り
おいけ #OKX*31 おいけ #JCN 御池
おいこ #M5r*3 追い込 #S5r*2 追い越 #M5r*1 追いこ #S5r*1 追いこ #M5r 老い込
おいさめ #OKX お諌め
おいし #KY*10 おいし #KY*9 美味し
おいしげ #R5 生い茂
おいしゅう #KYU*2 おいしゅう #KYU*1 美味しゅう
おいそがしい #T35*24 お忙しい
おいそがしゅう #KYU*3 お忙しゅう
おいそぎ #OKX*42 お急ぎ
おいそれと #F14*1 おいそれと
おいたち #T35*1 生い立ち
おいたて #KS*1 追いたて #KS 追い立て
おいたみ #OKX*24 お痛み #OKX*1 お悼み #OKX お傷み
おいため #OKX*1 お痛め お炒め #OKX お傷め
おいたわり #OKX*3 おいたわり
おいだ #S5r 追い出
おいつ #K5*2 追い付
おいつくしみ #OKX お慈しみ
おいつめ #KSr*3 追い詰め
おいて #F14*19 おいて
おいてきぼり #T35*2 おいてきぼり
おいてけぼり #T35*1 おいてけぼり
おいで #T35*2 おいで
おいとい #OKX*10 お厭い
おいとなみ #OKX*1 お営み
おいどみ #OKX お挑み
おいぬ #K5r*1 追い抜
おいのり #OKX*10 お祈り #T30*6 お祈り #OKX お祷り
おいはら #W5*1 追い払
おいぼれ #KSr 老いぼれ
おいましめ #OKX お戒め
おいめ #T35 負い目
おいもとめ #KS*1 追い求め
おいやし #OKX お癒し
おいやしめ #OKX お卑しめ
おいらー #JN オイラー
おいらだち #OKX*2 お苛立ち
おいり #OKX*21 お要り #OKX お煎り
おいる #T35*3 オイル
おいれ #OKX*255 お入れ
おいれかえ #OKX*27 お入れ換え お入れ替え
おいわい #T30*12 お祝い #OKX*9 お祝い #T30 御祝い
おいわけ #CN 追分
おう #ZX*61 応 #SUC*13 王 #KJ*4 横 #KJ*3 欧 #KJ*2 応 #KJ*1 凹 奥 旺 桜 應 #KJ 姶 央 往 押 殴 #JNS 王 #T35 王 #KJ 翁 襖 鴬 鴎 黄 鴨 皇 邑
おうい #T35*1 王位
おういつ #T30*1 横溢
おういん #KK*6 桜蔭 #T30 押印
おうう #CN 奥羽
おうえ #OKX*42 お植え
おうえん #T30*10 応援
おうおう #F14*1 往々
おうおうにして #F14*1 往々にして
おうか #T30*2 押下 #T30 欧化
おうかがい #OKX*40 お伺い #OKX*1 おうかがい
おうかくまく #T35 横隔膜
おうかび #OKX*11 お浮かび #OKX*1 おうかび
おうかべ #OKX*1 お浮かべ
おうかん #T30 往還 #T35 王冠
おうがい #JNM 鴎外
おうき #OKX*17 お浮き
おうきゅう #T35*5 応急 #T35*2 王宮
おうぎ #T35*3 扇
おうけ #OKX*101 お受け #OKX*6 おうけ #OKX お承け お請け
おうけとめ #OKX お受け止め
おうけとり #OKX*77 お受け取り
おうけながし #OKX お受け流し
おうこう #T30 横行 #T35 王侯
おうこく #T35*1 王国
おうごかし #OKX*92 お動かし
おうごき #OKX*255 お動き
おうごきだし #OKX*4 お動き出し
おうごん #T35*1 黄金
おうさか #JCN 逢坂
おうさま #T35*2 王様
おうざ #T35 王座
おうし #T35 牡牛
おうしざ #KK*1 牡牛座
おうしつ #T35 王室
おうしない #OKX*24 お失い #OKX お喪い
おうしゅう #T30*13 応酬 #T30 押収 #CN 欧州
おうしょう #T35*1 王将
おうしょく #T35*1 黄色
おうしん #T35*1 往信 #T30*1 往診
おうじ #T35*1 王子 #T35 往時 #CN 王寺
おうじゃ #T35*1 王者
おうじょう #T30*1 往生
おうすれ #OKX*2 お薄れ
おうせい #T05 旺盛 #T35 王制
おうせつ #T30*1 応接
おうたい #OKX*32 お歌い #OKX*2 お唱い #T30*2 応対 #OKX*1 お唄い お詠い #T35*1 黄体 #OKX お謡い
おうたがい #OKX*8 お疑い
おうだく #T30 応諾
おうだん #T30*3 横断
おうち #OKX*176 お打ち #OKX*5 おうち #OKX*1 お撃ち お討ち #OKX お伐ち #JCN 相知 邑智
おうちあけ #OKX*3 お打ち明け
おうちけし #OKX*1 お打ち消し
おうちとけ #OKX*2 お打ち解け
おうちゃく #T00*1 横着
おうちょう #T35 王朝
おうつくしゅう #KYU*2 お美しゅう
おうつし #OKX*61 お移し #OKX*33 お写し #OKX*4 お映し #OKX*2 おうつし
おうつり #OKX*77 お移り #OKX*15 お写り #OKX*6 お映り #OKX*2 おうつり
おうて #T35 王手
おうてん #T30 横転
おうとう #T30*42 応答 #JNS 大塔
おうとつ #T35*1 凹凸
おうとみ #OKX*1 お疎み
おうどいろ #T15 黄土色
おうどう #T35*3 王道 #T35 黄銅
おうなずき #OKX*1 おうなずき
おうなつ #T30 押捺
おうねん #T35 往年
おうばい #OKX*2 お奪い
おうひ #T35 王妃
おうふう #T35 欧風
おうふく #T30*18 往復
おうぶん #T35*3 欧文
おうへい #T05 横柄
おうほう #T30 応報
おうぼ #T30*21 応募
おうぼう #T05*1 横暴
おうまれ #OKX*23 お生まれ #OKX お産まれ
おうまれかわり #OKX お生まれ変わり
おうみ #OKX*3 お生み #OKX*1 お産み #JCN 近江 青海
おうみはちまん #CN 近江八幡
おうむ #T35*1 王蟲 鸚鵡 #JCN 雄武
おうめ #OKX*66 お埋め #JCN 青梅
おうもんきん #T35*1 横紋筋
おうやまい #OKX*1 お敬い
おうよう #T30*73 応用 #T05 鷹揚
おうら #JCN 邑楽
おうらい #T30*1 往来
おうらない #OKX*2 お占い
おうらみ #OKX*1 お怨み お恨み
おうらやみ #OKX お羨み
おうり #OKX*127 お売り #OKX*5 おうり
おうりょう #T30 横領
おうろ #T35 往路
おえ #KS*11 終え #JCN 麻植
おえかき #T30*1 お絵書き
おえがき #OKX*31 お描き
おえつ #T30 嗚咽
おえみ #OKX お笑み
おえらび #OKX*134 お選び #OKX お撰び
おえらびだし #OKX*1 お選び出し
おお #KY*255 多 #PRE*80 大 #W5r*33 覆 #KJ*21 大 #W5r*16 被 #CJ*3 おお
おおあさ #JCN 大朝
おおあざ #CNPRE*3 大字
おおあみしらさと #CN 大網白里
おおあめ #T35*2 大雨
おおあらい #JCN 大洗
おおい #OKX*177 お追い #T07 大い #JCN 大井 大飯
おおいがわ #JCN 大井川
おおいし #T35*4 大石 #JCN 大石
おおいしだ #JCN 大石田
おおいずみ #JCN 大泉
おおいそ #JCN 大磯
おおいそぎ #T35 大急ぎ
おおいた #CN 大分
おおいなる #RT*1 大いなる
おおいに #F14*5 大いに
おおいまつだ #CN 大井松田
おおいり #T35 大入
おおうだ #JCN 大宇陀
おおうち #JCN 大内
おおうちやま #JCN 大内山
おおうら #JCN 大浦
おおうりだし #T35*1 大売出し
おおえ #JCN 大江
おおおか #JCN 大岡
おおおかやま #CN 大岡山
おおおとこ #T35 大男
おおかぜ #T35 大風
おおかた #F14*1 大方 #T36 大方
おおかみ #T35*2 狼
おおかわ #T35*2 大川 #JCN 大川
おおかわら #JNS 大河原
おおが #JNS 大賀
おおがかり #T05*1 大がかり 大掛かり
おおがき #JCN 大垣 大柿
おおがた #T05*33 大型 #T35*1 大形 #JCN 大潟 大方
おおがたき #T35*1 大型機
おおがわた #CN 大河原
おおがわら #JNS 大河原
おおき #OKX*255 おおき お置き #OKX*102 お起き #KY*102 大き #JNS 大木
おおきあがり #OKX*1 お起き上がり
おおきな #RT*88 大きな
おおきに #F14 大きに
おおきゅう #KYU*3 大きゅう
おおぎみ #JCN 大宜味
おおく #T35*89 多く
おおくち #JCN 大口 #T35 大口
おおくぼ #JCN 大久保
おおくま #JCN 大熊 #JNS 大隈
おおくもの #RT*20 多くもの
おおくら #T35*1 大蔵 #JNS 大倉 #JCN 大蔵
おおくらしょう #KK*1 大蔵省
おおくらやま #CN 大倉山
おおくり #OKX*255 お送り #OKX*4 お贈り
おおくりとどけ #OKX*2 お送り届け
おおくれ #OKX*121 お遅れ #OKX お後れ
おおくわ #JCN 大桑
おおぐち #JCN 大口 #T35 大口
おおげさ #T05*5 大げさ #T05 大袈裟
おおこうち #JCN 大河内
おおこし #OKX*185 お起こし #JNS 大越
おおご #JCN 大胡
おおごえ #T35*1 大声 #JCN 大越
おおさ #JCN 大佐
おおさえ #OKX*16 お押え #OKX*8 お抑え #OKX*4 お押さえ
おおさか #JCN 大阪
おおさかさやま #CN 大阪狭山
おおさかだいがく #KK*6 大阪大学
おおさかふ #CN 大阪府
おおさき #JCN 大崎
おおさと #JCN 大郷 大里
おおさめ #OKX*9 お納め #OKX*6 お収め #OKX*5 お治め #OKX お修め
おおさわ #JCN 大沢 #JNS 大澤
おおさわぎ #T10*7 大騒ぎ
おおさわの #JCN 大沢野
おおざっぱ #T05*2 大雑把 #T05*1 おおざっぱ
おおざと #JCN 大里
おおし #OKX*241 お押し #KY 雄々し
おおしえ #OKX*255 お教え
おおしか #JCN 大鹿
おおしはかり #OKX お推し量り
おおしま #T35*3 大島 #JCN 大島 #JNS 大嶋
おおしゅう #KYU 雄々しゅう
おおじ #CN 王寺 #JNS 大路
おおすか #JCN 大須賀
おおすが #JCN 大須賀
おおすじ #T35*2 大筋
おおすみ #JNS 大角 #JCN 大隅
おおず #JCN 大洲
おおせ #T35*1 仰せ #KS おおせ
おおせつか #R5 仰せつか
おおせと #JCN 大瀬戸
おおぜい #T35*7 大勢
おおぜき #JNS 大関 #T35 大関
おおそれ #OKX*18 お恐れ
おおそわり #OKX*2 お教わり
おおぞら #T35 大空
おおた #JCN 太田 大田
おおたき #JNS 王滝 #JCN 大多喜 大滝 #T35 大滝 #JNS 大瀧
おおたけ #JCN 大竹
おおたに #JCN 大谷
おおたま #JCN 大玉
おおたわら #JCN 大田原
おおだ #JCN 大田
おおだい #JNS 大台
おおだか #JCN 大高
おおだて #JCN 大館
おおち #OKX*88 お落ち #OKX*3 お堕ち #OKX お墜ち #JCN 邑智
おおつ #JCN 大津
おおつか #JCN 大塚
おおつき #JCN 大月 #JNS 大槻
おおつち #JCN 大槌
おおつぼ #JNS 大坪
おおづ #JCN 大津
おおづめ #T35*1 大詰め
おおて #T35*2 大手
おおで #T35*1 大手
おおとう #JCN 大塔 大任
おおとし #OKX*91 お落し #OKX*3 お落とし #OKX*1 おおとし
おおとね #JCN 大利根
おおとも #JNS 大友
おおとよ #JCN 大豊
おおとり #T35 鴻 鳳 鵬
おおとろえ #OKX お衰え
おおどおり #T35 大通り
おおどしま #T35 大年増
おおどり #OKX*3 お踊り #OKX*1 おおどり #OKX お躍り
おおどろき #OKX*61 お驚き
おおなえ #JNS 大苗
おおなみ #T35 大波
おおにし #JCN 大西
おおぬき #JCN 大貫
おおぬま #JCN 大沼
おおねじめ #JCN 大根占
おおの #JCN 大野
おおのじょう #CN 大野城
おおのはら #JCN 大野原
おおのみ #JCN 大野見
おおはさ #JNS 大迫
おおはし #T35*10 大橋 #JCN 大橋
おおはた #JCN 大畑
おおはば #T05*23 大幅
おおはら #JCN 大原
おおはる #JCN 大治
おおば #JNS 大場 大庭
おおばたけ #JCN 大畠
おおばやし #JNS 大林
おおばん #T35*1 大判
おおひと #JCN 大仁
おおひら #JCN 大衡 大平
おおびけ #T35 大引け
おおふな #JCN 大船
おおふなと #JCN 大船渡
おおぶ #JCN 大府
おおほ #JCN 大穂
おおぼえ #OKX*70 お覚え #OKX*7 お憶え #OKX*4 おおぼえ
おおぼれ #OKX*2 お溺れ
おおま #JCN 大間
おおまか #T05*14 大まか
おおまがり #CN 大曲 #T35 大曲
おおまき #JN 大蒔
おおまた #T05 大股
おおまち #JCN 大町
おおまちがい #T10 大間違い
おおまま #JCN 大間々
おおみしま #CN 大三島
おおみず #T35 大水
おおみそか #T35 大晦日
おおみや #JCN 大宮
おおむかし #T35*1 大昔
おおむた #JCN 大牟田
おおむね #F14*8 おおむね #F14*3 概ね
おおむら #JCN 大村
おおめ #T35*2 大目
おおもい #OKX*255 お思い #OKX*3 お想い #OKX*2 おおもい
おおもいだし #OKX*18 お思い出し
おおもいつき #OKX*8 お思い付き #OKX*6 お思いつき
おおもいなおし #OKX*1 お思い直し
おおもじ #T35*23 大文字
おおもと #T35 大本
おおもの #T35*3 大物
おおもり #T10*1 大盛り #JCN 大森
おおや #T35*1 大家 #JCN 大屋 大谷 #JNS 大矢
おおやけ #T35*4 公
おおやすうり #T10 大安売り
おおやち #JNS 大矢知
おおやの #JCN 大矢野
おおやま #T35*6 大山 #JCN 大山
おおやまざき #JCN 大山崎
おおやまだ #JCN 大山田
おおゆき #T35 大雪
おおよそ #T35*2 おおよそ
おおよど #JCN 大淀
おおら #JN 邑楽
おおらか #T05*1 おおらか
おおり #OKX*102 おおり #OKX*27 お降り #OKX*20 お折り #OKX*12 お織り #OKX*7 お下り
おおれ #OKX*1 お折れ
おおろし #OKX*7 お降ろし #OKX*3 お下ろし #OKX お卸し
おおわだ #JCN 大和田
おおわに #JCN 大鰐
おおわらい #T10*3 大笑い
おおわらわ #T15 大童
おか #S5*34 侵 #T35*15 岡 #T35*12 丘 #S5*2 犯 冒 #JCN 岡 #KJ 峻 阜 陸
おかあさま #T35*2 お母様
おかあさん #T35*29 お母さん #T35 おかあさん
おかい #OKX*255 お買い #OKX*1 お飼い
おかいあげ #OKX*1 お買い上げ
おかいいれ #OKX お買い入れ
おかいかえ #OKX*3 お買い替え
おかいとり #OKX*1 お買い取り
おかえ #OKX*255 お変え #OKX*74 お替え #OKX*38 お換え #OKX*3 おかえ #OKX*2 お代え
おかえし #OKX*216 お帰し #T30*4 お返し
おかえり #OKX*255 お帰り お返り
おかかえ #OKX*18 お抱え
おかかげ #OKX*2 お掲げ
おかかり #OKX*113 おかかり #OKX*42 お掛かり
おかかわり #OKX*22 おかかわり #OKX*4 お係わり #OKX*2 お関わり お拘り
おかがき #JCN 岡垣
おかがみ #OKX お屈み
おかき #OKX*255 お書き #OKX*38 お描き #OKX*21 お欠き #OKX お掻き
おかきいれ #OKX*1 お書き入れ
おかきおとし #OKX お書き落とし
おかきかえ #OKX*49 お書き換え #OKX*30 お書換え
おかきこみ #OKX*161 お書き込み
おかきそえ #OKX お書き添え
おかきだし #OKX*14 お書き出し
おかきとめ #OKX*1 お書き留め
おかきぬき #OKX お書き抜き
おかきまぜ #OKX おかき混ぜ
おかぎり #OKX*130 お限り #OKX*1 おかぎり
おかくし #OKX*37 お隠し
おかくれ #OKX*5 お隠れ #OKX*3 おかくれ
おかけ #OKX*49 お掛け #OKX*47 おかけ #OKX*2 お賭け お翔け #OKX*1 お架け お駆け お懸け #OKX お駈け
おかけつけ #OKX*2 お駆けつけ
おかけより #OKX お駆け寄り
おかげ #T35*16 おかげ #T35*2 お蔭 #T35 御蔭
おかげさま #T35*11 お蔭様 #T35 御蔭様
おかげん #T35 お加減
おかこい #OKX*45 お囲い
おかこみ #OKX*48 お囲み
おかさね #OKX*13 お重ね
おかざき #JCN 岡崎
おかざり #OKX*17 お飾り
おかし #OKX*237 お貸し #KYmi*125 おかし #KYmi*4 可笑し
おかしこし #OKX お貸し越し
おかしだし #OKX*26 お貸し出し
おかしつけ #OKX お貸し付け
おかしな #RT*6 おかしな
おかしゅう #KYU*3 おかしゅう #KYU*1 可笑しゅう
おかじま #JNS 岡島 岡嶋
おかじょうき #T35 陸蒸気
おかじり #OKX*3 おかじり
おかず #T35 おかず
おかせぎ #OKX*5 お稼ぎ
おかぞえ #OKX*10 お数え
おかたづけ #OKX*1 お片付け
おかたむき #OKX*3 お傾き
おかため #OKX*2 おかため お固め
おかたより #OKX*2 お偏り
おかたり #OKX*4 お語り
おかだ #JCN 岡田
おかち #OKX*62 お勝ち
おかちまち #CN 御徒町
おかちめんこ #T35*1 オカチメンコ
おかって #T35 お勝手
おかっぴき #T35 岡っ引き
おかつぎ #OKX お担ぎ
おかどちがい #T05*3 お門違い
おかなしみ #OKX*27 お悲しみ #OKX*2 お哀しみ
おかなで #OKX お奏で
おかね #T35*55 お金 #OKX*24 お兼ね
おかねそなえ #OKX*2 お兼ね備え
おかの #JNS 岡野
おかはる #JCN 岡原
おかばい #OKX*1 お庇い
おかぶせ #OKX*2 おかぶせ
おかぶり #OKX*6 お被り
おかべ #JCN 岡部
おかぼ #T35 陸稲
おかまい #OKX*58 お構い #OKX*20 おかまい
おかまえ #OKX お構え
おかみ #OKX*11 お噛み
おかむら #JCN 岡村
おかもち #T35 岡持ち
おかもと #JCN 岡本
おかや #JCN 岡谷
おかやす #JNS 岡安
おかやま #JCN 岡山
おかよい #OKX*10 お通い
おかり #OKX*68 お借り #OKX*25 お苅り #OKX*1 お駆り #OKX お穫り #R5 お刈り #R5r お狩り
おかりあげ #OKX お借り上げ
おかりいれ #OKX お借り入れ
おかりうけ #OKX*1 お借り受け
おかりきり #OKX お借り切り
おかりこし #OKX お借り越し
おかりな #T35*1 オカリナ
おかると #T05 オカルト
おかれ #OKX*2 お枯れ
おかわかし #OKX*1 お乾かし
おかわき #OKX*1 お乾き #OKX お渇き
おかわし #OKX*7 お交わし #OKX*1 おかわし
おかわり #T30*5 お変わり #OKX*4 お代わり #OKX*3 おかわり #OKX*2 お換わり お替わり #T30*1 お代り お変り #OKX お替り
おかん #T35 悪寒
おかんがえ #OKX*255 お考え
おかんがえつき #OKX*2 お考え付き
おかんじ #OKX*4 お感じ
おかんじとり #OKX お感じとり
おが #M5*1 拝 #JCN 男鹿
おがさ #JCN 小笠
おがさわら #JCN 小笠原
おがた #JCN 緒方 #JNS 小方 #JN 尾形
おがち #JCN 雄勝
おがつ #JCN 雄勝
おがの #JCN 小鹿野
おがわ #T35*16 小川 #JCN 緒川 #JN 小河 #JCN 小川
おがわな #JNS 小川名
おき #KS*102 起き #T35*29 沖 #JCN 隠岐
おきあい #T35 沖合い
おきあが #R5*1 起き上が
おきかえ #KSr*49 置き換え #T30*1 置換え
おきかせ #OKX*121 お聞かせ
おきかわ #R5*3 置き換わ
おきがえ #OKX*3 お着替え
おきき #OKX*255 お聞き #OKX*175 お効き #OKX*43 おきき #OKX*13 お聴き #OKX*2 お利き
おききいり #OKX*1 お聞き入り
おききいれ #OKX*1 お聞き入れ
おききかえし #OKX お聞き返し
おききぐるしゅう #KYU*1 お聞き苦しゅう
おききとり #OKX*2 お聞き取り
おききながし #OKX*4 お聞き流し
おききわけ #OKX お聞き分け
おきこえ #OKX*20 お聞こえ #OKX*1 おきこえ #OKX お聴こえ
おきざみ #OKX*1 お刻み #OKX おきざみ
おきざり #T30 置き去り
おきしとしん #T35*1 オキシトシン
おきずき #OKX*11 お築き
おきせ #OKX お着せ
おきそい #OKX*2 お競い
おきた #JNS 沖田
おきたえ #OKX*6 お鍛え
おきたま #JCN 置賜
おきづき #OKX*29 お気づき お気付き
おきて #T35 掟
おきな #T35*1 翁
おきなわ #CN 沖縄
おきにいり #T15 お気に入り
おきば #T35 置場
おきまり #OKX*100 お決まり #OKX*3 おきまり #OKX*1 お決り
おきみ #JCN 沖美
おきめ #OKX*131 お決め
おきもち #T35*1 お気持
おきもの #T35 置物
おきゃく #T35*5 お客
おきゃくさま #T35*4 お客様
おきよめ #OKX*1 お清め
おきらい #OKX*3 お嫌い
おきり #OKX*152 お切り #OKX*6 おきり #OKX*1 お伐り #OKX お斬り
おきりかえ #OKX*69 お切替え #OKX*19 お切り替え
おきわすれ #KSr*1 置き忘れ
おきわめ #OKX*2 お究め #OKX*1 お極め
おぎ #T35*4 荻 #JN 荻 #JCN 小城 小木
おぎくぼ #CN 荻窪
おぎた #JNS 荻田
おぎな #W5*6 補
おぎの #JNS 荻野
おぎのめ #JNS 荻野目
おぎはら #JNS 荻原
おぎわら #JNS 荻原
おく #R5r*255 送 #T35*74 奥 #NN*13 億 #R5*4 贈 #S5*1 臆 #T30*1 臆 #KJ 屋 憶 #JCN 邑久
おくい #OKX お悔い #JNS 奥井 奥居
おくいとめ #OKX*1 お食い止め
おくがい #T35*1 屋外
おくぎ #T35*1 奥義
おくぐり #OKX*3 お潜り
おくさま #T35*5 奥様
おくさわ #JCN 奥沢
おくしが #CN 奥志賀
おくしり #JCN 奥尻
おくじょう #T35*1 屋上
おくずし #OKX*4 お崩し
おくそく #T30*3 憶測
おくそこ #T35 奥底
おくたま #CN 奥多摩
おくだ #JNS 奥田
おくだき #OKX お砕き
おくだし #OKX*6 お下し
おくだり #OKX*6 お下り #OKX お降り
おくち #T35 奥地
おくつ #JCN 奥津
おくつがえし #OKX*2 お覆し
おくつろぎ #OKX*1 お寛ぎ
おくてっと #T35*6 オクテット
おくない #T35*1 屋内
おくにがら #T35 お国柄
おくの #JN 奥乃 #JNS 奥野
おくば #T35*1 奥歯
おくばり #OKX*25 お配り
おくび #T35 おくび
おくびょう #T05*1 臆病
おくみ #OKX*7 お組み #OKX*5 お汲み #OKX お酌み
おくみたて #OKX*8 お組み立て
おくみとり #OKX*1 お汲み取り #OKX おくみ取り
おくむら #JNS 奥村
おくめん #T35 臆面
おくもらし #OKX お曇らし
おくもり #OKX*9 お曇り
おくやま #T35*1 奥山 #JCN 奥山
おくやみ #T30*2 お悔やみ
おくゆかし #KY 奥ゆかし
おくゆかしゅう #KYU 奥ゆかしゅう
おくゆき #T35*3 奥行き
おくらし #OKX*12 お暮らし
おくらべ #OKX*55 お比べ #OKX*17 お較べ
おくりあげ #OKX お繰り上げ
おくりいれ #OKX お繰り入れ
おくりかえし #OKX*80 お繰り返し
おくりこ #M5r*8 送り込
おくりさき #T35*7 送り先
おくりじょう #T35*1 送り状
おくりだ #S5r*11 送り出
おくりとどけ #KSr*2 送り届け
おくりぬし #T35 送り主
おくりむかえ #T30 送り迎え
おくりもの #T30*2 贈物
おくるい #OKX*6 お狂い
おくるしみ #OKX*11 お苦しみ
おくるしめ #OKX*1 お苦しめ
おくるまり #OKX*2 おくるまり
おくれ #KSr*121 遅れ #KSr 後れ
おくればせながら #F14*5 遅ればせながら #T36 遅ればせながら
おくわえ #OKX*122 お加え
おくわだて #OKX お企て
おくわわり #OKX*14 お加わり
おぐち #JNS 小口
おぐに #JCN 小国
おぐら #T35*2 小倉 #JNS 小倉 小椋
おぐり #JCN 小栗
おぐるしみ #OKX*1 お苦しみ
おぐろ #JNS 小黒
おけ #T35*29 桶
おけいこ #T30*1 おけいこ #T30 お稽古
おけがし #OKX*1 お汚し
おけがれ #OKX お汚れ
おけがわ #JCN 桶川
おけし #OKX*112 お消し
おけずり #OKX*9 お削り
おけと #JCN 置戸
おけなし #OKX*1 おけなし
おけり #OKX*6 お蹴り
おこ #R5*227 起こ #S5r*185 起こ #R5*137 怒 #R5*2 興 #S5*2 興
おこえ #OKX*99 お越え #OKX*10 お超え #OKX*1 お肥え
おこがし #OKX*1 お焦がし
おこがまし #KY*3 おこがまし
おこがましゅう #KYU*1 おこがましゅう
おこぎ #OKX*1 お漕ぎ
おこけ #OKX*13 おこけ
おこげ #OKX*1 お焦げ
おこころえ #OKX*1 お心得
おこころがけ #OKX*4 お心がけ
おこころざし #OKX*1 お志し
おこころみ #OKX*34 お試み
おこさま #T35*2 お子様
おこさん #T35*1 お子さん
おこし #OKX*94 お越し #OKX*1 お超し #OKX お漉し
おこた #R5r*1 怠
おこたえ #T30*23 お答え #OKX*2 お応え
おこっぺ #JCN 興部
おことづけ #OKX*1 お言付け
おことわり #T30*5 お断り #T30 お断わり
おこな #W5r*255 行 #W5r*216 行な
おこなし #OKX*5 おこなし
おこのぎ #JNS 小此木
おこのみ #OKX*23 お好み
おこばみ #OKX*2 お拒み
おこぼし #OKX*6 おこぼし
おこぼれ #OKX*3 おこぼれ
おこまり #OKX*123 お困り
おこまりはて #OKX お困り果て
おこめ #OKX お込め
おこらえ #OKX*1 お堪え
おこらしめ #OKX お懲らしめ
おこり #OKX*4 お懲り
おころがし #OKX*1 お転がし
おころし #OKX*24 お殺し
おころび #OKX*4 お転び
おこわし #OKX*9 お壊し
おご #R5r*2 おご
おごおり #JCN 小郡
おごせ #JCN 越生
おごそか #T05 厳か
おさ #KJ 長
おさえ #KSr*16 押え #OKX*11 お冴え #KS*8 抑え #KSr*4 押さえ
おさえぎり #OKX お遮り
おさえこ #M5r*1 押え込 #M5r 抑え込
おさか #JCN 小坂
おさかえ #OKX*1 お栄え
おさかし #OKX*29 お咲かし
おさからい #OKX*2 お逆らい
おさがし #OKX*91 お探し #OKX*6 お捜し
おさがり #OKX*9 お下がり
おさき #OKX*16 お割き #OKX*1 お裂き
おさきに #F14*9 お先に
おさぐり #OKX*3 お探り
おさけ #OKX*116 お避け
おさけび #OKX*7 お叫び
おさげ #OKX*13 お下げ #T35*1 おさげ #OKX*1 お提げ
おささえ #OKX*4 お支え
おささげ #OKX*1 お捧げ
おささやき #OKX*2 おささやき #OKX*1 お囁き お囁やき
おさし #OKX*103 お指し #OKX*6 お差し #OKX*4 おさし #OKX*2 お刺し お挿し
おさしこみ #OKX*1 お差し込み
おさししめし #OKX*1 お指し示し
おさしだし #OKX お差し出し
おさしつかえ #OKX*3 お差し支え
おさすり #OKX*1 おさすり
おさずかり #OKX*1 お授かり
おさずけ #OKX お授け
おさそい #T30*18 お誘い
おさそいあわせ #OKX お誘い合わせ
おさだ #JNS 長田
おさだまり #OKX*6 お定まり
おさだめ #OKX*37 お定め
おさっし #OKX*6 お察し
おさとし #OKX*1 お諭し
おさとり #OKX*19 お悟り
おさな #KY*2 幼
おさなご #T35 幼子
おさのう #KYU*1 幼う
おさばき #OKX*1 お裁き #OKX お捌き
おさふね #JCN 長船
おさま #R5r*7 収ま #R5r*2 納ま #R5r 治ま 修ま
おさまし #OKX*3 お覚まし
おさまたげ #OKX*4 お妨げ
おさむ #JNM 治 治虫 修
おさむう #KYU*2 お寒う
おさめ #KS*9 納め #KS*6 収め #OKX*5 お醒め #KS*5 治め #OKX*2 お覚め #KS 修め
おさらい #OKX*1 お拐い
おさらし #OKX お曝し
おさり #OKX*104 お去り
おさわがせ #OKX お騒がせ
おさわぎ #OKX*25 お騒ぎ
おさわり #OKX*20 お触り
おざき #JCN 尾崎
おざく #JCN 小作
おざわ #JNS 小沢 小澤
おし #KYmi*4 惜し #M5*3 惜し #KJ 唖 鴛 #T35 押し
おしあげ #OKX*4 お仕上げ #KS*1 押し上げ
おしあわせ #T03 お幸せ
おしい #R5r 押し入 #JNS 押井
おしいれ #OKX*7 お仕入れ #T35*1 押入
おしうり #T30 押売
おしえ #KSr*255 教え
おしか #JCN 牡鹿
おしかえ #S5 押し返
おしかけ #OKX*13 お仕掛け #KS*1 押しかけ
おしかり #OKX*19 お叱り #OKX*1 おしかり
おしかりつけ #OKX お叱りつけ
おしき #OKX*37 おしき #OKX*7 お敷き #R5r*1 押し切
おしきり #OKX*6 お仕切り
おしくも #F14 惜しくも
おしこ #M5r*5 押し込
おしこみ #OKX*2 お仕込み
おしこめ #KS 押し込め
おしごき #OKX*1 おしごき
おしごと #T35*18 お仕事
おしずまり #OKX お静まり #R5r お鎮まり
おしずみ #OKX*1 お沈み
おしずめ #OKX*1 お鎮め #OKX お静め お沈め
おしたい #OKX*1 お慕い
おしたがい #OKX*154 お従い #OKX*4 おしたがい
おしたしみ #OKX*5 お親しみ
おしたため #OKX*1 お認め
おしたて #OKX*2 お仕立て
おしだ #S5r*2 押し出
おしつけ #KSr*4 押しつけ #KSr*1 押し付け #KSr おしつけ
おしつま #R5*1 押し詰ま
おしとお #S5 押し通
おしどり #T35*1 鴛鴦
おしに #OKX*95 お死に
おしの #JCN 忍野
おしのぎ #OKX*5 おしのぎ #OKX*1 お凌ぎ
おしのけ #KS 押し退け
おしのび #OKX*4 お忍び #OKX*1 お偲び
おしはか #R5 推し量
おしはらい #OKX*3 お支払い
おしばり #OKX*4 お縛り
おしびれ #OKX*2 お痺れ
おしぶり #OKX*19 お渋り
おしぼり #OKX*5 おしぼり お絞り #OKX お搾り
おしま #JCN 渡島
おしまい #OKX*255 おしまい #OKX*1 お仕舞い
おしみず #JCN 押水
おしみな #KY 惜しみな
おしみのう #KYU 惜しみのう
おしむけ #OKX お仕向け
おしむらくは #CJ*1 惜しむらくは
おしめ #OKX*72 お占め #OKX*7 お閉め #OKX*3 お締め #OKX お絞め
おしめし #OKX*255 お示し
おしゃべり #T30*3 おしゃべり
おしゃまんべ #CN 長万部
おしゃれ #T00*6 おしゃれ #T05*6 お洒落
おしゅう #KYU*1 惜しゅう
おしょう #T35 和尚
おしょくじ #T30*2 お食事
おしよせ #KS*5 押し寄せ
おしらせ #T30*121 お知らせ #T30 おしらせ
おしらべ #OKX*245 お調べ
おしり #OKX*255 お知り #OKX*70 おしり
おしりあい #OKX*1 お知り合い
おしりぞき #OKX*1 お退き
おしりぞけ #OKX*1 お退け
おしるし #OKX*10 お記し
おしろい #T35 白粉
おしろすこーぷ #T35*1 オシロスコープ
おじ #T35*32 叔父 #T35 小父 伯父
おじいさん #T35*1 おじいさん #T35 お爺さん
おじか #JNS 牡鹿 #JCN 小値賀
おじぎ #T30*1 おじぎ
おじま #JNS 尾島
おじょうさま #T35*1 お嬢様
おじょうさん #T35*3 お嬢さん
おじょく #T35 汚辱
おす #T35*2 ♂ #T35*1 雄 #T35 牡
おすい #OKX*40 お吸い #T35 汚水
おすえ #OKX*1 お据え
おすかし #OKX*1 お透かし
おすき #OKX*67 おすき #OKX*7 お好き #OKX*4 お空き
おすぎ #OKX*133 お過ぎ #OKX*8 おすぎ
おすくい #OKX*4 お救い
おすぐれ #OKX*17 お優れ #OKX*1 おすぐれ
おすごし #OKX*19 お過ごし #T35*5 お過ごし
おすすぎ #OKX*1 お濯ぎ
おすすみ #OKX*147 お進み
おすすめ #OKX*10 お勧め お進め #OKX*5 おすすめ お奨め #OKX*3 お推め お薦め
おすずみ #OKX*1 お涼み
おすて #OKX*55 お捨て
おすてさり #OKX お捨て去り
おすばん #T35*1 オスバン
おすべり #OKX*18 お滑べり
おすまい #T35 お住い
おすまし #OKX*1 お済まし #OKX お澄まし
おすみ #OKX*255 お済み #OKX*57 お住み #OKX*45 おすみ #OKX お棲み
おすり #OKX*77 お擦り #OKX*72 お刷り #OKX*4 お摺り
おすわり #OKX*15 お座り #OKX お据わり
おせあにあ #CN オセアニア
おせいぼ #T35 御歳暮
おせち #T35 おせち
おせっかい #T05*3 お節介
おせっし #OKX*2 お接し
おせばめ #OKX お狭め
おせまり #OKX*9 お迫り
おせめ #OKX*9 お攻め #OKX*4 お責め
おせわ #T30*255 お世話
おせん #T30*3 汚染
おせんじ #OKX*1 お煎じ
おぜ #JCN 尾瀬
おそ #KY*201 遅 #W5*2 襲
おそう #KYU*3 遅う
おそえ #OKX*11 おそえ #OKX*3 お添え
おそくとも #F14*11 遅くとも
おそこない #OKX*3 お損ない
おそこね #OKX*6 お損ね
おそざき #T35 遅咲き
おそし #T35 遅し
おそそぎ #OKX*3 お注ぎ #OKX*1 おそそぎ
おそだち #OKX*3 お育ち
おそだて #OKX*11 お育て
おそなえ #OKX*10 お備え #OKX*1 お供え
おそなわり #OKX*2 お備わり
おそまり #OKX*1 お染まり
おそむき #OKX*1 お背き
おそめ #OKX*3 お染め
おそらく #F14*41 おそらく #F14*9 恐らく
おそらくは #F14 おそらくは 恐らくは
おそらし #OKX*1 お逸し #OKX お逸らし #S5 お反らし
おそり #OKX*8 お剃り #OKX お反り
おそれ #KSr*18 恐れ #OKX*3 お逸れ #T35*1 虞
おそれい #R5*11 恐れ入
おそれながら #CJ 恐れながら
おそろい #OKX*25 お揃い
おそろいで #F14 お揃いで
おそろえ #OKX*1 おそろえ お揃え
おそろし #KY*9 恐ろし
おそろしゅう #KYU*1 恐ろしゅう
おそわ #R5*2 教わ
おそん #T30 汚損
おぞまし #KY おぞまし 鈍まし 悍まし
おぞましゅう #KYU おぞましゅう 鈍ましゅう 悍ましゅう
おぞん #T35*1 オゾン
おた #JNS 織田
おたえ #OKX*12 お耐え #OKX*7 お絶え #OKX*3 お堪え
おたおし #OKX*15 お倒し
おたおれ #OKX*3 お倒れ
おたかめ #OKX*7 お高め
おたがい #T35*20 お互い #T35 おたがい
おたがいに #F14*3 お互いに
おたがやし #OKX お耕し
おたき #OKX*4 お炊き #OKX*1 お焚き
おたくらみ #OKX*1 お企み
おたくわえ #OKX*6 お蓄え #OKX*2 お貯え
おたぐり #OKX*10 お手繰り
おたけ #OKX*6 お長け
おたけび #T35 雄叫び
おたこう #KYU*3 お高う
おたし #OKX*39 お足し
おたしかめ #OKX*15 お確かめ
おたじま #JNS 小田島 小田嶋
おたすけ #OKX*24 お助け
おたすけあい #OKX*1 お助け合い
おたすけだし #OKX お助け出し
おたずさえ #OKX お携え
おたずさわり #OKX*3 お携わり #OKX*2 おたずさわり
おたずね #OKX*20 お尋ね #T30*4 お尋ね #OKX*2 お訪ね #OKX*1 おたずね
おたたえ #OKX*1 お讃え #OKX お称え
おたたかい #OKX*4 お戦い #OKX*1 お闘い
おたたき #OKX*27 お叩き #OKX*2 おたたき
おたたみ #OKX*2 お畳み
おただし #OKX*2 お正し #OKX お質し
おたち #OKX*145 お立ち #OKX*55 おたち #OKX*5 お発ち #OKX*3 お絶ち #OKX*1 お断ち #OKX お起ち #T5 お裁ち
おたちあい #OKX*3 お立ち会い
おたちあがり #OKX*37 お立ち上がり
おたちいり #OKX*2 お立ち入り
おたちさり #OKX お立ち去り
おたちどまり #OKX*1 お立ち止まり
おたちなおり #OKX*3 お立ち直り
おたちのき #OKX お立ち退き
おたっし #T30*2 お達し
おたて #OKX*21 お建て #OKX*6 お立て #OKX*1 おたて #OKX お点て
おたとえ #OKX*8 お例え #OKX*1 おたとえ
おたどり #OKX*12 おたどり #OKX*2 お辿り
おたのしみ #OKX*116 お楽しみ
おたのみ #OKX*67 お頼み
おたばね #OKX お束ね
おたびだち #OKX*4 お旅立ち
おたべ #OKX*119 お食べ #JNS 小田部
おたまい #OKX お給い
おたまり #OKX*33 お溜り #OKX*9 おたまり
おため #OKX*8 お貯め #OKX お矯め お溜め
おためし #OKX*76 お試し
おたもち #OKX*21 お保ち
おたやし #OKX お絶やし
おたより #OKX*24 お頼り #T30*2 お便り
おたり #OKX*51 お足り #JCN 小谷
おたる #CN 小樽
おたれ #OKX*3 お垂れ
おたわむれ #OKX お戯れ
おだ #JCN 小田 織田
おだか #JCN 小高
おだき #OKX*9 お抱き
おだきゅう #KK*2 小田急
おだぎり #JNS 小田桐
おだく #T35 汚濁
おだし #OKX*255 お出し #OKX*9 おだし
おだじま #JNS 小田島 小田嶋
おだち #OKX*3 お立ち
おだて #KSr*4 煽て
おだまし #OKX*5 おだまし
おだまり #OKX*4 お黙り
おだやか #T05*4 穏やか
おだわら #JCN 小田原
おち #KSr*88 落ち #KS*3 堕ち #JCN 越知 越智 #KS 墜ち
おちあ #W5*2 落ち合
おちあい #JCN 落合
おちい #R5*17 陥
おちかい #OKX*10 お誓い
おちかづき #OKX*31 お近付き
おちかより #OKX*1 お近寄り
おちがい #OKX*255 お違い
おちがえ #OKX お違え
おちぎり #OKX*1 お契り
おちこ #M5r*2 落ち込
おちつ #K5r*14 落ち着 #K5r おちつ
おちど #T35*1 落度
おちば #T35 落葉
おちぶれ #KS 落ちぶれ
おちゃ #T35*17 お茶
おちゃのみず #CN お茶の水 御茶ノ水
おちゃらけ #KSr おちゃらけ
おちゅうげん #T35*1 御中元
おちらかし #OKX*2 お散らかし
おちらし #OKX*2 お散らし
おちり #OKX*10 お散り
おぢや #JCN 小千谷
おっ #CJ*7 おっ
おっかけ #KSr 追っかけ
おっかな #KY おっかな
おっくう #T05 おっくう 億劫
おっこち #KS*2 落っこち #KS*1 おっこち
おっしゃ #L5*19 おっしゃ #L5*2 仰
おっつけ #CJ おっつけ
おって #T35*1 追っ手 追手 #F14 追って
おっと #T35*27 夫
おつ #T05*8 乙
おついやし #OKX*9 お費し #OKX*2 お費やし
おつうじ #OKX*28 お通じ
おつかい #OKX*255 お使い #OKX*1 お遣い #T35 お使い
おつかいわけ #OKX*3 お使い分け
おつかえ #OKX*24 お仕え
おつかまえ #OKX*12 お捕まえ
おつかまり #OKX*3 お捕まり
おつかみ #OKX*8 お掴み
おつかり #OKX*2 お浸かり
おつかれ #OKX*85 お疲れ
おつき #OKX*255 お付き #OKX*155 おつき #OKX*48 お着き #OKX*3 お尽き #OKX*2 お就き #OKX*1 お突き #OKX お衝き #K5 お附き
おつきあい #OKX*38 おつき合い #OKX*8 おつきあい
おつきそい #OKX お付き添い
おつぎ #OKX*48 お次ぎ #OKX*4 お継ぎ #OKX*1 お嗣ぎ #G5r お接ぎ #OKX お注ぎ
おつくし #OKX*7 お尽くし
おつくり #OKX*255 お作り #OKX*3 おつくり #OKX*2 お創り お造り
おつくりあげ #OKX*3 お作り上げ #OKX お造り上げ
おつくりだし #OKX*23 おつくり出し #OKX*5 お作り出し #OKX お創り出し
おつくろい #OKX お繕い
おつけ #OKX*57 お付け #OKX*4 おつけ #OKX*3 お漬け
おつけくわえ #OKX*22 お付け加え
おつけたし #OKX*4 お付け足し
おつげ #T30*6 お告げ
おつたえ #T30*26 お伝え
おつたわり #OKX*14 お伝わり
おつちかい #OKX*3 お培い
おつつしみ #OKX おつつしみ #M5r お謹み お慎み
おつつみ #OKX*3 お包み
おつづき #OKX*123 お続き
おつづけ #OKX*1 お続け
おつづり #OKX*6 お綴り #OKX*1 おつづり
おつとめ #OKX*10 お勤め #OKX*2 お努め #OKX*1 おつとめ #OKX お勉め お務め
おつどい #OKX*2 お集い
おつながり #OKX*120 おつながり #OKX*40 お継り #OKX*3 お繋がり
おつなぎ #OKX*73 おつなぎ #OKX*23 お繋ぎ
おつなげ #OKX*1 お繋げ
おつのり #OKX*4 お募り #OKX*1 おつのり
おつぶし #OKX*8 お潰し
おつぶり #OKX*4 おつぶり
おつぶれ #OKX*16 お潰れ
おつまずき #OKX*1 おつまずき
おつまみ #OKX*1 お摘み
おつまり #OKX*31 おつまり #OKX*6 お詰まり
おつみ #OKX*27 お積み #OKX*4 おつみ #OKX お摘み
おつむぎ #OKX*1 お紡ぎ
おつめ #OKX*28 お詰め #OKX*20 おつめ
おつもり #OKX*3 お積もり
おつよめ #OKX*4 お強め
おつらぬき #OKX*2 お貫き
おつらね #OKX*1 お連ね
おつり #OKX*3 おつり #OKX*1 お吊り お釣り
おつるし #OKX*1 お吊し
おつれ #OKX*47 おつれ #OKX*25 お連れ
おつれかえり #OKX*1 お連れ帰り
おつれそい #OKX お連れ添い
おつれもどし #OKX お連れ戻し
おてあげ #T15 お手上げ
おてかず #T35 お手数
おてがけ #OKX*1 お手掛け
おてきし #OKX*80 お適し
おてすう #T35*102 お手数
おてつだい #OKX*30 お手伝い
おてほん #T35*1 お手本
おてま #T35 お手間
おてまえ #T35 お点前
おてまどり #OKX*3 お手間取り
おてらい #OKX*1 お衒い
おてらし #OKX*10 おてらし #OKX*2 お照らし
おてれ #OKX お照れ
おてわたし #OKX*7 お手渡し
おてん #T35*2 汚点
おであい #OKX*7 お出会い
おであるき #OKX*1 お出歩き
おでかけ #T30 お出かけ
おでき #OKX*255 おでき #OKX*244 お出来
おでむかえ #OKX お出迎え
おでむき #OKX*2 お出向き
おでら #JNS 小寺
おでん #T35*2 おでん
おでんわ #T30*14 お電話
おと #S5*91 落 #T35*23 音 #R5*6 劣 #S5*3 落と #S5*1 おと
おとい #OKX*56 お問い
おといあわせ #OKX*69 お問い合わせ
おといかけ #OKX*1 お問いかけ
おといねっぷ #CN 音威子府
おとうさん #T35*5 お父さん #T35*4 おとうさん
おとうと #T35*9 弟
おとおし #OKX*255 お通し #OKX*1 おとおし
おとおり #OKX*255 お通り
おとかし #OKX*4 お融かし #OKX*2 お溶かし #OKX*1 お解かし
おとがめ #OKX*1 お咎め
おとき #OKX*111 お解き #OKX*5 お溶き #OKX*2 お説き #OKX お釈き
おとぎ #OKX お研ぎ
おとくに #CN 乙訓
おとけ #OKX*1 お融け #OKX お熔け
おとげ #OKX*1 お遂げ
おとこ #T35*53 男
おとこで #T35 男手
おとこのこ #T35*6 男の子
おとこらし #KY 男らし
おとこらしゅう #KYU 男らしゅう
おとさた #T35*4 音沙汰
おとざし #OKX お閉ざし
おとし #T35*6 お年
おとしいれ #KS*1 陥れ
おとしもの #T30 落し物
おとじ #OKX*28 お閉じ #OKX*5 お綴じ
おとずれ #KS*2 訪れ
おとつぎ #OKX*1 お嫁ぎ
おとづれ #KS 訪れ
おととい #T35*10 一昨日
おととし #T35*1 一昨年
おととのえ #OKX*4 お整え #OKX お調え
おとどき #OKX*255 お届き
おとどけ #OKX*2 お届け
おとどまり #OKX*2 お留まり
おとどめ #OKX*5 おとどめ
おとな #T35*9 大人
おとなえ #OKX*2 お唱え
おとなげな #KY*1 大人げな
おとなげのう #KYU*1 大人げのう
おとなし #KY*3 おとなし #KY*1 大人し #JCN 音無
おとなしゅう #KYU*1 おとなしゅう 大人しゅう
おとなりあわせ #OKX お隣合わせ
おとばし #OKX*18 お飛ばし
おとひめ #JN 乙姫
おとび #OKX*88 お飛び #OKX*12 おとび #OKX*2 お跳び
おとびこえ #OKX お飛び越え
おとびこし #OKX*1 お飛び越し
おとびこみ #OKX*2 おとびこみ お飛び込み
おとびだし #OKX*2 お飛び出し
おとふけ #JCN 音更
おとべ #JCN 乙部
おとまどい #OKX*3 お戸惑い #OKX おとまどい
おとまり #OKX*52 お止まり #OKX*37 お泊まり #OKX お留まり
おとまりこみ #OKX*2 お泊り込み
おとみ #OKX*12 お富み
おとむらい #OKX お弔い
おとめ #OKX*47 お止め #OKX*5 お泊め #T35*5 乙女 #OKX*1 おとめ お留め #JCN 乙女
おとめざ #KK*1 乙女座
おとらえ #OKX*37 おとらえ #OKX*6 お捉え #OKX*2 お捕らえ
おとり #OKX*255 おとり お取り #OKX*24 お採り #OKX*17 お撮り #OKX*1 お捕り
おとりあげ #OKX*4 お取り上げ #OKX*1 おとり上げ
おとりあつかい #T30 お取り扱い お取扱い
おとりいれ #OKX*27 お採り入れ
おとりかえ #OKX*3 お取り換え お取り替え
おとりこみ #OKX*47 お取り込み
おとりこみちゅう #T35 お取り込み中
おとりさげ #OKX*1 お取り下げ
おとりしらべ #T30 お取り調べ
おとりたて #OKX*1 お取り立て
おとりだし #OKX*79 お取り出し
おとりつぎ #OKX*2 お取り次ぎ
おとりなし #OKX お執り成し
おとりはからい #T30 お取り計らい
おとりひき #T30 お取り引き
おとりよせ #OKX*2 お取り寄せ
おとろえ #OKX 衰え
おとわ #JCN 音羽
おど #R5r*3 踊 #R5r*1 おど #S5r*1 脅 #R5 躍
おどか #S5r*1 脅か
おどろ #K5r*61 驚
おどろおどろし #KY おどろおどろし
おどろおどろしゅう #KYU おどろおどろしゅう
おどろか #S5 驚か
おない #JNS 尾内
おないどし #T15 同い年
おなおし #OKX*255 お直し #OKX*52 お治し #OKX*8 おなおし
おなおり #OKX*168 お直り #OKX*53 お治り #OKX*1 おなおり
おなか #T35*19 お腹
おなが #T35 オナガ 尾長
おながし #OKX*77 お流し
おながどり #T35 オナガドリ 尾長鶏
おながれ #OKX*115 お流れ
おながわ #JCN 女川
おなき #OKX*82 お泣き #OKX*7 お鳴き
おなくし #OKX*11 お無くし #OKX*6 おなくし #OKX お亡くし
おなくなり #OKX*16 お亡くなり #OKX*3 お無くなり
おなぐさめ #OKX*2 お慰め
おなぐり #OKX*3 お殴り
おなげ #OKX*9 お投げ
おなげき #OKX*2 お嘆き
おなごみ #OKX*1 お和み
おなじ #T05*255 同じ #T35*28 々 #T35 おなじ
おなじく #CJ*25 同じく #T35*1 〃
おなじみ #T16*2 おなじみ お馴染み
おなじよう #T05*41 同じよう
おなだめ #OKX おなだめ お宥め
おなつかしゅう #KYU*2 お懐かしゅう #KYU*1 おなつかしゅう
おなで #OKX*1 お撫で
おなにー #T30*1 オナニー
おなのり #OKX*4 お名乗り
おなまえ #T35*5 お名前
おなめ #OKX*2 おなめ #OKX お嘗め
おなやみ #OKX*14 お悩み
おならい #OKX*6 お習い
おならし #OKX*5 お鳴らし #OKX お均し
おならび #OKX*88 お並び
おならべ #OKX*1 お並べ #OKX お陳べ お列べ
おなり #OKX*255 おなり #OKX*12 お成り #OKX*5 お鳴り
おなりもん #CN 御成門
おなれ #OKX*30 お慣れ #OKX お馴れ
おに #T35*2 鬼
おにあい #OKX*21 お似合い
おにいさん #T35*2 お兄さん
おにがし #OKX*1 お逃し #OKX お逃がし
おにぎり #OKX*6 お握り
おにくみ #OKX*1 お憎み
おにげ #OKX*11 お逃げ
おにし #JCN 鬼石
おにせ #OKX*1 お似せ
おにづか #JNS 鬼塚
おにない #OKX*10 お担い
おにゅう #JCN 遠敷
おにらみ #OKX*3 お睨み
おぬい #OKX*3 お縫い
おぬかし #OKX*3 お抜かし
おぬき #OKX*14 お抜き
おぬぎ #OKX*2 お脱ぎ
おぬぐい #OKX*10 お拭い #OKX*1 おぬぐい
おぬけ #OKX*70 お抜け
おぬすみ #OKX*2 お盗み
おぬらし #OKX お濡らし
おぬり #OKX*8 お塗り
おぬれ #OKX*2 お濡れ
おね #T35*8 尾根
おねいり #OKX お寝入り
おねえさん #T35*10 お姉さん
おねがい #OKX*50 お願い
おねぎらい #OKX お労い
おねこみ #OKX*3 お寝込み
おねじり #OKX*1 おねじり #OKX お捻り
おねたみ #OKX おねたみ #M5r お妬み
おねだり #T30 おねだり
おねつき #OKX*1 お寝つき
おねむり #OKX*34 お眠り
おねらい #OKX*11 お狙い #OKX*4 おねらい
おねり #OKX*4 お練り
おの #T35*2 斧 #JCN 小野
おのえ #JCN 尾上
おのおの #T35*30 各々 #F14*1 おのおの
おのがし #OKX*5 お逃し
おのがみ #JCN 小野上
おのがれ #OKX*3 お逃れ
おのき #OKX*20 お退き
おのこし #OKX*117 お残し
おのこり #OKX*117 お残り
おのざわ #JNS 小野沢
おのずから #F14 自ずから
おのずと #F14*1 自ずと
おのせ #OKX*55 お載せ #OKX*15 お乗せ
おのぞき #OKX*80 お除き #OKX*15 お覗き
おのぞみ #OKX*17 お望み #OKX*1 お臨み
おのだ #JCN 小野田
おのづか #JNS 小野塚
おのでら #JNS 小野寺
おのばし #OKX*35 お伸ばし #OKX*7 お延ばし
おのび #OKX*9 お延び #OKX*4 お伸び
おのぼせ #OKX*1 おのぼせ
おのぼり #OKX*18 お登り #OKX*2 おのぼり #OKX*1 お昇り お上り
おのみ #OKX*163 お飲み #OKX*3 お呑み #OKX*2 おのみ
おのみち #JCN 尾道
おのり #OKX*168 お載り #OKX*140 お乗り #OKX*18 おのり
おのりかえ #OKX*13 お乗り換え
おのりこし #OKX お乗り越し
おのりこみ #OKX*2 お乗り込み
おのりつぎ #OKX*1 お乗り継ぎ
おのれ #T35*4 己
おのろい #OKX*2 お呪い
おはいり #OKX*255 お入り #OKX*3 おはいり
おはいりこみ #OKX*1 お入り込み
おはえ #OKX*2 お生え #OKX*1 お映え #OKX お栄え
おはかどり #OKX*2 お捗り
おはからい #OKX*1 お計らい
おはかり #OKX*42 おはかり #OKX*13 お計り #OKX*6 お図り #OKX*5 お測り #OKX お諮り #R5 お謀り #R5r お量り
おはがし #OKX*1 おはがし #OKX お剥し
おはき #OKX*100 お吐き #OKX*13 おはき #OKX*2 お履き #OKX お掃き #K5 お噴き
おはぎ #OKX*1 お剥ぎ
おはげ #OKX おはげ お禿げ お剥げ
おはげまし #OKX*2 お励まし
おはげみ #OKX*2 お励み
おはこび #OKX*14 お運び
おはさみ #OKX*5 おはさみ #OKX*3 お挟み
おはしゃぎ #OKX*10 おはしゃぎ
おはしり #OKX*74 お走り
おはじ #OKX*3 お恥じ
おはじき #OKX*3 お弾き #OKX*1 おはじき
おはじめ #OKX*184 お始め #OKX*148 おはじめ
おはずかしゅう #KYU*2 お恥ずかしゅう #KYU*1 おはずかしゅう お恥しゅう
おはずし #OKX*41 お外し
おはずみ #OKX*1 おはずみ #OKX お弾み
おはずれ #OKX*7 お外れ
おはたし #OKX*27 お果たし
おはたらき #OKX*33 お働き
おはて #OKX*10 お果て #OKX*4 おはて
おはなし #OKX*131 お話し #T30*96 お話 #OKX*58 お離し #OKX*7 お放し #OKX*1 おはなし
おはなしあい #OKX*15 お話し合い
おはなち #OKX*2 お放ち
おはなれ #OKX*16 お離れ #OKX お放れ
おはね #OKX*4 おはね #OKX*1 お跳ね
おはばみ #OKX*1 お阻み
おはぶき #OKX*11 お省き
おはめ #OKX*3 おはめ #OKX*1 お填め
おはやし #OKX*1 お生やし
おはやまり #OKX*2 お早まり
おはやめ #OKX*17 お早め #OKX*1 お速め
おはよう #KYU*34 おはよう #T35*8 おはよう #KYU お早う
おはら #JNS 小原
おはらい #OKX*31 お払い
おはらいこみ #OKX*1 お払い込み
おはらし #OKX*8 お晴らし
おはり #OKX*85 お張り #OKX*12 お貼り
おはりきり #OKX*3 お張り切り
おはれ #OKX*92 お晴れ #OKX*1 お腫れ
おば #T35*6 叔母 #T35 小母 伯母
おばあさん #T35*2 おばあさん #T35 お婆さん
おばけ #OKX*26 お化け #T35*2 オバケ #T35*1 おばけ
おばた #JNS 小幡 小畑 #JCN 小俣
おばたりあん #T35*1 オバタリアン
おばなざわ #JCN 尾花沢
おばま #JCN 小浜
おばら #JCN 小原
おぱーる #T35 オパール
おひかえ #OKX*21 お控え
おひかり #OKX*6 お光り
おひがみ #OKX*1 おひがみ #OKX お僻み
おひき #OKX*129 お引き #OKX*13 お弾き #OKX*4 おひき #OKX*2 お曳き #OKX*1 お惹き お退き #OKX お索き
おひきうけ #OKX*4 お引き受け #T30*1 お引受け
おひきかえ #OKX*3 おひきかえ #OKX*1 お引き替え
おひきかえし #OKX お引き返し
おひきたて #T30 お引き立て
おひきだし #OKX*9 お引き出し
おひきとめ #OKX*2 お引き留め
おひきとり #OKX*3 お引き取り
おひきのばし #OKX*1 お引き延ばし お引き伸ばし
おひきわたし #OKX*1 お引き渡し
おひくめ #OKX お低め
おひさしぶり #T35 お久しぶり
おひさしゅう #KYU*1 お久しゅう
おひそみ #OKX*3 お潜み
おひそめ #OKX お潜め
おひたし #OKX*1 お浸し
おひたり #OKX*2 お浸り
おひっこし #OKX*3 お引っ越し
おひつじざ #KK*1 牡羊座
おひとよし #T05*1 御人好し #T05 お人好し
おひねり #OKX*1 お捻り #OKX おひねり
おひめ #OKX*1 お秘め
おひやかし #OKX*2 お冷やかし
おひやし #OKX*2 お冷やし
おひらき #OKX*58 お開き #OKX*1 おひらき
おひるみ #OKX おひるみ #M5r お怯み
おひれ #T35*1 尾ひれ
おひろい #OKX*32 お拾い
おひろがり #OKX*1 お広がり
おひろげ #OKX*16 お広げ #OKX*1 おひろげ
おひろめ #OKX*3 お広め #T30 お披露目
おび #T35*2 帯 #KS*1 帯び
おびじょう #T35 帯状
おびただし #KY*1 おびただし #KY 夥し
おびただしゅう #KYU*1 おびただしゅう #KYU 夥しゅう
おびとけ #JCN 帯解
おびひろ #CN 帯広
おびやか #OKX*1 脅か
おびら #JCN 小平
おふ #T30*33 オフ
おふぃす #T35*9 オフィス
おふえ #OKX*94 お増え #OKX*1 おふえ
おふかまり #OKX*1 お深まり
おふかめ #OKX*3 お深め
おふき #OKX*58 お吹き #OKX*6 お拭き #OKX*1 お噴き
おふくみ #OKX*228 お含み
おふくらみ #OKX お膨らみ
おふくれ #OKX*1 おふくれ #OKX お膨れ
おふけ #OKX お老け
おふさぎ #OKX お塞ぎ
おふざけ #OKX*2 おふざけ
おふせぎ #OKX*15 お防ぎ
おふせっと #T35*29 オフセット
おふとり #OKX*32 お太り #OKX*1 おふとり
おふまえ #OKX*2 お踏まえ #OKX*1 おふまえ
おふみ #OKX*73 お踏み
おふやし #OKX*36 お増やし #OKX*1 おふやし #OKX お殖やし
おふらし #OKX お降らし
おふり #OKX*27 お振り
おふりこみ #OKX*6 お振り込み
おふりそそぎ #OKX お降り注ぎ
おふりむき #OKX*2 お振り向き #OKX*1 おふりむき
おふるい #OKX*12 お震い #OKX*2 お振るい #OKX*1 お奮い
おふるえ #OKX お震え
おふるび #OKX*1 お古び
おふるまい #OKX*14 お振舞い #OKX*1 お振る舞い #OKX おふるまい
おふれ #T30 お触れ
おふれこ #T35*2 オフレコ
おぶじぇくと #T35*255 オブジェクト
おぶせ #JCN 小布施
おぶつ #T35 汚物
おぷしょん #T35*199 オプション
おぷてぃまいず #T30*3 オプティマイズ
おへだて #OKX お隔て
おへや #T35*1 お部屋
おへらし #OKX*39 お減らし
おへり #OKX*29 お減り
おへんじ #T30*52 お返事 #T30 御返事
おべっか #T35 おべっか
おべんちゃら #T35 おべんちゃら
おぺこーど #T35*1 オペコード
おぺら #T35*9 オペラ
おぺらんど #T35*2 オペランド
おぺれーしょなる #T15*1 オペレーショナル
おぺれーしょん #T30*16 オペレーション
おぺれーた #T35*2 オペレータ
おぺれーたー #T35*1 オペレーター
おぺれーてぃんぐ #T35*5 オペレーティング
おぺれーてぃんぐしすてむ #T35*15 オペレーティングシステム
おほうり #OKX*4 お放り
おほこり #OKX*3 お誇り
おほし #OKX*25 お干し
おほそり #OKX*4 お細り
おほどき #OKX*6 お解き
おほどこし #OKX*14 お施し
おほねおり #OKX お骨折り
おほほえみ #OKX*2 お微笑み
おほめ #OKX*13 お褒め #OKX*3 お誉め
おほり #OKX*38 おほり #OKX*5 お掘り #OKX*1 お彫り
おほれ #OKX お惚れ
おほろぼし #OKX お滅ぼし
おぼえ #KSr*70 覚え #KSr*7 憶え #KSr*4 おぼえ
おぼえがき #T35*10 覚書
おぼし #KY*1 思し
おぼしゅう #KYU*1 思しゅう
おぼつかな #KY*1 おぼつかな
おぼつかのう #KYU*1 おぼつかのう
おぼれ #KS*2 溺れ
おぼろ #T06 おぼろ 朧ろ
おまい #OKX*34 お舞い
おまいり #OKX*14 おまいり #T30*1 お参り #T30 お詣り 御参り
おまえ #T35*4 お前
おまえざき #CN 御前崎
おまかせ #OKX*26 お任せ #OKX*2 おまかせ #OKX お委せ
おまかない #OKX*1 お賄い
おまがり #OKX*5 お曲がり #OKX お曲り
おまき #OKX*80 お巻き #OKX*4 おまき #OKX*1 お撒き お蒔き #OKX お捲き
おまぎらし #OKX お紛らし
おまぎれ #OKX*2 お紛れ
おまけ #OKX*25 お負け #OKX*8 お敗け #T30*6 おまけ
おまけに #CJ*1 おまけに
おまげ #OKX*3 お曲げ
おまごさん #T35 お孫さん
おまざり #OKX*9 お混ざり #OKX*1 お交ざり
おまし #OKX*33 お増し
おまじえ #OKX*4 お交え
おまじない #T35*1 おまじない
おまじり #OKX*4 お混じり #OKX*3 お交じり
おまじわり #OKX*2 お交わり
おまぜ #OKX*8 お混ぜ
おまた #JNS 小俣 尾股
おまたがり #OKX*19 お跨り
おまたぎ #OKX*10 お跨ぎ
おまたせ #OKX お待たせ
おまたたき #OKX*3 お瞬き
おまち #OKX*72 お待ち
おまちがい #OKX*213 お間違い
おまちのぞみ #OKX*1 お待ち望み
おまつり #T30*1 お祭り #OKX*1 お奉り
おまとめ #OKX*80 おまとめ
おまどい #OKX お惑い
おまなび #OKX*10 お学び
おまぬかれ #OKX お免れ
おまぬがれ #OKX*1 お免れ
おまねき #OKX*8 お招き
おまばたき #OKX*1 お瞬き
おまもり #OKX*1 おまもり お護り #T30 お守り
おまよい #OKX*14 お迷い
おまわし #OKX*94 お回し #OKX*16 おまわし #OKX*3 お廻し
おまわり #OKX*105 お回り #OKX*23 おまわり #T35*10 お巡り #OKX*3 お廻り
おみ #KJ 臣 #JNS 尾美 #JCN 麻績
おみあい #OKX*1 お見合い
おみうけ #OKX*2 お見受け
おみえ #OKX*150 お見え #OKX*1 おみえ
おみおくり #OKX*7 お見送り
おみおつけ #T35 おみおつけ
おみおとし #OKX*3 お見落とし #OKX*1 お見落し
おみかけ #OKX*6 お見かけ
おみがき #OKX*1 お磨き #OKX お研き
おみがわ #JCN 小見川
おみくじ #T35 おみくじ
おみくろん #T35*1 Ο ο
おみすて #OKX*2 お見捨て
おみせ #OKX*38 お見せ #OKX*3 おみせ
おみたて #OKX お見立て
おみだし #OKX*2 お乱し
おみだれ #OKX*11 お乱れ
おみち #OKX*1 お満ち
おみちがえ #OKX お見違え
おみちびき #OKX*6 お導き
おみつくろい #OKX お見繕い
おみつもり #OKX*9 お見積もり
おみとめ #OKX*29 お認め #OKX*4 おみとめ
おみとり #OKX お看取り
おみのがし #OKX*4 お見逃し
おみまい #T30*4 お見舞 #OKX お見舞い #T30 御見舞
おむかい #OKX*40 お向かい
おむかえ #OKX*6 お迎え
おむき #OKX*104 お向き #OKX お剥き
おむけ #OKX*27 お向け
おむし #OKX*3 お蒸し
おむすび #OKX*33 お結び #T35 おむすび
おむせび #OKX お咽び
おむにばす #T35*3 オムニバス
おむらいす #T35*1 オムライス
おむれつ #T35 オムレツ
おむろん #KK*11 ＯＭＲＯＮ #KK*4 オムロン
おめい #T35 汚名
おめいし #T35 お名刺
おめが #T35*1 Ω ω
おめぐみ #OKX*4 お恵み
おめぐり #OKX*11 おめぐり #OKX*1 お巡り
おめざし #OKX*18 お目指し
おめざめ #OKX*2 お目覚め
おめし #OKX*7 お召し
おめずらしゅう #KYU*2 お珍しゅう
おめだち #OKX*10 お目立ち
おめでとう #CJ*22 おめでとう
おめにかか #R5*5 お目にかか
おも #W5r*255 思 #T05*82 主 #KYmi*43 重 #W5r*3 想 #W5r*2 おも #KJ*1 重 面
おもいあた #R5r*1 思い当た
おもいおこ #S5*1 思い起こ
おもいかえ #S5 思い返
おもいがけな #KY*1 思いがけな
おもいがけのう #KYU*1 思いがけのう
おもいき #R5r*4 思い切
おもいきや #CJ 思いきや
おもいこ #M5r*4 思い込
おもいし #R5*1 思い知
おもいた #T5*1 思い立
おもいだ #S5*18 思い出 #S5*3 おもいだ #S5*1 思いだ
おもいちがい #T30*1 思い違い
おもいつ #K5r*8 思い付 #K5r*6 思いつ
おもいつめ #KS 思い詰め
おもいで #T35*4 想い出 #T35*3 思い出
おもいなお #S5r*1 思い直
おもいのこ #R5r 思い残 #S5r 思い残
おもいや #R5r*2 思いや
おもう #KYU*2 重う
おもうけ #OKX*31 お設け #OKX*5 お儲け #OKX*1 おもうけ
おもうしこみ #T30*65 お申し込み
おもえ #OKX*4 お燃え #OKX お萌え
おもおもし #KY 重々し
おもおもしゅう #KYU 重々しゅう
おもかげ #T35*1 面影
おもくるし #KY*1 重苦し
おもくるしゅう #KYU*1 重苦しゅう
おもぐり #OKX*1 お潜り
おもご #JCN 面河
おもざし #T35 面差し
おもしろ #KYmi*82 面白 #T35*62 面白 #KYmi*2 おもしろ #T35*1 おもしろ
おもしろう #KYU*2 面白う #KYU*1 おもしろう
おもた #KY*7 重た
おもだえ #OKX お悶え
おもち #OKX*255 お持ち #OKX*19 おもち
おもちあるき #OKX*1 お持ち歩き
おもちこみ #OKX*14 お持ち込み
おもちだし #OKX*1 お持ち出し
おもちはこび #OKX お持ち運び
おもちゃ #T35*7 玩具
おもて #T35*2 表 #T35 面
おもてがき #T30 表書き
おもてがわ #T35 表側
おもてぐち #T35*1 表口
おもてごう #JCN 表郷
おもてさんどう #CN 表参道
おもてじ #T35 表地
おもてむき #T35 表向き
おもとう #KYU*1 重とう
おもとめ #OKX*69 お求め
おもどし #OKX*182 お戻し
おもどり #OKX*248 お戻り
おもに #T35*2 重荷 #F14*1 おもに #F14 主に
おもね #R5r おもね 阿
おものがわ #CN 雄物川
おもみ #OKX*12 おもみ #OKX*3 お揉み
おもむ #K5*1 赴
おもむき #T35*2 趣
おもむろに #F14*2 おもむろに
おもめ #OKX*5 おもめ #OKX*1 お揉め
おももち #T35 面持ち
おもや #T35 母屋
おもやし #OKX お燃やし
おもらし #OKX*1 お漏らし
おもり #OKX*3 お盛り #OKX*2 おもり
おもれ #OKX*9 お洩れ
おもわく #T35*1 思惑
おもわし #KY*2 思わし
おもわしゅう #KYU*1 思わしゅう
おもわず #F14 思わず
おもん #ZX*3 重ん
おもんぱか #R5r*1 慮
おや #T35*61 親 #CJ*2 おや #T35*2 おや
おやかた #T35*1 親方
おやき #OKX*31 お焼き #OKX*10 お妬き #OKX*1 おやき お灼き
おやくそく #T30*4 お約束
おやけ #OKX*1 お焼け
おやこ #T35*5 親子
おやこうこう #T00*1 親孝行
おやじ #T35*4 親父
おやすまり #OKX お休まり
おやすみ #OKX*39 お休み #OKX*13 おやすみ #T35*12 おやすみ
おやすらぎ #OKX*1 お安らぎ
おやせ #OKX*3 お痩せ
おやだま #T35*1 親玉
おやとい #OKX*1 お雇い
おやどり #OKX*1 お宿り
おやばか #T05 親馬鹿
おやふこう #T00 親不孝
おやぶり #OKX*3 お破り
おやぶれ #OKX お敗れ
おやぶん #T35*2 親分
おやべ #JCN 小矢部
おやま #JCN 小山
おやみ #OKX*199 おやみ #OKX*1 お病み
おやめ #OKX*29 お止め #OKX*4 お辞め
おやもと #T35 親元
おやゆび #T35*1 親指
おやり #OKX*195 おやり #OKX お遣り
おゆ #T35*2 お湯
おゆい #OKX*13 お結い
おゆき #OKX*45 おゆき #OKX*22 お行き #OKX お往き #K5 お逝き
おゆすぎ #OKX お濯ぎ
おゆすり #OKX*1 おゆすり
おゆずり #OKX*7 お譲り
おゆだね #OKX*2 お委ね
おゆで #OKX*2 おゆで
おゆるし #T30*9 お許し
おゆるみ #OKX*4 お緩み
おゆるめ #OKX お緩め
おゆわえ #OKX お結わえ
およ #B5r*247 及 #G5r*8 泳 #B5r*2 およ
およい #OKX*68 お酔い
およけ #OKX*1 お避け
およごし #OKX*1 お汚し
およごれ #OKX*1 お汚れ
およし #OKX*70 およし #OKX お止し
およせ #OKX*10 お寄せ
およそ #F14*3 およそ #T35*3 およそ
および #CJ*200 および #OKX*8 および #CJ*2 及び #T30 お呼び
およびかけ #OKX*2 お呼びかけ
およびごし #T15 及び腰
およびだし #OKX*139 お呼び出し
およぼ #S5*6 及ぼ #S5*2 およぼ
およみ #OKX*255 お読み #OKX*2 お詠み
およみあげ #OKX*1 お読み上げ
おより #OKX*94 おより #OKX*64 お寄り #OKX*4 お依り #OKX お選り #R5r お撚り
およろこび #T30*2 お慶び #OKX*1 およろこび #T30*1 お喜び #OKX お悦び お歓び
およわめ #OKX*1 お弱め
およわり #OKX*11 お弱り
おらんうーたん #T35*3 オランウータン
おらんだ #CN オランダ
おり #KS*27 降り #KS*7 下り #T35*1 折 澱 #T35 檻
おりあ #W5r 折り合
おりえんてっど #T05*2 オリエンテッド
おりえんと #CN オリエント
おりおり #T35*1 折々
おりおん #JN オリオン
おりかえ #S5r*6 折り返
おりかさな #R5r 折り重な
おりがみ #T30*2 折り紙
おりこ #M5r*2 織り込 #M5r 折り込
おりこみ #T35*1 折込
おりしも #F14 折しも
おりじなる #T05*60 オリジナル
おりたた #M5r*3 折り畳
おりはら #JNS 折原
おりめ #T35 折り目
おりもの #T35*1 織物
おりんぱす #KK*1 オリンパス
おりんぴっく #T35*6 オリンピック
おるごーる #T35 オルゴール
おれ #T35*50 俺 #KSr*1 折れ
おれい #T30*11 お礼 #T30*4 御礼
おれせん #T35*3 折れ線
おれたち #T35 おれたち 俺達
おれまが #R5r 折れ曲が
おれんじ #T35*4 オレンジ
おろ #S5*7 降ろ #S5*3 下ろ #T35*1 悪露 #S5r 卸
おろか #T05*2 愚か
おろかし #KY 愚かし
おろかしゅう #KYU 愚かしゅう
おろし #T35 卸
おろしうり #T30*1 卸売
おろしそうば #T35 卸相場
おろしね #T35 卸値
おろそか #T05*2 疎か
おわ #R5r*160 終 #R5r*139 終わ
おわい #T35 汚穢
おわかし #OKX*1 お沸かし
おわかり #OKX*255 お分かり #OKX*199 おわかり #OKX*31 お判り #OKX*15 お解り #OKX*10 お分り
おわかれ #OKX*22 お別れ #OKX*10 お分かれ
おわけ #OKX*160 お分け #OKX*23 おわけ
おわすれ #OKX*188 お忘れ
おわずらい #OKX*1 お煩い #OKX お患い
おわせ #JCN 尾鷲
おわたし #OKX*255 お渡し
おわたり #OKX*233 お渡り #OKX*5 おわたり #OKX*1 お航り
おわび #T30*11 お詫び #OKX お詫び
おわらい #OKX*38 お笑い
おわり #OKX*14 お割り #T35*6 終 #CN 尾張
おわりあさひ #CN 尾張旭
おん #T35*37 恩 #T30*36 オン #KJ*12 温 #T35*11 音 #PRE*3 御 #KJ*1 穏 #KJ 遠 #SUC 音 #KJ 御
おんいん #T35*1 音韻
おんかい #T35*1 音階
おんかん #T35*1 音感
おんが #JCN 遠賀
おんがえし #T30 恩返し
おんがく #T35*19 音楽
おんきゅう #T35*1 恩給
おんきょう #T35*3 音響
おんぐすとろーむ #T35*1 Å
おんけい #T35*3 恩恵
おんけん #T05 穏健
おんこ #T35 恩顧
おんこう #T05 温厚
おんさいと #T35*2 オンサイト
おんし #T35 恩師 恩賜
おんしつ #T35*42 音質 #T35*6 温室
おんしゃ #T35*1 御社 #T35 恩赦
おんしょう #T35*1 温床 #T35 恩賞
おんしん #T35*1 音信
おんじゅく #JCN 御宿
おんじゅん #T05 温順
おんじょう #T35*1 温情 #T35 音声
おんじん #T35*1 恩人
おんすい #T35 温水
おんせい #T35*10 音声
おんせつ #T35*3 音節
おんせん #CN 温泉
おんそ #T35*1 音素
おんぞう #T35*1 音像
おんぞうし #T35*1 御曹司
おんぞん #T30 温存
おんたい #T35*1 温帯
おんだ #JNS 恩田
おんだん #T05 温暖
おんち #T35*1 音痴
おんちゅう #T35*2 御中
おんてい #T35 音程
おんてん #T35 恩典
おんとう #T05*1 穏当
おんど #T35*4 温度 #T35*2 音頭 #JCN 音戸
おんどけい #T35*1 温度計
おんな #T35*51 女 #JCN 恩納
おんなじ #T05*15 おんなじ
おんなのこ #T35*26 女の子
おんならし #KY 女らし
おんならしゅう #KYU 女らしゅう
おんねつ #T35 温熱
おんねん #T35*1 怨念
おんぱ #T35*1 音波
おんびん #T35*1 音便 #T05 穏便
おんぶ #T30*1 おんぶ
おんぷ #T35*1 ♪ #T35 音符
おんぷう #T35 温風
おんべつ #CN 音別
おんみつ #T05 隠密
おんみょう #T35 陰陽
おんみょうじ #T35 陰陽師
おんよく #T30 温浴
おんよみ #T30*1 音読み
おんらいん #T35*31 オンライン
おんりー #N2T16*2 オンリー
おんりょう #T35*2 音量
おんれい #T35*1 御礼
おんわ #T05 温和 穏和
か #S5*255 化 #T30*255 化 #K5*255 書 #W5r*255 買 #S5*237 貸 #S5*182 課 #T30*182 課 #T35*91 可 #T5r*62 勝 #KJ*59 火 #K5*38 描 #KJ*28 下 #R5*25 苅 #K5*21 欠 #KJ*16 加 #T35*14 香 #KJ*12 ヵ #M5*11 噛 #KJ*10 花 日 #KJ*8 ヶ #S5*7 科 #T30*7 科 #KJ*6 家 #KJ*4 夏 #T35*3 果 #KJ*3 茄 #T35*3 蚊 #KJ*3 掛 #KJ*2 仮 何 佳 嘉 珂 靴 #JS*1 価 #T30*1 嫁 #KJ*1 寡 歌 禾 箇 苛 華 蝦 嘩 #R5*1 駆 #W5*1 飼 #KJ 渦 #SUC 下 #N2T30 化 #KJ 伽 価 #SUC 可 家 科 #KJ 暇 #S5 架 #T30 架 #SUC 歌 #KJ 河 禍 稼 荷 菓 #JS 課 #SUC 課 #KJ 貨 迦 過 #PRE 過 #KJ 霞 #R5 穫 刈 #KJ 卦 袈 個 鹿 #R5r 狩 #K5 掻 #JS 日 #KJ 彼 耶
かー #T35*7 カー
かーそる #T35*255 カーソル
かーとりっじ #T35*14 カートリッジ
かーど #T35*48 カード
かーにばる #T35*1 カーニバル
かーねぎーめろん #KK*1 カーネギーメロン
かーねる #T35*47 カーネル
かーぶ #T30*1 カーブ
かーる #T30*1 カール
かあさん #T35*1 母さん
かあちゃん #T35*1 母ちゃん
かい #JS*255 回 #KJ*255 開 #T35*246 会 #S5*79 解 #T30*79 解 #T35*59 下位 #S5*51 介 #T30*51 介 #T35*24 階 #KJ*11 海 #T35*6 Χ #KJ*6 改 #T35*4 χ 怪 #KJ*3 快 #T35*3 界 #KJ*3 戎 #KJ*2 懐 潰 #T35*1 かい #S5*1 会 #T35*1 回 #KJ*1 界 皆 #T35*1 貝 甲斐 #SUC 会 #SX 会 #KJ 塊 壊 廻 #PRE 快 怪 #KJ 悔 恢 戒 拐 魁 晦 械 灰 #SUC 界 #KJ 絵 芥 蟹 #JS 階 #JCN 甲斐
かいあく #T30*1 改悪
かいあげ #KSr*1 買い上げ
かいい #T05*1 怪異 #T05 魁偉
かいいき #T35 海域
かいいれ #KSr 買い入れ
かいいん #T35*16 会員 #T30 改印 #T35 海員 #T30 開院
かいいんせい #T35*1 会員制
かいうん #T35 海運 開運
かいえん #T30*4 開宴 #T30*1 開演 #T30 開園
かいおうせい #KK*1 海王星
かいおき #T30 買い置き
かいか #T30 開化 開花 #T35 階下
かいかい #T30*2 開会
かいかえ #KSr*3 買い替え
かいかく #T30*6 改革
かいかけ #T35 買掛
かいかつ #T05 快活
かいかん #T35*10 会館 #T35*2 会舘 快感 #T30 開館
かいが #T35*2 絵画
かいがい #T35*60 海外
かいがら #T35*1 貝殻
かいがん #T35*11 海岸 #T30 開眼
かいがんせん #T35*1 海岸線
かいき #JS*2 回忌 #T35*1 会期 回忌 #T30*1 回帰 #T35*1 皆既 #T30 快気 #T05 怪奇 #R5r 買い切
かいきしょく #T35 皆既食
かいきせん #T35 回帰線
かいきゅう #JS*2 階級 #T35*2 階級 #T35 懐旧
かいきょ #T35*2 快挙
かいきょう #T35 回教 海峡
かいきょく #T30*1 開局
かいきん #T30*1 皆勤 #T30 解禁 #T35 開襟
かいぎ #T30*255 会議 #T30*2 懐疑
かいぎゃく #T35*1 諧謔
かいぎょう #T30*44 改行 #T30*1 開業
かいぎょうい #T35 開業医
かいぐい #T30 買い食い
かいぐん #T35 海軍
かいけい #T30*26 会計
かいけつ #T30*102 解決
かいけつさく #T35*5 解決策
かいけん #T30*1 会見 #T30 改憲
かいげん #T35 戒厳 #T30 改元
かいげんれい #T35*1 戒厳令
かいこ #T30*1 解雇 回顧 #T30 懐古 #T35 蚕 #M5 買い込
かいこう #T30*1 開口 開港 邂逅 #T30 回航 #T35 海港 海溝 #T30 開校 #JNS 開高
かいこん #T35 悔恨 #T30 開墾
かいご #T30*3 介護
かいごう #T30*7 会合
かいさ #T35 階差
かいさい #T30*39 開催
かいさく #T30*1 改作
かいさつ #T30*17 改札
かいさん #T30*4 解散
かいさんぶつ #T35*1 海産物
かいざい #T30*1 介在
かいざん #T35 海山
かいし #T30*125 開始 #T35 会誌
かいしつ #T30 開室
かいしめ #KSr 買い占め #T30 買占め
かいしゃ #T35*144 会社
かいしゃく #T30*82 解釈 #T30 介錯
かいしゅう #T30*14 回収 #T30*2 改修
かいしゅうごう #T35 開集合
かいしょ #T35 会所 楷書
かいしょう #T30*9 解消 #T35*1 甲斐性 #T30 改称
かいしょく #T30*1 会食 #T30 解職
かいしん #T30*1 回診 改新 #T35 会心 #T30 改心
かいじ #T30*2 開示 #T35 快事
かいじゅう #T35*1 怪獣 #T30 懐柔 #T05 晦渋
かいじょ #T30*16 解除
かいじょう #T35*31 会場 #T30*2 階乗 #T30*1 開場 #T35 階上
かいじり #JNS 海尻
かいすい #T35*1 海水
かいすいよく #T35*3 海水浴
かいすう #T35*49 回数 #T35*6 階数
かいせ #JNS 貝瀬
かいせい #T30*52 改正 #T30*8 改姓 #JS*6 回生 #T35*2 快晴 #T30 回生 #JCN 開成
かいせき #T30*107 解析 #T30*1 会席 #T35 懐石
かいせつ #T30*46 解説 #T30*2 開設 #T30*1 回折
かいせん #T35*91 回線 #T35*1 会戦 #JS*1 回戦 #T30 回旋 #JS 回線 #T30 改選 開戦
かいぜん #T30*52 改善
かいそ #T30*1 改組
かいそう #JS*48 階層 #T35*37 階層 #T30*2 回想 #T30*1 回送 改装 #T30 会葬 快走 #JCN 海草 #T35 海藻
かいそく #T05*9 快速 #T35*1 会則
かいぞう #T30*128 改造
かいぞうど #T35*2 解像度
かいぞく #T35*2 海賊
かいた #JCN 頴田 海田
かいたい #T30*2 解体 #T30 懐胎
かいたく #T30*1 開拓
かいだ #JCN 開田
かいだい #T35 解題 #T30 改題
かいだく #T30 快諾
かいだめ #T30*2 買い貯め
かいだん #T35*5 階段 #T30*2 会談 #T35 怪談
かいち #JN 嘉一
かいちく #T30 改築
かいちゅう #T30*2 懐中 #T35 海中
かいちょう #T35*10 会長 #T05*6 快調 #T35 階調
かいつう #T30*2 開通
かいつけ #T30 買付け
かいづ #JCN 海津
かいづか #JCN 貝塚 #T35 貝塚
かいて #T35*4 買い手
かいてい #T30*13 改定 #T30*9 改訂 #T35*1 海底 #T30 開廷 #T35 階悌
かいてき #T05*21 快適
かいてん #JS*6 回転 #T30*5 回転 #T30*4 開店
かいでん #T30 皆伝
かいと #R5r*1 買い取
かいとう #T30*177 回答 #T30*8 解凍 解答 #T35*1 会頭 #T30*1 解糖
かいとうしゃ #T35 回答者
かいどう #T35*2 街道 #CNSUC1*1 街道 #T35 会堂
かいどく #T30*2 解読 #T05*1 買い得
かいなら #S5 飼い慣ら
かいなん #CN 海南 #T35 海難
かいにゅう #T30*1 介入
かいにん #T30 解任 懐妊
かいぬし #T35 飼い主
かいね #T35 買値
かいはい #T30 改廃
かいはくしょく #T15 灰白色
かいはつ #T30*255 開発
かいはつぶ #T35 開発部
かいはん #T30*16 改版
かいばつ #T35 海抜
かいばら #JCN 柏原
かいひ #T30*52 回避 #T35*16 会費
かいひょう #T30*1 開票
かいひん #T35*10 海浜
かいび #T05 快美
かいふ #T30 回付 #JCN 海部
かいふう #T30*1 開封
かいふく #T30*9 回復 #T30 快復
かいぶつ #T35*4 怪物
かいぶんしょ #T35 怪文書
かいへい #T30*2 開閉 #T30 開平
かいへん #T30*4 改編 #T30*2 改変
かいほう #T30*32 開放 #T30*20 解放 #T35*10 解法 #T35*2 会報 #T30 介抱 #T35 快方
かいぼう #T30*1 解剖
かいまく #T30*1 開幕
かいまみ #KS かいま見
かいみん #T30 快眠
かいむ #T05*1 皆無 #T35 会務
かいめい #T30*5 解明 #T30*1 改名 #T05 開明
かいめつ #T30*2 壊滅
かいめん #T35*1 界面 #T35 海棉 海面
かいもく #F14 皆目
かいもど #S5*1 買い戻
かいもの #T30*24 買物 #T30*10 買いもの
かいもん #JCN 開聞 #T30 開門
かいもんだけ #CN 開聞岳
かいやく #T30*1 解約 #T30 改訳
かいゆ #T30 快癒
かいゆう #T30*1 回遊
かいよう #T35*8 潰瘍 #T35*1 海洋
かいらい #T35 傀儡
かいらく #T35 快楽
かいらん #T30*51 回覧 #T35*1 介卵
かいり #T30*1 解離 乖離 #JS カイリ 海里 浬
かいりく #T35 海陸
かいりゅう #T35 海流
かいりょう #T30*29 改良
かいれい #T35 海嶺
かいろ #T35*53 回路 #CN カイロ #T35 懐炉 海路
かいろう #T35*1 回廊
かいわ #T30*33 会話
かいわい #T35*1 界隈
かいん #T35 下院
かうんた #T35*18 カウンタ
かうんと #T30*11 カウント
かえ #R5r*255 帰 #KS*255 変え #R5r*255 返 #S5*255 返 #S5*216 帰 #KSr*74 替え #KSr*38 換え #KS*3 かえ #KSr*2 代え #JNM 華絵
かえだま #T35 替玉
かえって #F14*11 かえって #F14*2 却って
かえで #T35 楓
かえりざ #K5r 返り咲
かえりざき #T30 返咲き
かえりみ #KS 顧み 省み
かえる #T35*9 蛙
かえん #T35*1 火炎 火焔
かお #T35*51 顔 #R5r*5 香 #R5*3 薫
かおあわせ #T30*3 顔合せ
かおいろ #T35*1 顔色
かおう #KK*4 花王 #JCN 鹿央
かおく #T35*1 家屋
かおす #T35*1 カオス
かおだち #T35 顔立ち
かおぶれ #T35*2 顔ぶれ
かおみしり #T35*1 顔見知り
かおむけ #T30 顔向け
かおやく #T35 顔役
かおり #JNM 薫理 香織
かおる #JNM かおる 馨 薫 香
かおん #T30*2 加温
かおんばん #T35*1 加温盤
かか #R5*113 かか #R5*42 掛か #R5*10 架か #R5*1 係 #S5*1 欠か #R5 懸か
かかい #T35 下界
かかえ #KS*18 抱え
かかく #T35*37 価格
かかげ #KS*2 掲げ
かかぢ #CN 香々地
かかと #T35 踵
かかみがはら #CN 各務原
かかり #T35*14 係
かかりいん #T35*1 係員
かかりかん #T35 係官
かかりちょう #T35*2 係長
かかる #RT かかる
かかわ #R5r*22 かかわ #R5r*4 係わ #R5r*2 関わ #R5*2 拘 #R5r 関
かかん #T05*4 可換 #T05 果敢
かが #JCN 加賀 #M5 屈
かがい #T35*2 加害 #T35 課外
かがく #T30*71 科学 #T35*30 化学 #T35*1 下顎
かがみ #T35*7 鏡 #JCN 鏡 香我美
かがみいし #JCN 鏡石
かがみの #JCN 鏡野
かがや #K5r*4 輝 #JCN 加賀谷
かがやか #S5 輝か
かがやかし #KY 輝かし
かがやかしゅう #KYU 輝かしゅう
かがわ #JCN 香川
かき #T35*47 下記 #T35*7 垣 #T35*6 柿 #T35*2 花器 #T35*1 牡蛎 蛎 #T35 夏期 夏季 火気 花卉 堵
かきあげ #KS*3 書き上げ #KSr*2 かき揚げ
かきあつめ #KS*1 かき集め
かきあらわ #S5*1 書き表
かきいれ #KSr*1 書き入れ #KSr 掻き入れ
かきいれどき #T35 掻き入れ時
かきおき #T30 書置
かきおと #S5r 書き落と
かきおろ #S5r 書き下ろ
かきかえ #KSr*49 書き換え #KSr*30 書換え
かきかた #T35*40 書き方
かきくだ #S5*1 書き下
かきけ #S5*1 かき消
かきこ #M5r*161 書き込
かきこみけん #T35 書き込み権
かきざき #JCN 柿崎
かきそえ #KSr 書き添え
かきぞめ #T35 書初め
かきたて #KS 書き立て
かきだ #S5r*14 書き出
かきつけ #KSr 書き付け #T35 書付け
かきつばた #T35 カキツバタ 杜若
かきつらね #KSr 書き連ね
かきとめ #KS*1 書き留め #T35 書留
かきとり #T35 書取
かきぬ #K5r 書き抜
かきぬま #JNS 柿沼
かきね #T35*3 垣根
かきのき #JCN 柿木
かきのこ #S5r 書き残
かきまぜ #KSr かき混ぜ
かきまわ #S5r*1 かき回
かきみだ #S5 かき乱
かきゅう #T35*2 下級 #T15*1 火急
かきょう #T35*3 家教 #T35*1 佳境 #T30 架橋 #T35 華僑
かきょく #T35 歌曲
かきよせ #KS かき寄せ
かきわき #JNS 垣脇
かきわけ #KSr かき分け
かきん #T30*8 課金 #T35*1 家禽
かぎ #R5r*130 限 #T35*17 鍵 #R5r*1 かぎ #T35*1 かぎ カギ #T35 鈎
かぎかっこ #T35*7 』 #T35*6 『 #T35*2 「 」 #T35 鈎括弧
かぎゃく #T15*1 可逆 #T35 加虐 #T05 苛虐
かぎょう #T35*3 稼業 #T35 家業
かぎりな #KY*2 限りな
かぎりのう #KYU*1 限りのう
かく #PRE*231 各 #T35*224 各 #S5*37 隠 #T35*31 角 #T35*10 核 #KJ*9 確 #S5*5 画 #T30*5 画 #T35*5 格 #KJ*3 隔 #F14*1 かく #JS*1 画 #KJ*1 劃 獲 覚 較 郭 閣 鶴 #KJ 嚇 廓 拡 撹 #SUC 格 #T35 殻 #F12 確 #KJ 穫 #JS 角 #KJ 赫 革 掴 塙
かくあげ #T30 格上げ
かくい #T35*18 各位
かくいつてき #T05*1 画一的
かくいん #T35*1 各員 #T35 客員
かくう #T35*1 架空
かくえき #T35*3 各駅
かくか #T35*1 角化 #T35 各課
かくかい #T35 各階
かくかぞく #T35 核家族
かくぎ #T35*1 閣議
かくけん #T35 各県
かくげつ #T35*1 隔月
かくげん #T35*1 格言 #T30*1 確言
かくこう #T35 各校 各項
かくご #T30*5 覚悟
かくさ #T35*2 格差
かくさく #T30*1 画策
かくさげ #T30 格下げ
かくさん #T30*2 拡散 #T35 核酸
かくざい #T35 角材
かくざとう #T35 角砂糖
かくし #T35 各市 各紙
かくしき #T35*1 格式
かくしつ #T35 各室 #T30 確執 #T35 角質
かくして #CJ*2 かくして
かくしゃ #T35*28 各社
かくしゃく #F00 矍鑠
かくしゅ #T35*64 各種
かくしゅう #T35*2 各週 隔週 #T30 確執
かくしょ #T35*5 各所 #T35 各署 #JNS 角所
かくしょう #T30*3 確証
かくしょうちょう #T35 各省庁
かくしん #T30*4 確信 #T30*2 革新 #T35*1 核心
かくじ #T35*33 各自
かくじっけん #T30 核実験
かくじつ #T05*34 確実 #T35 隔日
かくじゅう #T30*2 拡充
かくじょし #T35 格助詞
かくじん #T35*4 各人 #T35*1 核仁
かくすう #T35*11 画数
かくせい #T30 拡声 覚醒 #T35 隔世
かくせいざい #T35 覚醒剤
かくせん #T35 各線
かくせんそう #T35 核戦争
かくぜつ #T30 隔絶
かくぜん #F00 画然
かくそう #T35*23 各層
かくた #JCN 角田
かくだ #JCN 角田
かくだい #T30*42 拡大
かくだん #T35*3 格段
かくち #T35*1 各地
かくちゅう #T35*1 角柱
かくちょう #T30*255 拡張 #T35*1 格調
かくちょうし #T35 拡張子
かくつき #T35 各月
かくづけ #T30 格付け
かくてい #T30*228 確定
かくてん #T35 各店 各点
かくとう #T30*5 格闘 #T35 各党 #T30 確答
かくとく #T30*8 獲得
かくど #T35*13 角度 #T35*1 確度
かくない #T35*1 核内 #T35 閣内
かくにん #T30*255 確認
かくねん #T35 隔年
かくのう #T30*88 格納
かくのだて #JCN 角館
かくは #T35 各派
かくはん #T30*2 撹拌 #T35 各般
かくぶ #T35*6 各部
かくぶんれつ #T30 核分裂
かくへいき #T35*1 核兵器
かくへき #T35 隔壁
かくべつ #T06*2 格別
かくほ #T30*79 確保
かくほう #T35*1 確報
かくまく #T35 角膜
かくめい #T35*1 革命
かくめん #T35*1 各面
かくやく #T30*1 確約
かくやす #T05*4 格安
かくゆうごう #T30 核融合
かくらん #T30 撹乱
かくり #T30*1 隔離
かくりつ #T30*133 確立 #T35*62 確率
かくりょう #T35*1 閣僚
かくれ #KS*5 隠れ #KS*3 かくれ
かくれが #T35 隠れ家
かくれんぼ #T30*1 かくれんぼ
かくろん #T35*1 各論
かぐ #T35 家具
かぐら #T35*1 神楽
かけ #KS*116 かけ #KS*49 掛け #KSr*2 欠け 賭け #KS*2 翔け #KS*1 架け #KJ*1 掛 #KS*1 駆け 懸け #JCN 加計 #KS 駈け #T35 賭
かけあ #W5r 掛け合
かけあし #T10 駆け足
かけあわ #S5r 掛け合わ
かけい #T35*4 家系 #T35*2 家計 #KJ*2 筧 #JNS 筧
かけいぼ #T35 家計簿
かけおち #T30*1 駆け落ち
かけがわ #JCN 掛川
かけきん #T35 掛金
かけこ #M5r*1 駆け込
かけごえ #T35*1 かけ声
かけごと #T35 賭事
かけざん #T30*1 かけ算 #T30 掛け算 掛算
かけじく #T35 掛軸
かけず #T35 掛図
かけずりまわ #R5*1 駆けずりまわ 駆けずり回
かけだ #S5r 駆け出
かけつ #T30*1 可決
かけつけ #KSr*2 駆けつけ
かけぬけ #KS*1 駆け抜け
かけね #T35 掛け値
かけのぼ #R5r 駆け登
かけはし #T35 架け橋
かけはなれ #KSr*1 かけ離れ
かけひき #T30*2 駆け引き #T30 駆引き
かけへだて #KSr かけ隔て
かけまわ #R5 駆け回
かけも #T5r 掛け持
かけもち #T30 掛持ち
かけや #JCN 掛合
かけよ #R5 駆け寄
かけら #T35*1 かけら
かける #T35*24 ×
かけん #KK*2 科研
かけんひ #KK*2 科研費
かげ #T35*23 影 #T35*2 蔭 陰 #T35*1 かげ #JNS 鹿毛
かげえ #T35 影絵
かげき #T05*4 過激 #T35 歌劇
かげぐち #T30*1 陰口
かげつ #JS*19 ヵ月 #JS か月 カ月 ヶ月
かげぼし #T30 陰干し
かげやま #JNS 影山
かげり #T35*1 かげり
かげろう #T35*1 陽炎
かげん #T30*8 加減 #T35*4 下限 #T35 下弦
かげんじょうじょ #T30 加減乗除
かげんち #T35 下限値
かこ #M5r*48 囲 #W5r*45 囲 #T35*27 過去 #T5 かこ #JCN 加古
かこう #T30*5 加工 #T35*1 河口 火口 #T30 下降
かこうがん #T35 花崗岩
かこうせん #T35 下降線
かこがわ #JCN 加古川
かこく #T05*1 苛酷 #JS カ国 ヶ国 #T05 過酷
かこん #T35 禍根
かご #T35*4 篭 籠 #KJ*1 籠 #T35 加護
かごう #T30 化合
かごうぶつ #T35 化合物
かごしま #CN 鹿児島
かごたに #JNS 篭谷 籠谷
かごめ #KK*1 カゴメ
かごん #T35*1 過言
かさ #T35*8 傘 #T35*1 笠 嵩 #JCN 加佐
かさい #T35*2 家裁 #T35*1 火災 #JCN 加西 #JNS 河西 #JCN 笠井 葛西
かさいりゅう #T35 火砕流
かさおか #JCN 笠岡
かさかけ #JCN 笠懸
かさぎ #JCN 笠置
かさく #T35*1 佳作 #T35 家作 #T05 寡作
かささ #JCN 笠沙
かさな #R5r*20 重な
かさね #KS*13 重ね
かさねて #F14 重ねて
かさはら #JCN 笠原
かさま #JCN 笠間
かさまつ #JCN 笠松
かさみ #JN 嵩
かさり #JCN 笠利
かさん #T30*16 加算 #T15 可算
かざ #R5r*17 飾
かざい #T35 家財
かざかみ #T35*1 風上
かざぐるま #T35 風車
かざしも #T35*1 風下
かざま #T35*1 風間 #JNS 風間
かざまうら #CN 風間浦
かざみ #T35 風見
かざみどり #T35 風見鶏
かざむき #T35 風向き
かざりつけ #KSr*1 飾り付け
かざわ #JCN 鹿沢
かざん #T35*1 火山
かざんばい #T35 火山灰
かし #T35*12 可視 #T35*9 菓子 #T35*7 歌詞 #T35*2 瑕疵 #T35*1 樫 #T35 仮死 河岸 華氏 橿 #G5 傾 #G5r 炊 #T35 貸し
かしお #KK*1 カシオ
かしおか #JNS 柏岡
かしかた #T35*1 貸方
かしかん #T35 下士官
かしきり #T30 貸切り
かしきん #T35*2 貸し金
かしげ #KS*1 かしげ #KS 傾げ
かしこ #KY*7 賢 #S5r 貸し越
かしこう #KYU*1 賢う
かしこまる #R5r 畏ま
かしだ #S5r*26 貸し出
かしちん #T35 貸賃
かしつ #T30*2 加湿 #T35*1 過失
かしつけ #KSr 貸し付け
かしはら #JCN 橿原
かしば #JCN 香芝
かしほん #T35 貸本
かしま #JCN 嘉島 鹿島 #JNS 鹿嶋 #T35 貸間
かしまし #KY 姦し
かしましゅう #KYU 姦しゅう
かしまだい #CN 鹿島台
かしも #CN 加子母
かしゃ #T35 貨車
かしゃく #T35 苛責
かしや #T35 貸家
かしゅ #T35*1 歌手
かしゅう #T35*1 歌集
かしょ #JS*50 箇所 #T35*25 箇所 #T35*1 個所 #JS ヶ所
かしょう #T35*6 仮称 #T05*1 過小 #T30 歌唱 #T05 過少
かしょく #T35 華燭 #T30 過食
かしょくしょう #T35 過食症
かしょくのてん #T35 華燭の典
かしら #CJ*19 かしら #T35*3 頭
かしわ #T35*5 柏 #T35 栢 #JCN 柏
かしわぎ #JCN 柏木
かしわざき #JCN 柏崎
かしわばら #JCN 柏原
かしわもち #T35 柏餅
かしわら #JCN 柏原
かしん #T30*1 過信 #T35 家臣
かじ #T35*13 舵 #T35*4 家事 #R5r*3 かじ #T35*1 火事 梶 #JNS 梶 #T35 柁 鍛冶
かじい #JNS 梶井
かじうら #JNS 梶浦
かじか #T35*1 鮖 #M5r かじか #T35 鰍
かじかざわ #JCN 鰍沢
かじかわ #JNS 加冶川
かじき #JCN 加治木
かじたに #JNS 梶谷
かじつ #T35*1 過日 #T35 佳日 果実
かじはら #JNS 梶原
かじま #JCN 鹿島 #JNS 鹿嶋
かじもと #JNS 梶本
かじゅ #T35 果樹
かじゅう #T30*1 加重 #T35 果汁 荷重 #T05 過重
かじょ #T30 加除
かじょう #T05*2 過剰 #T35*1 箇条 #JS か条 ヶ条 箇条
かじょうがき #T30*2 箇条書
かじわら #JCN 梶原
かじん #T35*1 歌人
かじんはくめい #T35*1 佳人薄命
かす #T35*8 カス #M5r*1 霞 #JN カス #T35 糟 粕 #R5r 掠
かすい #T30 下垂
かすいぶんかい #T30*1 加水分解
かすう #T35*3 仮数
かすか #T05*3 かすか #T05*1 微か
かすかべ #JCN 春日部
かすかわ #JCN 粕川
かすが #T35*3 春日 #JCN 春日
かすがい #JCN 春日井 春日居
かすがのつぼね #JN 春日局
かすけーど #T05*1 カスケード
かすたまいず #T30*255 カスタマイズ
かすたむ #T35*4 カスタム
かすたむか #T30*2 カスタム化
かすてら #T35*1 カステラ
かすみ #T35*3 霞 #JCN 香住
かすみがうら #CN 霞ヶ浦
かすみがせき #CN 霞ヶ関
かすや #JCN 粕屋 #JNS 粕谷
かず #T35*162 数 #KJ*15 和 #T35*4 下図
かずあき #JNM 和明
かずえ #JNM 和恵 和枝
かずお #JNM 一男 一夫 一雄 和男 和夫 和雄
かずおお #KY*1 数多
かずおおく #T35*4 数多く
かずおみ #JNM 和臣
かずかず #T35*3 数々
かずき #JNM 一樹
かずこ #JNM 量子 和子
かずさ #CN 上総
かずし #JNM 一司
かずしげ #JNM 和茂
かずすくな #KY 数少な
かずすくのう #KYU 数少のう
かずとし #JNM 和利
かずのこ #T35*1 鯑
かずのり #JNM 一憲 和徳
かずひこ #JNM 一彦 #JN 和比古 #JNM 和彦
かずひさ #JNM 一久 一寿 和久
かずひで #JNM 和秀
かずひと #JNM 一仁 和仁
かずひろ #JNM 一宏 一弘 一博 和広 和弘 和裕
かずま #JNM 和馬
かずまさ #JNM 一正 和正
かずみ #JNM 一三 一美 和美
かずみち #JNM 一路
かずや #JNM 一也 数也 和也 和弥
かずゆき #JNM 一行 和幸 和之
かずよし #JNM 一義 和義 和美
かずら #T35 葛
かせ #G5r*5 稼 #T35*3 枷 #JNS 加瀬
かせい #T35*6 家政 #T35*4 化成 #KK*2 火星 #T35 仮性 #T30 加勢
かせき #T35*1 化石
かせだ #JCN 加世田
かせっと #T35*2 カセット
かせつ #T35*6 仮説 #T30*1 架設 #T30 仮設
かせん #T35*10 下線 #T30*4 寡占 #T35*3 河川 #T35 化繊 #T30 架線
かぜ #T35*33 風邪 #T35*27 風
かぜい #T30 課税
かぜいん #T35*1 カゼイン
かぜむき #T35 風向き
かそ #T35*2 過疎
かそう #T30*59 仮想 #T35*2 下層 #T30 仮装 火葬
かそうきおく #T35*10 仮想記憶
かそく #T30*2 加速
かそくど #T35*1 加速度
かぞ #JCN 加須
かぞう #T30 家蔵
かぞえ #KSr*10 数え
かぞく #T35*17 家族 #T35 華族
かぞくづれ #T35*2 家族連れ
かた #T35*227 型 方 #D2T35*144 方 #T35*29 肩 #R5r*4 語 #KY*4 硬 #KY*3 難 #KY*2 堅 #T35*1 形 #KYmi*1 固 #T15 かた 過多 #T35 潟 #PRE 片 #T35 片 #JNSUC 方
かたあし #T35 片足
かたいじ #T18 片意地
かたいれ #T30 肩入れ
かたうで #T35 片腕
かたおか #JCN 片岡
かたおもい #T30*13 片想い #T30 片思い
かたかな #T35*64 カタカナ #T35*4 片仮名
かたがき #T35*1 肩書
かたがた #T35*41 方々 #CJ*18 方々
かたがみ #T35 型紙
かたがわ #T35*2 片側
かたがわり #T30*2 肩代り
かたきうち #T30 敵討ち
かたぎ #T05*1 堅気 #T35 気質
かたぎり #JCN 片桐
かたく #T30 仮託 #T35 家宅
かたくずれ #T30 型くずれ
かたくな #T05 かたくな
かたくり #T35 片栗
かたくりこ #T35 片栗粉
かたくるし #KY*2 堅苦し
かたくるしゅう #KYU*1 堅苦しゅう
かたこと #T35*1 片言
かたこり #T35*1 肩こり
かたしな #JCN 片品
かたじけな #KY かたじけな
かたじけのう #KYU かたじけのう
かたすみ #T35*1 片隅
かたず #T35 固唾
かたた #JCN 堅田
かたち #T35*106 形
かたちづく #R5r*1 形作
かたっぱし #T35 片っ端
かたつ #T35*1 可達
かたづ #K5*5 片付
かたづけ #KSr*1 片付け
かたて #T35*2 片手
かたてま #T35 片手間
かたとき #T35 片時
かたど #R5r 型ど #R5 象
かたどおり #T05 型どおり
かたな #T35*2 刀
かたの #JCN 交野 #JNS 片野
かたはし #T35 片端
かたはば #T35 肩幅
かたばん #T35*3 型番
かたひがし #JCN 潟東
かたぶつ #T15 堅物
かたほう #T35*5 片方
かたま #R5r*12 固ま
かたまち #T35 片町
かたまり #T35*2 塊
かたみ #T35*1 肩身 #T35 形見 片身
かたみち #T35*6 片道
かたむ #K5r*3 傾
かため #KSr*2 かため 固め #T35*1 片目 #T35 堅め
かためん #T35*1 片面
かたやぶり #T05*1 型破り
かたやま #JNS 片山
かたやまづ #JNS 片山津
かたよ #R5r*2 偏
かたら #W5r 語ら
かたり #T35 語り
かたりあ #W5r 語り合
かたりつ #G5 語り継
かたろぐ #T35*7 カタログ
かたわら #T35 傍ら
かたん #T35*2 下端 #T35*1 加胆 #T30*1 荷担
かだい #T35*48 課題 #T05*1 過大
かだん #T35*1 歌壇 花壇
かち #T35*31 価値
かちあ #W5r*2 かち合
かちかん #T35*4 価値観
かちき #T05 勝気
かちく #T35*1 家畜
かちこ #S5r 勝ち越
かちこし #T30 勝ち越し
かちすす #M5 勝ち進
かちと #R5 勝ち取
かちぬ #K5r*1 勝ち抜
かちぼし #T35 勝ち星
かちまけ #T35 勝ち負け
かちめ #T35 勝ち目
かちゅう #T35 渦中
かちょう #T35*53 課長 #T05*3 可聴
かちょうきん #T35 課徴金
かちり #F14*1 かちり
かちん #F06*1 カチン
かっ #S5*10 渇 #T30*10 渇
かっか #T35*1 角化 閣下
かっかい #T35 各回 各界
かっかそうよう #T35 隔靴掻痒
かっき #T35*1 活気
かっきてき #T05*5 画期的
かっきゅう #T35*1 割球
かっきょ #T30 割拠
かっきょう #T35 活況
かっきょく #T35*1 各局
かっくう #T30 滑空
かっけつ #T30*1 喀血
かっこ #T35*51 【 #T35*49 】 #T35*29 括弧 #T35*18 カッコ #T35*13 （ ） #T35*6 〔 #T35*5 〕 ］ #T35*3 ［ #T35*1 ｛ ｝ 〈 〉 《 》 各個 #F02 確固
かっこう #T05*5 恰好 #T35*3 格好 #T35 カッコウ 郭公 #T30 滑降
かっこく #T35*8 各国
かっさい #T30*1 喝采 #JCN 葛西
かっしゃ #T35 滑車
かっしょく #T35 褐色
かっすい #T30 渇水
かっせい #T35*16 活性
かっせん #T30*2 合戦
かっそう #T30 滑走
かっそうろ #T35*1 滑走路
かった #JCN 刈田
かったー #T35*2 カッター
かったつ #T05 闊達
かって #T05*126 勝手 #T05*2 かって
かっと #T30*21 カット
かっとう #T35*3 葛藤
かっとば #S5 かっ飛ば
かっぱ #T35*1 Κ κ 河童 #T30 喝破
かっぱつ #T05*6 活発
かっぱら #W5r かっぱら
かっぱん #T35 活版
かっぷ #T30*4 割賦
かっぷく #T30 割腹 #T35 恰幅
かっぷる #T35*5 カップル
かっぽう #T35*1 割烹
かつ #F14*22 かつ #KJ*10 喝 #KJ*7 活 #KJ*3 克 #KJ*2 葛 #KJ*1 滑 猾 #KJ 割 恰 括 #T35 活 #T30 渇 #KJ 褐 轄 且 #F14 且つ #G5r 担
かつあい #T30*2 割愛
かつあき #JNM 勝昭
かついち #JNM 勝一
かつうら #JCN 勝浦
かつえ #KSr 渇え
かつお #T35 鰹 #JNM 勝男 勝夫 勝雄
かつおぶし #T35 鰹節
かつぎこ #M5r かつぎ込
かつこ #JNM 克子 勝子
かつしか #JCN 葛飾
かつじ #T35*4 活字 #JNM 勝次 勝治
かつじょう #T30 割譲
かつぞう #JNM 勝三
かつた #JCN 勝田
かつたろう #JNM 勝太郎
かつて #F14*4 かつて #T36 かつて
かつとし #JNM 勝利
かつどう #T30*47 活動
かつぬま #JCN 勝沼
かつひこ #JNM 克彦 勝彦
かつひさ #JNM 勝久
かつひろ #JNM 克宏 克洋
かつぶし #T35 鰹節
かつぼう #T30 渇望
かつまた #JNS 勝又
かつみ #JNM 克己 克美 克巳 勝美 勝巳
かつもと #JNS 勝本
かつや #JNM 克哉 勝哉 勝也
かつやく #T30*9 活躍
かつやま #JCN 勝山
かつよう #T30*58 活用
かつよし #JNM 勝義 勝吉
かつら #JCN 桂 #T35 桂
かつらお #JCN 葛尾
かつらぎ #CN かつらぎ #JCN 葛城
かつりょく #T35 活力
かつれん #JCN 勝連
かつろ #T35 活路
かづけ #JS 日付
かづさ #JCN 加津佐
かづの #JCN 鹿角
かて #T35*2 糧
かてい #T30*85 仮定 #T35*52 家庭 #T35*32 過程 #T35*4 課程
かていか #T35*1 家庭科
かてごり #T35*5 カテゴリ
かててくわえて #CJ かてて加えて
かでな #CN 嘉手納
かでん #T35*1 家電 #T35 瓜田 家伝 #T30 架電 #T35 荷電
かと #T35*3 過渡
かとう #T30*1 加糖 #KYU*1 堅う 固う 硬う 難う #T05 下等 #JCN 加東 #JNS 加藤 #T35 果糖 #JCN 河東 #T05 過当
かとき #T35*2 過渡期
かとく #T35 家督
かとくせい #T35*1 可読性
かとり #JCN 香取
かとりっく #T35*1 カトリック
かど #T35*10 角 #T05*3 過度 #T35 門
かどう #T30*38 稼働 #T35*2 華道
かどかわ #JNS 角川
かどがわ #JCN 門川
かどく #T05*2 可読
かどぐち #T35 門口
かどち #T35 角地
かどで #JNS 門出 #T30 門出 #T35 門出
かどま #JCN 門真
かどまつ #T35 門松
かどみうむ #T35*1 カドミウム
かどわき #JNS 門脇
かな #T35*255 かな #T35*104 仮名 #T35*23 カナ #KJ*9 金 #W5*6 かな #KJ*4 哉 #W5*1 叶 #JN 加奈 #W5 敵 適
かなあみ #T35 金網
かない #T35*2 家内 #T35 課内 #JCN 金井
かなう #KJ 叶
かなえ #T35*255 鼎
かながき #T30*1 カナ書き #T30 仮名書き
かながわ #CN 神奈川
かなぎ #JCN 金城 金木
かなぐ #T35*1 金具
かなこ #JNM 佳那子
かなさき #JNS 金崎
かなさごう #CN 金砂郷
かなざわ #JCN 金沢
かなし #KYmi*42 悲し #M5*27 悲し #KYmi*3 かなし 哀し #M5*2 哀し
かなしゅう #KYU*2 悲しゅう #KYU*1 かなしゅう 哀しゅう
かなた #T35*2 彼方
かなだ #CN カナダ #JCN 金田
かなづ #JCN 金津
かなで #KS 奏で
かなまる #JCN 金丸
かなめ #T35*3 要 #JN 要
かなもの #T35 金物
かなもり #JNS 金森
かなや #JCN 金屋 金谷
かなやま #JCN 金山
かならず #F14*125 必ず
かならずしも #F14*20 必ずしも
かならずや #F14 必ずや
かなり #T06*121 かなり
かなん #CN 河南
かに #T35*2 カニ #T35*1 蟹 #JCN 可児
かにえ #JCN 蟹江
かにく #T35 果肉
かにざ #KK*1 蟹座
かにた #JCN 蟹田
かにゅう #T30*24 加入
かぬま #JCN 鹿沼
かね #T35*30 金 #KS*24 兼ね #T35*1 鐘 #JNM かね
かねあい #T35*2 兼ね合い
かねがさき #CN 金ヶ崎
かねがね #F14*1 かねがね
かねきち #JN 兼吉
かねこ #JCN 金子
かねじゃく #T35 曲尺
かねそなえ #KS*2 兼ね備え
かねだ #JCN 金田
かねつ #T30*2 加熱 #T30*1 過熱
かねづかい #T35 金遣い
かねまる #JCN 金丸
かねまわり #T35 金回り
かねみつ #JN 金光
かねもうけ #T30*7 金儲け
かねもち #T30*2 金持ち
かねやま #JCN 金山 兼山
かねん #JS か年 ヶ年
かの #JCN 鹿野
かのあし #JCN 鹿足
かのう #T05*255 可能 #T30*29 化膿 #JCN 加納 #JNS 狩野
かのうせい #T35*112 可能性
かのじょ #T35*65 彼女
かのせ #JCN 鹿瀬
かのや #JCN 鹿屋
かはいらん #T35*1 過排卵
かはん #T15*2 可搬 #T35*1 過半
かはんしん #T35 下半身
かはんすう #T35*1 過半数
かば #T35*2 樺 #KJ*1 椛 #T35*1 蒲 #W5*1 庇
かばー #T30*17 カバー
かばしら #T35 蚊柱
かばと #CN 樺戸
かばね #T35 屍 姓
かばやき #T35 蒲焼き
かばん #T35*1 鞄
かひ #T35*5 可否 #T35*1 瑕疵 痂皮
かひつ #T30*4 加筆
かひょう #T35*1 下表
かび #T35*1 カビ #T05 華美 #T35 黴
かびん #T35*3 花瓶 #T05 過敏
かふ #T30 下付 #T35 寡婦
かふう #T35 家風
かふぇ #T35 カフェ
かふぇいん #T35*1 カフェイン
かふぇてりあ #T35*2 カフェテリア
かふそく #T30*2 過不足
かふちょうせい #T35*1 家父長制
かふばん #T15 可付番
かふん #T35*1 花粉
かふんしょう #T15 花粉症
かぶ #JS*92 株 #T35*9 下部 #R5*6 被 #T35*1 株 #SUC 株 #T35 蕪
かぶか #T35*1 株価
かぶき #T35*2 歌舞伎
かぶけん #T35 株券
かぶしき #T35*7 株式
かぶしきがいしゃ #T35*28 株式会社
かぶせ #KS*2 かぶせ
かぶと #T35*1 甲 #T35 兜
かぶぬし #T35*1 株主
かぶら #T35 鏑 蕪
かぶん #T35*1 寡聞 #T05 過分
かぷしっど #T35*1 カプシッド
かぷせる #T35*5 カプセル
かへい #JCN 加平 #T35 貨幣
かへん #T05*11 可変
かべ #T35*8 壁
かべん #T35 花弁
かほ #JCN 嘉穂 #JN 果歩
かほう #T35*1 加法 家宝
かほく #CN 河北 香北 鹿北
かほご #T05 過保護
かぼちゃ #T35*1 かぼちゃ
かま #W5*58 構 #W5*20 かま #T35*3 釜 #T35*1 蒲 鎌 窯 #T35 竃
かまいし #CN 釜石
かまえ #JCN 蒲江 #KSr 構え
かまがや #CN 鎌ヶ谷
かまがり #JCN 蒲刈
かまきり #T35 カマキリ
かまくら #JCN 鎌倉
かまけ #KS かまけ
かます #T35*1 鰤
かまた #JCN 蒲田 鎌田
かまだ #JNS 鎌田
かまぼこ #T35 かまぼこ 蒲鉾
かまもと #T35 窯元
かみ #T35*64 紙 #T35*53 神 #T35*28 上 #T35*9 髪 #T30*1 加味 #T35*1 守 #JCN 加美 #JN 可美 #T35 祇 #JN 香美 #CNPRE 上 #JNS 上
かみあがた #CN 上県
かみいしづ #CN 上石津
かみいずみ #JCN 神泉
かみいそ #JCN 上磯
かみいた #JCN 上板
かみいち #CN 上市
かみいな #CN 上伊那
かみうけな #CN 上浮穴
かみうら #JCN 上浦
かみおか #JCN 神岡
かみかつ #JCN 上勝
かみかわ #JCN 上川 神川
かみかわち #CN 上河内
かみがた #T35*4 髪型 #T35*1 上方
かみき #T35*3 上期
かみきた #CN 上北
かみきたやま #CN 上北山
かみぎょう #CN 上京
かみくいしき #CN 上九一色
かみくだ #K5 噛み砕
かみこしき #JCN 上甑
かみごうり #JCN 上郡
かみごとう #CN 上五島
かみさと #CN 上郷 #JCN 上里
かみさま #T35*3 神様
かみざ #T35 上座
かみしひ #CN 上志比
かみしほろ #CN 上士幌
かみしめ #KS 噛みしめ
かみしろ #JNS 神城
かみじょう #JNS 上条 上條
かみす #CN 神栖
かみすながわ #CN 上砂川
かみそり #T35 剃刀
かみた #JNS 上田
かみたいら #JCN 上平
かみたかい #CN 上高井
かみたから #JCN 上宝
かみだな #T35 神棚
かみだのみ #T30 神頼み
かみつ #T05 過密
かみつが #CN 上都賀
かみつしま #CN 上対馬
かみて #T35*1 上手
かみとんだ #CN 上富田
かみなか #JCN 上中 #CN 上那賀
かみなり #T35*2 雷
かみにいかわ #CN 上新川
かみのかわ #CN 上三川
かみのくに #CN 上ノ国
かみのせき #JCN 上関
かみのほ #JCN 上之保
かみのやま #JCN 上山
かみはやし #JCN 神林
かみはんき #T35*1 上半期
かみふくおか #CN 上福岡
かみふらの #CN 上富良野
かみへい #CN 上閉伊
かみましき #CN 上益城
かみみね #JCN 上峰
かみみのち #CN 上水内
かみむら #JCN 上村
かみや #T35 紙屋 #JCN 神谷
かみやく #CN 上屋久
かみやちょう #CN 神谷町
かみやはぎ #JCN 上矢作
かみやま #JCN 神山
かみやまだ #JCN 上山田
かみゆうべつ #CN 上湧別
かみん #T30 仮眠 夏眠
かめ #T35*10 亀 #T35*1 瓶
かめい #T35*4 仮名 #T30*2 加盟 #T35 家名 #JNS 亀井
かめおか #JCN 亀岡
かめきち #JN 亀吉
かめじろう #JNM 亀次郎
かめたろう #JNM 亀太郎
かめだ #JCN 亀田
かめやま #JCN 亀山
かめら #T35*14 カメラ
かめん #T35*5 仮面
かも #T35*2 鴨 #JCN 加茂 賀茂 #S5 醸
かもい #T35 鴨居 #JCN 神威
かもう #JCN 蒲生
かもえない #CN 神恵内
かもがた #JCN 鴨方
かもがわ #CN 加茂川 賀茂川 #JCN 鴨川
かもく #JS*2 科目 #T35*2 科目 #T05 寡黙 #T35 課目
かもしだ #S5 醸し出
かもじま #JCN 鴨島
かもつ #T35*1 貨物
かもつせん #T35 貨物船
かもと #JCN 鹿本
かもみーる #T35*1 カモミール
かもめ #T35 鴎
かもん #T35*1 家紋
かや #T35*2 茅 #T35*1 蚊帳 #KJ*1 萱 #T35*1 萱 #JCN 加悦
かやく #T05*4 可約 #T35*1 火薬
かやべ #JCN 茅部
かやもり #JNS 萱森
かゆ #KYmi*2 かゆ #KYmi*1 痒 #T35 かゆ 粥
かゆう #KYU*1 かゆう 痒う
かよ #W5r*10 通 #JNM 佳代
かよう #T35*7 火曜 #T05*1 可溶 #T35*1 歌謡 #JCN 賀陽
かようきょく #T35*1 歌謡曲
かようせい #T35*4 可用性
かようび #T35*12 火曜日
から #T35*255 〜 #T35*141 空 #KJ*10 唐 #M5r*6 絡 #KYmi*3 辛 #T35*1 殻 #S5 枯ら
からー #T35*49 カラー
からおけ #T35*5 カラオケ
からかさ #T35 からかさ 傘
からかみ #T35 唐紙
からくち #T05 辛口
からくり #T35*1 からくり
からくわ #JCN 唐桑
からげ #R5 からげ
からさわ #JCN 唐沢
からし #T35 芥子
からす #T35*1 カラス 烏 鴉 #JCN 香良洲
からすま #JCN 烏丸
からすみ #T35 からすみ
からすやま #JCN 烏山
からたけわり #T30*2 幹竹割り
からだ #T35*46 体 #T35*8 身体 #T35*1 からだ #T35 躯
からっかぜ #T35 からっ風
からっぽ #T05 空っぽ
からつ #JCN 唐津
からて #T35*1 空手
からふと #CN 樺太
からぶり #T30 空振り
からま #R5r*1 絡ま
からまわり #T10 空回り
からみ #T35 空身
からみあ #W5r 絡み合
からみつ #K5r 絡み付
からむ #T35*16 カラム
からめ #KS 絡め
かり #KSr*68 借り #T35*34 仮 #T35*1 刈 雁 狩 #PRE 仮 #T35 苅 猟
かりあげ #KSr 刈り上げ 借り上げ
かりあつめ #KS かり集め
かりいれ #KSr 刈り入れ 借り入れ #T30 借入れ
かりいれきん #T35 借入金
かりうけ #KSr*1 借り受け
かりかた #T35*1 借方
かりき #R5r 借り切
かりきゅらむ #T35 カリキュラム
かりこ #M5r 刈り込 #S5 借り越
かりしゃくほう #T30 仮釈放
かりしょぶん #T30 仮処分
かりすま #T35*1 カリスマ
かりずまい #T30 仮住い
かりたて #KS*1 駆り立て
かりだ #S5 駆り出
かりちょういん #T30 仮調印
かりて #T35 借り手
かりと #R5r 刈り取
かりぬい #T30 仮縫い
かりばら #T35*1 借腹
かりばらいきん #T35*1 仮払金
かりふぉるにあ #CN カリフォルニア
かりめんきょ #T35 仮免許
かりや #JCN 刈谷 #JNS 苅屋
かりゅう #T35*1 下流 顆粒
かりょう #T30 加療 #T35 科料
かりょく #T35*6 火力
かりわ #JCN 刈羽
かる #KYmi*26 軽
かるいざわ #CN 軽井沢
かるう #KYU*2 軽う
かるがりー #CN カルガリー
かるがる #F06 軽々
かるがるし #KY*1 軽々し
かるがるしゅう #KYU*1 軽々しゅう
かるはずみ #T05 軽はずみ
かるぴす #KK*1 カルピス
かるふぉるにあ #CN カルフォルニア
かるまい #JCN 軽米
かるわざ #T35 軽業
かれ #T35*118 彼 #KS*2 枯れ #KS 渇れ
かれー #T35*6 カレー
かれい #T05*3 華麗 #T35*1 加齢 鰔 鰈
かれいど #T35*1 カレイド
かれき #T35 枯木
かれし #T35*2 彼氏
かれつ #T05*1 苛烈
かれは #T35 枯葉
かれら #T35*22 彼ら #T35 彼等
かれん #T05*3 可憐
かれんだー #T35*4 カレンダー
かれんと #T35*138 カレント
かろう #KYU*1 辛う #T35 家老
かろうじて #F14*1 かろうじて
かろやか #T05 軽やか
かろりー #JS*1 カロリー #T35*1 カロリー
かわ #R5*255 変わ #R5r*136 代わ #T35*65 川 #R5r*25 かわ #T35*20 河 #T35*9 側 #S5*7 交わ #R5r*6 代 #T35*4 皮 #R5*2 換わ 替わ 変 #S5*1 かわ #T35*1 革 #K5*1 乾 #K5 渇 #R5 替
かわい #KY*27 可愛 #KY*4 かわい #JNS 河井 #JCN 河合 川井 川合
かわいそう #T05*21 可哀想 #T05 可哀相
かわうえ #JCN 川上
かわうち #JCN 河内 川内
かわうら #JCN 河浦
かわか #S5r*1 乾か
かわかみ #JNS 河上 #JCN 川上 #T35 川上
かわきた #JCN 川北
かわくら #JNS 川倉
かわぐち #T35*11 川口 #T35*1 河口 #JCN 川口
かわぐちこ #CN 河口湖
かわぐつ #T35 革靴
かわげ #JCN 河芸
かわごえ #JCN 川越
かわさき #JCN 川崎
かわさと #JCN 川里
かわざんよう #T30 皮算用
かわしま #JNS 河島 #JCN 川島 #JNS 川嶋
かわしも #T35 川下
かわじま #JNS 川島
かわじり #JCN 川尻
かわず #T35 蛙
かわせ #T35*1 為替 #JNS 川瀬
かわそえ #JCN 川副
かわぞこ #T35 川底
かわた #JNS 河田
かわたな #JCN 川棚
かわだ #JNS 河田 川田
かわち #JCN 河内
かわちながの #CN 河内長野
かわづ #JNS 川津
かわな #JNS 川名
かわなべ #JCN 川辺
かわにし #JNS 河西 #JCN 川西
かわぬま #JCN 河沼
かわね #JCN 川根
かわの #JNS 河野 川野
かわのえ #CN 川之江
かわはら #JNS 河原 #JCN 川原
かわば #CN 川場
かわばた #T35*5 川端 #JCN 川端 #JNS 川畑
かわひがし #JCN 河東
かわべ #JCN 河辺 川辺
かわまた #JCN 川俣 #JNS 川又
かわみなみ #JCN 川南
かわむら #JNS 河村 川村
かわもと #JCN 川本
かわや #KJ*1 厠 #T35*1 厠
かわゆう #KYU*2 可愛う #KYU*1 かわゆう
かわら #T35*8 河原 #T35*2 瓦 #JCN 香春 #T35 川原
かわり #T35 変わり
かん #T35*255 完 #CNSUC1*170 間 #SX*142 関 #F09*11 冠 #T35*11 管 #KJ*7 漢 #T35*6 間 館 #T35*5 缶 #T35*4 勘 #KJ*4 官 #T35*4 感 #ZX*4 感 #KJ*3 刊 換 歓 #KJ*2 乾 勧 環 韓 舘 奸 #T35*1 寒 #KJ*1 寛 幹 慣 #T35*1 棺 #KJ*1 澗 看 簡 緩 翰 #T35*1 観 #ZX*1 観 #KJ*1 貫 閑 函 瞰 #KJ 侃 #JS 巻 #KJ 喚 堪 #T30 姦 #SUC 官 #KJ 干 患 #SUC 感 #KJ 憾 敢 柑 桓 款 汗 潅 甘 監 竿 #JS 缶 #KJ 肝 #T35 艦 #KJ 莞 #SUC 観 #KJ 諌 #JS 貫 #KJ 還 鑑 #JSSUC 間 #KJ 陥 #SUC 館 #T35 甲 #KJ 潤 神 #JNS 菅
かんあん #T35 勘案
かんい #T05*9 簡易
かんいち #JNM 貫一
かんいっぱつ #T35 間一髪
かんいん #T30 姦淫 #T35 館員
かんえつ #CN 関越
かんえん #T35*2 肝炎
かんおけ #T35*1 棺桶
かんおんじ #JCN 観音寺
かんか #T30*1 感化 #T30 看過
かんかい #T35 官界 #T30 寛解
かんかく #T35*36 間隔 #T30*12 感覚
かんかつ #T30*1 管轄
かんがい #T35*14 感慨 #T35 寒害 干害
かんがえ #KSr*255 考え
かんがえかた #T35*23 考え方
かんがえこ #M5*1 考え込
かんがえだ #S5*2 考え出
かんがえつ #K5r*2 考え付
かんがえつめ #KS 考え詰め
かんがえなお #S5r*3 考え直
かんがえぬ #K5 考え抜
かんがく #T35 官学
かんがみ #KS*2 鑑み
かんき #T35*1 寒気 #T30*1 喚起 換気 歓喜 #T35 乾期
かんきせん #T35*3 換気扇
かんきつ #T35 柑橘
かんきゃく #T35*2 観客
かんきゅう #T35 緩急
かんきょ #T30*26 官許 #T30 閑居
かんきょう #T35*255 環境 #T35 感興
かんきん #T30*1 換金 #T35*1 桿菌 #T30 監禁
かんぎゅうじゅうとう #T35 汗牛充棟
かんぎょう #T35*3 勧業 #T35 官業
かんく #JS 管区 #T35 管区
かんぐ #R5r 勘ぐ
かんけい #T30*255 関係
かんけいな #KY 関係な
かんけつ #T05*4 簡潔 #T30*2 完結 #T35*1 間欠
かんげい #T30*23 歓迎
かんげき #T30*3 感激 #T30*2 観劇 #T35*2 間隙
かんげん #T30*8 換言 #T35*1 甘言 #T30*1 諌言 還元
かんげんがく #T35 管弦楽
かんこ #T30 歓呼
かんこう #T30*9 観光 #T30*2 刊行 #T35*1 慣行 #T30 完工 感光 敢行 緩行
かんこうきゃく #T35 観光客
かんこうちょう #T35 官公庁
かんこうへん #T35 肝硬変
かんこうれい #T35*1 箝口令
かんこく #T30*9 勧告 #CN 韓国
かんこつだったい #T35*1 換骨奪胎
かんこどり #T35*1 閑古鳥
かんこんそうさい #T35*1 冠婚葬祭
かんご #T30*1 看護 #KK 漢語
かんごく #T35 監獄
かんごふ #T35*2 看護婦
かんさ #T30*2 監査 #T30*1 感作 #T30 鑑査
かんさい #T30 完済 #T35 簡裁 #CN 関西
かんさつ #T30*9 観察 #T30 監察 #T35 鑑札
かんさん #T30*4 換算 #F00*1 閑散
かんざい #T35 管財
かんざき #JCN 神崎 神埼
かんざん #T30 換算
かんし #T30*29 監視 #T35*1 鉗子
かんしき #T35 鑑識
かんしつ #T35*1 間質 #T35 乾湿
かんして #T35*90 関して
かんしゃ #T30*21 感謝 #T35 官舎
かんしゃく #T35 かんしゃく 癇癪
かんしゃくだま #T35 かんしゃく玉 癇癪玉
かんしゅ #T30*1 看取 #T35*1 肝腫 #T35 看守
かんしゅう #T35*16 慣習 #T30*1 監修 #T35 観衆
かんしょ #T35 甘薯
かんしょう #T30*3 鑑賞 #T30*2 干渉 観賞 #T35 感傷 環礁 緩衝
かんしょうえき #T35*1 緩衝液
かんしょうてき #T05*1 感傷的
かんしょく #T30*4 間食 #T35*2 感触 #T35 閑職
かんしん #T05*16 関心 #T30*12 感心 #T35 歓心
かんしんじ #T35*2 関心事
かんじ #T35*255 漢字 #KS*248 感じ #T35*214 幹事 #T35*76 感じ #T30*11 換字 #T35*1 監事 #JNM 完治 寛治
かんじと #R5 感じと
かんじゃ #T35*2 患者
かんじゃく #T05 閑寂
かんじゅ #T30*1 甘受
かんじゅく #T30 完熟
かんじゅせい #T35 感受性
かんじょ #T35 官女 #T30 寛恕
かんじょう #T35*14 感情 #T30*6 勘定 #T35*1 環状
かんじょうみゃく #T35*1 冠静脈
かんじん #T05*15 肝心 #T05 肝腎
かんすい #T30*1 冠水 完遂 #T30 潅水
かんすう #T35*255 関数
かんすうがた #T35*10 関数型
かんすうし #T35 冠数詞
かんせい #T30*46 完成 #T35*1 乾性 喚声 感性 #T30*1 管制 #T35 官製 慣性 歓声 #T05 閑静
かんせつ #T35*4 間接 #T35*1 関節
かんせん #T30*3 感染 #T35*1 幹線 #T30*1 観戦 #T35 艦船
かんせんか #T35*1 感染価
かんぜい #T35 関税
かんぜん #T05*80 完全 #F00*1 敢然
かんそ #T05*10 簡素
かんそう #T35*21 感想 #T30*3 乾燥 #T35*1 乾草 #T30 完走 歓送
かんそく #T30*14 観測
かんそん #T35 寒村
かんぞう #T35 肝臓
かんたい #T30*1 歓待 #T35*1 艦隊 #T35 寒帯
かんたく #T30 干拓
かんたん #T05*255 簡単 #T30*3 感嘆
かんだ #JCN 苅田 神田
かんだい #T05*3 寛大
かんだん #T35*1 寒暖 #T30 歓談 #T35 間断
かんち #T30*3 感知 関知
かんちがい #T30*17 勘違い
かんちく #T35*1 患畜
かんちゅう #T35*1 寒中
かんちょう #T35*6 官庁 #T30*5 浣腸 #T35 干潮 管長 艦長 間諜 館長
かんつう #T30*1 貫通
かんづ #K5r 勘づ 感づ
かんづめ #T35 缶詰
かんてい #T30*2 鑑定 #T35 官邸 艦艇
かんてつ #T30 貫徹
かんてん #T35*18 観点
かんでん #T30 感電
かんでんち #T35*1 乾電池
かんとう #T35*8 冠頭 #T35*4 巻頭 #T30*1 敢闘 #JNS 関東
かんとく #T30*6 監督
かんとん #CN 広東
かんど #T35*2 感度
かんどう #T30*14 感動 #T30 勘当
かんどうみゃく #T35*1 冠動脈
かんな #T35 鉋
かんない #T35*1 館内 #T35 管内 艦内 #JCN 関内
かんなづき #T35*1 神無月
かんなな #KK*1 環七
かんなべ #JCN 神辺
かんなみ #CN 函南
かんなり #JCN 金成
かんにゅう #T30 貫入
かんにん #T30 堪忍
かんぬし #T35 神主
かんねつ #T35*1 乾熱
かんねん #T30*1 観念
かんの #JCN 菅野
かんのう #T35*1 感能 #T30 完納 #T35 官能 #T30 感応 #T35 間脳
かんのん #T35*3 観音
かんば #T35 悍馬
かんばし #KYmi*1 芳し
かんばしゅう #KYU*1 芳しゅう
かんばつ #T30 簡抜 間伐 #T35 旱魃
かんばやし #JCN 上林 #JNS 神林
かんばら #JCN 蒲原
かんばん #T35*12 看板
かんぱ #T35*2 寒波 #T30 カンパ
かんぱい #T30*4 乾杯 #T30*1 完敗
かんぱく #T35 関白
かんぱち #KK*1 環八
かんぱん #T35 甲板
かんび #T10*3 完備
かんびょう #T30 看病
かんぶ #T35*2 幹部 #T35 患部
かんぶつ #T35 乾物
かんぶん #T35*3 漢文
かんぷ #T35 乾布 完膚 #T30 還付
かんぷう #T35 寒風 #T30 完封
かんぷく #T30*2 感服
かんべ #JCN 神戸
かんべつ #T30 鑑別
かんべん #T30*14 かんべん #T30*3 勘弁 #T05*2 簡便
かんぺき #T05*20 完璧 #T05*1 完ぺき
かんぼう #T35*1 官房 #T35 感冒
かんぼく #T35 潅木
かんぼつ #T30 陥没
かんぽう #T35*2 漢方 #T35 官報
かんぽうのまじわり #T35 管鮑の交わり
かんぽうやく #T35 漢方薬
かんま #T35*5 カンマ
かんまき #JCN 上牧
かんまく #T35*1 間膜
かんまつ #T35*2 巻末
かんまん #T35 干満 #T05 緩慢
かんみ #T35 甘味
かんみん #T35 官民
かんむり #T35*1 冠
かんむりょう #T05*2 感無量
かんめい #T30*1 感銘 #T05*1 簡明
かんもん #T35*1 関門 #T30 喚問
かんやく #T30*1 監訳 簡約 #T30 完訳
かんゆ #T30*4 換喩
かんゆう #T30*2 勧誘
かんよ #T30*2 関与
かんよう #T05*7 肝要 #T05*1 寛容 #T30 慣用
かんら #JCN 甘楽
かんらく #T35*1 乾酪 #T35 歓楽 #T30 陥落
かんらん #T30 観覧
かんり #T30*189 管理 #T35 官吏
かんりしゃ #T35 管理者
かんりしょく #T35*2 管理職
かんりにん #T35*5 管理人
かんりゃく #T05*10 簡略
かんりゃくか #T30 簡略化
かんりゅう #T30*1 潅流 #T35 寒流
かんりょう #T30*66 完了 #T35*3 官僚
かんるい #T35*1 感涙
かんれい #T05*2 寒冷 #T35*2 慣例
かんれき #T35*1 還暦
かんれん #T30*239 関連
かんろ #T35*1 寒露 #T35 甘露
かんろく #T35 貫禄
かんろに #T35 甘露煮
かんわ #T30*3 緩和 #T35 漢和
かんわきゅうだい #CJ*1 閑話休題
が #T35*105 画 #KJ*16 ヶ #T35*15 賀 #KJ*10 牙 #KJ*2 芽 #KJ*1 ヵ 峨 #SX*1 臥 #T35*1 蛾 #KJ*1 雅 #T35*1 駕 #KJ 俄 #T35 我 #KJ 餓
がーぜ #T35*1 ガーゼ
がーたー #T35*1 ガーター
がーど #T30*3 ガード
がーりっく #T35*1 ガーリック
がい #KJ*22 外 #S5*11 害 #T30*11 害 #JS*7 階 #KJ*4 街 #SX*2 概 #KJ*1 咳 涯 蓋 該 #KJ 凱 劾 #SUC 外 #KJ 崖 慨 概 碍 #SUC 街 #KJ 鎧 骸
がいあく #T35*2 害悪
がいあつ #T35*1 外圧
がいいん #T35*1 外因
がいいんぶ #T35*1 外陰部
がいえん #T05*2 外縁
がいか #T35 凱歌 外貨
がいかい #T35*1 外界
がいかく #T35*3 外郭 #T35 外角
がいかつ #T30 概括
がいかん #T30*11 概観 #T35*2 外観 #T35 外患
がいき #T35 外気
がいきょう #T35 概況
がいきょく #T35 外局
がいきん #T30*2 外勤
がいけい #T35 外形
がいけん #T35*2 外見
がいこう #T30*1 外交 #T35 外向
がいこうかん #T35 外交官
がいこく #T35*15 外国
がいこつ #T35*1 骸骨
がいご #T35 外語
がいさい #T35 外債
がいさん #T30*1 概算
がいざい #T35 外材
がいし #T35*2 外資 #T35 外紙 碍子
がいしゃ #SUC*8 会社 #T35*3 外車 #T35*1 会社
がいしゅう #T35*2 外周
がいしゅつ #T30*13 外出
がいしょう #T35*1 外相 #T35 外傷
がいしょく #T30*2 外食
がいしん #T35 外信 外心
がいじ #T35*91 外字
がいじゅ #T35 外需
がいじん #T35*3 外人
がいすう #T35*1 概数
がいせい #T30 外征 #T35 外声
がいせき #T35*3 外積 #T35 外戚
がいせつ #T30*3 概説 #T30 外接
がいせん #T35*7 外線 #T30 凱旋
がいそう #T30*4 外挿 #T35*2 外層
がいそく #T35*1 外側
がいためほう #T35*1 外為法
がいたん #T30 慨嘆
がいちゅう #T30*9 外注 #T35 害虫
がいてき #T05*1 外的 #T35 外敵
がいでん #T35 外伝 外電
がいとう #T30*27 該当 #T35*1 街頭 #T35 外灯 街灯
がいど #T30*20 ガイド
がいどく #T35 害毒
がいねん #T35*60 概念
がいはいよう #T35*1 外胚葉
がいはく #T30*2 外泊
がいひ #T35*1 外皮 外被
がいひょう #T30 概評
がいぶ #T35*73 外部
がいぶん #T35 外聞
がいへき #T35 外壁
がいまい #T35 外米
がいむ #T35 外務
がいむしょう #KK*1 外務省
がいめん #T35 外面
がいや #T35*1 外野
がいゆう #T30*1 外遊
がいよう #T35*100 概要
がいらい #T35*12 外来
がいらん #T35*2 外乱
がいりゃく #T35*12 概略
がいりょく #T35 外力
がいろ #T35 街路
がいろん #T30*2 概論
がえん #ZX 肯ん
がえんじ #KS 肯んじ
がお #JS*2 顔
がおか #CNSUC1 が丘 ヵ岡 ヵ丘 ヶ岡 ヶ丘
がか #T35*1 画家
がかり #SUC*3 係
ががく #T35 雅楽
がき #N2T30*32 書き
がく #SUC*18 額 #KJ*5 岳 #KJ*3 楽 #T35*2 学 #T35*1 額 #SUC 学 #KJ 顎 鍔 鰐
がくい #T35*2 学位
がくいん #T35*10 学院
がくえん #T35*7 学園
がくぎょう #T35 学業
がくげい #T35*1 学芸
がくげき #T35 楽劇
がくさい #T35*2 学才
がくし #T35*2 学士 #T35 学資
がくしき #T35*1 学識
がくしゃ #T35*2 学者
がくしゅう #T30*241 学習
がくしょう #JS*1 楽章 #T35*1 楽章
がくじゅつ #T35*6 学術
がくせい #T35*31 学生 #T35 学制
がくせき #T35*2 学籍
がくせつ #T35 学説
がくぜん #F03 愕然
がくそう #T35 学窓
がくそく #T35 学則
がくそつ #T35 学卒
がくたい #T35 楽隊
がくだん #T35 楽団
がくちょう #T35*2 学長
がくてき #T05*2 学的
がくどう #T35 学堂 学童 楽堂
がくない #T35*10 学内
がくねん #JS*2 学年 #T35*2 学年
がくばつ #T35 学閥
がくひ #T35 学費
がくふ #T35*1 楽譜
がくぶ #T35*19 学部
がくぶち #T35 額縁
がくむ #T35*1 学務
がくめい #T35 学名
がくめん #T35 額面
がくもん #T30*2 学問
がくや #T35*1 楽屋
がくゆう #T35 学友
がくようひん #T35 学用品
がくりょく #T35 学力
がくれき #T35*8 学歴
がくわり #T35 学割
がけ #SUC*10 がけ #T35*1 崖
がけっぷち #T35 崖っ淵
がごう #T35 雅号
がさい #T35 画才
がざい #T35 画材
がし #T30*1 餓死
がしつ #T35*1 画質
がしゅ #T35 雅趣
がしゅう #T35*1 画集
がしょう #T35 画商 賀正
がしんしょうたん #T30*3 臥薪嘗胆
がじょう #T35*1 牙城 賀状
がす #T35*11 ガス #T35*1 瓦斯
がせんし #T35 画仙紙
がぜん #F02 俄然
がそ #T35*6 画素
がそりん #T35*13 ガソリン
がぞう #T35*24 画像
がた #T35*41 型 #D2KY*6 がた #SUC 型 #D2KY 難 #SUC 方 #T35 方
がだい #T35*1 画題
がち #N2T16*3 がち #N2T16*2 勝ち
がちがち #T03 がちがち
がちっと #F12 がちっと
がちょう #T35 鵞鳥
がっか #T35*29 学科 #T35 学課
がっかい #T35*27 学会 #T35 学界
がっかり #T30*3 がっかり
がっき #T35*7 楽器 #JS*4 学期 #T35*4 学期
がっきゅう #JS 学級 #T35 学級
がっく #JS 学区 #T35 学区 楽句
がっくり #T30*1 がっくり
がっこう #T35*21 学校
がっさく #T30 合作
がっさん #T30*1 合算 #JCN 月山
がっしゅうこく #T35*1 合州国 合衆国
がっしゅく #T30*23 合宿
がっしょう #T30*2 合唱 #T30*1 合掌
がっしょうれんこう #T30*1 合従連衡
がっそう #T30 合奏
がったい #T30*1 合体
がっち #T30*4 合致
がっと #T35*1 ガット
がっぺい #T30*1 合併
がっぽり #F06 がっぽり
がつ #JS*255 月 #KJ*6 月
がつがつ #F04 がつがつ
がつごう #JS 月号
がつまつ #JS*5 月末
がね #SUC*1 金
がはく #T35 画伯
がへい #T35 画餅
がほう #T35*1 芽胞
がまがえる #T35 蝦蟇
がまぐち #T35 がま口
がまごおり #JCN 蒲郡
がまし #KY*1 がまし
がましゅう #KYU*1 がましゅう
がまん #T30*9 我慢
がみがみ #F04 がみがみ
がめ #SUC*2 瓶 #KS*1 がめ #T35 亀
がめん #T35*200 画面
がもう #JCN 蒲生
がや #CNSUC1*3 ヶ谷
がよい #N2T30*2 通い
がようし #T35 画用紙
がよく #T35 我欲
がら #T35*14 柄
がらくた #T35 がらくた
がらくとしだーぜ #T35*1 ガラクトシダーゼ
がらす #T35*1 ガラス 硝子
がらっと #F14 がらっと
がらり #F06 がらり
がらん #T35*1 伽藍
がらんどう #T15 がらんどう 伽藍堂
がりばー #JN ガリバー
がりゅう #T35 我流
がりょうてんせい #T35 画竜点睛 画龍点睛
がりれお #JN ガリレオ
がれーじ #T35*1 ガレージ
がれき #T35*1 瓦礫
がろう #T35 画廊
がわ #T35*255 側 #CNSUC1*22 河 #CNSUC1 川 #SUC 側
がん #T35*6 癌 #KJ*4 眼 #T35*2 ガン #KJ*1 含 玩 岩 顔 #KJ 丸 岸 巌 翫 贋 #T35 雁 #KJ 頑 #T35 願 #KJ 元 厳
がんい #T30*10 含意
がんえん #T35 岩塩
がんか #T35*1 眼下 眼科
がんかい #T35 眼科医
がんがん #F04 がんがん
がんきゅう #T35 眼球
がんきょう #T35*3 元凶 #T05*1 頑強 #T35 眼鏡
がんきん #T35 元金
がんぐ #T35 玩具
がんけん #T35*1 眼瞼 #T05 頑健
がんこ #T05*1 頑固
がんこう #T35 眼光
がんこうしゅてい #T15 眼高手低
がんさく #T30 贋作
がんしゅう #T35 含羞
がんしょ #T35*1 願書
がんしょう #T35 岩礁
がんしょく #T35 顔色
がんじがらめ #T06 がんじがらめ
がんじつ #T35*1 元日
がんじょう #T05*1 頑丈
がんせき #T35 岩石
がんそ #T35 元祖
がんたん #T35*1 元旦
がんちく #T35*2 含蓄
がんちゅう #T35*1 眼中
がんてい #T35 眼底
がんねん #T35*3 元年
がんば #R5r*77 頑張
がんぴし #T35 雁皮紙
がんぺき #T35 岸壁
がんぼう #T35*1 眼房 #T30*1 願望
がんぽん #T35 元本
がんま #T35*5 γ #T35*1 Γ
がんめい #T05*1 頑迷
がんめいころう #T05 頑迷固陋
がんめん #T35 顔面
がんもく #T35 眼目
がんやく #T35 丸薬
がんゆう #T30 含有
がんらい #T35*1 元来
がんり #T35 元利
がんりき #T35 眼力
がんりょう #T35*1 含量 #T35 顔料
がんろう #T30*1 玩弄
き #T35*255 気 #K5*255 聞 #kxi*255 来 #K5*175 効 #R5*152 切 #T35*129 基 #KS*117 着 #T35*100 木 #T35*50 奇 #K5*43 き #T35*35 機 #T35*25 喜 #kxi*18 き #T35*18 期 #T35*15 記 #SX*13 期 #K5*13 聴 #T35*7 樹 #R5*6 き #KJ*6 貴 #KJ*5 奇 鬼 #KJ*4 危 #T35*4 器 #KJ*4 既 #S5*4 帰 #SX*4 帰 #KJ*4 紀 #SX*4 記 #KJ*3 岐 希 #T35*3 忌 #K5r*2 利 #T35*1 黄 #KJ*1 寄 幾 旗 棋 毅 #T35*1 季 #KJ*1 規 起 己 槻 #R5*1 伐 #KJ*1 綺 騏 麒 #KJ 企 伎 #SUC 器 #JS 基 #KJ 嬉 揮 机 #PRE 既 #JS 期 #KJ 棄 #JS 機 #SUC 機 #KJ 汽 畿 祈 稀 徽 #SUC 記 #PRE 貴 #KJ 軌 輝 飢 #JS 騎 #KJ 騎 亀 祁 碁 碕 #R5 斬 #K5 訊 #KJ 生 #PRE 生 #KJ 其 肌 姫 鰭 箕
きー #T35*255 キー
きーたっち #T35*1 キータッチ
きーぼーど #T35*62 キーボード
きーわーど #T35*31 キーワード
きあい #T35*4 気合い
きあつ #T35*1 気圧
きあん #T30*1 起案
きい #T05*2 奇異 #JCN 紀伊 #T35 貴意
きいいち #JNM 基一
きいち #JNM 喜一 #JN 喜市
きいちろう #JNM 喜一郎
きいと #T35 生糸
きいながしま #CN 紀伊長島
きいれ #JCN 喜入
きいろ #T05*7 黄色 #KYmi*3 黄色
きいん #T30*2 起因
きう #T35*1 気宇
きうち #JNS 木内
きうん #T35 機運 気運
きえ #KSr*47 消え
きえい #T35 機影 #T05 気鋭 #R5 消え入
きえさ #R5 消え去
きえん #T35 気炎
きお #W5r*1 気負
きおいた #T5 気負い立
きおう #T35 既往
きおうしょう #T35 既往症
きおく #T30*65 記憶
きおくれ #T30 気遅れ
きおち #T30*1 気落ち
きおん #T35*6 気温
きか #T35*6 幾何 #T30*1 気化 #T35*1 貴下 #T30 帰化
きかい #T35*99 機械 #T35*95 機会 #T05*1 奇怪 #JCN 喜界 #T35 器械
きかいか #T30*2 機械化
きかいご #T35*1 機械語
きかがく #T35*1 幾何学
きかく #T30*93 企画 #T35*59 規格
きかざ #R5r 着飾
きかでーた #T35*3 幾何データ
きかん #T35*60 期間 #T35*52 機関 #T30*5 帰還 #T35*3 基幹 #T35*1 器官 季刊 #T35 既刊 気管 貴簡
きかんし #T35 気管支
きかんしえん #T35 気管支炎
きかんしゃ #T35 機関車
きが #T35 飢餓
きがい #T35*1 危害 #T35 気概
きがえ #KSr*3 着替え #T30*1 着替え
きがかり #T05*1 気がかり
きがく #T35*1 器楽
きがね #T30 気兼ね
きがまえ #T35 気構え
きがる #T05*6 気軽
きがん #T30*2 祈願
きき #T35*42 機器 #T35*9 危機 #KK 記紀 #T35 効き
ききい #R5*1 聞き入 #R5 聴き入
ききいれ #KSr*1 聞き入れ
ききかえ #S5r 聞き返
ききぐるし #KY*1 聞き苦し
ききぐるしゅう #KYU*1 聞き苦しゅう
ききこ #M5r*3 聴き込 #M5r 聞き込
ききだ #S5*1 聞き出
ききて #T35*2 聞き手
ききと #R5r*2 聞き取
ききなが #S5r*4 聞き流
ききめ #T35*1 効き目
ききゃく #T30*2 棄却
ききゅう #T30*1 希求 #T35 気球
ききょ #T30 起居
ききょう #T35*4 桔梗 #T30*1 帰京 帰郷 #T05 奇矯
ききわけ #KSr 聞き分け
ききん #T35*1 基金 #T35 寄金
ききんぞく #T35*1 貴金属
きぎょう #T35*22 企業 #T30*4 起業
きぎょうか #T35*3 起業家
きぎょうたい #T35 企業体
きく #T35*10 菊 #T30*3 掬 #JNM きく #T35 規矩 鞠
きくお #JNM 喜久男 喜久雄 菊男
きくか #JCN 菊鹿
きくがわ #JCN 菊川
きくじ #JN 喜久治
きくすい #JCN 菊水
きくた #JNS 菊田
きくち #JNS 菊地 #JCN 菊池
きくばり #T30*4 気くばり #T30*2 気配り
きくま #JCN 菊間
きくよう #JCN 菊陽
きくん #T35*1 貴君
きぐ #T35*2 器具 #T30 危惧 #T35 機具
きぐう #T05*1 奇遇
きぐらい #T35*1 気位
きぐろう #T30 気苦労
きけい #T35*1 貴兄 #T35 奇形
きけつ #T30*1 帰結 #T35 既決
きけん #T05*34 危険 #T30*1 棄権 #T35 気圏
きげき #T35*1 喜劇
きげん #T35*42 期限 #T35*24 機嫌 #T35*1 紀元 起源
きこ #M5*2 着込
きこう #T35*61 機構 #T30*2 寄稿 #T35*2 気候 #T35*1 紀行 貴校 #T30*1 起工 #T30 寄港 帰港 #T35 気功 気孔 稀覯
きこうし #T35 貴公子
きこうぞう #T35*2 木構造
きこうぼん #T35 稀覯本
きこえ #KSr*20 聞こえ #KSr*1 きこえ #KSr 聴こえ
きこえよがし #T15 聞こえよがし
きこく #T30*11 帰国 #T35 貴国
きこしめ #S5 聞こし召
きこつ #T35 気骨
きこな #S5r 着こな
きこない #JCN 木古内
きこん #T35*2 既婚
きご #T35*1 季語
きごう #T35*149 記号
きごころ #T35 気心
きさ #JCN 吉舎
きさい #T30*17 記載 #T35*2 鬼才 #T35 奇才 既済 #T30 起債 #JCN 騎西
きさかた #JCN 象潟
きさき #T35 后 妃
きさく #T05*3 気さく #JCN 喜作
きさぶろう #JNM 喜三郎
きさま #T35 貴様
きさらぎ #T35 如月
きさらづ #CN 木更津
きさわ #JCN 木沢
きさん #T30*4 起算 #T30 帰参
きざ #M5r*1 刻 #S5r*1 兆 #M5r きざ #S5 きざ #T05 キザ 気障 #S5 萌
きざい #T35*3 機材
きざみ #T35 刻
きざみこ #M5r*1 刻み込
きざみつけ #KSr 刻みつけ
きし #T35*8 騎士 #T35*4 岸 #M5r きし #R5r きし #JNS 岸 #T35 棋士 貴志 #JN 来住 #M5r 軋 #R5r 軋
きしかいせい #T30*1 起死回生
きしがわ #JCN 貴志川
きしだ #JNS 岸田
きしつ #T35*1 基質 #T35 気質
きしべ #T35 岸辺
きしま #JCN 杵島
きしもと #JCN 岸本
きしゃ #T35*19 貴社 #T35*6 汽車 #T30*2 帰社 #T35*2 記者
きしゃく #T30 希釈 棄釈
きしゅ #T35*45 機種 #T35*1 旗手 機首 #T35 奇手 期首
きしゅう #T30*1 奇襲 #CN 紀州
きしゅく #T30 寄宿 #CN 岐宿
きしゅつ #T30*2 既出
きしょ #T35 貴書
きしょう #T35*2 気象 #T05*1 希少 #T30*1 起床 #T35 気性 記章
きしょうだい #T35 気象台
きしょうちょう #KK 気象庁
きしょうてんけつ #T35*1 起承転結
きしょく #T35 気色
きしわだ #JCN 岸和田
きしん #T30 寄進 #T35 貴信
きじ #T35*116 記事 #T35*1 生地 雉 #T35 木地
きじく #T35*4 基軸 #T35*1 機軸
きじつ #T35*10 期日
きじばと #T35 キジバト 雉鳩
きじま #JCN 木島 #JNS 木嶋
きじまだいら #JCN 木島平
きじゅ #T35*2 喜寿
きじゅつ #T30*210 記述 #T35 奇術
きじゅん #T35*22 基準 #T35*4 規準
きじょ #T35*2 貴女 #T35*1 機序
きじょう #T35*5 机上 #T05*1 気丈 #T35 機上 軌条 #JCN 木城
きじん #T35*2 奇人
きす #T30*3 キス #T35*1 鱚
きすい #T35 既遂
きすう #T35*6 奇数 #T35 帰趨
きすき #JCN 木次
きすみ #JNS 木栖
きず #K5*11 築 #T35*3 傷 #T35*2 瑕
きずあと #T35 傷跡
きずきあげ #KS 築き上げ
きずくり #JNS 木造
きずぐち #T35 傷口
きずつ #K5*4 傷つ
きずつけ #KS*1 傷つけ
きずな #T35*1 絆
きせ #KS 着せ
きせい #T35*15 既成 #T30*13 帰省 #T30*7 規制 #JS*1 期生 #T35 奇声 #T30 寄生 #T35 既製 気勢 #JCN 紀勢 #T30 規正
きせき #T35*2 軌跡 #T35*1 奇跡
きせつ #T35*12 季節 #T35*2 既設
きせのん #T35*1 キセノン
きせん #T35*1 汽船 #T35 基線
きぜつ #T30 気絶
きぜん #F00*1 毅然
きそ #T35*99 基礎 #T30*2 起訴 #W5r*2 競 #JCN 木曾 木曽 #JNS 木祖
きそう #T35*1 気相 #T30*1 起草 #T30 寄贈
きそうてんがい #T05 奇想天外
きそがわ #CN 木曽川
きそく #T35*225 気息 #T35*75 規則
きそざき #CN 木曾岬
きそふくしま #CN 木曾福島
きそん #T30*10 既存 #T30*2 毀損
きぞう #T30*1 寄贈
きぞく #T35*3 貴族 #T30*1 帰属
きぞん #T00*32 既存
きた #T35*51 北 #S5*7 来 #S5*4 きた #CNPRE*4 北 #S5*1 来た #JCN 喜多 #CNSUC1 北 #JCN 北 木田
きたあいき #CN 北相木
きたあいづ #CN 北会津
きたあきた #CN 北秋田
きたあずみ #CN 北安曇
きたあだち #CN 北足立
きたあまべ #CN 北海部
きたありま #CN 北有馬
きたい #T30*102 期待 #T35*1 奇胎 機体 #T35 危殆 #T05 奇体 奇態 #T35 気体 稀代
きたいち #T35 期待値
きたいばらき #CN 北茨城
きたうおぬま #CN 北魚沼
きたうら #JCN 北浦
きたうわ #CN 北宇和
きたえ #KS*6 鍛え
きたえあげ #KS 鍛え上げ
きたかぜ #T35 北風
きたかた #JCN 喜多方 北方
きたかつしか #CN 北葛飾
きたかつらぎ #CN 北葛城
きたかみ #JCN 北上
きたかるいざわ #CN 北軽井沢
きたかわべ #CN 北川辺
きたかんばら #CN 北蒲原
きたがた #JCN 北方
きたがわ #T35*1 北側 #JNS 北河 #JCN 北川
きたきつね #T35*1 キタキツネ
きたきゅうしゅう #CN 北九州
きたく #T30*4 帰宅 #T30 寄託
きたくわだ #CN 北桑田
きたぐち #T35*2 北口
きたぐんま #CN 北群馬
きたけ #T35 着丈
きたこま #CN 北巨摩
きたごう #JCN 北郷
きたさいたま #CN 北埼玉
きたさく #CN 北佐久
きたざわ #JCN 北沢
きたしおばら #CN 北塩原
きたしげやす #CN 北茂安
きたしたら #CN 北設楽
きたじま #JCN 北島 #JNS 北嶋
きたそうま #CN 北相馬
きたたかき #CN 北高来
きたたかぎ #CN 北高来
きたたちばな #CN 北橘
きたたま #CN 北多摩
きただいとう #CN 北大東
きたつがる #CN 北津軽
きたつる #CN 北都留
きたな #KY*16 汚
きたなかぐすく #CN 北中城
きたならし #KY*1 汚らし
きたならしゅう #KYU*1 汚らしゅう
きたにほん #CN 北日本
きたの #JCN 北野
きたのう #KYU*2 汚う
きたはた #CN 北波多
きたはら #JNS 北原
きたはんきゅう #T35 北半球
きたひやま #CN 北桧山
きたまち #T35*1 北町
きたまつうら #CN 北松浦
きたまわり #T30 北回り
きたみ #JCN 北見
きたみまき #CN 北御牧
きたむら #JCN 北村
きたむらやま #CN 北村山
きたむろ #CN 北牟婁
きため #JCN 北目
きたもと #JCN 北本
きたもろかた #CN 北諸県
きたやま #JCN 北山
きたら #JN 喜多羅
きたる #RT*2 来る
きたろう #JNM 喜太郎
きたん #T35*5 忌憚
きだ #JNS 木田
きだい #T35*1 貴台
きだて #T35 気だて
きだん #T35 気団
きち #T35*25 基地 #T05*5 既知 #T35*3 吉 #T35 機知 貴地
きちがい #T35 気違い
きちじ #T35 吉事
きちじつ #T35*1 吉日
きちじょうじ #CN 吉祥寺
きちっと #F12 きちっと
きちのすけ #JNM 吉之助
きちゃく #T30*2 帰着
きちゅう #T35 忌中
きちょう #T35*8 基調 #T05*5 貴重 #T30*3 記帳 #T35*1 機長
きちょうめん #T05 きちょうめん 几帳面
きちろう #JNM 吉郎
きちんと #F12*45 きちんと
きっ #SX 喫
きっかけ #T35*3 きっかけ #T35*1 切っ掛け
きっかり #F14 きっかり
きっかわ #JNS 吉川
きっきょう #T35 吉凶
きっく #T30*6 キック
きっこう #T30*1 拮抗
きっさ #T35*2 喫茶
きっさき #T35 切先
きっさてん #T35*3 喫茶店
きっす #T30*1 キッス
きっすい #T35 喫水 生粋
きっせき #T30 詰責
きったか #JNS 橘高
きっちょう #T35 吉兆
きっちり #F04*4 きっちり
きっちん #T35*2 キッチン
きって #T35*4 切手
きっと #F14*77 きっと #T35*28 キット
きっぱり #F04 きっぱり
きっぷ #T35*14 切符 #T35 気っ風
きっぽう #T35 吉報
きつ #KY*7 きつ #KJ*1 橘 #KJ 吃 喫 桔 詰
きつう #KYU*1 きつう
きつえん #T30*3 喫煙
きつかわ #JNS 橘川
きつき #JCN 杵築
きつけ #T30*1 着付け
きつつき #T35 キツツキ 啄木鳥
きつね #T35*1 狐
きつもん #T30*1 詰問
きつれがわ #JCN 喜連川
きづ #K5*29 気づ 気付 #JCN 木津
きづか #W5r*2 気遣
きづかれ #T30 気疲れ
きづけ #T35*5 気付
きてい #T30*114 規定 #T35*13 基底 #T35*5 既定 #T35 規程
きていち #T35 既定値
きてき #T35*3 汽笛
きてん #T35*16 起点 #T35*2 機転 #T35 基点 貴店
きでら #JNS 紀寺
きでん #T35*1 貴殿
きと #T30 企図 #T35 帰途
きとう #T30*1 祈祷 #JCN 木頭
きとく #T05*4 奇特 #T35*2 既得 #T35 危篤
きど #R5r*4 気取 #T35*2 輝度 #T35*1 木戸 #R5r 気ど
きどう #T30*255 起動 #T35*10 軌道 #T35*1 機動 気道
きどうたい #T35 機動隊
きどるい #T35*1 希土類
きない #T35*1 機内
きなが #T05*3 気長
きなさ #JCN 鬼無里 #R5 来なさ
きにい #R5*25 気に入
きにち #T35 忌日
きにゅう #T30*71 記入
きにん #T30*1 帰任
きぬ #T35*2 絹 #T35 衣
きぬいと #T35 絹糸
きぬた #T35 砧
きね #T35 杵
きねま #T35*1 キネマ
きねん #T30*11 記念 #T30*2 祈念
きねんひ #T35 記念碑
きねんひん #T35*2 記念品
きねんび #T35*1 記念日
きのう #T30*255 機能 #T35*150 昨日 #T30*4 帰納
きのえ #JCN 木江
きのくにや #KK*3 紀伊国屋 #KK*1 紀伊國屋
きのこ #T35*1 キノコ #T35 茸
きのさき #JCN 城崎
きのした #JNS 木下
きのどく #T05*1 気の毒
きのり #T30 気乗り
きはく #T05*1 希薄 #T35 気迫
きはじめ #KSr*1 来はじめ #KSr 来始め
きはた #JN 木幡
きはだ #T35 木肌
きはちろう #JNM 喜八郎
きはつ #T30*2 揮発
きはつゆ #T35 揮発油
きはら #JNS 樹原 #JCN 木原
きはん #T35*1 規範
きば #T35*2 騎馬 #R5*1 気張 #M5r 黄ば #T35 牙 木場
きばく #T35 起爆
きばたらき #T35*1 気ばたらき
きばつ #T05*1 奇抜
きばらし #T30 気晴らし
きばん #T35*30 基板 基盤
きひ #T30 忌避
きひつ #T30 起筆
きひん #T35*1 気品 #T35 貴賓
きび #T35 機微 #JCN 吉備 #T35 黍
きびきび #F04*1 きびきび
きびし #KY*6 厳し #KY*4 きびし
きびしゅう #KYU*1 きびしゅう 厳しゅう
きびす #T35 踵
きびたき #T35 キビタキ 黄鶲
きびょう #T35 奇病
きびん #T05 機敏
きふ #T30*3 寄付
きふう #T35 気風
きふく #T30 起伏
きぶくれ #T30 着膨れ
きぶっせい #T05 気ぶっせい
きぶとり #T30 着太り
きぶん #T35*30 気分
きへい #T35*1 騎兵
きべ #JNS 峡部
きべん #T35 詭弁
きほう #T35*17 記法 #JCN 紀宝 #T35 貴方 #JNS 木方
きほく #JCN 輝北
きほん #T35*255 基本
きぼ #T35*31 規模
きぼう #T30*80 希望
きぼね #T35 気骨
きぼり #T35 木彫り
きま #R5r*100 決ま #R5r*3 きま #R5r*1 決 #R5 極
きまえ #T35*1 気前
きまぐれ #T05*3 気まぐれ
きまず #KY 気まず
きまずう #KYU 気まずう
きまって #F14 決って
きまつ #T35 期末
きまま #T05*1 気まま
きみ #T35*42 君 #T35*1 気味 #KJ*1 公 #JNM きみ キミ #T35 黄身 #JNM 希美 貴美
きみお #JNM 公男 公雄
きみがわる #KY 気味が悪
きみがわるう #KYU 気味が悪う
きみこ #JNM きみ子 喜美子 君子 公子
きみしま #JNS 君島 君嶋
きみじか #T05 気短
きみじま #JNS 君島 君嶋
きみた #JCN 君田
きみたち #T35*1 君達
きみつ #T35*1 機密 #JCN 君津
きみどり #T15 黄緑
きみひこ #JNM 公彦
きみょう #T05*4 奇妙
きむずかし #KY 気難し
きむずかしゅう #KYU 気難しゅう
きむち #T35*1 キムチ
きむづかし #KY*1 気難し
きむづかしゅう #KYU*1 気難しゅう
きむら #JNS 木村
きめ #KS*131 決め #T35*1 肌理 #T30 決め #T35 木目
きめい #T30*1 記名
きめこ #M5*1 決め込
きめこまか #KY*1 きめ細か #T07*1 きめ細か
きめこまこう #KYU*1 きめ細こう
きめつけ #KSr*1 決めつけ
きめて #T35*1 決め手
きめら #T35*1 キメラ
きも #T35*7 肝
きもち #T35*46 気持 #T35*19 気持ち
きもったま #T35 肝っ玉
きもつき #JCN 肝属
きもと #JNS 木本
きもの #T35*3 着物
きもべつ #CN 喜茂別
きもん #T35 鬼門
きゃ #KJ*8 伽
きゃく #T35*12 客 #KJ*1 脚 #KJ 却 #JS 客 脚
きゃくあし #T35 客足
きゃくいん #T35 客員 脚韻
きゃくしつ #T35*1 客室
きゃくしゃ #T35 客車
きゃくしょく #T30 脚色
きゃくじん #T35 客人
きゃくせき #T35 客席
きゃくせん #T35*1 客船 #T35 脚線
きゃくちゅう #T35*9 脚注
きゃくほん #T35 脚本
きゃくま #T35 客間
きゃしゃ #T05*3 きゃしゃ #T05*1 華奢
きゃすと #T30*5 キャスト
きゃたつ #T35*1 脚立
きゃっか #T30*3 却下
きゃっかん #T30*4 客観
きゃっこう #T35 脚光
きゃっしゅ #T30*34 キャッシュ
きゃっしんぐ #T30*3 キャッシング
きゃっち #T30*1 キャッチ
きゃっちふれーず #T35*1 キャッチフレーズ
きゃどきー #KK*2 キャドキー
きゃのん #KK*1 キャノン
きゃばれー #T35*1 キャバレー
きゃびねっと #T35*2 キャビネット
きゃぷてん #T35*1 キャプテン
きゃらくた #T35*91 キャラクタ
きゃりー #T35*1 キャリー
きゃりあー #T35*8 キャリアー
きゃんせる #T30*37 キャンセル
きゃんでぃ #T35*1 キャンディ
きゃんでぃー #T35*1 キャンディー
きゃんばす #T35*1 キャンバス
きゃんぱす #T35*3 キャンパス
きゃんぺーん #T30*1 キャンペーン
きやく #T35*30 規約 #T05*11 既約
きやす #KY*1 気安
きやすう #KYU*1 気安う
きやすめ #T35*1 気休め
きやせ #T30 着痩せ
きやま #JCN 基山
きやまち #CN 木屋町
きゅー #T35*7 キュー
きゅーぶ #T35*2 キューブ
きゅう #T35*64 旧 #T05*51 急 #KJ*26 休 #NN*14 九 #T30*13 仇 #S5*7 給 #T30*7 給 #KJ*5 吸 #T35*4 球 #KJ*3 久 #SX*2 休 #KJ*1 宮 求 #T35*1 灸 #KJ*1 嗅 #KJ 扱 丘 及 弓 #PRE 急 #KJ 救 朽 汲 泣 #JS 球 #KJ 究 #S5 窮 #T30 窮 #KJ 笈 #JS 級 #SUC 級 #T35 級 #KJ 糾 #PRE 旧 #KJ 鳩
きゅうあい #T30*1 求愛
きゅういん #T30*1 吸引
きゅうえん #T30 休演 救援
きゅうか #T35*45 休暇 #T35 旧家
きゅうかい #T30*2 休会
きゅうかく #T35*1 嗅覚 #T35 球殻
きゅうかな #T35 旧カナ 旧仮名
きゅうかん #T30*1 休刊 #T35 休閑 #T30 休館 #T35 急患 旧館
きゅうかんちょう #T35*1 九官鳥
きゅうがく #T30*1 休学
きゅうき #T30*5 吸気
きゅうきゅう #T35*2 救急 #F03 窮々
きゅうきゅうしゃ #T35 救急車
きゅうきょ #F14*3 急きょ #F14 急拠
きゅうきょく #T35*5 究極
きゅうきん #T35 給金
きゅうぎ #T35 球技
きゅうぎょう #T30*1 休業
きゅうくつ #T05*1 窮屈
きゅうけい #T30*4 休憩 #T35 急啓 #T30 求刑 #T35 球形
きゅうけつ #T30*1 吸血
きゅうけつき #T35*1 吸血鬼
きゅうげき #T05*4 急激
きゅうげん #T30 急減
きゅうこう #T30*2 急行 #T30*1 休講 #T35*1 吸光 #T30 休校 休耕
きゅうこうか #T30 急降下
きゅうこく #T30*2 急告
きゅうこん #T30*1 求婚 #T35 球根
きゅうご #T30*1 救護
きゅうごう #T30 糾合 鳩合
きゅうさい #T30*1 救済 #T30 休載
きゅうし #T30*1 休止 #T35 臼歯 #T30 急死
きゅうしき #T05 旧式
きゅうしふ #T35 休止符
きゅうしゅう #T30*45 吸収 #CN 九州
きゅうしゅつ #T30*5 救出
きゅうしゅん #T05*1 急峻
きゅうしょ #T35*2 急所
きゅうしょく #T30*1 求職 給食 #T30 休職
きゅうしん #T30*1 休診 #T30 急進
きゅうじ #T30 給仕
きゅうじつ #T35*13 休日
きゅうじょ #T30 救助
きゅうじょう #T35*1 球場 球状 窮状 #T30 休場 #T35 宮城
きゅうじょうしょう #T30 急上昇
きゅうじょたい #T35 救助隊
きゅうじん #T30*1 求人
きゅうす #T35*9 急須
きゅうすい #T30 給水
きゅうすう #T35*1 級数
きゅうせい #T35*6 旧姓 #T35*1 急性 #T30*1 急逝 #T35 旧制
きゅうせいしゅ #T35 救世主
きゅうせいちょう #T30 急成長
きゅうせき #T35 旧跡
きゅうせん #T30 休戦
きゅうそう #T30 急送
きゅうそく #T05*2 急速 #T30*1 休息
きゅうぞう #T30*1 急増
きゅうたい #T35*1 旧態 #T15 球体
きゅうたろう #JCN 久太郎
きゅうだい #KK*2 九大 #T30*1 及第
きゅうだん #T35*1 球団 #T30 糾弾
きゅうち #T35 窮地 旧知
きゅうちゃく #T30*1 吸着
きゅうちゅう #T35 宮中
きゅうちょう #T35 級長
きゅうてい #T35*3 宮廷
きゅうていしゃ #T30 急停車
きゅうてん #T30 急転
きゅうでん #T35 宮殿
きゅうとう #T30 急騰 #T35 給湯
きゅうどう #T35*2 弓道 #T35*1 旧道
きゅうにゅう #T30*1 吸入
きゅうねん #T35*1 旧年
きゅうは #T30 急派
きゅうはく #T30 急迫 窮迫
きゅうば #T35*1 急場
きゅうばん #T35*2 旧版 #T35 吸盤
きゅうびょう #T35 急病
きゅうふ #T35*1 宮阜 #T30*1 給付
きゅうふきん #T35 給付金
きゅうへん #T30*1 急変
きゅうほう #T30 急報
きゅうほうじ #CN 久宝寺
きゅうぼ #T30 急募
きゅうぼう #T30 窮乏
きゅうみん #T30 休眠
きゅうむ #T35*1 急務
きゅうめい #T30*5 究明 #T30 救命 糾明
きゅうめん #T35*1 球面
きゅうゆ #T30*1 給油
きゅうゆう #T35 級友 旧友 梟雄
きゅうよ #T30*2 給与
きゅうよう #T30*2 休養 #T35*1 急用
きゅうらい #T35*3 旧来
きゅうらぎ #JCN 厳木
きゅうらく #T30*1 急落
きゅうり #T35 胡瓜
きゅうりゅう #T35 急流
きゅうりょう #T35*8 給料 #T35*1 丘陵
きゅうりょうび #T35 給料日
きゅうれい #T30 急冷
きゅうれき #T35 旧暦
きゅりー #JN キュリー
きゆう #T35*1 杞憂
きょ #T35*58 居 #KJ*3 許 #KJ*1 巨 拠 #T35*1 挙 虚 #KJ 嘘 去 拒 渠 距 鋸
きょう #T35*255 今日 #CNSUC1*62 教 #T35*17 京 #KJ*12 共 #S5*9 供 #T30*9 供 #T35*8 境 #T35*7 卿 #KJ*5 協 恭 #KJ*4 郷 #T35*3 強 興 #KJ*1 亨 享 #T35*1 凶 #KJ*1 匡 恐 #T30*1 狂 #ZX*1 興 #KJ*1 鏡 #T30*1 饗 #T35*1 経 #KJ 杏 叶 侠 僑 兇 競 叫 喬 峡 #JSSUC 強 #PRE 強 #KJ 彊 怯 挟 #JNSUC 教 #KJ 教 橋 況 狭 矯 胸 脅 蕎 響 驚 兄 梗 劫
きょうあい #T05 狭隘
きょうい #T35*1 胸囲 脅威 #T35 強意 驚異
きょういく #T30*87 教育
きょういち #JNM 恭一
きょういん #T35*1 教員
きょうえい #T30*1 競泳 #T30 共栄
きょうえき #T35 共益
きょうえん #T30*1 競演 共演 #T35 饗宴
きょうか #T30*57 強化 #T30 教化 #T35 教科
きょうかい #T35*56 境界 #T35*16 協会 #T35*3 教会 #T30 教誨
きょうかく #T35 胸郭
きょうかしょ #T35*3 教科書
きょうかつ #T30*1 恐喝
きょうかん #T35*6 教官 #T30*2 共感
きょうが #T35 恭賀
きょうがい #T35 境涯
きょうがく #T35 共学 #T30 驚愕
きょうがせ #CN 京ヶ瀬
きょうき #T35*1 狂気 #T35 侠気 凶器 #T30 狂喜 驚喜
きょうきゃく #T35 侠客 橋脚
きょうきゅう #T30*21 供給
きょうきん #T35 胸襟
きょうぎ #T30*16 競技 #T30*4 協議 #T35*2 狭義 #T35*1 教義 #T35 経木
きょうぎょう #T30 協業
きょうくう #T35*1 胸腔
きょうくん #T30*2 教訓
きょうぐう #T35 境遇
きょうけつ #T30 供血
きょうけん #T35*1 鏡検 #T05 強健 #T35 強権
きょうけんびょう #T35 狂犬病
きょうげき #T35 京劇 #T30 挟撃
きょうげん #T35 狂言
きょうこ #T05*3 強固 #JNM 京子 恭子 響子 今日子
きょうこう #T30*4 強行 #T35*2 恐慌 #T35 凶行 #T05 強硬
きょうこうぐん #T35 強行軍
きょうこく #T35*1 強国 #T35 峡谷
きょうこつ #T35 胸骨
きょうご #T30 教護
きょうごう #T30*13 競合 #T35 強豪
きょうごく #JCN 京極
きょうさ #T30 教唆
きょうさい #T35*2 恐妻 #T30*1 共催 #T35*1 共済
きょうさく #T30*1 狭窄 #T30 競作 #T35 凶作
きょうさん #T35*1 共産 #T30 協賛
きょうさんとう #KK*1 共産党
きょうざい #T35*5 教材
きょうざつ #T30 夾雑
きょうざぶろう #JNM 恭三郎
きょうざめ #KS*1 興ざめ #T11 興ざめ
きょうし #T35*14 教師
きょうしき #T35 胸式
きょうしつ #T35*11 教室
きょうしゃ #T35 強者
きょうしゅう #T35*1 郷愁
きょうしゅうじょ #T35 教習所
きょうしゅく #T10*25 恐縮
きょうしゅつ #T30*1 供出
きょうしょ #T35 教書
きょうしょう #T05 狭小
きょうしょく #T35*1 教職
きょうしん #T30*2 共振 #T35 強心 強震
きょうしんしょう #T35 狭心症
きょうじ #T30*2 教示 #T35 共時 凶事 矜持
きょうじゃく #T35 強弱
きょうじゅ #T30*20 教授 #T30*2 享受
きょうじゅつ #T30 供述
きょうじゅん #T35 恭順
きょうじょう #T35 教場 教条
きょうじん #T05*1 強靭 #T35 狂人
きょうせい #T30*29 強制 #T30*1 共棲 共生 矯正 #T30 強請 #T35 教生
きょうせら #KK*1 京セラ
きょうそ #T35 教祖
きょうそう #T30*2 競争 #T30*1 競走 狂躁 #T05 強壮
きょうそうきょく #T35*1 協奏曲
きょうそく #T35 教則
きょうそん #T30 共存
きょうぞう #T35 胸像 鏡像
きょうぞめ #T35 京染
きょうぞん #T30*4 共存
きょうたい #T35 狂態
きょうたく #T30 供託
きょうたん #T30*1 驚嘆
きょうだ #T30 強打 #T15 怯懦
きょうだい #KK*16 京大 #T35*12 兄弟 #T35*3 橋台 #T05 強大 #T35 鏡台
きょうだん #T35*1 教団 #T30 凶弾 #T35 教壇
きょうち #T35*1 境地
きょうちゅう #T35*1 胸中
きょうちょ #T30 共著
きょうちょう #T30*56 強調 #T30*16 協調 #T35 凶兆
きょうつう #T00*91 共通
きょうてい #T35*1 競艇 #T30*1 協定 #T35 教程
きょうてき #T35 強敵
きょうてん #T35 経典
きょうと #T35*4 教徒 #CN 京都
きょうとう #T30*1 共闘 #JNS 京濤 #T35 教頭 #T30 驚倒
きょうとうほ #T35 橋頭堡
きょうとふ #CN 京都府
きょうど #T35*4 強度 #T35 郷土
きょうどう #T30*32 共同 #T30*1 協同 #T35 経堂
きょうどうたい #T35 共同体
きょうどうつうしん #KK*1 共同通信
きょうねん #T35 享年
きょうはく #T30*5 脅迫 #T30*1 強迫
きょうはん #T35 共犯
きょうばい #T30 競売
きょうばて #CN 京終
きょうふ #T30*3 恐怖
きょうふう #T35*1 強風
きょうぶ #T35 胸部
きょうべん #T30 強弁 #T35 教鞭
きょうほ #T30 競歩
きょうほん #T30*3 狂奔
きょうぼう #T05*1 凶暴 #T30 共謀 #T05 狂暴
きょうぼく #T35 喬木
きょうま #T35 京間
きょうまく #T35*1 莢膜
きょうみ #T35*39 興味
きょうむ #T35*1 教務
きょうめい #T30*1 共鳴 #T35 響鳴
きょうめん #T35*1 鏡面
きょうやく #T05*2 共役 #T30*1 供役 協約
きょうゆ #T35 教諭
きょうゆう #T30*58 共有 #T30 享有
きょうよ #T30*1 供与
きょうよう #T30*43 共用 #T35*6 教養 #T30*2 強要
きょうらく #T30 享楽
きょうらん #T30*1 供卵 #T30 供覧 狂乱
きょうり #T35*2 郷里
きょうりつ #T35*5 共立
きょうりゅう #T35*1 恐竜
きょうりょう #T35 橋梁 #T05 狭量
きょうりょく #T05*45 強力 #T30*36 協力
きょうれつ #T05*3 強烈
きょうれん #T30 教練
きょうわ #T30*2 協和 #JCN 共和 #T35 共和 #JCN 協和
きょうわこく #T35*1 共和国
きょうわせい #T35 共和制
きょえい #T35 虚栄
きょか #T30*34 許可
きょがく #T05*1 巨額
きょきょじつじつ #T35*1 虚々実々
きょぎ #T35 虚偽
きょぎょう #T35*4 虚業
きょく #JS*21 曲 #T35*16 極 #T30*3 局 #T30*2 曲 #SUC 局
きょくいん #T35 局員
きょくぎ #T35 曲技
きょくげい #T35 曲芸
きょくげん #T35*2 極限 #T30 局限 極言
きょくし #CN 旭志
きょくしょ #T35*11 局所
きょくしょう #T05*1 極小 #T05 極少
きょくせい #T35*1 極性
きょくせつ #T30*1 曲折
きょくせん #T35*11 曲線
きょくたい #T35*1 極体
きょくたん #T05*4 極端
きょくだい #T05*1 極大
きょくち #T35 局地 極地 極致
きょくちょう #T35*3 曲調 #T35*1 局長
きょくてん #T35 極点
きょくど #T35*1 極度
きょくどめ #T30*2 局止め
きょくない #T35*1 局内
きょくばん #T35*1 局番
きょくぶ #T35 局部
きょくめん #T35*7 局面 #T35*2 曲面
きょくもく #T35*2 曲目
きょくりつ #T35 曲率
きょくりょく #F14*5 極力
きょくろん #T30*1 極論
きょげん #T30 虚言
きょこう #T30 挙行 #T35 虚構
きょこん #T35 許婚
きょごう #T05 倨傲
きょし #T30 挙止
きょしき #T30*1 挙式
きょしつ #T35*1 居室
きょしてき #T05 巨視的
きょしゅ #T30 挙手
きょしゅう #T35*1 去就
きょしゅつ #T30 拠出
きょしょ #T35 居所
きょしょう #T35 巨匠 裾礁
きょしょく #T35 虚飾
きょしょくしょう #T35 拒食症
きょしんたんかい #T15*1 虚心坦懐
きょじつ #T35 虚実
きょじゃく #T05 虚弱
きょじゅう #T30*1 居住
きょじん #T35*3 巨人
きょすう #T35 虚数
きょせい #T30 去勢 #T35 巨星 虚勢
きょぜつ #T30*2 拒絶
きょぞう #T35 巨象 虚像
きょたく #T35 居宅
きょだい #T05*12 巨大
きょだく #T30*4 許諾
きょだつ #T30 虚脱
きょっかい #T30 曲解
きょっけい #T35 極刑
きょてん #T35*4 拠点
きょとう #T35 巨頭 挙党
きょどう #T35*27 挙動
きょなん #JCN 鋸南
きょねん #T35*18 去年
きょひ #T30*4 拒否 #T35 巨費
きょふ #T35 巨富
きょほう #T35 巨峰 虚報
きょむ #T35 虚無
きょよう #T30*3 許容
きょらい #T30 去来
きょり #T35*46 距離 #T35 巨利
きょりゅう #T30 居留
きょれい #T35 虚礼
きょろきょろ #F04*1 きょろきょろ #F12*1 きょろきょろ
きよ #KY*63 清 #T30*6 寄与 #JNM きよ キヨ 喜代 #KJ 清
きよう #KYU*2 清う #T05 器用 #T30 起用
きよかわ #JCN 清川
きよこ #JNM 喜代子 清子
きよさと #JCN 清里
きよし #JNM 喜好 喜代志 潔 清 清司 清志 清茂
きよじ #JNM 喜代治
きよす #JCN 清洲
きよせ #JCN 清瀬
きよたか #JNM 清隆
きよたけ #JCN 清武
きよだ #JNS 清田
きよと #JNM 清人
きよね #JCN 清音
きよのぶ #JN 清信
きよはら #JNS 清原
きよまつ #JNM 清松
きよみ #JCN 清見 #JNM 清美
きよみず #JCN 清水
きよめ #KSr*1 清め
きよらか #T05 清らか
きよわ #T05*1 気弱
きら #W5r*3 嫌 #JCN 吉良 #S5 切ら
きらい #T05*37 嫌い #T35*1 きらい #T30*1 帰来
きらく #T05*3 気楽
きらびやか #F06 きらびやか
きらめ #K5r*2 きらめ #K5r*1 燐 煌
きり #T35*9 霧 #T35*8 桐 #T35*4 雰 #T35 切り
きりあげ #KSr*1 切り上げ #T30 切上げ
きりおと #S5r 切り落と
きりかえ #KSr*69 切替え #KSr*19 切り替え #T35*7 切替
きりかぶ #T35*1 切株
きりかわ #R5r*4 切り替わ
きりきざ #M5*1 切り刻
きりきず #T35 切傷
きりぎりす #T35*1 キリギリス
きりくず #S5r 切り崩
きりくち #T35*1 切口
きりこ #M5r*1 切り込
きりさげ #KSr*1 切り下げ #T30 切下げ
きりさめ #T35 霧雨
きりしま #JNS 桐島 #JCN 霧島
きりすて #T30*4 切捨て
きりすと #JN キリスト
きりた #T5 切り立
きりたに #JNS 桐谷
きりたんぽ #T35 切りたんぽ
きりだ #S5r*9 切り出
きりつ #T35*1 規律 #T30*1 起立
きりつめ #KSr*1 切り詰め
きりと #R5r*1 切り取
きりとり #T30*1 切り取り 切取り
きりぬ #K5r*2 切りぬ 切り抜
きりぬき #T30*1 切抜き
きりはな #S5r*13 切り離 #S5*1 切り放
きりはり #T30*2 切り貼り
きりひら #K5r 切り開
きりふだ #T35 切札
きりぶち #JNS 桐渕
きりまわ #S5r 切り回
きりもり #T30*1 切盛り
きりゅう #T35*1 気流 #JCN 桐生
きりょう #T35*1 器量
きりょく #T35*2 気力
きりわけ #KSr 切り分け
きりん #T35*2 麒麟 #T35*1 キリン
きれ #JNS 喜連
きれーた #T35*1 キレータ
きれーと #T30*1 キレート
きれい #T05*22 きれい #T05*15 綺麗 #T05*3 奇麗
きれぎれ #T05*1 切れ切れ
きれこ #M5r 切れ込
きれっぱし #T35 切れっ端
きれつ #T35 亀裂
きれはし #T35 切れ端
きれま #T35 切れ間
きれめ #T35*5 切れ目
きろ #JS*15 キロ #T35*1 帰路 #T35 岐路
きろく #T30*77 記録
きろめーとる #JS キロメートル 粁
きわ #T35*5 際 #JCN 紀和
きわだ #T5*1 際だ
きわど #KY*10 きわど
きわどう #KYU*2 きわどう
きわま #R5r 窮ま 極ま
きわみ #T35 極み
きわめ #KS*2 究め #KS*1 極め
きわめつき #T35 極めつき
きわめつく #S5 窮め尽く
きわめて #F14*10 極めて #F14*2 きわめて
きん #T35*196 金 #KJ*13 近 #KJ*6 欽 #KJ*3 筋 緊 #T35*2 禁 #ZX*2 禁 #KJ*2 今 #KJ*1 勤 均 #T35*1 菌 #KJ 僅 巾 錦 #JS 斤 #KJ 斤 欣 琴 禽 芹 衿 襟 謹 金 #SUC 金 #JCN 金武
きんいち #JNM 金一
きんいつ #T05*1 均一
きんいろ #T35 金色
きんいん #T35 近因
きんえい #T35 近影
きんえき #T35*1 菌液
きんえん #T30*3 禁煙 #T35*1 近縁
きんか #T35 近火 金貨
きんかい #CN 琴海 #T35 近海 金塊
きんかん #T35 近刊 金冠
きんが #T35*3 謹賀
きんがく #T35*24 金額
きんがん #T35*1 近眼
きんき #T30 欣喜 #CN 近畿
きんきゅう #T05*8 緊急
きんきょう #T35*4 近況
きんきょり #T35 近距離
きんきん #F14 近々
きんぎょ #T35 金魚
きんぎん #T35 金銀
きんく #T35*1 禁句
きんぐ #T35*6 キング
きんけい #T35 謹啓 近景
きんけん #T35*1 金券 金権 #T35 近県
きんげん #T05 謹厳
きんこ #T35*1 金庫 #T30 禁固
きんこう #T30*1 均衡 #T35*1 近郊 #T35 金工 金鉱
きんこうけい #T35*1 近交系
きんこつ #T35 筋骨
きんこんしき #T35 金婚式
きんご #JNM 金吾
きんさ #T35 僅差
きんさく #T35*1 近作 #JNM 金作 #T30 金策
きんざい #T35 近在
きんし #T30*27 禁止 #T35*1 菌糸 近視
きんしつ #T05*1 均質
きんしゅ #T30*1 禁酒
きんしゅく #T30*1 緊縮
きんしょう #T05 僅少 #T35 金賞
きんしん #T30*1 謹慎 #T35 近親
きんじ #KS*33 禁じ #T30*19 近似 #JNM 金次 金治
きんじち #T35 近似値
きんじつ #T35*7 近日
きんじとう #T35*1 金字塔
きんじょ #T35*8 近所
きんじろう #JNM 金次郎
きんせい #T30*1 謹製 #KK*1 金星 #T35 均整 #T30 禁制 #T35 近世 金星
きんせつ #T30*1 近接
きんせん #T35*1 金銭
きんぜん #F02 欣然
きんそく #T35*5 禁則 #T30 禁足
きんそくれい #T30 禁足令
きんぞう #JNM 金蔵 金造
きんぞく #T35*2 金属 #T30 勤続
きんたい #T30 勤怠
きんたろう #JNM 金太郎
きんだい #T35*2 近代
きんだいち #JNS 金田一
きんだん #T30*1 禁断
きんちゃく #T35 近着
きんちょ #T35*1 近著
きんちょう #T30*6 緊張 #T30 金打
きんてい #T30 欽定 謹呈
きんてつ #KK*8 近鉄
きんとう #T05*2 均等
きんとん #T35 きんとん
きんにく #T35*2 筋肉
きんにくつう #T35 筋肉痛
きんねん #T35*6 近年
きんのすけ #JNM 金之助
きんぱく #T30 緊迫 #T35 金箔
きんぱつ #T35 金髪
きんぴん #T35 金品
きんぶん #T30 均分
きんべん #T05*1 勤勉
きんぺん #T35*6 近辺
きんぼう #T35*2 近傍
きんぼし #T35 金星
きんぽう #JCN 金峰
きんまく #T35 筋膜
きんみつ #T05*1 緊密
きんみゃく #T35 金脈
きんむ #T30*12 勤務
きんむさき #T35*2 勤務先
きんもつ #T35 禁物
きんや #JNM 欽也 金弥
きんゆ #T30 禁輸
きんゆう #T35*9 金融
きんゆうぎょう #T35 金融業
きんよう #T35*7 金曜
きんようび #T35*60 金曜日
きんよく #T30 禁欲 #T35 金欲
きんらい #T35 近来
きんり #T35*1 金利
きんりょう #T35 禁漁 禁猟
きんりょく #T35 筋力 金力
きんりん #T35*1 近隣
きんろう #T30*2 勤労
ぎ #KJ*102 技 #T35*54 義 #T35*8 儀 #KJ*4 偽 #S5*4 議 #T30*4 議 #R5r*4 切 #T05*2 偽 #KJ*2 蟻 #KJ*1 妓 戯 #SUC*1 着 #KJ 伎 宜 擬 #SX 擬 #KJ 欺 犠 疑 祇 誼 祁
ぎあ #T35*4 ギア
ぎあく #T35*1 偽悪
ぎあん #T35 議案
ぎいち #JNM 義一
ぎいん #T35*1 議員 #T35 議院
ぎえんきん #T35 義援金
ぎおん #T35 擬音 #CN 祇園
ぎかい #T35*2 議会
ぎかん #T35*1 技官
ぎきょうしん #T35 義侠心
ぎきょく #T35*1 戯曲
ぎぎ #T35 疑義
ぎけつ #T30 議決
ぎけん #KK*4 技研
ぎこう #T35 技巧
ぎごく #T35 疑獄
ぎごちな #KY*1 ぎごちな
ぎごちのう #KYU*1 ぎごちのう
ぎさく #T30 偽作
ぎざ #T35*3 ギザ
ぎざぎざ #T00*1 ギザギザ
ぎし #T35*3 技士 技師 #T35 義肢
ぎしき #T35*1 儀式
ぎしゅ #T35 義手
ぎしょ #T35 偽書
ぎしょう #T30 偽称 偽証
ぎしん #T35 疑心
ぎしんあんき #T35*1 疑心暗鬼
ぎじ #T35*25 疑似 #T35*4 議事
ぎじどう #T35*2 議事堂
ぎじゅく #KK*19 義塾
ぎじゅつ #T35*217 技術
ぎじょう #T35 議場
ぎじろく #T35*21 議事録
ぎじん #T35 擬人
ぎすぎす #SX ぎすぎす
ぎせい #T35*4 犠牲
ぎせき #T35 議席
ぎぜん #T35 偽善
ぎそう #T30 偽装 艤装
ぎそく #T35 義足
ぎぞう #T30*3 偽造
ぎたー #T35*1 ギター
ぎだい #T35*3 議題
ぎちょう #T35*2 議長
ぎっしり #F04 ぎっしり
ぎてい #T30 議定
ぎなん #CN 岐南
ぎにんしん #T30*1 擬妊娠
ぎねん #T35 疑念
ぎのう #T35*1 技能
ぎのざ #CN 宜野座
ぎのわん #CN 宜野湾
ぎふ #CN 岐阜 #T35 義父
ぎふと #T30 ギフト
ぎふん #T35 義憤
ぎほう #T35*4 技法 #T35*2 技報
ぎぼ #T35 義母
ぎまん #T35*3 欺瞞
ぎみ #T35*9 気味
ぎむ #T35*4 義務
ぎむづけ #KSr*1 義務づけ
ぎめい #T35*6 偽名
ぎもん #T15*34 疑問
ぎもんし #T30 疑問視
ぎゃく #T35*80 逆 #KJ 虐
ぎゃくさつ #T30*1 虐殺
ぎゃくさん #T30*1 逆算
ぎゃくさんかく #T35*1 ▽ ▼
ぎゃくしゅう #T30*2 逆襲
ぎゃくじゅん #T35*3 逆順
ぎゃくじょう #T30 逆上
ぎゃくすう #T35*2 逆数
ぎゃくせつ #T30 逆接 #T35 逆説
ぎゃくたい #T30*1 虐待
ぎゃくてん #T30*5 逆転
ぎゃくふう #T35 逆風
ぎゃくもどり #T30 逆戻り
ぎゃくよう #T30 逆用
ぎゃくり #T35 逆理
ぎゃくりゅう #T30 逆流
ぎゃぐ #T35*3 ギャグ
ぎゃっきょう #T35 逆境
ぎゃっこう #T30*1 逆行
ぎゃっぷ #T35*3 ギャップ
ぎゃんぶる #T35*1 ギャンブル
ぎゅう #T35*1 牛
ぎゅうご #T35 牛後
ぎゅうにく #T35*1 牛肉
ぎゅうにゅう #T35*4 牛乳
ぎゅうば #T35 牛馬
ぎゅうほ #T30 牛歩
ぎょ #KJ*14 漁 #KJ*8 御 #KJ*1 魚 #KJ 禦
ぎょい #CJ*3 御意
ぎょう #T35*255 行 #JS*13 行 #KJ*4 暁 #KJ*1 仰 業 #KJ 凝 尭 #JN 暁 #SUC 業 #KJ 形 #SUC 行
ぎょうかい #T35*11 業界
ぎょうかく #T35*1 行革
ぎょうかん #T35*2 行間
ぎょうぎ #T35*2 行儀
ぎょうぎょうし #KY 仰々し
ぎょうぎょうしゅう #KYU 仰々しゅう
ぎょうけつ #T30 凝結 凝血
ぎょうこ #T30 凝固
ぎょうざ #T35*1 餃子
ぎょうし #T30 凝視
ぎょうしゃ #T35*11 業者
ぎょうしゅ #T35*1 業種
ぎょうしゅう #T30*1 凝集
ぎょうしゅく #T30*1 凝縮
ぎょうしょ #T35 行書
ぎょうしょう #T30 行商
ぎょうじ #T35*4 行事 #T35 行司
ぎょうずい #T30 行水
ぎょうせい #T35*4 行政
ぎょうせき #T35*2 業績
ぎょうそう #T35 形相
ぎょうだ #JCN 行田
ぎょうてん #T30 仰天
ぎょうとう #T35*4 行頭
ぎょうまつ #T35*18 行末
ぎょうむ #T35*54 業務
ぎょうれつ #T30*10 行列
ぎょかい #T35 魚介
ぎょがん #T35 魚眼
ぎょきょう #T35 漁協
ぎょぎょう #T35*1 漁業
ぎょく #T35*2 玉
ぎょくさい #T30 玉砕
ぎょくざ #T35 玉座
ぎょくせきこんこう #T35*1 玉石混淆
ぎょくとう #JCN 玉東
ぎょぐ #T35 漁具
ぎょぐん #T35 魚群
ぎょこう #T35 漁港
ぎょしょう #T35 漁礁
ぎょじ #T35 御璽
ぎょじょう #T35 漁場
ぎょせん #T35 漁船
ぎょそん #T35 漁村
ぎょにく #T35 魚肉
ぎょふ #T35 漁夫
ぎょぶつ #T35 御物
ぎょみん #T35 漁民
ぎょめい #T35 御名
ぎょらい #T35*1 魚雷
ぎょるい #T35 魚類
ぎょろぎょろ #F12*1 ぎょろぎょろ
ぎり #T35*1 義理 #T30 切り
ぎりぎり #T05*2 ぎりぎり
ぎりしゃ #CN ギリシャ
ぎりょう #T35*3 技量 #T35 技倆 #T30 議了
ぎれい #T35 儀礼
ぎろん #T30*80 議論
ぎわ #D2T35*1 際
ぎわく #T35*2 疑惑
ぎん #T35*52 銀 #KJ*2 吟 #ZX 吟
ぎんいろ #T35*1 銀色
ぎんえい #T30 吟詠
ぎんか #T35 銀貨
ぎんが #T35*2 銀河
ぎんぎん #T00*1 ギンギン
ぎんこう #T35*35 銀行
ぎんこんしき #T35 銀婚式
ぎんざ #CN 銀座
ぎんざん #T35*2 銀山
ぎんしょう #T30 吟唱 #T35 銀賞
ぎんせい #T35 銀製
ぎんぞう #JNM 銀蔵
ぎんばん #T35 銀盤
ぎんぱく #T35 銀箔
ぎんぱつ #T35 銀髪
ぎんまく #T35 銀幕
ぎんみ #T30*1 吟味
く #W5r*94 食 #T35*42 区 #KJ*22 久 #M5r*7 組 #NN*6 九 #M5*5 汲 #T35*4 句 #R5*4 繰 #T35*3 苦 #KJ*1 駆 #W5*1 喰 #KJ 供 倶 #JS 句 #CNSUC1 区 #JS 区 #KJ 狗 玖 矩 躯 駈 駒 口 孔 工 貢 #M5 酌
くーでた #T35 クーデタ
くーぽん #T35*1 クーポン
くーら #T35*1 クーラ
くーる #T05*1 クール
くい #KSr 悔い #JCN 久井 #T35 杭
くいーん #T35*1 クイーン
くいあ #W5r 食い合
くいい #R5 食い入
くいき #T35*2 区域
くいこ #M5r*1 食い込
くいさが #R5r 食い下が
くいず #T35*1 クイズ
くいちが #W5r*2 食い違
くいつ #K5r*1 食いつ
くいとめ #KSr*1 食い止め
くいんと #KK*2 くいんと
くう #T35*18 空 #KJ*1 腔
くうい #T35 空位
くういき #T35 空域
くうかん #T35*64 空間
くうき #T35*18 空気
くうきょ #T05 空虚
くうぎょう #T35*5 空行
くうぐん #T35*1 空軍
くうこう #T35*6 空港
くうしゃ #T35 空車
くうしゅう #T30 空襲
くうせき #T35*1 空席
くうぜん #T35 空前
くうそ #T05 空疎
くうそう #T30*2 空想
くうちゅう #T35*1 空中
くうちょう #T35*2 空調
くうてん #T30*1 空転
くうどう #T35 空洞
くうはく #T05*34 空白
くうふく #T35*1 空腹
くうほう #T35 空砲
くうぼ #T35 空母
くうゆ #T30 空輸
くうらん #T35 空欄
くうれい #T30*1 空冷
くうろ #T35 空路
くうろん #T35*1 空論
くぇすちょん #T35*1 クェスチョン
くぇっしょん #T35*1 クェッション
くえい #T35 区営
くえすと #T35*2 クエスト
くぉーと #T35*3 クォート
くおーと #T35*2 “ #T35*1 ‘ ’ ” #T30*1 クオート
くおん #T15 久遠
くか #JCN 久賀
くかく #T30*3 区画
くかん #T35*8 区間
くが #JCN 玖珂 #JNS 玖賀
くがく #T30*1 苦学
くき #T35*5 茎 #JCN 久喜
くきざき #JCN 茎崎
くきょう #T35*1 苦境
くぎ #R5r*34 区切 #T35*1 釘
くぎづけ #T30 釘付け
くぎの #JCN 久木野
くぎり #T35*8 区切り #T35 句切り
くく #R5*10 括
くぐ #R5*3 潜
くぐい #T35*1 鵠
くぐの #JCN 久々野
くけい #T35*5 矩形
くげん #T35 苦言
くさ #KYmi*20 臭 #R5*13 腐 #T35*7 草
くさか #JCN 日下
くさかべ #JCN 日下部
くさかり #T30 草刈り
くさき #T35 草木
くさつ #JCN 草津
くさの #JCN 草野
くさばな #T35 草花
くさむら #T35 叢
くさもち #T35 草餅
くさやなぎ #JNS 草柳
くさり #T35*1 鎖
くさわけ #T35*1 草分け
くし #T30*4 駆使 #T35*2 串 #T35 櫛 #KJ 釧
くしがた #JCN 櫛形
くしきの #JCN 串木野
くしくも #CJ*1 奇しくも
くしだ #JNS 串田
くしはら #JCN 串原
くしびき #JCN 櫛引
くしま #JCN 串間
くしもと #JCN 串本
くしゃくしゃ #T11 くしゃくしゃ
くしゃみ #T30*2 くしゃみ
くしやま #JNS 串山
くしゅう #T35 句集
くしょう #T30*1 苦笑
くしら #JCN 串良
くしろ #CN 釧路
くしん #T30*1 苦心
くじ #JCN 久慈
くじびき #T30*2 くじ引き
くじゃく #T35*1 孔雀
くじゅう #JCN 久住 #T30 苦渋
くじゅうくり #CN 九十九里
くじょ #T30*7 駆除
くじょう #T35*6 苦情
くじら #T35*3 鯨 鯀
くじらじゃく #T35 鯨尺
くす #T35*23 樟 #T35*3 楠 #JCN 玖珠 楠
くすくす #F14*2 くすくす
くすぐ #R5r*2 くすぐ
くすのき #JCN 楠
くすもと #JNS 楠本
くすり #T35*11 薬
くすりゆび #T35 薬指
くず #S5*4 崩 #T35*1 葛 屑
くずう #JCN 葛生
くずてつ #T35 屑鉄
くずは #JCN 楠葉
くずまき #JCN 葛巻
くずみ #JCN 久住
くずもち #T35 葛餅
くずれ #KSr*6 崩れ
くずれおち #KSr 崩れ落ち
くずれさ #R5 崩れ去
くせ #T35*10 癖 #JCN 久世
くせつ #T30*2 苦節
くせもの #T35*1 曲者
くせん #T30*1 苦戦
くぜ #JCN 久世 久瀬
くそ #T35*2 糞
くたくた #T11 くたくた
くだ #R5r*6 下 #S5*6 下 #T35*2 管 #R5 降 #K5 砕
くだくだし #KY くだくだし
くだくだしゅう #KYU くだくだしゅう
くだけ #KS*2 砕け
くださ #L5*255 くださ 下さ #R5 くださ 下さ
くだまつ #JCN 下松
くだもの #T35*1 果物 #T35 くだもの
くだらな #KY*1 下らな
くだらのう #KYU*1 下らのう
くだん #KJ*1 件 #CN 九段
くだんした #CN 九段下
くち #JS*44 口 #KS*1 朽ち #T35 口
くちあたり #T35*1 口当り
くちえ #T35 口絵
くちかず #T35*1 口数
くちがね #T35 口金
くちく #T30*1 駆逐
くちぐせ #T35*1 口ぐせ 口癖
くちぐるま #T35 口車
くちさがな #KY 口さがな
くちさがのう #KYU 口さがのう
くちすう #T35 口数
くちずさ #M5*2 口ずさ
くちづけ #T30*1 口づけ
くちのつ #CN 口之津
くちはばった #KY 口はばった
くちばし #R5*1 口走 #T35 喙 嘴
くちび #T35 口火
くちびる #T35*3 唇
くちべに #T35 口紅
くちもと #T35 口元
くちゅう #T30*1 駆虫 #T35 苦衷
くちょう #T35*1 区長 口調
くちわ #JCN 口和
くっきょう #T05 屈強
くっきょく #T30 屈曲
くっきり #F04 くっきり
くっさく #T30 掘削
くっし #T35 屈指
くっしゃろこ #CN 屈斜路湖
くっしょん #T35*1 クッション
くっしん #T30*1 屈伸
くっせい #T35*1 屈性
くっせつ #T30*2 屈折
くったく #T30 屈託
くっちゃん #CN 倶知安
くっつ #K5r*2 くっつ
くっつけ #KSr*2 くっつけ
くっぷく #T30 屈伏 屈服
くつ #T35*9 靴 #KJ*1 屈 掘 #KJ 窟 沓 堀
くつう #T05*3 苦痛
くつがえ #S5*2 覆 #R5 覆
くつき #JCN 朽木
くつした #T35 靴下
くつじゅう #T30 屈従
くつじょく #T35*1 屈辱
くつろ #G5r*1 寛
くつわ #T35 轡
くてん #T35*6 区点 #T35*4 句点
くとう #T30*1 苦闘
くとうてん #T35*3 句読点
くど #KY*3 くど #K5r 口説
くどう #T30*13 駆動 #KYU*1 くどう #JCN 久遠 #JNS 工藤
くどお #JCN 久遠
くどやま #CN 九度山
くない #T35*2 区内
くないちょう #KK*1 宮内庁
くなしり #CN 国後
くなん #T30 苦難
くに #T35*66 国 #KJ*2 邦 #KJ*1 國 #JNM 久邇 #CN 六合
くにい #JN 国井
くにえだ #JN 国枝
くにお #JNM 国男 国夫 国雄 邦男 邦夫 邦雄
くにがみ #JCN 国頭
くにがら #T35 国柄
くにく #T35*1 苦肉
くにぐに #T35*1 国々
くにこ #JNM 邦子
くにさき #JCN 国東
くにさだ #JNS 国定
くにたち #CN 国立
くにとみ #JCN 国富
くにひこ #JNS 国彦 #JNM 邦彦
くにひさ #JNM 邦久
くにひと #JNM 邦仁
くにひろ #JN 国博
くにみ #JCN 国見
くによし #JNM 國吉
くの #JNS 久野
くのう #T30*1 苦悩
くのへ #JCN 九戸
くば #R5*25 配
くび #T35*7 首 #T35 頚
くびき #JCN 頚城
くびすじ #T35 首筋
くびったけ #T15 首ったけ
くびっぴき #T35 首っ引き
くふう #T30*27 工夫
くぶくりん #T35 九分九厘
くぶん #T30*21 区分
くべ #KS*2 くべ
くべつ #T30*40 区別
くほんぶつ #CN 九品仏
くぼ #M5r くぼ #JNS 久保 #KJ 窪 #M5r 窪
くぼかわ #JCN 窪川
くぼさわ #JNS 久保沢
くぼた #KK*4 クボタ #JCN 久保田 #JNS 窪田
くま #T35*6 熊 #KJ*4 隈 #T35*1 々 #CN 久万 球磨
くまいし #JCN 熊石
くまがい #JNS 熊谷
くまがや #JCN 熊谷
くまくら #JNS 熊倉
くまげ #JCN 熊毛
くまさわ #JCN 熊沢
くまざわ #JCN 熊沢
くまだ #JNS 熊田
くまとり #JCN 熊取
くまなく #F14 くまなく
くまの #JCN 熊野
くまのがわ #CN 熊野川
くまばち #T35 熊蜂
くまもと #JCN 熊本
くまやま #JCN 熊山
くみ #JS*53 組 #T35*47 組 #JN 久美
くみあ #W5r 組み合
くみあい #T35*11 組合
くみあげ #KSr 汲み上げ 組み上げ
くみあわ #S5r*28 組み合わ
くみあわせ #T35*25 組合せ
くみいれ #KSr*2 組み入れ
くみかた #T35*1 組み方
くみこ #M5r*52 組み込 #JN 久美子
くみたて #KSr*8 組み立て
くみつ #K5r 組み付
くみと #R5r*1 汲み取 #R5r くみ取
くみなお #S5*1 組み直
くみはま #CN 久美浜
くみはん #T30*1 組版
くみやま #CN 久御山
くみん #T35*1 区民
くめ #JCN 久米 #KJ 粂
くめなん #CN 久米南
くめん #T30 工面
くも #T35*9 雲 #R5r*9 曇 #T35*1 蜘蛛
くもつ #T35 供物
くもら #S5 曇ら
くもり #T35 曇
くもん #JNS 久門 #T30 苦悶
くや #M5r*2 悔や
くやくしょ #T35*2 区役所
くやし #KYmi*2 悔し
くやしゅう #KYU*1 悔しゅう
くよう #T30*2 供養
くよくよ #F04 くよくよ
くら #KY*27 暗 #T35*14 倉 #S5r*12 暮ら #T35*2 鞍 #W5*2 食ら #T35*2 蔵 #M5*1 くら
くらい #T35*42 位 #T35*3 くらい
くらいあんと #T35*255 クライアント
くらいし #JCN 倉石
くらいつ #K5r 食らいつ
くらいまっくす #T35*1 クライマックス
くらがえ #T30 鞍替え
くらがり #T35 暗がり
くらく #T35 苦楽
くらくしょん #T35*3 クラクション
くらくら #F05 くらくら
くらげ #T35*1 クラゲ 鮓
くらさわ #JNS 倉沢
くらしき #JCN 倉敷
くらしっく #T05*2 クラシック
くらしのてちょう #KK 暮しの手帖
くらす #T35*107 クラス
くらすた #T35*25 クラスタ
くらた #JNS 倉田
くらたけ #JCN 倉岳
くらっか #T35 クラッカ
くらっしゅ #T35*21 クラッシュ
くらっち #T35*2 クラッチ
くらて #JCN 鞍手
くらはし #JCN 倉橋
くらはら #JNS 倉原
くらふと #T35*1 クラフト
くらぶ #T35*8 クラブ #T35*5 倶楽部
くらぶち #JCN 倉淵
くらべ #KS*55 比べ #KS*17 較べ #JNS 倉部
くらま #JCN 鞍馬
くらもち #JN 倉持
くらもと #JNS 倉本 蔵本
くらやみ #T35*2 暗闇
くらよし #JCN 倉吉
くり #T35*17 栗 #KJ 繰
くりーにんぐ #T30*5 クリーニング
くりーむ #T35*5 クリーム
くりーん #T05*2 クリーン
くりあ #T00*40 クリア
くりあが #R5r 繰り上が
くりあげ #KSr 繰り上げ
くりいれ #KSr 繰り入れ
くりかえ #S5r*80 繰り返
くりかえし #T35*5 々 #T35*1 ヽ ヾ ゝ ゞ
くりこ #S5r*2 繰り越 #M5r 繰り込
くりこし #T30*1 繰越
くりこま #JCN 栗駒
くりごと #T30 繰言
くりさが #R5r 繰り下が
くりさげ #KSr 繰り下げ
くりさわ #JCN 栗沢
くりすたる #T35*1 クリスタル
くりすます #T35*6 クリスマス
くりた #JCN 栗太 #JNS 栗田
くりだ #S5*1 繰り出
くりっく #T30*48 クリック
くりっぴんぐ #T30*3 クリッピング
くりっぷ #T30*2 クリップ
くりつ #T35*1 区立
くりにっく #T35*1 クリニック
くりの #JCN 栗野
くりのべ #T30*1 繰り延べ #KSr 繰り延べ
くりはし #JCN 栗橋
くりはら #JCN 栗原
くりばやし #JNS 栗林
くりひろげ #KS*1 繰り広げ
くりみあ #CN クリミア
くりもと #JCN 栗源 #JNS 栗本
くりやま #JCN 栗山 #T35 栗山
くりょ #T30 苦慮
くる #kxuru*247 来る #kxuru*21 くる #W5r*6 狂 #M5*1 くる
くるー #T35*2 クルー
くるくる #F14*1 くるくる
くるし #KYmi*22 苦し #M5r*11 苦し
くるしめ #KS*1 苦しめ
くるしゅう #KYU*2 苦しゅう
くるま #T35*107 車 #R5*2 くるま #T35*1 クルマ
くるまだい #T35 車代
くるまやま #JCN 車山
くるみ #T35 くるみ 胡桃
くるめ #CN 久留米
くれ #KS*32 くれ #kxure*28 来れ #kxure*22 くれ #KSr*9 暮れ #JCN 呉
くれーたー #T35*1 クレーター
くれーむ #T35*5 クレーム
くれおぱとら #JN クレオパトラ
くれぐれ #F14*2 くれぐれ
くれじっと #T30*2 クレジット
くれぞーる #T35 クレゾール
くれない #T35*1 紅
くれむりん #CN クレムリン
くろ #KYmi*30 黒 #T35*26 黒
くろーず #T30*21 クローズ
くろいし #JCN 黒石 #T35 黒石
くろいそ #JCN 黒磯
くろいわ #JCN 黒岩
くろう #T30*26 苦労 #KYU*2 暗う 黒う
くろうじ #JNS 黒氏
くろうと #T35*1 玄人
くろかわ #JCN 黒川
くろき #JNS 黒木
くろぐ #JNS 黒木
くろさき #JCN 黒崎
くろさわ #JNS 黒沢 黒澤
くろざとう #T35 黒砂糖
くろしお #T35 黒潮
くろじ #T35*1 黒字
くろす #T30*4 クロス #JNS 黒須
くろず #M5r 黒ず
くろせ #JCN 黒瀬
くろたき #JCN 黒滝
くろだ #JCN 黒田
くろだしょう #CN 黒田庄
くろっく #T35*10 クロック
くろつち #T35 黒土
くろばね #JCN 黒羽
くろべ #JCN 黒部
くろほね #JCN 黒保根
くろぼし #T35 黒星
くろまく #T35 黒幕
くろまつ #JCN 黒松
くろまつない #JCN 黒松内
くろまめ #T35 黒豆
くろめ #T35 黒目
くろやなぎ #JNS 黒柳
くろやま #T35*1 黒山
くわ #T35*7 桑 #T35*1 鍬
くわえ #KS*122 加え
くわけ #T30*4 区分け
くわし #KY*107 詳し #KY*1 くわし
くわしゅう #KYU*3 詳しゅう #KYU*1 くわしゅう
くわた #JNS 桑田
くわだ #JNS 桑田
くわだて #KSr 企て
くわな #JCN 桑名
くわはら #JNS 桑原
くわばら #JNS 桑原
くわり #T30 区割り
くわわ #R5*14 加わ
くん #JNSUC*255 君 #T35*4 訓 #T35*2 君 #ZX*2 訓 #KJ*1 勲 #KJ 釧 薫 #ZX 薫
くんいく #T30 訓育
くんかい #T30 訓戒
くんこく #T30 訓告
くんしゅ #T35*1 君主 #T30 暈酒
くんしょう #T35 勲章
くんじ #T30*1 訓示
くんじょう #T30 薫蒸 燻蒸
くんせい #T35 薫製 燻製
くんとう #T30 薫陶
くんどう #T30 訓導
くんねっぷ #CN 訓子府
くんぷう #T35 薫風
くんよみ #T30*1 訓読み
くんりん #T30*1 君臨
くんれい #T30*1 訓令
くんれん #T30*3 訓練
くんわ #T30 訓話
ぐ #T30*46 具 #T35*3 愚 #KJ*1 倶 愚 虞 #KJ 惧
ぐあい #T15*27 具合 #T35 具合い
ぐあてまら #CN グアテマラ
ぐう #KJ*3 偶 #KJ*2 宮 #S5*1 遇 #T30*1 遇 #S5 寓 #T30 寓 #KJ 隅
ぐういん #T35 偶因
ぐうじ #T35 宮司
ぐうすう #T35*6 偶数
ぐうぜん #T35*3 偶然
ぐうぞう #T35 偶像
ぐうはつ #T30 偶発
ぐきょ #T35*1 愚挙
ぐこう #JN 愚公 #T30 愚考
ぐさ #D2T35*10 種
ぐさりと #F14 ぐさりと
ぐざい #T35*1 Ξ ξ
ぐしかみ #JCN 具志頭
ぐしかわ #JCN 具志川
ぐしゃ #T35 愚者
ぐしゃぐしゃ #T05 ぐしゃぐしゃ
ぐしょう #T35*1 具象
ぐしん #T30 具申
ぐじょう #CN 郡上
ぐすくべ #JCN 城辺
ぐず #R5r ぐず #T05 愚図
ぐせい #T35 愚生
ぐたい #T35*7 具体
ぐたいか #T30*3 具体化
ぐたいてき #T05*83 具体的
ぐたいれい #T35 具体例
ぐち #CNSUC1*34 口 #T35*6 愚痴 #SUC 口
ぐちゃぐちゃ #T05 ぐちゃぐちゃ
ぐちょく #T05 愚直
ぐっしょり #F06 ぐっしょり
ぐっと #F14*1 ぐっと
ぐっど #T05 グッド
ぐどん #T05 愚鈍
ぐび #T30 具備
ぐまい #T05 愚昧
ぐみ #JNSUC*31 組
ぐらいだー #T35*1 グライダー
ぐらうんど #T35*15 グラウンド
ぐらす #T35*1 グラス
ぐらふ #T35*27 グラフ
ぐらふぃかる #T05*23 グラフィカル
ぐらふぃくす #T35*2 グラフィクス
ぐらふぃっく #T35*10 グラフィック
ぐらふぃっくす #T35*19 グラフィックス
ぐらまー #T05*4 グラマー
ぐらむ #JS*29 グラム #T35*29 グラム
ぐらんど #T35*6 グランド
ぐりーん #T05*4 グリーン
ぐりかん #T35*1 グリカン
ぐりこーげん #T35*1 グリコーゲン
ぐりせりん #T35*1 グリセリン
ぐりっでぃんぐ #T30*2 グリッディング
ぐりっど #T35*22 グリッド
ぐりる #T30 グリル
ぐる #T35*3 ぐる
ぐるーぴんぐ #T30*4 グルーピング
ぐるーぷ #T35*255 グループ
ぐるい #N2T30*1 狂い
ぐるぐる #F04*1 ぐるぐる
ぐるこーす #T35*1 グルコース
ぐるここるちこいど #T35*1 グルココルチコイド
ぐるし #KY*1 苦し #M5*1 苦し
ぐるしゅう #KYU*1 苦しゅう
ぐるめ #T35*1 グルメ
ぐるり #T35 ぐるり
ぐれ #KJ*1 昏
ぐれー #T05*3 グレー
ぐれーぷ #T35*1 グレープ
ぐれい #T05*1 グレイ
ぐれん #T35 紅蓮
ぐろ #KYmi*3 黒
ぐろーばる #T05*19 グローバル
ぐろう #KYU*1 黒う #T30 愚弄
ぐろっきー #T15 グロッキー
ぐん #T35*16 郡 #T35*11 群 #SUC 群 #CNSUC1 軍 #T30 軍 #CNSUC1 郡
ぐんい #T35 軍医
ぐんか #T35 軍歌
ぐんかく #T30 軍拡
ぐんかん #T35 軍艦
ぐんきょ #T30 群居
ぐんぐん #F06*1 ぐんぐん
ぐんこう #T35 軍港
ぐんこく #T35*1 軍国
ぐんしきん #T35 軍資金
ぐんしゅう #T35 群衆 群集
ぐんしゅく #T30*1 軍縮
ぐんしょう #T35 群小
ぐんしん #T35 軍神
ぐんじ #T30*1 軍事 #JN 郡司
ぐんじゅ #T35 軍需
ぐんじん #T35*2 軍人
ぐんせい #T30 群生
ぐんそう #T35*1 軍曹
ぐんぞう #T35 群像
ぐんたい #T35*1 軍隊
ぐんだん #T35 軍団
ぐんと #F14 ぐんと
ぐんとう #T35 群島
ぐんばい #T35 軍配
ぐんぱつ #T30 群発
ぐんび #T30 軍備
ぐんぶ #T30 群舞 #T35 郡部
ぐんま #JCN 群馬
ぐんもう #T35*1 群盲
ぐんゆう #T35 群雄
ぐんよう #T35*1 軍用
ぐんらく #T35 群落
け #S5*112 消 #JNSUC*90 家 #KJ*12 ヶ #KJ*8 気 #R5r*6 蹴 #T35*6 毛 #KJ*1 華 卦 #KJ 化 仮 家 袈 祁 懸
けーき #T35*9 ケーキ
けーじ #T35*1 ケ−ジ
けーす #T35*22 ケース
けーぱびりてぃ #T35*22 ケーパビリティ
けーぶる #T35*40 ケーブル
けい #T35*56 系 #KJ*33 計 #KJ*22 罫 #S5*17 啓 #T30*17 啓 #KJ*16 形 #KJ*10 軽 #S5*4 慶 #T30*4 慶 #S5*4 敬 #T30*4 敬 #KJ*3 経 #KJ*2 佳 掲 景 #KJ*1 京 卿 係 径 渓 蛍 警 #PRE*1 軽 #KJ*1 鶏 勁 #KJ 馨 掛 競 境 傾 #T35 刑 #KJ 兄 #JNM 圭 #KJ 珪 型 契 #SUC 形 #JNM 恵 #KJ 慧 憩 携 桂 畦 稽 #SUC 系 #KJ 継 繋 茎 荊 #SUC 計 #KJ 詣 頚
けいあい #T30 敬愛
けいい #T35*4 経緯 #T30*2 敬意
けいいち #JNM 啓一 圭一 恵一 慶一 敬一
けいいん #T30 契印
けいえい #T30*16 経営
けいえん #T30*1 敬遠
けいおう #KK*81 慶応 #KK*6 慶應 #KK*2 京王
けいおうぎじゅく #KK*6 慶應義塾
けいか #T30*18 経過
けいかい #T30 警戒 #T05 軽快
けいかく #T30*79 計画
けいかん #T35*1 警官 頚管 #T35 景観
けいが #T30*1 慶賀
けいがい #T35*1 形骸
けいがく #KK*2 啓学
けいがん #T05 慧眼
けいき #T35*1 契機 景気 #T35 刑期 計器
けいきょもうどう #T30 軽挙妄動
けいぐ #T35*3 敬具
けいけい #F02 炯々
けいけん #T30*52 経験 #T05 敬虔
けいげん #T30*17 軽減
けいこ #T30*3 けいこ #T30*1 稽古 #JNM 啓子 圭子 恵子 慶子 敬子 景子 桂子
けいこう #T35*8 傾向 #T30 携行 #T35 蛍光 鶏口
けいこうぎょう #T35 軽工業
けいこうとう #T35*2 蛍光灯
けいこく #T30*15 警告 #T35*10 渓谷
けいご #T35*1 敬語 #JNM 啓吾 #T30 警護
けいごう #T30 契合
けいさい #T30*17 掲載
けいさつ #T35*6 警察
けいさつかん #T35 警察官
けいさつしょ #T35 警察署
けいさつちょう #KK 警察庁
けいさん #T30*94 計算 #T35*1 経産
けいさんき #T35*147 計算機
けいさんぎ #T35*1 計算木
けいさんしょ #T35 計算書
けいざい #T35*16 経済
けいざいかい #T35*1 経済界
けいざいめん #T35*1 経済面
けいし #T30*1 軽視 #T35 警視
けいしき #T35*255 形式
けいしちょう #KK*3 警視庁
けいしつ #T35 形質
けいしゃ #T30*1 傾斜
けいしゅく #T30 慶祝
けいしゅつ #T30*1 掲出
けいしょう #T30*43 継承 #T35*13 敬称 #T35*1 警鐘 軽傷 #T35 形象 景勝 軽症
けいしょく #T30*1 軽食
けいしん #T35 軽震
けいじ #T30*7 掲示 #T35*2 刑事 #T35*1 経時 #JNM 啓司 啓二 圭二 #T35 慶事 #JNM 慶治 敬二
けいじじょう #T35*2 形而上
けいじょう #T35*8 形状 #T30*4 計上 #T35*1 経常 #T30 啓上
けいす #T35*1 ケイス
けいすい #T35 軽水
けいすう #T35*2 係数 #T35*1 計数
けいすけ #JNM 佳佑 佳祐 啓介
けいず #T35 系図
けいせい #T30*7 形成 #T35 傾城 形勢
けいせき #T35*1 形跡
けいせつ #T35 蛍雪
けいせん #T35*42 ─ #T35*35 罫線 #T35*24 │ #T35*4 ┌ #T35*3 ┃ #T35*2 ━ #T35*1 経線 #JCN 桂川
けいそ #T35 硅素
けいそう #T30*1 軽装 #T30 係争 #T35 形相
けいそく #T30*10 計測
けいそつ #T05*1 軽率
けいぞう #JNM 啓三 恵三 #T30 恵贈 #JNM 敬三
けいぞく #T30*28 継続
けいた #JNM 慶太
けいたい #T30*42 携帯 #T35*32 形態
けいたいそ #T35*7 形態素
けいだい #T35*1 境内 #KK*1 慶大 #T30*1 継代
けいだんれん #KK*1 経団連
けいちつ #T35*1 啓蟄
けいちゅう #T30*1 傾注
けいちょう #T35*4 慶弔 #T30 傾聴
けいちょうふはく #T05 軽佻浮薄
けいつい #T35*1 頚椎
けいてい #T35 兄弟
けいてき #T35 警笛
けいでんき #T35*1 継電器
けいと #T35*1 毛糸
けいとう #T35*19 系統 #T30 傾倒
けいど #T05*1 軽度 #T35 経度
けいねん #T35*1 経年
けいはく #T05*1 軽薄
けいはつ #T30*2 啓発
けいはん #KK*6 京阪 #CN 京阪
けいはんざい #T35 軽犯罪
けいはんしん #CN 京阪神
けいば #T35*2 競馬
けいばい #T30 競売
けいばつ #T35 刑罰 閨閥
けいひ #T35*10 経費
けいひん #T35*1 景品 #CN 京浜
けいび #T30*1 警備 #T15 軽微
けいふ #T35*1 系譜
けいふく #KK*1 京福 #T30*1 敬服
けいぶ #T35 警部
けいべつ #T30*1 軽蔑
けいべん #T05 軽便
けいほう #T35 刑法 #T30 警報
けいほく #CN 京北
けいぼう #T35 閨房
けいみょう #T05 軽妙
けいむしょ #T35 刑務所
けいもう #T30 啓蒙
けいやく #T30*15 契約
けいやくしょ #T35*3 契約書
けいゆ #T30*41 経由 #T35*1 軽油
けいよ #T30 恵与
けいよう #T30*35 形容 #T30*1 掲揚 #CN 京葉
けいようし #T35*22 形容詞
けいようせん #KK 京葉線
けいらん #T35 鶏卵
けいり #T30*6 経理
けいりゃく #T35 計略
けいりゅう #T30*1 稽留 #T30 係留 #T35 渓流 #T30 繋留
けいりょう #T30*4 計量
けいりん #T35*1 競輪
けいるい #T35*1 係累
けいれい #T30 敬礼
けいれき #T35*1 経歴
けいれつ #T35*23 系列
けいれん #T30*1 痙攣
けいろ #T35*85 経路
けいろう #T35*2 敬老
けおと #S5r け落と 蹴落と
けおりもの #T35 毛織物
けが #T30*5 怪我 #S5*1 汚
けがらわし #KY 汚らわし
けがらわしゅう #KYU 汚らわしゅう
けがれ #KSr 汚れ
けがわ #T35 毛皮
けさ #T35*34 今朝 #T35*1 袈裟
けし #T35*2 消し #T35 芥子
けしいん #T35*4 消印
けしき #T35*2 景色 #T35 気色
けしきば #M5 気色ば
けしごむ #T35 消しゴム
けしとめ #KSr 消し止め
けしょう #T30*3 化粧
けしょうひん #T35*3 化粧品
けしん #T30*1 化身
けず #R5r*9 削
けずりと #R5*2 削り取
けせん #CN 気仙
けせんぬま #CN 気仙沼
けそう #T30 懸想
けた #JS*22 桁 #T35*13 桁
けたか #CN 気高
けたぐ #R5r けたぐ
けたたまし #KY けたたまし
けたたましゅう #KYU けたたましゅう
けたちがい #T05*1 桁違い
けだか #KY 気高
けだこう #KYU 気高う
けっ #S5 決 #SX 決
けっか #T30*255 結果 #T30 欠課
けっかい #T30 決壊 #T35 結界
けっかく #T35*1 欠格 結核
けっかん #T35*1 欠陥 #T35 血管
けっき #T30 決起
けっきゅう #T35*1 血球
けっきょく #T35*103 結局
けっきん #T30 欠勤
けっこう #T05*255 結構 #T35*4 血行 #T05 けっこう #T30 欠航 決行
けっこん #T30*32 結婚 #T35 血痕
けっこんしき #T35*17 結婚式
けっさい #T30*52 決裁 #T30 決済
けっさく #T05*3 傑作
けっさつ #T30*1 結紮
けっさん #T30*2 決算
けっし #T35 決死
けっしきそ #T35*1 血色素
けっして #F14*29 決して #F14*1 けっして
けっしゃ #T35 結社
けっしゅう #T30*2 結集
けっしゅつ #T30 傑出
けっしょう #T35*26 決勝 #T30*3 結晶 #T35*1 血症 血漿
けっしょうせん #T35*2 決勝戦
けっしょうばん #T35 血小板
けっしょく #T30 欠食 #T35 血色
けっしん #T30*3 決心 #T30*1 結審
けっせい #T30*2 結成 #T35*1 血清
けっせき #T30*9 欠席 #T35 結石
けっせつ #T35*1 結節
けっせん #T30*1 決戦 #T35 血栓
けっそう #T35*1 血相
けっそく #T30 結束
けっそん #T30 欠損
けったく #T30 結託
けっちゃく #T30*2 決着
けっちゅう #T35*1 血中
けっちょう #T35*1 結腸
けっちん #T35 血沈
けってい #T30*181 決定
けってん #T35*24 欠点
けっとう #T35*4 血糖 #T30 決闘 結党 #T35 血統
けっとば #S5r 蹴っ飛ば
けっぱく #T05 潔白
けっぺき #T05*1 潔癖
けつ #T35*4 決 #KJ*2 欠 血 #KJ*1 傑 結 #KJ 桔 潔 穴 訣 蕨
けつあつ #T35*1 血圧
けつい #T30*2 決意
けついん #T35 欠員
けつえき #T35*1 血液
けつえきがた #T35*3 血液型
けつえん #T35*1 血縁
けつぎ #T30*1 決議
けつごう #T30*63 結合
けつごうし #T35*2 結合子
けつじつ #T30*1 結実
けつじょ #T30*2 欠如
けつじょう #T30*1 欠場
けつぜい #T35 血税
けつぞく #T35 血族
けつだん #T30*2 決断 #T30 結団
けつばん #T35*1 欠番
けつべつ #T30 決別 訣別
けつぼう #T30*1 欠乏
けつまくえん #T35 結膜炎
けつまつ #T35*2 結末
けつみゃく #T35 血脈
けつめい #T30 結盟
けつらく #T30*2 欠落
けつりゅう #T35 血流
けつれい #T30 欠礼
けつれつ #T30 決裂
けつろ #T30*1 結露
けつろん #T30*34 結論
けとば #S5r 蹴飛ば
けど #CJ*241 けど
けどういん #JCN 祁答院
けな #S5*1 けな #S5 貶
けなげ #T05*2 健気
けなみ #T35 毛並
けねでぃ #JN ケネディ
けねん #T30*2 懸念
けはい #T35*4 気配
けびょう #T35*1 仮病
けぶか #KY 毛深
けぶこう #KYU 毛深う
けむ #R5*2 煙
けむし #T35 毛虫
けむた #KY 煙た
けむとう #KYU 煙とう
けむり #T35*2 煙
けもの #T35*1 獣
けやき #T30 欅
けらい #T35 家来
けり #T35*1 けり
けれど #CJ*3 けれど
けれども #CJ*2 けれども
けわし #KY*2 険し
けわしゅう #KYU*1 険しゅう
けん #T35*255 件 #KJ*27 権 #T35*16 券 #CNSUC1*11 県 #KJ*10 研 #T35*5 軒 #KJ*4 憲 #T30*4 検 #T35*3 健 剣 #KJ*2 堅 建 見 #KJ*1 倹 兼 圏 献 #T35*1 賢 #KJ*1 鍵 険 #T30*1 験 #KJ 絢 鰹 巻 #JS 間 #KJ 間 #JS 件 #KJ 倦 #JNM 健 #SUC 兼 券 #KJ 喧 #SUC 圏 #KJ 嫌 懸 拳 捲 #SUC 権 #KJ 牽 犬 #ZX 献 #JNSUC 研 #SUC 研 #KJ 硯 絹 #JS 県 #T35 県 #KJ 肩 謙 #JNM 賢 #JS 軒 #KJ 遣 顕 鹸 繭
けんあく #T05 険悪
けんあん #T35*3 懸案 #T30 検案
けんい #T35*2 権威
けんいち #JNM 健一 剣一 憲一 研一 謙一 賢一
けんいん #T30 検印 牽引
けんえい #T30 兼営 #T35 県営
けんえき #T30 検疫
けんえつ #T30*1 検閲
けんえん #T30*8 嫌煙 #T35 犬猿
けんお #T30*16 嫌悪
けんおん #T30 検温
けんか #T30*3 喧嘩 #T35*1 県下
けんかい #T35*3 見解 #T35 県会
けんかく #T30 懸隔
けんかごし #T15 喧嘩腰
けんがい #T35*1 圏外 #T35 懸崖 県外
けんがく #T30*35 見学
けんがん #T30 検眼
けんき #T35*1 嫌気
けんきち #JNM 健吉
けんきゃく #T05 健脚
けんきゅう #T30*196 研究
けんきゅうかい #T35*14 研究会
けんきゅうしつ #T35*143 研究室
けんきゅうしょ #T35*117 研究所
けんきゅうじょ #T35*24 研究所
けんきょ #T30*1 検挙 #T05*1 謙虚
けんきん #T30 献金
けんぎ #T35*2 嫌疑 #T30 建議 #T35 県議
けんぎゅう #JN 牽牛
けんぎょう #T30 兼業
けんけい #T35*1 県警
けんけつ #T30*2 献血
けんげん #T35*3 権限 #T30*1 顕現
けんこう #T05*14 健康 #T35*2 軒昂
けんこうほけん #T35*3 健康保険
けんこく #T30 建国
けんご #T05 堅固
けんさ #T30*109 検査
けんさく #T30*85 検索 #T30 献策 #T35 研削
けんさつ #T30*1 検察 賢察 #T30 検札
けんさつかん #T35 検察官
けんさつちょう #KK 検察庁
けんざい #T05*1 健在 #T35 建材 #T30 顕在
けんざぶろう #JNM 健三郎
けんざん #T30 検算
けんし #T30 検死 検視 #T35 犬歯
けんしき #T35*1 見識
けんしゅう #T30*24 研修 #T30*17 検収
けんしゅつ #T30*28 検出
けんしょう #T30*196 検証 #T35*1 腱鞘 #T05 健勝 #T35 憲章 懸賞 肩章
けんしょく #T30 兼職
けんしん #T30*2 検診 #T30*1 献身 #T30 検針
けんじ #T35*1 健児 #T30*1 堅持 顕示 #JNM 健司 健次 健治 健二 健之 建司 憲司 憲治 憲二 #T35 検事 献辞 #JNM 研治 謙治 謙二 賢司 賢次 賢治 賢二
けんじつ #T05 堅実
けんじゃ #T35 賢者
けんじゅう #T35 拳銃
けんじょう #T30 献上 #T05 謙譲
けんじろう #JNM 健次郎
けんじん #T35*1 賢人 #T35 県人
けんすい #T30 懸垂
けんすう #T35*25 件数 #T35 軒数
けんすけ #JNM 健介 堅助
けんせい #T30*2 牽制 #T35 権勢 県政
けんせつ #T30*6 建設
けんせつしょう #KK 建設省
けんせん #T35 健羨
けんぜん #T05*3 健全
けんそ #T05*1 倹素 #T05 険阻
けんそう #T35*1 喧騒
けんそん #T30 謙遜
けんぞう #T30*1 建造 #JNM 健三 健蔵 謙三 賢三
けんぞうぶつ #T35 建造物
けんぞく #T35 眷族
けんたい #T35*1 検体 #T30 倦怠
けんたいかん #T35 倦怠感
けんたいき #T35 倦怠期
けんたっきー #CN ケンタッキー
けんたろう #JNM 健太郎 謙多郎
けんたん #T05 健啖
けんち #T35*4 見地 #T30 検知
けんちく #T30*2 建築
けんちくし #T35 建築士
けんちょ #T05*4 顕著
けんちょう #T05*3 健聴 #T35*1 県庁
けんてい #T30*3 検定 #T30 献呈
けんでん #T30*1 喧伝
けんとう #T30*255 検討 #T30*14 見当 #T30*5 健闘
けんとし #T35 ケント紙
けんどう #T35*1 剣道 #T35 県道
けんどじゅうらい #T30 捲土重来
けんどちょうらい #T30 捲土重来
けんない #T35*1 県内 #T35 圏内
けんなん #T05 険難
けんにょう #T30 検尿
けんにん #T30 兼任
けんのう #T30 献納
けんのん #T05 剣呑
けんばいき #T35 券売機
けんばん #T35 鍵盤
けんぱ #T30*1 検波
けんび #T30*1 兼備 #T35*1 顕微
けんびきょう #T35 顕微鏡
けんぶち #CN 剣淵
けんぶつ #T30*1 見物
けんぶりっじ #CN ケンブリッジ
けんぶん #T30*1 見聞 #T30 検分
けんぷ #T35 絹布
けんべん #T30 検便
けんぺいりつ #T35 建蔽率
けんぼうしょう #T35 健忘症
けんぽ #T35*2 健保
けんぽう #T35*3 憲法 #T35 拳法
けんま #T30 研磨
けんみん #T35 県民
けんむ #T30 兼務
けんめい #T35*15 件名 #T05*4 賢明 #T05*3 懸命
けんもん #T30 検問
けんやく #T30 倹約
けんよう #T30*2 兼用
けんり #T35*20 権利
けんりつ #T35*1 県立
けんりょう #T35 見料
けんりょく #T35*3 権力
けんろう #T05 堅牢
げ #T35*43 下 #KJ 解 外
げーて #JN ゲーテ
げーでる #JN ゲーデル
げーと #T35*22 ゲート
げーとあれい #T35*2 ゲートアレイ
げーとうぇい #T35*94 ゲートウェイ
げーむ #T30*37 ゲーム
げい #T35*4 芸 #KJ*2 鯨 #KJ*1 睨 藝 #KJ 迎
げいいんばしょく #T30 鯨飲馬食
げいげき #T30 迎撃
げいごう #T30*1 迎合
げいごと #T35*2 芸事
げいしゃ #T35*1 迎車 #T35 芸者
げいしゅん #T35 迎春
げいじゅつ #T35*8 芸術
げいせい #CN 芸西
げいつ #JN ゲイツ
げいとう #T35*1 芸当
げいにん #T35*1 芸人
げいのう #T35*1 芸能 #CN 芸濃
げいのうじん #T35*1 芸能人
げいは #T35 鯨波
げいひん #T35 迎賓
げいひんかん #T35*1 迎賓館
げいほく #CN 芸北
げいめい #T35 芸名
げか #T35*2 外科
げかい #T35*3 下界 #T35*1 外界 #T35 外科医
げかん #T35*1 下巻
げき #T35*1 劇 #KJ*1 撃 #S5*1 激 #T30*1 激 #KJ 逆 戟 隙 #T35 檄
げきか #T30 激化
げきから #T05*2 激辛
げきが #T35 劇画
げきげん #T30*3 激減
げきさん #T30 激賛
げきしょ #T35 激暑
げきしょく #T35 激職
げきしん #T30 激震
げきじょう #T35*5 劇場 #T30 激情
げきせん #T30 激戦
げきぞう #T30 激増
げきたい #T30*1 撃退
げきだん #T35*1 劇団
げきつい #T30 撃墜
げきつう #T35 激痛
げきてき #T05*2 劇的
げきとう #T30 激闘
げきとつ #T30 激突
げきど #T30*1 激怒
げきどう #T30 激動
げきは #T30 撃破
げきはつ #T30*1 激発
げきへん #T30 激変
げきむ #T35 激務
げきめつ #T30 撃滅
げきやく #T35 劇薬
げきりゅう #T35 激流
げきりん #T35 逆鱗
げきれい #T30*1 激励
げきれつ #T05*1 激烈
げきろん #T30*1 激論
げこ #T35*1 下戸
げこう #T30*1 下校
げざい #T35 下剤
げざん #T30 下山
げし #T35 夏至
げしゃ #T30*2 下車
げしゅく #T30*5 下宿
げじゅん #T35*6 下旬
げすい #T35 下水
げすいどう #T35*2 下水道
げすと #T35*1 ゲスト
げせん #T30 下船
げた #T35*4 下駄 #T35*1 〓
げだん #T35*1 下段
げっか #T35*1 月下
げっかん #T35*22 月刊 #T35*1 月間
げっきゅう #T35 月給
げっけい #T35 月経
げっけいじゅ #T35*1 月桂樹
げっこう #T30 激昂 #T35 月光
げっさん #T35 月産
げっしゃ #T35*3 月謝
げっしゅう #T35 月収
げっしょく #T35 月食
げっしるい #T35*1 げっ歯類 齧歯類
げっぷ #T35 月賦
げっぺい #T35*3 月餅
げっぽう #T35*3 月俸 月報
げつ #KJ*120 月
げつがく #T35*1 月額
げつじ #T35*1 月次
げつまつ #T35*5 月末
げつよう #T35*30 月曜
げつようび #T35*43 月曜日
げつり #T35 月利
げつれい #T35*1 月例 #T35 月齢
げどく #T30 解毒
げねつ #T30 解熱
げねつざい #T35 解熱剤
げのむ #T35*1 ゲノム
げば #T30*1 下馬
げひん #T05*1 下品
げらく #T30 下落
げり #T35 下痢
げりゃく #T35 下略
げりら #T35*1 ゲリラ
げる #T35*1 ゲル
げるちん #T35*1 ゲル沈
げれつ #T05*2 下劣
げれんで #T35*1 ゲレンデ
げろ #CN 下呂
げん #JS*20 元 #T35*17 言 #T35*12 現 #KJ*11 原 限 #T35*4 減 #ZX*3 減 #KJ*3 源 #T35*2 弦 玄 #KJ*1 幻 #ZX*1 現 #KJ*1 言 #KJ 元 #PRE 原 #KJ 厳 #JS 弦 #SUC 源 #JN 玄 #PRE 現 #KJ 絃 舷 諺 #SUC 限
げんあつ #T30 減圧
げんあん #T35*3 原案
げんいん #T30*131 原因
げんえい #T35 幻影
げんえき #T35*2 現役 #T35 原液 #T30 減益
げんか #T35*2 原価 #SUC*1 喧嘩 #T35*1 現下 #T30 減価 #T35 言下
げんかい #T35*26 限界 #T30*1 厳戒 #T35 厳戒 #CN 玄海
げんかいち #T35 限界値
げんかく #T05*1 厳格 #T35*1 幻覚
げんかしょうきゃく #T30*1 減価償却
げんかん #T35*7 玄関 #T35*1 厳寒
げんが #T35 原画
げんがい #T35 言外
げんがく #T35*1 弦楽 #T30*1 減額 #T35*1 衒学
げんき #T05*118 元気 #T35*1 原基
げんきゃく #T30 減却
げんきゅう #T30*2 言及 #T30 減給
げんきょう #T35*1 現況 #T35 元凶
げんきん #T05*9 現金 #T30 厳禁
げんぎょう #T35*1 現業
げんけい #T35*1 原型 原形 #T30 減刑
げんこう #T35*171 原稿 #T35*6 現行 #T35 言行
げんこく #T35 原告
げんご #T35*49 言語 #T35*3 原語
げんごう #T35*1 元号
げんさい #T30*15 減殺
げんさく #T35*4 原作
げんさん #T35*1 原産 #T30 減産
げんさんち #T35 原産地
げんざい #T35*255 現在 #T35 原罪
げんざいりょう #T35 原材料
げんざん #T30*1 減算
げんし #T35*17 原始 #T35*1 原子 #T35 原紙 #T30 幻視 減資
げんしかく #T35*1 原子核
げんしゅ #T30*7 厳守 #T35 元首 原種 原酒
げんしゅう #T30*1 減収
げんしゅく #T05 厳粛
げんしょ #T35*1 原書 #T35 原初
げんしょう #T35*57 現象 #T30*12 減少
げんしょく #T35 原色 #T30 減食 #T35 現職
げんしりょく #T35*10 原子力
げんしろ #T35*2 原子炉
げんじ #T35*3 源氏 #JNM 源治
げんじつ #T35*25 現実
げんじつみ #T35*2 現実味
げんじてん #T35 現時点
げんじゅう #T05*2 厳重
げんじゅうしょ #T35*1 現住所
げんじゅうみん #T35 原住民
げんじょう #T35*96 現状 #T35 原状 #T30 言上
げんすい #T35*1 元帥 #T30*1 減衰 #T30 減水
げんすいばく #T35 原水爆
げんすう #T30 減数
げんすん #T35 原寸
げんず #T35 原図
げんせ #T35 現世
げんせい #T05*1 厳正
げんせき #T35*1 原石
げんせつ #T35*1 言説
げんせん #T35*4 源泉 #T30 厳選
げんぜい #T30*3 減税
げんぜん #F03*1 厳然
げんそ #T35*1 元素
げんそう #T35*3 幻想 #T30 現送
げんそく #T35*33 原則 #T30*1 減速
げんそん #T30 減損 現存
げんぞう #T30 現像
げんぞく #T30 還俗
げんぞん #T30*2 現存
げんたい #T30*2 減退
げんたつ #T30 厳達
げんたん #T30 減反
げんだい #T35*5 現代 #T35*3 原題
げんだいじん #T35 現代人
げんち #T35*3 現地 #T35 言質
げんちゅう #T35*1 原虫
げんちょ #T35*1 原著
げんちょう #T35 幻聴
げんつき #T35*4 原付
げんてい #T30*71 限定
げんていし #T35*1 限定子
げんていばん #T35 限定版
げんてん #T35*4 原点 #T35*1 原典 #T30*1 減点
げんとう #T35 厳冬 幻灯
げんど #T35*4 限度
げんどう #T35*2 言動 #T35*1 原動
げんどうき #T35 原動機
げんどうりょく #T35*1 原動力
げんに #F14*2 現に #F14*1 げんに
げんにん #T35 現任
げんば #T35*11 現場
げんばく #T35 原爆
げんばつ #T30 厳罰
げんばん #T35 原盤
げんぱい #T30 減配
げんぱつ #T35*13 原発
げんぴ #T30 厳秘
げんぴん #T35 現品
げんぶ #KK 玄武
げんぶがん #T35 玄武岩
げんぶつ #T35*2 現物
げんぶん #T35*3 原文
げんぷ #T35 厳父
げんぷう #T30 厳封
げんぺい #KK 源平
げんぼ #T35 原簿
げんぼく #T35 原木
げんぽう #T35*1 減法 #T30 減俸
げんぽん #T35*14 原本
げんまい #T35 玄米
げんみつ #T05*9 厳密
げんめい #T30*1 言明 #T30 厳命
げんめつ #T30 幻滅
げんめん #T30*1 減免
げんや #T35 原野
げんゆ #T35*1 原油
げんゆう #T30*1 現有
げんよう #T30 現用
げんり #T35*19 原理
げんりゅう #T35*1 源流
げんりょう #T35*2 原料 #T30*2 減量 限量
げんりょうし #T35*13 限量子
げんろく #T35 元禄
げんろん #T35*4 原論 #T35*1 言論
げんわく #T30 幻惑
こ #T35*255 個 子 #kxo*157 来 #M5r*118 込 #S5*94 越 #R5r*39 凝 #T35*22 小 #CNSUC1*19 湖 #M5r*14 こ #T35*8 弧 #M5*6 混 #T35*6 娘 #KJ*5 古 #U5*5 乞 #KJ*3 仔 #T35*2 呼 #kxo*1 こ #KJ*1 固 #T35*1 戸 #KJ*1 菰 虎 #T30*1 鼓 #G5r*1 漕 #S5*1 超 #KY*1 濃 #T35*1 粉 #KJ*1 呱 #KJ 箇 去 拠 虚 乎 #JS 個 #KJ 姑 孤 己 庫 #SUC 庫 #JS 戸 #KJ 故 #PRE 故 #KJ 枯 湖 狐 糊 袴 股 胡 誇 跨 鈷 雇 顧 瑚 #S5 漉 #PRE 小 #U5 請 #KJ 壷 #U5 恋
こーす #T35*6 コース
こーすうぇあ #T35*1 コースウェア
こーだ #T35 コーダ
こーち #T30*2 コーチ
こーちぞる #T35*1 コーチゾル
こーちん #T35 コーチン
こーでぃねーしょん #T30*2 コーディネーション
こーでぃんぐ #T30*34 コーディング
こーと #T30*27 コート
こーど #T35*255 コード
こーな #T35*4 コーナ
こーひー #T35*11 コーヒー #T35*3 珈琲
こーぽ #T35*2 コーポ
こーらす #T30*1 コーラス
こーる #T30*60 コール
こーるど #T35*1 コールド
こーん #T35*2 コーン
こあ #T35*49 コア
こあざ #CNPRE 小字
こあら #T35*2 コアラ
こい #T30*22 恋 #T35*4 故意 #kxoi*4 来い #T35*3 鯉 #kxoi*1 こい #T35 請い
こいけ #T35*8 小池 #JNS 小池
こいけだ #JNS 小池田
こいし #T35 小石 #KY 恋し
こいしわら #JCN 小石原
こいじ #T35*1 恋路
こいずみ #JCN 小泉
こいつ #T35*6 こいつ
こいで #JCN 小出
こいぬ #T35*1 仔犬
こいびと #T35*2 恋人
こいん #T35*1 コイン
こう #T30*255 こう #T35*72 校 #T35*35 甲 #CNSUC1*35 高 #KJ*32 廣 #KJ*27 好 #T35*22 光 #T35*21 項 #ZX*16 講 #KJ*13 公 #KJ*10 耕 #KJ*7 後 稿 #T30*4 抗 #KJ*4 綱 #KJ*3 孝 晃 構 浩 硬 #KJ*2 交 #T35*2 功 #KJ*2 効 向 孔 幸 広 恒 更 #T30*2 航 #KJ*2 行 香 滉 #KJ*1 黄 興 侯 #T35*1 候 #KJ*1 光 厚 口 垢 巧 康 攻 昂 杭 江 洪 #T35*1 港 #KJ*1 皇 酵 降 鴻 皐 縞 衝 #KYU*1 濃う #KJ*1 敲 狡 #KJ 袷 鈎 恰 仰 佼 候 倖 #JN 功 #KJ 勾 #T35 口 #KJ 后 喉 坑 宏 工 #SUC 工 #KJ 巷 庚 弘 慌 拘 控 #ZX 昂 #JS 校 #SUC 校 #KJ 梗 #CNSUC1 港 #SUC 港 #KJ 溝 糠 紅 紘 絞 考 肯 肱 腔 膏 荒 衡 講 貢 購 郊 鉱 砿 鋼 閤 #JS 項 #SUC 項 #KJ 高 #PRE 高 #ZX 高 #KJ 肴 塙 #T35 頁 #KJ 湊
こうあつ #T15*1 高圧
こうあん #T30*3 考案 #T35 公安
こうい #T30*62 行為 #T05*8 高位 #T35*2 好意 #T30*2 更衣 #T35 厚意 校医
こういう #RT*54 こういう
こういき #T15*22 広域
こういしょう #T35 後遺症
こういち #JNM 光一 公一 功一 孝一 幸一 幸市 弘一 晃一 浩一 耕一 講一
こういちろう #JNM 幸一郎 晃一朗
こういってん #T35 紅一点
こういん #T30*1 拘引 #T35 公印 工員 行員
こうう #T30 降雨
こううん #T05*10 好運 #T05*4 幸運
こううんき #T35 耕耘機
こうえい #T35*3 光栄 公営
こうえき #T30 交易 #T35 公益 公役
こうえつ #T30 校閲
こうえん #T35*38 公園 #T30*10 講演 #T30*2 公演 #T30 後援 好演
こうおつ #T35 甲乙
こうおん #T15*1 高温 #T35*1 高音 #T15 恒温
こうか #T35*80 効果 #T05*13 高価 #T30*3 硬化 #T35*2 校歌 #T30*2 降下 #T35*1 工科 #CN 甲賀 #T35 硬貨 考課 高架
こうかい #T30*65 公開 #T30*27 後悔 #T30*1 更改 #T05*1 高階 #T35 公海 #T30 航海
こうかいどう #T35*1 公会堂
こうかがく #T35 光化学
こうかきかえ #T35*2 項書換え
こうかく #T35*1 甲殻 #T15 広角 #T30 降格
こうかつ #T05*1 狡猾
こうかふこうか #CJ 幸か不幸か
こうかん #T30*95 交換 #T30*2 交感 #T30*1 交歓 #T35*1 好感 鋼管 #T35 高官
こうかんき #T35*80 交換機 #T35*5 交換器
こうが #CN 黄河 #JCN 甲賀 #T35 高雅
こうがい #T35*4 郊外 #T35*2 公害 梗概 #T30*1 口外 #T35 校外 構外 #T30 慷慨
こうがく #T35*166 工学 #T35*2 光学 #T05*1 高額 #T35 後学 向学
こうがくか #KK*26 工学科
こうがくねん #T35 高学年
こうがくぶ #T35*68 工学部
こうがっか #T35*25 工学科
こうがん #T05 厚顔 #T35 紅顔 睾丸
こうがんむち #T05*2 厚顔無恥
こうき #T35*3 後期 広軌 #T05*2 高貴 #T30*1 後記 #T35*1 好奇 好気 #T35 好機 工期
こうきしん #T35*1 好奇心
こうきち #JNM 厚吉 孝吉 幸吉
こうきゅう #T05*10 高級 #T30*6 講究 #T35*2 恒久 #T35 後宮 公休 硬球 高給
こうきょ #T35*1 皇居 #T30 薨去
こうきょう #T35*13 公共 #T05*1 好況 #T35*1 広狭 #T35 交響
こうきょうがく #T35 交響楽
こうきょうきょく #T35*1 交響曲
こうきょうぶつ #T35 公共物
こうきん #T35 公金 #T30 拘禁
こうきんり #T35 高金利
こうぎ #T15*6 広義 #T30*6 講義 #T30*2 抗議 #T05*1 恒偽 #T35 高誼
こうぎいん #KK*1 工技院
こうぎょう #T35*88 工業 #T30*16 興行 #T35*1 興業 #T35 鉱業
こうく #T35*1 校区
こうくう #T35*25 航空 #T35*1 口腔 #T35 高空
こうくうき #T35*2 航空機
こうくうけん #T35*1 航空券
こうぐ #T35*1 工具
こうぐう #T35 皇宮
こうぐん #T30 行軍
こうけい #T35*4 光景 #T35*2 後継 #JS 口径 #T35 口径
こうけつ #T05 高潔
こうけつあつ #T35 高血圧
こうけん #T30*12 貢献 #T30 後見
こうげい #T35*3 工芸
こうげき #T30*8 攻撃
こうげん #T35*6 高原 #T35*1 光源 #T30*1 公言 #T35*1 抗原
こうげんびょう #T35 膠原病
こうこ #T35*1 江湖 考古 #T35 公庫
こうこう #T35*19 高校 #T30*2 航行 #T00*1 孝行 #T30 後攻 #T35 膏肓 #F02 煌々
こうこうせい #T35*2 高校生
こうこく #T30*19 広告 #T30*4 公告 #T35 公国
こうこつ #T05 硬骨 #F02 恍惚
こうご #T35*5 口語 #T35*4 交互
こうごう #T35*1 皇后
こうごうし #KY 神々し
こうごうしゅう #KYU 神々しゅう
こうごうせい #T30 光合成
こうさ #T30*11 交差 #T30*1 考査 #T35 較差 #JNS 甲佐
こうさい #T30*11 交際 #T35*1 虹彩 #T35 公債 #CN 甲西 #T35 高裁
こうさいかい #KK 弘済会
こうさいぼう #T35*1 膠細胞
こうさか #JNS 上阪
こうさく #T30*4 工作 #T30*2 耕作 #T30 交錯 #JNM 幸作 耕筰 #T35 鋼索
こうさつ #T30*32 考察 #T30 絞殺 #T35 高察 高札
こうさん #T35*1 好酸 #T30*1 降参 #T35 公算 鉱産
こうざ #T35*14 口座 #T35*4 講座 #JCN 高座
こうざい #T35 鋼材
こうざき #JCN 神崎
こうざつ #T30*1 交雑
こうざぶろう #JNM 幸三郎
こうざん #T35*1 鉱山 #CN 甲山 #T35 高山
こうし #T35*9 講師 #T35*6 格子 #T35*2 公司 公私 #T35*1 後肢 #T30*1 行使 #T35 公使 厚志 #JN 孔子 #T35 皇子 #JNM 合志 #T35 仔牛
こうしえん #CN 甲子園
こうしき #T05*10 公式 #T35 硬式
こうしつ #T05*1 硬質 #T35 皇室
こうしてん #T35*1 格子点
こうしど #T35*1 格子戸
こうしゃ #T35*23 後者 #T35*1 公社 校舎 #T30*1 降車
こうしゃく #T35*1 候爵 #T35 公爵 #T30 講釈
こうしゅ #T35 攻守
こうしゅう #T30*65 講習 #T35*8 公衆 #T35 口臭 #CN 甲州
こうしゅうかい #T35 講習会
こうしゅうでんわ #T35*1 公衆電話
こうしょ #T35*1 高所 #T35 向暑
こうしょう #T30*17 交渉 #T30*4 公称 #T35 公証 工廠 校章 #T30 考証 #T05 高尚 #T30 哄笑
こうしょく #T30 交織 #T35 公職 #T05 好色 #CN 更埴
こうしろう #JNM 幸四郎
こうしん #T30*255 更新 #T30*68 交信 #T05*2 恒真 #T30*1 後進 行進 亢進 #T30 高進
こうしんじょ #T35 興信所
こうしんらい #T05*2 高信頼
こうしんりょう #T35 香辛料
こうじ #T30*27 工事 #T35*2 麹 #T15*1 高次 #T35*1 小路 #T30 公示 #JNM 孝次 孝治 孝二 宏次 幸司 幸次 幸治 幸二 康司 康治 浩司 浩志 浩次 浩二 滉二 #T35 糀
こうじえん #KK*7 広辞苑
こうじつ #T35*4 口実
こうじまち #CN 麹町
こうじゅ #T30 口授
こうじゅつ #T30*5 後述 #T30 口述 講述
こうじゅん #T05*2 降順 #T35*1 公準
こうじょ #T35*4 公序 #T30*3 控除
こうじょう #T30*55 向上 #T35*20 工場 #T35*2 恒常 #T35*1 口上 #T35 厚情
こうじょうせん #T35 甲状腺
こうじろう #JNM 幸次郎
こうじん #T05*1 幸甚 #CN 荒神
こうすい #T35 硬水 降水 香水
こうすいじゅん #T05*1 高水準
こうすう #T35*45 工数
こうすけ #JNM 江介 廣祐
こうず #T35*1 構図
こうずい #T35*1 洪水
こうずか #T35 好事家
こうせい #T30*255 構成 #T30*66 校正 #T35*6 厚生 #T35*2 恒星 #T35*1 後生 #T05*1 公正 #T35*1 向性 抗生 攻勢 #T30*1 更生 #T35*1 校生 硬性 #T35 後世 #T30 更正 #JCN 甲西
こうせいいん #T35*3 構成員
こうせいし #T35*2 構成子
こうせいしょう #KK*1 厚生省
こうせいせき #T15 好成績
こうせいのう #T05*2 高性能
こうせいぶっしつ #T35*1 抗生物質
こうせき #T35*1 功績 #T35 鉱石
こうせつ #T35*1 巧拙 #T30 降雪
こうせん #T35*6 高専 #T35*3 光線 #T35*1 交線 #T30 交戦 公選
こうぜん #T05*4 公然
こうそ #T30 公訴 控訴 #T35 酵素
こうそう #T30*6 構想 #T30 後送 #T05 広壮 #T30 抗争 #T35 降霜 高僧 #T15 高層
こうそく #T05*58 高速 #T30*16 拘束 #T35*1 光速 校則 #T30 梗塞
こうそしん #T35 控訴審
こうそつ #T35*6 高卒
こうぞう #T35*241 構造 #JNM 孝三 幸三
こうぞうか #T30*14 構造化
こうぞうたい #T35*186 構造体
こうぞく #T30*2 後続 #T35*1 皇族
こうた #T35*2 小歌 #JCN 幸田
こうたい #T30*4 交替 #T30*2 後退 #T30*1 交代 #T35*1 抗体
こうたいか #T35*1 抗体価
こうたいごう #T35*1 皇太后
こうたいし #T35 皇太子
こうたく #T35 光沢
こうたろう #JNM 孝太郎 幸太郎
こうたん #T35*2 後端 #T30 降誕
こうだ #JNS 古宇田 #JCN 幸田 甲田 #JNS 行田 神田
こうだい #CNSUC1*2 工大 #T05*1 広大
こうだん #T35*1 公団 講談 #T35 後段 講壇
こうだんし #T35 好男子
こうだんしゃ #KK*2 講談社
こうち #T30*7 後置 #T35*4 巧遅 #JCN 河内 #T30 拘置 #T35 耕地 #CN 高知
こうちく #T30*113 構築
こうちゃ #T35*3 紅茶
こうちゃく #T30*1 膠着 #T30 降着
こうちゅうきゅう #T35*1 好中球
こうちょう #T05*2 好調 #T35*2 校長 #T30 紅潮 #T35 高潮
こうちょうかい #T35*1 公聴会
こうちょうは #T35 高調波
こうちょく #T30 硬直
こうちん #T35*1 工賃
こうつう #T30*14 交通
こうつうきかん #T35*2 交通機関
こうつうもう #T35 交通網
こうつごう #T05*2 好都合
こうづき #JCN 上月 #JNS 神月
こうづけ #JCN 上野
こうづしま #JCN 神津島
こうてい #T35*32 工程 #T30*19 肯定 #T35*2 皇帝 高低 #T35*1 行程 #T30 公定 #T35 公邸 校庭 #T30 校訂
こうていえき #T35*1 口蹄疫
こうてき #T05*2 公的 #T05*1 好適
こうてきしゅ #T35 好敵手
こうてつ #T30 更迭 #T35 鋼鉄
こうてん #T35*5 交点 #T30*2 公転 好転 #T35 後天 好天
こうでい #T30 拘泥
こうでら #JCN 香寺
こうでん #T35*1 香典
こうとう #T35*10 江東 #T35*7 口頭 #T05*2 恒等 高等 #T30*1 高騰 #T35 喉頭 #JCN 江東
こうとうしき #T35*3 恒等式
こうとうぶ #T35 後頭部
こうとうむけい #T05 荒唐無稽
こうとうよう #T35 後頭葉
こうとく #T35 公徳
こうど #T05*25 高度 #T35 黄土
こうどう #T30*60 行動 #T35*3 講堂 #T35*1 黄道 #T35 公道 坑道
こうどく #T30*6 購読
こうない #T35*4 構内 #T35 校内 港内
こうないえん #T35 口内炎
こうなん #CN 江南 港南 甲南 #T35 硬軟 #CN 香南
こうにゅう #T30*77 購入
こうにん #T35*2 後任 #T30*2 公認
こうぬ #JCN 甲奴
こうねつ #T35*12 光熱 #T35*2 高熱
こうねん #T35*1 高年 #T35 後年 #JS 光年
こうねんき #T35 更年期
こうの #JNS 河野
こうのう #T35*2 効能
こうのす #JCN 鴻巣
こうは #T35*2 光波 #T35 硬派
こうはい #T35*21 後輩 #T30*1 荒廃 #T35 高配
こうはく #T35*1 紅白
こうはん #T35*10 後半 #T30*1 公判 #T05*1 広範 #T30 撹拌
こうはんい #T05*4 広範囲
こうば #T35*1 工場
こうばい #T35*1 勾配 #T30*1 購買 #T35 紅梅
こうばし #KYmi 香ばし
こうばしゅう #KYU 香ばしゅう
こうばん #T35*6 交番
こうひ #T35*1 公費 #T35 工費
こうひつ #T35 硬筆
こうひょう #T30*6 公表 #T05*4 好評 #T30 講評
こうひん #T35 公賓
こうび #T35*2 後尾 #T30 交尾
こうふ #T30*1 交付 公布 #CN 江府 甲府
こうふう #T35 校風
こうふく #T05*1 幸福 #T30*1 降伏
こうふん #T30*2 興奮
こうぶ #T35*1 後部
こうぶつ #T35*1 好物 鉱物
こうぶん #T35*22 構文
こうぶんぎ #T35*6 構文木
こうぶんしょ #T35 公文書
こうへい #T05*4 公平 #JNM 公平 幸平 耕平
こうへいれつ #T35*3 高並列
こうへん #T35 後編
こうべ #JCN 神戸
こうほ #T35*255 候補
こうほう #T30*3 広報 #T35*1 後方 #T35 公報 工法
こうほく #CN 江北 港北
こうぼ #T30*3 公募 #T35 酵母
こうぼう #T30*3 攻防 #T30*2 興亡 #T30*1 好防 #T35*1 工房
こうぼく #T35 高木
こうまい #T05 高邁
こうまん #T05 高慢
こうみ #T35*1 香味 #JCN 小海
こうみつ #T35 高密
こうみゃく #T35 鉱脈
こうみょう #T05*1 巧妙
こうみん #T35*1 公民
こうみんかん #T35 公民館
こうむ #R5*1 被 #T35 公務
こうむいん #T35*2 公務員
こうむてん #T35*1 工務店
こうめい #T05 公明 高名
こうめいとう #KK 公明党
こうもく #T35*255 項目
こうもくすう #T35 項目数
こうもり #T35 コウモリ 蝙蝠
こうもん #T35*2 肛門 #T35*1 校門
こうや #T35*3 高野 #T35*1 荒野 #T35 広野 #JCN 高野
こうやぎ #JCN 香焼
こうやく #T30*2 公約 #T30 口約
こうやぐち #CN 高野口
こうやま #JNS 光山 #JCN 高山
こうゆう #T30*1 交遊 #T30 交友 公有
こうよう #KK*2 甲陽 #T30*1 紅葉 高揚 #T35 公用 効用 #JNS 宏陽 #T30 昂揚
こうら #JCN 甲良
こうらく #T30 行楽
こうらくえん #CN 後楽園
こうらん #T30 撹乱 #T35 高覧
こうり #T35*42 公理 #T30*2 小売 #T35 功利 行李 高利
こうりき #JNS 高力
こうりつ #T35*115 効率 #T35*1 公立 #T05 高率
こうりつか #T30 効率化
こうりゃく #T30 後略 攻略
こうりゅう #T30*17 交流 #T30 拘留
こうりょ #T30*49 考慮
こうりょう #F00*1 荒涼 #T35*1 講料 #JCN 広陵 #T35 稿料 香料
こうりょく #T35*1 効力
こうりん #T30 降臨
こうれい #T35*2 恒例 高齢 #T35*1 好例 #T30 降霊
こうれつ #T35 後列
こうろ #T35*10 航路 #KK*1 香露 #T35*1 高炉 #T35 香炉
こうろう #T35*1 功労
こうろん #T30*1 口論 #T35 公論
こうわ #KK*2 興和 #T30 講和 講話
こうわん #T35 港湾
こえ #KS*99 越え #T35*49 声 #KS*10 超え #KS*1 肥え #T35 肥
こえがわり #T30 声変り 声変わり
こえだ #T35 小枝
こお #R5*1 凍 氷
こおう #T30*1 呼応
こおげ #JNS 郡家
こおとこ #T35 小男
こおり #T35*2 氷 #T35*1 郡 #JCN 桑折
こおりざとう #T35 氷砂糖
こおりつ #K5r*1 凍り付
こおりやま #JCN 郡山
こか #T30*1 固化
こかげ #T35 木陰
こかつ #T30*1 枯渇
こかわ #JCN 粉河
こが #S5*1 焦が #JCN 古河 #JNS 古賀
こがい #T35*2 個飼い #T35*1 戸外
こがいしゃ #T35*1 子会社
こがた #T05*4 小型 #T35 小形
こがたき #T35 小型機
こがね #T35*1 黄金
こがねい #JCN 小金井
こがら #T05 小柄
こがらし #T35 木枯し
こき #T35*1 呼気 #T35 古希 古稀
こきざみ #T15*1 小刻み
こきつか #W5*2 こき使
こきゃく #T35*4 顧客
こきゅう #T30*2 呼吸 #T35 胡弓
こきょう #T35*1 故郷
こきんしゅう #KK*1 古今集
こぎって #T35*2 小切手
こぎつ #K5*1 こぎ着
こぎみよ #KY 小気味よ
こぎみよう #KYU 小気味よう
こく #SUC*42 国 #KJ*6 酷 #T05*3 酷 #T30*1 刻 #KJ*1 告 国 穀 黒 哭 國 #KJ 克 鵠 石 谷
こくい #T35*1 国威
こくいん #T30 刻印
こくう #T35*1 穀雨 #T35 虚空
こくえい #T35*1 国営
こくえき #T35 国益
こくおう #T35*1 国王
こくがい #T35*1 国外
こくぎ #T35 国技
こくぎかん #KK 国技館
こくこ #T35*1 国庫
こくご #T35*10 国語
こくさい #T35*85 国際 #T35 国債
こくさいか #T30*19 国際化
こくさいせん #T35*1 国際線
こくさく #T35 国策
こくさん #T35*1 国産
こくさんしゃ #T35 国産車
こくし #T30*3 酷使
こくしじうむ #T35*1 コクシジウム
こくしむそう #T35*1 国士無双
こくしょ #T35*2 国書 #T35 酷暑
こくしょう #JNS 国生
こくじ #T35*1 国事 #T30 告示 #T35 国璽 #T30 酷似
こくじょう #T35 国情
こくじん #T35*1 黒人
こくせい #T35*1 国勢 国政
こくせき #T35*3 国籍
こくせん #T35 国選
こくぜい #T35*1 国税
こくぜいきょく #KK 国税局
こくぜいちょう #KK 国税庁
こくそ #T30 告訴
こくたい #KK 国体
こくだい #T35*1 国大
こくち #T30 告知
こくてい #T35 国定
こくてつ #KK*1 国鉄
こくでん #T35 国電
こくど #T35*1 国土
こくどう #T35*1 国道
こくどちょう #KK*1 国土庁
こくない #T35*34 国内
こくないせん #T35*1 国内線
こくはく #T30*1 告白 #T05 酷薄
こくはつ #T30*1 告発
こくばん #T35*4 黒板
こくひ #T35 国費
こくひょう #T30 酷評
こくひん #T35 国賓
こくふ #JNS 国府
こくふく #T30*4 克服
こくぶ #JCN 国分
こくぶんじ #CN 国分寺
こくべつ #T30*1 告別
こくほ #T35*3 国保
こくほう #T35 国宝
こくぼ #JNS 小久保
こくぼう #T35 国防
こくみん #T35*5 国民
こくむ #T35 国務
こくむしょう #KK 国務省
こくもつ #T35*1 穀物
こくゆう #T35 国有
こくら #JCN 小倉
こくらきた #CN 小倉北
こくらみなみ #CN 小倉南
こくりつ #T35*4 国立
こくるい #T35*1 穀類
こくれん #T35 国連
こくろう #KK 国労
こくろん #T35 国論
こぐち #T05*1 小口
こぐれ #JNS 小暮
こけ #KS*13 こけ #T35*1 苔
こけい #T35 固形
こけつ #JNS 纐纈
こけら #T35 柿
こげ #KSr*1 焦げ
こげつ #K5r 焦げ付
ここ #T35*136 ここ #T35*63 個々
ここあ #T35*1 ココア
ここいら #T35*1 ここいら
ここく #T35*1 故国
ここち #T35*3 心地
ここちよ #KY*3 心地よ
ここちよう #KYU*1 心地よう
ここに #CJ*4 ここに
ここのえ #JCN 九重
ここのか #T35 9日 ９日 九日
ここのそじ #T35 九十路
ここのつ #T35*1 九つ
こころ #T30*48 心 #T35*1 こころ
こころいき #T35 心意気
こころえ #KSr*1 心得
こころから #F14 心から
こころがけ #KSr*4 心がけ #KSr 心掛け
こころがまえ #T35*3 心構え
こころざ #S5*1 志
こころざし #T35 志
こころづけ #T35 心付け
こころづよ #KY*2 心強
こころづよう #KYU*1 心強う
こころな #KY*1 心無
こころね #T35 心根
こころのう #KYU*1 心無う
こころのこり #T05 心残り
こころぼそ #KY*1 心細
こころぼそう #KYU*1 心細う
こころまち #T35 心待ち
こころみ #KSr*34 試み
こころもち #T35 心持ち
こころもとな #KY*1 心許な
こころもとのう #KYU*1 心許のう
こころゆ #K5 心行
こころよ #KY*1 快
こころよう #KYU*1 快う
ここん #T35*1 古今
こご #T35*1 古語
こごえ #KS 凍え
こごた #JCN 小牛田
こごと #T30*1 小言
こさい #JCN 湖西
こさか #JCN 小坂
こさざ #JCN 小佐々
こさつ #T35 古刹
こさめ #T35*1 小雨
こさん #T35 古参
こざ #JCN 古座
こざいく #T30 小細工
こざかい #JCN 小坂井
こざがわ #JCN 古座川
こし #T35*10 腰 #T35*1 輿 #JNS 古志 #T30 枯死
こしかけ #KSr*1 腰かけ #T35 腰掛け
こしかわ #JNS 越川
こしがや #JCN 越谷
こしき #T35 甑
こしじ #JCN 越路
こしたんたん #F02*1 虎視眈々
こしつ #T35*5 個室 #T30*4 固執
こしぬけ #T05 腰抜け
こしの #JCN 越廼
こしば #JNS 小柴
こしみず #CN 小清水
こしゃく #T15 こしゃく 小癪
こしゅ #T30*1 固守 #T35 戸主
こしゅう #T30*1 固執
こしょ #T35*2 古書
こしょう #T30*39 故障 #T30*8 呼称 #T35 胡椒
こじ #T35*11 古寺 #T35*1 故事 #T30*1 誇示 #R5 こじ #T35 居士 #T30 固持 固辞 #T35 孤児
こじあけ #KS こじ開け
こじき #KK*1 古事記 #T35*1 乞食
こじま #T35*5 小島 #JCN 児島 #JNS 児嶋 #JCN 小島 #JNS 小嶋
こじゅうと #T35 小姑
こじょう #T35 古城
こじれ #KSr こじれ
こじん #T35*165 個人 #T35*1 故人
こす #R5r こす 擦
こすい #T35 湖水
こすう #T35*16 個数 #T35 戸数
こすぎ #JCN 小杉
こすげ #JNS 小菅
こすたりか #CN コスタリカ
こすっから #KY こすっ辛
こすっかろう #KYU こすっ辛う
こすと #T35*30 コスト
こすど #JCN 小須戸
こすも #T35*1 コスモ
こずえ #T35 梢
こせい #T35*5 個性
こせき #T35*4 古跡 #T35*1 戸籍 #JNS 小関
こせこせ #F00 こせこせ
こせん #T35 古銭
こせんきょう #T35 跨線橋
こぜに #T35 小銭
こそ #G5 こそ
こそく #T05*2 姑息
こそげ #KS こそげ
こそこそ #F04 こそこそ
こそだて #T30*1 子育て
こそばゆ #KY こそばゆ
こそばゆう #KYU こそばゆう
こぞう #T35*2 小僧
こぞって #F14 こぞって
こたい #T35*1 個体 固体
こたえ #KSr*98 答え #T35*4 答 #KSr*2 応え #KS 堪え
こたけ #JCN 小竹
こたつ #T35 コタツ 炬燵
こたに #JNS 小谷
こだい #T35*3 古代
こだいら #JCN 小平
こだか #JNS 小高 #KY 小高
こだこう #KYU 小高う
こだち #T35*1 木立
こだま #JCN 児玉 #JNS 小玉
こだわ #R5r*11 拘
こち #T35*1 故知 鯒
こちこち #T00*2 コチコチ #T05 こちこち
こちゃく #T30 固着
こちょう #T30*1 誇張 #T35 胡蝶
こちら #T35*178 こちら
こちんだ #CN 東風平
こっか #T35*4 国家 #T35*2 国歌
こっかい #T35*4 国会 #T35 黒海
こっかく #T35*2 骨格
こっかん #T35 酷寒
こっき #T35*1 国旗
こっきょう #T35*1 国境
こっくぴっと #T35*1 コックピット
こっけい #T05 滑稽
こっけん #T35 国権
こっこ #T35 国庫
こっこう #T35 国交
こっこうりつ #T35 国公立
こっこく #T35*1 刻々
こっし #T35*1 骨子
こっせつ #T30*1 骨折
こっそり #F06*9 こっそり
こっち #T35*39 こっち
こっちょう #T35 骨頂
こってり #F04 こってり
こっとう #T35*1 骨董
こっぱ #T35 木端
こっぷ #T35*5 コップ
こっぺん #T35 骨片
こつ #T35*2 コツ #T35*1 骨 #KJ 乞 忽 惚
こつこつ #F14*2 こつこつ コツコツ
こつずい #T35 骨随
こつつぼ #T35 骨壷
こつにく #T35 骨肉
こつばん #T35 骨盤
こつまく #T35 骨膜
こづかい #T35 小遣い
こづつみ #T35 小包
こて #T35*1 小手 #T35 鏝
こてい #T30*38 固定
こてき #T35 鼓笛
こてさき #T35 小手先
こてん #T35*3 古典 #T35 個展
こと #T35*245 事 #T35*95 こと #T35*5 異 #T35*3 琴 #KJ*3 言 #T35*1 古都 #T35 殊
ことあげ #T30 言挙げ
ことう #T35*1 孤島 #CN 湖東
ことおか #JCN 琴丘
ことか #K5 事欠
ことがら #T35*3 ことがら 事柄
ことこまか #T05 事細か
ことさら #T06*1 殊更 #T06 ことさら
ことし #T35*72 今年
ことたり #KS*1 事足り
ことづけ #KSr*1 言付け #T30*1 言付け
ことな #R5r*163 異な #R5r*2 異
ことなみ #CN 琴南
ことば #T35*86 言葉 #T35*4 ことば #T35*1 詞
ことひら #CN 琴平
ことぶき #T35*6 寿
ことほ #G5r 祝
こともなげ #T05 こともなげ
ことり #T35 小鳥
ことわ #R5r*10 断 #R5r 断わ
ことわざ #T35*2 諺
こどう #T30 鼓動
こどうぐ #T35*1 小道具
こどく #T05*1 孤独
こどまり #JCN 小泊
こども #T35*50 子ども #T35*18 子供
こな #T35*8 粉 #S5*5 こな
こないだ #T35 こないだ
こなか #JNS 小中
こながい #JCN 小長井
こなごな #T05 粉々
こなつ #JNS 小夏
こにくらし #KY 小憎らし
こにくらしゅう #KYU 小憎らしゅう
こにし #JNS 小西
こにゃっく #T35 コニャック
こぬま #JCN 小沼
こね #T35*2 コネ #KSr こね
こねあげ #KSr こね上げ
こねく #R5 こねく
こねくしょにすと #T35*1 コネクショニスト
こねくしょにずむ #T35*2 コネクショニズム
こねくしょん #T35*57 コネクション
こねくた #T35*6 コネクタ
こねくと #T30*10 コネクト
こねまわ #S5r こね回
この #RT*255 この #M5r*23 好 #RT*1 此の
このあいだ #T35*25 この間
このうえう #KYU この上無う
このうえな #KY この上な この上無
このうえのう #KYU この上のう
このうら #JCN 金浦
このこと #T35*13 このこと
このごろ #T35*24 この頃
このさい #F14 この際
このたび #T35*17 この度
このため #T35*34 このため
このはな #JCN 此花
このほど #T35*1 このほど
このまし #KY*12 好まし
このましゅう #KYU*2 好ましゅう
このまま #T35*17 このまま
このみ #T35*1 木の実
このもし #KY 好もし
このもしゅう #KYU 好もしゅう
このよう #T05*139 このよう #T05*5 この様
このんで #F14 好んで
こはだ #T35 こはだ
こはら #JNS 小原
こはる #T35 小春
こはん #T35*1 湖畔
こはんとき #T35 小半刻
こば #M5*2 拒 #JNS 古場
こばなし #T35 小話
こばやかわ #JNS 小早川
こばやし #JNS 古林 #JCN 小林
こばり #JNS 小針
こばると #T35*2 コバルト
こばん #T35*3 小判
こばんざめ #T35 コバンザメ 小判鮫
こび #KSr*2 媚び
こびりつ #K5r こびり付
こぴー #T30*251 コピー
こぴーらいと #T35*20 コピーライト
こふう #T05*2 古風
こふきいも #T35 粉吹き芋
こふん #T35*1 古墳
こぶし #T35 拳
こぶちざわ #JCN 小淵沢
こぶり #T05 小降り
こぶん #T35 古文
こぷろせっさ #T35*2 コプロセッサ
こべつ #T15*20 個別 #T35 戸別
こほく #CN 湖北
こぼ #S5*6 こぼ
こぼり #JNS 小堀
こぼれ #KSr*3 こぼれ
こぼれおち #KSr こぼれ落ち
こま #R5*123 困 #T35*11 駒 #T35*8 コマ #KJ*1 狛 #JCN 巨摩
こまーしゃる #T35*2 コマーシャル
こまあらし #JNS 駒嵐
こまえ #JCN 狛江
こまか #KY*45 細か #T05*29 細か
こまがだけ #CN 駒ヶ岳
こまがね #CN 駒ヶ根
こまき #JCN 小牧
こまぎれ #T05*1 細切れ
こまく #T35*1 鼓膜
こまこう #KYU*2 細こう
こまごま #F04*3 こまごま
こまごめ #JCN 駒込
こまた #T05 小股
こまっしゃくれ #KS こまっしゃくれ
こまつ #T35*1 小松 #JCN 小松
こまつざき #JNS 小松崎
こまつしま #CN 小松島
こまぬ #K5 拱
こまね #K5 拱
こまば #JCN 駒場
こまめ #T05*3 こまめ
こまやか #T05 細やか
こまりはて #KS 困り果て
こまわり #T30 小回り
こまんど #T35*255 コマンド
こみ #T35*9 込
こみあ #W5r*2 込み合
こみあげ #KS こみ上げ
こみい #R5*1 込み入
こみち #T35*1 小路 #T35 小道
こみっく #T35*3 コミック
こみっと #T30*2 コミット
こみね #JCN 小峰 #JNS 小峯
こみや #JNS 小宮
こみやま #JNS 小宮山
こみゅにけーしょん #T30*18 コミュニケーション
こみゅにけーと #T30 コミュニケート
こむぎ #T35 小麦
こむぎこ #T35 小麦粉
こむそう #T35 虚無僧
こむろ #JCN 小室
こめ #T35*78 ※ #T35*11 米 #KS*1 こめ #KS 込め 篭め 籠め
こめじるし #T35*16 ※
こめでぃー #T35*1 コメディー
こめでぃあん #T35*1 コメディアン
こめんと #T30*103 コメント
こも #R5*5 籠 #KJ 菰 薦 #R5 篭
こもじ #T35*11 小文字
こもち #JCN 子持
こもの #T35*2 小物 #JCN 菰野
こもり #T30*1 子守 #JNS 小森
こもろ #CN 小諸
こもん #T35*1 顧問 #T35 小紋
こもんじょ #T35*1 古文書
こや #T35*1 小屋 #S5 肥 #S5r 肥や
こやなぎ #JNS 小柳
こやま #T35*1 小山 #JCN 小山
こゆ #JNS 児湯
こゆう #T05*30 固有
こゆび #T35*2 小指
こよい #T35*1 今宵
こよう #T30*2 雇用
こよみ #T35*2 暦
こら #CJ*4 こら #S5*2 凝ら #S5 懲ら
こらーじゅ #T30 コラージュ
こらい #T35 古来
こらえ #KS*1 堪え #KSr こらえ
こらしめ #KSr 懲らしめ
こらむ #T35*4 コラム
こり #KS*4 懲り
こりー #T35*1 コリー
こりじょん #T35*1 コリジョン
こりつ #T30*2 孤立
こりゅーじょん #T35*1 コリュージョン
こりょ #T30 顧慮
こりょう #CN 湖陵
こりょうり #T35 小料理
こるーちん #T35 コルーチン
こるせっと #T35 コルセット
こるちこいど #T35*1 コルチコイド
これ #T35*255 これ #KJ*1 此 是 之 #KJ 惟
これいじょう #T35*10 これ以上
これくしょん #T30*1 コレクション
これくた #T35*1 コレクタ
これっぽっち #T35 これっぽっち
これほど #F14*20 これほど #T36 これほど
これみよがし #T05 これみよがし #T15 これ見よがし
これら #T35*115 これら #T35*2 コレラ
これらの #RT*20 これらの
ころ #T35*126 ころ #T35*66 頃 #S5r*24 殺 #B5*4 転
ころう #T35 古老
ころが #R5r*1 転が #S5r*1 転が
ころがりこ #M5*1 転がり込
ころげこ #M5 転げ込
ころな #T35*1 コロナ
ころにー #T35*1 コロニー
ころも #T35*1 衣
ころもがえ #T30*1 衣更え
ころらど #CN コロラド
ころんぶす #JN コロンブス
こわ #KY*25 恐 #S5*9 壊 #KY*6 怖 #KY*1 こわ
こわごわ #F06 こわごわ
こわば #R5r*1 こわば
こわれ #KSr*24 壊れ
こん #KJ*252 今 #SUC*15 コン #T35*5 根 #KJ*2 婚 梱 #ZX*2 混 #T35*2 紺 #KJ*1 恨 懇 魂 #KJ 近 金 献 #JNS 今 #KJ 困 坤 墾 昏 昆 混 痕 艮
こんい #T05 懇意
こんいん #T30*1 婚姻
こんか #T35 今夏 婚家
こんかい #T35*255 今回
こんかれんと #T35*1 コンカレント
こんかん #T35*1 根幹
こんがらが #R5r こんがらが
こんがり #F06*1 こんがり
こんがん #T30 懇願
こんき #T35*4 根気 #T35*1 今期 #T35 婚期
こんきゃく #T30 困却
こんきゅう #T30*1 困窮
こんきょ #T35*8 根拠
こんぎ #T35 婚儀
こんく #T30 困苦
こんくーる #T35*1 コンクール
こんくりーと #T35*1 コンクリート
こんけつ #T35 混血
こんげつ #T35*45 今月
こんげん #T35*1 根源
こんこう #T30*1 混交 #CN 金光
こんこん #F02 渾々
こんご #T35*102 今後
こんごう #T30*6 混合 #T35*2 根号
こんさーと #T35*5 コンサート
こんさるたんと #T35*1 コンサルタント
こんさるてぃんぐ #T30*1 コンサルティング
こんざい #T30*31 混在
こんざつ #T30*14 混雑
こんしゅう #T35*97 今週 #T35 今秋
こんしゅん #T35*1 今春
こんしん #T35*4 懇親 #T30*1 混信
こんじ #T35 今時
こんじき #T35*1 金色
こんじゃく #T35*1 今昔
こんじょう #T35*48 今生 #T35*4 根性 #T35 懇情
こんすい #T30 昏睡
こんすたんと #T05*3 コンスタント
こんすとれいんと #T35*1 コンストレイント
こんせい #T30*1 混成 #T30 懇請 #T35 混声
こんせき #T35 今夕 痕跡
こんせつ #T05 懇切
こんせぷと #T35*3 コンセプト
こんせん #T35*4 混戦
こんせんさす #T35*2 コンセンサス
こんせんと #T35*3 コンセント
こんぜつ #T30*1 根絶
こんぜん #F02 混然 渾然
こんそーしあむ #T35*4 コンソーシアム
こんそーる #T35*26 コンソール
こんそしあむ #T35 コンソシアム
こんそめ #T35 コンソメ
こんたくと #T30*6 コンタクト
こんたん #T35*2 魂胆
こんだ #JCN 今田
こんだく #T30 混濁 溷濁
こんだて #T35*1 献立
こんだん #T30*3 懇談
こんちゅう #T35 昆虫
こんてい #T35*2 根底
こんてきすと #T35*11 コンテキスト
こんてくすと #T35*90 コンテクスト
こんてすと #T30*2 コンテスト
こんてな #T35*1 コンテナ
こんてんつ #T35*25 コンテンツ
こんでぃしょん #T35*2 コンディション
こんでんさ #T35*1 コンデンサ
こんとら #T35*1 コントラ
こんとらすと #T35*2 コントラスト
こんとろーら #T35*4 コントローラ
こんとろーる #T30*29 コントロール
こんとん #F02*1 混沌 #T35*1 混沌 #F02 渾沌
こんど #T35*163 今度 #T35*2 こんど
こんどう #T30*4 混同 #T35*1 金堂 #JNS 近藤
こんどる #T35 コンドル
こんな #RT*141 こんな #T05*100 こんな
こんなん #T05*28 困難
こんにち #T35*20 今日
こんにちは #CJ*101 こんにちは #T35*68 こんにちは
こんにちわ #CJ*5 こんにちわ #T35*2 こんにちわ
こんにゃく #T35 こんにゃく 蒟蒻
こんにゅう #T30*1 混入
こんねん #T35 今年
こんねんど #T35*4 今年度
こんの #JNS 今野 紺野
こんばーじょん #T35*4 コンバージョン
こんばーた #T35*3 コンバータ
こんばーと #T30*15 コンバート
こんばん #T35*4 今晩
こんばんは #T35*9 こんばんは #CJ*7 こんばんは
こんばんわ #CJ*2 こんばんわ #T35 こんばんわ
こんぱ #T35*4 コンパ
こんぱい #T30 困憊
こんぱいら #T35*68 コンパイラ
こんぱいらー #T35*3 コンパイラー
こんぱいる #T30*193 コンパイル
こんぱく #T35 魂魄
こんぱくと #T05*3 コンパクト
こんぱす #T35*8 コンパス
こんぱち #T05*8 コンパチ
こんぱちびりてぃ #T35*2 コンパチビリティ
こんぱちびりてぃー #T35*1 コンパチビリティー
こんぱちぶる #T05*4 コンパチブル
こんぱにおん #T35*3 コンパニオン
こんぱん #T35 今般
こんび #T35*3 コンビ
こんびなーと #T35 コンビナート
こんびに #T35 コンビニ
こんびにえんす #T35*1 コンビニエンス
こんびねーしょん #T35 コンビネーション
こんびねーた #T35*1 コンビネータ
こんぴゅーた #T35*154 コンピュータ
こんぴゅーたー #T35*2 コンピューター
こんふぁれんす #T35 コンファレンス
こんふぃぎゅれーしょん #T30*6 コンフィギュレーション
こんふぃぐれーしょん #T30*3 コンフィグレーション
こんふりくと #T30*2 コンフリクト
こんぶ #T35*3 昆布
こんぷりーしょん #T35*1 コンプリーション
こんぷりーと #T00*1 コンプリート
こんぷりしょん #T35*1 コンプリション
こんぷりめんと #T35*1 コンプリメント
こんぷれす #T30*4 コンプレス
こんぷれっくす #T35*1 コンプレックス
こんべあ #T35 コンベア
こんべんしょなる #T05*6 コンベンショナル
こんべんしょん #T35*2 コンベンション
こんぺき #T35*1 紺碧
こんぼう #T30 懇望 混紡 #T35 棍棒
こんぽーねんと #T35*15 コンポーネント
こんぽう #T30*3 梱包
こんぽん #T35*6 根本
こんま #T35*3 コンマ
こんめい #T30 混迷
こんもう #T30 懇望 #T35 根毛
こんもり #F04 こんもり
こんや #T35*4 今夜
こんやく #T30*3 婚約
こんよう #T30*1 混用
こんらん #T30*15 混乱
こんりゅう #T30 建立 #T35 根粒
こんりんざい #F14*1 金輪際
こんれい #T35*1 婚礼
こんろ #T35*7 コンロ
こんわ #T30*1 懇話 #T30 混和
こんわく #T30*1 困惑
ご #JS*255 語 #NN*190 五 #SUC*154 後 #KJ*62 御 #KJ*13 誤 #S5*7 伍 #T30*7 伍 #T35*7 語 #KJ*3 悟 護 #T35*2 碁 #KJ*1 互 吾 後 檎 醐 宕 #PRE ご #KJ 胡 午 呉 娯 #JSSUC 後 #PRE 御 #KJ 梧 瑚 #SUC 語 #PRE 誤 #KJ 冴
ごーじゃす #T05*2 ゴージャス
ごーすと #T35*2 ゴースト
ごーどん #JN ゴードン
ごーる #T30*31 ゴール
ごーるでん #PRE*1 ゴールデン
ごーるど #T05*2 ゴールド
ごあいこ #T30*1 ご愛顧 #T30 御愛顧
ごあいさつ #T30*4 ご挨拶 #T30*1 御挨拶
ごあいよう #T30*2 ご愛用 #T30 御愛用
ごあんじ #OKX*3 ご案じ
ごあんない #T30*2 ご案内 #T30 御案内
ごい #T35*31 語彙 #T35*1 語意
ごいちどうさま #T35 御一同様
ごいっしょ #T10*2 ご一緒 #T10*1 御一緒
ごいっとうさま #T35 御一統様
ごいっぽう #T30*5 御一報
ごいらい #T30*3 御依頼 #T30*2 ご依頼
ごう #JS*89 号 #KJ*11 郷 #T35*2 合 豪 #KJ*1 強 業 #T35*1 剛 #T35 業 #KJ 劫 #S5 号 #SUC 号 #T30 号 #JS 合 #KJ 壕 拷 濠 轟
ごうい #T30*10 合意
ごういん #T05*8 強引
ごうう #T35*1 豪雨
ごうか #T05*4 豪華
ごうかい #T05*3 豪快
ごうかく #T30*5 合格
ごうかん #JS*10 号館 #T30*1 強姦
ごうがい #T35 号外
ごうき #T05 剛毅
ごうきゅう #T30 号泣
ごうきん #T35 合金
ごうぎ #T30 合議 #T05 豪儀
ごうけい #T30*55 合計
ごうけつ #T35 豪傑
ごうけん #T05 剛健
ごうご #T30*1 豪語
ごうごう #F00*1 轟々
ごうし #T30*1 合資
ごうしがいしゃ #T35 合資会社
ごうしつ #JS*4 号室
ごうしゃ #T05 豪奢
ごうしゅう #CN 豪州
ごうしょう #JS 号証 #T35 豪商
ごうじょう #T05 強情
ごうすう #T35 号数
ごうせい #T30*16 合成 #T15 豪勢
ごうせつ #T35 豪雪
ごうせん #JS*2 号線
ごうそ #T30 強訴
ごうぞく #T35 豪族
ごうたん #T05 剛胆 豪胆
ごうだ #JNS 郷田
ごうだつ #T30*1 強奪
ごうちょく #T05 剛直
ごうつ #JCN 江津
ごうつくばり #T05 業つくばり
ごうとう #T30*1 強盗 #JS 号棟
ごうど #JCN 神戸
ごうどう #T30*7 合同
ごうのうら #CN 郷ノ浦
ごうはら #T05 業腹
ごうはん #T35 合板
ごうひ #T35 合否
ごうひつ #T30 合筆
ごうべん #T35 合弁
ごうほう #T05*4 合法 #T35*1 号砲 #T05 豪放
ごうほうらいらく #T05*1 豪放磊落
ごうまん #T05*4 ごう慢 傲慢
ごうめい #T35 合名
ごうめいがいしゃ #T35 合名会社
ごうもん #T30*1 拷問
ごうよく #T05 強欲
ごうり #T35*11 合理
ごうりき #T05 強力
ごうりゅう #T30*2 合流
ごうりゅうし #T35*1 合流子
ごうれい #T30 号令
ごうわん #T05 剛腕
ごえい #T30*1 護衛
ごえいじ #OKX*2 ご映じ #OKX ご詠じ
ごえいてん #T30 ご栄転 御栄転
ごえつどうしゅう #T15 呉越同舟
ごえん #T30*1 誤嚥
ごえんじ #OKX*5 ご演じ
ごえんじょ #T30 ご援助 御援助
ごおうじ #OKX*61 ご応じ
ごおうぼ #T30*2 ご応募 #T30 御応募
ごおんこ #T35 ご恩顧 御恩顧
ごか #CN 五霞
ごかい #T30*20 誤解
ごかいそう #T35 ご会葬 御会葬
ごかいとう #T30*4 御回答 #T30 ご回答
ごかくにん #T30*2 ご確認 #T30 御確認
ごかしょう #CN 五個荘
ごかせ #CN 五ヶ瀬
ごかぞく #T35*1 ご家族 #T35 御家族
ごかつやく #T30*1 ご活躍 #T30 御活躍
ごかてい #T35 ご家庭 御家庭
ごかん #T35*179 互換 #T35*13 語幹 #T35*1 語感
ごかんじ #OKX*4 ご感じ #OKX*1 ご観じ
ごかんじょ #T35 ご寛恕 御寛恕
ごかんせい #T35 互換性
ごかんづき #K5r ご勘づき ご感づき
ごかんよう #T35*1 ご寛容 #T35 御寛容
ごがく #T35*7 語学
ごがん #T35 護岸
ごき #T35*2 語気 #T30 誤記
ごきげん #T15 ご機嫌
ごきげんよう #CJ*1 ごきげんよう
ごきたい #T30*5 御期待 #T30 ご期待
ごきにゅう #T30*2 ご記入 #T30 御記入
ごきぶり #T35*7 ゴキブリ
ごきぼう #T30*7 ご希望 #T30*2 御希望
ごきょうりょく #T30*18 御協力 #T30*12 ご協力
ごきんじ #OKX*33 ご禁じ
ごぎょう #T35 五行
ごぎんじ #OKX ご吟じ
ごく #F14*19 ごく #F14*4 極 #T35*1 語句 #SUC*1 国 #KJ*1 獄
ごくあく #T05 極悪
ごくい #T35*1 極意
ごくしゃ #T35 獄舎
ごくひ #T05*6 極秘
ごくひん #T35 極貧
ごくらく #T35 極楽
ごくろう #T05*35 ご苦労 #T05*13 御苦労
ごくろうさま #T15 ご苦労さま 御苦労様
ごけ #T35 後家
ごけい #T35 互恵
ごけっこん #T30*2 ご結婚 #T30 御結婚
ごけっせき #T30 ご欠席 御欠席
ごけん #T35 護憲
ごけんしょう #T35*1 ご健勝 #T35 御健勝
ごけんとう #T30*6 ご検討 御検討
ごげん #T35*2 語源
ごこう #T35 後光
ごこうい #T35*2 ご厚意 #T35*1 ご好意 御厚意 御好意
ごこうぎ #T35 ご高誼 御高誼
ごこうし #T35 ご厚志 御厚志
ごこうじ #OKX*16 ご講じ
ごこうじょう #T35 ご厚情 御厚情
ごこうはい #T35 ご高配 御高配
ごこうらん #T35*1 御高覧 #T35 ご高覧
ごこく #T35 五穀 護国
ごこち #T35*2 心地
ごころ #SUC*2 心
ごこんじょう #T35 ご懇情 御懇情
ごこんせつ #T05 ご懇切 御懇切
ごご #T35*102 午後 #T35*1 午后
ごさ #T35*2 誤差
ごさい #T35*1 後妻
ごさっし #OKX*6 ご察し
ごさんか #T30*3 御参加 #T30 ご参加
ごさんじ #OKX ご散じ
ごさんれつ #T30 ご参列 御参列
ござ #L5*255 ござ #L5*12 御座
ございしょ #JCN 御在所
ごし #T35 越し 腰
ごしえん #T30*2 ご支援 #T30 御支援
ごしき #CN 五色
ごしっく #KK*5 ゴシック
ごしつもん #T30*4 ご質問 #T30*3 御質問
ごしどう #T30*2 御指導 #T30 ご指導
ごしゃく #T30 語釈
ごしゅうにん #T30 ご就任 御就任
ごしゅじん #T35*4 ご主人 #T35 御主人
ごしゅっせき #T30*4 ご出席 #T30 御出席
ごしょ #T35 御所
ごしょう #T35*1 後生
ごしょうかい #T30*3 御紹介 #T30*1 ご紹介 #T30 ご照会 御照会
ごしょうじ #OKX*61 ご生じ
ごしょうたい #T30*1 ご招待 #T30 御招待
ごしょうち #T30*4 御承知 #T30 ご承知
ごしょがわら #JCN 五所川原
ごしょく #T30 誤植
ごしょのうら #JCN 御所浦
ごしよう #T35*14 御使用 #T35 ご使用
ごしん #T30 誤信 誤診 #T35 護身
ごしんじ #OKX*163 ご信じ
ごじ #T35*6 誤字 #T30 護持
ごじあい #T30 ご自愛 御自愛
ごじつ #T35*10 後日
ごじゅう #T35*1 五重
ごじゅうおん #T35*2 五十音
ごじゅうしょ #T35 ご住所 御住所
ごじゅっぽひゃっぽ #T35*1 五十歩百歩
ごじゅん #T35 語順
ごじゅんじ #OKX ご殉じ
ごじょ #T30*2 互助
ごじょう #CN 五条 #JCN 五條 #T30 互譲
ごじょうじ #OKX*4 ご乗じ
ごじょうめ #CN 五城目
ごじん #T35*1 御仁
ごすう #T35*12 語数
ごせ #JCN 御所
ごせい #T35 悟性
ごせいうん #T35 ご盛運 御盛運
ごせいえい #T35*1 ご清栄 御清栄
ごせいきゅう #T35*2 ご請求 #T35 御請求
ごせいぎょう #T35 ご盛業 御盛業
ごせいしょう #T35 ご清勝 ご清祥 御清勝 御清祥
ごせいだい #T05 ご盛大 御盛大
ごせいらん #T35*1 御清覧
ごせっし #OKX*2 ご接し
ごせん #JCN 五泉 #T30 互選
ごせんし #T35 五線紙
ごせんぞ #T35 ご先祖 御先祖
ごせんふ #T35*1 五線譜
ごぜん #T35*88 午前
ごぜんやま #CN 御前山
ごそう #T30 護送
ごそうさ #T30*3 誤操作
ごそうだん #T30*4 ご相談 #T30 御相談
ごそうふ #T30*1 ご送付 #T30 御送付
ごそくろう #T35 ご足労 御足労
ごそんじ #OKX*8 ご損じ
ごぞんじ #OKX*25 ご存知 #OKX*15 御存知 #OKX*12 ご存じ
ごたく #T35*1 御託
ごたごた #T35 ゴタゴタ
ごたしょう #T35 ご多祥 御多祥
ごたぼう #T05*2 ご多忙 #T05 御多忙
ごたようちゅう #T35 ご多用中 御多用中
ごたんじ #OKX ご嘆じ
ごたんだ #CN 五反田
ごだつ #T35 誤脱
ごだんかつよう #T30 五段活用
ごだんじ #OKX*2 ご断じ
ごちそう #T35*8 ご馳走 #T35 御馳走
ごちゃごちゃ #F04*3 ごちゃごちゃ
ごちゅうい #T30*23 ご注意 #T30 御注意
ごちゅうもん #T30*3 ご注文 #T30*1 御注文
ごちょう #T35*1 語長 #T35 語調
ごちょうじ #OKX ご長じ
ごっそり #F02*1 ごっそり
ごったに #T35 ごった煮
ごっちゃ #T18 ごっちゃ
ごつうち #T30 ご通知 御通知
ごつごう #T35*2 御都合 #T35*1 ご都合
ごてん #T35 御殿
ごてんば #CN 御殿場
ごと #SUC*153 毎 #SUC ごと 事 #KJ 如 毎 #T15 毎
ごとう #JCN 五島 #JNS 後藤 #T35 語頭
ごとうじ #OKX*1 ご投じ
ごとうちゃく #T30*1 御到着 #T30 ご到着
ごとき #RT 如き
ごとく #F14 如く
ごとし #F14 如し
ごどうさ #T30 誤動作
ごどうじ #OKX*3 ご動じ
ごなんじ #OKX*13 ご難じ
ごにん #T30*2 誤認
ごねんじ #OKX*22 ご念じ
ごのへ #JNS 五戸
ごはいりょ #T30*2 ご配慮 #T30 御配慮
ごはさん #T35 ご破算 御破算
ごはっし #OKX*4 ご発し
ごはってん #T30 ご発展 御発展
ごはん #T35*35 ご飯 #T35*2 御飯
ごはんえい #T30 ご繁栄 御繁栄
ごはんじ #OKX*1 ご判じ
ごはんじょう #T30 ご繁盛 御繁盛
ごばっし #OKX*1 ご罰し
ごばん #T35 碁盤
ごび #T35*7 語尾
ごびゅう #T35 誤謬
ごふうじ #OKX ご封じ
ごふく #T35 呉服
ごぶさた #T30*29 御無沙汰
ごぶん #T30*1 誤聞
ごへい #T35 語弊
ごへんかん #T30 誤変換
ごへんじ #OKX*247 ご変じ #T30*4 ご返事 #T30 御返事
ごへんそう #T30*1 ご返送 #T30 御返送
ごべんたつ #T35*1 御鞭撻
ごほう #T35*1 誤報
ごほうこく #T30*14 御報告 #T30 ご報告
ごほうし #T30 ご奉仕 #T35 ご芳志 #T30 御奉仕 #T35 御芳志
ごほうじ #OKX*10 ご報じ #OKX ご奉じ ご崩じ
ごほうねん #T35 ご放念 御放念
ごほうめい #T35 ご芳名 御芳名
ごほく #CN 吾北
ごぼう #CN 御坊
ごま #T35 胡麻
ごまか #S5*3 ごまか 誤魔化
ごまつ #T35 語末
ごみ #T35*27 ゴミ #JNS 五味 #T35 塵
ごみばこ #T35*3 ゴミ箱
ごむ #T35*3 ゴム
ごめいじ #OKX*2 ご命じ #OKX*1 ご銘じ
ごめいわく #T05*28 御迷惑 #T05*2 ご迷惑
ごめん #T35*85 ごめん #T35*6 御免
ごめんじ #OKX*29 ご免じ
ごもく #T35*1 五目
ごやく #T30*1 誤訳
ごよう #T35*3 御用 #T30*1 誤用
ごようおさめ #T35 御用納め
ごようたつ #T35 御用達
ごようはじめ #T35 御用始め
ごようめい #T35*1 ご用命 #OKX ご用命 #T35 御用命
ごらいこう #T35 ご来光 御来光
ごらいじょう #T30 ご来場 御来場
ごらいてん #T30*1 御来店 #T30 ご来店
ごらいりん #T30 ご来臨 御来臨
ごらく #T35*1 娯楽
ごらん #T35*37 御覧 #T35*2 ごらん #T35 ご覧
ごりかい #T35*4 御理解 #T35*2 ご理解
ごりむちゅう #T15 五里霧中
ごりやく #T35*2 ご利益
ごりゅう #CN 五竜
ごりゅうせい #T35*1 御隆盛 #T35 ご隆盛
ごりょうしょう #T30*5 御了承 #T30 ご了承
ごりょうしん #T35*2 ご両親 #T35 御両親
ごりよう #T30*13 ご利用 #T30*3 御利用
ごりん #T35*2 五輪
ごるばちょふ #JN ゴルバチョフ
ごるふ #T35*5 ゴルフ
ごれんらく #T30*31 御連絡 #T30*6 ご連絡
ごろ #SUC*22 ごろ #T35*4 語呂 #SUC 頃 #T15 頃
ごろう #JNM 五郎 伍朗
ごろく #T35*1 語録
ごろごろ #T30*5 ゴロゴロ
ごろね #T30*3 ごろ寝
ごろんじ #OKX*52 ご論じ
ごろんと #F14 ごろんと
ごん #KJ*2 権 #KJ*1 言 #KJ 厳 魂
ごんぎょう #T35 勤行
ごんげ #T35*2 権化
ごんどう #JNS 権藤
ごんどわな #CN ゴンドワナ
さ #T35*154 差 #R5*104 去 #S5*103 指 #K5*16 割 #KJ*12 査 #K5*8 咲 #S5*6 差 #KJ*5 左 #S5*4 さ #KJ*4 佐 砂 #KJ*3 鎖 #KJ*2 沙 #S5*2 刺 挿 #KJ*1 狭 嵯 #PRE*1 左 #KJ*1 作 茶 #K5*1 裂 #KJ 些 叉 唆 瑳 詐 裟 挫 #S5 射 #KJ 紗 早 蓑
さーかむすくらいぶ #T30*6 サーカムスクライブ
さーかむすくりぷしょん #T35*16 サーカムスクリプション
さーが #T35 サーガ
さーきっと #T35*1 サーキット
さーくる #T35*2 サークル
さーじ #T35*1 サージ
さーち #T30*23 サーチ
さーば #T35*255 サーバ
さーばー #T35*122 サーバー
さーびす #T30*138 サービス
さーふぃん #T30*2 サーフィン
さーぶ #T30*1 サーブ
さーべい #T35*2 サーベイ
さあ #CJ*6 さあ
さい #T35*255 際 #KJ*135 再 #JS*32 歳 #SX*26 際 #T35*8 差異 #KJ*8 採 #KJ*6 細 #KJ*5 済 #F00*4 最 #T35*3 サイ #KJ*2 彩 才 #KJ*1 債 催 斎 菜 裁 載 西 斉 齋 鰓 #JCN 佐井 #PRE 再 #KJ 最 #PRE 最 #KJ 哉 塞 妻 宰 #JS 才 #KJ 栽 歳 災 采 犀 砕 砦 祭 #SUC 祭 #KJ 財 殺 晒 偲 責 切
さいあい #T35*1 最愛
さいあく #T05*10 最悪
さいう #T35 最右
さいえん #T30 再演 #T35 才媛 菜園
さいえんす #T35*2 サイエンス
さいえんてぃすと #T35*1 サイエンティスト
さいか #T05*1 最下 #T30 裁可
さいかい #T30*8 再開 #T30*4 再会 #CN 西海
さいかぎょう #T35 最下行
さいかく #T30 才覚
さいがい #T35*3 最外 #T35*2 災害
さいがわ #JCN 犀川
さいき #T30*53 再帰 #T30*1 再起 #JCN 佐伯 #T35 才気 #JNS 斎木 斉木
さいきゅう #T35*1 鰓弓
さいきょう #T35*3 最強
さいきん #T35*191 最近 #T35*16 細菌
さいぎ #T35*1 猜疑
さいく #T30*2 細工
さいくつ #T30*1 採掘
さいくりんぐ #T30*2 サイクリング
さいくる #T35*17 サイクル
さいぐさ #JNS 三枝
さいけい #T30*2 再掲
さいけつ #T30*1 採血 #T30 採決 裁決
さいけん #T35 債券 債権 #T30 再建
さいげつ #T35*1 歳月
さいげん #T30*27 再現 #T35*1 際限
さいこ #T35 最古
さいこう #T30*29 再考 #T05*21 最高 #T30 再興
さいこうさい #T35*1 最高裁
さいこうちょう #T35*1 最高潮
さいこく #T30 催告
さいこん #T30 再婚
さいご #T35*190 最後 #T35*2 最期
さいごう #JCN 西郷
さいごく #CN 西国
さいさ #T35*2 最左 #T35 歳差
さいさん #T35*2 採算 #T35*1 再三
さいし #T35*1 妻子
さいしゅ #T30*2 採取
さいしゅう #T35*98 最終 #T30*5 採集
さいしゅうかい #T35 最終回
さいしゅつ #T35 歳出
さいしょ #T35*245 最初
さいしょう #T35*25 最小 #T35*4 最少
さいしょうげん #T35*7 最小限
さいしょうち #T35*2 最小値
さいしょく #T30*1 菜食 #T35 才色
さいしん #T35*88 最新 #T30 再審 #T05 細心
さいじつ #T35*2 祭日
さいじょ #T35*1 才女
さいじょう #T05*1 最上 #JCN 西城 西条
さいすん #T30 採寸
さいず #T35*141 サイズ
さいせい #T30*6 再生 #T35*1 採精 #T30 再製
さいせいき #T35*1 最盛期
さいせき #T30 採石 砕石
さいせん #T30 再選
さいせんたん #T35*1 最先端
さいぜん #T35*3 最前 #T35*2 最善
さいぜんせん #T35*1 最前線
さいそう #T30*5 再送
さいそく #T30*4 催促
さいた #T35 最多 #JCN 財田
さいたい #T35*1 臍帯 #T30 妻帯
さいたく #T30*1 採択
さいたま #JCN 埼玉
さいたん #T35*2 最短
さいだ #JNS 斎田 斉田
さいだー #T35 サイダー
さいだい #T35*59 最大 #T35 細大
さいだいげん #T35*1 最大限
さいだいじ #CN 西大寺
さいだいち #T35*10 最大値
さいだん #T35 祭壇 #T30 裁断
さいち #JNM 佐一 #T35 才知
さいちゅう #T35*26 最中
さいちょう #T35*2 最長
さいてい #T05*21 最低 #T30*1 裁定
さいていげん #T35*9 最低限
さいてき #T05*18 最適
さいてん #T30*2 採点 #T35*1 祭典
さいと #T35*148 サイト #JCN 西都
さいとう #JNS 斎藤 斉藤 齋藤 齊藤
さいとぞる #T35*1 サイトゾル
さいど #T35*23 再度 #T35*7 サイド
さいどく #T30 再読
さいなん #T35*1 災難
さいなんたん #T35 最南端
さいにゅう #T35*1 歳入
さいにん #T30 再任
さいねん #T30 再燃
さいねんしょう #T35 最年少
さいねんちょう #T35 最年長
さいのう #T35*2 才能
さいはい #T30 采配
さいはいち #T30*1 再配置
さいはく #JCN 西伯
さいはつ #T30*2 再発
さいはん #T30 再販
さいばー #PRE*1 サイバー
さいばい #T30*1 栽培
さいばし #R5 才走
さいばん #T30*4 裁判
さいばんかん #T35 裁判官
さいばんしょ #T35*1 裁判所
さいひ #T35*1 採否 #T35 歳費
さいふ #T35*3 財布
さいぶ #T35*6 細部
さいぶん #T30*5 細分
さいへん #T30*7 再編
さいほう #T30*1 裁縫
さいほくたん #T35 最北端
さいぼう #T35 細胞
さいぼうしつ #T35*1 細胞質
さいまつ #T35*1 歳末
さいみつ #T05 細密
さいみん #T35 催眠
さいむ #T35*2 債務
さいもく #T35*1 細目
さいもん #JNS 紫門
さいゆうき #KK*1 西遊記
さいゆうほう #T35*1 最尤法
さいよう #T30*61 採用
さいらい #T30*1 再来
さいりょう #T05*1 最良 #T30*1 裁量
さいれい #T35 祭礼
さいろく #T30*1 採録 載録
さいわい #T05*30 幸い #T00*25 幸い #CN 幸 #T00 幸
さいん #T30*5 サイン
さうんど #T35*6 サウンド
さえ #KSr*11 冴え
さえき #T35*1 差益 #JCN 佐伯
さえぎ #R5 遮
さえぐさ #JNS 三枝
さえこ #JNM さえ子 小枝子
さえつ #T30*23 査閲
さお #JS*1 竿 #T35 竿 #JS 棹
さおり #JN 佐織
さか #S5*29 咲か #T35*19 坂 #KJ*2 阪 #JN 坂 #R5r 盛
さかい #T35*14 堺 #T35*2 境 #JCN 境 #JNS 左海 #JCN 坂井 #JNS 阪井 #JCN 堺 #JNS 酒井
さかいがわ #JCN 境川
さかいで #JCN 坂出
さかいみなと #CN 境港
さかうえ #JNS 坂上
さかうち #JCN 坂内
さかえ #T35*8 栄 #KSr*1 栄え #JCN 栄
さかがみ #JNS 坂上
さかき #JCN 坂城 #T35 榊
さかきた #JCN 坂北
さかきばら #JCN 榊原
さかぐち #JNS 坂口 阪口
さかさ #T15*1 逆さ
さかさい #JCN 逆井
さかした #JCN 坂下
さかしら #T05*1 さかしら
さかずき #T35 杯
さかた #JNS 坂田 #JCN 酒田
さかづき #T35 杯
さかど #JCN 坂戸
さかな #T35*11 魚 #T35*1 肴
さかにわ #JNS 坂庭
さかの #JNS 坂野
さかのぼ #R5*3 遡
さかば #T35*1 酒場
さかべ #JNS 坂部
さかほぎ #JCN 坂祝
さかま #K5 逆巻
さかみち #T35 坂道
さかもと #JCN 坂本
さかや #T35*1 酒屋
さから #W5*2 逆ら
さかり #T35*2 盛り
さかわ #JNS 佐川
さかん #T05*4 盛ん #T05 さかん #T35 佐官 左官
さが #S5r*91 探 #R5r*9 下が #S5r*6 捜 #T35*3 性 #JCN 佐賀 嵯峨
さがえ #JCN 寒河江
さがく #T35*1 差額
さがしあて #KSr*1 捜し当て
さがしだ #S5r*1 捜し出
さがの #JCN 嵯峨野
さがのせき #CN 佐賀関
さがみ #CN 相模
さがみこ #CN 相模湖
さがみはら #CN 相模原
さがら #JCN 相良
さがる #CNSUC1*1 下ル
さがわ #JCN 佐川
さき #T35*255 先 #KJ*26 崎 #SUC*6 先 #T35*2 前 #KJ*1 咲 #T35 左記 #KJ 埼 碕 #JNM 早紀 #KJ 嵜
さきおくり #T30*2 先送り
さきがけ #KJ 魁 #KSr 先駆け
さきごろ #T35*1 先頃
さきさま #T35 先様
さきざき #T35*1 先々
さきだ #T5*3 先立
さきだて #KS 先立て
さきち #JNS 佐吉
さきと #JCN 崎戸
さきどり #T30 先取り
さきばし #R5r 先走
さきまわり #T30 先回り
さきもの #T35 先物
さきや #JNS 崎谷
さきゅう #T35*2 砂丘
さきゆき #T35*1 先行き
さきょう #JCN 左京
さきん #ZX*1 先ん
さぎ #T35*2 詐欺 #T35 鷺
さぎょう #T30*255 作業 #T35 サ行
さぎょうぎ #T35 作業着
さく #T35*27 作 #T35*26 策 #KJ*7 咋 #KJ*5 昨 #KJ*1 炸 #JCN 佐久 #KJ 削 搾 朔 #T35 柵 #KJ 窄 #T35 索 #KJ 錯 酢 乍
さくい #T30*1 作為 #T35 作意
さくいおん #T35 錯イオン
さくいん #T35*31 索引
さくが #T30*4 作画
さくがら #T35 作柄
さくぎ #JNS 昨木
さくげん #T30*8 削減
さくご #T30*4 錯誤
さくし #T30*2 作詞 #T35*1 策士
さくしゃ #T35*11 作者
さくしゅ #T30 搾取
さくじ #T35 作事
さくじつ #T35*33 昨日
さくじょ #T30*255 削除
さくず #T30*28 作図
さくせい #T30*255 作成 #T30*6 作製
さくせん #T35*2 作戦
さくそう #T30*1 錯綜
さくつけ #T30 作付
さくてい #T30*3 策定
さくとう #JCN 作東 #T35 昨冬
さくどう #T30 策動
さくにゅう #T30 搾乳
さくねん #T35*24 昨年
さくねんど #T35*2 昨年度
さくばん #T35 昨晩
さくひょう #T30*2 作表
さくひん #T35*18 作品
さくふ #T30*1 作譜
さくふう #T35*1 作風
さくぶん #T30*4 作文
さくま #JCN 佐久間
さくもつ #T35 作物
さくや #T35*3 昨夜
さくゆ #T30 搾油
さくら #T35*9 桜 #T35*3 櫻 #T35*1 さくら #JCN 佐倉 桜
さくらい #JCN 桜井 #JNS 櫻井
さくらえ #JCN 桜江
さくらがい #T35*4 桜貝
さくらがわ #JCN 桜川
さくらしんまち #CN 桜新町
さくらじま #JCN 桜島
さくらだ #JCN 桜田
さくらだい #JCN 桜台
さくらづつみ #JNS 桜堤
さくらもち #T35 桜餅
さくらん #T30 錯乱
さくりゃく #T30*1 策略
さくれい #T35 作例
さくれつ #T30*1 炸裂
さぐ #R5r*3 探
さぐりあて #KS 探り当て
さぐりだ #S5*1 探り出
さけ #KS*116 避け #T35*50 酒 #B5r*7 叫 #T35*1 鮭
さけい #T30 左傾
さけかわ #JCN 鮭川
さげ #KS*13 下げ #KS*1 提げ
さこ #JNS 佐古 #KJ 迫
さこう #JNS 酒向 酒匂
さこく #T30 鎖国
ささ #R5*13 刺さ #T35*4 笹 #KJ 篠
ささい #T05*5 些細
ささえ #KSr*4 支え
ささお #JNS 笹尾
ささおか #JNS 笹岡
ささかみ #JCN 笹神
ささがわ #JNS 笹川
ささき #JNS 佐々木
ささくら #JCN 笹倉
ささぐり #JCN 篠栗
ささげ #KS*1 捧げ
ささだ #JNS 笹田
ささつ #T30 査察
ささはら #JNS 笹原
ささもと #JNS 笹本
ささや #K5*2 ささや #K5*1 囁 囁や
ささやか #T05*3 ささやか
ささやま #JCN 篠山
ささら #T35 簓
さざ #CN 佐々
さざなみ #T35*3 さざ波 #T35 漣
さざんか #T35*1 サザンカ
さざんぶろっと #T35*1 サザンブロット
さし #D2T35*6 さし
さしあげ #KS*8 差し上げ
さしあたり #F14*1 さしあたり #T36 さしあたり
さしいれ #KSr 差入れ
さしえ #T35*4 挿絵
さしおさえ #KSr 差し押え #T30 差押
さしかえ #KSr*3 差し替え
さしかけ #KS 差しかけ
さしがね #T35 差金
さしき #JCN 佐敷 #T30 挿し木
さしこ #M5r*1 差し込
さしさわり #T35 差障り
さししめ #S5*1 指し示
さしず #T30 指図
さしずめ #CJ さしずめ
さしせま #R5*1 差し迫
さしたる #RT*1 さしたる
さしだ #S5 差し出
さしだしにん #T35 差出人
さしつかえ #KSr*3 差し支え
さしづめ #CJ さしづめ
さして #F14 さして
さしでがまし #KY さしでがまし 差し出がまし
さしでがましゅう #KYU さしでがましゅう 差し出がましゅう
さしとめ #KSr 差し止め
さしどめ #T30 差止め
さしひ #K5r 差し引
さしひかえ #KS*1 差し控え
さしひき #T35*1 差引
さしま #JCN 猿島
さしまわ #S5 差し回
さしみ #T35*3 刺身
さしむかい #T10 差向い
さしむけ #KS 差し向け
さしもど #S5r 差し戻
さしもどし #T35 差戻し
さしゅ #T30 詐取
さしゅう #T30*2 査収
さしょう #T05*1 些少 #T30 査証
さしわたし #T30 差渡し
さしんしつ #T35*1 左心室
さじ #T35*1 些事 #JCN 佐冶 #T35 匙
さじき #T35*1 桟敷
さじま #JCN 佐島
さす #R5*1 さす
さすが #T05*29 さすが #T05*1 流石
さすぺんど #T30*5 サスペンド
さずか #R5*1 授か
さずけ #KS 授け
させ #KS*1 させ #JNS 佐瀬
させつ #T30*2 左折
させぼ #JCN 佐世保
させん #T35*4 鎖線 #T30*1 左遷
さそ #W5r*49 誘
さそいあわ #S5r 誘い合わ
さそいかけ #KSr 誘いかけ
さそいこ #M5r 誘い込
さそいだ #S5r 誘い出
さそく #T35*1 左側
さそり #T35*2 蠍
さそん #T35*1 差損
さた #T30*2 沙汰 #JNS 佐屋 #JCN 佐多
さたけ #JNS 佐竹
さたみさき #CN 佐多岬
さたろう #JNM 佐太郎
さたん #T35*7 左端
さだ #KJ*1 定 #JCN 佐田 #KJ 偵 禎
さだお #JNM 貞男 貞夫 貞雄 定男 定夫 定雄
さだか #T05*5 定か
さだきち #JNM 定吉
さだこ #JNM 貞子
さだま #R5r*6 定ま
さだみつ #JCN 貞光
さだめ #KSr*37 定め
さだゆき #JNM 貞行
さち #KJ*3 幸 #KJ*1 倖
さちお #JNM 幸雄
さちこ #JNM 幸子
さっ #SX*6 察
さっか #T35*2 作家 #T30 擦過
さっかー #T35*2 サッカー
さっかく #T30*2 錯覚
さっき #T35*53 さっき #T35 殺気
さっきゅう #T05*2 早急
さっきょく #T30*2 作曲
さっきん #T30 殺菌
さっこん #T35*2 昨今
さっさと #F14*3 さっさと
さっし #T35*1 冊子 #T35 察し
さっしん #T30*2 刷新
さっすう #T35 冊数
さっそう #F02 さっそう 颯爽
さっそく #F14*44 早速 #F14*10 さっそく #T36 さっそく 早速
さっち #T30 察知
さって #CN 幸手
さっと #F14 さっと
さっとう #T30 殺到
さっぱり #F04*4 さっぱり
さっぴ #K5 さっ引
さっぽろ #CN 札幌
さつ #T35*49 冊 #T35*3 札 #KJ*2 殺 #JS*1 刷 #T35*1 察 #KJ*1 撮 薩 #JS 冊 #KJ 刷 拶 擦 撒
さつい #T35*1 殺意
さつえい #T30*9 撮影
さつがい #T30 殺害
さつき #JN 五月 #T35 皐月
さつきがおか #CN 五月丘
さつじん #T35*2 殺人
さつじんき #T35 殺人鬼
さつばつ #F00*1 殺伐
さつま #JCN 薩摩
さつまいも #T35 サツマイモ 薩摩芋
さつりく #T30 殺戮
さて #CJ*64 さて
さてぃ #JN サティ
さてい #T30*2 査定
さておき #CJ さておき
さと #R5r*19 悟 #T35*10 里 #T35*1 郷 #KJ*1 聡 #KY*1 聡 #S5*1 諭 #KY 敏 #JN 里
さとう #T35*6 砂糖 #KYU*1 聡う #JNS 佐藤
さとこ #JNM 佐登子 砂登子
さとし #JNM 覚 悟 聡 智 智嗣 哲 敏 諭
さとしょう #JCN 里庄
さとみ #JNM 恵規 聡美 智美 #JCN 里美
さとる #JNM 覚 悟 哲 理
さど #JCN 佐渡
さどう #T30*1 作動 #T35 茶道
さどく #T30*4 査読
さどわら #CN 佐土原
さなえ #JNM 早苗
さなぎ #T35*1 蛹
さなごうち #CN 佐那河内
さなだ #JCN 真田
さぬき #JCN 讃岐
さね #KJ 実
さねとう #JNS 実藤 實藤
さの #JCN 佐野
さのう #T35 砂嚢
さは #T35*1 左派
さはい #T30 差配
さはら #JNS 佐原 砂原
さはんじ #T35*2 茶飯事
さば #T35*2 鯆 #K5r*1 裁 #JCN 佐波 #T35 鯖 #K5r 捌
さばえ #JCN 鯖江
さばき #KJ 捌
さばく #T35*13 砂漠
さび #T35*3 錆 #KS 錆び
さびし #KY*11 寂し #KY*8 淋し #KY*3 さびし
さびしゅう #KYU*2 寂しゅう #KYU*1 さびしゅう 淋しゅう
さびしろ #JCN 淋代
さびつ #K5r 錆びつ
さびれ #KS 寂れ
さぴえんす #KK*2 サピエンス #T35*1 サピエンス
さふぁいあ #T35 サファイア
さぶ #PRE*52 サブ
さぶじぇくと #T35*2 サブジェクト
さぶろう #JNM 三朗 三郎
さぶん #T35*9 差分
さぷらい #T30*1 サプライ
さへん #T35*2 左辺
さべつ #T30*6 差別
さほう #T35*1 作法
さほど #F14*1 さほど #T36 さほど
さぼ #R5r*5 サボ #R5r*4 さぼ
さぼう #T35 砂防
さぽーと #T30*171 サポート
さま #JNSUC*213 様 #T35*4 さま #S5r*3 覚ま #T35*2 様 #S5r*1 冷ま #JNSUC さま
さまがわり #T30 様変り 様変わり
さまざま #T05*68 さまざま #T05*26 様々
さまたげ #KSr*4 妨げ
さまつ #T05 瑣末
さまに #CN 様似
さまよ #W5*3 さ迷 #W5*1 さまよ
さまりー #T35*1 サマリー
さみし #KY*3 寂し 淋し
さみしゅう #KYU*1 寂しゅう 淋しゅう
さみず #JCN 三水
さみだれ #T35 五月雨
さむ #KY*19 寒 #T35*4 サム
さむう #KYU*2 寒う
さむえ #T35 作務衣
さむかわ #JCN 寒川
さむけ #T35*1 寒気
さむさ #T35*1 寒さ
さむらい #T35*1 侍
さめ #T35*11 鮫 #KS*5 醒め #KS*2 覚め 冷め
さめかわ #JCN 鮫川
さめざめ #F06 さめざめ
さめしま #JNS 鮫島
さめはだ #T15 鮫肌
さも #F14*1 さも
さもなければ #CJ*1 さもなければ
さもん #T30 査問
さや #T35*4 鞘
さやあて #T35 鞘当て
さやか #JNM 沙也加
さやま #JCN 狭山 #JNS 佐山
さゆう #T30*32 左右
さゆり #JNM さゆり 早百合
さよう #T30*8 作用 #JCN 佐用
さようなら #CJ*4 さようなら #T30*3 さようなら
さよく #T35 左翼
さよなら #CJ*6 さよなら #T30*6 さよなら
さら #T35*72 さら #JS*19 皿 #T35*16 皿 #W5*1 拐 #KJ*1 更 #S5*1 晒 #S5 曝
さらいげつ #T35*1 再来月
さらいしゅう #T35*4 再来週
さらいねん #T35 再来年
さらうんど #T35*1 サラウンド
さらけだ #S5*2 さらけだ #S5 さらけ出
さらさら #F14*2 更々
さらし #T35 晒
さらしな #KK*1 更科 #JCN 更級
さらだ #T35*4 サラダ
さらに #F14*22 更に
さらば #CJ*1 さらば
さらべつ #CN 更別
さらりーまん #T35*1 サラリーマン
さらりと #F12 さらりと
さりげな #KY*4 さりげな
さりげのう #KYU*1 さりげのう
さる #T35*6 猿 #T35*1 さる サル #CN 沙流
さること #T35*1 さること 然ること
さるすべり #T35*1 サルスベリ 百日紅
さるぢえ #T35 猿知恵
さるとる #JN サルトル
さるふぁざい #T35*1 サルファ剤
さるふつ #JCN 猿払
されど #CJ*1 されど
さろま #CN 佐呂間
さわ #KJ*33 澤 #G5r*25 騒 #R5r*20 触 #T35*19 沢 #JCN 佐波 #R5r 障 #JNS 沢
さわい #JCN 沢井
さわうち #JCN 沢内 #JNS 澤内
さわかい #T35 茶話会
さわがし #KY*1 騒がし
さわがしゅう #KYU*1 騒がしゅう
さわぐち #JNS 沢口
さわた #JCN 佐和田
さわたり #JCN 沢渡
さわだ #JNS 沢田 澤田
さわむら #JNS 沢村
さわやか #T05*4 爽やか #T05*1 さわやか
さわら #T35*1 鰆 #JCN 佐原 早良
さわん #T35*2 左腕
さん #JNSUC*255 さん #NN*153 三 #CNSUC1*12 山 #KK*6 サン #T30*3 参 #KJ*3 讃 #ZX*2 参 #T30*2 算 #KJ*2 纂 #KJ*1 山 撒 散 桟 #T30*1 賛 #T35*1 酸 #SUC さん #KJ 傘 参 惨 #ZX 散 #KJ 燦 珊 #CNSUC1 産 #T30 産 #KJ 蚕 餐 蒜
さんい #T35*2 賛意
さんいつ #T30*1 散逸
さんいん #T35 参院 #CN 山陰 #T35 産院
さんか #T30*255 参加 #T35*3 傘下 #T35*2 讃歌 #JS*2 酸化 #T30*1 酸化 #T35 賛歌
さんかい #T30*2 散会 #T30 参会
さんかく #T35*96 △ #T35*48 ▲ #T35*4 三角
さんかくちゅう #T35*1 三角柱
さんかしゃ #T35 参加者
さんかん #T35*2 山間 #T30 参観
さんかんしおん #T35*2 三寒四温
さんが #T30 参賀 #T35 山河
さんがい #T35 三界 惨害
さんがく #T35 山岳 産額
さんがにち #T35*1 三が日
さんがわ #JCN 寒川
さんきゅう #T30 産休
さんきょう #T35 山峡
さんぎいん #KK 参議院
さんぎょう #T35*20 産業
さんぐう #T30 参宮
さんぐらす #T30*2 サングラス
さんけ #T35 産気
さんけい #T30 参詣 #KK 産経
さんけん #T30*2 散見 #T35 三権
さんげき #T35 惨劇
さんこう #T30*127 参考 #CN 三光
さんこうしょ #T35*1 参考書
さんこつ #T30 散骨
さんご #T35*1 珊瑚 #T35 産後
さんごう #CN 三郷
さんごしょう #T35 珊瑚礁
さんさい #T35*1 山菜
さんさく #T30*3 散策
さんさん #F02 燦々
さんざい #T30 散在 散財
さんざん #T05*3 散々
さんし #T35*1 産仔 #T35 蚕糸
さんしゅう #T30*2 参集
さんしゅつ #T30*5 算出 #T30*2 産出
さんしょう #T30*255 参照 #T30 三唱
さんしょううお #T35*1 鯢
さんしょく #T35*1 三食 #T30 蚕食
さんしん #T35 三振
さんじ #T35*2 参事 #T30*1 賛辞 #JNM 三次 #T35 惨事 産児
さんじゅ #T35 傘寿
さんじゅつ #T35*4 算術
さんじょ #T30*1 賛助
さんじょう #T30*3 参上 #T35*1 山上 惨状 #JCN 三条
さんじょく #T35 産褥
さんすい #T30 撒水 散水
さんすう #T35*1 算数
さんせい #T10*40 賛成 #T30*3 産生 #T35*1 酸性 #T35 参政
さんせいどう #KK*1 三省堂
さんせき #T30*1 山積
さんせん #T30 参戦
さんせんそうもく #T35 山川草木
さんぜん #F02*2 燦然
さんそ #T35*1 酸素
さんそう #T35 山荘
さんそん #T35 山村
さんぞく #T35*1 山賊
さんたい #T35*1 算体
さんたくろーす #JN サンタクロース
さんたん #T02 惨澹 #T30 賛嘆
さんだ #JCN 三田
さんだつ #T30 簒奪
さんだん #T35*3 散弾 #T30*1 算段
さんだんばら #T15 三段腹
さんち #T35*1 産地 #T35 山地
さんちゅう #T35*1 山中
さんちょう #T35*1 山頂
さんてい #T30*1 算定
さんてん #T35*2 …
さんとう #JCN 三島 山東
さんとりー #KK*1 サントリー
さんど #T35 サンド
さんどう #T35*6 参道 #T30*6 賛同
さんない #CN 山内
さんなん #CN 山南
さんにゅう #T30*1 参入 #T30 算入
さんにん #T35*8 三人
さんにんしょう #T35 三人称
さんのう #CN 山王
さんのへ #JCN 三戸
さんのみや #CN 三の宮
さんはんきかん #T35 三半規管
さんば #T35*1 産婆
さんばし #T35*1 桟橋
さんばん #T35*3 三番
さんぱい #T30 参拝 惨敗
さんぱつ #T30*1 散発 #T30 散髪
さんび #T30 賛美
さんぴ #T35*2 賛否
さんふじんか #T35*1 産婦人科
さんふらんしすこ #CN サンフランシスコ
さんぶ #JNS 山武
さんぶつ #T35*2 産物
さんぶん #T30 三分 #T35 散文
さんぷ #T30*2 散布 #T35*1 算譜 #T35 産婦
さんぷく #T35 山腹
さんぷりんぐ #T30*13 サンプリング
さんぷる #T35*58 サンプル
さんぺい #JCN 三平
さんぼう #T35 参謀
さんぼく #JNS 山北
さんぼんぎ #JCN 三本木
さんぽ #T30*1 散歩
さんぽう #T35*10 算法
さんまいくろしすてむず #KK*1 サンマイクロシステムズ
さんまん #T05*1 散慢
さんみ #T35 三位 酸味
さんみゃく #T35*1 山脈
さんめんろっぴ #T35 三面六膂
さんもんばん #T35 三文判
さんゆ #T35 産油
さんよ #T30 参与
さんよう #KK*1 三洋 #CN 山陽 #T30 算用
さんようちゅう #T35 三葉虫
さんらん #T30*4 散乱 #T30 産卵
さんらんけい #T35*1 産卵鶏
さんりく #CN 三陸
さんりん #T35 山林
さんれつ #T30*1 参列
さんろく #T35 山麓
さんわ #KK*1 三和 #JCN 三和
ざ #T30*33 座 #KJ*3 坐 #KJ*1 挫
ざあざあ #F06 ざあざあ
ざい #KJ*3 剤 #SUC*3 罪 #KJ*2 在 #T35*2 財 #SUC*1 材 #SUC 剤 #T35 材 罪
ざいあく #T05*1 罪悪
ざいい #T30 在位
ざいか #T35 財貨
ざいかい #T35*1 財界
ざいがい #T35*1 在外
ざいがく #T30*1 在学
ざいきょう #T30 在京
ざいきん #T30 在勤
ざいけい #T35*1 財形 財経
ざいげん #T35*2 財源
ざいこ #T35*7 在庫
ざいこう #T30 在校
ざいさん #T35*3 財産
ざいしつ #T35 材質
ざいしょ #T35 在所
ざいしょく #T30*1 在職
ざいじゅう #T30*2 在住
ざいじょう #T35 罪状
ざいせい #T35*2 財政
ざいせき #T30 在籍
ざいたく #T30*3 在宅
ざいだん #T35*1 財団
ざいだんほうじん #T35*2 財団法人
ざいにち #T30 在日
ざいにん #T30 在任 #T35 罪人
ざいばつ #T35 財閥
ざいほう #T35*4 財宝
ざいむ #T35*1 財務
ざいもく #T35*1 材木
ざいもくや #JNS 材木谷
ざいらい #T35*1 在来
ざいりゅう #T30 在留
ざいりょう #T35*13 材料
ざいりょく #T35 財力
ざおう #CN 蔵王
ざか #CNSUC1*2 坂
ざき #T35*9 崎
ざくろ #T35 ザクロ 石榴
ざけ #T35 酒
ざこう #T35 座高
ざしき #T35*1 座敷
ざしょう #T30 座礁 #T35 挫傷
ざせき #T35*8 座席
ざせつ #T30*2 挫折
ざぜん #T30 坐禅 座禅
ざぞう #T35 座像
ざた #T35 沙汰
ざだん #T30*1 座談
ざちょう #T35*2 座長
ざっか #T35 雑貨
ざっき #T30*1 雑記
ざっきょ #T30*1 雑居
ざっくばらん #T05 ざっくばらん
ざっし #T35*27 雑誌
ざっしゅ #T35*1 雑種
ざっしょく #T30 雑食
ざっそう #T35 雑草
ざった #T05*2 雑多
ざっと #F14*3 ざっと
ざっとう #T30 雑踏
ざっぱい #T35 雑俳
ざっぱく #T08 雑駁
ざっぴ #T35*1 雑費
ざっぽう #T35 雑報
ざつ #T05*5 雑
ざつえき #T35 雑益
ざつおん #T35*1 雑音
ざつがく #T35 雑学
ざつじ #T35*1 雑事
ざつぜん #F00*2 雑然
ざつだん #T30*3 雑談
ざつねん #T35*1 雑念
ざつぶん #T35 雑文
ざつむ #T35*2 雑務
ざつよう #T35*11 雑用
ざひょう #T35*43 座標
ざぶとん #T35 座蒲団 座布団
ざま #D2T35*1 様 #T35 ざま #JCN 座間
ざまみ #JNS 座間見
ざやく #T35 座薬
ざゆう #T35*1 座右
ざら #T05 ざら #T35 皿
ざわ #KJ 沢
ざわめ #K5r*3 ざわめ
ざん #T35*19 残 #CNSUC1*10 山 #KJ*1 斬 #KJ 惨 #SUC 算 #KJ 暫
ざんがい #T35*2 残骸
ざんがく #T35 残額
ざんき #T35*1 残基
ざんきょう #T35 残響
ざんきん #T35*1 残金
ざんぎゃく #T05 残虐
ざんぎょう #T30*29 残業
ざんげ #T30 懺悔
ざんげん #T30 讒言
ざんこく #T05*2 残酷
ざんごう #T35 塹壕
ざんさい #T35 残滓
ざんさつ #T30 惨殺
ざんし #T35 残滓
ざんしょ #T35*3 残暑
ざんしょう #T35 残照
ざんしん #T05 斬新
ざんじ #T35 暫時
ざんせつ #T35 残雪
ざんそん #T30*2 残存
ざんぞう #T35 残像
ざんぞん #T30*2 残存
ざんだか #T35*17 残高
ざんてい #T30*6 暫定
ざんにん #T05 残忍
ざんねん #T05*87 残念
ざんぱい #T30 惨敗
ざんぱん #T35 残飯
ざんぴん #T35 残品
ざんぶ #T35 残部
ざんぼう #T30 讒謗
ざんまい #T15*2 三昧
ざんむ #T35 残務
ざんよ #T35 残余
ざんりゅう #T30*1 残留
し #sxi*255 し #R5*255 知 #CNSUC1*242 市 #T35*143 氏 #N5*95 死 #JNSUC*80 氏 #R5*70 し #SUC*61 子 #T35*59 死 #K5*37 し #M5*25 染 #KJ*23 詞 #T35*12 志 #NN*10 四 #KJ*8 紙 #T35*7 士 #K5*7 敷 #KJ*4 子 #KJ*3 始 #T35*3 師 #KJ*3 紫 試 #KJ*2 仕 使 司 #T35*2 市 #KJ*1 史 嗣 支 斯 私 肢 #T35*1 至 詩 #KJ*1 誌 #S5*1 資 #T30*1 資 #T35*1 巳 #KJ 祇 此 匙 仔 伺 刺 #CNSUC1 史 #SUC 史 士 #KJ 姉 姿 屍 #SUC 師 #KJ 思 指 孜 施 旨 枝 #SUC 枝 #KJ 止 #SX 死 #KJ 獅 祉 糸 #JS 紙 #KJ 脂 視 #N2T30 視 #SUC 詞 #JS 誌 #SUC 誌 #KJ 諮 賜 雌 飼 歯 次 示 自 偲 柴 芝 笥 弛 砥 #M5 凍 #KJ 之 覗 髭 矢
しーく #T30*2 シーク
しーくぇんす #T35*1 シークェンス
しーけんさ #T35*23 シーケンサ
しーけんしゃる #T05*1 シーケンシャル
しーけんす #T35*31 シーケンス
しーずん #T35*3 シーズン
しーた #T35*2 θ #T35*1 Θ
しーと #T35*17 シート
しーとふぃーだ #T35*1 シートフィーダ
しーど #T30*1 シード
しーま #KK*1 シーマ
しーるど #T30*3 シールド
しーん #T35*13 シーン
しあい #T30*6 試合
しあが #R5r*3 仕上 #R5r*2 仕上が
しあげ #KSr*4 仕上げ
しあたー #T35*1 シアター
しあわせ #T05*12 幸せ #T05*2 しあわせ
しあん #T35*3 私案 #T35*1 シアン #T30*1 思案
しい #KS*35 強い #T35*3 椎 #T35*1 しい 私意 恣意 #T35 四囲 #T30 思惟
しいか #T35*1 詩歌
しいく #T30 飼育
しいたけ #T35 椎茸
しいたげ #KS 虐げ
しいだ #JNS 椎田
しいな #JCN 椎名
しいの #JNS 椎野
しいば #JCN 椎葉
しいれ #KSr*7 仕入れ
しいん #T35*5 子音 #T35 死因 #T30 試飲
しうち #T35*1 仕打
しうら #JCN 市浦
しうんじ #CN 紫雲寺
しうんてん #T30 試運転
しぇーでぃんぐ #T30*2 シェーディング
しぇあ #T35*4 シェア
しぇありんぐ #T30*1 シェアリング
しぇぱーど #T35*1 シェパード
しぇる #T35*108 シェル
しぇるた #T35 シェルタ
しえい #T35 市営 私営
しえき #T30 使役 #T35 私益
しえん #T30*91 支援 #T35 私怨 紫煙
しお #T35*10 塩 #T35*3 潮 #JNM 史生 #JCN 志雄 #T35 汐
しおから #KYmi 塩辛 #T35 塩辛
しおかわ #JCN 塩川
しおがま #JCN 塩竃 塩釜
しおくり #T30 仕送り
しおけ #T35 塩気
しおざわ #JCN 塩沢
しおじり #JCN 塩尻
しおた #JCN 塩田
しおだ #JCN 塩田
しおづけ #T30 塩漬け
しおどめ #CN 汐留
しおの #JCN 塩野
しおのえ #JCN 塩江
しおはま #JCN 塩浜
しおばら #JCN 塩原
しおひがり #T35*1 潮干狩
しおみ #JCN 塩見
しおみず #T35 塩水
しおもと #JNS 塩本
しおや #JCN 塩屋 塩谷
しおやき #T30 塩焼
しおり #T35*2 栞
しおん #T35*2 紫苑
しか #R5*19 叱 #T35*8 歯科 #T35*2 鹿 #R5*1 しか #T35 市価 #JCN 志賀
しかい #T30*8 司会 #T35*2 視界 #T35 市会 歯科医
しかえし #T30*2 仕返し
しかおい #JCN 鹿追
しかく #T35*180 □ #T35*42 ■ #T35*26 資格 #T35*19 視覚 #T05*18 四角 #KYmi*16 四角 #T35 死角
しかくう #KYU*2 四角う
しかけ #KSr*13 仕掛け #T35 仕掛
しかし #CJ*100 しかし
しかしながら #CJ*11 しかしながら
しかた #T35*64 仕方
しかっけい #T35*3 四角形
しかつ #JCN 師勝
しかの #JCN 鹿野
しかばね #T35 屍
しかべ #JCN 鹿部
しかま #JCN 飾磨 #JNS 色麻
しかまち #CN 鹿町
しかみ #JNS 爾見
しかも #CJ*32 しかも
しからば #CJ しからば
しかり #T35 然り
しかりつけ #KS 叱りつけ
しかるべき #RT*2 然るべき
しかん #T35*2 士関 #T30 仕官 #T35 士官 #T30 弛緩
しが #JCN 四賀 志賀 滋賀
しがい #T35*5 市街 #T35*1 市外 #T35 死骸
しがいせん #T35*1 紫外線
しがく #T35 史学 志学 私学 歯学
しがこうげん #CN 志賀高原
しがらき #JCN 信楽
しがらみ #T35*1 しがらみ
しがん #T30*1 志願
しき #T35*32 式 #T30*25 指揮 #R5r*6 仕切 #T35*2 四季 #JS*2 式 #KJ*2 色 #T35*1 始期 死期 #JCN 磯城 #KK 史記 #JCN 志木 #SUC 式 #KJ 識 織 敷 鋪
しきい #T35 敷居
しきいき #T35 識閾
しきいち #T35*4 閾値
しききん #T35 敷金
しきさい #T35*1 色彩
しきし #T35*3 色紙
しきしま #JCN 敷島
しきしゃ #T35*1 識者
しきじ #T35 式辞
しきじょう #T35*1 式場
しきそ #T35 色素
しきそう #T35 色相
しきだ #JNS 敷田
しきち #T35 敷地
しきちょう #T35*1 色調
しきつめ #KS*1 敷き詰め
しきてん #T35 式典
しきふ #T35 敷布
しきぶ #T35*2 式部
しきべつ #T30*34 識別
しきべつし #T35*27 識別子
しきもう #T35 色盲
しきゅう #T05*19 至急 #T30*8 支給 #T35*1 子宮
しきゅうかく #T35*1 子宮角
しきょ #T30 死去
しきょう #T35 市況
しきょく #T35 支局
しきり #T35 仕切り
しきりに #F14 しきりに
しきん #T35*2 資金 #T35 至近
しきんぐり #T30*1 資金繰り
しぎ #T35*1 仕儀 #T35 市議 鴫 #JCN 信貴
しぎゃく #T35 嗜虐
しぎょう #T30*5 始業
しぎん #T35 詩吟
しく #M5r*14 仕組 #T35 市区
しくしく #F14*2 しくしく
しくはっく #T30*1 四苦八苦
しぐさ #T35*1 しぐさ #T35 仕草
しぐなる #T35*18 シグナル
しぐま #T35*87 Σ #T35*1 σ
しぐれ #T35*1 時雨
しけい #T35 死刑
しけつ #T30 止血
しけん #T30*55 試験 #T35*2 私見
しけんかん #T35 試験管
しけんじょう #T35 試験場
しげ #KJ*5 茂 #KJ*4 重 #R5*2 しげ 茂 #R5*1 繁 #KJ 繁
しげお #JNM 重男 重夫 重雄 繁男 繁夫 繁雄 茂男 茂夫 茂雄
しげかず #JNM 重一
しげき #T30*3 刺激 #JNM 重樹 茂樹
しげこ #JNM シゲ子 茂子
しげし #JNS 茂志
しげしげ #F06 しげしげ
しげた #JNS 重田
しげのぶ #JCN 重信
しげはる #JNM 重治
しげひこ #JNM 重彦
しげまつ #JN 重松
しげみ #T30 茂み
しげみつ #JNM 重光
しげゆき #JNM 重行 茂行
しげよし #JNM 重義
しげる #JNM 滋 繁 茂
しげん #T35*52 至言 #T35*38 資源
しこ #M5r*2 仕込
しこう #T30*123 思考 #T30*63 指向 #T30*29 試行 #T30*6 施行 #T30*4 嗜好 #T30*1 志向 #T05*1 至高
しこく #CN 四国
しこたん #CN 色丹
しこん #T35*2 紫紺 #T35 歯根
しご #T35*16 死後 #K5r*1 しご #K5r 扱 #T30 私語
しごう #JNS 紫合
しごく #F14*1 しごく 至極
しごと #T30*231 仕事
しさ #T30*6 示唆
しさい #T05*1 子細 #T35 司祭
しさく #T30*11 試作 #T30*1 思索 施策
しさつ #T30*1 視察 #T30 刺殺
しさん #T35*11 資産 #T30*1 試算 #T30 四散
しざい #T35*1 資材 #T35 私財
しし #T35*2 獅子 #T35 四肢 #KJ 宍
ししくい #CN 宍喰
ししつ #T35*1 支質 資質
ししど #JNS 宍戸
ししまい #T35 獅子舞
ししゃ #T35*16 支社 #T35*1 使者 死者 #T30*1 試写 #T30 試射
ししゃく #T35*1 子爵
ししゃごにゅう #T30*2 四捨五入
ししゃちょう #T35 支社長
ししゅ #T30 死守
ししゅう #T30 刺繍 #T35 屍臭 詩集
ししゅうき #T35*1 思秋期
ししゅつ #T30*7 支出
ししゅんき #T35*1 思春期
ししょ #T35*1 支所 #T35 司書 支署
ししょう #T35*7 支障 #T35*2 師匠 #T30*1 死傷 #T35 刺傷
ししょく #T30*1 試食
ししょばこ #T35 私書箱
ししん #T35*8 私信 #T35*5 指針 #T35 私心 #T30 視診
しじ #T30*65 指示 #T35*53 私事 #T30*6 支持 #T30 師事
しじみ #T35*1 蜆
しじゅう #T35*1 四重 始終
しじゅうから #T35 シジュウカラ 四十雀
しじゅうそう #T35*1 四重奏
しじゅく #T35 私塾
しじゅん #T35 示準
しじょ #T35 子女
しじょう #T35*6 市場 #T35*1 史上 #JCN 四条 #T35 私情 紙上 至上 #T30 試乗 #T35 誌上
しじょうなわて #CN 四條畷
しじん #T35*1 詩人
しすい #CN 酒々井 #JCN 泗水
しすう #T35*4 指数
しすていん #T35*1 システイン
しすてまてぃっく #T05 システマティック
しすてむ #T35*255 システム
しすてむず #T35*6 システムズ
しすとりっく #T05*1 シストリック
しず #M5*1 沈 #JNM 静
しずえ #JNM 静江 静枝
しずお #JNM 静男 静夫 静雄
しずおか #JCN 静岡
しずか #T05*4 静か #JNM 静 静香
しずがわ #JCN 志津川
しずく #T35 雫 滴
しずくいし #JCN 雫石
しずけさ #T35*1 静けさ
しずこ #JNM 静子
しずない #CN 静内
しずま #R5r 静ま 鎮ま
しずまりかえ #R5 静まり返
しずめ #KSr*1 鎮め #KSr 静め #KS 沈め
しせい #T35*4 姿勢 #T35*1 雌性 #T35 市井 市制 市政 #T30 施政 #T35 私製
しせいかつ #T35*1 私生活
しせいかん #T35 死生観
しせいじ #T35 私生児
しせき #T35 史跡
しせつ #T30*3 施設 #T35 使節 私設
しせる #RT 死せる
しせん #T35*2 視線 #CN 四川 #T35 支線 死線
しぜん #T05*39 自然
しそ #T35 紫蘇
しそーらす #T35*1 シソーラス
しそう #T35*6 思想 #T30 試走 #JCN 宍粟
しそうか #T35 思想家
しそく #T35*1 子息
しそん #T35*2 子孫
しぞう #T30 死蔵
しぞく #T35*1 氏族 #T35 士族
した #T35*255 下 #T35*11 舌 #PRE*3 下 #W5*1 慕
したい #T35*2 姿態 #T35 死体 肢体
したうけ #T30*1 下請け
したえ #T35 下絵
したが #W5*154 従 #W5*4 したが
したがき #T30*1 下書き
したがって #CJ*28 従って #CJ*4 したがって
したぎ #T35 下着
したく #T30*3 支度
したざわり #T35*1 舌触り
したし #KYmi*6 親し #M5r*5 親し
したしゅう #KYU*1 親しゅう
したしらべ #T30 下調べ
したじ #T35*1 下地
したじき #T35*1 下敷
したた #R5r*1 滴
したたか #T05 強か
したため #KS*1 認め
したたらず #T05 舌足らず
しただ #JCN 下田
したづみ #T30 下積み
したて #KSr*2 仕立て #T05 下手
したどり #T30*1 下取り
したね #T35 下値
したび #T05*1 下火
したまち #T35*1 下町
したまわ #R5r*1 下回
したみ #T30*4 下見
したむき #T35*2 下向き
したや #T35*7 ↓
したら #JCN 設楽
したわし #KY 慕わし
したわしゅう #KYU 慕わしゅう
しだ #JCN 志太 志田
しだい #D2T35*74 次第 #T35*17 次第 #T35*1 私大
しだいすう #T35*1 始代数
しだし #T30 仕出し
しだれ #KSr 垂れ
しだん #T35*1 師団 #T30 指弾
しち #NN*4 七 #T35 質
しちがはま #CN 七ヶ浜
しちけん #T35 質権
しちしょう #T35 七生
しちじょう #JCN 七城
しちずん #KK シチズン
しちてんばっとう #T30*1 七転八倒
しちなん #T35*1 七難
しちのへ #JCN 七戸
しちめんどう #T05 しち面倒
しちや #T35 質屋
しちゅー #T35*2 シチュー
しちゅう #T35 市中 支柱
しちょう #T35*1 市長 #T30*1 視聴 #T35 思潮 支庁 #T30 試聴
しちょうそん #T35*1 市町村
しちょく #T35 司直
しちよう #T35 七曜
しちろう #JNM 七郎
しっかく #T30*2 失格
しっかり #F04*23 しっかり
しっかん #T35 疾患
しっき #T35 漆器
しっきゃく #T30 失脚
しっきん #T30 失禁
しっくり #T30*1 しっくり
しっけい #T00 失敬
しっこう #T30*3 執行 #T30*1 失効
しっこく #T35 桎梏
しっさく #T30 失策
しっしょう #T30 失笑
しっしょく #T30 失職
しっしん #T30*1 失神 #T35 湿疹
しっせい #T35 失政
しっせき #T30*1 叱責
しっそ #T05 質素
しっそう #T30*2 疾走 #T30*1 失踪
しっそく #T30 失速
しった #T30*1 叱咤
しったい #T35*1 失態
しっち #T35 湿地
しっちょう #T30 失調
しっつい #T30*1 失墜
しってん #T30 失点
しっと #T30*1 嫉妬
しっとう #T30 執刀
しっとり #T30 しっとり
しっぱい #T30*154 失敗
しっぴつ #T30*43 執筆
しっぴつしゃ #T35 執筆者
しっぷ #T30*4 湿布
しっぷう #T35*1 疾風
しっぺい #T35 疾病
しっぽ #T35 尻尾
しっぽう #CN 七宝
しっぽり #F06 しっぽり
しつ #T35*38 室 #SUC*18 質 #KJ*2 失 #T35*1 質 #KJ 叱 執 嫉 #SUC 室 #KJ 悉 湿 漆 疾 膝
しつい #T35 失意
しつう #T35 止痛
しつおん #T35 室温
しつかん #T35*3 質感
しつき #CN 後月
しつぎ #T30*3 質疑
しつぎょう #T30 失業
しつけ #KSr しつけ #T30 仕付 #T35 躾 #KS 躾け
しつげん #T35*1 湿原 #T30 失言
しつこ #KY*4 しつこ
しつこう #KYU*1 しつこう
しつごしょう #T35*1 失語症
しつじ #T35 執事
しつじつ #T05 質実
しつじゅん #T05 湿潤
しつちょう #T35*5 室長
しつてき #T05*2 質的
しつでん #T35 湿田
しつど #T35*1 湿度
しつない #T35*2 室内
しつないがく #T35*1 室内楽
しつねん #T30*2 失念
しつぼう #T30*1 失望
しつむ #T30 執務
しつめい #T30 失明
しつもん #T30*216 質問
しつよう #T05*2 執拗 #T05 執よう
しつりょう #T35*2 質量
しつれい #T00*52 失礼
しつれん #T30 失恋
してぃ #T35*2 シティ
してい #T30*255 指定 #T35 師弟 私邸
していせき #T35*1 指定席
してき #T30*75 指摘 #T05*4 私的 #T15*1 至適 #T05 史的 #T16 詩的
してつ #T35 私鉄
してん #T35*35 視点 #T35*31 支店 #T35*5 始点 #T35*3 支点
してんちょう #T35 支店長
してんのう #T35*1 四天王
しでか #S5 しでか
しでん #T35 市電
しと #T35*1 使徒 #T35 使途
しとう #T30 死闘
しとね #KJ 褥
しとみ #T35 蔀
しど #JCN 志度
しどう #T30*20 指導 #T30*1 始動 #T35 私道
しな #T35*20 品 #D2T35*1 しな #KK*1 シナ #KJ*1 科 #W5 しな #JS 品
しない #T35*30 市内 #T35 竹刀
しなお #S5 しなお し直
しなかず #T35*1 品数
しながわ #JCN 品川
しなぎれ #T30*1 品切れ
しなこ #JNM 信成子
しなさだめ #T30 品定め
しなじな #T35 品々
しなの #CN 信濃
しなび #KS 萎び
しなぷす #T35*1 シナプス
しなもの #T35*4 品物
しなりお #T35*42 シナリオ
しなん #T05*1 至難 #T30 指南
しなんばん #T35 指南番
しにあ #T35 シニア
しにせ #T35 老舗
しにたえ #KS 死に絶え
しにはて #KS 死に果て
しにゅう #T30*1 刺入
しにょう #T35 し尿
しにん #T35*1 死人 #T30*1 視認
しね #KS 死ね
しねま #T35*1 シネマ
しねん #T30 思念
しの #T35*14 篠 #G5r*5 しの #B5r*4 忍 #B5*1 偲 #G5r*1 凌 #JN 篠
しのざき #JCN 篠崎
しのだ #JNS 篠田
しのづか #JNS 篠塚
しのにむ #T35*1 シノニム
しのはら #JCN 篠原
しのびこ #M5r*1 忍び込
しのびよ #R5 忍び寄
しのぶ #T35*1 忍 #KJ 偲 #JN 忍
しはい #T30*4 支配 #T35 紙背 賜杯
しはいにん #T35*1 支配人
しはつ #T35*5 始発
しはら #W5r*3 支払
しはらい #T35*5 支払
しはらいにん #T35 支払人
しはん #T30*7 市販 #T35*1 紫斑 #T35 師範 死斑
しはんだい #T35 師範代
しば #T35*8 芝 #R5r*4 縛 #T35*3 柴 #JNS 柴
しばい #T30*2 芝居
しばうら #JCN 芝浦
しばかわ #JCN 芝川
しばさき #JCN 柴崎
しばざき #JNS 柴崎
しばし #F14*1 しばし #T36 しばし #F14 暫し #T36 暫し
しばしば #F14*13 しばしば
しばた #JCN 柴田 新発田
しばはら #JNS 芝原
しばふ #T35*6 芝生
しばまた #CN 柴又
しばやま #JCN 柴山 芝山
しばらく #F12*40 しばらく #F12*9 暫く #T36 しばらく 暫く
しばりつけ #KSr 縛り付け
しひ #T35*1 市費
しひょう #T35*8 指標
しびあ #T05*2 シビア
しびっく #KK*1 シビック
しびと #T35 死人
しびれ #KSr*2 痺れ
しふく #T35*1 私服 #T30*1 雌伏 #T35 私腹 至福
しふと #T30*50 シフト
しふん #T35 私憤 脂粉
しぶ #KYmi*21 渋 #R5*19 渋 #T35*19 渋 #T35*3 支部 #T35 市部
しぶう #KYU*2 渋う
しぶかわ #JCN 渋川 #T35 渋皮
しぶがき #T35 渋柿
しぶし #CN 志布志
しぶしぶ #F06*1 渋々 #F06 しぶしぶ
しぶつ #T35*2 私物
しぶと #KY しぶと
しぶとう #KYU しぶとう
しぶや #JCN 渋谷
しぶん #T35 詩文
しぶんぎ #T35 四分儀
しへい #T35 紙幣
しべ #T35 蕊
しべちゃ #CN 標茶
しべつ #CN 士別 #T30 死別 #CN 標津
しべりあ #CN シベリア
しべん #T30 思弁 #T05 至便
しほう #T35*1 司法 四方 視法 #T35 至宝
しほうしょし #T35 司法書士
しほろ #CN 士幌
しほん #T35*1 資本
しほんか #T35 資本家
しほんきん #T35*1 資本金
しぼ #R5*5 しぼ 絞 #T30*2 思慕 #M5 萎 #R5 搾
しぼう #T30*2 志望 #T35*2 脂肪 #T30*1 死亡
しぼうこう #T35 志望校
しぼつ #T30 死没
しぼりあげ #KS 絞り上げ 搾り上げ
しぼりだ #S5*1 絞り出
しま #W5r*255 しま #T35*90 島 #R5r*5 閉ま #T35*4 縞 #W5r*1 仕舞 #JCN 志摩 #R5r 締ま #JNS 島 嶋 #KJ 嶋 嶌
しまい #T35*2 姉妹 #T35 終い
しまうま #T35*1 シマウマ
しまかげ #T35 島陰
しまがはら #CN 島ヶ原
しまぐに #T35 島国
しまさき #JNS 島崎 嶋崎
しまざき #JNS 島崎 嶋崎
しまじり #JCN 島尻
しまだ #JCN 島田 #JNS 嶋田
しまつ #T30*2 始末
しまづ #JNS 島津 嶋津
しまね #JCN 島根
しまの #JNS 島野 嶋野
しまばら #JCN 島原
しままき #CN 島牧
しままと #JNS 島本
しまむら #JNS 島村 嶋村
しみ #KS*1 凍み #T35 紙魚 #KSr 浸み 染み
しみこ #M5r 染み込
しみじみ #F04 しみじみ
しみず #T35*10 清水 #JCN 清水
しみつ #K5r 染み着
しみどうふ #T35 凍み豆腐
しみぬき #T30*1 染み抜き
しみゃく #T35 支脈
しみゅれーしょん #T30*25 シミュレーション
しみゅれーた #T35*5 シミュレータ
しみゅれーと #T30*2 シミュレート
しみょう #T15 至妙
しみん #T35*1 市民
しみんぜい #T35 市民税
しむかっぷ #CN 占冠
しむけ #T35 仕向 #KS 仕向け
しむら #JCN 志村
しめ #S5*255 示 #KS*72 占め #KS*7 閉め #R5r*4 湿 #KSr*3 締め #T35*1 〆 #KS 絞め #CN 志免 #T35 締
しめい #T35*18 氏名 #T30*3 指名 #T35*1 使命 #T35 指命
しめき #R5r*1 締め切 #R5r 閉め切
しめきり #T35*33 〆切 #T35*22 締切
しめきりび #T35*2 締切日
しめくく #R5r*1 締めくく
しめし #T35 示し
しめだ #S5r*1 締め出
しめっぽ #KY 湿っぽ
しめつ #T30 死滅
しめつけ #KSr 締めつけ 締め付け
しめて #F14 締めて
しめび #T35 締め日
しめやか #T05 しめやか
しめん #T35*1 紙面 誌面
しめんそか #T35*1 四面楚歌
しも #T35*29 下 #CNPRE*1 下 #T35*1 霜 #JNS 下
しもーぬ #JN シモーヌ
しもあがた #CN 下県
しもいち #JCN 下市
しもいな #CN 下伊那
しもうさ #CN 下総
しもお #JNS 下尾
しもかまがり #CN 下蒲刈
しもかわ #JCN 下川
しもがれ #T30 霜枯れ
しもき #T35*2 下期
しもきた #CN 下北
しもきたざわ #CN 下北沢
しもきたやま #CN 下北山
しもぎょう #CN 下京
しもげ #JCN 下毛
しもこしき #JCN 下甑
しもごう #CN 下郷
しもざ #T35 下座
しもじ #JCN 下地
しもじょう #JCN 下条 #JNS 下條
しもすわ #CN 下諏訪
しもたかい #CN 下高井
しもだ #JCN 下田
しもだて #JCN 下館
しもつ #JCN 下津
しもつが #JCN 下都賀
しもつき #T35 霜月
しもつけ #JCN 下野
しもつじ #JNS 下辻
しもつま #JCN 下妻
しもづま #JCN 下妻
しもて #T35 下手
しもなり #JNS 下成
しもにいかわ #CN 下新川
しもにた #CN 下仁田
しものせき #CN 下関
しもはんき #T35 下半期
しもばしら #T35 霜柱
しもふさ #CN 下総
しもへい #CN 下閉伊
しもべ #JCN 下部
しもましき #JCN 下益城
しもみのち #CN 下水内
しもむら #JCN 下村
しもやけ #T30*1 霜焼け
しもやま #JCN 下山
しもん #T30*4 諮問 #T30*3 試問 #T35 指紋
しゃ #T35*171 社 #T35*134 者 #T35*11 車 #SUC*8 者 #SUC*6 社 #KJ*4 射 #SUC*2 舎 #KJ*2 写 謝 #KJ*1 舎 赦 斜 #T35*1 紗 #KJ*1 者 #SX*1 謝 #KJ*1 遮 #KJ 沙 砂 捨 煮
しゃーぷ #T05*108 シャープ #T35*1 ♯
しゃい #T35*1 謝意
しゃいん #T35*18 社員
しゃうん #T35 社運
しゃえい #T30*1 射影
しゃおく #T35 社屋
しゃおん #T30*1 謝恩 #T30 遮音
しゃか #T35*1 釈迦
しゃかい #T35*40 社会
しゃかいとう #KK 社会党
しゃがい #T35*24 社外
しゃがいひ #T35*2 社外秘
しゃきっと #F14 しゃきっと
しゃきょう #T30 写経
しゃぎょう #T35 社業
しゃく #T35*2 尺 #KJ 借 #JS 勺 尺 #T35 杓 #KJ 灼 爵 #T35 酌 #KJ 釈 錫 昔 石 赤 #T19 癪
しゃくい #T35*1 爵位
しゃくざい #T30 借財
しゃくし #T35*2 杓子
しゃくしじょうぎ #T15 杓子定規
しゃくじょう #T35*1 錫杖
しゃくぜん #F00 釈然
しゃくど #T35*1 尺度
しゃくねつ #T30*1 灼熱
しゃくはち #T35 尺八
しゃくほう #T30 釈放
しゃくめい #T30 釈明
しゃくやく #T35 シャクヤク 芍葯
しゃくよう #T30*5 借用
しゃくりあげ #KS しゃくりあげ
しゃくりょう #T30 酌量
しゃけ #T35 鮭
しゃけん #T30*2 車検
しゃげき #T30*1 射撃
しゃこ #T35*1 車庫
しゃこう #T30 斜交 斜行 #T35 社交 #T30 遮光
しゃこたん #CN 積丹
しゃさい #T35 社債
しゃさつ #T30 射殺
しゃざい #T30*1 謝罪
しゃし #T35 斜視 社史 奢侈
しゃしゅ #T35*1 車種
しゃしゅつ #T30 射出
しゃしょう #T35*1 車掌 #T30 捨象
しゃしょく #T30*2 写植
しゃしん #T35*45 写真
しゃじ #T35*4 社寺 #T35*3 謝辞
しゃじく #T35 車軸
しゃじつ #T35 写実
しゃじょう #T35*1 謝状
しゃせい #T30*1 写生 射精
しゃせつ #T35*1 社説
しゃせん #T35*2 斜線 車線
しゃぜつ #T30 謝絶
しゃそう #T35 社葬 車窓
しゃそく #T35*1 車速
しゃぞう #T30*25 写像
しゃたい #T35*2 斜体 #T35*1 車体
しゃたく #T35*1 社宅
しゃだん #T35*1 社団 #T30*1 遮断
しゃだんほうじん #T35 社団法人
しゃち #T35*2 鯱
しゃちゅう #T35*1 車中 #T35 社中
しゃちょう #T35*14 社長
しゃっかん #T35*2 借款
しゃっきん #T30*7 借金
しゃっくり #T30 しゃっくり
しゃっけい #T35 借景
しゃっこう #T35 赤口
しゃったー #T35*2 シャッター
しゃっぽ #T35 シャッポ
しゃつ #T35*5 シャツ
しゃてい #T35*1 射程
しゃてき #T35 射的
しゃどう #T35 車道
しゃな #CN 紗那
しゃない #T35*79 社内 #T35*5 車内
しゃにくさい #T35*1 謝肉祭
しゃにむに #F14 しゃにむに
しゃひ #T35 社費
しゃふ #T30 写譜
しゃふう #T35 社風
しゃふつ #T30 煮沸
しゃへい #T30 遮蔽
しゃへん #T35 斜辺
しゃべ #R5r*13 喋
しゃみ #T35 沙弥 三味
しゃみせん #T35*1 三味線
しゃめい #T35*2 社名
しゃめん #T35 斜面
しゃよう #T35*16 社用
しゃらく #JN 写楽
しゃらくさ #KY しゃらくさ
しゃり #CN 斜里
しゃりき #CN 車力
しゃりょう #T35*3 車両
しゃりん #T35 車輪
しゃれ #KSr*2 洒落 #KSr*1 しゃれ
しゃれい #T30*1 謝礼
しゃれき #T35 社歴
しゃわー #T30*1 シャワー
しゃんそん #T35 シャンソン
しゃんはい #CN 上海
しゃんぱん #T35 シャンパン
しゃんぷー #T30*1 シャンプー
しゃんぺん #T35 シャンペン
しや #T35*5 視野
しやく #T35 試薬
しやくしょ #T35 市役所
しやけ #JNS 初宿
しゅ #JS*180 種 #T35*80 種 #T35*31 主 #T35*2 朱 #KJ*1 取 珠 腫 趣 #PRE 主 #SUC 主 #KJ 守 手 #SUC 手 #KJ 殊 狩 酒 #SUC 酒 #JS 首 #KJ 首 諏
しゅーず #T35*1 シューズ
しゅーべると #JN シューベルト
しゅい #T35*3 首位 #T35*1 趣意 #T35 主意
しゅいん #T35 主因
しゅう #T35*121 週 #T35*19 秀 #KJ*12 終 #T35*11 周 #JS*6 州 #T35*6 州 #T35*5 集 #KJ*2 就 修 秋 蒐 #KJ*1 収 宗 臭 衆 襲 醜 #KJ 鰍 鍬 執 囚 #JS 周 #KJ 愁 拾 #T35 洲 #KJ 繍 習 舟 #SUC 衆 #KJ 讐 蹴 輯 #JS 週 #KJ 酋 酬 #SUC 集 #KJ 摺
しゅうい #T35*2 周囲 #T35 拾遺
しゅういち #JNM 修一 秀一
しゅういつ #T05*1 秀逸 #JN 秀逸
しゅういん #T35 衆院
しゅうう #T35 驟雨
しゅうえいしゃ #KK*1 集英社
しゅうえき #T35*2 収益 #T30 就役
しゅうか #T30 集荷
しゅうかい #T30*16 集会 #T30*1 周回
しゅうかく #T30*1 収穫
しゅうかん #JS*62 週間 #T35*29 週間 #T35*8 習慣 #T35 週刊
しゅうかんし #T35*1 週刊誌
しゅうがく #T30 就学 修学
しゅうき #T35*4 周期 #T35*1 終期 #JS 周忌 #T35 周忌 #JS 周期
しゅうきゅう #T35*5 週給 #T35*1 週休 #T35 蹴球
しゅうきょう #T35*3 宗教
しゅうきょうほうじん #T35 宗教法人
しゅうきょく #T30*1 終局 #T35 終曲 #T30 褶曲
しゅうきん #T30*5 集金
しゅうぎ #T35*1 祝儀
しゅうぎいん #KK 衆議院
しゅうぎょう #T30*2 終業 #T30 就業 修業
しゅうけい #T30*10 集計
しゅうけつ #T30*2 集結 #T30*1 終結
しゅうけん #T35*1 集権
しゅうげき #T30*1 襲撃
しゅうげん #T35 祝言
しゅうこ #JNM 修子
しゅうこう #T30 周航 就航 修好 #T35 醜行
しゅうごう #T30*143 集合
しゅうさい #T35*1 秀才 #T30 収載
しゅうさく #T35*1 秀作
しゅうさん #T35 衆参 #T30 集散
しゅうし #T35*16 修士 #T30*10 終止 #T30*4 終始 #T35*2 収支
しゅうしゅう #T30*13 収集 #T30*5 収拾 #T30*1 蒐集
しゅうしゅく #T30 収縮
しゅうしょう #T35*2 愁傷 #T30 周章
しゅうしょく #T30*29 就職 #T30*4 修飾
しゅうしん #T30*1 就寝 #T35*1 終身 #T30 執心
しゅうじ #T35*1 修辞 #JNM 周二 修司 修二 #T35 習字 衆耳
しゅうじつ #T35*1 終日
しゅうじゅく #T30 習熟
しゅうじょし #T35 終助詞
しゅうじん #T35 囚人
しゅうせい #T30*255 修正 #T35*1 習性 #T30 修整 #T35 終生 #T30 集成
しゅうせき #T30*3 集積
しゅうせん #T30 周旋 終戦
しゅうぜん #T30 修繕
しゅうそ #T35 愁訴 臭素
しゅうそう #JCN 周桑
しゅうそく #T30*3 収束 #T30 終息 集束
しゅうぞう #JNM 修三
しゅうぞく #T35 習俗
しゅうたい #T35 醜態
しゅうたいせい #T30 集大成
しゅうたん #T35*16 終端 #T30*2 愁嘆
しゅうだいすう #T35*1 終代数
しゅうだん #T35*7 集団
しゅうち #T30*19 周知 #T30*1 羞恥 #JCN 周智 #T35 習知
しゅうちく #T30 修築
しゅうちゃく #T30 執着 #T35 終着
しゅうちゅう #T30*30 集中
しゅうちょう #T35 酋長
しゅうてん #T35*2 終点
しゅうでん #T35*1 終電
しゅうと #T35 姑
しゅうとう #T05*1 周到 #JCN 周東
しゅうとく #T30*3 習得 #T30*1 修得 拾得 #T30 収得
しゅうとめ #T35*1 姑
しゅうどう #T30 修道
しゅうにゅう #T35*9 収入
しゅうにん #T30*3 就任
しゅうねん #JS*2 周年 #T35*1 周年 #T35 執念
しゅうのう #T30*3 収納
しゅうは #T35*2 周波 #T35 宗派 愁波
しゅうはい #T30*1 集配
しゅうはすう #T35*11 周波数
しゅうばつ #T05 秀抜
しゅうばん #T35*1 終盤
しゅうふく #T30*2 修復
しゅうぶん #T35*1 秋分 #T35 醜聞
しゅうへき #T35 習癖
しゅうへん #T35*6 周辺
しゅうほう #T35*16 週報 #CN 秋芳
しゅうぼう #T35 衆望 醜貌
しゅうまい #T35 シュウマイ 焼売
しゅうまく #T35 終幕
しゅうまつ #T35*17 週末 #T35 終末
しゅうみん #T30 就眠
しゅうめい #T30 襲名
しゅうもく #T35*1 衆目
しゅうや #T35*1 終夜
しゅうやく #T30*4 集約
しゅうゆう #T30*1 周遊
しゅうよう #T30*3 収容 #T30 収用 修養
しゅうらい #T30 襲来
しゅうらく #T35*1 集落
しゅうり #T30*12 修理
しゅうりょう #T30*255 終了 #T30*3 修了 #T35 収量
しゅうりょうご #T35 終了後
しゅうりん #T35 秋霖
しゅうれい #T35*1 週令 #T35 秋冷
しゅうれん #T30 収斂 修練
しゅうろう #T30 就労
しゅうろく #T30*4 収録 #T30 集録
しゅうろん #T35*7 修論
しゅうわい #T30 収賄
しゅえい #T35*3 守衛
しゅえん #T30*1 主演
しゅかい #T35 主魁
しゅかく #T35*1 主格
しゅかん #T35*10 主幹 #T35*4 主観
しゅがー #T35*1 シュガー
しゅがん #T35*3 主眼
しゅき #T30*1 手記
しゅきゅうは #T35 守旧派
しゅぎ #T35*10 主義 #T35*1 手技
しゅぎょう #T30*5 修行
しゅく #T30*8 宿 #KJ*6 縮 #S5*4 宿 #S5*2 祝 #T30*2 祝 #KJ*1 淑 #KJ 叔 夙 粛
しゅくあ #T35 宿痾
しゅくい #T35 祝意
しゅくえん #T35 祝宴
しゅくが #T30*2 祝賀
しゅくがん #T35 宿願
しゅくぎ #T35 祝儀
しゅくさつ #T30 縮刷
しゅくしゃ #T35*1 宿舎 #T30 縮写
しゅくしゃく #T30*6 縮尺
しゅくしゅ #T35*1 宿主
しゅくしょう #T30*17 縮小
しゅくじ #T35*2 祝辞
しゅくじつ #T35*2 祝日
しゅくじょ #T35 淑女
しゅくず #T35 縮図
しゅくせい #T30 粛正
しゅくぜん #F03 粛然
しゅくたい #T30*2 縮退
しゅくだい #T30*17 宿題
しゅくちょく #T30 宿直
しゅくてき #T35 宿敵
しゅくてん #T35*1 祝典
しゅくでん #T35 祝電
しゅくはい #T35 祝杯
しゅくはく #T30*17 宿泊
しゅくば #T35 宿場
しゅくふく #T30*1 祝福
しゅくほう #T35 祝砲
しゅくぼう #T30 宿望
しゅくめい #T35*2 宿命
しゅくん #T35*1 主君 #T35 殊勲
しゅけい #T35 主計
しゅけん #T35*1 主権
しゅげい #T35 手芸
しゅこう #T35*1 趣向
しゅご #T35*3 主語 #T30 守護
しゅごしん #T35 守護神
しゅさ #T35*1 主査
しゅさい #T30*7 主催 #T30*1 主宰 #T35 主菜
しゅさん #T35 主餐
しゅざい #T30*1 取材
しゅざん #T35 珠算
しゅし #T35*13 趣旨 #T35*5 主旨 #T35*1 種子
しゅしゃ #T30*1 取捨
しゅしゃせんたく #T30*1 取捨選択
しゅしゅ #T35*5 種々
しゅしょう #T35*4 首相 #T05*3 殊勝 #T30 主唱 #T35 主将
しゅしょく #T35 主食
しゅしん #T35 主審
しゅじ #T35*25 主辞 #T35 主事
しゅじい #T35 主治医
しゅじく #T35 主軸
しゅじゅ #T05*7 種々
しゅじゅう #T35*1 主従
しゅじゅつ #T30*2 手術 #T35 主述
しゅじょう #T35 衆生
しゅじん #T35*1 主人
しゅじんこう #T35*2 主人公
しゅせい #T15*2 守勢
しゅせき #T35*1 主席 #T35 首席
しゅぜんじ #CN 修善寺
しゅぞう #T35 酒造
しゅぞく #T35*2 種族
しゅたい #T35*5 主体
しゅだい #T35*2 主題 #T30*1 腫大
しゅだん #T35*84 手段
しゅちにくりん #T35*1 酒池肉林
しゅちゅう #T35*1 手中
しゅちょう #T30*28 主張 #T35*1 主徴 #T35 主潮 首長
しゅっか #T30*74 出荷 #T30 出火
しゅっかん #T30 出棺
しゅっきん #T30*9 出勤 #T30*2 出金
しゅっけつ #T35*4 出欠 #T30 出血
しゅっこ #T30 出庫
しゅっこう #T30*5 出向 #T30 出稿 出航
しゅっこく #T30*1 出国
しゅっさん #T30*2 出産
しゅっし #T30*2 出資
しゅっしゃ #T30*7 出社
しゅっしょ #T35*1 出処 #T30 出所
しゅっしょう #T30 出生
しゅっしん #T35*8 出身
しゅっすい #T30 出水
しゅっせ #T30*13 出世
しゅっせい #T30*1 出生 #T30 出征
しゅっせき #T30*73 出席
しゅっせきしゃ #T35 出席者
しゅっそう #T30 出走
しゅったい #T30 出来
しゅっちょう #T30*57 出張 #T35 出超
しゅってん #T30*8 出展 #T35*4 出典 #T30 出店
しゅっとう #T30 出頭
しゅっぱつ #T30*27 出発
しゅっぱん #T30*18 出版 #T30 出帆
しゅっぴ #T30*2 出費
しゅっぴん #T30*2 出品
しゅっぽん #T30 出奔
しゅつ #T35*37 出
しゅつえん #T30*5 出演
しゅつがん #T30*4 出願
しゅつげん #T30*12 出現
しゅつごく #T30 出獄
しゅつじ #T35 出自
しゅつじょう #T30*1 出場
しゅつじん #T30 出陣
しゅつだい #T30 出題
しゅつどう #T30 出動
しゅつにゅう #T30 出入
しゅつにゅうこく #T30 出入国
しゅつば #T30 出馬
しゅつぼつ #T30*4 出没
しゅつらい #T30 出来
しゅつりょう #T30*2 出猟 #T30 出漁
しゅつりょく #T30*224 出力
しゅつりょくせん #T35*68 出力線
しゅつるい #T30 出塁
しゅと #T35*9 首都
しゅとう #T30 種痘 #JNS 首藤
しゅとく #T30*58 取得
しゅとけん #T35*1 首都圏
しゅとして #F14 主として
しゅどう #T35*13 手動 #T30*2 主導
しゅにん #T35*42 主任
しゅのう #T35*1 首脳
しゅはん #T35 主犯
しゅひ #T35*1 守秘 #T35 種皮
しゅひつ #T35 主筆
しゅひん #T35*1 主賓
しゅび #T30*6 守備 #T35*1 首尾
しゅふ #T35*4 主婦 #T35*1 主夫 #T35 首府
しゅぶん #T35 主文
しゅぷーる #T35*1 シュプール
しゅべつ #T30*22 種別
しゅほう #T35*31 手法 #T35 主峰
しゅぼう #T30*2 首謀
しゅみ #T35*20 趣味
しゅみれーしょん #T30*4 シュミレーション
しゅむ #T35 主務
しゅもく #T35*3 種目
しゅやく #T35*3 主役
しゅよう #T05*12 主要
しゅよく #T35 主翼
しゅら #T35*1 修羅
しゅらん #T35 酒乱
しゅりゅう #T35*5 主流
しゅりょう #T30*4 狩猟 #T35*2 首領
しゅりょく #T35*1 主力
しゅるい #JS*124 種類 #T35*62 種類
しゅろ #T35 シュロ 棕櫚
しゅわ #T35 手話
しゅん #T35*5 旬 #KJ*3 駿 #KJ*2 春 #KJ*1 俊 瞬 竣 舜 #KJ 峻
しゅんいち #JNM 俊一
しゅんえい #T35 俊英
しゅんかしゅうとう #T35 春夏秋冬
しゅんかん #T35*19 瞬間
しゅんき #T35*1 春季 #JN 峻輝
しゅんきょ #T30 峻拒
しゅんこう #T30 竣工
しゅんさい #T35 俊才
しゅんじ #T35*2 瞬時 #JNM 俊二
しゅんじゅう #T35*1 春秋
しゅんじゅん #F00 逡巡
しゅんせつ #T35 春雪 #T30 浚渫
しゅんそく #T35 俊足 #T05 駿足
しゅんたろう #JNM 俊太郎
しゅんだん #T35 春暖
しゅんとう #T35 春闘
しゅんびん #T05 俊敏
しゅんぶん #T35 春分
しゅんべつ #T30 峻別
しゆう #T30*2 私有 #T35 雌雄
しゆうち #T35 私有地
しょ #T35*152 書 #KJ*67 初 #S5*25 処 #T30*25 処 #KJ*10 諸 #T35*9 所 #KJ*1 暑 曙 #CNSUC1*1 署 #KJ 黍 #PRE 初 #SUC 所 #KJ 渚 庶 緒 #SUC 署 #T35 署 #SUC 書 #KJ 薯 藷 #PRE 諸 #KJ 岨
しょー #T35*10 ショー
しょーと #T00*2 ショート
しょあく #T35*1 諸悪
しょい #T35 所為
しょいん #T35*1 書院 #T35 所員 署員
しょう #T35*198 章 #S5*53 称 #T30*53 称 #T35*51 小 #CNSUC1*13 小 #KJ*10 昭 #KJ*6 醤 #S5*5 賞 #T30*5 賞 #ZX*5 生 #T35*4 ショウ #KJ*4 尚 #T35*3 将 #KJ*3 消 #KJ*2 勝 唱 少 庄 招 昌 #S5*2 証 #T30*2 証 #KJ*2 象 憔 翔 #KJ*1 妾 承 抄 晶 湘 焼 照 症 #T35*1 省 #KJ*1 硝 礁 祥 笑 肖 蒋 蕉 衝 生 装 誦 #KJ 鍬 錆 篠 傷 償 #JS 勝 #KJ 匠 #JS 升 #KJ 召 哨 商 #SUC 商 #KJ 嘗 奨 娼 宵 #PRE 小 #KJ 床 廠 彰 掌 捷 昇 松 梢 樟 樵 沼 渉 焦 #SUC 症 #PRE 省 #SUC 省 #JS 章 #KJ 粧 紹 菖 裳 訟 #SUC 証 #KJ 詔 詳 #SUC 賞 #KJ 鉦 鍾 鐘 障 鞘 姓 性 #SUC 性 #T35 性 #KJ 政 正 精 相 #SUC 相 #KJ 蛸 憧 橡
しょうあく #T30 掌握
しょうあつ #T30*1 昇圧
しょうい #T35 小異
しょういち #JNM 庄一 昇一 昭一 章一 正一
しょういちろう #JNM 昌一郎 正一郎
しょういん #T35 勝因
しょうえん #T35*1 招宴 #T35 小宴 #T30 荘園
しょうおう #JCN 勝央 #T30 照応
しょうおん #T30 消音
しょうか #T30*9 消化 #T30*2 消火
しょうかい #T30*64 紹介 #T30*10 照会 #T35*8 商会 #T30 詳解
しょうかく #T30*2 昇格
しょうかどう #KK*1 松香堂
しょうかわ #JCN 荘川
しょうかん #T30*2 召喚 #T35*2 小寒 #T30 償還 #T35 将官
しょうが #T35 生姜
しょうがい #T35*62 障害 #T35*1 渉外 生涯 #T30 傷害 #T35 障碍
しょうがく #T35*1 奨学 小学 #T05 小額 少額
しょうがくかん #KK*1 小学館
しょうがくせい #T35*3 小学生 #T35*1 奨学生
しょうがっこう #T35*2 小学校
しょうがつ #T35*7 正月
しょうがな #KY*25 しょうがな
しょうがのう #KYU*2 しょうがのう
しょうがわ #JCN 庄川
しょうき #T35 正気
しょうきち #JNM 昭吉 正吉
しょうきぼ #T05*6 小規模
しょうきゃく #T30 償却 消却 焼却
しょうきゅう #T30 昇級 昇給
しょうきょ #T30*39 消去
しょうきょく #T35*1 消極
しょうきん #T35*1 賞金 #T35 償金
しょうぎ #T35*1 将棋
しょうぎょう #T35*2 商業
しょうぐん #T35*4 将軍
しょうけい #T35*9 小計 #T35 象形 #T30 憧憬
しょうけつ #T30 焼結
しょうけん #T35*3 証券 #T35*2 證券
しょうげき #T35*1 衝撃
しょうげきは #T35 衝撃波
しょうげん #T30*1 証言
しょうこ #T35*13 証拠 #JNM 祥子 聖子 翔子
しょうこう #T30*2 昇降 #T35 商工 将校 小康 #T30 消光 焼香 #T35 症候
しょうこうぐん #T35*1 症候群
しょうこうねつ #T35 猩紅熱
しょうこく #T35*1 小国
しょうこん #T35 商魂
しょうご #T35*1 正午 #JNM 昭吾 省吾 正吾
しょうごいん #CN 聖護院
しょうごう #T30*4 照合 #T35*1 称号 #T35 商号
しょうさ #T30*1 照査 #T35 小差 少佐
しょうさい #T05*135 詳細 #T35 商才
しょうさん #T35*1 勝算 硝酸 #T30*1 称賛 #T30 消散 賞賛
しょうざぶろう #JNM 正三郎
しょうし #T30 焼死 #T15 笑止 #T35 証紙
しょうしつ #T30*1 消失 焼失
しょうしゃ #T35*1 商社 #T30*1 照射 #T35 勝者 小社 #T30 抄写 #T05 瀟洒
しょうしゅう #T30*1 招集 #T30 召集 消臭
しょうしゅくしゃく #T15 小縮尺
しょうしゅつ #T30 抄出
しょうしょ #T35*1 小暑 #T35 証書
しょうしょう #T35*38 少々 #T35 少将
しょうしょく #T35 小職 #T05 小食 少食
しょうしん #T30*2 昇進 #T35*1 正真 #T30 傷心 #T05 小心 #T30 焼身 焦心
しょうじ #KS*61 生じ #T35*2 商事 #T35*1 障子 #JNM 小路 尚司 #JN 庄司 #JNM 昌二 昭司 昭次 昭治 昭二 祥治 章司 章二 正次 正治 正二 #JN 東海林
しょうじき #T05*13 正直
しょうじゃ #T35*1 精舎
しょうじゃく #CN 正雀
しょうじゅう #T35*1 小銃
しょうじゅつ #T30*1 詳述
しょうじゅん #T05*3 昇順 #T30*1 照準
しょうじょ #T35*16 少女
しょうじょう #T35*46 症状 #T35*1 賞状
しょうじろう #JNM 章治郎 正次郎
しょうじん #T30*2 精進
しょうすい #T30*2 憔悴
しょうすう #T05*19 小数 #T05*13 少数
しょうず #CN 小豆
しょうせい #T35*1 小生 #T30*1 招請
しょうせつ #T35*10 小説 #T35*1 小雪 #T30*1 詳説 #JS 小節 #T35 小節
しょうせつか #T35 小説家
しょうせん #T35*4 商船
しょうそ #T30 勝訴
しょうそう #T05*4 尚早 #T30*1 焦燥
しょうそういん #KK 正倉院
しょうそく #T35*1 消息
しょうぞう #T35*1 肖像 #JNM 尚三 彰三 昭三 省三 正三
しょうぞく #T35*1 装束
しょうたい #T30*8 招待 #T35*2 正体 #T35*1 小体 小隊
しょうたろう #JNM 正太郎
しょうたん #T30 賞嘆
しょうだく #T30*3 承諾
しょうだん #T30*1 商談
しょうち #T30*8 承知 #T30 招致
しょうちく #KK*2 松竹
しょうちゅう #T35 掌中 焼酎
しょうちょ #T35 小著
しょうちょう #T35*2 省庁 #T30*2 象徴 #T35 小腸 消長
しょうちん #T30 消沈
しょうてきほう #T35*1 小滴法
しょうてん #T35*2 商店 焦点 #T30*1 昇天 #T35 衝天
しょうとう #T30*1 消灯 #CN 松涛
しょうとくたいし #JNS 聖徳太子
しょうとつ #T30*8 衝突
しょうど #T35 焦土 照度
しょうどう #T35*1 衝動
しょうどく #T30 消毒
しょうない #JCN 庄内 #T35 渉内
しょうなごん #T35 少納言
しょうなり #T35*1 ＜
しょうなりいこーる #T35*3 ≦
しょうなん #CN 沼南 湘南
しょうに #T35 小児
しょうにゅうどう #T35*1 鍾乳洞
しょうにん #T30*41 承認 #T35*6 証人 #T35*2 商人 #T30*1 昇任 #T35 小人
しょうね #T35 性根
しょうねつ #T35 焦熱
しょうねん #T35*2 少年
しょうねんば #T35*1 正念場
しょうのう #T35 小脳 樟脳 #T30 笑納
しょうは #T30 小破
しょうはい #T35*1 勝敗 #T35 賞杯
しょうはち #JNM 正八
しょうばい #T30*9 商売
しょうばつ #T35 賞罰
しょうばら #JCN 庄原
しょうばん #T30 相伴
しょうひ #T30*19 消費
しょうひしゃ #T35*2 消費者
しょうひぜい #T35*14 消費税
しょうひょう #T35*14 商標
しょうひん #T35*15 商品 #T35*1 賞品
しょうび #T35*3 焦眉 #T30 賞美
しょうびょう #T35 傷病
しょうふ #T35*3 娼婦
しょうふく #T35*1 妾腹 #T30 承伏
しょうふだ #T35 正札
しょうぶ #T30*7 勝負 #JCN 菖蒲
しょうぶん #JNM 昌文 #T35 性分
しょうへい #T30*2 招聘 #T35 将兵 #JNM 章平 正平
しょうへき #T35*2 障壁
しょうへん #T35 掌篇 掌編
しょうべん #T35 小便
しょうほう #T35*1 商法
しょうほん #T35 抄本
しょうぼう #T35*1 消防 #T30 焼亡
しょうぼうしょ #T35*1 消防署
しょうまん #T35*1 小満
しょうみ #T30 賞味 #T35 正味
しょうめい #T30*80 証明 #T30*2 照明 #T35*1 正銘
しょうめつ #T30*8 消滅
しょうめん #T35*3 正面
しょうもう #T30*6 消耗
しょうもな #KY しょうもな
しょうものう #KYU しょうものう
しょうもん #T35*1 証文
しょうや #T35*1 庄屋
しょうやく #T30 抄訳 #T35 生薬
しょうゆ #T35*2 醤油
しょうよ #T35 賞与
しょうよう #T35*2 商用 #T35*1 小葉
しょうらい #T35*60 将来 #T30 招来
しょうらん #T30 照覧 笑覧
しょうり #T30*3 勝利
しょうりき #JNS 正力
しょうりゃく #T30*63 省略
しょうりょ #T30*1 焦慮
しょうりょう #T05*2 小量 少量
しょうりょく #T35 省力
しょうれい #T30*2 奨励 #T35 省令
しょうろく #T30 抄録
しょうろん #T30 詳論
しょうわ #KK*27 昭和 #T30 唱和 #CN 庄和 昭和
しょうわくせい #T35 小惑星
しょうわる #T18 性悪
しょえん #T30 初演
しょか #T35*2 初夏 #T35 書架
しょかい #T35*1 初回 #T35 初会
しょかつ #T30 所轄
しょかん #T35*5 所感 #T35*1 書簡 #T30 所管
しょが #T35 書画
しょがい #T35*2 所外
しょき #T35*95 初期 #T35*58 書記 #T35 所期
しょきか #T30*149 初期化
しょきち #T35*10 初期値
しょきちょう #T35*1 書記長
しょきゅう #T35*2 初級
しょぎ #T35 諸儀
しょく #KJ*6 色 #KJ*5 飾 #JS*5 食 #KJ*1 殖 #T35*1 職 #KJ*1 蝕 #KJ 喰 #T30 嘱 #KJ 埴 拭 植 燭 織 #SUC 職 #JS 色 #SUC 色 #KJ 触 #S5 食 #SUC 食 #T30 食 属
しょくあん #T35 職安
しょくい #T35 職位
しょくいん #T35*13 職員
しょくえん #T35 食塩
しょくぎょう #T35*4 職業
しょくご #T35*1 食後
しょくさつ #T35*1 喰殺
しょくし #T35*1 食指
しょくしゅ #T35*1 職種 #T35 触手
しょくしょう #T30*1 食傷 #T35 職掌
しょくしん #T30 触診
しょくじ #T30*28 食事 #T30 植字
しょくじゅ #T30*1 植樹
しょくじゅう #T35 職住
しょくじょ #JN 織女
しょくせい #T35 植生 職制
しょくせいかつ #T35*3 食生活
しょくせき #T35 職責
しょくぜん #T35*1 食前 #T35 食膳
しょくたく #T30*1 嘱託 #T35*1 食卓
しょくだい #T35 燭台
しょくち #T35 諸口
しょくちゅうどく #T30 食中毒
しょくつう #T35 食通
しょくどう #T35*10 食堂 #T35 食道
しょくにん #T35 職人
しょくのう #T35 職能
しょくはつ #T30 触発
しょくば #T35*7 職場
しょくばい #T35 触媒
しょくひ #T35*3 食費
しょくひん #T35*2 食品
しょくぶつ #T35*3 植物
しょくぶん #T35 職分
しょくほう #T30*3 触法
しょくぼう #T30 嘱望
しょくみんち #T35*1 植民地
しょくむ #T35*1 職務
しょくもく #T30 属目
しょくもつ #T35*1 食物
しょくよう #T35 食用
しょくよく #T35*2 食欲
しょくりょう #T35*4 食糧 #T35 食料
しょくりょうなん #T35*1 食糧難
しょくりょうひん #T35 食料品
しょくりん #T30 植林
しょくれき #T35 職歴
しょくん #T35*1 諸君
しょけい #T30*1 処刑 #T35 諸兄
しょけん #T30*3 書見 #T35 初見 所見
しょこ #T35*3 書庫
しょこく #T35*1 諸国
しょこん #T35 初婚
しょさい #T35 所載 書斎
しょさん #T35 所産
しょさんべつ #CN 初山別
しょざい #T35*1 所在
しょざいち #T35*1 所在地
しょざん #T35*1 初産
しょし #T35*3 諸氏 #T35 初志
しょしき #T35*31 書式 #T35 諸式
しょしゃ #T30 書写
しょしゅう #T35*2 所収 #T35 初秋
しょしゅつ #T30*2 初出
しょしゅん #T35 初春
しょしょ #T35*5 諸処 #T35*1 処暑
しょしょう #T35 書証
しょしん #T35*8 初心 #T35*2 初診 #T35 初審 所信 書信
しょしんしゃ #T35*10 初心者
しょじ #T30 所持 #T35 諸事
しょじゅん #T35*1 初旬
しょじょ #T35 処女
しょじょう #T35 書状
しょせい #T35*1 初生 #T35 処世 書生
しょせき #T35*10 書籍
しょせつ #T35 諸説
しょせん #F14*5 所詮 #T35 緒戦
しょぜい #T35 諸税
しょそう #T35 諸相
しょそく #T35 初速
しょぞう #T30*1 所蔵
しょぞく #T30*32 所属
しょぞん #T35*2 所存
しょたい #T35*13 書体 #T35*1 所帯
しょたいめん #T35*2 初対面
しょだい #T35 初代
しょだな #T35 書棚
しょだん #T35 初段
しょち #T30*18 処置
しょちゅう #T35*4 暑中 #T35 書中
しょちょう #T35*3 所長 #T35 初潮 署長
しょっかく #T35 触覚
しょっかん #T35 触感 食間
しょっき #T35*6 食器
しょっきり #T35 初っ切り
しょっく #T05*9 ショック
しょっくし #T30 ショック死
しょっけん #T35 職権 食券
しょっこう #T35 職工
しょっちゅう #F14*17 しょっちゅう
しょっつる #T35 しょっつる
しょっと #T30*9 ショット
しょっぱ #KYmi しょっぱ
しょっぴ #K5 しょっ引
しょっぴんぐ #T30*1 ショッピング
しょっぷ #T35*28 ショップ
しょて #T35*1 初手
しょてい #T35*4 所定
しょてん #T35*7 書店
しょとう #T15*2 初等 #T35*1 初冬 初頭 諸島
しょとく #T35*3 所得
しょどう #T35 書道
しょない #T35*2 所内
しょにち #T35*3 初日
しょにゅう #T35*1 初乳
しょにんきゅう #T35 初任給
しょねんど #T35*4 初年度
しょはん #T35*27 諸般 #T35*6 初版 #T35 初犯
しょばつ #T30 処罰
しょぱん #JN ショパン
しょひょう #T35*2 諸表 #T30*1 書評
しょぶん #T30*5 処分
しょほ #T35*3 初歩
しょほう #T30 処方 #T35 諸方
しょほうせん #T35 処方箋
しょぼう #T35*1 書房
しょみん #T35*3 庶民
しょむ #T35*2 庶務
しょめい #T30*15 署名 #T35*4 書名
しょめん #T35*1 書面
しょもう #T30 所望
しょもつ #T35*2 書物
しょゆう #T30*8 所有
しょゆうち #T35 所有地
しょよう #T35*5 所用 #T35*3 所要
しょり #T30*255 処理
しょるい #T35*24 書類
しょろう #T35 初老
しょんべん #T35 しょんべん 小便
しょんぼり #T30 しょんぼり
しよう #T30*255 使用 #T35*188 仕様 #T30*12 私用 #T30*4 試用 #T35*1 脂溶 #T30 止揚
しようしゃ #T35 使用者
しようしょ #T35*219 仕様書
しよく #T35 私欲
しら #S5*4 知ら #M5*1 白
しらい #JCN 白井
しらいし #JCN 白石
しらおい #CN 白老
しらおか #JCN 白岡
しらかば #T35*1 白樺
しらかばこ #CN 白樺湖
しらかわ #JCN 白河 白川
しらが #T35*1 白髪
しらき #JNS 白木
しらくも #T35 白癬
しらけ #KSr*1 白け
しらこ #JCN 白子
しらさぎ #T35 白鷺
しらさわ #JCN 白沢
しらじらし #KY 白々し
しらじらしゅう #KYU 白々しゅう
しらたか #JCN 白鷹
しらたき #JCN 白滝
しらたま #T35 白玉
しらたまこ #T35 白玉粉
しらと #JNS 白戸
しらとり #JCN 白鳥
しらぬか #CN 白糠
しらぬひ #CN 不知火
しらね #CN 白根
しらは #T35 白羽
しらはま #CN 白浜
しらぶる #T35*1 シラブル
しらべ #KSr*245 調べ
しらみね #JNS 白峰
しり #T35*9 尻 #T35 私利
しりーず #T35*42 シリーズ
しりあ #W5r*1 知り合
しりあい #T35*7 知合い
しりあす #T05*1 シリアス
しりあらいず #T30*1 シリアライズ
しりある #T05*17 シリアル
しりうち #JCN 知内
しりきしない #CN 尻岸内
しりこん #T35*3 シリコン
しりこんばれー #KK*1 シリコンバレー
しりごみ #T30*1 後込み 尻込み
しりぞ #K5*1 退
しりぞけ #KS*1 退け #KS 斥け
しりつ #T35*7 市立 #T35*2 私立
しりつぼみ #T15 尻つぼみ
しりぬ #K5 知り抜
しりぬぐい #T30 尻拭い
しりべし #CN 後志
しりめ #T35 後目
しりめつれつ #T05*1 支離滅裂
しりゅう #T35 支流
しりょ #T30*1 思慮
しりょう #T35*155 資料 #T35*2 死霊 #T35*1 飼料 #T35 史料 #T30 思量
しりょく #T35*1 視力 #T35 死力 資力
しる #S5*10 記 #T35*3 汁
しるえっと #T35 シルエット
しるく #T35 シルク
しるし #T35*26 印
しるばー #T05*1 シルバー
しるべ #T35*1 標
しれい #T30*3 指令 #T30 司令
しれとこ #CN 知床
しれわた #R5 知れ渡
しれん #T35 試練
しろ #T35*24 白 #KY*22 白 #KJ*12 代 #T35*9 城 #SX しろ
しろい #JCN 白井
しろいし #JCN 白石
しろう #KYU*2 白う #JNM 司郎 史郎 四郎 #T35 屍蝋 #JNM 志郎
しろうと #T35*4 素人
しろうま #JCN 白馬
しろかわ #JCN 城川
しろがね #JCN 白金
しろくじちゅう #T35 四六時中
しろくま #T35*1 シロクマ
しろくろ #T35*4 白黒
しろざとう #T35 白砂糖
しろじ #T35*1 白地
しろた #JNS 白田
しろっぽ #KY 白っぽ
しろとり #JCN 白鳥
しろね #JCN 白根
しろばい #T35 白バイ
しろぼし #T35 白星
しろみ #T35*1 白身
しろむく #T35*2 白無垢
しろもの #T35*2 代物
しろやま #T35*1 城山 #JCN 城山
しろん #T35*1 史論 #T35 私論
しわ #T35 しわ #JCN 紫波 #T35 皺
しわく #T30 思惑
しわけ #T30*9 仕訳 #T30*1 仕分け #KSr 仕分け
しわざ #T35*1 仕業
しわす #T35*1 師走
しわひめ #CN 志波姫
しわよせ #T30 しわ寄せ
しん #T35*165 信 #ZX*163 信 #T35*120 新 #KJ*75 進 #T35*34 真 #T35*16 心 #JS*11 進 #KJ*6 秦 #KJ*5 神 #T35*4 伸 #KJ*4 審 #KJ*3 晋 #KJ*2 深 #KJ*1 寝 慎 森 申 #T05*1 真 #T35*1 芯 #KJ*1 槙 沁 眞 #JNM 伸 #KJ 侵 唇 娠 振 #JN 新 #PRE 新 #KJ 榛 浸 疹 紳 臣 薪 親 診 身 辛 針 震 辰 津 賑 矧
しんあい #T05*2 親愛
しんあさひ #CN 新旭
しんあん #T35 新案
しんい #T35*6 深意 #T35*2 真意
しんいち #JNM 伸一 信一 慎一 新一 #JCN 新市 #JNM 真一 進一
しんいちろう #JNM 慎一郎 新一郎
しんいり #T30 新入り
しんうおのめ #CN 新魚目
しんうち #T35*1 真打ち
しんえい #T05 新鋭 #T35 親衛
しんえん #T05*1 深遠
しんおう #T35 震央
しんおおさか #CN 新大阪
しんおん #T35 心音
しんか #T30*4 進化 #T30*1 深化 #T35 真価 臣下
しんかい #T35 深海
しんかいち #T35 新開地
しんかく #T35*1 真核
しんかぶ #T35 新株
しんかわ #JCN 新川
しんかん #T35*4 新刊 #T35 新館
しんかんせん #T35*6 新幹線
しんがー #T35 シンガー
しんがい #T30*2 侵害 #T05*1 心外
しんがお #T35 新顔
しんがく #T30*1 進学 #T35 神学
しんがくぎほう #T35*3 信学技法
しんがくろん #T35*1 信学論
しんがた #T35*2 新型
しんがぽーる #CN シンガポール
しんき #T05*99 新規 #T05*1 新奇 #T35 心機
しんきち #JNM 信吉 新吉 真吉
しんきゅう #T35*4 新旧 #T30*1 進級
しんきょ #T35*3 新居
しんきょう #T35*2 心境
しんきょく #T35 新曲
しんきろう #T35*1 蜃気楼
しんきろく #T35*1 新記録
しんきん #T35*1 真菌 #T30*1 親近 #T35 信金 心筋
しんぎ #T35*4 真偽 #T30*1 審議
しんぎかい #T35*1 審議会
しんぎじゅつ #T35*1 新技術
しんぎたい #T35 心技体
しんぎん #T30 呻吟
しんく #T35*1 真紅 #T30 辛苦
しんくう #T15 真空
しんくうかん #T35*1 真空管
しんぐ #T35 寝具
しんぐう #JCN 新宮 #T35 新宮
しんぐる #T35*14 シングル
しんぐん #T30 進軍
しんけい #T35*7 神経
しんけいしつ #T35*1 神経質
しんけいつう #T35 神経痛
しんけん #T05*9 真剣 #T35 親権
しんげき #T30 進撃
しんげん #JNM 信玄
しんげんち #T35 震源地
しんこう #T30*31 進行 #T30*8 信仰 #T30*2 振興 #T35*1 新興 #T30 侵攻 #T35 親交 親好
しんこきゅう #T30 深呼吸
しんこく #T30*8 申告 #T05*4 深刻 #T30 親告
しんこっちょう #T35*1 真骨頂
しんこやす #CN 新子安
しんこん #T35*5 新婚
しんご #JNM 紳悟
しんごう #T35*62 信号 #JCN 新郷 神郷
しんさ #T30*4 審査
しんさい #T35*4 震災
しんさく #T30*1 新作
しんさつ #T30*3 診察
しんさん #T35 辛酸
しんざん #T35*3 新参
しんし #T35*1 紳士
しんしき #T05 新式
しんしつ #T35*1 寝室 心室
しんしのつ #CN 新篠津
しんしゃ #T30*1 新車 #T30 深謝
しんしゃく #T30 斟酌
しんしゅ #T35*1 新種 #T35 新酒 進取
しんしゅう #CN 信州
しんしゅく #T30*18 伸縮
しんしゅつ #T30*2 進出 #T30*1 滲出 #T30 侵出 浸出
しんしゅん #T35*1 新春
しんしょ #T35*1 新書
しんしょう #T30*1 辛勝 #T35 心証
しんしょうしゃ #T35 身障者
しんしょうひん #T35 新商品
しんしょく #T30*1 侵食 #T30 侵蝕 寝食 浸食 浸蝕
しんしろ #JCN 新城
しんしん #T35*8 心身 #F00*2 津津 #T35*1 新進
しんじ #JCN 宍道 #KS 信じ #JNM 信次 信治 信二 新治 真司 真治
しんじけーと #T35 シンジケート
しんじつ #T35*4 真実
しんじゃ #T35*3 信者
しんじゅ #T35*2 真珠
しんじゅう #T30*1 心中
しんじゅく #JCN 新宿
しんじょう #T35*2 信条 #T35*1 心情 身上 #JCN 新庄 #T30 進上
しんじろう #JNM 新次郎
しんじん #T35*42 新人 #T30*1 信心
しんすい #T30*8 浸水 #T30 心酔 進水
しんすいせい #T05*1 親水性
しんすう #JS*10 進数
しんずい #T35*1 真髄 #T35 心髄 神髄
しんせい #T30*31 申請 #T35*1 新生 #T05*1 神聖 #T35 新制 新星 真性 #CN 真正 #T05 真正
しんせいじ #T35 新生児
しんせいひん #T35*2 新製品
しんせき #T35*3 親戚
しんせさいざ #T35 シンセサイザ
しんせつ #T05*16 親切 #T30*10 新設 #T35 新説 新雪
しんせん #T05*8 新鮮 #T35*1 振戦 芯線 #T35 神仙 #JCN 神泉 #T30 震顫
しんぜん #T35 神前 親善
しんそう #T35*2 深層 真相 #T35 深窓
しんそこ #T35 心底
しんそつ #T35 新卒
しんぞう #T35*4 心臓
しんぞく #T35*1 親族
しんたい #T35*1 身体 #T30*1 進退 #T35 神体
しんたいせい #T35*2 新体制
しんたいそう #T35 新体操
しんたく #T30*6 信託 #JNS 新宅 #T35 神託
しんたくぎんこう #T35 信託銀行
しんたっくす #T35*14 シンタックス
しんたろう #JNM 信太郎 新太郎
しんだい #T35*1 寝台
しんだん #T30*31 診断
しんち #CN 新地 #T35 新地
しんちく #T30 新築
しんちゅう #T35*1 心中 #T35 真鍮 #T30 進駐
しんちょう #T30*13 伸長 #T05*8 慎重 #T35*8 身長 #KK*3 新潮 #T30*2 新調 #T05 深長
しんちょく #T30*68 進捗
しんちんたいしゃ #T30 新陳代謝
しんつう #T30 心痛
しんてい #T30*1 進呈
しんてき #T05*4 心的
しんてん #T30*5 進展 #T35*1 親展
しんでれら #JN シンデレラ
しんでん #T35*1 神殿 #JCN 新田
しんでんず #T35 心電図
しんとう #T30*3 浸透 #T35 心頭 新党 #JCN 榛東 #JNS 真藤 #JS 親等 #T35 親等
しんとく #JCN 新得
しんとつかわ #CN 新十津川
しんとね #CN 新利根
しんとみ #CN 新富
しんど #T35*2 進度 #KY しんど #T35 震度
しんどう #T30*1 振動 #JNS 真藤 #T35 神童 #JNS 進藤 #T30 震動
しんなー #T35 シンナー
しんなんよう #CN 新南陽
しんにちは #T35 親日派
しんにってつ #KK*1 新日鉄
しんにほん #KK*4 新日本
しんにゅう #T30*5 侵入 #T35*2 新入 #T30*1 浸入 進入
しんにゅうがく #T35 新入学
しんにゅうせい #T35 新入生
しんにん #T30 信任 新任
しんねん #T35*21 新年 #T35*3 信念
しんねんかい #T35*7 新年会
しんのう #T35*1 心嚢 #T35 親王
しんはつばい #T30 新発売
しんばし #T35*8 新橋
しんばん #T35*1 新盤
しんぱ #T35*1 新派
しんぱい #T00*64 心配
しんぱん #T30*2 審判 #T30*1 侵犯
しんび #T35*1 審美
しんぴ #T05*1 神秘 #T35 真否
しんぴょう #T30 信憑
しんぴん #T05*1 新品
しんふぜん #T35 心不全
しんぶ #T35*2 深部
しんぶつ #T35*1 神仏
しんぶん #T35*31 新聞
しんぷ #T35*7 新譜 #T35*3 新婦 #T35 神父
しんぷう #T35 新風
しんぷく #T35*1 振幅
しんぷる #T05*10 シンプル
しんぺい #JNM 新平
しんぺん #T35 身辺
しんぼ #JCN 新保
しんぼう #T35*4 信望 #T30*2 辛抱 #T35 心房 心棒
しんぼく #T30*6 親睦 #T35 神木
しんぼりっく #T05*21 シンボリック
しんぼりっくす #KK*2 シンボリックス
しんぼる #T35*23 シンボル
しんぽ #T30*14 進歩
しんぽう #T30 信奉 #T35 新法
しんぽじうむ #T35*24 シンポジウム
しんぽん #T35 新本
しんまい #T35 新米
しんまち #JCN 新町
しんみ #T05 親身
しんみつ #T05 親密
しんみなと #CN 新湊
しんみり #T00 しんみり
しんめ #T35 新芽
しんめい #T35*1 身命
しんもつ #T35 進物
しんもん #T30 審問
しんや #T35*8 深夜 #JNM 伸也 信也 真哉 進也
しんゆう #T35 親友
しんよう #T30*9 信用
しんよこはま #CN 新横浜
しんよしとみ #CN 新吉富
しんらい #T30*50 信頼
しんらつ #T05*1 辛辣
しんらばんしょう #T35*1 森羅万象
しんらん #JNS 親鸞
しんり #T30*2 審理 #T35*2 心理 #T35*1 真理
しんりゃく #T30 侵略
しんりょ #T35 深慮 神慮
しんりょう #T30*1 診療
しんりょく #T35 新緑
しんりん #T35*1 森林
しんるい #T35*1 親類
しんれい #T35 心霊
しんろ #T35*4 進路
しんろう #T35*2 新郎 #T30*1 心労
しんわ #T30*3 親和 #T35*1 神話 #JCN 新和
じ #SUC*255 時 #T35*190 字 #KJ*64 次 #KJ*32 自 #KJ*25 事 #KJ*16 寺 #KJ*10 治 #S5*6 持 #SX*6 持 #KJ*6 辞 #KJ*4 磁 #CNSUC1*3 路 #KJ*2 慈 #T35*2 時 #KJ*1 時 滋 #T35*1 痔 #KJ*1 示 而 耳 #S5*1 辞 #SX*1 辞 #T35*1 地 #KJ*1 峙 #KJ 餌 似 侍 #JS 児 #KJ 児 #JS 字 時 次 #PRE 次 #SUC 次 #KJ 爾 璽 蒔 迩 路
じーた #T35*1 Ζ ζ
じあい #T35 慈愛 #T30 自愛
じい #T35*2 爺 #T35*1 じい #T30*1 自慰 #T35*1 辞意 #KJ*1 爺
じいしき #T35 自意識
じいん #T35*2 寺院
じぇっと #T35*5 ジェット
じぇっとき #T35*1 ジェット機
じぇねれーしょん #T30*2 ジェネレーション
じぇねれーた #T35*4 ジェネレータ
じぇねれーと #T30*1 ジェネレート
じぇらしー #T35*1 ジェラシー
じえい #T30*1 自営 #T30 自衛
じえいたい #KK*2 自衛隊
じえん #T30*1 自演
じか #T35*4 じか #T35*2 時下 #T35 時価 自家
じかい #T35*80 次回
じかく #T30*2 自覚 #T35 字画
じかけ #SUC*1 仕掛け
じかせい #T35*1 自家製
じかつ #T30 自活
じかよう #T35*1 自家用
じかようしゃ #T35*1 自家用車
じかん #JS*255 時間 #T35*75 時間 #T35 次官
じかんがい #T35*7 時間外
じかんぎれ #T35*1 時間切れ
じかんたい #T35*4 時間帯
じかんわり #T35 時間割
じが #T35*3 自我 #T35*1 自画
じがく #T30 自学
じがぞう #T35 自画像
じがため #T30 地固め
じき #T35*92 時期 #T35*48 磁気 #T35*34 次期 #T35*2 時機 #KJ 喰 #T35 磁器 #T30 自棄
じきさん #T35 直参
じきしあん #KK*1 直指庵
じきじき #T15 直々
じきそ #T30 直訴
じきひつ #T35 直筆
じきゅう #T35*2 時給 #T30*1 自給 #T30 持久
じきょ #T30 辞去
じきょう #T30 自供
じきょく #T35 時局 磁極
じぎ #T35*8 字義 #T35 辞儀
じぎゃく #T30*1 自虐
じぎょう #T35*59 事業
じぎょうしょ #T35*1 事業所
じぎょうぬし #T35 事業主
じぎょうぶ #T35 事業部
じく #T35*64 軸 #T35*3 字句 #KJ 竺
じくさく #T35*1 軸索
じくじ #F10 忸怩
じぐざぐ #T00*1 ジグザグ
じぐそー #T35*9 ジグソー
じけい #T35*2 次兄
じけいれつ #T35*1 時系列
じけつ #T30 自決
じけん #T35*7 事件
じげん #T35*36 次元 #JS*1 時限 #T35 時限 #JS 次元
じこ #T35*107 事故 #T35*35 自己
じこう #T35*95 事項 #T35*1 時候 時効 次項
じこく #T35*36 時刻 #T35*2 自国
じご #T35*5 事後
じごう #T35*1 次号
じごうじとく #T05*1 自業自得
じごく #T35*3 地獄
じさ #T35*4 時差
じさく #T30*3 自作
じさげ #T30 字下げ
じさつ #T30*3 自殺
じさん #T30*2 持参 #T30*1 自賛
じざい #T05*1 自在
じざけ #T35*1 地酒
じし #T35 次子
じしつ #T35*1 自室 #T30 自失
じしゃ #T35*3 自社
じしゃく #T35*1 磁石
じしゅ #T35*7 自主 #T35 字種 #T30 自首
じしゅう #T30*2 自習
じしゅく #T30 自粛
じしょ #T35*255 辞書 #T35*6 地所 #T30*1 自署
じしょう #T35*57 事象 #T30*1 自称 #T35*1 自象 #T35 次章
じしょく #T30*1 辞職
じしん #T35*118 自身 #T35*31 自信 #T35*12 地震 #T35 磁針
じじ #T35*3 時事 #T35*2 爺
じじこくこく #T35*1 時々刻々
じじつ #T35*30 事実
じじつじょう #T35*3 事実上
じじゅう #T35 侍従
じじゅん #T35 耳順
じじょ #T35 次女 自序
じじょう #T35*33 事情 #T30 自乗
じじょでん #T35 自叙伝
じすい #T30*2 自炊
じすう #T35*11 次数
じすぎ #JS*1 時すぎ
じせい #T30*6 自制 #T35*2 時制 #T35*1 時世 #T35 時勢
じせき #T35 次席 自席 #T30 自責
じせつ #T35*4 時節 #T35 自説
じせん #T30*1 自選 #T30 自薦
じぜん #T35*28 事前 #T15*1 次善 #T35 慈善
じそう #T35*8 時相
じそく #T35*2 時速
じそん #T30 自存 #T35 自尊
じぞう #T35 地蔵
じぞく #T30*1 持続
じた #T35*1 自他
じたい #T35*73 自体 #T35*7 事態 #T35*3 字体 #T30*2 辞退
じたく #T35*40 自宅
じたばた #F12 じたばた
じだい #T35*38 時代 #T35*1 次代 #T35 地代
じだいげき #T35 時代劇
じだらく #T05 自堕落
じだん #T35*1 次段 #T35 示談
じち #T35*1 自治
じちかい #T35*1 自治会
じちしょう #KK 自治省
じちたい #T35*1 自治体
じちょう #T35*1 次長 #T30*1 自重 #T30 自嘲
じっ #NN*5 十 #PRE 実
じっか #T35*5 実家
じっかん #T30*8 実感
じっき #T35*7 実機
じっきょう #T35*1 実況
じっくり #F04*3 じっくり
じっけい #T35 実刑 実兄
じっけん #T30*105 実験 #T30 実検 #T35 実権
じっこう #T30*255 実行 #T35*3 実効
じっさい #T35*255 実際
じっさく #T30 実作
じっし #T30*40 実施 #T30*2 実視 #T35 実子
じっしつ #T35*11 実質
じっしゃ #T30*1 実写
じっしゅう #T30*10 実習 #T35 実収
じっしょう #T30*3 実証
じっすう #T35*4 実数
じっせき #T35*26 実績
じっせん #T30*5 実践 #T35*4 実線 #T30*1 実戦
じっそう #T30*75 実装
じっそく #T30*1 実測
じったい #T35*123 実態 #T35*44 実体
じっち #T35 実地
じっちょく #T05 実直
じっと #F14*3 じっと
じっぴ #T35*4 実費
じっぷ #T35 実父
じつ #T35*241 実 #KJ*17 日 #KJ*1 實
じついん #T35 実印
じつえき #T35*1 実益
じつえん #T30*2 実演
じつがい #T35*1 実害
じつぎ #T35*1 実技
じつぎょう #T35 実業
じつぎょうか #T35 実業家
じつげん #T30*255 実現
じつざい #T30*6 実在
じつじょう #T35*4 実情 #T35 実状
じつぞう #T35 実像
じつぞん #T30*1 実存
じつだん #T35 実弾
じつどう #T30*2 実働
じつぶつ #T35*4 実物
じつぼ #T35 実母
じつむ #T35*2 実務
じつめい #T35*2 実名
じつよう #T30*32 実用
じつり #T35 実利
じつりょく #T35*1 実力
じつれい #T35*2 実例
じつろく #T30*1 実録
じつわ #T35*2 実話
じづら #T35*2 字面
じてき #T30 自適
じてん #T35*81 時点 #T35*6 辞典 #T35*1 事典 #T35 次点 #T30 自転
じてんしゃ #T35*7 自転車
じでん #T35*1 自伝
じとう #T30 自答
じとく #T30 自得
じとり #T35*1 地鶏
じどう #T05*255 自動 #T35*1 児童
じどうし #T35*8 自動詞
じどうしゃ #T35*31 自動車
じどうしゃどう #T35*1 自動車道
じなり #T30 地鳴り
じなん #T35 次男
じにん #T30*1 自認 #T30 自任 辞任
じぬし #T35 地主
じねつ #T35 地熱
じはく #T30*1 自白
じはだ #T35 地肌
じはつ #T35 自発
じば #T35 地場
じばく #T30 自爆
じばん #T35*1 地盤
じひ #T35*1 慈悲 自費
じひつ #T35 自筆
じひょう #T35*1 辞表
じびか #T35 耳鼻科
じびき #T35*1 字引
じびょう #T35*1 持病
じふ #T30*2 自負
じぶつ #T35*1 事物
じぶん #T35*255 自分 #T35 時分
じへいしょう #T35*1 自閉症
じへん #T35*7 事変
じほう #T35*1 時報
じぼう #T35 自暴
じま #T35*7 島
じまい #T35*1 仕舞い
じまえ #T35*2 自前
じまく #T35*1 字幕
じまん #T30*3 自慢
じみ #T05*2 地味
じみち #T05*2 地道
じみん #KK*1 自民
じみんとう #KK*2 自民党
じむ #T35*24 事務 #T35*1 ジム
じむき #T35*1 事務機
じむきょく #T35*8 事務局
じむしょ #T35*2 事務所
じめい #T05*3 自明
じめつ #T30 自滅
じめん #T35*2 地面
じもく #T35 耳目
じもくじ #CN 甚目寺
じもと #T35*2 地元
じもん #T35 自問
じゃ #KJ*7 邪 #T35*4 蛇
じゃーなりすと #T35*1 ジャーナリスト
じゃーなる #T35*2 ジャーナル
じゃあ #CJ*13 じゃあ
じゃあく #T05*1 邪悪
じゃいあんと #T05*1 ジャイアント
じゃがいも #T35*1 じゃがいも
じゃき #T35 邪気
じゃく #T35*3 弱 #KJ*1 雀 #KJ 若 寂 #JSSUC 弱 #PRE 弱 #KJ 惹
じゃくし #T35 弱視
じゃくしゃ #T35*1 弱者
じゃくしょう #T05*1 弱小
じゃくしん #T35 弱震
じゃくそん #KK*1 ジャクソン
じゃくてん #T35*2 弱点
じゃくにくきょうしょく #T15 弱肉強食
じゃくねん #T35*1 若年
じゃくはい #T35*1 若輩
じゃくやく #T30 雀躍
じゃぐち #T35 蛇口
じゃけっと #T35*1 ジャケット
じゃけん #T05 じゃけん 邪慳
じゃしん #T35 邪心
じゃじゃうま #T35*1 じゃじゃ馬
じゃすい #T30*3 邪推
じゃすと #T35*4 ジャスト
じゃすみん #T35*1 ジャスミン
じゃず #T35*2 ジャズ
じゃっかん #T35*40 若干
じゃっき #T35 ジャッキ #T30 惹起
じゃっく #T35 惹句
じゃどう #T35*4 邪道
じゃま #T00*23 邪魔
じゃむ #T35*13 ジャム
じゃり #T35 砂利
じゃんぐる #T35*1 ジャングル
じゃんぱー #T35*1 ジャンパー
じゃんぷ #T30*2 ジャンプ
じゃんぼ #T05*1 ジャンボ
じゃんる #T35*8 ジャンル
じゅ #KJ*35 受 #KJ*2 呪 #KJ*1 寿 授 樹 #KJ 儒 綬 需 嬬
じゅーくぼっくす #T35*29 ジュークボックス
じゅーす #T35*7 ジュース
じゅう #NN*119 十 #CNSUC1*42 中 #T35*15 重 #KJ*4 住 充 #T35*2 銃 #KJ*1 従 柔 渋 獣 縦 #NN 拾 #KJ 什 戎 汁 #JS 重 #KJ 重 #PRE 重 #SUC 中 #T35 中
じゅうあつ #T35 重圧
じゅうい #T35 獣医
じゅうおう #T35*1 縦横 #CN 十王
じゅうかん #T30 獣姦 縦貫 #T15 重患
じゅうき #T35*1 什器 #T35 銃器
じゅうきち #JNM 重吉
じゅうきょ #T35*2 住居
じゅうぎょういん #T35*3 従業員
じゅうけい #T35*1 重刑
じゅうけいしょう #T35 重軽傷
じゅうけつ #T30 充血
じゅうげき #T30 銃撃
じゅうこう #T05*1 重厚 #T35 銃口
じゅうこうぎょう #T35 重工業
じゅうさつ #T30 銃殺
じゅうざい #T35 重罪
じゅうし #T30*4 重視
じゅうしやま #CN 十四山
じゅうしょ #T35*30 住所
じゅうしょう #T35*3 重傷 #T35*2 重症 #T30 重唱
じゅうしょく #T35 住職
じゅうしょろく #T35*2 住所録
じゅうしん #T35*1 重心 #T35 銃身
じゅうじ #T30*3 従事 #T35 十字 十時
じゅうじつ #T30*24 充実
じゅうじゅう #F14 重々
じゅうじゅん #T05*1 従順
じゅうじょう #T30*3 重畳
じゅうすい #T35 重水
じゅうせい #T35 銃声
じゅうせき #T35 重責
じゅうぜい #T35 重税
じゅうぜん #T15 十全
じゅうそう #T30*1 重層 #JCN 十三 #T30 縦走 重奏 #T35 重曹
じゅうそく #T30*1 充足
じゅうぞう #T35*4 重像 #JNM 十三
じゅうぞく #T30*1 従属
じゅうぞくし #T35*1 従属子
じゅうたい #T30*4 渋滞 #T05 重体
じゅうたく #T35*4 住宅
じゅうたくがい #T35 住宅街
じゅうたくち #T35*1 住宅地
じゅうたん #T35*4 絨毯 #T35*1 じゅうたん
じゅうだい #T05*6 重大
じゅうだん #T35*1 銃弾 #T30 縦断
じゅうてん #T35*7 重点 #T30*1 充填
じゅうでん #T30*2 充電
じゅうとう #T30 充当
じゅうとく #T35*1 重篤
じゅうどう #T35*6 柔道
じゅうなん #T05*16 柔軟
じゅうにし #T35*1 十二支
じゅうにしちょう #T35 十二指腸
じゅうにん #T35*2 住人
じゅうばこ #T35*1 重箱
じゅうばつ #T35 重罰
じゅうびょう #T35 重病
じゅうふく #T30*5 重複
じゅうぶん #T05*90 十分 #T05*34 充分
じゅうほう #T35*1 銃砲
じゅうまん #T30*1 充満
じゅうみん #T35*3 住民
じゅうみんか #T35 住民課
じゅうみんぜい #T35 住民税
じゅうめん #T35 渋面
じゅうもう #T35 絨毛
じゅうもうまく #T35*1 絨毛膜
じゅうもんじ #T35*1 十文字 #CN 十文字
じゅうやく #T35 重役
じゅうゆ #T35 重油
じゅうよう #T05*79 重要 #T30 充用
じゅうようし #T30*1 重要視
じゅうらい #T35*120 従来
じゅうらん #T30 縦覧
じゅうりょう #T35*5 重量
じゅうりょうあげ #T35 重量挙げ
じゅうりょうぜい #T35 従量税
じゅうりょく #T35*3 重力
じゅうりん #T30 蹂躙
じゅうろう #JNM 十郎
じゅかい #T35 樹海
じゅきゅう #T35*1 需給 #T30 受給
じゅぎょう #T30*15 授業
じゅく #T35*19 塾 #S5*19 熟 #KJ*2 熟 #SX*2 熟 #T35*1 宿
じゅくこう #T30*2 熟考
じゅくご #T35*2 熟語
じゅくし #T35 熟柿
じゅくじくん #T35 熟字訓
じゅくすい #T30*1 熟睡
じゅくせい #T30*1 熟成
じゅくたつ #T30*1 熟達
じゅくち #T30*2 熟知
じゅくどく #T30*1 熟読
じゅくねん #T35 熟年
じゅくりょ #T30*1 熟慮
じゅくれん #T30*2 熟練
じゅけい #T30 受刑
じゅけん #T30*2 受験 #T30 受検
じゅけんせい #T35*1 受験生
じゅこう #T30*4 受講
じゅし #T35 樹脂
じゅしょう #T30*2 受賞 授賞
じゅしん #T30*30 受信 #T30*1 受診
じゅじゅ #T30*2 授受
じゅじゅつ #T35*1 呪術
じゅすい #T30 入水
じゅず #T35*1 数珠
じゅせい #T30*1 受精 #T30 授精
じゅせいらん #T35*1 受精卵
じゅぞう #T30*1 受像
じゅたい #T30 受胎
じゅたく #T30*2 受託
じゅだく #T30 受諾
じゅちゅう #T30*23 受注
じゅっ #NN*5 十
じゅっかい #T30*1 述懐
じゅつ #T35*16 術 #KJ*3 述
じゅつご #T35*43 述語 #T35*3 術語
じゅどう #T35*2 受動
じゅどうたい #T35*2 受動態
じゅなん #T30*1 受難
じゅにあ #T35*1 ジュニア
じゅにゅう #T30 授乳
じゅねーぶ #CN ジュネーブ
じゅのう #T30*1 受納
じゅひ #T35 樹皮
じゅひょう #T35*1 樹氷
じゅふん #T30 受粉
じゅみょう #T35*2 寿命
じゅもく #T35*1 樹木
じゅもん #T35*2 呪文
じゅよ #T30 授与
じゅよう #T30*16 受容 #T35*8 需要
じゅり #T30*2 受理
じゅりつ #T30 樹立
じゅりょう #T30*1 受領
じゅりょうしょ #T35*1 受領書
じゅれい #T35 樹齢
じゅわき #T35*1 受話器
じゅん #T35*29 順 #T35*17 淳 #T05*7 純 #ZX*6 準 #T35*4 準 #KJ*2 巡 #T35*1 准 #ZX*1 准 #KJ*1 旬 #T35*1 潤 #KJ 閏 #PRE 准 #KJ 循 楯 殉 #ZX 殉 #JNM 淳 #PRE 準 #JNM 潤 #KJ 盾 #JNM 純 #PRE 純 #KJ 遵 醇 #JNM 順 #SUC 順 #KJ 惇 馴 隼
じゅんあい #T35*1 純愛
じゅんい #T35*15 順位
じゅんいち #JNM 淳一 純一 順一
じゅんいつ #T05 純一
じゅんえき #T35 純益
じゅんえん #T30*2 順延 #T30 巡演
じゅんか #T30*1 馴化 #T30 純化 順化
じゅんかい #T30*2 巡回
じゅんかつ #T05*1 潤滑
じゅんかつゆ #T35 潤滑油
じゅんかん #T30*5 循環 #T35 准看
じゅんきょ #T30*11 準拠
じゅんきん #T35 純金
じゅんぎょう #T30 巡業
じゅんぐり #T15 順繰り
じゅんけっしょう #T35 準決勝
じゅんけつ #T05 純潔 #T35 純血
じゅんこ #JNM 淳子 純子 順子
じゅんこう #T30*1 巡行
じゅんさ #T35 巡査
じゅんさい #T35 蓴菜
じゅんさつ #T30 巡察
じゅんし #T30*1 殉死 #T30 巡視
じゅんしゅ #T30*1 遵守
じゅんしょく #T30 殉職 #T35 純色
じゅんしん #T05*1 純真
じゅんじ #F14*34 順次
じゅんじゅん #F14*2 順々 #F06 じゅんじゅん 諄々
じゅんじゅんけっしょう #T35 準々決勝
じゅんじょ #T35*57 順序
じゅんじょう #T05*1 純情
じゅんすい #T05*7 純粋 #T35*1 純水
じゅんせい #T05*4 純正
じゅんせつ #T30 順接
じゅんぞう #JNM 淳三
じゅんたいげんじょし #T35 準体言助詞
じゅんたく #T05 潤沢
じゅんち #T30 馴致
じゅんちょう #T05*9 順調
じゅんとう #T05*1 順当
じゅんど #T35 純度
じゅんのう #T30*1 順応
じゅんばん #T35*63 順番
じゅんぱく #T05 純白
じゅんび #T30*112 準備
じゅんびきん #T35*1 準備金
じゅんふどう #T15*1 順不同
じゅんぷう #T35 順風
じゅんぼく #T05 純朴
じゅんぽう #T35 遵法 順法
じゅんよう #T30 準用
じゅんようかん #T35 巡洋艦
じゅんり #T35 純利
じゅんりょう #T05 純良 順良
じゅんれい #T30*1 巡礼
じゅんろ #T35 順路
じゆう #T05*62 自由 #T35*1 事由
じゆうがおか #CN 自由が丘 自由ヶ丘
じゆうぎょう #T35 自由業
じょ #T35*14 所 #JS*8 女 #S5*4 序 #T30*4 序 #KJ*3 如 #KJ*1 助 徐 #S5*1 除 #T30*1 除 #SUC 所 #PRE 助 #S5 叙 #T30 叙 #KJ 女 #SX 恕 #KJ 鋤 汝
じょーく #T35*1 ジョーク
じょーじ #JN ジョージ
じょい #T35*5 女医
じょいす #JN ジョイス
じょう #T35*145 上 #T35*47 情 #T35*25 嬢 #T35*17 場 #T35*5 状 #ZX*4 乗 #KJ*4 常 #KJ*2 乗 #KJ*1 丈 冗 城 擾 条 譲 醸 #JS*1 錠 #KJ*1 茸 #JS*1 帖 #KJ*1 貞 #PRE 上 #SUC 上 #KJ 丞 剰 #JN 城 #SUC 場 #KJ 壌 #JS 条 #KJ 杖 浄 #SUC 状 #JS 畳 #KJ 畳 穣 蒸 #T35 錠 #KJ 成 静 帖 定 瀞 縄
じょうあい #T35 情愛
じょうい #T35*38 上位 #T30 譲位
じょういん #T35*1 乗員 #T35 上院
じょうえい #T30*2 上映
じょうえつ #CN 上越
じょうえん #T30*2 上演
じょうおん #T35 常温
じょうか #T30 浄化
じょうかい #T35 上界
じょうかそう #T35 浄化槽
じょうかまち #T35 城下町
じょうかん #T35*1 情感 #T35 上巻 上官
じょうがい #T35 場外
じょうがしま #CN 城ヶ島
じょうき #T35*113 上記 #T30 上気 #T35 蒸気
じょうきゃく #T35*1 乗客
じょうきゅう #T35 上級
じょうきょう #T35*183 状況 #T30*5 上京
じょうきん #T30*1 常勤
じょうぎ #T35*1 定規
じょうくう #T35*1 上空
じょうけい #T35*3 情景
じょうけん #T35*162 条件
じょうげ #T30*13 上下 #CN 上下
じょうげん #T35*4 上限 #T35 上弦
じょうこう #T35*16 情工 #T35*3 上皇 #T35*2 条項 #T30*1 上向 乗降 #T30 上行
じょうこく #T30*1 上告
じょうご #T35*1 上戸 #T35 漏斗
じょうさい #T35 城砦
じょうさん #T30*1 乗算 #T30 蒸散
じょうざい #T30*1 常在 #T35 錠剤
じょうざん #T30*8 乗算
じょうし #T35*20 上司 #T30 上梓 情死
じょうしき #T35*16 常識
じょうしつ #T05*1 上質
じょうしゃ #T30*1 乗車
じょうしゅ #T35*1 城主
じょうしょ #KK*4 情処 #T35 情緒
じょうしょう #T30*4 上昇
じょうしょく #T30 常食
じょうしょろん #T35*1 情処論
じょうしん #T30 上申
じょうしんえつ #CN 上信越
じょうしんこ #T35 上新粉
じょうじ #T35*3 情事 #T35*2 常時
じょうじま #JCN 城島
じょうじゅ #T30 成就
じょうじゅつ #T30*6 上述
じょうじゅん #T35*4 上旬
じょうじょ #T30 乗除
じょうじょう #T30*1 上場 #T05 上々 上乗 #T35 情状 #F02 嫋々
じょうじん #T35 常人
じょうすい #T30 浄水
じょうすいどう #T35 上水道
じょうすう #T35*2 乗数 #T35*1 定数 #T35 常数
じょうず #T05*6 上手
じょうせい #T35*2 情勢 #T35*1 上清 #T35 状勢 #T30 醸成
じょうせき #T35*1 定石 #T35 上席
じょうせつ #T30*1 常設
じょうせん #T30*1 乗船
じょうぜつ #T05 饒舌
じょうそ #T30 上訴
じょうそう #T35*1 上層 #T35 情操
じょうぞう #T30 醸造
じょうたい #T35*255 状態 #T35*1 上体 #T35 常態
じょうたつ #T30*1 上達
じょうたん #T35*1 上端
じょうだん #T35*12 冗談 #T35*3 上段
じょうち #KK*4 上智 #T30*2 常置 #JNS 上智
じょうちゃく #T30 蒸着
じょうちゅう #T30*8 常駐
じょうちょ #T35*1 情緒
じょうちょう #T05*8 冗長
じょうてんき #T15 上天気
じょうでき #T05*1 上出来
じょうと #T30*1 譲渡
じょうとう #T05*2 上等 #T00*2 常套 #CN 上道 #JCN 城東
じょうとうく #T35 常套句
じょうど #T35*1 浄土
じょうどじ #CN 浄土寺
じょうない #T35*1 場内 #JNS 城内 #T35 城内
じょうなん #CN 城南
じょうにん #T30 常任
じょうねつ #T35*2 情熱
じょうねん #T35 情念
じょうのう #T30 上納
じょうのうち #JNM 城之内
じょうはつ #T30*1 蒸発
じょうはな #JCN 城端
じょうはんしん #T35 上半身
じょうば #T30 乗馬
じょうばん #JNS 常盤
じょうひ #T35*1 上皮
じょうひん #T05*4 上品
じょうび #T30 常備
じょうぶ #T35*4 上部 #T05*3 丈夫
じょうぶつ #T30*1 成仏
じょうぶん #T35*1 条文
じょうへき #T35 城壁
じょうへん #JCN 城辺
じょうほ #T30*1 譲歩
じょうほう #T35*255 情報 #T35*2 乗法 #T35*1 常法 #T35 上方
じょうほうしょりがっかい #KK*2 情報処理学会
じょうほうじ #CN 浄法寺
じょうほく #CN 常北
じょうぼう #CN 上房
じょうみゃく #T35 静脈
じょうみん #T35 常民
じょうむ #T30 乗務 #T35 常務
じょうやく #T30*1 条約
じょうよ #T35*1 剰余 #T30*1 譲与
じょうよう #T30*2 乗用 常用 #CN 上陽 城陽
じょうようしゃ #T35 乗用車
じょうよく #T35 情欲
じょうらん #T30 上覧 擾乱
じょうり #T35*1 条理
じょうりく #T30*1 上陸
じょうりゅう #T35*1 上流 #T30 蒸留
じょうりょく #T35*1 常緑
じょうりょくじゅ #T35 常緑樹
じょうるり #T35 浄瑠璃
じょうれい #T35 常例 条例
じょうれん #T35 常連
じょうろ #T35 ジョウロ
じょえん #T30 助演
じょおう #T35*2 女王
じょかく #T35*1 除核
じょがい #T30*3 除外
じょがくせい #T35 女学生
じょきょ #T30*7 除去
じょきょうじゅ #T35*3 助教授
じょく #KJ 辱
じょくん #T35 叙勲
じょげん #T30*5 助言
じょこう #T30*1 徐行
じょさい #T35*1 如才
じょさんぷ #T35*1 助産婦
じょざん #T30*2 除算
じょし #T35*38 女子 #T35*19 助詞 #T35*3 女史
じょしつ #T35*1 除湿
じょしゅ #T35*5 助手
じょしょう #T35*1 序章
じょじ #T35*1 女児 #T35 叙事
じょじゅつ #T30*1 叙述
じょじょう #T35 叙情
じょじょに #F14*3 徐々に
じょすうし #T35 助数詞
じょせい #T35*68 女性 #T30 助勢 助成 #T35 女声
じょせき #T30 除籍
じょせつ #T35*1 序説 #T30*1 除雪
じょそう #T30*1 女装 #T30 助走 除草
じょたい #T30 除隊
じょたんぱく #T35*1 除蛋白
じょちゅう #T35*1 女中
じょちょう #T30 助長
じょっき #T35 ジョッキ
じょどうし #T35*4 助動詞
じょばん #T35 序盤
じょぶ #T35*17 ジョブ
じょぶず #JN ジョブズ
じょぶん #T35*2 序文
じょほう #T35*6 除法
じょまく #T35 序幕
じょまくしき #T35 除幕式
じょめい #T30*1 助命 除名
じょや #T35 除夜
じょやく #T35 助役
じょゆう #T35*1 女優
じょりゅう #T35*1 女流
じょりょく #T30*1 助力
じょれつ #T35*1 序列
じょろん #T35*1 序論
じょん #JN ジョン
じよう #T35*1 滋養
じら #S5r*2 じら
じらい #T35*1 地雷
じりき #T35*3 自力 #T35 地力
じりつ #T30*64 自立 #T35*1 自律 #T35 而立
じりつご #T35 自立語
じりゅう #T35 時流 自流
じりょく #T35 磁力
じる #SUC*2 汁
じるし #SUC*9 印
じれい #T35*7 辞令 #T35*4 事例
じれんま #T35*1 ジレンマ
じろう #JNM 次郎 治郎 二郎
じろじろ #F06 じろじろ
じろん #T35*1 持論
じん #CNSUC1*63 人 #T35*28 仁 #T35*4 陣 #KJ*1 神 尋 尽 #SUC 人 #T35 人 #JNM 仁 #KJ 刃 塵 壬 甚 腎 訊 迅 #JS 陣 #SUC 陣 #KJ 靭
じんい #T35*2 人為
じんいち #JNM 仁一
じんいん #T35*2 人員
じんう #T35*1 腎盂
じんえい #T35*1 陣営
じんえん #T35*1 腎炎
じんか #T35 人家
じんかい #T35 塵芥
じんかいせんじゅつ #T35 人海戦術
じんかく #T35*1 人格
じんがさ #T35 陣笠
じんぎ #T35*1 仁義
じんくす #T35*1 ジンクス
じんぐう #T35*2 神宮 #JNS 神功
じんけん #T35*1 人権 #T35 人件
じんこう #T35*27 人工 #T35*4 人口
じんこつ #T35 人骨
じんさい #T35 人災
じんざい #T35*4 人材
じんしゅ #T35*1 人種
じんしん #T35*1 人身
じんじ #T35*7 人事
じんじいん #KK*1 人事院
じんじか #T35 人事課
じんじゃ #T35*1 神社
じんじょう #T05*1 尋常
じんせい #T35*8 人生
じんせいかん #T35 人生観
じんせいろん #T35 人生論
じんせき #JCN 神石 #T35 人跡
じんせん #T30*1 人選
じんそく #T05*2 迅速
じんぞう #T30*4 人造 #T35*1 腎臓
じんたい #T35 人体
じんだい #T05 甚大
じんち #T35*2 人知 #T35 陣地
じんちく #T35*1 人畜
じんちゅう #T35 陣中
じんちょうげ #T35 ジンチョウゲ 沈丁花
じんつう #T35 陣痛
じんてい #T35 人定
じんてき #T05*2 人的
じんとう #T35 陣頭
じんとく #T35*1 人徳
じんどう #T35*1 人道
じんば #JNS 神馬 #T35 人馬
じんぶつ #T35*17 人物
じんぶん #T35*1 人文
じんぼ #JNS 神保
じんぼう #T35 人望
じんぼうちょう #CN 神保町
じんましん #T35*1 蕁麻疹
じんみゃく #T35*1 人脈
じんみん #T35*3 人民
じんめい #T35*24 人名 #T35*1 人命
じんもん #T30*1 尋問
じんりきしゃ #T35*1 人力車
じんりょく #T30*1 尽力 #T35 人力
じんるい #T35*3 人類
す #M5r*255 済 #T35*119 洲 #R5*77 擦 #R5r*72 刷 #K5*67 す #PRE*62 素 #M5*57 住 #M5r*45 す #W5*40 吸 #M5*8 澄 #K5*7 好 #KJ*5 須 #K5r*4 空 #R5*4 摺 #T35*3 巣 #T35*2 酢 素 #KJ*1 守 #T35*1 州 #KJ*1 栖 #K5r 漉 #KJ 笥 諏 棲 #M5 棲 #K5 透 #KJ 簾
すーつ #T35*30 スーツ
すーぱ #PRE*4 スーパ
すーぱー #PRE*22 スーパー #T35*13 スーパー
すーぱーばいざ #T35*1 スーパーバイザ
すーぱーゆーざ #T35*10 スーパーユーザ
すーぱばいざ #T35*1 スーパバイザ
すーぷ #T35*1 スープ
すあし #T35 素足
すい #SUC*116 水 #KJ*3 推 #KJ*1 吹 垂 帥 睡 #T05*1 粋 #KJ*1 翠 衰 酔 悴 #KJ 出 #T35 水 #KJ 炊 遂 錐 錘 誰 穂
すいーぷ #T30*3 スイープ
すいあげ #KSr*6 吸い上げ
すいあつ #T35 水圧
すいい #T35*6 水位 #T30*4 推移
すいいき #T35 水域
すいいりつ #T35*1 推移律
すいえい #T30*13 水泳
すいおん #T35*1 水温
すいか #T30*1 垂下 #T35 すいか 西瓜 #T30 誰何
すいがい #T35 水害
すいがら #T35 吸殻
すいきゅう #T35 水球
すいきょ #T30 推挙
すいぎん #T35 水銀
すいけい #T30 推計
すいげん #T35 水源
すいこ #M5r*1 吸い込
すいこう #T30*8 推敲 #T30*6 遂行 #T30*1 推考
すいこでん #KK 水滸伝
すいさいが #T35 水彩画
すいさつ #T30*2 推察
すいさん #T30*1 推算 #T35*1 水産 水酸
すいさんか #T30 水酸化
すいし #T30 水死
すいしつ #T35 水質
すいしょう #T30*5 推奨 #T30 推賞 #T35 水晶
すいしん #T30*22 推進 #T35 垂心 水深
すいじ #T30 炊事
すいじゃく #T30*1 衰弱
すいじゅん #T35*10 水準
すいじょう #T35*1 水上
すいじょうき #T35 水蒸気
すいす #CN スイス
すいせい #KK*1 水星 #T35*1 彗星 #T35 水性
すいせん #T35*6 垂線 #T30*5 推薦 #T30*1 水洗 #T35 水仙
すいぜん #T30 垂涎
すいそ #T35*1 水素
すいそう #T35*1 水層 #T30 吹奏 #T35 水槽 #T30 水葬
すいそく #T30*4 推測
すいぞう #T35 膵臓
すいぞくかん #T35*1 水族館
すいた #JCN 吹田
すいたい #T30*1 衰退
すいちゅう #T35 水中
すいちゅうよくせん #T35 水中翼船
すいちょく #T05*16 垂直
すいっち #T30*67 スイッチ
すいっちんぐ #T30*16 スイッチング
すいてい #T30*9 推定 #T35 水底
すいてき #T35 水滴
すいでん #T35 水田
すいと #R5r*1 吸い取
すいとう #T30*1 出納 #T35*1 水稲 #T35 水筒
すいどう #T35*4 水道
すいなん #T30 水難
すいはんき #T35*2 炊飯器
すいばく #T35 水爆
すいばら #JCN 水原
すいばん #T35 水盤
すいふ #T35*1 水夫 #JCN 水府
すいへい #T05*16 水平
すいほう #T35*1 水胞 水疱
すいぼう #T30 衰亡
すいぼくが #T35 水墨画
すいぼつ #T30 水没
すいま #T35*1 睡魔
すいません #CJ*29 すいません #T35*25 すいません
すいみん #T30*4 睡眠
すいめん #T35*1 水面
すいもん #T35 水門
すいよう #T35*7 水曜 #T35*2 水溶
すいようえき #T35 水溶液
すいようび #T35*23 水曜日
すいらん #KK*1 翠嵐
すいり #T30*2 推理
すいりゅう #T35*1 水流
すいりょう #T30 推量 #T35 水量
すいりょく #T35*13 推力 #T35*2 水力
すいろ #T35 水路
すいろん #T30*47 推論
すいんぐ #T30 スイング
すう #SUC*253 数 #T35*180 数 #KJ*1 枢 #KJ 崇 嵩 趨 雛
すうぇーでん #CN スウェーデン
すうかい #T35*4 数回
すうがく #T35*26 数学
すうき #T05 数奇
すうこ #T35*4 数個
すうこう #T05*1 崇高
すうし #T35*8 数詞
すうしき #T35*13 数式
すうじ #T35*73 数字
すうじく #T35*1 枢軸
すうじつ #T35*6 数日
すうせい #T35*3 趨勢
すうだい #T35*2 数台
すうち #T35*42 数値
すうにん #T35*4 数人
すうはい #T30 崇拝
すうめい #T35*6 数名
すうり #T35*19 数理
すうりかがっか #KK*1 数理科学科
すうりょう #T35*5 数量
すうれつ #T35*2 数列
すえ #SUC*11 末 #KS*1 据え #JN 須恵 #KJ 据 #T35 陶 末
すえお #K5r*1 据え置 #JNM 末男
すえきち #JNM 末吉 #T35 末吉
すえじ #JNM 末治
すえつけ #KSr 据え付け
すえなが #KY*1 末永
すえなごう #KYU*1 末永う
すえひろ #T35*3 末広 #JCN 末広
すえまつ #JN 末松
すえよし #JCN 末吉
すおう #CN 周防
すか #S5r*1 透か
すかーと #T35*5 スカート
すかうと #T30*1 スカウト
すかがわ #JCN 須賀川
すかさず #F14*1 すかさず
すかっちゃーどぷろっと #T35*1 スカッチャードプロット
すかとろじー #T35 スカトロジー
すから #T35*11 スカラ
すかんぴん #T15 素寒貧
すが #JCN 須賀 #JNS 菅
すがい #JNS 菅井
すがお #T35*1 素顔
すがた #T35*24 姿 #JNS 菅田
すがだいら #JCN 菅平
すがの #JCN 菅野
すがはら #JNS 菅原
すがも #JCN 巣鴨
すがや #JNS 菅谷
すがわら #JCN 菅原
すき #T05*110 好き #T35 鍬 隙 #KJ 漉 #T35 鋤 #JCN 須木
すきー #T30*42 スキー
すきーま #T35*2 スキーマ
すきーやー #T35*1 スキーヤー
すきずき #T05*1 好きずき
すきっぱら #T35 空きっ腹
すきっぷ #T30*6 スキップ
すきとお #R5 透き通
すきま #T35*1 すき間 #T35 隙間
すきゃな #T35*11 スキャナ
すきゃん #T30*2 スキャン
すきゃんだる #T35 スキャンダル
すきる #T35*5 スキル
すぎ #KSr*133 過ぎ #T35*20 杉 #KSr*8 すぎ #KJ 椙
すぎうら #JNS 杉浦
すぎえ #JN 杉江
すぎさ #R5*2 過ぎ去
すぎさき #JNS 杉崎
すぎざき #JNS 杉崎
すぎた #JCN 杉田
すぎと #JCN 杉戸
すぎとう #JNS 杉藤
すぎなみ #JCN 杉並
すぎの #JNS 杉野
すぎのした #JNS 杉之下
すぎはら #JNS 杉原
すぎむら #JNS 杉村
すぎもと #JCN 杉本
すぎやま #T35*6 杉山 #JNS 杉山
すく #W5r*4 救 #W5*2 巣食
すくーぷ #T30 スクープ
すくーる #T35*4 スクール
すくいだ #S5r 救い出
すくな #KY*122 少な #KY*20 少 #KY すくな
すくのう #KYU*3 少のう #KYU*2 少う #KYU すくのう
すくも #JCN 宿毛
すくらっぷ #T30 スクラップ
すくりーにんぐ #T30*1 スクリーニング
すくりーん #T35*20 スクリーン
すくりぷと #T35*42 スクリプト
すくろーる #T30*49 スクロール
すぐ #F14*117 すぐ
すぐさま #F14*1 すぐさま
すぐに #F14*25 すぐに
すぐる #JNM 英
すぐれ #KS*17 優れ #KS*1 すぐれ
すけ #KJ*4 介 #KJ*3 助 #KSr*3 透け #KJ*1 輔 佑 祐 #KJ 丞 甫 亮
すけーと #T30*1 スケート
すけーりんぐ #T30*1 スケーリング
すけーる #T35*1 スケール
すけじゅーら #T35*1 スケジューラ
すけじゅーりんぐ #T30*3 スケジューリング
すけじゅーる #T30*38 スケジュール
すけっち #T30*1 スケッチ
すけるとん #T35*3 スケルトン
すげ #JNS 菅 #KJ 菅
すげぬま #JNS 菅沼
すこーぴんぐ #T35*2 スコーピング
すこーぷ #T35*6 スコープ
すこあ #T35*3 スコア
すこし #T35*255 少し #F14*13 すこし
すこしも #F14 少しも
すこぶる #F14*2 すこぶる
すこやか #T05 健やか
すご #KYmi*44 凄 #KYmi*37 すご #S5*19 過ご
すごう #KYU*2 すごう 凄う #JCN 菅生
すごろく #T35*1 すごろく
すさ #JCN 須佐
すさき #JCN 須崎
すさまじ #KY*2 すさまじ #KY*1 凄まじ
すさまじさ #T35*1 凄まじさ
すさまじゅう #KYU*1 すさまじゅう 凄まじゅう
すさみ #CN すさみ
すざか #JCN 須坂
すざく #KK 朱雀
すし #T35*8 寿司 #T35*2 鮨
すじ #SUC*6 筋 #T35*1 筋
すじがき #T35*1 筋書き
すじみち #T35*1 筋道
すじょう #T35 素性
すじわる #T18 筋悪
すす #M5r*147 進 #R5*2 すす #T35*2 煤 #G5r*1 濯
すすき #T35 ススキ 薄
すすすっと #F14*1 すすすっと
すすみだ #S5*1 進み出
すすむ #JNM 晋 進
すすめ #KSr*10 勧め #KS*10 進め #KSr*5 すすめ 奨め #KSr*3 推め 薦め
すず #T35*6 鈴 #M5*1 涼 #T35 スズ 錫 #CN 珠洲
すずか #JCN 鈴鹿
すずかぜ #T35 涼風
すずき #T35*1 鱸 #JCN 鈴木
すずさわ #KK*1 すずさわ
すずし #KY*2 涼し
すずしゅう #KYU*1 涼しゅう
すずめ #T35*2 雀
すずめばち #T35 スズメバチ 雀蜂
すずらん #T35*1 鈴蘭
すずり #T35*1 硯
すそ #T35*1 裾
すその #JCN 裾野 #T35 裾野
すた #R5r 廃
すたー #T35*24 スター
すたーと #T30*8 スタート
すたーとれっく #KK*1 スタートレック
すたーりにずむ #T35 スターリニズム
すたいる #T35*19 スタイル
すたじお #T35*2 スタジオ
すたっく #T35*41 スタック
すたっくうぇあ #T35*1 スタックウェア
すたっふ #T35*4 スタッフ
すたてぃっく #T05*12 スタティック
すたでぃ #T30*4 スタディ
すたぶ #T35*6 スタブ
すたみな #T35 スタミナ
すたれ #KS*1 廃れ
すたんだーど #T05*1 スタンダード
すたんど #T35*3 スタンド
すたんふぉーど #KK*3 スタンフォード
すたんぷ #T35*3 スタンプ
すだ #JCN 須田 #T5r 巣立
すだま #JCN 須玉
すだれ #T35 簾
すちーる #T35*1 スチール
すちろーる #T35*1 スチロール
すっかり #F14*9 すっかり
すっきり #F04*5 すっきり
すっつ #CN 寿都
すっと #F12 すっと
すっぱ #KYmi*3 すっぱ
すっぽか #S5r すっぽか
すっぽり #F06 すっぽり
すっぽん #T35 すっぽん
すて #KS*55 捨て
すてーしょん #T35*139 ステーション
すてーじ #T35*13 ステージ
すてーたす #T35*17 ステータス
すてーと #T35*1 ステート
すてーとめんと #T35*1 ステートメント
すてぃっく #T35 スティック
すていたす #T35*5 ステイタス
すてき #T05*7 素敵 #T05*2 すてき
すてさ #R5 捨て去
すてっかー #T35*4 ステッカー
すてっぱ #T35*1 ステッパ
すてっぷ #T30*34 ステップ
すててこ #T35 ステテコ
すてばち #T05 捨て鉢
すてら #KK*6 ステラ
すてれお #T35*3 ステレオ
すてろいど #T35*1 ステロイド
すてんれす #T35*2 ステンレス
すで #T35*3 素手
すでに #F14*136 既に #F14*87 すでに
すと #T35*7 スト
すとーぶ #T35 ストーブ
すとーりー #T35*5 ストーリー
すとあ #T30*4 ストア
すとあー #T35*1 ストアー
すとっきんぐ #T35*1 ストッキング
すとっく #T30*3 ストック
すとっぷ #T30*5 ストップ
すとらいき #T30 ストライキ
すとらくちゃ #T35*1 ストラクチャ
すとらてじー #T35*1 ストラテジー
すとりーと #T35*1 ストリート
すとりーま #T35*1 ストリーマ
すとりーまー #T35*1 ストリーマー
すとりーむ #T35*33 ストリーム
すとりんぐ #T35*13 ストリング
すとれーと #T05*1 ストレート
すとれす #T35*4 ストレス
すとろーく #T30*8 ストローク
すとろーぶ #T35*1 ストローブ
すとろべりー #T35*1 ストロベリー
すどう #JNS 須藤
すどおり #T30 素通り
すな #T35*3 砂 #KJ 沙
すなお #T05*12 素直
すなが #JNS 須永
すながわ #T35*1 砂川 #JCN 砂川
すなじ #T35 砂地
すなっく #T35*2 スナック
すなっぷ #T30*6 スナップ
すなどけい #T35 砂時計
すなはま #T35*1 砂浜
すなはら #JNS 砂原
すなば #T35 砂場
すなみ #JCN 巣南
すなやま #T35 砂山
すなわち #F14*52 すなわち #F14*10 即ち
すにーかー #T35 スニーカー
すのっぶ #T15 スノッブ
すのまた #JCN 墨俣
すはだ #T35 素肌
すばや #KY*17 素早
すばらし #KY*20 素晴らし #KY すばらし
すばらしゅう #KYU*2 素晴らしゅう #KYU すばらしゅう
すばる #T35*7 昴
すぱーく #T30*2 スパーク
すぱい #T30*1 スパイ
すぱげってぃ #T35*2 スパゲッティ
すぱん #T35*5 スパン
すぴーか #T35*2 スピーカ
すぴーち #T30*1 スピーチ
すぴーでぃ #T05 スピーディ
すぴーど #T35*24 スピード
すぴん #T30*3 スピン
すぷーら #T35*2 スプーラ
すぷーる #T30*6 スプール
すぷらいん #T35*13 スプライン
すべ #KS*24 統べ #R5r*18 滑べ #T35*3 術
すべから #SX すべから
すべき #T15 すべき #T18 すべき
すべて #T35*255 全て #T35*14 すべて #T35*1 ∀
すべりおち #KSr 滑り落ち
すべりこ #M5r 滑り込
すべりだ #S5r 滑り出
すぺーす #T35*43 スペース
すぺあ #T35 スペア
すぺいん #CN スペイン
すぺいんご #KK*1 スペイン語
すぺくとる #T35*1 スペクトル
すぺしゃりすと #T35*1 スペシャリスト
すぺっく #T35*45 スペック
すぺりんぐ #T35*2 スペリング
すぺる #T35*9 スペル
すぽーつ #T30*7 スポーツ
すぽっと #T35*2 スポット
すぽんさ #T35 スポンサ
すぽんさー #T35*2 スポンサー
すま #S5*1 済ま #JCN 須磨 #S5r 澄ま
すまーと #T35*1 スマート
すまい #T35*2 住い #T35*1 住まい
すみ #SUC*75 済 #KJ*17 住 #T35*8 隅 #T35*6 角 #KJ*6 澄 #T35*2 炭 #T35*1 墨 #JNM スミ #T35 済
すみえ #JNM 澄江 #T35 墨絵
すみか #T35 住処 #KJ 栖
すみかわ #JCN 澄川
すみこ #M5r 住み込 #JNM 澄子
すみす #JN スミス
すみずみ #T35 隅々
すみた #JCN 住田
すみだ #JCN 墨田
すみつ #K5 住み着
すみとも #KK*11 住友
すみなれ #KS 住み慣れ
すみのえ #JCN 住之江
すみやか #T05*1 速やか
すみよう #JCN 住用
すみよし #JCN 住吉
すむーず #T05*6 スムーズ
すめあー #T35*1 スメアー
すめたな #JN スメタナ
すもう #T35*1 相撲
すもっぐ #T35 スモッグ
すもと #JCN 洲本 栖本
すもも #T35*2 李
すもん #JCN 守門
すやま #JNS 須山
すらいす #T30*1 スライス
すらいど #T30*1 スライド
すらすら #F06 すらすら
すらっしゅ #T35*3 スラッシュ
すらりと #F12 すらりと
すり #T35*3 摺
すりかえ #KSr*3 すり替え
すりきれ #KSr 擦り切れ
すりこ #M5r 擦り込
すりっぷ #T30*1 スリップ
すりぬけ #KSr すり抜け #R5r すり抜け
すりばち #T35*1 すりばち すり鉢 擂鉢
すりへ #R5r すり減
すりへら #S5 すり減ら
すりむ #K5r 擦りむ
すりりんぐ #T05*1 スリリング
すりる #T35*1 スリル
する #sxuru*255 する
するーぷっと #T35*3 スループット
するが #JCN 駿河
すると #CJ*14 すると
するど #KY*1 鋭
するどう #KYU*1 鋭う
するめ #T35*1 鯣
すれーぶ #T35*21 スレーブ
すれあ #W5r 擦れ合
すれいぶ #T35*1 スレイブ
すれすれ #T05 すれすれ
すれちが #W5r*4 すれ違
すれっからし #T15 すれっからし
すれっしょるど #T35 スレッショルド
すれっど #T35*6 スレッド
すろーがん #T35*1 スローガン
すろっと #T35*7 スロット
すろっとる #T35*2 スロットル
すわ #R5r*15 座 #JCN 諏訪 #R5 据わ
すわこ #CN 諏訪湖
すわっぷ #T30*16 スワップ
すわりこ #M5r*1 座り込
すん #JS*2 寸 #T35*2 寸
すんか #T35 寸暇
すんし #T35 寸志
すんじ #T35 寸時
すんぜん #T35*3 寸前
すんだい #JCN 駿台
すんだん #T30 寸断
すんてつ #T35 寸鉄
すんとう #CN 駿東
すんなり #F04 すんなり
すんぶん #T35 寸分
すんぽう #T35*2 寸法
すんわ #T35 寸話
ず #T35*157 図 #SUC*13 図 #T35*1 頭 #KJ 厨 逗 豆
ずーむ #T30*1 ズーム
ずあん #T35*1 図案
ずい #T35*2 随 #KJ*1 瑞 #KJ 蕊 #T35 髄
ずいい #T05*1 随意
ずいいち #T15 随一
ずいいん #T35 随員
ずいこう #T30 随行
ずいしょ #T35*1 随所
ずいじ #T35*3 随時
ずいそう #T30 随想
ずいはん #T30*1 随伴
ずいひつ #T35*1 随筆
ずいぶん #F06*77 ずいぶん #T05*74 ずいぶん #F06*14 随分 #T05*8 随分
ずうずうし #KY*2 図々し
ずうずうしゅう #KYU*1 図々しゅう
ずかい #T30 図解
ずかん #T35 図鑑
ずが #T35*2 図画
ずがいこつ #T35*1 頭蓋骨
ずがら #T35*1 図柄
ずき #N2T16*4 好き
ずく #SUC*2 ずく
ずくめ #N2T16*1 ずくめ
ずけい #T35*154 図形
ずこう #T35 図工
ずし #T35*4 逗子 #T30*1 図示 #T35 寿司 #CN 逗子
ずしき #T35*2 図式
ずしりと #F12 ずしりと
ずじょう #T35*2 図上 #T35 頭上
ずっきーに #T35*1 ズッキーニ
ずっしり #F04*2 ずっしり
ずっと #F14*22 ずっと
ずつう #T35*2 頭痛
ずのう #T35*1 頭脳
ずばぬけ #KS ずば抜け
ずばり #F04*1 ずばり
ずひょう #T35*3 図表
ずふ #T35 図譜
ずほう #T35 図法
ずぼし #T15 図星
ずぼら #T05 ずぼら
ずぼん #T35*2 ズボン
ずまい #SUC 住い
ずみ #SUC*7 住み #T35 済 #SUC 済み #T35 済み
ずめん #T35*7 図面
ずら #S5r*6 ずら
ずらりと #F14*1 ずらりと
ずる #KY*3 ずる
ずるう #KYU*1 ずるう
ずるずる #F04*3 ずるずる
ずれ #KSr*7 ずれ
ずれこ #M5r ずれ込
ずろく #T35 図録
ずんぐり #F04 ずんぐり
せ #JS*25 畝 #T35*22 背 #T35*17 瀬 #R5r*7 競 #sxe*4 せ #KJ*1 世 #KJ 施 畝 脊
せーたー #T35 セーター
せーふ #T30 セーフ
せーぶ #T30*16 セーブ
せーる #T30*1 セール
せーるす #T30*3 セールス
せい #T35*108 性 #T35*88 生 #T35*71 せい #CNSUC1*31 製 #T35*15 正 #T35*13 精 #KJ*9 成 #T35*6 姓 #KJ*5 清 #T35*4 所為 #KJ*4 聖 #S5*3 制 #T30*3 制 #KJ*3 西 靖 #KJ*2 政 星 #KJ*1 省 勢 #T30*1 征 #KJ*1 整 晴 盛 誠 青 静 斉 #KJ 井 甥 歳 犀 錆 鉦 #JS 世 #KJ 世 凄 #SUC 制 性 星 #KJ 棲 栖 #PRE 正 #KJ 牲 声 #T35 製 #KJ 誓 請 逝 醒 瀞 婿
せいあつ #T30 制圧
せいあん #T35 成案 #CN 西安
せいい #T35*2 誠意
せいいく #T30 成育 生育
せいいち #JNM 清一 精一 誠一
せいいちろう #JNM 清一郎 誠一郎
せいいっぱい #T35*1 精いっぱい
せいいつ #T05 斉一
せいいん #T35*1 成員 #T35 成因 正員
せいうん #T35 星雲 盛運 青雲
せいえい #T35*2 清栄 #T35*1 精鋭
せいえき #T35 精液
せいえん #T30*1 声援 #T30 製塩
せいおう #CN 西欧
せいおん #T05 静穏
せいか #T35*28 成果 #T35*1 聖火 #T35 正価 正課 生家 生花 盛夏 #JCN 精華 #T35 製菓 青果
せいかい #T30*15 正解 #T35*1 政界 #T35 盛会 #T30 精解
せいかく #T05*99 正確 #T35*79 性格
せいかつ #T30*21 生活
せいかつかん #T35*1 生活環
せいかん #T30*1 生還 #T35*1 精管 #T30*1 静観 #T05 精悍 #CN 青函
せいがいは #KK*1 青海波
せいがく #T35*4 声楽
せいがん #T30 請願
せいき #T05*67 正規 #JS*9 世紀 #T35*1 性器 #T35 世紀 生気 #JNS 生起 #T35 精機 精気
せいきち #JNM 清吉
せいきまつ #T35 世紀末
せいきゅう #T30*36 請求 #T05*2 性急
せいきょ #T30*1 逝去 #T35 盛挙
せいきょう #T35*8 生協 #T05*2 盛況
せいきょく #T35 政局
せいきん #T30 精勤
せいぎ #T35*1 正義
せいぎょ #T30*255 制御
せいぎょう #T30*6 成業 #T35 盛業
せいくう #T30 制空
せいぐん #T35 西軍
せいけい #T30*16 整形 #T30*1 成形 #T35*1 生計 #KK 成蹊
せいけつ #T05*1 清潔
せいけん #T35*25 政権 #T35 政見 #JNS 生研
せいげん #T30*112 制限 #T35 正弦
せいこ #JNM 聖子
せいこー #KK*19 セイコー
せいこう #T30*69 成功 #KK*6 星光 #T30 性交 #T05 生硬 精巧 #T35 精鋼 #T30 製鋼
せいこううどく #T35*2 晴耕雨読
せいこく #T35 正鴻
せいこつ #T35 整骨
せいこん #T30 成婚 #T35 精魂
せいご #T30*40 正誤 #T35*2 生後 #JNM 清吾
せいごう #T30*18 整合
せいごく #T35 生国
せいさ #T30*4 精査 #T35*2 性差
せいさい #T30*2 制裁 #T35*2 正妻 #T05*1 精細
せいさく #T30*17 製作 #T30*1 制作 #T35*1 政策 #JNM 清作
せいさん #T30*51 生産 #T30*12 清算 #T30*1 精算 #T05 凄惨 #T35 成算 正餐
せいさんだか #T35*1 生産高
せいさんち #T35 生産地
せいざ #T35*1 星座 #T30 正座
せいざい #T30*1 製材
せいざいかい #T35 政財界
せいざぶろう #JNM 清三郎
せいし #T30*4 正視 #T30*2 静止 #T30 制止 #T35 生死 精子 製紙
せいしき #T05*70 正式
せいしつ #T35*23 性質
せいしゃいん #T35 正社員
せいしゃく #T35 正釈
せいしゅ #T35 清酒
せいしゅん #T35*1 青春
せいしょ #T30*22 清書 #T35*2 聖書
せいしょう #T35*1 精漿 #T30*1 斉唱 #T35 清勝 清祥
せいしょうなごん #JNS 清少納言
せいしょうねん #T35*1 青少年
せいしょく #T30*1 生殖 #T35*1 生食 #T35 生色
せいしん #T35*14 精神 #T35*1 誠心 #T35 星辰 #JNM 聖心
せいしんか #T35 精神科
せいじ #T35*8 政治 #JNM 清司 清次 清治 清二 誠司 誠治 誠二 #T35 青磁
せいじか #T35*1 政治家
せいじつ #T05*1 誠実
せいじゃ #T35 正邪 聖者
せいじゃく #T05 静寂
せいじゅく #T30*2 成熟
せいじゅん #T05*1 清純
せいじょ #T30 整除
せいじょう #T05*116 正常 #T35*1 性状 政情 #T05*1 清浄 #CN 成城
せいじょうたい #T35*1 星状体
せいじろう #JNM 清次郎
せいじん #T30*1 成人
せいすい #T35*1 聖水 #T30 盛衰
せいすう #T35*35 整数 #T35*1 正数
せいすけ #JNM 清助
せいず #T35*1 星図 #T30 製図
せいせい #T30*213 生成 #F12 せいせい #T30 清々 精製
せいせいどうどう #F00 正々堂々
せいせき #T35*3 成績
せいせつ #T35 正接
せいせん #T35*1 性腺 #T05 生鮮 #T30 精選
せいぜい #F14*4 せいぜい
せいぜん #F00*1 整然 #T35 生前
せいそ #T05*7 整礎 #T05*1 清楚
せいそう #T35*3 政争 #T30*2 清掃 #T30*1 正装 #T35*1 精巣 #T30 盛装
せいそうけん #T35 成層圏
せいそく #T30*2 生息 #T30*1 棲息 #T05*1 正則
せいぞう #T30*22 製造 #JNM 清三 清蔵 誓三
せいぞろい #T30 勢ぞろい
せいぞん #T30*1 生存
せいたい #T35*2 生体 #T35*1 成体 #T35 生態 声帯
せいたいごう #JN 西太后
せいたかくけい #T35*1 正多角形
せいたかくちゅう #T35*1 正多角柱
せいたろう #JNM 清太郎
せいだい #T05 盛大
せいだん #CN 西淡
せいち #T35*1 生地 #T05*1 精緻 #T35*1 聖地 #T30*1 静置 #T30 整地
せいちゃ #T35 製茶
せいちゅう #T35*2 成虫
せいちょう #T30*8 成長 #T35 整調 正調 #T05 清澄 #T30 生長 静聴
せいつう #T30*1 精通
せいてい #T30*3 制定
せいてき #T05*25 静的 #T05*1 性的
せいてつ #T30*1 製鉄
せいてん #T35*38 晴天 #T35 盛典
せいでんき #T35*1 静電気
せいと #T35*14 生徒 #CN 成都
せいとう #T05*17 正当 #T35*1 政党 正統 #T30 正答 製糖
せいとん #T30*2 整頓
せいど #T35*23 制度 #T35*13 精度
せいどう #T30 制動 #T35 聖堂 青銅
せいどうき #T35 青銅器
せいどく #T30 精読
せいないじ #CN 清内路
せいなる #RT 聖なる
せいにく #T35 精肉
せいねん #T35*2 青年 #T35 成年
せいねんがっぴ #T35*2 生年月日
せいの #JNS 清野
せいのう #T35*75 性能
せいは #T30*2 制覇
せいはく #T30 精白
せいはん #T30 製版
せいはんたい #T05*1 正反対
せいばい #T30 成敗
せいばつ #T30 征伐
せいひ #T35*2 成否 正否 #T35*1 性比 #JCN 西彼
せいひつ #T05 静謐
せいひょう #T30 製氷
せいひん #T35*157 製品 #T05 清貧
せいび #T30*33 整備
せいふ #T35*16 政府 #T35 正負
せいふう #JNS 清風
せいふく #T35*1 制服 #T30 征服
せいふん #T30 製粉
せいぶ #T35*5 西武 #T35*2 西部 #T35 声部
せいぶつ #T35*4 生物 #T35 静物
せいぶん #T35 成分
せいへき #T35 性癖
せいべつ #T35*2 性別
せいほう #T35 正方 製法 西方
せいほうけい #T35*3 正方形
せいほん #T30*4 製本
せいぼ #T35 歳暮 生母
せいぼう #T35 制帽 声望
せいまい #T30 精米
せいみつ #T05*2 精密
せいめい #T35*10 姓名 #T35*2 生命 #T30*1 声明 #T30 清明
せいめん #T30 製麺
せいもん #T35*2 正門
せいやく #T30*36 制約 #T30 成約 #T35 製薬 #T30 誓約
せいゆ #T35 製油
せいゆう #T35*1 声優
せいよう #T30*1 静養
せいよく #T35*8 性欲
せいらい #T35*1 生来
せいり #T30*43 整理 #T35*2 生理
せいりつ #T30*17 成立
せいりゃく #T35 政略
せいりゅう #T35 清流
せいりょう #T05*1 清涼 #T35 声量 #KK 青龍
せいりょく #T35*2 勢力 #T35*1 精力
せいれい #T35*1 精霊 #T35 政令 #T30 精励
せいれき #T35*2 西暦
せいれつ #T30*3 整列
せいれん #T30*1 精錬 #T05 清廉
せいろ #T35 蒸篭
せいろう #CN 聖篭
せいろん #T35*2 正論
せいわ #CN 勢和 清和
せお #W5*2 背負 #JNS 瀬尾
せおりー #T35*1 セオリー
せかい #T35*71 世界
せかせか #F04 せかせか
せかんど #T35 セカンド
せがわ #JNS 瀬河 瀬川
せき #T35*75 席 #T35*16 積 #T35*4 関 #T35*3 咳 #KJ*3 石 責 #T35*2 籍 #KJ*2 赤 #T35*1 堰 責 #KJ*1 蹟 #KJ 威 #JCN 関 #T35 寂 #JS 隻 #T35 隻 #KJ 惜 戚 斥 昔 析 績 脊 跡 碩 夕
せきか #T30 石化
せきかわ #JCN 関川
せきがいせん #T35*1 赤外線
せきがく #T35 碩学
せきがね #JCN 関金
せきがはら #CN 関ヶ原
せきがわ #JNS 関川
せきぐち #JCN 関口
せきこう #T35*1 積項
せきさい #T30 積載
せきさく #T35 脊索
せきさん #T30*2 積算
せきざい #T35*1 石材
せきしゅつ #T30 析出
せきしょく #T05*1 赤色
せきじゅうじ #T35 赤十字
せきじょう #T35*1 席上 #JNS 関上 #JCN 関城
せきずい #T35 脊髄
せきせつ #T30*2 積雪
せきぜん #JCN 関前
せきぞう #T35*1 石像
せきたん #T35*1 石炭
せきつい #T35 脊椎
せきてい #T35*1 石庭
せきとめ #KSr*1 せき止め
せきとり #T35*2 関取
せきどう #T35*1 赤道
せきにん #T35*35 責任
せきにんしゃ #T35 責任者
せきね #JNS 関根
せきねつ #T30 赤熱
せきの #JNS 関野
せきのみや #JCN 関宮
せきはい #T30 惜敗
せきはん #T35 赤飯
せきばん #T35 石版
せきひ #T35 石碑
せきひん #T35 赤貧
せきぶん #T30*2 積分
せきべつ #T30 惜別
せきむ #T35*1 責務
せきめ #JCN 関目
せきめん #T30*2 赤面
せきもと #JCN 関本
せきや #JCN 関屋 #JNS 関谷
せきやど #JCN 関宿
せきゅりてぃ #T35*14 セキュリティ
せきゆ #T35*4 石油
せきら #T05 赤裸
せきらら #T05 赤裸々
せきり #T35*1 赤痢
せきりょく #T35 斥力
せきわけ #T35*1 関脇
せくしょん #T35*12 セクション #T35*1 §
せくはら #T30 セクハラ
せぐち #JNS 瀬口
せぐめんてーしょん #T35*2 セグメンテーション
せぐめんと #T35*19 セグメント
せけん #T35*8 世間
せこう #T30*1 施工
せこむ #KK*1 セコム
せざんぬ #JN セザンヌ
せしゅう #T30 世襲
せしょん #T35*33 セション
せじ #T35*2 世辞
せじょう #T30*2 施錠 #T35 世情
せすじ #T35*2 背筋
せそう #T35 世相
せぞく #T35 世俗
せた #JCN 瀬田 #JNS 勢多
せたい #JS*1 世帯 #T35*1 世帯
せたか #JCN 瀬高
せたがや #JCN 世田谷
せたけ #T35 背丈
せたな #JCN 瀬棚
せだい #JS*11 世代 #T35*10 世代
せちばる #JCN 世知原
せっ #SX*2 接
せっかい #T35*1 石灰 #T30 切開
せっかく #T36*16 せっかく #T36*6 折角
せっかち #T05*1 せっかち
せっかん #T30 折檻
せっき #T35*4 節気 #T35 石器
せっきゃく #T30 接客
せっきょう #T30*1 説教
せっきょく #T35*19 積極
せっきん #T30*1 接近
せっく #T35 節句
せっくす #T30*1 セックス
せっけい #T30*234 設計
せっけいし #T35 設計士
せっけいず #T35 設計図
せっけっきゅう #T35 赤血球
せっけん #T35*1 石鹸 #T30 接見
せっこう #T35*1 石膏 #T30 斥候 #T35 拙稿
せっこつ #T30 接骨
せっさく #T30 切削 #T35 拙作
せっさたくま #T30 切磋琢磨
せっし #T35 摂氏 #T30 窃視
せっしゃ #T35 拙者
せっしやくわん #T30 切歯扼腕
せっしゅ #T30*1 摂取 #T30 接種
せっしゅう #T30 接収
せっしょう #T30*3 折衝 #T10 殺生 #T35 摂政
せっしょく #T30*13 接触 #T30 節食
せっしょん #T35*17 セッション
せっすい #T30 節水
せっせ #F14*1 せっせ
せっせい #T30 摂生 節制
せっせん #T35*2 接線 #T30 接戦
せっそう #T35*2 節操
せっそく #T05*1 拙速 #T35 節足
せったい #T30*1 接待
せったく #T35*1 拙宅
せっち #T30*22 設置 #T30*1 接地
せっちゃく #T30 接着
せっちゅう #T30 折衷
せっちょ #T35*1 拙著
せっちん #T35 雪隠
せっつ #K5r せっつ #JCN 摂津
せってい #T30*255 設定
せってん #T35*2 接点 #T35*1 節点
せっと #T30*255 セット
せっとう #T30*6 接頭 #T30 窃盗
せっとうご #T35 接頭語
せっとく #T30*5 説得
せっぱく #T30*1 切迫
せっぱつま #R5 切羽詰ま
せっぱん #T30 折半
せっぴ #T30*1 切皮
せっぷく #T30 切腹
せっぷん #T30*2 接吻
せっぺん #T35*1 切片
せつ #T35*79 節 #JNSUC*27 説 #KJ*18 接 #KJ*7 雪 #T05*2 切 #KJ*1 拙 摂 設 #T35*1 説 #JNM せつ セツ #KJ 折 窃 #JS 節 #SUC 説 #KJ 鱈
せつえい #T30*1 設営
せつお #JNM 節男 節夫
せつがん #T30 接岸
せつげん #T35*1 雪原 #T30 節減
せつこ #JNM 節子
せつごう #T30*7 接合
せつじ #T35*1 接辞
せつじつ #T05*1 切実
せつじゅ #T30 接受
せつじょ #T30 切除
せつじょく #T30 雪辱
せつせつ #F00 切々
せつぜい #T30*1 節税
せつぞく #T30*255 接続
せつぞくし #T35*7 接続詞
せつだん #T30*5 切断
せつでん #T30*1 節電
せつど #T35*1 節度
せつどう #T30 摂動
せつな #KY*1 切な #T35*1 刹那
せつのう #KYU*1 切のう
せつび #T30*40 接尾 #T30*16 設備
せつびご #T35 接尾語
せつぶん #T35*1 節分
せつへいめん #T35*2 接平面
せつぼう #T30*1 切望
せつめい #T30*255 説明
せつめいかい #T35 説明会
せつめいしょ #T35 説明書
せつもん #T30 設問
せつやく #T30*14 節約
せつり #T35 摂理
せつりつ #T30*8 設立
せつわ #T35 説話
せと #T35*10 瀬戸 #JCN 瀬戸
せとうち #JCN 瀬戸内
せとぎわ #T35*1 瀬戸際
せとだ #CN 瀬戸田
せとないかい #CN 瀬戸内海
せともの #T35 瀬戸物
せなか #T35*3 背中
せのび #T30 背伸び
せばま #R5*1 狭ま
せばめ #KS 狭め
せびろ #T35*1 背広
せぴあ #T35*1 セピア
せふぃーろ #KK*1 セフィーロ
せふり #CN 背振
せぼね #T35*1 背骨
せま #KY*34 狭 #R5*9 迫
せまくるし #KY 狭苦し
せまくるしゅう #KYU 狭苦しゅう
せまふぉ #T35*2 セマフォ
せまふぉあ #T35 セマフォア
せまんてぃくす #T35*3 セマンティクス
せまんてぃっく #T35*1 セマンティック
せまんてぃっくす #T35*1 セマンティックス
せみ #T35*1 蝉
せみころん #T35*2 セミコロン
せみこんだくた #T35*1 セミコンダクタ
せみな #T35*4 セミナ
せみなー #T35*26 セミナー
せみね #JCN 瀬峰
せめ #KSr*9 攻め #KSr*4 責め #T35 責
せめおと #S5*1 攻め落と
せめこ #M5r*1 攻め込
せめたて #KSr 責めたて
せめつけ #KSr 責めつけ
せめて #F14*3 せめて #T36 せめて
せめてもの #RT せめてもの
せめんと #T35 セメント
せもう #KYU*2 狭う
せや #JCN 瀬谷
せら #JCN 世羅
せらにし #CN 世羅西
せらみっく #T35*2 セラミック
せり #T35*39 芹
せりあ #W5r 競り合
せりあげ #KSr せり上げ
せりざわ #JNS 芹沢 芹澤
せりふ #T35*3 セリフ #T35*1 台詞
せる #T35*5 セル
せるふちぇっきんぐ #T30*10 セルフチェッキング
せれくた #T35*6 セレクタ
せれくと #T30*2 セレクト
せれなーど #T35*1 セレナード
せろん #T35*1 世論
せわ #T30*4 世話
せわしな #KY せわしな
せわしのう #KYU せわしのう
せん #CNSUC1*121 線 #NN*35 千 #T35*33 線 #KJ*14 先 #T35*9 栓 #KJ*7 戦 #S5*6 宣 #T30*6 宣 #KJ*5 船 遷 #KJ*3 泉 煎 #KJ*2 洗 #KJ*1 釧 専 #T35*1 選 #KJ*1 鮮 僭 #KJ 蝉 仙 占 尖 川 #SUC 戦 #KJ 扇 #T30 撰 #KJ 栴 浅 染 潜 煽 旋 穿 箭 #SUC 線 #KJ 繊 羨 腺 舛 #SUC 船 #KJ 薦 詮 賎 践 #JS 選 銭 #KJ 銭 銑 閃 揃 苫 亘
せんい #T30*54 遷移 #T35*4 繊維 #T35*1 線維
せんいそ #T35*1 線維素
せんいん #T35*1 船員
せんえき #T35*2 戦役
せんえつ #T05*1 僭越
せんか #T35*1 戦果 #T35 専科 戦火 戦禍
せんかい #T30*1 旋回
せんかく #T35 先覚
せんかん #T35*1 戦艦
せんが #T35*1 線画
せんがい #T35 選外
せんがく #T35 先学 #T05 浅学
せんがん #T30*1 洗顔
せんき #T35*1 戦記
せんきゃく #T35 船客
せんきゃくばんらい #T35*1 千客万来
せんきょ #T30*2 占拠 #T30*1 選挙
せんきょうし #T35 宣教師
せんきょく #T30*2 選曲 #T35 戦局
せんぎ #T30 詮議
せんぎょ #T35 鮮魚
せんぎょう #T35 専業
せんく #T30*2 先駆
せんぐう #T30 遷宮
せんけい #T35*4 線形
せんけつ #T30*3 先決
せんけん #T35*1 先見
せんげつ #T35*15 先月
せんげん #T30*69 宣言 #T35*2 選言
せんこう #T30*5 先行 #T30*4 先攻 専攻 #T30*2 選考 #T30 穿孔 #T35 線香 閃光
せんこく #T35*1 先刻 #T30*1 宣告
せんこん #T35 戦棍
せんご #T35*1 戦後
せんごく #T35*1 戦国
せんさ #T35*8 センサ
せんさー #T35*2 センサー
せんさい #T05*2 繊細 #T35 戦災
せんさく #T30*1 詮索
せんさばんべつ #T05*1 千差万別
せんざい #T30*8 潜在 #T35*2 洗剤 #T35 千載
せんざいいちぐう #T35 千載一遇
せんし #T35*3 戦士 #T30*1 戦死
せんしきめん #T35*4 線織面
せんしつ #T35 船室
せんしゃ #T35*2 戦車
せんしゅ #T35*4 選手 #T35*1 船首 #T30 先取
せんしゅう #T35*33 先週 #T30*1 専修 #T35*1 選集 #CN 泉州
せんしゅうらく #T35 千秋楽
せんしゅつ #T30*2 選出
せんしょう #T30 先勝 #T35 僭称
せんしょく #T35 染織 #T30 染色
せんしょくたい #T35 染色体
せんしん #T35*1 先進 千進 #T30 専心
せんしんこく #T35*1 先進国
せんじ #KS*1 煎じ #T35 戦時
せんじつ #T35*112 先日
せんじゃ #T35 選者
せんじゅう #T30 専従
せんじゅつ #T35*1 仙術 #T30*1 先述 #T35*1 戦術
せんじょう #T30*1 洗浄 #T35*1 船上 #T35 戦場 扇状
せんじん #T35*2 先人 #T35*1 千尋
せんす #T35*11 センス #T35*1 扇子
せんすい #T30*1 潜水
せんすいかん #T35*1 潜水艦
せんず #T35 先途
せんせーしょなる #T05 センセーショナル
せんせーしょん #T35 センセーション
せんせい #T35*143 先生 #T30*2 宣誓
せんせいじゅつ #T35*1 占星術
せんせん #T30 宣戦 #T35 戦線
せんせんしゅう #T35 先々週
せんぜん #T35*1 戦前
せんそう #T30*4 戦争
せんそく #T35*1 栓塞
せんぞ #T35*2 先祖
せんぞがえり #T30 先祖返り
せんぞく #T35 専属
せんた #T35*55 センタ
せんたー #T35*60 センター
せんたい #T35*2 船体
せんたく #T30*255 選択 #T30*14 洗濯
せんたくき #T35*5 洗濯機
せんたくし #T35*14 選択肢 #T35*1 選択子
せんたん #T35*2 先端 #T35 尖端
せんだい #T35*1 先代 #CN 仙台 川内
せんだがや #CN 千駄ヶ谷
せんだぎ #CN 千駄木
せんち #JS*5 センチ #T05*2 センチ #T35 戦地
せんちめーとる #JS センチメートル 糎
せんちゃ #T35*1 煎茶
せんちゃく #T30*2 先着
せんちょう #CN 千丁 #T35 船長
せんつう #T35*1 疝痛
せんて #T35*6 先手
せんてい #T30*13 選定
せんてん #T35 先天
せんでき #T35 洗滌
せんでん #T30*10 宣伝
せんと #T35*1 ¢ #T35 セント #T30 遷都
せんとう #T35*86 先頭 #T30*4 戦闘 #T35 尖塔 銭湯
せんとうき #T35 戦闘機
せんど #T35 センド 鮮度
せんどう #T30*1 先導 #T35*1 船頭
せんない #T35 船内
せんなん #CN 仙南 泉南
せんにちまえ #CN 千日前
せんにゅう #T30 潜入
せんにゅうかん #T35*1 先入観
せんにん #T30*2 専任 選任 #T35*1 仙人
せんねん #T30*5 専念
せんのう #T30*2 洗脳
せんはた #JCN 千畑
せんば #T35*1 船場
せんばい #T30*1 専売
せんばつ #T30 選抜
せんばづる #T35 千羽鶴
せんばん #T05 千万 #T35 旋盤
せんぱい #T35*27 先輩
せんぱく #T35*1 船舶
せんぱつ #T30*2 洗髪 #T30*1 先発
せんぱん #T35*2 先般
せんび #T35*1 船尾
せんびん #T35 船便
せんぴょう #T35*14 線表 #T30*1 選評
せんぶん #T35*16 線分
せんぷ #T35 先負 戦斧
せんぷう #T35*2 旋風
せんぷうき #T35*1 扇風機
せんぷく #T30 潜伏
せんべい #T35 煎餅
せんべつ #T30*2 選別 #T35 餞別
せんべん #T35 先鞭
せんぺい #T35*2 尖兵
せんほく #CN 泉北
せんぼう #T30*1 羨望
せんぼうきょう #T35*1 潜望鏡
せんぼく #CN 仙北 泉北
せんぼつ #T30 戦没
せんぽう #T35*2 先方 #T35 戦法 旋法
せんぽく #CN 泉北
せんまや #JCN 千厩
せんみん #T35 賎民 選民
せんむ #T35*3 専務
せんめい #T05*2 鮮明
せんめつ #T35 殲滅
せんめん #T30*2 洗面
せんもう #T35*1 剪毛 #T35 繊毛 譫妄
せんもん #T35*32 専門
せんもんか #T35*6 専門家
せんやく #T35*1 先約
せんゆう #T30*8 占有 #T30*1 専有 #T35*1 戦友
せんよう #T10*71 専用
せんり #T35*1 千里 戦利
せんりがん #T35 千里眼
せんりつ #T30*1 戦慄 #T35*1 旋律
せんりゃく #T35*30 戦略
せんりゅう #T35*1 川柳
せんりょう #T35*29 線量 #T30*1 占領 #T35 染料
せんりょく #T35 戦力
せんれい #T35*1 先例 #T30*1 洗礼
せんれつ #T05 鮮烈
せんれん #T30*3 洗練
せんろ #T35*2 線路
ぜ #T35*9 是
ぜい #SUC*4 税 #KJ*2 脆 #T35 税
ぜいかん #T35*1 税関
ぜいがく #T35 税額
ぜいきん #T35*5 税金
ぜいげん #T35 税源
ぜいこみ #T35 税込み
ぜいしゅう #T35*1 税収
ぜいじゃく #T35 脆弱
ぜいせい #T35*1 税制
ぜいたく #T00*5 贅沢 #T00*3 ぜいたく
ぜいにく #T35*1 ぜい肉
ぜいほう #T35 税法
ぜいむ #T35 税務
ぜいむしょ #T35 税務署
ぜいめい #T35*1 喘鳴
ぜいりし #T35 税理士
ぜいりつ #T35 税率
ぜげん #T35 女衒
ぜせい #T30*1 是正
ぜぜ #JCN 膳所
ぜっ #SX*4 絶
ぜっきょう #T30 絶叫
ぜっく #T30 絶句
ぜっけい #T35*1 絶景
ぜっこう #T05*1 絶好 #T30 絶交
ぜっこうちょう #T05*1 絶好調
ぜっさん #T30 絶賛
ぜっしょう #T30 絶唱
ぜっしょく #T30 絶食
ぜっせい #T35 絶世
ぜったい #T35*83 絶対
ぜったいぜつめい #T15 絶体絶命
ぜったいち #T35*3 絶対値
ぜっちょう #T35 絶頂
ぜっぱん #T30*1 絶版
ぜっぴつ #T30*1 絶筆
ぜっぴん #T05*1 絶品
ぜっぺき #T35 絶壁
ぜっぽう #T35 舌鋒
ぜつ #KJ*3 絶 #KJ*1 舌
ぜつえん #T30*1 絶縁
ぜつだい #T05*2 絶大
ぜつぼう #T30*5 絶望
ぜつみょう #T05*4 絶妙
ぜつむ #T15 絶無
ぜつめい #T30*1 絶命
ぜつめつ #T30*1 絶滅
ぜに #T35*2 銭
ぜにん #T30*1 是認
ぜねすと #T30 ゼネスト
ぜひ #T35*86 是非 #T35*11 ぜひ
ぜみ #T35*13 ゼミ
ぜみなーる #T35*4 ゼミナール
ぜろ #T35*18 ゼロ #T35*4 零 #T35*3 〇
ぜろっくす #KK*5 ゼロックス
ぜん #T35*70 全 #T35*47 前 #T35*25 善 #JS*3 膳 #T35*2 千 膳 #KJ*1 喘 #KJ 蝉 #PRE 前 #KJ 漸 然 #SUC 然 #PRE 全 #KJ 禅 #T35 禅 #KJ 繕
ぜんあく #T35*1 善悪
ぜんい #T35*1 善意
ぜんいき #T35*3 全域
ぜんいち #JNM 善一
ぜんいん #T35*50 全員
ぜんえい #T35*1 前衛
ぜんか #T35 前科
ぜんかい #T35*36 前回 #T30*1 全開 #T35 全会 #T30 全壊 全快
ぜんかく #T35*60 全角 #T35*1 前核
ぜんかくき #T35*1 前核期
ぜんかしき #T35*1 漸化式
ぜんかん #T35 全巻 全館
ぜんがく #T35*1 全学 全額
ぜんき #T35*3 前期 #T35*1 前記
ぜんきち #JNM 善吉
ぜんきょうれん #KK*3 全共連
ぜんきょく #T35*6 全曲 #T35 全局
ぜんきん #T30*2 漸近
ぜんく #T30 前駆 #T35 全区
ぜんけい #T30 前傾 前掲 #T35 全形 全景
ぜんけつ #T35 全血
ぜんけん #T30 全検 #T35 全権 全県
ぜんげつ #T35*2 前月
ぜんげん #T30*1 漸減 #T35 前言
ぜんこ #T35 全戸
ぜんこう #T35*1 善行 #T35 前項 全校
ぜんこく #T35*28 全国
ぜんご #T30*19 前後
ぜんごさく #T35 善後策
ぜんさい #T35*1 前菜
ぜんさく #T35*1 前作
ぜんし #T35*1 前肢 #T35 全市 全紙
ぜんしつ #T35*1 前室
ぜんしゃ #T35*15 前者 #T35*3 全社 #T35*2 全射
ぜんしゅう #T35*2 前週 #T35 全集 禅宗
ぜんしょ #T30*2 善処 #T35 全書
ぜんしょう #T35*48 前章 #T30*1 全勝 全焼
ぜんしょうせん #T35 前哨戦
ぜんしろう #JNM 善四郎
ぜんしん #T30*2 前進 #T35*1 前身 全身 #T30 漸進
ぜんじ #T35*1 漸時 漸次
ぜんじつ #T35*9 前日 #T35 全日
ぜんじゅつ #T30*5 前述
ぜんじろう #JNM 善次郎
ぜんせいき #T35*1 全盛期
ぜんせき #T35*1 全席
ぜんせん #T35*2 全線 #T35 前線
ぜんぜん #F14*119 全然
ぜんぜんしつ #T35*1 前々室
ぜんそう #T35*1 前奏
ぜんそく #T35*1 喘息
ぜんぞう #T30 漸増
ぜんたい #T35*154 全体
ぜんたろう #JNM 善太郎
ぜんたん #T35*4 前端
ぜんだい #T35*1 前代
ぜんだま #T35 善玉
ぜんち #T30*3 前置 #T30 全治
ぜんちし #T35*4 前置詞
ぜんちょ #T35 前著
ぜんちょう #T35*8 前兆 #T35*1 全長
ぜんつう #T30 全通
ぜんつうじ #CN 善通寺
ぜんてい #T35*30 前提
ぜんてん #T35 全店
ぜんでら #T35 禅寺
ぜんと #T35*1 前途
ぜんとう #T35 全島
ぜんとうよう #T35 前頭葉
ぜんど #T35*1 全土
ぜんどう #T30*1 蠕動 #T30 善導
ぜんにちせい #T35 全日制
ぜんにっくう #KK*2 全日空
ぜんにほん #T35*2 全日本
ぜんにん #T35 前任 善人
ぜんねんど #T35*1 前年度
ぜんのう #T30 前納
ぜんはん #T35*4 前半
ぜんぱい #T30*1 全廃 #T30 全敗
ぜんぱん #T35*18 全般 #T35 前半
ぜんびん #T35 全便
ぜんぴん #T35 全品
ぜんぶ #T35*138 全部
ぜんぶん #T35*5 全文 #T35 前文
ぜんぺん #T35*1 前編 全編
ぜんぼう #T35*1 全貌
ぜんぽう #T35*2 前方
ぜんめつ #T30*6 全滅
ぜんめん #T35*4 前面 全面
ぜんや #T35*1 前夜
ぜんやく #T30*1 全訳
ぜんよう #T30 善用 #T35 全容
ぜんら #T35 全裸
ぜんりゃく #T35*4 前略
ぜんりょう #T05*1 善良
ぜんりょく #T35*1 全力
ぜんれい #T35*2 前例
ぜんれき #T35*1 前歴
ぜんれつ #T35 前列
そ #T35*195 祖 #U5*153 沿 #W5*153 沿 #KJ*33 疎 #R5r*8 剃 #KJ*7 素 #KJ*4 租 #U5*3 添 #W5*3 添 #T05*2 素 #KJ*1 曽 粗 蘇 #KJ 姐 酢 噌 塑 岨 措 曾 楚 狙 疏 礎 組 訴 阻 遡 鼠 #R5r 反
そーす #T35*255 ソース
そーせーじ #T35 ソーセージ
そーと #T30*24 ソート
そーぷ #T35*1 ソープ
そあく #T05 粗悪
そいつ #T35*2 そいつ
そいん #T35 素因 訴因
そいんし #T35 素因子
そいんすう #T35 素因数
そう #T30*196 そう #PRE*42 総 #T35*41 総 #T30*18 相 #T35*15 層 #KJ*10 装 #KJ*8 送 #KJ*5 早 #T35*4 荘 #KJ*3 双 倉 #KJ*2 創 #S5*2 奏 #T30*2 奏 #KJ*2 想 曹 走 #KJ*1 曽 挿 操 槍 聡 #T30*1 草 #KJ*1 騒 聰 躁 輳 #KJ 鯵 繰 桑 #T35 宗 #KJ 庄 曾 #T35 僧 #KJ 叢 喪 壮 爽 宋 #SUC 層 #KJ 匝 惣 捜 掃 掻 巣 #SUC 槽 #T35 槽 #KJ 漕 燥 争 痩 窓 糟 綜 #SUC 荘 #KJ 葬 蒼 藻 遭 鎗 霜 贈 爪 葱 蚤 湊
そうあい #T35 相愛
そうあん #T35*4 草案
そうい #T30*9 相違 #T35*1 創意
そういう #RT*71 そういう
そういち #JNM 宗一
そういちろう #JNM 総一朗
そういん #T35 総員
そううつ #T05*1 躁鬱
そうおう #T05*1 相応
そうおん #T35*1 騒音
そうか #JCN 草加
そうかい #T35*4 総会 #T05*2 壮快 #T05 爽快 #T30 掃海
そうかつ #T30*18 総括 #T30 総轄
そうかん #T05*4 壮観 #T35*4 層間 #T30*2 相関 #T30 創刊 相姦 #T35 総監 #T30 送還
そうが #T35 挿画
そうがく #T35*1 総額
そうがんきょう #T35 双眼鏡
そうき #T35*10 早期 #T30 想起
そうきゅう #T05*20 早急
そうきょ #T35 壮挙
そうきょく #T35*1 総局
そうきょくし #T35 双極子
そうきょくせん #T35*2 双曲線
そうきん #T30*2 送金
そうぎ #T30 争議 #T35 葬儀
そうぎょう #T30*1 創業 #T30 操業
そうぐう #T30*2 遭遇
そうけ #T35 宗家
そうけい #KK 早慶 #T30 総計
そうけん #T05*1 壮健 #T30 送検
そうげい #T30*3 送迎
そうげん #T35*8 草原
そうこ #T35*2 倉庫
そうこう #T30*12 走行 #T30*5 壮行 #T35*1 草稿 霜降 #T35 操行 糟糠
そうご #T35*69 相互
そうごう #T30*33 総合 #T35 相好
そうごん #T05*1 荘厳
そうさ #T30*255 操作 #SX*2 走査 #T30*1 捜査 #CN 匝瑳
そうさい #T35*1 総裁 #T30 相殺
そうさく #T30*2 捜索 #T30*1 創作
そうざい #T35 惣菜
そうし #T30*1 創始 #T35 相思 草紙
そうしき #T35*1 葬式
そうしたら #CJ そうしたら
そうしつ #T30*1 喪失
そうして #CJ そうして
そうしゃ #T30*1 操車 #T35 奏者 走者
そうしゅうへん #T35 総集編
そうしゅつ #T30*10 送出
そうしゅん #T35*1 早春
そうしょ #T35*2 草書
そうしょう #T30*5 総称 #T30 宗匠
そうしょく #T30*10 装飾 #T30 草食
そうしん #T30*65 送信 #T30 痩身 装身
そうじ #T30*23 掃除 #T30 相似
そうじしょく #T30*1 総辞職
そうじつはい #T35*1 桑実胚
そうじて #F14 総じて
そうじゃ #CN 総社
そうじゅ #T30*2 送受
そうじゅう #T30*1 操縦
そうじゅく #T05 早熟
そうじゅしん #T30*5 送受信
そうじょう #T35*1 僧上 #T30*1 相乗 #T30 騒擾
そうすう #T35*5 総数
そうず #T35 挿図
そうせい #T30*1 早世 #T30 創製 叢生 早逝
そうせいじ #T35 双生児
そうせき #JNM 漱石
そうせつ #T30*2 創設
そうぜい #T35*1 総勢
そうぜつ #T05*1 壮絶
そうぜん #F00 騒然
そうそう #T35*41 早々 #T35*2 草々 #F02 錚々
そうぞう #T30*20 想像 #T30*2 創造 #T30*1 送像
そうぞうし #KY 騒々し
そうぞうしゅう #KYU 騒々しゅう
そうぞく #T30*2 相続
そうたい #T35*28 相対 #T30*22 早退 #T35 総体
そうたいち #T35 相対値
そうたいてき #T15 相対的
そうたつ #T30*1 送達
そうだ #T30 操舵
そうだい #T05*1 壮大 #KK*1 早大 #T35 総代
そうだか #T35 総高
そうだち #T35 総立ち
そうだつ #T30 争奪
そうだん #T30*57 相談
そうだんしょ #T35 相談所
そうち #T30*74 装置 #T30 送致
そうちゃく #T30*1 装着
そうちょう #T35*7 早朝 #T35 総長 #T05 荘重
そうつい #T05*1 双対
そうてい #T30*17 想定 #T30 装丁 送呈
そうてん #T35*1 争点
そうで #T35 総出
そうですが #CJ そうですが
そうでん #T30*1 送電
そうと #T35 壮図 壮途
そうとう #T00*27 相当 #T35*1 双頭 #T30 掃討 #T35 総統
そうとく #T35 総督
そうどう #T35*1 相同 #T35 騒動
そうなん #T30*1 遭難
そうにゅう #T30*102 挿入
そうねん #T35 壮年
そうは #T30 掻爬 走破
そうはい #T30 送配
そうはん #T30*1 相反
そうば #T35*2 相場
そうばん #T35*1 早晩
そうひょう #T35*1 総評
そうび #T30*3 装備
そうびょう #T35 躁病
そうふ #T30*121 送付
そうふう #T30 送風
そうべつ #T30*23 送別 #CN 壮瞥
そうほ #T30*2 相補
そうほう #T35*3 双方
そうほうこう #T35*11 双方向
そうほさ #T35*1 相補鎖
そうま #JCN 相馬
そうむ #T35*11 総務 #KK*10 創夢
そうめい #T05*1 聰明 #T05 聡明
そうめん #T35*4 素麺
そうや #JCN 宗谷
そうよう #T35*1 掻痒 #T30*1 装用
そうらく #CN 相楽
そうらん #T30 総覧 #T35 騒乱
そうり #T35*2 総理
そうりつ #T30 創立
そうりふ #T35*1 総理府
そうりょ #T35*1 僧侶
そうりょう #T35*4 送料 #T35*1 総量 #JCN 総領
そうりょく #T35*1 総力
そうる #CN ソウル
そうるい #T30 走塁
そうれつ #T05 壮烈
そうろう #T35*2 候
そうろん #T30*1 総論
そうわ #T35*2 総和 #T35*1 挿話 #CN 総和
そえ #KSr*11 そえ #KSr*3 添え
そえかみ #CN 添上
そえじ #T35*10 添字
そえだ #JCN 添田
そえん #T05 疎遠
そお #CN 曾於 曽於
そかい #T30*2 疎開
そかく #T30 疎隔 組閣
そが #JNS 曾我 曽我 #JCN 蘇我
そがい #T30*4 阻害 #T30*2 疎外
そきゅう #T30 遡及
そく #T30*47 即 #KJ*4 側 #S5*4 則 #T35*4 則 #KJ*2 捉 速 #KJ*1 則 #T35*1 息 束 #KJ*1 測 #JS*1 足 #T35*1 足 #KJ 塞 促 #JS 束
そくい #T30 即位
そくおう #T30 即応
そくおん #T35 促音
そくさ #T35*1 側鎖
そくざ #T15*5 即座
そくし #T30 即死
そくしん #T30*5 促進
そくじ #T35*2 即時
そくせい #T30*1 促成
そくせき #T35 即席 足跡
そくたい #T35 束帯
そくたつ #T35*1 速達
そくだく #T30 即諾
そくだん #T30*1 即断
そくち #T35*2 即値 #T30*1 測地
そくてい #T30*30 測定
そくとう #T30*1 即答 #T30 速答
そくとうよう #T35 側頭葉
そくど #T35*52 速度 #T35*2 測度
そくはつ #T30 即発
そくばい #T30*1 即売
そくばく #T30*13 束縛
そくぶつてき #T15 即物的
そくほう #T30 速報
そくめん #T35*5 側面
そくらてす #JN ソクラテス
そくりょう #T30 測量
そくりょく #T35 速力
そくろう #T35*1 足労
そぐ #W5*2 そぐ
そけっと #T35*17 ソケット
そげき #T30 狙撃
そこ #T35*117 そこ #T35*29 底
そこう #T35 素行
そこく #T35 祖国
そこそこ #T35*1 そこそこ
そこぢから #T35 底力
そこで #CJ*29 そこで
そこな #W5*3 損な
そこね #KSr*6 損ね #T35 底値
そこはかとな #KY そこはかとな
そこはかとのう #KYU そこはかとのう
そこびえ #T30 底冷え
そこびき #T35 底引き
そこら #T35*4 そこら
そご #T30 齟齬
そごう #KK*2 そごう
そさい #T35 蔬菜
そざい #T35*4 素材
そざつ #T05 粗雑
そし #T35*14 素子 #T30*2 阻止 #R5r*1 誹
そしき #T30*119 組織
そしつ #T35*1 素質
そして #CJ*51 そして
そしな #T35*1 粗品
そしゃく #T30*1 咀嚼 #T30 租借
そしょう #T30*1 訴訟
そしょく #T30 粗食
そじ #T35 素地
そじょう #T35 訴状 俎上 #T30 溯上
そすい #T35 疎水
そすう #T35 素数
そせい #T35 粗製 #T30 組成 蘇生
そせん #T35*1 祖先
そそ #G5*3 注 #G5*1 そそ #F00*1 楚々
そそう #T30 粗相
そそぎぐち #T35 注ぎ口
そそのか #S5 唆
そそりた #T5 そそり立
そぞろ #T16 そぞろ
そだ #T5r*3 育
そだい #T05*2 粗大
そだて #KSr*11 育て
そだてあげ #KSr 育て上げ
そち #T30*4 措置
そちら #T35*101 そちら
そっき #T30 速記
そっきゅう #T05*1 速急
そっきょう #T35 即興
そっきん #T35 側近 即金
そっくり #T06*2 そっくり
そっけつ #T30 即決
そっけな #KY*1 そっけな
そっけのう #KYU*1 そっけのう
そっこう #T30*2 速攻 #T35*1 即効
そっこうじょ #T35 測候所
そっこく #T35 即刻
そっせん #T30 率先
そっち #T35*10 そっち
そっちのけ #T15 そっちのけ
そっちゅう #T35 卒中
そっちょく #T05*1 率直
そっと #F14*2 そっと
そっとう #T30 卒倒
そっぷがた #T15 ソップ形
そっぽ #T35 そっぽ
そつ #T35*14 卒 #KJ 率
そつう #T30*1 疎通
そつぎょう #T30*14 卒業
そつぎょうしき #T35 卒業式
そつじゅ #T35 卒寿
そつろん #T35*22 卒論
そてー #T35 ソテー
そてい #T30 措定
そで #T35*1 袖
そでがうら #CN 袖ヶ浦
そと #T35*67 外
そとがわ #T35*5 外側
そとづら #T35 外面
そとまわり #T35*1 外回り
そとめ #CN 外海
そなー #T35 ソナー
そなえ #KSr*10 備え #KSr*1 供え
そなえつけ #KSr 備え付け
そなた #T35*1 ソナタ
そなわ #R5r*2 備わ
そに #CN 曾爾
そにー #KK*11 ソニー
そね #M5r そね 嫉 #JNS 曾根 #JCN 曽根
そねざき #JCN 曽根崎
その #RT*255 その #T35*2 園 #RT*1 其 #KJ 苑 薗
そのうち #F14*26 そのうち
そのぎ #CN 彼杵
そのこと #T35*6 そのこと
そのた #T35*217 その他
そのため #F14*63 そのため #T36*57 そのため
そのだ #JNS 園田
そのべ #JCN 園部
そのまま #T35*78 そのまま #F14*62 そのまま
そのもの #T35*12 そのもの
そのよう #T05*33 そのよう #T05*6 その様
そば #T35*33 そば #T35*1 蕎麦 側
そばこ #T35 そば粉 蕎麦粉
そびえた #T5 そびえ立
そびえと #CN ソビエト
そびょう #T30 素描
そびれ #KS そびれ
そふ #T35*4 祖父
そふぃすてぃけーと #T30 ソフィスティケート
そふぃすと #T35 ソフィスト
そふと #T05*95 ソフト
そふとうぇあ #T35*200 ソフトウェア
そふとうえあ #T35*12 ソフトウエア
そふとばんく #KK*1 ソフトバンク
そふぼ #T35*1 祖父母
そぶえ #JCN 祖父江
そへん #T35*2 素片
そほう #T05 粗放
そぼ #T35*2 祖母
そぼう #T05*1 粗暴
そぼく #T05*4 素朴
そぼろ #T35 そぼろ
そま #R5*1 染ま
そまつ #T05*4 粗末
そむ #K5*1 背
そめ #KS*3 初め 染め
そめだ #S5r 染め出
そめつけ #KSr 染め付け
そめもの #T30 染物
そめや #JNS 染谷
そもそも #F14*5 そもそも #T35*4 そもそも
そや #T05 粗野
そよ #G5r*1 そよ
そよう #T35*1 素養 #CN 蘇陽
そよかぜ #T35*1 そよ風
そら #T35*26 空 #S5*1 逸 #S5 逸ら 反ら
そらおそろし #KY そら恐ろし
そらおそろしゅう #KYU そら恐ろしゅう
そらち #CN 空知
そらなみだ #T35 空涙
そりかえ #R5r 反り返
そりっど #T35*4 ソリッド
そりゃく #T05 粗略
そりゅうし #T35*1 素粒子
そる #KK*2 ソル
それ #T35*243 それ #KSr*80 それ #KSr*3 逸れ
それぞれ #T35*86 それぞれ #T35*16 各々
それだけ #T35 それだけ
それほど #F14*18 それほど #T36 それほど
それら #T35 それら
それらの #RT*19 それらの
それん #CN ソ連
そろ #W5r*25 揃
そろえ #KSr*1 そろえ 揃え
そろそろ #F14*28 そろそろ
そん #CNSUC1*28 村 #T10*10 損 #ZX*8 損 #KJ*2 尊 #T30*1 存 #KJ*1 孫 遜 邨 #KJ 噂 損 #T35 村
そんえい #T35 村営
そんえき #T35 損益
そんかい #T30 損壊
そんがい #T30*1 損害
そんぐ #T35*1 ソング
そんけい #T30*2 尊敬
そんげん #T05*1 尊厳
そんざい #T30*255 存在
そんしつ #T30*1 損失
そんしょう #T30*3 損傷 #T30 尊称
そんしょく #T35*1 遜色
そんぞく #T30*1 存続 #T35 尊属
そんたく #T30 忖度
そんだい #T05*1 尊大
そんち #T30 存置
そんちょう #T30*6 尊重 #T35 村長
そんとく #T35 損得
そんな #RT*74 そんな #T05*65 そんな
そんぼう #T35*1 存亡
そんみん #T35*1 村民
そんもう #T30 損耗
そんらく #T35 村落
そんりつ #T30*1 存立 #T35 村立
ぞい #SUC*7 沿い
ぞう #T30*49 蔵 #T35*12 像 #T35*11 象 増 #KJ*4 噌 #KJ*1 憎 造 #KJ 雑 臓 贈
ぞういん #T30*1 増員
ぞうえい #T30 造営
ぞうえき #T30 増益
ぞうえん #T30 増援 造園
ぞうお #T30*1 憎悪
ぞうか #T30*51 増加 #T35 造花
ぞうかん #T30*1 増刊
ぞうがく #T30 増額
ぞうき #T35 雑木 臓器
ぞうきばやし #T35 雑木林
ぞうきょう #T30*6 増強
ぞうきん #T35 ぞうきん 雑布
ぞうけい #T35*2 造詣 #T30 造形
ぞうけつ #T30 増結
ぞうげ #T35*1 象牙
ぞうげん #T30*1 増減
ぞうご #T30*1 造語
ぞうごん #T30 雑言
ぞうさ #T35 造作
ぞうさく #T30 増作 造作
ぞうさつ #T30*2 増刷
ぞうさん #T30*2 増産
ぞうし #T30 増資
ぞうしゅう #T30 増収
ぞうしょ #T30*2 蔵書
ぞうしょう #T35 蔵相
ぞうしょく #T30*1 増殖
ぞうしん #T30*1 増進
ぞうすい #T30*6 増水 #T35*2 雑炊
ぞうせい #T30*1 増生 #T30 造成
ぞうせつ #T30*12 増設
ぞうせん #T30*1 造船
ぞうぜい #T30*1 増税
ぞうだい #T30*13 増大
ぞうちく #T30 増築
ぞうちょう #T30 増長
ぞうてい #T30 贈呈
ぞうとう #T30 贈答
ぞうに #T35 雑煮
ぞうはつ #T30 増発
ぞうびん #T30 増便
ぞうふく #T30*1 増幅
ぞうぶん #T35*2 増分
ぞうほ #T30 増補
ぞうむし #T35 象虫
ぞうよ #T30*1 贈与
ぞうり #T35*2 草履
ぞうりょく #T30 増力
ぞうりん #T30 造林
ぞうわい #T30 贈賄
ぞく #S5*55 属 #T30*53 属 #KJ*3 続 #T05*2 俗 #T35*2 賊 #JS 束 足 #SUC 族 #T35 族
ぞくあく #T05 俗悪
ぞくご #T35*1 俗語
ぞくしゅう #T35 俗習
ぞくしゅつ #T30*1 続出
ぞくしょう #T30 俗称
ぞくしん #T30 続伸
ぞくじ #T35 俗字
ぞくじん #T35 属人
ぞくせい #T35*108 属性
ぞくせいち #T35*7 属性値
ぞくせつ #T30*1 俗説
ぞくぞく #F14*4 続々 #F05 ぞくぞく
ぞくち #T35 属地
ぞくとう #T30 続投 続騰
ぞくはつ #T30 続発
ぞくぶつ #T35 俗物
ぞくへん #T35*2 続編
ぞくほう #T30*4 続報
ぞくらく #T30 続落
ぞくりゅう #T35*1 粟粒 #T35 俗流
ぞくろん #T30*1 俗論
ぞっか #T30 俗化
ぞっこう #T30*3 続行
ぞめ #D2T35 初め
ぞる #T35*1 ゾル
ぞろあすたー #KK*1 ゾロアスター
ぞろえ #SUC*4 揃え #T35 揃え
ぞろぞろ #F04 ぞろぞろ
ぞん #ZX*12 存 #KJ*4 存
ぞんがい #T05 存外
ぞんざい #T05 ぞんざい
ぞんじあげ #KS*1 存じ上げ
ぞんぶん #T15 存分
ぞんめい #T30 存命
た #T35*255 他 #T5*145 立 #T35*95 田 #T5*55 た 経 #S5r*39 足 #T5*14 発 #KJ*13 多 #KJ*9 太 #R5*5 足 #K5*4 炊 #T5*3 絶 #KJ*2 汰 #T5*1 建 断 #K5*1 焚 #T5 起 裁 #PRE 他 多 #KJ 詫
たーげっと #T35*13 ターゲット
たーとる #T35*3 タートル
たーびん #T35*1 タービン
たーぼ #T35*1 ターボ
たーみなる #T35*23 ターミナル
たーみねーた #T35*4 ターミネータ
たーむ #T35*1 ターム
たーん #T30*16 ターン
たあい #T35*2 他愛
たあいな #KY 他愛な
たあいのう #KYU 他愛のう
たい #T30*245 対 #KJ*16 態 #T30*14 体 #KJ*12 大 #T30*6 帯 #KJ*3 太 #T35*2 他意 #KJ*2 耐 泰 #T35*2 隊 #PRE*1 対 #KJ*1 待 滞 退 #T35*1 鯛 #CN タイ #KJ 碓 載 汰 #SUC 体 #KJ 堆 岱 怠 戴 替 胎 腿 苔 袋 貸 逮 #SUC 隊 #KJ 黛 台 諦
たいあたり #T30*1 体当り
たいあっぷ #T30*1 タイアップ
たいあん #T35*3 大安 #T35 対案
たいい #T30*1 退位 #T35 体位 大意
たいいき #T35*8 帯域 #T15*4 大域
たいいきち #T35 大域値
たいいく #T35*4 体育
たいいくかん #T35*1 体育館
たいいん #T30*8 退院 #T35*2 隊員
たいえい #T30 退嬰 #CN 大栄
たいえき #T35 体液
たいおう #T30*255 対応
たいおん #T35*1 体温
たいか #T30*1 耐過 #T35*1 大家 #T35 耐火 #T30 滞貨 退化 #T35 大火 大過
たいかい #T35*41 大会 #T35*1 大海 #T30 退会
たいかく #T35*25 対格 #T35*1 対角 #T35 体格
たいかくせん #T35*2 対角線
たいかん #T30*1 退官 #T35 耐寒 #T30 戴冠 #CN 大観
たいかんしき #T35 戴冠式
たいが #T35*2 大河
たいがい #T35*10 体外 #T35*2 対外 #T15*2 大概
たいがく #T30*1 退学
たいがん #T35*1 対岸
たいき #T30*3 待機 #T35*1 大気 #JCN 大樹
たいきゃく #T30 退却
たいきゅう #T35*2 耐久
たいきょ #T30*1 退去 #T30 大挙
たいきょう #T35 胎教
たいきょく #T35*2 対極 #T35*1 大局 #T35 太極 #T30 対局
たいきょくけん #T35 太極拳
たいきん #T35 大金
たいぎめいぶん #T35*1 大義名分
たいくう #T35*1 体腔
たいくつ #T00*1 退屈
たいぐう #T30 待遇
たいぐん #T35 大群
たいけい #T35*38 体系 #T35*1 体型 #T15 大慶
たいけつ #T30 対決
たいけん #T30*5 体験 #T35 大圏
たいげん #T30*2 体現 #T35*2 体言
たいこ #T35*3 太古 #T35*1 太鼓
たいこう #T30*5 対抗 #T35*2 大公 #T30*1 退行 #T35*1 大綱
たいこく #T35*1 大国
たいこしょう #T35*1 耐故障
たいこばら #T35 太鼓腹
たいこばん #T35 太鼓判
たいさ #T35*8 大佐 #T35*3 大差
たいさく #T30*34 対策 #T35*10 大作
たいさん #T30 退散
たいざ #T30 対座
たいざい #T30*6 滞在
たいし #CN 太子 #T35 大使 大志
たいしかん #T35 大使館
たいした #RT*21 大した
たいしつ #T35*1 体質
たいしゃ #T30*4 退社 #T30 代謝 #T35 大赦 #CN 大社
たいしゃく #T30 貸借
たいしゃくてん #JN 帝釈天
たいしゅう #T35*2 大衆 #T35 体臭
たいしゅつ #T30 退出
たいしょ #T30*55 対処 #T35*1 大暑
たいしょう #T35*211 対象 #T35*62 大将 #T30*11 対照 #T05*10 対称 #KK*3 大正 #T35*1 対症 #T30 大勝 #T35 大賞 #JCN 大正
たいしょうがい #T15 対象外
たいしょく #T30*11 退職 #T05 大食
たいしょくかん #T15 大食漢
たいしょくきん #T35*1 退職金
たいしん #T15 耐震 #CN 大信
たいじ #T30*3 退治 #T30*2 対峙 #T35*1 胎仔 胎児 #CN 太地 #JNM 泰次
たいじゅ #T35*1 大樹
たいじゅう #T35*5 体重
たいじょう #T30*1 退場
たいじん #T35*1 対人
たいすい #T35*1 耐水
たいすう #T35*3 対数
たいすけ #JNM 泰介 泰佑
たいせい #T35*7 体制 #T35*1 耐性 大勢 #T30*1 大成 #T35 体勢 態勢 #JCN 大成
たいせいよう #CN 大西洋
たいせき #T35*1 体積 #T30*1 堆積 #T30 滞積 退席
たいせつ #T05*25 大切 #T35*1 体節 #T35 大雪
たいせん #T30*3 対戦 #T35*1 大戦
たいぜん #F02 泰然
たいそう #T35*4 大喪 #T30*3 体操 #F14*1 たいそう #T16*1 大層
たいぞう #JNM 泰三 #T30 退蔵 #JN 大造
たいだ #T05 怠惰
たいだん #T30*2 退団 #T30*1 対談
たいち #T30*1 対置 #JNM 太一
たいちょ #T35 大著
たいちょう #T35*6 体調 #T35*3 体長 #T35*1 隊長
たいてい #T05*23 大抵 #T05*3 たいてい
たいてき #T35 大敵
たいでん #T30*1 帯電
たいと #T05 タイト #T35 泰斗
たいとう #T05*3 対等 #T30*1 台頭 #T30 帯刀 #JCN 台東
たいとく #T30 体得
たいとりーじょん #T35*1 タイトリージョン
たいとる #T35*67 タイトル
たいど #T35*18 態度
たいどう #T30 胎動
たいない #T35*2 胎内 #T35*1 体内 #T35 対内
たいにち #T35*1 対日 #T30 滞日
たいにん #T30*1 退任 #T35 大任
たいねつ #T35 耐熱
たいのう #T30 滞納
たいは #T30 大破
たいはん #T35*3 大半
たいばつ #T35 体罰
たいばん #T35 胎盤
たいひ #T30*47 対比 #T30*13 退避
たいびょう #T35 大病
たいぴすと #T35 タイピスト
たいふう #T35*11 台風
たいぶ #T35*1 体部 #T05 大部
たいぶつ #T35 対物
たいぷ #T30*76 タイプ
たいぷらいた #T35*1 タイプライタ
たいへい #T05 太平 #JN 大平
たいへいよう #CN 太平洋
たいへん #T05*255 大変 #T05*13 たいへん
たいべい #T35*1 対米
たいべつ #T30*1 大別
たいほ #T30*1 逮捕 #T30 退歩
たいほう #T35 大砲 大鵬
たいぼう #T30*2 待望 #T30 耐乏 #T35 大望
たいぼく #T35 大木
たいぽぐらふぃ #T35*1 タイポグラフィ
たいま #T35*10 タイマ #T35 大麻 #CN 当麻
たいまん #T05*1 怠慢
たいみんぐ #T35*25 タイミング
たいむ #T35*28 タイム
たいむあうと #T30*4 タイムアウト
たいむり #T05*1 タイムリ
たいめい #CN 岱明
たいめん #T35*1 体面 #T30*1 対面
たいもう #T35*1 大網
たいや #T35*6 タイヤ
たいやく #T30*3 対訳 #T35*2 大役
たいゆう #JCN 大雄
たいよ #T30*1 貸与
たいよう #T35*9 太陽 #T35*1 大洋 #JCN 大洋 #T35 大要
たいようけい #T35 太陽系
たいら #T35*3 平 #T05*1 平ら #JCN 平
たいらげ #KS 平らげ
たいりく #T35*1 大陸
たいりつ #T30*1 対立
たいりゃく #T35 大略
たいりゅう #T30 滞留
たいりょう #T05*19 大量
たいりょく #T35*8 体力
たいりん #T35*1 大輪
たいる #T35*3 タイル
たいれい #T35*2 胎齢
たいろう #T35 大老
たいわ #T30*32 対話 #JCN 大和
たいわん #CN 台湾
たう #T35*10 τ #T35*4 Τ 多雨
たうん #T35*3 タウン
たえ #KS*12 耐え #KS*7 絶え #KS*3 堪え #KJ 妙
たえこ #JNM 妙子
たえず #F14*1 絶えず
たえなる #RT 妙なる
たえま #T35*1 絶えま 絶え間
たお #S5*15 倒
たおやか #T05 たおやか
たおる #T35*4 タオル
たおれ #KS*3 倒れ
たか #KYmi*170 高 #KJ*33 高 #T35*2 鷹 #KJ*2 隆 #JNM たか タカ #KJ 尭 嵩 #JNS 多可 #T35 多寡
たかあき #JNM 孝明 隆明
たかい #T30*1 他界 #JNS 高井
たかいし #JCN 高石
たかいち #JCN 高市 #JNM 隆一
たかいど #CN 高井戸
たかお #JNM 孝男 孝夫 孝雄 #JCN 高尾 #JNM 隆男 隆夫 隆雄
たかおか #JCN 高岡
たかおの #CN 高尾野
たかが #F14*1 たかが
たかき #JNS 高木 高来
たかぎ #JCN 喬木 #JNS 高木 #JCN 高来
たかく #T35*2 多角 #JNS 高久
たかくけい #T35*1 多角形
たかくすい #T35*3 多角錐
たかくすいだい #T35*3 多角錐台
たかくちゅう #T35*2 多角柱
たかこ #JNM 貴子 孝子
たかさか #JCN 高坂
たかさき #JCN 高崎
たかさご #T35*1 高砂 #CN 高砂
たかさと #JCN 高郷
たかさわ #JNS 高沢
たかざわ #JNS 高沢
たかし #JNM 貴史 貴士 喬 敬 孝 孝司 孝志 高志 尚 崇 卓 隆 隆司 隆史 隆志
たかしお #T35*1 高潮
たかしま #JCN 高島 #JNS 高嶋 #JCN 鷹島
たかしまだいら #CN 高島平
たかしみず #JCN 高清水
たかじょう #JCN 高城
たかす #JNS 高鷲 #JCN 鷹栖
たかせ #JN*1 高瀬 #JCN 高瀬
たかた #JCN 高田
たかだ #JCN 高田
たかだい #T35 高台
たかだか #F14*5 高々
たかだのばば #CN 高田馬場
たかちほ #JCN 高千穂
たかっけい #T35*3 多角形
たかつ #JCN 高津
たかつき #JCN 高月 高槻
たかとう #JCN 高遠 #JNS 高藤
たかとし #JNM 孝利
たかとび #T30*1 高飛び
たかとみ #JCN 高富
たかとり #JCN 高取
たかな #R5r 高鳴
たかなか #JNS 高中
たかなし #JNS 高梨
たかなべ #JCN 高鍋
たかなみ #T35 高波
たかなわ #JCN 高輪
たかね #JCN 高根 #T35 高値
たかねざわ #JNS 高根沢
たかの #JCN 高野
たかのす #JCN 鷹巣
たかのぞみ #T10 高望み
たかのり #JNM 隆則
たかはぎ #JCN 高萩
たかはし #JNS 高橋 #JCN 高梁
たかはた #JNS 高畑 #JCN 高畠
たかはま #JCN 高浜 #JNS 高濱
たかはら #JCN 高原
たかはる #JCN 高原
たかばやし #JNS 高林
たかひで #JNM 貴英
たかひろ #JNM 孝宏
たかふみ #JNM 貴文 隆文
たかぶ #R5r 昂ぶ
たかま #R5r*3 高ま
たかまつ #JCN 高松
たかみ #JCN 高見
たかみち #JNM 孝通
たかみね #JNS 高嶺
たかみや #JCN 高宮
たかむら #JNS 高村
たかめ #KS*7 高め
たかもと #JNS 高本
たかもり #JCN 高森
たかや #JNS 高谷
たかやす #JCN 高安
たかやなぎ #JCN 高柳
たかやま #JCN 高山
たかゆき #JNM 孝幸 孝行 孝之 隆之
たかよし #JNM 孝義
たから #T35*10 宝 #JNM 財
たからか #T05 高らか
たからがいけ #CN 宝が池
たからじま #T35*1 宝島
たからづか #JCN 宝塚
たからべ #JCN 財部
たからもの #T35*1 宝物
たかり #T35*1 たかり
たかん #T05*1 多感
たが #W5*1 違 #JCN 多賀
たがい #T35*12 互い
たがいちがい #T35 互い違い
たがく #T05*2 多額
たがじょう #JCN 多賀城
たがた #JCN 田方
たがね #T35 鏨
たがみ #JCN 田上
たがや #S5r 耕
たがわ #JCN 田川
たき #T35*5 瀧 #T35*4 滝 #T35*1 多岐 #JNS 多喜 多岐 #JCN 多気 多紀 #JNS 滝 瀧
たきかわ #T35*1 滝川 #JCN 滝川 #JNS 瀧川
たきがわ #JNS 滝川 瀧川
たきぎ #T35 薪
たきぐち #JNS 滝口 瀧口
たきさわ #JNS 滝沢
たきざわ #JCN 滝沢 #JNS 瀧沢
たきしーど #T35*2 タキシード
たきた #JNS 滝田 瀧田
たきだし #T30 炊出し
たきね #JCN 滝根
たきの #JCN 滝野
たきび #T30 焚火
たきもと #JNS 滝本
たきょう #T35*1 他郷
たきょく #T35 他局 #T15 多極
たぎせい #T35 多義性
たく #JNSUC*43 宅 #S5*3 託 #S5*1 托 #KJ*1 拓 沢 琢 #JCN 多久 #JS 卓 #KJ 卓 啄 #T35 宅 #KJ 托 択 濯 託 #SX 託 #KJ 鐸 柘 擢
たくい #JNS 宅井
たくいつ #T30*1 択一
たくえつ #T30 卓越
たくお #JNM 卓雄
たくさん #T06*61 たくさん #T06*15 沢山
たくしー #T35*4 タクシー
たくしゅつ #T30 卓出
たくしょく #T30*1 拓殖
たくじ #T35 託児
たくじょう #T35*3 卓上
たくせつ #T35 卓説
たくせん #T35 託宣
たくそう #JNM 拓宋 #T30 託送
たくち #T35 宅地
たくはい #T30 宅配
たくはつ #T30 托鉢
たくばつ #T05 卓抜
たくほん #T35 拓本
たくぼ #JNS 田窪
たくぼく #JNM 啄木
たくま #JCN 詫間
たくまし #KY たくまし
たくましゅう #KYU たくましゅう
たくみ #T05*2 巧み #JNM 巧 #T35 匠
たくや #JNM 卓也
たくら #M5r*1 企
たくわえ #KSr*6 蓄え #KSr*2 貯え
たぐ #T35*29 タグ #R5*10 手繰
たぐい #T35*4 類
たぐち #JCN 田口
たけ #T35*39 竹 #T35*15 岳 #KJ*7 武 #KS*6 長け #KJ*5 茸 #CNSUC1*1 岳 #T35*1 丈 #JNM たけ タケ #T35 他家 #R5 猛
たけい #JNS 武井
たけいち #JNS 竹市 武一 武市
たけうち #JNS 竹内 武内
たけお #JNS 威雄 #JNM 毅雄 健夫 丈夫 #JN 竹男 竹雄 #JNM 武男 武夫 #JCN 武雄
たけおか #JCN 竹岡
たけざわ #JNS 竹沢
たけし #JNM 毅 健 健史 健志 剛 竹志 武 武司 武史 武志 #JCN 武石 #JNM 猛
たけした #JNS 竹下
たけじ #JNM 武治
たけじろう #JNM 竹次郎
たけぞの #JNS 竹園
たけた #JCN 竹田
たけだ #JCN 竹田 武田
たけと #JNM 丈人
たけとみ #JCN 竹富
たけとよ #JCN 武豊
たけなか #JNS 竹中
たけなわ #T05*1 たけなわ
たけの #JCN 竹野
たけのぶ #JN 武信
たけはら #JCN 竹原
たけばやし #JNS 竹林
たけひこ #JNM 武彦
たけひご #T35 竹ひご
たけひさ #JNM 武久
たけひで #JNM 武英 武秀
たけひろ #JNM 武裕
たけふ #JCN 武生
たけべ #JCN 建部
たけむら #JNS 竹村
たけもと #JNS 竹本
たけもり #JNS 竹森
たけやり #JNS 武鑓
たけん #T35*1 他県
たげ #T05 たげ
たげい #T05 多芸
たげん #T35 多元 #T30 多言
たこ #T35*23 蛸 #T35*1 凧 鮹 #JNS 多古
たこう #KYU*3 高う #T35*1 他校 #T05 多幸
たこうしき #T35*2 多項式
たこく #T35*2 他国
たごん #T30 他言
たさい #T05*3 多彩 #T05*1 多才
たさく #T05 多作
たさつ #T35 他殺
たざ #T30 打坐
たざき #JNS 田崎
たざわ #JNS 田沢
たざわこ #CN 田沢湖
たし #T35*10 他紙 #T35*1 足し #T35 他誌
たしか #T06*180 確か #T06*6 たしか
たしかめ #KSr*15 確かめ
たしざん #T30*1 足し算 足算
たしつ #T05 多湿
たしゃ #T35*14 他社 #T35 他者 #T30 多謝
たしゅ #T05*4 多種 #T35 他種
たしゅつ #T30 他出
たしょう #T35*31 多少 #T35 他生 多祥
たしろ #JCN 田代
たじ #T35*2 他事 #T05 多事
たじつ #T35 他日
たじま #JCN 田島 #JNS 田嶋
たじみ #JCN 多治見
たじゅう #T35*29 多重
たじょう #T05 多情
たじり #JCN 田尻
たすう #T35*29 多数
たすうけつ #T30*1 多数決
たすか #R5r*19 助か
たすく #T35*43 タスク
たすけ #KSr*24 助け
たすけあ #W5r*1 助け合
たすけだ #S5 助け出
たずさえ #KS 携え
たずさわ #R5r*3 携わ #R5r*2 たずさわ
たずね #KSr*20 尋ね #KSr*2 訪ね #KSr*1 たずね
たせん #T30 他薦
たぜい #T35 多勢
たそーと #T30*1 多ソート
たぞえ #JNS 田添
たた #K5r*27 叩 #F14*5 多々 #K5r*2 たた #M5*2 畳 #R5r 祟
たたえ #KSr*1 讃え #KS たたえ #KSr 称え #KS 湛え
たたか #W5r*4 戦 #W5r*1 闘
たたきこ #M5r たたき込
たたきだ #S5r*1 たたき出
たたみ #T35*1 畳
たたみかけ #KSr 畳み掛け
たたみこ #M5r*1 畳み込
ただ #F14*53 ただ #T35*47 ただ #T35*3 只 #KJ*3 唯 #S5*2 正 #KJ 惟 #S5 質 #JNS 多田
ただい #T05*3 多大
ただいま #T35*28 只今 #F14*19 ただいま #T35*1 ただ今
ただお #JNM 忠男 忠夫 忠雄
ただおか #JCN 忠岡
ただごと #T35 唯事
ただし #KY*255 正し #CJ*109 ただし #CJ*43 但し #CJ*1 但 #JNM 惟史 正 忠 忠司 忠志 直 貞 禎
ただしゅう #KYU*3 正しゅう
ただす #JN 匡
ただちに #F14*12 直ちに
ただはる #JNM 忠治
ただひこ #JNM 忠彦
ただひろ #JNM 忠広
ただまさ #JNM 忠正
ただみ #JCN 只見
ただみち #JNM 直道
ただよ #W5*6 ただよ #W5*1 漂
ただよし #JNM 忠義 忠良
ただよわ #S5 漂わ
ただれ #KSr ただれ
たち #JNSUC*197 たち #KJ*34 達 #T15*12 多値 #T35*2 太刀 #SUC たち #T35 質 #JNSUC 達 #SUC 達
たちあ #W5r*3 立ち会
たちあいにん #T35 立会人
たちあが #R5r*37 立ち上が
たちあがり #T35 立上り
たちあげ #KSr 立ち上げ
たちあらい #JCN 大刀洗
たちい #R5r*2 立ち入 #T30*1 立ち居 立居
たちうり #JNS 立売 #T30 立売
たちおうじょう #T30 立往生
たちおくれ #KSr 立ち後れ
たちかけ #JNS 太刃掛
たちかわ #JCN 立川
たちき #R5r*1 断ち切
たちぎえ #T35*1 立ち消え
たちぎき #T30*1 立ち聞き
たちぐい #T30 立ち食い
たちさ #R5 立ち去
たちすく #M5r 立ちすく
たちどころに #F14 たちどころに
たちどま #R5r*1 立ち止ま
たちなお #R5r*3 立ち直
たちなら #B5*1 立ち並 #B5 建ち並
たちの #K5r 立ち退
たちはだか #R5*1 立ちはだか
たちば #T35*29 立場
たちばな #T35*22 橘 #JCN 橘 立花
たちばなし #T30*3 立ち話
たちまち #F14*1 たちまち #T35*1 立待ち
たちまわ #R5r*1 立ち回
たちみ #T30*2 立見
たちむか #W5r*1 立ち向か
たちよ #R5 立ち寄
たっ #SX*15 達
たっきゅう #T30*2 卓球
たっきゅうびん #T35*3 宅急便
たっけん #T35 卓見
たっしゃ #T05*1 達者
たっせい #T30*4 達成
たった #F14*1 たった #T36 たった
たっち #T30*5 タッチ
たっての #RT たっての
たっと #B5 貴 #KY 貴 #B5 尊 #KY 尊
たっぴつ #T05 達筆
たっぷり #F04*2 たっぷり
たつ #KJ*6 達 #T35*1 龍 #T35 辰 竜
たつお #JNM 達男 達夫 達雄 辰男 辰夫 辰雄 竜夫 竜雄 龍雄
たつかわ #JNS 立川
たつこ #JN 田子
たつごう #JNS 竜郷
たつし #JNM 達志
たつじ #JNM 辰二
たつじん #T35*2 達人
たつの #JCN 辰野 #JNS 竜野 #JCN 龍野
たつのくち #JCN 辰口
たつのり #JNM 達徳
たつぶん #T35 達文
たつまき #T35*1 竜巻
たつみ #T35 巽 #JNM 立見
たつや #JNM 達也 辰也 立哉 竜也 龍也
たつやま #JNS 竜山
たつゆき #JNM 達行
たつるはま #CN 田鶴浜
たつろう #JNM 達郎
たづな #T35*1 手綱
たて #T35*28 縦 #KS*21 建て #KS*6 立て #KS*1 たて #T35*1 舘 盾 #D2T35 たて #T35 館 楯 #KJ 竪 #KS 点て #D2T35 立て
たてあき #JNM 賢明
たていし #KK*4 立石 #JNS 建石 #JCN 立石
たていしでんき #KK*8 立石電機
たていわ #JCN 館岩
たてうち #JN*1 館内
たてうり #T35 建売り
たてかえ #KSr*22 立て替え #T30*2 立替 #KSr*1 建て替え
たてかけ #KS 立てかけ
たてけい #T35*1 │ ┃
たてこ #M5r*1 たて込
たてこも #R5r 立てこも
たてしな #JCN 立科
たてじま #T35*1 縦縞
たてつ #K5 盾突
たてつけ #T35 建てつけ 立付け
たてつぼ #T35 建坪
たてなが #T05*3 縦長
たての #JCN 館野 #JNS 舘野
たてばやし #JCN 館林
たてまえ #T35 建前
たてまし #T30 建増し
たてまつ #R5 奉
たてもの #T35*4 建物
たてやくしゃ #T35 立役者
たてやま #JCN 館山 立山
たで #T35*2 蓼
たとえ #F14*24 たとえ #KS*8 例え #KS*1 たとえ
たとえば #F14*226 例えば #CJ*5 たとえば
たとみ #JCN 田富
たど #R5*12 たど #R5*2 辿 #JCN 多度
たどうし #T35*3 他動詞
たどく #T30 多読
たどころ #JNS 田所
たどつ #JCN 多度津
たどりつ #K5*6 たどり着
たな #T35*10 棚 #T35 店
たなあげ #T30 棚上げ
たなおろし #T30 棚卸し
たなか #JCN 田中
たなき #JNS 棚木
たなぐら #JCN 棚倉
たなごころ #T35 掌
たなし #CN 田無
たなだ #JNS 棚田
たなばた #T35*4 七夕
たなび #K5 棚引
たなべ #JCN 田辺
たなん #T05*1 多難
たに #T35*22 谷 #JNS 谷
たにうち #JNS 谷内
たにがわ #T35*2 谷川 #JCN 谷川
たにぐち #JCN 谷口
たにぐみ #JCN 谷汲
たにま #T35 谷間
たにもと #JNS 谷本
たにやま #JCN 谷山
たにん #T35*28 他人
たぬき #T35*1 狸
たぬしまる #CN 田主丸
たぬま #JCN 田沼
たね #T35*5 種 #T35 胤 #JN 種子
たねあかし #T30 種明かし 種明し
たねいち #JCN 種市
たねがしま #CN 種子島
たねん #T15 多年
たの #M5r*67 頼 #JCN 多野 田野
たのう #T05 多能
たのうら #JCN 田浦
たのし #KYmi*116 楽し #M5r*116 楽し #KYmi*1 愉し #KYmi たのし
たのしゅう #KYU*3 楽しゅう #KYU*1 愉しゅう #KYU たのしゅう
たのはた #JCN 田野畑
たのみこ #M5r 頼み込
たのもし #KY*3 頼もし
たのもしゅう #KYU*1 頼もしゅう
たはつ #T30*2 多発
たはら #JCN 田原
たば #JS*1 束 #T35*1 束
たばいと #T15*26 多バイト
たばか #R5 たばか 謀
たばこ #T35*2 タバコ 煙草
たばた #JCN 田端 #JNS 田畑
たばね #KS 束ね
たばやま #CN 丹波山
たび #T35*31 たび #T35*23 度 #T30*9 旅 #T35 足袋
たびかさな #R5*1 度重な
たびさき #T35 旅先
たびじ #T35*1 旅路
たびたび #T06*6 たびたび 度々
たびだ #T5r*4 旅立
たびびと #T35*1 旅人
たびら #JCN 田平
たふ #T05*3 タフ
たぶ #T35*9 タブ
たぶー #T05*1 タブー
たぶせ #JCN 田布施
たぶち #JNS 田渕
たぶん #F14*153 多分 #T35*101 多分 #F14*19 たぶん #T35*2 他聞
たべ #KS*119 食べ
たべもの #T35*6 たべもの
たべん #T05 多弁
たほう #T35*6 他方
たぼう #T05*4 多忙
たま #R5r*33 溜 #R5r*9 たま #T35*7 玉 #T35*4 球 #T35*1 珠 #T35 丸 #W5 給 賜 #JCN 多摩 #T35 弾 #R5 貯 #KJ 霊
たまい #JNS 玉井
たまかし #JNS 玉柏
たまかわ #JNS 玉川
たまがわ #JCN 玉川 #CN 田万川
たまき #JCN 玉城 #JNS 玉木
たまぐすく #JCN 玉城
たまげ #KS たまげ 魂消
たまご #T35*12 卵 #T35 玉子
たましい #T35*2 魂
たまたま #T06*7 たまたま
たまち #CN 田町
たまつき #T30 球撞き
たまつくり #JCN 玉造
たまづくり #JCN 玉造
たまてっく #KK*1 多摩テック
たまな #JCN 玉名
たまに #F14*16 たまに
たまねぎ #T35 タマネギ 玉葱
たまの #JCN 玉野
たまのうら #CN 玉之浦
たまのこし #T35*2 玉の輿
たまほ #JCN 玉穂
たまむし #T35*1 玉虫
たまむら #JNS 玉村
たまもの #T35 賜
たまやま #JCN 玉山
たまゆ #JCN 玉湯
たまり #JCN 玉里 #T35 溜
たまわ #R5*2 賜わ #R5 賜
たみ #T35 民
たみお #JNM 民男 民雄
たむし #T35 田虫
たむら #JNS 田村
たむろ #T30*2 屯
ため #T35*255 ため #S5*76 試 #T35*53 為 #KS*8 貯め #T35*1 溜 #KS 矯め 溜め
ためいき #T30*1 ため息
ためし #T35 例
ためら #W5r*1 ためら
ためん #T35*2 他面 #T05*1 多面
ためんたい #T35*3 多面体
たも #T5*21 保
たもつ #JN 保
たもり #JNS タモリ
たや #S5 絶や
たやす #KY*1 たやす 容易
たやすう #KYU*1 たやすう
たよ #R5r*24 頼
たよう #T05*4 多様 #T30*2 多用
たより #T35*3 便り
たよりな #KY*1 たよりな #KY 頼りな
たよりのう #KYU*1 たよりのう #KYU 頼りのう
たら #T35*11 鱈 #S5*2 垂ら #JCN 太良
たらぎ #JCN 多良木
たらず #N2T16*7 足らず
たらま #JCN 多良間
たらみ #JCN 多良見
たり #KS*51 足り
たりき #T35*1 他力
たりょう #T05*3 多量
たる #T35*1 樽
たるい #JCN 垂井
たるみ #JCN 垂水
たるみず #JCN 垂水
たれ #KSr*3 垂れ
たれさが #R5r 垂れ下が
たれんと #T35*1 タレント
たろう #JCN 太郎 #CN 田老
たわ #M5r*1 たわ
たわー #T35*2 タワー
たわい #T35*1 たわい
たわいな #KY たわいな
たわいのう #KYU たわいのう
たわごと #T35*1 たわごと 戯言
たわむれ #KSr 戯れ
たわら #T35*4 俵 #JCN 田原
たわらもと #JCN 田原本
たわわ #T05 たわわ
たん #T35*217 単 #T35*9 短 #KJ*5 端 #KJ*4 丹 #KJ*2 坦 譚 #KJ*1 但 嘆 歎 淡 #T35*1 胆 #JS*1 反 #T35*1 痰 #ZX 嘆 #KJ 担 探 旦 湛 炭 #SUC 端 #KJ 箪 綻 耽 蛋 誕 鍛 椴
たんい #T35*101 単位 #T35*2 単為
たんいつ #T05*28 単一
たんいつか #T30*4 単一化
たんか #T35*4 単価 #T35*1 ＠ 短歌 #T35 担架 #T30 炭化 #T35 啖呵
たんかー #T35 タンカー
たんかん #T35*2 単漢
たんかんじ #T35 単漢字
たんがん #T30 嘆願
たんき #T15*3 短期 #T05*1 短気
たんきゅう #T30*2 探求 #T30*1 探究
たんく #T35*3 タンク
たんぐすてん #T35*1 タングステン
たんけん #T30*3 探検 #T30*1 探険
たんげい #T30 端倪
たんげん #T35 単元
たんこう #T05*3 単項 #T35*2 単向 #T30*2 単行 #T35*1 炭鉱 #T35 炭坑
たんご #T35*255 単語 #CN 丹後
たんさ #T30 探査
たんさい #T35 淡彩
たんさく #T30*16 探索
たんさくぎ #T35*6 探索木
たんさん #T35 炭酸
たんざく #T35 短冊
たんざにあ #CN タンザニア
たんし #T35*17 端子
たんしゃ #T35*3 単射 #T35 単車
たんしゅく #T30*8 短縮
たんしょ #T35*2 短所 #T35 端緒
たんしょう #T30 探勝 #T05 短小
たんしん #T35*2 単身 #T35 短針
たんじかん #T15*3 短時間
たんじゅう #T35*1 胆汁 #T35 短銃
たんじゅん #T05*37 単純
たんじょう #T30*7 誕生
たんじょうび #T35*21 誕生日
たんす #T35*2 箪笥
たんすい #T35 淡水
たんすう #T35 単数
たんせい #T35*1 丹青 #KK*1 淡青 #T05*1 端正 #T30 丹精
たんせいど #T05*1 単精度
たんせき #T35 胆石
たんせん #T35 単線
たんぜん #CN 丹前 #T35 丹前 #F00 端然
たんそ #T35*1 炭素 炭疽
たんそく #T30 嘆息
たんたい #T35*3 単体 #T35*1 担体
たんたん #F00*1 淡々
たんだい #T35*12 短大
たんち #T30*1 探知
たんちょう #T05*3 単調 #T35 丹頂 短調
たんちょうづる #T35 丹頂鶴
たんてい #T35*1 単蹄 #T30*1 探偵
たんてき #T05*2 端的
たんてん #T35*1 端点
たんでき #T35 耽溺
たんでん #T35*1 炭田
たんとう #T30*255 担当 #T35*2 短刀 #CN 但東
たんとうしゃ #T35 担当者
たんとうちょくにゅう #T05 単刀直入
たんどく #T35*16 単独 #T35 丹毒
たんなる #RT*10 単なる
たんなん #CN 丹南
たんにん #T30*1 担任
たんねん #T05 丹念
たんの #JNS 丹野 #JCN 端野
たんのう #T30 堪能 胆嚢
たんば #JCN 丹波
たんばいち #CN 丹波市
たんばら #JCN 丹原
たんぱ #T35 短波
たんぱく #T35*1 タンパク 蛋白 #T05 淡白
たんび #T35 耽美
たんぴょう #T35*1 単票 #T30*1 短評
たんぴん #T35 単品
たんぶん #T35*2 単文 #T35 短文
たんぺん #T35*1 短篇 短編 #T35 短辺
たんぼ #T35*9 田んぼ
たんぼう #T30 探訪
たんぽ #T30 担保
たんぽぽ #T35 タンポポ 蒲公英
たんまつ #T35*151 端末
たんむ #T30*1 坦務
たんもの #T35 反物
たんようき #T35 単葉機
たんらく #T30*4 短絡
たんりょく #T35 胆力
たんれい #T05 端麗
たんれん #T30 鍛錬
だ #S5r*255 出 #T35*95 田 #JS*66 打 #S5*9 だ #K5*9 抱 #KJ*5 駄 #T5r*3 立 #KJ*1 雫 蛇 妥 舵 楕 騨 #KJ 唾 堕 惰 打 柁 陀
だーい #PRE*1 だーい
だーく #T05*2 ダーク
だーす #JS*2 ダース
だい #KJ*201 第 #T35*76 大 #T35*43 台 #T35*18 代 #S5*10 題 #T30*10 題 #KJ*1 醍 #KJ 岱 #JS 代 #SUC 代 #CNSUC1 台 #JSSUC 台 #CNSUC1 大 #PRE 大 #SUC 大 #NNPRE 第 #JS 題 #KJ 弟 内 廼
だいあぐらむ #T35*5 ダイアグラム
だいある #T30*12 ダイアル
だいあろぐ #T30*5 ダイアログ
だいあん #T35*4 代案 #CN 大安
だいいち #T35*118 第一
だいえい #CN 大栄
だいえっと #T30*4 ダイエット
だいえん #T35 大円
だいおーど #T35*2 ダイオード
だいおう #CN 大王
だいおうじょう #T30*1 大往生
だいか #T35*1 代価
だいかん #T35*2 大寒
だいがえ #T30 代替
だいがく #T35*45 大学
だいがくいん #T35*26 大学院
だいがくせい #T35*3 大学生
だいきぼ #T05*17 大規模
だいきゅう #T30*4 代休
だいきん #T35*5 代金
だいく #T35*1 大工
だいくすとら #JN ダイクストラ
だいけい #T15 台形
だいこう #T30*4 代行 #T30 代講
だいこく #T35*1 大黒
だいこん #T35*5 大根
だいご #CN 大子
だいごみ #T35*3 醍醐味
だいざい #T35*1 題材
だいし #T35 台紙
だいしゃ #T35*1 台車
だいしゅくしゃく #T15 大縮尺
だいしょ #T30 代書
だいしょう #T35*9 大小 #T30*3 代償
だいじ #T05*27 大事 #T35 題字
だいじぇすと #T30*4 ダイジェスト
だいじょうぶ #T05*199 大丈夫
だいじん #T35*3 大臣
だいすう #T35*115 台数 #T35*35 代数
だいすき #T05*15 大好き
だいず #T35 大豆
だいせつ #T35*1 大雪
だいせん #CN 大山
だいそれた #RT 大それた
だいぞう #JNM 大蔵
だいたい #T35*138 大体 #T30*23 代替 #T35*7 だいたい #T35*1 大腿 大隊
だいたん #T05*5 大胆
だいだい #T35*3 代々 橙
だいち #T35*4 大地 #T35 台地
だいちょう #T35*2 台帳 #T35 大腸
だいとう #JCN 大東
だいとうりょう #T35*1 大統領
だいどころ #T35*1 台所
だいなごん #T35 大納言
だいなし #T05 台無し
だいなぶっく #KK*2 ダイナブック
だいなみっく #T05*34 ダイナミック
だいなみっくす #T35*1 ダイナミックス
だいなり #T35*2 ＞
だいなりいこーる #T35*4 ≧
だいに #T35*111 第二
だいにっぽん #KK*3 大日本
だいにゅう #T30*32 代入
だいのう #T30 代納 #T35 大脳
だいひつ #T30*1 代筆
だいひょう #T30*23 代表
だいふく #T35 大福
だいふくちょう #T35 大福帳
だいぶ #F14*15 だいぶ #F14*5 大部
だいぶつ #T35*3 大仏
だいぶぶん #T35*7 大部分
だいぶん #F14*1 大分
だいべん #T30*1 代弁 #T30 大便
だいほん #T35 台本
だいみょう #T35*1 大名
だいめい #T35*6 題名
だいめいし #T35*5 代名詞
だいもく #T35*5 題目
だいもん #JCN 大門 #T35 大門
だいもんじ #CN 大文字
だいや #T35*3 ダイヤ
だいやく #T30*1 代役
だいやぐらむ #T35*16 ダイヤグラム
だいやもんど #T35*1 ダイヤモンド
だいやる #T30*4 ダイヤル
だいよう #T30*6 代用
だいり #T30*22 代理
だいりせき #T35*1 大理石
だいりてん #T35*3 代理店
だいれくと #T05*5 ダイレクト
だいわ #JCN 大和
だう #T35 ダウ
だうん #T30*37 ダウン
だうんひる #T35*1 ダウンヒル
だえき #T35 唾液
だえん #T35*10 楕円
だえんこ #T35*2 楕円弧
だおれ #D2T35*1 倒れ
だか #KY*2 高 #SUC 高
だかい #T30*2 打開
だから #CJ*14 だから #T35*1 だから
だが #CJ*1 だが
だがー #T35*1 †
だき #T30 唾棄
だきこ #M5r 抱き込
だきしめ #KSr*2 だきしめ #KSr*1 抱きしめ #KSr 抱き締め
だきょう #T30*2 妥協
だく #KJ*4 濁 #T30 諾
だくおん #T35*3 濁音
だくすい #T35 濁水
だくてん #T35*2 ゛ 濁点
だくと #T35*1 ダクト
だくりゅう #T35 濁流
だけ #CNSUC1*8 岳
だけつ #T30 妥結
だけど #CJ*9 だけど
だけん #T30*9 打鍵
だげき #T35*2 打撃
だこう #KYU*1 高う #T30 蛇行
ださく #T05*1 駄作
ださん #T30*3 打算
だざい #JNS 太宰
だざいふ #CN 太宰府
だし #T35 山車 出汁
だしあ #W5r*2 出し合
だしいれ #T30*1 出し入れ
だしき #R5r*1 出し切
だしぬ #K5r 出し抜
だしもの #T35*2 出し物
だしゃ #T35*1 打者
だしん #T30*2 打診
だじゃく #T05 懦弱
だじゃれ #T35*3 駄洒落
だすてぃん #JN ダスティン
だせい #T35*1 惰性
だせき #JS 打席 #T35 打席
だそく #T05*3 蛇足
だだ #T35*3 だだ
だちょう #T35 駝鳥
だちん #T35 駄賃
だっ #SX*2 脱
だっかい #T30 奪回 脱会
だっかく #T35 奪格
だっきゃく #T30 脱却
だっきゅう #T30*1 脱臼
だっこう #T30*1 脱稿 #T30 脱肛
だっこく #T30 脱穀
だっし #T30 脱脂
だっしゅ #T30*2 ダッシュ
だっしゅう #T30*1 脱臭
だっしゅつ #T30*3 脱出
だっしょく #T30 脱色
だっすい #T30 脱水
だっせん #T30*1 脱線
だっそう #T30 脱走
だったい #T30*1 脱退
だったら #CJ だったら
だっちょう #T30 脱腸
だって #CJ だって
だっぴ #T30 脱皮
だつ #KJ*2 脱 #KJ 奪
だつい #T30 脱衣
だつごく #T30*1 脱獄
だつじ #T35*1 脱字
だつぜい #T30 脱税
だつぼう #T30*1 脱帽
だつもう #T30 脱毛
だつらく #T30 脱落
だつりゃく #T30 奪略
だて #T05*19 伊達 #KSr*1 立て #JCN 伊達
だてまき #T35 伊達巻
だてん #JS 打点
だでん #T30 打電
だとう #T00*10 妥当 #T30 打倒
だとうせい #T35*8 妥当性
だに #CNSUC1*1 谷 #T35*1 谷
だは #T30*2 打破
だび #T35 荼毘
だびんぐ #T30*2 ダビング
だぶつ #K5r*1 ダブつ
だぶる #T35*15 ダブル
だぶるだがー #T35*1 ‡
だほ #T30 拿捕
だぼく #T30 打撲
だま #S5r*5 だま #R5*4 黙
だましと #R5r だまし取
だまり #T35 溜り
だみー #T35*4 ダミー
だみごえ #T35 だみ声
だむ #T35*1 ダム
だめ #T05*91 ダメ #T05*55 駄目 #T35 溜め
だめーじ #T35*13 ダメージ
だらく #T30 堕落
だらけ #SUC*3 だらけ
だらしな #KY だらしな
だらしのう #KYU だらしのう
だらす #CN ダラス
だらだら #F04 だらだら
だらりと #F12 だらりと
だり #JN ダリ
だりあ #T35 ダリア
だりょく #T35 惰力
だる #KYmi*2 だる
だるう #KYU*1 だるう
だれ #T35*255 誰 #T35*14 だれ
だれしも #F14 誰しも
だん #JS*87 段 #T35*25 談 #T35*18 弾 #ZX*13 談 #ZX*4 弾 #T35*2 断 #ZX*2 断 #T35*1 暖 段 #KJ*1 男 #KJ 騨 旦 #SUC 団 #T35 団 壇 #KJ 檀 椴 灘 #JNS 團
だんあつ #T30*1 弾圧
だんかい #JS*54 段階 #T35*29 段階 #T35*1 団塊
だんがい #T30*2 弾劾 #T35 断崖
だんがん #T35*1 弾丸
だんき #T35 暖気
だんぎ #T30 談義 談議
だんけつ #T30*1 団結
だんげん #T30*2 断言
だんこ #F00*2 断固
だんこう #T30 団交 断交 断行
だんこん #T35 弾痕 男根
だんご #T35*1 だんご 団子
だんごう #T30*1 談合
だんごうざか #CN 談合坂
だんごばな #T35*1 だんご鼻
だんさい #T30*1 断裁
だんし #T35*31 男子
だんしゃく #T35*1 男爵
だんしょう #T30 談笑
だんじ #T35*1 男児
だんじき #T30 断食
だんじて #F14*2 断じて
だんじょ #T35*12 男女
だんじょう #T35 壇上
だんす #T30*6 ダンス
だんせい #T35*17 男性 #T35*2 弾性 #T35 男声
だんせん #T30 断線
だんぜつ #T30 断絶
だんぜん #F14*2 断然
だんそう #T35 断層
だんぞく #T30*2 断続
だんたい #T35*15 団体
だんだん #F14*15 段々 #F14*2 だんだん
だんち #T35*3 団地
だんちがい #T05 段違い
だんちょう #T35*2 団長 #T35 断腸
だんてい #T30*2 断定
だんとう #T35*1 暖冬
だんとうだい #T35 断頭台
だんとつ #T05 断トツ
だんどり #T30*1 段取り
だんな #T35*8 旦那
だんねん #T30*2 断念
だんぷ #T30*59 ダンプ
だんぺん #T35*3 断片
だんぼう #T30 暖房
だんめん #T35*1 断面
だんゆう #T35 男優
だんらく #T35*8 段落
だんりゅう #T35 暖流
だんりょく #T35*1 弾力
だんろ #T35 暖炉
だんわ #T30*1 談話
ち #SUC*255 値 #T35*25 知 #KJ*13 茅 #T35*12 地 #R5*10 散 #T35*9 血 #KJ*7 千 #KJ*6 池 #KJ*4 遅 #T30*3 治 #KJ*2 智 稚 #KJ*1 弛 恥 置 魑 #KJ 値 #T35 値 #SUC 地 #KJ 痴 致 蜘 馳 薙
ちーず #T35*4 チーズ
ちーふ #T35*1 チーフ
ちーむ #T35*14 チーム
ちあき #JN 千秋
ちあん #T35 治安
ちい #T35*5 地位 #T35 地衣
ちいき #T35*20 地域 #T35*16 値域
ちいさ #KY*62 小さ
ちいさがた #CN 小県
ちいさな #RT*26 小さな
ちいそう #KYU*2 小そう
ちぇーん #T35*2 チェーン
ちぇあまん #T35*1 チェアマン
ちぇす #T35*1 チェス
ちぇっか #T35*8 チェッカ
ちぇっかー #T35*1 チェッカー
ちぇっく #T30*248 チェック
ちぇるのぶいり #CN チェルノブイリ
ちぇんじ #T30*2 チェンジ
ちえ #T35*9 知恵 #JNM 千絵
ちえこ #JNM 千恵子 千枝子 智恵子
ちえぶくろ #T35 知恵袋
ちえり #JNM 智恵理
ちえん #T30*30 遅延 #T35*1 地縁
ちか #KY*106 近 #W5r*10 誓 #T35*8 地下 #KJ*2 近 #T35*1 地価
ちかい #T35*2 地階
ちかく #T35*18 近く #T30*2 知覚 #T35 地殻
ちかけい #T35 地下茎
ちかこ #JNM 千加子
ちかごろ #T35*2 近頃
ちかし #KY 近し
ちかしつ #T35*3 地下室
ちかしゅう #KYU 近しゅう
ちかぢか #F14*16 近々
ちかぢかと #F14 近々と
ちかぢかの #F14 近々の
ちかづ #K5*31 近付
ちかづけ #KS 近づけ 近付け
ちかてつ #T35*8 地下鉄
ちかば #T35 近場
ちかみち #T30*1 近道
ちかやま #JNS 近山
ちかよ #R5*1 近寄
ちから #T35*26 力 #JN 力
ちからぞえ #T30 力添え
ちからづよ #KY*1 力強
ちからづよう #KYU*1 力強う
ちからぶそく #T00*1 力不足
ちからもち #T15 力持ち
ちからわざ #T35 力業
ちかん #T30*36 置換 #T35*1 痴漢
ちが #W5r*255 違
ちがいな #KY*5 違いな
ちがいのう #KYU*1 違いのう
ちがえ #KS 違え
ちがさき #CN 茅ヶ崎
ちき #T35*2 知己
ちきゅう #T35*3 地球
ちきゅうぎ #T35 地球儀
ちきん #T35*5 チキン
ちぎ #R5r*1 契
ちぎり #T35*1 契
ちく #CNSUC1*17 地区 #KJ*4 筑 #KJ*2 築 #KJ*1 竹 蓄 #T35 地区 #KJ 畜 逐
ちくいち #F14 逐一
ちくご #CN 筑後
ちくさ #JCN 千種
ちくさん #T30 畜産
ちくし #JCN 筑紫
ちくしの #CN 筑紫野
ちくじ #F14*255 逐次
ちくじょう #CN 築上 #T30 築城
ちくせき #T30*14 蓄積
ちくぜん #JCN 筑前
ちくぞう #T30 築造
ちくでん #T30 蓄電
ちくねん #F14 逐年
ちくほ #CN 筑穂
ちくま #JCN 筑摩
ちくら #JCN 千倉
ちぐさ #T35 千草
ちぐはぐ #T05 ちぐはぐ
ちけい #T35*1 地形
ちけっと #T35*25 チケット
ちけん #T35*1 知見
ちこう #KYU*3 近う
ちこく #T30*4 遅刻
ちご #T35 稚児
ちさい #T35 地裁
ちさこ #JNM 千佐子
ちさと #JCN 千里
ちさん #T30 遅参
ちし #T35*1 致死
ちしお #T35 血潮
ちしき #T35*85 知識
ちしつ #T35*1 地質
ちしま #CN 千島
ちじ #T35*2 知事
ちじく #T35 地軸
ちじょう #T35*4 地上 #T35 痴情
ちじょく #T35 恥辱
ちじん #T35*2 知人
ちすい #T30 治水
ちすじ #T35 血筋
ちず #T35*22 地図 #JCN 智頭
ちせい #T35*2 知性
ちせつ #T05*1 稚拙
ちそう #T35*1 地層 #T30 馳走
ちた #CN 知多
ちたい #T35*2 地帯 #T30*1 遅滞
ちだ #JNS 千田
ちだい #T35 地代
ちち #T35*11 父 #F00*1 遅々 #T35*1 乳
ちちおや #T35*7 父親
ちちばなれ #T30 乳離れ
ちちぶ #JCN 秩父
ちちゅう #T35*1 地中
ちちゅうかい #CN 地中海
ちぢ #M5r*38 縮
ちぢま #R5r*2 縮ま
ちぢめ #KSr 縮め
ちぢわ #CN 千々石
ちっきょ #T30 蟄居
ちっさ #KY*1 ちっさ
ちっさな #RT*1 ちっさな
ちっそ #T35*1 窒素
ちっそく #T30 窒息
ちっとも #F14*6 ちっとも
ちっぷ #T30*15 チップ
ちっぷべつ #CN 秩父別
ちっぽけ #T05 ちっぽけ
ちつ #T35*4 膣 #KJ*1 秩 膣 #KJ 窒
ちつじょ #T35*1 秩序
ちつせん #T35*1 膣栓
ちづこ #JNM 千鶴子
ちてい #T35 地底
ちてき #T05*8 知的
ちてん #T35*3 地点
ちと #F14*1 ちと
ちとし #JCN 千歳
ちとせ #T35*2 千歳 #JCN 千歳
ちどうせつ #T35 地動説
ちどり #T35*1 千鳥
ちな #M5*8 因 #CN 知名
ちなつ #JNM 千夏
ちなみに #CJ*71 ちなみに #CJ*22 因みに
ちにちは #T35 知日派
ちねつ #T35*1 地熱
ちねん #JCN 知念
ちの #JCN 茅野 #JN 千野
ちのう #T35*6 知能
ちはい #T30 遅配
ちはら #JNS 茅原
ちば #JCN 千葉
ちばし #R5r*1 血走
ちばん #T35*8 地番
ちひょう #T35 地表
ちひろ #JCN 千尋
ちぶ #CN 知夫 #T35 恥部
ちぶさ #T35*1 乳房
ちへい #T35 地平
ちへん #T35 地変
ちべっと #CN チベット
ちほう #CNSUC1*16 地方 #T35*1 痴呆 #T35 地方
ちまき #T35 粽
ちまた #T35 巷
ちまよ #W5 血迷
ちみつ #T05*1 緻密
ちみもうりょう #T35*1 魑魅魍魎
ちめい #T35*22 地名 #T35*5 致命 #T35 知名 知命
ちめいど #T35 知名度
ちゃ #T35*23 茶
ちゃーじ #T30*1 チャージ
ちゃーたー #T30*1 チャーター
ちゃーと #T35*6 チャート
ちゃーはん #T35*2 チャーハン 炒飯
ちゃいな #T35*2 チャイナ #CN チャイナ
ちゃいろ #T15 茶色
ちゃき #T35 茶器
ちゃく #CNSUC1*35 着 #KJ*2 嫡 #T30*1 着 #JS 着 #KJ 摘
ちゃくい #T30 着衣
ちゃくがん #T30*2 着眼
ちゃくし #T35 嫡子
ちゃくしゅ #T30*2 着手
ちゃくしゅつ #T30*1 嫡出
ちゃくしょう #T30*1 着床
ちゃくしょく #T30 着色
ちゃくしん #T30*4 着信
ちゃくじつ #T05*2 着実
ちゃくすい #T30 着水
ちゃくせき #T30 着席
ちゃくそう #T30*1 着想
ちゃくだつ #T30*1 着脱
ちゃくち #T30*1 着地
ちゃくちゃく #F14*10 着々
ちゃくにん #T30 着任
ちゃくばらい #T30*1 着払
ちゃくもく #T30*4 着目
ちゃくよう #T30 着用
ちゃくりく #T30*1 着陸
ちゃくりゅう #T35 嫡流
ちゃしゃく #T35 茶杓
ちゃじゅ #T35 茶寿
ちゃせん #T35 茶筅
ちゃたん #CN 北谷
ちゃっか #T30 着火
ちゃっかり #F04*4 ちゃっかり
ちゃっこ #T30*13 着呼
ちゃっこう #T30*1 着工
ちゃねる #T35*25 チャネル
ちゃのま #T35*1 茶の間
ちゃぺる #T35*1 チャペル
ちゃや #T35*1 茶屋
ちゃりてぃ #T35 チャリティ
ちゃれんじ #T30*8 チャレンジ
ちゃわん #T35*2 茶碗
ちゃん #JNSUC*255 ちゃん
ちゃんく #T35*4 チャンク
ちゃんす #T35*3 チャンス
ちゃんすう #T35*1 昌洙
ちゃんと #F12*45 ちゃんと
ちゃんねる #T35*10 チャンネル
ちゅーとりある #T35*16 チュートリアル
ちゅーな #T35*4 チューナ
ちゅーりっぷ #T35 チューリップ
ちゅーりんぐ #JN チューリング
ちゅーん #T30*3 チューン
ちゅう #T35*255 中 #S5*161 注 #T30*161 注 #T35*43 忠 #KJ*2 昼 #T35*1 宙 #KJ*1 抽 柱 虫 #KJ 丑 #S5 沖 #T30 沖 #KJ 厨 #CNSUC1 中 #PRE 中 #SUC 中 #KJ 仲 衷 註 酎 鋳 駐 #PRE 駐 #KJ 紬 肘 紐
ちゅうい #T30*160 注意
ちゅういち #JNM 忠一
ちゅういぶか #KY*2 注意深
ちゅういぶこう #KYU*1 注意深う
ちゅういほう #T35 注意報
ちゅうおう #T35*22 中央 #CN 中央
ちゅうか #T35*10 中華 #CN 中和
ちゅうかい #T30*2 仲介 #T30 注解
ちゅうかく #T35*2 中核 #T35*1 中隔
ちゅうかん #T35*42 中間
ちゅうかんし #T35*1 中間子
ちゅうがえり #T35*1 宙返り
ちゅうがく #T35*1 中学
ちゅうがくせい #T35*3 中学生
ちゅうがた #T15 中型
ちゅうがっこう #T35*1 中学校
ちゅうき #T35*1 中期 #T30 注記
ちゅうきち #JNM 忠吉
ちゅうきぼ #T05*2 中規模
ちゅうきゅう #T35*1 中級
ちゅうきょう #CN 中京
ちゅうきょり #T15 中距離
ちゅうきん #T30 忠勤
ちゅうきんとう #CN 中近東
ちゅうぎ #T30 忠義
ちゅうくう #T05*1 中空
ちゅうけい #T30*10 中継
ちゅうけん #T35*20 中堅 #T35*12 中研
ちゅうげん #T35 中元
ちゅうこ #T35*3 中古
ちゅうこく #T30*2 忠告
ちゅうごく #CN 中国
ちゅうごし #T15*1 中腰
ちゅうさ #T35 中佐
ちゅうさい #T30*1 仲裁
ちゅうざ #T30 中座
ちゅうざい #T30*1 駐在
ちゅうし #T30*35 中止 #T30*1 注視
ちゅうしゃ #T30*7 駐車 #T30 注射
ちゅうしゃく #T30*10 注釈
ちゅうしゃしん #T35*1 注射針
ちゅうしゃじょう #T35*46 駐車場
ちゅうしゃとう #T35*1 注射筒
ちゅうしゅう #T35 仲秋
ちゅうしゅつ #T30*14 抽出
ちゅうしょう #T30*31 抽象 #T30*25 中傷 #T35*2 中小
ちゅうしょく #T30*11 昼食
ちゅうしん #T35*59 中心 #T35 忠臣 衷心
ちゅうしんぐら #KK 忠臣蔵
ちゅうじ #T35 中耳 仲仕 #JNM 忠治
ちゅうじつ #T05*5 忠実
ちゅうじゅん #T35*8 中旬
ちゅうじょう #T35 中将
ちゅうすい #T30 注水 #T35 虫垂
ちゅうすう #T35*1 中枢
ちゅうず #CN 中主
ちゅうせい #T35*2 中世 #T15*1 中性 #T05*1 忠誠
ちゅうせいし #T35*1 中性子
ちゅうせん #T30*6 抽選
ちゅうぜつ #T30 中絶
ちゅうそく #T30 中速
ちゅうそつ #T35 中卒
ちゅうぞう #JNM 忠三 #T30 鋳造
ちゅうたい #T30*1 中退 #T35*1 中隊
ちゅうだん #T30*20 中断
ちゅうち #T30*1 中置
ちゅうちょ #T30*1 躊躇
ちゅうっぱら #T15 中っ腹
ちゅうてつ #T35 鋳鉄
ちゅうてん #T35 中天 中点
ちゅうと #T35*10 中途
ちゅうとう #T15 中等
ちゅうどく #T30*1 中毒
ちゅうなごん #T35 中納言
ちゅうなん #CN 仲南
ちゅうなんべい #CN 中南米
ちゅうにち #T30 駐日
ちゅうにゅう #T30*2 注入
ちゅうねん #T35*1 中年
ちゅうのう #T35 中濃
ちゅうはいよう #T35*1 中胚葉
ちゅうばつ #T30 誅伐
ちゅうばん #T35 中盤
ちゅうひ #T35*1 中皮
ちゅうふく #T35 中腹
ちゅうぶ #T35*7 中部 #CN 中部
ちゅうぶう #T35 中風
ちゅうぶん #T35 中文
ちゅうべい #CN 中米
ちゅうぼう #T35 厨房
ちゅうもく #T30*36 注目
ちゅうもん #T30*24 注文 #T35*1 中門
ちゅうや #T35 昼夜
ちゅうりつ #T30*1 中立
ちゅうりゃく #T30*31 中略
ちゅうりゅう #T35 中流
ちゅうりょく #T30 注力
ちゅうるい #CN 忠類
ちゅうわ #T30 中和
ちゅにじあ #CN チュニジア
ちゆ #T30*1 治癒
ちょ #JNSUC*31 著 #KJ*3 猪 #KJ*1 樗 #KJ 緒 瀦 苧 #T35 著 #KJ 貯 箸 儲
ちょーく #T35*1 チョーク
ちょい #JSSUC*3 ちょい
ちょいと #F14*1 ちょいと
ちょう #T35*114 町 #T35*68 長 #KJ*13 超 #CNSUC1*6 町 #KJ*3 丁 #NN*3 兆 #T35*3 庁 #T30*2 朝 #T30*1 弔 #KJ*1 張 潮 聴 #T35*1 腸 蝶 #KJ*1 調 鳥 #KJ 重 澄 #JS 丁 #KJ 凋 #T35 喋 #T30 寵 #JS 帖 #KJ 帖 #SUC 帳 #T35 帳 #SUC 庁 #KJ 彫 #T30 徴 #KJ 懲 挑 暢 牒 眺 脹 #SUC 調 #KJ 諜 #PRE 超 #KJ 跳 銚 #SUC 長 #ZX 長 #KJ 頂 蔦 挺 微
ちょうあい #T30 寵愛
ちょうい #T30*1 弔慰 #T35 弔意
ちょういん #T30*1 調印
ちょうえい #T35 町営
ちょうえき #T35 懲役
ちょうえつ #T30*1 超越
ちょうおん #T30*2 聴音 #T35*1 長音
ちょうおんぱ #T35*1 超音波
ちょうか #T30*1 超過
ちょうかい #T30 懲戒 #T35 朝会 町会 #JCN 鳥海
ちょうかいりょく #T35*8 聴解力
ちょうかく #T35*2 聴覚
ちょうかん #T35*1 朝刊 腸管 長官 #T30 鳥瞰
ちょうかんまく #T35*1 腸間膜
ちょうき #T15*2 長期
ちょうきち #JNM 長吉
ちょうきょう #T30 調教
ちょうきょり #T15*3 長距離
ちょうきん #T30 彫金 超勤
ちょうけし #T30 帳消し
ちょうげん #T30 調弦
ちょうこう #T35 兆候 #T30 聴講 #CN 長江 #T30 長考
ちょうこうぜつ #T35 長広舌
ちょうこうそく #T05*1 超高速
ちょうこく #T30*1 彫刻 #T30 超克
ちょうごう #T30*1 調合 #T35 調号
ちょうさ #T30*246 調査
ちょうさん #T30 逃散
ちょうし #T35*35 調子 #JCN 銚子 #T35 銚子 長子
ちょうしゃ #T35*1 庁舎
ちょうしゅ #T30 聴取
ちょうしゅう #T35*6 聴衆 #T30*2 徴収
ちょうしょ #T35*16 調書 #T35*4 長所
ちょうしょう #T35 弔鐘 #T30 長嘯 嘲笑
ちょうしょく #T30*2 朝食
ちょうしん #T35 弔心 #T05 長身
ちょうじ #T35 弔辞 #JNM 長治
ちょうじかん #T15*4 長時間
ちょうじゃ #T35*1 長者
ちょうじゅ #T05*1 長寿
ちょうじょ #T35*1 長女
ちょうじょう #T35*3 頂上 #T30 重畳
ちょうじり #T35*1 帳尻
ちょうじろう #JNM 長次郎
ちょうじん #T35*1 超人
ちょうせい #T30*32 調整 #T30 調製 #JCN 長生
ちょうせつ #T30*6 調節
ちょうせん #T30*7 挑戦 #CN 朝鮮
ちょうぜん #F00 超然
ちょうそん #T35 町村 長孫
ちょうぞう #T35 彫像 #JNM 長蔵
ちょうたく #T30 彫琢
ちょうたつ #T30*9 調達
ちょうたろう #JNM 長太郎
ちょうたん #T35*1 長短
ちょうだい #T30*6 頂戴 #T05*1 長大
ちょうちゃく #T30 打擲
ちょうちょう #T35*1 蝶々 #T30 喋々 #T35 町長 長調
ちょうちょうはっし #F03 打々発止
ちょうてい #T30*1 調停 #T35 朝廷
ちょうてん #T35*8 頂点
ちょうでん #T35 弔電
ちょうでんどう #T35*1 超電導
ちょうど #T06*22 ちょうど #T06*8 丁度 #T35 調度
ちょうどきゅう #T15 超弩級
ちょうない #T35*1 腸内 #T35 町内
ちょうなん #T35*2 長男 #JCN 長南
ちょうは #T35 長波
ちょうはつ #T30*1 挑発 #T30 徴発
ちょうば #T35 帳場
ちょうばつ #T30 懲罰
ちょうひょう #T35*57 帳票
ちょうふ #JCN 調布
ちょうふく #T30*27 重複
ちょうぶつ #T35*2 長物
ちょうぶん #T35*2 長文
ちょうへい #T30*1 徴兵 #JNM 長平
ちょうへん #T35*1 長編 #T35 長辺
ちょうほう #T00*2 重宝 #T35 諜報
ちょうほうけい #T35*6 長方形
ちょうほんにん #T35*1 張本人
ちょうぼ #T35*1 帳簿
ちょうみ #T30*1 調味
ちょうみりょう #T35*4 調味料
ちょうみん #T35*1 町民
ちょうめ #JS*16 丁目 #T35*13 丁目
ちょうめん #T35 帳面
ちょうもく #T35*1 鳥目
ちょうもん #T30 弔問
ちょうやく #T30 跳躍
ちょうよう #CN 町陽
ちょうらく #T30 凋落
ちょうり #T30*2 調理
ちょうりつ #T35*1 町立 #T30 調律
ちょうりゅう #T35*1 潮流
ちょうりょう #KK*1 潮陵 #T30 跳梁
ちょうりょく #T35 張力 聴力
ちょうれい #T30*1 朝礼
ちょうれいぼかい #T35*1 朝令暮改
ちょうろう #T35*1 長老 #T30 嘲弄
ちょうわ #T30*4 調和
ちょきん #T30*2 貯金
ちょく #KJ*16 直 #KJ*2 勅 #KJ*1 捗
ちょくえい #T30 直営
ちょくげき #T30*1 直撃
ちょくご #T35*18 直後 #T35*1 勅語
ちょくさい #T05 直截
ちょくし #T30 直視
ちょくしゃ #T30*1 直射
ちょくしん #T30*1 直進
ちょくじ #T30*1 直示
ちょくせき #T35*3 直積
ちょくせつ #T15*224 直接 #T05 直截
ちょくせん #T35*13 直線
ちょくぜん #T35*44 直前
ちょくそう #T30*1 直送
ちょくぞく #T30*1 直属
ちょくちょく #F14*3 ちょくちょく
ちょくつう #T30*2 直通
ちょくとう #T30 直答
ちょくはん #T30*1 直販
ちょくばい #T30 直売
ちょくほうたい #T35*2 直方体
ちょくめい #T35 勅命
ちょくめん #T30*3 直面
ちょくやく #T30*1 直訳
ちょくりつ #T30 直立
ちょくりゅう #T35*1 直流
ちょくれつ #T30*3 直列
ちょくわ #T35*2 直和
ちょこ #T35*38 チョコ #T35 猪口
ちょこれーと #T35*3 チョコレート
ちょさく #T30*11 著作
ちょしゃ #T35*21 著者
ちょしょ #T35*2 著書
ちょじゅつ #T30 著述
ちょぞう #T30*1 貯蔵
ちょちく #T30 貯蓄
ちょっか #T35*5 直下
ちょっかく #T15*1 直角
ちょっかつ #T30 直轄
ちょっかん #T30*115 直観 #T30*4 直感
ちょっき #T30*5 直帰
ちょっけい #T35*3 直径 直系
ちょっけつ #T30*13 直結
ちょっこう #T30*10 直行 #T30*3 直交
ちょっと #F14*220 ちょっと #T31*157 ちょっと
ちょっぴり #F14*2 ちょっぴり
ちょとつもうしん #T35*1 猪突猛進
ちょびっと #F12 ちょびっと
ちょめい #T05*1 著明 #T05 著名
ちょりゅう #T30*1 貯留
ちよ #T35*1 千代 #JNM ちよ チヨ #JN 千代
ちよかわ #CN 千代川
ちよくら #JNS 千代倉
ちよこ #JNM 千代子
ちよだ #JCN 千代田
ちよの #JNM 千代乃
ちら #S5*2 散ら
ちらか #R5*2 散らか #S5*2 散らか
ちらし #T35*2 チラシ
ちらっ #F06*2 ちらっ
ちらつ #K5r*1 ちらつ
ちらば #R5r 散らば
ちらり #F06*1 ちらり
ちらん #JCN 知覧
ちり #T35*10 地理 #T35*1 塵 #CN チリ
ちりばめ #KS*1 散りばめ
ちりゅう #JCN 知立
ちりょう #T30*4 治療
ちるちる #JN チルチル
ちわ #T35*1 痴話
ちん #T30*10 チン #T35*3 朕 #KJ*3 賃 #T35*1 珍 #KJ*1 陳 椿 #KJ 砧 沈 珍 鎮 #ZX 陳
ちんあげ #T30*1 賃上げ
ちんあつ #T30 鎮圧
ちんか #T30 沈下 鎮火
ちんき #T35 チンキ #T05 珍奇
ちんきゃく #T35 珍客
ちんぎん #T35*1 賃金
ちんこう #T30 沈降
ちんさ #T35*1 沈渣
ちんさげ #T30 賃下げ
ちんざ #T30*3 鎮座
ちんしゃ #T30*1 陳謝
ちんしゃく #T30 賃借
ちんじ #T35 珍事
ちんじゅ #T35 鎮守
ちんじゅつ #T30*1 陳述
ちんじょう #T30 陳情
ちんせい #T30 鎮静
ちんぜい #JCN 鎮西
ちんたい #T30*1 沈滞 賃貸
ちんだん #T35 珍談
ちんちょう #T30*1 珍重
ちんつう #T30 鎮痛
ちんでん #T30 沈澱
ちんとう #T35 枕頭
ちんぼつ #T30 沈没
ちんぽこ #T35*1 似指
ちんみ #T35 珍味
ちんもく #T30*2 沈黙
ちんれつ #T30 陳列
ぢ #T35*14 痔
っこ #D2T35*6 っこ #KY*1 っこ
っぱなし #T05*2 っぱなし
っぽ #KY*6 っぽ
つ #K5*255 付 #K5*155 つ #JS*114 つ #G5r*48 次 #K5*48 着 #M5*27 積 #T35*20 津 #KJ*10 都 #M5*4 つ #G5r*4 継 #R5*3 つ #K5*2 就 #G5*1 嗣 #R5r*1 吊 釣 #K5r*1 突 #KJ 晦 #M5r 詰 #K5 衝 #G5r 接 #G5 注 #JCN 津 #KJ 柘 #M5 摘 #K5r 撞 #K5 附
つーりすと #T35*1 ツーリスト
つーりんぐ #T30*3 ツーリング
つーる #T35*6 ツール
つーるきっと #T35 ツールキット
つあー #T30*12 ツアー
つい #JS*76 対 #F14*62 つい #T35*42 対 #KJ*1 追 #KJ 堆 墜 椎 槌 鎚
ついおく #T30 追憶
ついか #T30*255 追加
ついかんばん #T35 椎間板
ついき #T30*8 追記 #JCN 築城 #JNS 立木
ついきゅう #T30*6 追求 #T30*3 追及 #T30*1 追究
ついげき #T30 追撃
ついし #T30*2 追試 #T30*1 墜死
ついしけん #T30 追試験
ついしょう #T30 追従
ついしん #T35*16 追伸
ついじゅう #T30*2 追従
ついずい #T30*1 追随
ついせき #T30*4 追跡
ついそ #T30 追訴
ついたち #T35*1 1日 一日 #T35 １日
ついちょう #T30 追徴
ついつい #F14*2 ついつい
ついては #CJ*10 ついては
ついで #T35*6 ついで
ついでに #F14*36 ついでに
ついとう #T30 追悼
ついとつ #T30 追突
ついに #F14*3 遂に
ついにん #T30*2 追認
ついば #M5 啄
ついほう #T30 追放
ついや #S5*9 費 #S5*2 費や
ついらく #T30*2 墜落
ついろく #T30 追録
ついん #T35*3 ツイン
つう #JS*101 通 #ZX*28 通 #KJ*2 痛 #SUC 通 #T35 通
つういん #T30*3 通院
つうえん #T30 通園
つうか #T30*31 通過 #T35*1 通貨
つうかい #T05*1 痛快
つうかく #T35 痛覚
つうかん #T30*2 痛感 #T35*1 通巻 #T35 通関
つうがく #T30*2 通学
つうき #T35 通気
つうきん #T30*28 通勤
つうけん #KK*2 通研
つうげき #T30 痛撃
つうこう #T30*1 通行
つうこく #T30*2 通告 #T30 痛哭
つうこん #T30 痛恨
つうさん #T35*1 通産 #T30*1 通算
つうさんしょう #KK*6 通産省 #T35*4 通産相
つうしょう #T30*4 通称 #T35*1 通商
つうしん #T30*255 通信
つうしんもう #T35 通信網
つうじ #T35 通時
つうじょう #T05*102 通常
つうせつ #T05 痛切 #T35 通説
つうぞく #T05 通俗
つうたつ #T30*2 通達
つうち #T30*32 通知
つうちょう #T35*10 通帳
つうてん #T35 痛点
つうてんかく #CN 通天閣
つうでん #T30 通電
つうどく #T30 通読
つうねん #T35*1 通念
つうふう #T35 通風
つうほう #T30*2 通報
つうやく #T30*6 通訳
つうよう #T30*5 通用
つうれい #T35*1 通例
つうれつ #T05*1 痛烈
つうろ #T35*3 通路
つうわ #T30*2 通話
つえ #T35 杖
つか #W5r*255 使 #T35*9 塚 #M5r*8 掴 #R5*2 浸か #W5r*1 遣 #KJ*1 束 #R5 漬か #KJ 柄
つかいがって #T35*4 使い勝手
つかいこ #M5r*3 使い込
つかいこな #S5r*3 使いこな
つかいわけ #KSr*3 使い分け
つかえ #KS*24 仕え
つかぐち #JCN 塚口
つかこし #JNS 塚越
つかごし #JNS 塚越
つかさ #T35*11 司 #JNM 司
つかさど #R5*1 司
つかだ #JCN 塚田
つかはら #JCN 塚原
つかま #R5r*3 捕ま
つかまえ #KSr*12 捕まえ
つかみあ #W5r つかみ合
つかもと #JNS 塚本
つかれ #KSr*85 疲れ
つが #KJ*1 栂 #JCN 都賀
つがい #T35 番
つがる #JCN 津軽
つがわ #JCN 津川
つき #JS*155 月 #T35*3 月 #KSr*3 尽き #SUC つき #T35 つき #KJ 槻 #SUC 付 #T35 付き
つきあ #W5r*38 つき合 #W5r*8 つきあ #W5r 付き合
つきあげ #KSr 突き上げ
つきあわせ #KSr 突き合わせ
つきがけ #T35 月掛け
つきがせ #CN 月ヶ瀬
つきがた #JCN 月形
つきぎめ #T35 月極め
つきぎり #T35 月限
つきさ #S5r 突き刺
つきさむ #CN 月寒
つきじ #JCN 築地
つきすす #M5*1 突き進
つきそ #W5r 付き添
つきそい #T30*1 付添 #T30 付添い
つきだ #S5r*1 突き出
つきだて #JNS 築館
つきつけ #KS*2 突き付け
つきつめ #KSr*1 突き詰め
つきづき #T35*1 月々
つきとめ #KS*2 突き止め
つきはな #S5r*1 突き放
つきばらい #T30 月払い
つきひ #T35*4 月日
つきべつ #T15*1 月別
つきまと #W5*1 付きまと
つきやま #T35 築山
つきよ #T35*1 月夜
つきよの #CN 月夜野
つぎ #T35*255 次 #T35*2 つぎ
つぎお #JNM 次男 次夫 次雄
つぎこ #M5r*2 つぎこ
つぎた #S5r*1 継ぎ足
つぎつぎ #T35*4 次々
つぎめ #T35 継目
つく #R5*255 作 #S5*7 尽く #R5r*3 つく #R5*2 創 #R5r*2 造 #KJ*1 筑
つくい #JCN 津久井
つくいこ #CN 津久井湖
つくえ #T35*42 机
つくし #JCN 筑紫
つくだ #T35*3 佃
つくだに #T35 佃煮
つくづく #F14*1 つくづく
つくで #JCN 作手
つくば #CN つくば #JCN 筑波
つくばし #CN つくば市
つくばね #KK 筑波嶺
つくぼ #JCN 都窪
つくみ #JCN 津久見
つくり #KJ*6 旁 #T35 作り
つくりあげ #KS*3 作り上げ #KS 造り上げ
つくりかた #T35*20 作り方
つくりだ #S5*23 つくり出 #S5*5 作り出 #S5 創り出
つくろ #W5r 繕
つぐ #JCN 津具
つぐな #W5r*1 償
つぐみ #T35 ツグミ 鶫
つけ #KS*57 付け #KSr*4 つけ #KS*3 漬け #D2T35 つけ #KJ 漬 #KS 点け
つけかた #T35*5 付け方
つけくわえ #KSr*22 付け加え
つけこ #M5r 漬け込 #M5 付け込
つけた #S5r*4 付け足
つけち #JCN 付知
つけねら #W5*1 付け狙
つけもの #T35*2 漬物
つけやきば #T35 付け焼き刃 付焼刃
つげ #KS*6 告げ #JCN 拓殖 #KJ 柘 #JCN 都祁
つげぐち #T30 告げ口
つごう #T30*74 都合
つしま #JCN 対馬 津島
つじ #T35*5 辻 #JCN 辻
つじい #JNS 辻井
つじつま #T35*2 つじつま 辻褄
つじの #JNS 辻野
つた #T35*3 蔦
つたう #JNM 伝
つたえ #KS*64 伝え
つたな #KY*2 拙
つたのう #KYU*1 拙う
つたわ #R5*14 伝わ
つだ #JCN 津田
つだぬま #CN 津田沼
つち #T35*3 槌 土 #KJ 壌 椎 #T35 鎚
つちうら #JCN 土浦
つちか #W5*3 培
つちくれ #T35 土くれ
つちだ #JNS 土田
つちのえ #T35 戊
つちや #JCN 土屋
つちやま #JCN 土山
つっかえ #S5*1 突っ返
つっかか #R5 突っかか
つっけんどん #T05 つっけんどん 突慳貪
つっこ #M5r*4 突っ込
つっつ #K5*1 突っつ
つっぱ #R5r*1 突っ張
つっぱし #R5*1 つっ走
つっぷ #S5 つっぷ つっ伏
つつ #T35*3 筒 #M5r*3 包
つつい #JCN 筒井
つつうらうら #T35 津々浦々
つつが #JCN 筒賀
つつがな #KY*2 つつがな
つつがのう #KYU*1 つつがのう
つつし #M5r つつし 謹 慎
つつしんで #F14*1 謹んで
つつみ #T35*2 堤 #JNS 堤 #T35 包
つつみこ #M5r 包み込
つづ #K5r*123 続 #R5r*6 綴 #KSr*2 つづ #R5r*1 つづ
つづき #JCN 綴喜 #JNS 都築
つづけ #KS*1 続け
つづみ #T35 鼓
つづり #T35*5 綴
つて #T35 つて 伝手
つとに #F14 つとに
つとま #R5*1 努ま #R5 勤ま
つとむ #JNM 勤 努 勉 務 力
つとめ #KSr*10 勤め #KSr*2 努め #KSr*1 つとめ #KSr 勉め 務め
つとめさき #T35*1 勤め先
つとめて #F14 努めて
つど #T35*6 都度 #W5r*2 集
つな #G5r*73 つな #G5r*23 繋 #T35*1 綱 #JCN 津名
つなが #R5r*120 つなが #R5*40 継 #R5r*3 繋が
つなぎ #T35*1 繋 #JCN 津奈木
つなぎとめ #KSr つなぎ止め
つなげ #KS*1 繋げ
つなしま #JCN 綱島 #JNS 綱嶋
つなみ #T35 津波
つなん #CN 津南
つね #T35*53 常 #JNM つね #T35 恒
つねあき #JNM 常明
つねお #JNM 恒男 恒夫 恒雄 常男 常夫 常雄
つねきち #JNM 常吉
つねずみ #JCN 常澄
つねづね #T35*2 常々
つねとう #JNS 恒藤 恒任
つねとし #JNM 恒敏
つの #R5*4 募 #R5*1 つの #T35*1 角 #JCN 都濃 都農
つのだ #JNS 角田
つば #T35 唾 鍔
つばき #T35 椿
つばさ #T35*2 翼
つばた #JCN 津幡
つばめ #CN 燕 #T35 燕
つぶ #S5r*8 潰 #JS*5 粒 #R5*4 つぶ #T35*3 粒
つぶて #T35 礫
つぶや #K5r*2 つぶや
つぶより #T05 粒選り
つぶら #T05 つぶら
つぶれ #KSr*16 潰れ
つべこべ #F06 つべこべ
つべつ #JCN 津別
つぼ #JS*3 坪 #T35*3 坪 #T35 壷
つぼい #JNS 坪井
つぼすう #T35 坪数
つぼた #JNS 坪田
つま #R5r*31 つま #T35*8 妻 #R5r*6 詰ま #M5r*1 摘 #M5r つま #KJ 嬬 #JCN 都万
つまごい #JCN 嬬恋
つまさき #T35 爪先
つまず #K5r*1 つまず
つまづ #K5r つまづ 躓
つまみだ #S5 つまみ出
つまらな #KY*6 つまらな
つまらのう #KYU*1 つまらのう
つまり #CJ*61 つまり
つみ #T05*19 罪
つみあげ #KSr*1 積み上げ
つみかえ #T30 積み替え
つみかさな #R5r 積み重な
つみかさね #KSr*1 積み重ね
つみき #T35 積木
つみこ #M5r 積み込
つみたて #KSr*3 積み立て #T30*2 積立て
つみだ #S5r 積み出
つみに #T35 積荷
つみのこ #S5r*1 積み残 #R5r 積み残
つみほろぼし #T30*1 罪滅ぼし
つむ #G5*1 紡
つむぎ #T35 紬
つむら #JNS 津村
つめ #KSr*28 詰め #KSr*20 つめ #T35*2 爪
つめあわせ #KSr 詰め合わせ #T30 詰合せ
つめかえ #KSr 詰め替え
つめかけ #KS 詰めかけ
つめこ #M5r*4 詰め込
つめご #T35 詰碁
つめしょ #T35 詰所
つめしょうぎ #T35 詰将棋
つめた #KY*14 冷た
つめとう #KYU*2 冷とう
つも #R5*3 積も
つもり #T35*76 つもり
つや #T30*1 通夜 #T35 艶
つやざき #JCN 津屋崎
つやま #JCN 津山
つゆ #T35*6 梅雨 #T35*1 露
つよ #KYmi*45 強
つよう #KYU*2 強う
つよき #T05*1 強気
つよし #JNM 毅 強 剛
つよま #R5r*1 強ま
つよめ #KSr*4 強め
つら #KY*21 辛 #KY*2 つら #T35*1 面
つらがまえ #T35 面構え
つらな #R5r*3 連な
つらぬ #K5*2 貫
つらぬきとお #S5 貫き通
つらね #KS*1 連ね
つり #KJ*2 吊 釣
つりー #T35*7 ツリー
つりあい #T35*1 釣合
つりあが #R5r つり上が
つりあげ #KSr つり上げ
つりだ #S5r 吊り出 釣り出
つる #T35*6 鶴 #S5r*1 吊 #T35 弦 #JCN 都留 #KJ 敦 蔓
つるい #JCN 鶴居
つるおか #JCN 鶴岡
つるが #JCN 敦賀
つるがしま #CN 鶴ヶ島
つるぎ #T35*1 剣 #JCN 鶴来
つるた #JCN 鶴田
つるだ #JCN 鶴田
つるっぱげ #T15 つるっ禿
つるはし #JCN 鶴橋
つるべ #T35 釣瓶
つるまい #JNS 鶴舞
つるみ #JCN 鶴見
つれ #KS*47 つれ #KSr*25 連れ
つれかえ #R5*1 連れ帰
つれこ #M5r 連れ込
つれそ #W5r 連れ添
つれだ #S5r 連れ出
つれづれぐさ #KK 徒然草
つれもど #S5r 連れ戻
つろう #KYU*2 辛う #KYU*1 つろう
つわの #JCN 津和野
つんのめ #R5 つんのめ
つんぼ #T15*1 聾
づ #T35*32 津
づか #T35*6 塚
づかい #SUC*6 使い #N2T10 づかい
づき #KJ 月
づくえ #T35*3 机
づくり #N2T30*3 作り
づけ #JS*5 付 #KSr づけ #SUC づけ #N2T30 漬け #KSr 付け
づめ #T30*16 詰め
づよ #KY*2 強
づよう #KYU*1 強う
づら #D2KY*6 辛 #D2KY づら
て #T35*255 手 #T35*22 〒 #D2T35*10 手 #R5r*10 照 #JS 手
てーぶる #T35*157 テーブル
てーぷ #T35*76 テープ
てーま #T35*28 テーマ
てあし #T35*1 手足
てあつ #KY*1 手厚
てあつう #KYU*1 手厚う
てあて #T30*10 手当
てあら #T05*1 手荒
てあらい #T30 手洗い
てぃー #T35*2 ティー
てぃしゅ #T35*1 ティシュ
てぃしゅー #T35*1 ティシュー
てぃっしゅ #T35*1 ティッシュ
てい #KJ*34 定 #KJ*18 低 #S5*13 呈 #T30*13 呈 #KJ*10 亭 #KJ*5 貞 #T30*5 訂 #KJ*4 帝 #JNSUC*2 邸 #KJ*2 鼎 #T35*1 体 丁 #KJ*1 綴 停 堤 提 梯 禎 程 #KJ 鵜 滞 #PRE 低 #KJ 偵 剃 #T35 底 #KJ 庭 廷 弟 悌 抵 #T30 挺 #KJ 汀 碇 締 艇 諦 蹄 逓 #T35 邸 #KJ 鄭 釘 薙
ていあん #T30*75 提案
ていいち #JNM 貞一
ていいん #T35*5 定員
ていえん #T35 庭園
ていおう #T35*2 帝王
ていおん #T05*1 低音 #T05 低温
ていか #T30*11 低下 #T35*6 定価
ていかく #T35 定格
ていかん #T35 定款
ていがく #T35*2 定額 #T05 低額
ていがくねん #T35*1 低学年
ていき #T35*32 定期 #T30*8 提起
ていきあつ #T35*1 低気圧
ていきゅう #T05*1 低級 #T35 庭球
ていきゅうび #T35*2 定休日
ていきょう #T30*255 提供
ていきん #T35 提琴
ていぎ #T30*255 定義
ていくう #T35 低空
ていけい #T35*4 定型 #T30*2 提携
ていけつ #T30*1 締結
ていけん #T35 定見
ていげん #T30*3 低減 #T30*1 提言
ていこ #JNM 貞子
ていこう #T30*14 抵抗 #T35*4 定項
ていこく #T35*3 帝国 #T35 定刻
ていさい #T35*11 体裁
ていさつ #T30*1 偵察
ていざいは #T35 定在波
ていし #T30*35 停止
ていしき #T35*16 定式
ていしつ #T05 低湿
ていしゃ #T30*1 停車
ていしゅ #T35*3 亭主
ていしゅかんぱく #T15 亭主関白
ていしゅく #T05 貞淑
ていしゅつ #T30*167 提出
ていしゅつび #T35 提出日
ていしょう #T30*2 提唱
ていしょく #T35*2 定食 #T30*2 抵触 #T35 定職
ていしん #T35*1 逓信
ていじ #T35*31 定時 #T30*5 提示 #T30*1 呈示 #JNM 貞次 貞治 貞二
ていじせい #T35 定時制
ていじゅう #T30 定住
ていじょう #T05*2 定常 #T30 呈上
ていすいじゅん #T05*1 低水準
ていすう #T35*57 定数
ていせい #T30*34 訂正 #T35*1 定性
ていせいてき #T15 定性的
ていせつ #T35*1 定説
ていせん #T30 停戦
ていそ #T30 提訴
ていそう #T35*1 貞操
ていそく #T05*5 低速
ていぞく #T05 低俗
ていたい #T30*2 停滞
ていたく #T35*1 邸宅
ていたらく #T35 ていたらく
ていだん #T30 鼎談
ていち #T30*1 定置 #T35 低地
ていちゃく #T30*2 定着
ていちょう #T05*2 丁重 #T05 低調
ていっぱい #T15 手一杯
ていてん #T35 定点
ていでん #T30*4 停電
ていと #T35 帝都
ていとう #T30 低頭 #T35 抵当
ていど #T15*154 程度 #T15*1 ていど
ていどさ #T35 程度差
ていね #CN テイネ 手稲
ていねい #T05*5 丁寧
ていねん #T35*1 定年
ていのう #T05 低能
ていはく #T30 停泊
ていばん #T35 定番
ていひょう #T35*1 定評
ていへん #T35 底辺
ていぼう #T35 堤防
ていめい #T30*1 低迷
ていめん #T35*1 底面
ていもう #T30*1 剃毛
ていらく #T30 低落
ていり #T35*52 定理 #T15 低利
ていりつ #T05 低率
ていりゅう #T30*4 停留 #T35*1 底流
ていりょう #T35*1 定量
ていりょうてき #T15 定量的
ていれ #T30*1 手入れ
ていれい #T35 定例
てうす #T05 手薄
ておくれ #T15*1 手遅れ
ておち #T35 手落ち
ており #T35*1 手織
てかげん #T30*3 手加減
てがかり #T35*2 手がかり #T35 手掛かり
てがき #T30*5 手書き
てがけ #KS*1 手掛け #KS 手がけ
てがた #T35*2 手形 #KY 手堅
てがとう #KYU 手堅う
てがみ #T35*38 手紙
てがら #T35*1 手柄
てがる #T05*7 手軽
てき #T15*228 的 #T30*96 敵 #T30*91 適 #S5*80 適 #KJ*50 的 #JS*5 滴 #KJ*1 摘 #KJ 嫡 擢 滴 #N2T16 的 #KJ 笛 鏑
てきい #T35 敵意
てきおう #T30*2 適応
てきかく #T05*4 的確 #T35*1 適格
てきがいしん #T35 敵愾心
てきき #T35*1 適期
てきぎ #T05*10 適宜
てきごう #T30*15 適合
てきさす #CN テキサス
てきざい #T35 適材
てきし #T30 敵視
てきしゅつ #T30*2 摘出
てきしゅつりつ #T35 摘出率
てきじ #T05*1 適時
てきじょ #T30 滌除
てきじん #T35 敵陣
てきすちゃ #T35*2 テキスチャ
てきすと #T35*255 テキスト
てきせい #T05*3 適正 #T35 適性
てきせつ #T05*19 適切
てきたい #T30*1 敵対
てきち #T35 適地
てきちゅう #T30 的中
てきとう #T05*115 適当
てきど #T05*5 適度
てきにん #T05 適任
てきはつ #T30*1 摘発
てきぱき #F06*1 てきぱき
てきひ #T35*1 適否
てきびし #KY 手厳し
てきびしゅう #KYU 手厳しゅう
てきほう #T05 適法
てきやく #T35 適役
てきよう #T30*68 適用 #T30*2 摘要
てきりょう #T35*8 適量
てきれい #T35*2 適齢
てきれいき #T35*1 適齢期
てぎわ #T35 手際
てくすちゃ #T35*1 テクスチャ
てくとろにくす #KK*1 テクトロニクス
てくにかる #T05*5 テクニカル
てくにっく #T35*2 テクニック
てくの #T35*3 テクノ
てくのさいえんす #KK*1 テクノサイエンス
てくのろじ #T35*2 テクノロジ
てくのろじー #T35*5 テクノロジー
てくのろじず #KK*2 テクノロジズ
てくび #T35 手首
てぐすね #T35*1 てぐすね
てぐち #T35 手口
てこ #T35*4 梃
てごたえ #T35 手応え
てごま #T35 手駒
てごめ #T35 手篭め
てごろ #T05*2 手頃 #T05 手ごろ
てごわ #KY*1 手ごわ 手強
てさき #T35 手先
てさげ #T35 手提げ
てざわり #T35*1 手触り
てしお #CN 天塩
てしかが #CN 弟子屈
てしごと #T30 手仕事
てした #T35*2 手下
てしま #JNS 手嶋
てしろぎ #JNS 手代木
てじか #T05*1 手近
てじな #T30 手品
てじま #JNS 手島 手嶋
てじゅん #T35*93 手順
てじゅんしょ #T35 手順書
てじょう #T35 手錠
てすう #T35*5 手数
てすと #T30*255 テスト
てすり #T35*1 てすり
てぜま #T05 手狭
てだし #T30 手出し
てだすけ #T30*2 手助け
てだて #T30*1 手立て
てちがい #T35*2 手違い
てちょう #T35*3 手帳 #T35 手帖
てぢか #T05*1 手近
てっ #S5*5 徹 #SX*5 徹
てっかい #T30*1 撤回
てっかん #T35 鉄管
てっき #T35*1 適期 #T35 敵機 鉄器
てっきょ #T30*1 撤去
てっきょう #T35 鉄橋
てっきり #F14*1 てっきり
てっきん #T35 鉄筋
てっこう #T35*1 鉄鋼 #T35 鉄工 鉄鉱
てっこつ #T35 鉄骨
てっしゅう #T30 撤収
てっしょう #T30 徹宵
てっせい #CN 哲西 #T35 鉄製
てっせん #T35 鉄扇 鉄線
てっそく #T35 鉄則
てったい #T30*2 撤退
てってい #T30*5 徹底
てっとう #T35 鉄塔
てっとうてつび #F14 徹頭徹尾
てっとりばや #KY*3 手っ取り早
てっぱい #T30*1 撤廃
てっぱん #T35*1 鉄板
てっぺい #T30*1 撤兵
てっぽう #T35*2 鉄砲
てつ #T35*57 鉄 #KJ*2 哲 #KJ*1 徹 轍 #KJ 綴 撤 迭 姪
てつお #JNM 哲男 哲夫 哲雄 鉄男 鉄夫 鉄雄
てつがく #T35*10 哲学
てつけきん #T35 手付金
てつこ #JNM 徹子
てつごうし #T35*3 鉄格子
てつざい #T35 鉄材
てつし #JNM 哲史
てつじ #JNM 哲二
てつじん #T35 哲人 鉄人
てつた #CN 哲多
てつたろう #JNM 鉄太郎
てつだ #W5r*30 手伝
てつづき #T30*37 手続き #T30*20 手続
てつどう #T35*1 鉄道
てつびん #T35 鉄瓶
てつぼう #T35 鉄棒
てつみ #JNM 哲美
てつや #T30*8 徹夜 #JNM 哲也
てつり #T35 哲理
てつろう #JNM 哲朗 哲郎
てづか #JNS 手塚
てづくり #T30*4 手作り
てづまり #T10 手詰り
てどり #T35*2 手取り
てなおし #T30*5 手直し
てにす #T30*83 テニス
てぬき #T10*2 手抜き
てぬぐい #T35*1 手ぬぐい #T35 手拭い
てのーる #T35 テノール
てのひら #T35*1 手の平 #T35 掌
てはい #T30*16 手配
てはじめ #T35*2 手始め
てばな #S5r*1 手放
てびき #T35*71 手引 #T30*4 手引き
てふだ #T35 手札
てぶくろ #T35*2 手袋
てべんとう #T35*1 手弁当
てほん #T35*2 手本
てま #T35*27 手間
てまえ #T35*4 手前
てまど #R5*3 手間取
てまり #T35*1 手毬
てまわし #T30 手回し
てみじか #T05*1 手短
てむか #W5 手向か
てもち #T35 手持ち
てもと #T35*31 手元
てら #S5*10 てら #T35*6 寺 #S5*2 照ら #W5r*1 衒
てらい #JCN 寺井
てらうち #JNS 寺内
てらお #JNS 寺尾
てらおか #JNS 寺岡
てらさか #JNS 寺坂
てらさき #JNS 寺崎
てらさわ #JNS 寺沢
てらしま #JCN 寺島 #JNS 寺嶋
てらだ #JCN 寺田
てらどまり #CN 寺泊
てらもと #JNS 寺本
てらやま #JNS 寺山
てりかえ #S5r 照り返
てりつけ #KSr 照りつけ
てる #T35*3 照 #JNM てる #JN テル #T35 輝 #JN 照
てるお #JNM 輝男 輝夫 輝雄 照男 照夫 照雄
てるこ #JNM てる子 輝子 照子
てるみ #JNM 照美
てれ #KSr 照れ
てれか #T35 テレカ
てれっくす #T35 テレックス
てれぱしー #T35*1 テレパシー
てれび #T35*28 テレビ
てれびきょく #T35*2 テレビ局
てれびじょん #T35*1 テレビジョン
てわけ #T30*1 手分け
てわた #S5r*7 手渡
てん #T35*255 点 #KJ*111 典 #T35*32 天 #JNSUC*6 展 #KJ*5 転 #KJ*4 店 #T35*2 ・ #T35*1 テン #KJ*1 展 添 貼 #ZX*1 転 #KJ*1 槙 #KJ 辿 填 #SUC 店 #KJ 纏 甜 顛 #JS 点 #SUC 点 #ZX 点 #KJ 殿
てんい #T30 転移
てんいむほう #T05*1 天衣無縫
てんいん #T35*3 店員
てんえい #CN 天栄
てんか #T35*2 天下 #T30*2 転嫁 #T30*1 添加 転化 点火
てんかい #T30*33 展開 #T30 転回
てんかん #T30*4 転換
てんき #T35*74 天気 #T30*2 転記 #T35 転機
てんきー #T35 テンキー
てんきょ #T30*1 転居
てんきん #T30*3 転勤
てんぎょう #T30 転業
てんくう #T35 天空
てんくぎ #T35 天釘
てんぐ #T35*1 天狗
てんけい #T35*12 典型
てんけん #T30*3 点検
てんこ #T30*1 点呼
てんこう #T35*5 天候 #T30*2 転向 転校
てんごく #T35*2 天国
てんさい #T30*7 転載 #T35*3 天才 #T35 天災
てんさく #T30*7 添削
てんざい #T30*1 点在
てんし #T35*2 天使
てんしゃ #T30*1 転写
てんしゅ #T35*3 店主
てんしゅつ #T30 転出
てんしゅどう #T35*1 天主堂
てんしょ #T30 添書 #T35 篆書
てんしょう #T30 転生
てんしょく #T30*2 転職 #T35*1 天職
てんしん #T30*1 転身 #CN 天津 #T30 転進
てんしんらんまん #T05*1 天真爛漫
てんじ #T30*32 展示 #T35 点字
てんじく #CN 天竺
てんじゅ #T35 天寿
てんじょう #T35*5 天井 #T35*1 点状
てんじん #T35*1 天神
てんすい #CN 天水
てんすう #T35*3 点数
てんせい #T35 天性 #T30 転成
てんせき #T35 転石 #T30 転籍
てんせん #T35*10 点線
てんそう #T30*129 転送
てんそる #T35*2 テンソル
てんぞく #T30*1 転属
てんたい #T35 天体
てんたん #F02 恬淡
てんち #T30*10 転置 #T35*1 天地 #T30 転地
てんちゅう #T35 天誅
てんちょう #T30 転調
てんてき #T35 天敵 #T30 点滴
てんてこまい #T10*1 てんてこまい
てんてん #T35*255 … #T35*8 ‥
てんと #T35*7 テント
てんとう #T30*2 転倒 #T30*1 点灯 #T35 店頭
てんどう #JCN 天童
てんどうせつ #T35 天動説
てんどん #T35 天丼
てんない #T35 店内
てんにゅう #T30 転入
てんにょ #T35*1 天女
てんにん #T30 転任
てんねん #T35 天然
てんねんとう #T35*1 天然痘
てんのう #T35*2 天皇 #CN 天王
てんのうじ #JCN 天王寺
てんのうせい #KK*1 天王星 #T35 天皇制
てんばい #T30 転売
てんぱく #CN 天白
てんぱん #T35 典範
てんびき #T30 天引き
てんびん #T35 天秤
てんぴ #T35*1 天火 #T35 天日
てんぶくろ #T35 天袋
てんぶん #T35 天分
てんぷ #T30*49 添付
てんぷく #T30 転覆
てんぷら #T35*3 てんぷら
てんぷれーと #T35*13 テンプレート
てんぺん #T30 転変
てんぺんちい #T35 天変地異
てんぼう #T30*2 展望
てんぽ #T35*1 テンポ #T35 店舗
てんぽらり #T35*1 テンポラリ
てんま #CN 天満
てんまつ #T35 顛末
てんまばやし #CN 天間林
てんまんぐう #T35*4 天満宮
てんめい #JCN 天明
てんめつ #T30*5 点滅
てんもん #T35*1 天文
てんゆうしんじょ #T35 天祐神助
てんよう #T30*1 転用
てんらく #T30 転落
てんらん #T30*2 展覧
てんり #T35*42 天理 #CN 天理
てんりゅう #JCN 天竜 #JNS 天龍
てんれつ #T35 点列
で #KSr*255 出 #KS*78 で
でーた #T35*174 データ
でーたふろー #T35*22 データフロー
でーたべーす #T35*32 データベース
でーと #T30*16 デート
でーもん #T35*13 デーモン
であ #W5r*7 出会
であし #T35 出足
である #K5r*1 出歩
でぃーりんぐ #T35*1 ディーリング
でぃくしょなり #T35*2 ディクショナリ
でぃくす #T35*1 ディクス
でぃじたる #T05*11 ディジタル #T35*1 ディジタル
でぃすかっしょん #T30*3 ディスカッション
でぃすく #T35*175 ディスク
でぃすくりぷた #T35*4 ディスクリプタ
でぃすけっと #T35*5 ディスケット
でぃすこ #T35*2 ディスコ
でぃすてぃねーしょん #T35*1 ディスティネーション
でぃすとりびゅーしょん #T30*2 ディストリビューション
でぃすぱっち #T30*19 ディスパッチ
でぃすぱっちゃ #T35*13 ディスパッチャ
でぃすぷれー #T30*2 ディスプレー
でぃすぷれーすめんと #T35*1 ディスプレースメント
でぃすぷれい #T30*66 ディスプレイ
でぃずにー #JN ディズニー
でぃっしゅ #T35*1 ディッシュ
でぃなー #T35*2 ディナー
でぃふぉると #T35*2 ディフォルト
でぃぷりーしょん #T35*1 ディプリーション
でぃれい #T35*2 ディレイ
でぃれくた #T35*2 ディレクタ
でぃれくとり #T35*255 ディレクトリ
でい #KJ*1 泥 禰
でいすい #T30 泥酔
でいねい #T35 泥濘
でいり #T30*2 出入り
でいりぐち #T35*1 出入口
でか #KY*7 でか
でかけ #KS*9 出かけ
でかせぎ #T30*1 出稼ぎ
でかると #JN デカルト
でき #KSr*255 でき #KSr*244 出来 #KJ 溺
できあい #T30*1 溺愛 #T35 出来合い
できあが #R5r*13 出来上が
できごと #T35*2 出来事 #T35*1 できごと
できし #T30 溺死
できだか #T35 出来高
できばえ #T35*1 出来栄え
でく #T35 木偶
でくりめんと #T30*3 デクリメント
でぐち #T35*7 出口 #JNS 出口
でこーだ #T35*1 デコーダ
でこーど #T30*1 デコード
でこぼこ #T30*1 凸凹
でこんぱいら #T35*1 デコンパイラ
でさき #T35 出先
でざいな #T35*3 デザイナ
でざいなーず #T35*1 デザイナーズ
でざいん #T30*23 デザイン
でし #T35*3 弟子
でしじょん #T35*1 デシジョン
でじたる #T15*8 デジタル
でじま #JCN 出島
ですから #CJ*4 ですから
ですが #CJ*30 ですが
ですく #T35*10 デスク
ですけれど #CJ ですけれど
ですてぃねーしょん #T35*16 デスティネーション
でそろ #W5 出そろ
でたらめ #T05 でたらめ デタラメ 出鱈目
でっか #KY*1 でっか
でっき #T35*2 デッキ
でっち #T35 丁稚
でっちあげ #KSr でっちあげ
でっどろっく #T30*3 デッドロック
でっぱ #R5r*1 出っぱ #T15*1 出っ歯
でつく #S5 出尽く
でとろいと #CN デトロイト
でどころ #T35 出どころ 出所
でなお #S5r*1 出直
でなければ #CJ でなければ
では #CJ*7 では
でばいす #T35*85 デバイス
でばぐ #T30*19 デバグ
でばっが #T35*21 デバッガ
でばっく #T30*7 デバック
でばっぐ #T30*50 デバッグ
でばん #T35*2 出番
でぱーと #T35*2 デパート
でびゅー #T30*3 デビュー
でふぉーると #T35*8 デフォールト
でふぉると #T35*131 デフォルト
でふれ #T35 デフレ
でぶ #T05*2 デブ
でぶしょう #T05 出不精
でま #T35*1 デマ 手間
でまわ #R5r*6 出回
でまんど #T35*3 デマンド
でみせ #T35 出店
でむ #K5*2 出向
でむかえ #KSr 出迎え #T30 出迎え
でむら #JNS 出村
でめきん #T35 出目金
でも #T30*84 デモ #CJ*23 でも
でもじゅれーた #T35*4 デモジュレータ
でもんすとれーしょん #T30*4 デモンストレーション
でら #KJ 寺
でらっくす #T05*1 デラックス
でりーと #T30*1 デリート
でりみた #T35*1 デリミタ
でるた #T35*2 デルタ #T35*1 Δ δ
でわ #JCN 出羽
でん #KJ*43 電 #KJ*3 田 #KJ*2 伝 #KJ 殿 澱
でんあつ #T35*27 電圧
でんい #T35*2 電位
でんえん #T35*2 田園
でんか #T30*1 電化 #T35*1 電荷 #T35 殿下
でんき #T35*101 電器 #T35*63 電気 #T35*25 電機
でんきえいどう #T30*1 電気泳動
でんきゅう #T35*2 電球
でんきょく #T35 電極
でんけん #T35*1 電顕
でんげき #T35 電撃
でんげん #T35*33 電源
でんこう #KK*4 電興 #KK*2 電工
でんごん #T30*9 伝言
でんさん #T35*4 電算
でんさんき #T35*4 電算機
でんし #T35*135 電子
でんしとめーた #T35*1 デンシトメータ
でんしゃ #T35*28 電車
でんしょう #T30 伝承
でんしょばと #T35 伝書鳩
でんしん #T35*10 電信
でんじき #T35*1 電磁気
でんじは #T35*1 電磁波
でんじゅ #T30*1 伝授
でんせつ #T35*4 伝説
でんせん #T35*2 電線 #T30 伝染
でんせんびょう #T35 伝染病
でんそう #T30*26 伝送 #KK*1 電装 #T30*1 電送
でんそうけん #KK*74 電総研
でんたく #T35*5 電卓
でんたつ #T30*6 伝達
でんち #T35*2 電池
でんちゅう #T35*1 電柱 #T35 殿中
でんつう #KK 電通
でんつうだい #KK*1 電通大
でんてつ #T35 電鉄
でんでん #KK*2 電々
でんとう #T35*4 伝統 #T35*3 電灯
でんどう #T35*6 電動 #T30 伝導 #T35 殿堂
でんねつ #T35 電熱
でんぱ #T30*2 伝播 #T35*2 電波
でんぱん #T30*8 伝搬
でんぴょう #T35*9 伝票
でんぶん #T30*1 伝聞 #T35 電文
でんぽう #T35*1 電報
でんらい #T30 伝来
でんりゅう #T35*25 電流
でんりょく #T35*25 電力
でんれい #T35 伝令
でんわ #T30*255 電話
でんわきょく #T35*2 電話局
でんわだい #T35 電話代
でんわちょう #T35 電話帳
と #R5*255 と 取 #T35*127 戸 #K5*111 解 #B5*88 飛 #U5r*56 問 #T35*31 都 #R5*24 採 #R5*17 撮 #B5*12 と #M5*12 富 #K5*5 溶 #KJ*3 登 #KJ*2 図 #K5*2 説 #B5*2 跳 #KJ*2 渡 途 #KJ*1 砿 兎 堵 徒 #JS*1 斗 #KJ*1 杜 #S5*1 賭 #SX*1 賭 #KJ*1 砥 土 #R5*1 捕 #G5r 研 #K5 釈 #R5 摂 #R5r 奪 #KJ 吐 塗 妬 屠 斗 菟 #JS 都 #KJ 鍍 #R5r 盗 #U5 訪
とーくん #T35*10 トークン
とーたる #T00*3 トータル
とーとろじー #T35*1 トートロジー
とーらす #T35*1 トーラス
とある #RT*2 とある
とい #T35*2 樋 #T35*1 問 #JNS 戸井 #JCN 土肥
といあわせ #KSr*69 問い合わせ #T30*32 問い合わせ #T30*13 問合せ
といかけ #KSr*1 問いかけ
といき #T30*1 吐息
といし #T35 砥石
といただ #S5 問い質 問い正
といれ #T35*6 トイレ
といれっと #T35*2 トイレット
とう #JS*255 等 #T35*30 当 #KJ*19 東 #T35*4 塔 #KJ*4 島 筒 #KJ*3 盗 #KJ*2 登 #T35*2 党 唐 #KJ*2 湯 踏 頭 #KJ*1 稲 #T35*1 十 #KJ*1 倒 冬 #T35*1 刀 #KJ*1 嶋 #ZX*1 投 #KJ*1 淘 #T35*1 灯 #KJ*1 答 藤 謄 任 #KJ 桶 兜 桐 沓 諮 訊 逗 凍 塘 套 宕 悼 #T35 投 #KJ 搭 桃 梼 #JS 棟 #T35 棟 #KJ 涛 燈 #PRE 当 #KJ 痘 祷 #SUC 等 #T35 等 糖 #KJ 統 到 董 蕩 討 豆 逃 透 鐙 陶 #JS 頭 #KJ 騰 闘 萄 読 樋 訪
とうあ #KK*2 東亜
とうあん #T35*1 答案
とうい #T35 糖衣
とういち #JNM 東一
とういつ #T30*53 統一
とういん #T35 党員 #CN 東員 #T35 頭韻
とうえい #KK*2 東映 #T30*1 投影 #CN 東栄
とうおう #CN 東欧
とうおんせん #T35 等温線
とうか #T05*44 等価 #T00*16 透過 #T35*1 10日 １０日 十日 #T30 投下 #T35 灯火
とうかい #T30*25 倒壊 #JCN 東海 #T30 韜晦
とうかいどう #CN 東海道
とうかく #T30 倒閣 #T15 当確 #T35 頭角
とうかつ #T30*2 統括
とうかん #T30*1 投函 #T35 等閑
とうがい #T35*2 当該
とうがね #CN 東金
とうがらし #T35 唐辛子
とうがん #T35 冬瓜
とうき #T35*3 当期 #T35*1 当機 陶器 #T30 登記 #T35 冬期 冬季 #T30 投棄 騰貴
とうきち #JNM 藤吉
とうきゅう #KK*5 東急 #T30*3 投球 #JS*2 等級 #T35*2 等級
とうきょう #CN 東京
とうきょうと #CN 東京都
とうきょく #T35*4 当局
とうぎ #T30*3 討議
とうぎゅう #T35*1 闘牛
とうぎょ #T30 統御
とうぐん #T35 東軍
とうけい #T30*22 統計 #T35*8 東経
とうけつ #T30*2 凍結
とうげ #T35*1 峠 #CNSUC1 峠
とうげい #T35*1 陶芸
とうげんきょう #T35 桃源境
とうこう #T30*65 投稿 #T30*42 登校 #T30 投降
とうこうせん #T35*2 等高線
とうこうだい #KK*84 東工大
とうこん #T35 闘魂
とうご #T35*1 統語
とうごう #T30*52 統合 #T35*16 等号 #JCN 東郷
とうごうしゃ #T35 統合者
とうさい #T30*17 搭載
とうさく #T30*3 倒錯 #JNM 東作 #T30 盗作
とうさま #T35*7 父様
とうさん #T35*4 父さん #T30 倒産
とうざ #T35*1 当座
とうざい #T35*3 東西
とうし #T30*10 投資 #T30*2 透視 #T30*1 凍死 #T35*1 闘士 #T35 闘志
とうしき #T35*3 等式
とうしつ #T35*1 糖質 #T05 等質
とうしば #KK*15 東芝
とうしゃ #T35*29 当社 #T30 投射 透写
とうしゃばん #T35*1 謄写版
とうしゅ #T35*1 投手 #T35 党首 当主
とうしゅう #T30*1 踏襲
とうしょ #T35*14 当初 #T30 投書
とうしょう #T35*1 凍傷
とうしん #T30*2 答申 #T30 投身 #JS 等親 頭身
とうしんだい #T35 等身大
とうじ #T35*17 当時 #T30*1 湯治 #T35*1 統辞 #JNS 田路 #T35 杜氏 冬至
とうじき #T35 陶磁器
とうじしゃ #T35*1 当事者
とうじつ #T35*44 当日
とうじょう #T30*19 登場 #T30 搭乗 #JCN 東城 東条 #JNS 東條 棟上
とうじん #T30 蕩尽
とうすい #T30 陶酔
とうすう #T35*1 頭数
とうせい #T35*3 当世 #T30*2 統制 #T35 陶製
とうせき #T30*10 投石 #T30*1 透析
とうせん #T30*5 当選
とうぜん #T06*96 当然
とうそう #T35*4 党争 #T30*1 逃走 #T30 闘争
とうそく #T35*1 頭側
とうそくるい #T35 頭足類
とうそつ #T30 統率
とうぞく #T35*1 盗賊
とうた #T30*1 淘汰
とうたつ #T30*16 到達
とうだい #KK*57 東大 #T35 灯台 燈台 当代
とうだいじ #CN 東大寺
とうち #T35*1 当地 #T30*1 統治 #T30 倒置
とうちゃく #T30*64 到着
とうちゃん #T35*1 父ちゃん
とうちょう #T30*1 盗聴 #T30 登頂
とうちょうよう #T35 頭頂葉
とうちょく #T30 当直
とうてい #F14*2 到底 #F14*1 とうてい
とうてき #T30 投擲
とうてん #T35*1 読点 #T35 当店
とうと #B5*1 貴 #KY*1 貴 #B5*1 尊 #KY*1 尊
とうとう #F14*4 とうとう #SUC*2 等々 #F06 滔々
とうとうみ #JNS 遠江
とうとつ #T05*4 唐突
とうどり #T35 頭取
とうない #T35 党内
とうなん #T35*10 東南 #T35*1 盗難
とうにゅう #T30*12 投入
とうにん #T35*2 当人
とうねん #T35 当年
とうの #JNS 東野
とうのしょう #JCN 東庄
とうは #T35 党派 #T30 踏破
とうはく #JCN 東伯
とうはつ #T35 頭髪
とうはん #T30 登攀
とうばく #T30 倒幕
とうばつ #T30 討伐
とうばん #T30*1 当番
とうひ #T30*3 逃避 #T35 当否
とうひこう #T35 逃避行
とうひょう #T30*2 投票
とうびょう #T30*1 闘病
とうふ #T35 とうふ 豆腐
とうぶ #T35*9 頭部 #T35*1 東部 #CN 東部 #T35 当部
とうぶん #T35*19 当分 #T10*2 等分 #T35 糖分
とうへき #T35 盗癖
とうへん #JS 等辺
とうへんぼく #T35*1 唐変木
とうべつ #CN 当別
とうべん #T30 答弁
とうほう #T35*6 当方 #KK*2 東宝 東邦 #T35 東方
とうほく #T35*19 東北 #CN 東北
とうほん #T35 謄本
とうほんせいそう #T30*1 東奔西走
とうぼう #T30*1 逃亡
とうま #JCN 当麻
とうみん #T30 冬眠
とうめい #T05*10 透明 #CN 東名
とうめん #T30*9 当面
とうもく #T35 頭目
とうもろこし #T35 とうもろこし 玉蜀黍
とうや #CN 洞爺
とうやく #T30 投薬
とうやま #JNS 遠山
とうゆ #T35 灯油
とうよ #T30 投与 #JCN 東予
とうよう #T30*2 当用 #T30 登用 #JCN 東洋 東陽
とうよこ #KK*5 東横
とうらい #T30*1 到来
とうらく #T35 当落
とうりつ #T30*1 倒立
とうりゅう #T30 逗留
とうりゅうもん #T35*2 登竜門
とうりょう #T30 投了 #T15 等量 #T35 頭領
とうるい #T30 盗塁
とうろう #T35 灯篭 灯籠
とうろく #T30*255 登録
とうろん #T30*9 討論
とうわ #JCN 東和
とうわく #T30*1 当惑
とえ #T35 十重
とえい #T35 都営
とお #R5*255 通 #S5*255 通 #KY*40 遠 #T35*2 十 #S5*1 とお
とおあさ #T05 遠浅
とおか #T35 10日 １０日 十日
とおかいち #CN 十日市
とおかまち #CN 十日町
とおく #T35*5 遠く
とおざか #R5*1 遠ざか
とおざけ #KS*2 遠ざけ
とおし #T35 通し
とおだ #JCN 遠田
とおで #T30*1 遠出
とおの #K5*1 遠の #JCN 遠野
とおまき #T15 遠巻き
とおまわし #T10 遠回し
とおまわり #T10 遠回り
とおやま #T35*1 遠山 #JNS 遠山
とおり #JS*139 通り #T35*4 とおり 通り
とおりすぎ #KS*1 通り過ぎ
とおりぬけ #KSr 通り抜け
とおる #JNM 亨 享 達 徹
とおわ #CN 十和
とか #S5*4 融か #S5*2 溶か #S5*1 解か #T35 都下
とかい #T35*5 都会
とかく #F14*11 とかく
とかげ #T35 蜥蜴
とかしき #JCN 渡嘉敷
とかち #CN 十勝
とが #R5 尖 #JCN 利賀
とがうち #JCN 戸河内
とがくし #JCN 戸隠
とがし #JNS 富樫 冨樫
とがめ #KSr*1 咎め
とがわ #JNS 戸川
とき #T35*255 時 #T35*184 とき #JCN 土岐 #T35 鴇
ときえだ #JNM 時枝
ときお #JNM 時男 時夫 時雄
ときおり #F14*1 時おり 時折 #T36 時おり 時折
ときがわ #JCN 都幾川
ときこ #JNM 時子
ときた #JNS 時田
ときだ #JNS 時田
ときどき #T06*28 時々 #T06*1 ときどき
ときめき #T35*3 ときめき
ときわ #JNS 常盤 #JCN 常葉
とぎ #T35*1 砥 #JCN 富来
とぎつ #JCN 時津
とぎれ #KSr*1 途切れ
とぎん #T35*1 都銀
とく #T35*255 特 #T10*20 得 #T35*10 徳 #JNM とく #KJ 匿 涜 #T30 督 #KJ 禿 篤
とくい #T05*6 得意 #T05*1 特異
とくいげ #T05*1 得意気
とくいち #JNM 徳一
とくお #JNM 徳男 徳雄
とくがわ #JNS 徳川
とくぎ #T35*1 特技
とくさく #T15*1 得策
とくさぶろう #JNM 徳三郎
とくさん #T35 特産
とくしつ #T35*1 特質
とくしま #JCN 徳島
とくしゅ #T05*37 特殊
とくしゅう #T30*3 特集
とくしょう #T35*1 特賞
とくしょく #T35*2 特色
とくしん #T30 得心 特進
とくじ #JNM 徳治 #JCN 徳地
とくじゅ #T35*1 特需
とくじろう #JCN 徳次郎
とくせい #T35*22 特性 #T35*1 特製
とくせつ #T30 特設
とくせん #T30 特選
とくそう #T35 特捜
とくそく #T30 督促
とくぞう #JNM 徳蔵
とくたろう #JNM 徳太郎
とくだ #JNS 徳田
とくだい #T05 特大
とくちゅう #T30 特注
とくちょう #T35*77 特徴 #T35*6 特長
とくてい #T10*59 特定
とくてん #T30*8 得点 #T35*2 特典
とくとう #T35 特等
とくなが #JNS 徳永
とくに #F14*28 とくに
とくのしま #CN 徳之島
とくは #T30*1 特派
とくばい #T30*2 特売
とくひつ #T30*1 特筆
とくべつ #T06*41 特別
とくまる #JNM 得丸
とくみ #JNM 徳美
とくめい #T35*4 匿名 #T35*1 特命
とくやく #T30*1 特約
とくやす #JNM 徳康
とくやま #JCN 徳山
とくゆう #T05*8 特有
とくよう #T05 徳用
とくら #JNS 都倉
とくれい #T35*1 特例 #T30 督励
とぐち #T35 戸口
とぐら #JCN 戸倉
とけ #KS*1 融け #KS 熔け
とけい #T35*9 時計
とけつ #T30 吐血
とげ #T35*1 刺 #KS*1 遂げ
とこ #T35*9 床 #KJ 常
とこう #T30*2 渡航
とことん #F14*1 とことん
とこなつ #T35 常夏
とこなめ #JCN 常滑
とこばしら #T35 床柱
とこや #T35*5 床屋
ところ #T35*252 所 #T35*20 ところ #T35*3 処 #JN 所 #JCN 常呂
ところが #CJ*3 ところが
ところざわ #JCN 所沢
ところで #CJ*12 ところで
ところどころ #T35*1 所々
とさ #JCN 土佐
とさしみず #CN 土佐清水
とさつ #T30 屠殺
とさやま #CN 土佐山
とさやまだ #CN 土佐山田
とざ #S5 閉ざ
とざと #JNS 外里
とざま #T35 外様
とざわ #JCN 戸沢
とざん #T30*5 登山
とし #T35*54 年 #T35*53 都市 #KJ*14 敏 #KJ*5 俊 #T35*4 歳 #KJ*3 利 #JNM とし #KJ 夙 駿 肇 稔
としあき #JNM 俊昭 俊章 俊明 年昭 敏昭 敏明 利昭 利明
としあけ #T35 年明け
としうえ #T35*29 年上 #T05*1 歳上
としえ #JNM 季衣
としお #JNM 寿男 俊男 俊夫 俊雄 敏男 敏夫 敏雄 利男 利夫 利雄 利朗
としかず #JNM 俊一 利一
としかつ #JNM 利勝
としき #JNM 戸敷 俊樹
としこ #JNM とし子 トシ子 寿子 淑子 俊子 年子 敏子 利子
としごろ #T35*3 年頃 #T35*2 としごろ
としただ #JNM 利忠
としつき #T35*1 年月
としつぐ #JNM 敏嗣
として #CJ*11 として
としなみ #T35 年波
としのぶ #JNM 敏伸 利伸
としのり #JNM としのり 俊範 年紀
としはる #JNM 寿晴 俊治 利治
としひこ #JNM 俊彦 敏彦 利彦
としひと #JNM 敏仁
としひろ #JNM 俊博 敏宏 敏弘
としふみ #JNM 俊史 俊文
としま #T35*1 年増 #JCN 十島 豊島 利島
としまさ #JNM 敏正
としみつ #JNM 俊光
としや #JNM 俊哉 敏哉
としゆき #JNM 俊幸 俊行 俊之 敏幸 敏行 敏之 利幸 利之
としょ #T35*13 図書
としょかん #T35*8 図書館
としより #T35*2 年寄り
としろう #JNM 俊郎 敏郎 利郎
としん #T35*1 都心
とじ #KS*28 閉じ #KSr*5 綴じ #T35 刀自
とじき #JNS 戸敷
とじこめ #KS*4 閉じ込め
とじまり #T30 戸締り
とじょう #T35*1 途上
とじょうこく #T35 途上国
とす #JCN 鳥栖
とぜつ #T30*1 途絶
とそ #T35 屠蘇
とそう #T30 塗装
とたん #T35*21 途端 #T35 トタン 塗炭
とたんに #F14*2 とたんに
とだ #JCN 戸田
とだえ #KS*2 途絶え
とだな #T35 戸棚
とち #T35*10 土地 #KJ 栃 橡
とちお #JCN 栃尾
とちぎ #JCN 栃木
とちゅう #T35*84 途中
とちょう #T35 都庁
とっか #T35 特化 特価
とっかい #T35 特解
とっかん #T30 突貫
とっき #T30*1 突起
とっきゅう #T35*4 特急
とっきょ #T30*19 特許
とっくに #F14*1 とっくに
とっくみあ #W5r 取っ組み合
とっくり #T35*1 徳利
とっくん #T30*1 特訓
とっけん #T35*2 特権
とっこうやく #T35*1 特効薬
とっしゅつ #T30 突出
とっしん #T30 突進
とっとり #JCN 鳥取
とっぱ #T30*2 突破
とっぱつ #T30 突発
とっぱん #KK*3 凸版
とっぴょうし #T35 突拍子
とっぷ #T35*35 トップ
とっぷう #T35 突風
とっぷだうん #T05*3 トップダウン
とっぷり #F06 とっぷり
とつ #T35*3 凸 #G5*1 嫁 #KJ*1 突
とつか #JCN 戸塚
とつかわ #JCN 十津川
とつけつごう #T35*1 凸結合
とつじょ #F14*1 突如
とつじょとして #F14*1 突如として
とつぜん #T06*56 突然 #F02*31 突然
とつにゅう #T30*2 突入
とつへいほう #T35*2 凸閉包
とてい #T35 徒弟
とてつ #T35*1 とてつ
とても #F14*53 とても
ととの #W5*16 整 #W5 調
ととのえ #KSr*4 整え #KSr 調え
とど #K5*255 届 #KJ*1 椴
とどうふけん #T35*1 都道府県
とどけ #SUC*6 届 #KSr*2 届け #T35*1 届
とどけで #T30 届出
とどこお #R5r*1 滞
とどほっけ #CN 椴法華
とどま #R5*2 留ま
とどまつ #T35 椴
とどめ #KSr*5 とどめ #KSr 止め #KS 留め
とどろ #K5r*4 轟
とどろき #JCN 轟 等々力
とない #T35*5 都内
となえ #KS*2 唱え
となき #JCN 渡名喜
となみ #JCN 砺波 #JNS 礪波
となり #T35*53 隣 #T35*3 となり
となりあ #W5r 隣り合
となりあわせ #KSr 隣合わせ
となん #CN 都南
とにかく #F14*29 とにかく
とね #JNS 刀根 #JCN 利根
とねやま #JNS 刀根山
との #T35*12 殿
とのしょう #JCN 土庄
とば #S5r*18 飛ば #JCN 鳥羽
とばく #T35*1 賭博
とばた #JCN 戸畑
とばっちり #T35 とばっちり
とばり #T35 帳
とび #T35 鳶
とびおり #KSr*1 飛び降り
とびか #W5*7 飛び交
とびこ #M5r*2 とびこ 飛び込 #S5r*1 飛び越
とびこえ #KSr 飛び越え
とびしま #JCN 飛島
とびた #JCN 飛田
とびだ #S5r*2 飛び出
とびち #R5r*1 飛び散
とびつ #K5r*2 飛びつ
とびの #R5r 飛び乗
とびはね #KS 飛び跳ね
とびまわ #R5 飛び回
とびら #T35*1 扉
とぴっくす #T35*1 トピックス
とぶくろ #T35 戸袋
とべ #JCN 砥部
とべい #T30*3 渡米
とほ #T35*5 徒歩
とほう #T35*1 途方
とぼし #KY*1 乏し
とぼしゅう #KYU*1 乏しゅう
とぼとぼ #F06 とぼとぼ
とぽろじー #T35*2 トポロジー
とま #R5*52 止ま #R5r*37 泊ま #KJ 苫 #R5 留ま
とまこまい #CN 苫小牧
とまた #JCN 苫田
とまつ #T30*1 塗沫
とまと #T35*2 トマト
とまど #W5r*3 戸惑 #W5r とまど
とままえ #JCN 苫前
とまむ #CN トマム
とまり #JCN 泊 #T35 泊り
とまりこ #M5r*2 泊り込
とみ #T35*2 富 #JNM とみ トミ #JN 登美 富 #KJ 冨
とみあい #JCN 富合
とみうら #JCN 富浦
とみえ #JCN 富江
とみお #JNM 富男 富夫 #JCN 富雄
とみおか #JCN 富岡 #JNS 冨岡
とみか #JCN 富加
とみかわ #JNS 富川
とみぐすく #CN 豊見城
とみこ #JNM 富子 富美子
とみさと #JCN 富里
とみさわ #JNS 富沢 冨沢
とみざわ #JCN 富沢
とみじ #JNM 富治
とみた #JNS 富田 冨田
とみなが #JNS 富永 冨永
とみに #F14 とみに
とみや #JCN 富谷
とみやま #JNS 富山
とみん #T35 都民
とむら #JNS 外村 戸村 #W5r 弔
とめ #KS*47 止め #KS*5 泊め #KS*1 とめ 留め #JNM とめ #KS 停め #JCN 登米
とめきち #JNM 留吉
とめこ #JNM 留子
とめじ #JNM 留治
とめそで #T35*1 留袖
とも #T35*100 とも #T35*43 共 #KJ*34 智 #T35*11 友 #T35*2 供 #KJ*2 朋
ともあれ #CJ*1 ともあれ
ともいし #JNS 友石
ともいち #JNM 友一
ともえ #T35 巴
ともかく #F14*10 ともかく
ともかず #JNM 友和
ともきち #JNM 友吉
ともこ #JNM ともこ 知子 智子 朝子 朋子 友子
ともじ #JNM 友治
ともだち #T35*37 友達
ともち #JCN 砥用
ともども #F14*2 共々
ともな #W5*45 伴
ともなが #JNS 朝永
ともに #CJ*3 ともに
ともはる #JNM 知治
ともばたらき #T30*1 共働き
ともひこ #JNM 友彦
ともひろ #JNM 智博 友洋
ともびき #T35 友引
ともべ #JCN 友部
ともゆき #JNM 智之 朋之
ともゆみ #JNM 智之
ともよ #JNM 知世 知代
とや #JNS 戸谷
とやかく #F14 とやかく
とやま #JNS 外山 #JCN 富山 #JNS 冨山
とやまけん #CN 富山県
とよ #T35*10 豊 #JNM トヨ
とよあき #JN 豊明
とよあけ #JCN 豊明
とようら #JCN 豊浦
とよおか #JCN 豊岡 豊丘
とよかず #JNM 豊和
とよかわ #JCN 豊川
とよき #JNM 豊樹
とよきち #JNM 豊吉
とよこ #JNM 豊子
とよころ #JCN 豊頃
とよさか #JCN 豊栄
とよさく #JNS 豊作
とよさと #JCN 豊郷 豊里
とよしな #JCN 豊科
とよしま #JNS 豊島 豊嶋
とよじ #JN 豊治
とよた #KK*1 トヨタ #JCN 豊田
とよたま #JCN 豊玉
とよだ #JCN 豊田
とよつ #JCN 豊津
とよとみ #JNS 豊臣 #JCN 豊富
とよなか #JCN 豊中
とよね #JCN 豊根
とよの #JCN 豊能 豊野
とよはし #JCN 豊橋
とよはま #JCN 豊浜
とよひら #JCN 豊平
とよま #JCN 登米
とよまつ #JCN 豊松
とよやま #JCN 豊山
とら #T35*4 虎 #T35*2 寅
とらい #T30*2 トライ #T30 渡来
とらえ #KS*37 とらえ #KS*6 捉え #KS*2 捕らえ #KS 捕え
とらお #JNM 虎雄 寅男 寅雄
とらきち #JNM 寅吉
とらくた #T35*1 トラクタ
とらくたー #T35*1 トラクター
とらっきんぐ #T30*1 トラッキング
とらっく #T35*3 トラック
とらっぷ #T30*3 トラップ
とらのもん #CN 虎ノ門
とらばーゆ #T30*2 トラバーユ
とらひこ #JNM 寅彦
とらひっく #T35*33 トラヒック
とらひめ #JCN 虎姫
とらふぃっく #T35*25 トラフィック
とらぶる #T35*53 トラブル
とらわれ #KSr*1 因われ 捕らわれ
とらんく #T35*3 トランク
とらんざくしょん #T35*15 トランザクション
とらんしーば #T35*16 トランシーバ
とらんしるばにあ #CN トランシルバニア
とらんじぇんと #T05 トランジェント
とらんじすた #T35*8 トランジスタ
とらんすふぁ #T30*1 トランスファ
とらんすふぁー #T30*1 トランスファー
とらんすぽーと #T30*8 トランスポート
とらんすれーたー #T35*1 トランスレーター
とらんすれーと #T30*1 トランスレート
とらんぷ #T35*2 トランプ
とり #T35*23 鳥 #KJ 禽 #T35 酉
とりあ #W5r*3 取り合
とりあえず #F14*97 とりあえず #F14*27 取り敢えず #T36 とりあえず 取り敢えず
とりあげ #KSr*4 取り上げ #KSr*1 とり上げ
とりあつか #W5r*56 取り扱 #W5r*8 取扱
とりあつかい #T35*1 取扱
とりあつめ #KSr 取り集め
とりい #JCN 鳥居
とりいそぎ #F14*2 取り急ぎ
とりいれ #KSr*27 採り入れ #KSr*4 取り入れ #T30 取入れ
とりうみ #JNS 鳥海
とりえ #T35*1 とりえ 取り柄
とりお #T35*1 トリオ
とりおこな #W5*1 執り行
とりおと #S5r 取り落と
とりかえ #KSr*3 取り換え 取り替え #S5r*1 取り返
とりかか #R5r*2 取り掛か
とりかこ #M5r*1 取り囲
とりかわ #S5r*1 取り交わ
とりが #T30*4 トリガ
とりがー #T35*1 トリガー
とりきめ #KSr*6 取り決め
とりく #M5r*4 取り組
とりくず #S5 取り崩
とりけ #S5r*9 取り消
とりけし #T35*14 取消
とりこ #M5r*47 取り込 #T35*1 虜
とりこわ #S5r 取り壊
とりごえ #JCN 鳥越
とりさ #R5*2 取り去
とりさげ #KSr*1 取り下げ
とりざた #T30 取りざた
とりしき #R5r*1 取り仕切
とりしま #R5r 取り締ま
とりしまり #T30 取締り
とりしまりやく #T35*3 取締役
とりしらべ #KSr 取り調べ #T35 取調
とりそろ #W5r 取り揃
とりそろえ #KSr 取り揃え
とりたて #KSr*1 取り立て #T35 取立
とりたに #JNS 鳥谷
とりだ #S5r*79 取り出 #S5r*1 とりだ
とりちがえ #KSr*1 取り違え
とりつ #T35*11 都立 #G5r*2 取り次 #K5r 取り付
とりつくろ #W5r 取り繕
とりつけ #KSr*3 取り付け #T35 取付
とりで #T35*1 砦 #JCN 取手
とりとめ #KSr*2 取り留め
とりな #S5 執り成
とりのこ #S5r*1 取り残
とりのぞ #K5r*25 取り除
とりはから #W5r 取り計ら
とりはず #S5r*1 取り外
とりはだ #T35*1 鳥肌
とりはら #W5r*1 取り払
とりひき #T30*3 取り引き #T35*1 取引
とりひきさき #T35*1 取引先
とりぷる #T35*1 トリプル
とりま #K5r*1 取り巻
とりまぜ #KS*1 取り混ぜ
とりまとめ #KSr*34 取りまとめ
とりみだ #S5 取り乱
とりめ #T35 鳥目
とりも #T5r 取り持
とりもど #S5r*2 取り戻
とりもの #T35*1 捕物
とりや #JCN 鳥屋
とりやま #JNS 鳥山
とりやめ #KSr 取り止め
とりょう #T35 塗料
とりよせ #KSr*2 取り寄せ
とりわき #JNS 鳥脇
とりわけ #F14*2 とりわけ
とるく #T35 トルク
とるこ #CN トルコ
とれーしんぐ #T30*2 トレーシング
とれーす #T30*23 トレース
とれーどおふ #T30*2 トレードオフ
とれーなー #T35*1 トレーナー
とれーにんぐ #T30*2 トレーニング
とれい #T35*1 トレイ
とれらんと #T35*4 トレラント
とれんど #T35*6 トレンド
とろ #T30 吐露 #JCN 登呂
とろーち #T35 トローチ
とろう #T35*1 徒労
とろけ #KS*1 とろけ
とろふぃー #T35*1 トロフィー
とろんと #CN トロント
とわだ #JCN 十和田
とわだこ #CN 十和田湖
とん #KJ*3 噸 #JS トン 噸 #KJ 屯 惇 敦 沌 豚 遁 頓
とんかつ #T35 豚カツ
とんだばやし #JCN 富田林
とんちゃく #T30 頓着
とんちんかん #T05*2 とんちんかん 頓珍漢
とんでもな #KY*2 とんでもな
とんでものう #KYU*1 とんでものう
とんでんへい #T35*1 屯田兵
とんと #F14 とんと
とんねる #T35*2 トンネル
とんばら #JCN 頓原
とんび #T35 鳶
とんぼ #T35*2 とんぼ #T35 蜻蛉
とんぼがえり #T30 とんぼ返り 蜻蛉返り
ど #JS*146 度 #KJ*80 土 #T30*7 度 #KJ*4 戸 #JS*1 ° #T35*1 ° #KJ*1 努 怒 #KJ 堵 #SUC 度 #KJ 奴
どーなつ #T35*1 ドーナツ
どーむ #T35*1 ドーム
どあ #T35*8 ドア
どあい #T35*1 度合
どあほ #T05*1 ど阿呆
どい #JCN 土井 土居
どいつ #CN ドイツ
どいつご #KK*4 ドイツ語
どう #SX*221 どう #F14*116 どう #T35*96 同 #T35*10 動 #SUC*10 道 #KJ*6 導 #ZX*3 動 #T35*3 銅 #T35*2 々 #KJ*2 洞 童 #T35*2 胴 #KJ*1 憧 道 #KJ 桐 藤 働 同 #ZX 同 #KJ 堂 #SUC 堂 #KJ 撞 瞳 萄
どう゛ぉるざーく #JN ドヴォルザーク
どうい #T15*3 同位 #T30*2 同意 #T35 胴衣
どういう #RT*43 どういう
どういつ #T05*58 同一
どういつし #T30*1 同一視
どういん #T30*1 動員
どうおん #T35*4 同音
どうか #F14*26 どうか #T30*1 どうか 同化 #T35 銅貨
どうかく #T05*2 同格
どうかつ #T30 恫喝
どうかん #T30*7 同感
どうが #T35*3 動画 #T35*1 童画
どうがく #T15 同額
どうがん #T35 童顔
どうき #T35*95 動機 #T30*54 同期 #T35 動悸
どうきゅう #T35*2 同級 #T30 撞球
どうきょ #T30*3 同居
どうきょう #T35 同郷
どうきん #T30 同衾
どうぎ #T05*2 同義 #T35 動議
どうぎょう #T35*1 同業
どうくつ #T35*2 洞窟
どうぐ #T35*20 道具
どうぐち #JNS 洞口
どうけ #T35 道化
どうけい #T05*1 同型 #T05 同形 同系
どうけん #T35*9 同県
どうげつ #T35 同月
どうげんざか #CN 道玄坂
どうこう #T35*13 動向 #T30*2 同行 #T35*1 同好 #T35 瞳孔
どうこうのし #T35*1 同好の士
どうこうほう #T35 道交法
どうこく #T35 同国 #T30 慟哭
どうこん #T15 同根 #T30 同梱
どうさ #T30*255 動作
どうさつ #T30*2 洞察
どうさん #T35 動産
どうし #T35*76 動詞 #T35*33 同士 #T35*8 どうし #T35*1 同市 同志 同氏 #JCN 道志
どうししゃ #KK*1 同志社
どうした #JNS 堂下
どうしつ #T30 同室 #T05 同質
どうして #F14*139 どうして
どうしゃ #T35*1 同社
どうしゃく #T15 同尺
どうしゅ #T15*1 同種
どうしゅつ #T30*6 導出
どうしょう #T35 銅賞
どうしょういむ #T35 同床異夢
どうしょく #T35 同色
どうしょくぶつ #T35*2 動植物
どうしん #T35 童心
どうしんえん #T35*2 同心円
どうじ #T05*107 同時 #T35 童子
どうじく #T35*1 同軸
どうじつ #T35*2 同日
どうじま #JCN 堂島
どうじょ #T35 童女
どうじょう #T35*12 同上 #T30*4 同乗 同情 #T35*1 仝 道場
どうじん #T35*1 同人
どうすう #T15*1 同数
どうせ #F14*10 どうせ
どうせい #T35*2 同姓 #T35*1 同性 #T30*1 同棲 #T35 動静 同声 童声 銅製
どうせき #T30*2 同席
どうせん #T35 導線
どうぜん #T05*2 同然
どうそう #T35*4 同窓 #T05 同相
どうぞ #F14*21 どうぞ
どうぞう #T35 銅像
どうたい #T35*1 動態 導体 #T35 胴体
どうたく #T35 銅鐸
どうだん #T05 同断
どうち #T05*2 同値 #T35*1 ⇔ #T35 同地
どうちゅう #T35*1 道中
どうちょう #T30*1 同調 #T35 道庁
どうつう #T30 導通
どうてい #T30*4 同定 #T35*4 童貞 #T35*1 道程
どうてき #T05*44 動的
どうてん #T30*1 動転 #T15 同点
どうでん #T30 導電
どうでんたい #T35*1 導電体
どうとう #T05*16 同等
どうとく #T35 道徳
どうどう #F00*2 堂々 #T36 堂々
どうどうめぐり #T30 堂々巡り
どうない #T35*1 道内
どうにか #F14*1 どうにか
どうにも #F14 どうにも
どうにゅう #T30*99 導入
どうにゅうご #T35 導入後
どうねん #T35 同年
どうは #T30 道破
どうはい #T35 同輩
どうはん #T30*1 同伴
どうひょう #T35*1 道標
どうふう #T30*12 同封
どうふく #T35*1 同腹
どうぶつ #T35*10 動物
どうほう #T35*10 同報 #T35 同胞
どうまごえ #T35 胴間声
どうみゃく #T35 動脈
どうみゃくこうか #T30*1 動脈硬化
どうめい #T30*16 同盟 #T35*6 同名
どうも #F14*109 どうも
どうもう #T05 獰猛
どうもく #T30 瞠目
どうもと #T35 胴元
どうやら #F14 どうやら
どうゆう #T35*1 同友
どうよう #T05*136 同様 #T35*3 童謡 #T30*2 動揺
どうらく #T30 道楽
どうらん #T35 動乱
どうり #T35*20 道理
どうりつ #T15 同率
どうりょう #T35*3 同僚
どうりょく #T35*1 動力
どうるい #T15*1 同類
どうれつ #T35*1 同列
どうろ #T35*5 道路
どうわ #T35*2 童話 #T35 同和
どお #KY*1 遠
どおり #CNSUC1*25 通り #CNSUC1*1 通 #SUC どおり #T15 どおり #SUC 通り #T15 通り
どかた #T35 土方
どき #T35*16 土器 #SUC*4 時
どきっと #F12 どきっと
どきどき #T30*3 ドキドキ
どきゅめんてーしょん #T35*2 ドキュメンテーション
どきゅめんと #T35*111 ドキュメント
どきょう #T35*1 度胸
どぎつ #KY どぎつ
どぎつう #KYU どぎつう
どく #S5*57 毒 #T30*57 毒 #KJ*13 読 #KJ*3 独
どくえん #T30 独演
どくかい #T30*1 読解
どくがく #T30 独学
どくご #T30 独語
どくさい #T30*1 独裁
どくしゃ #T35*8 読者
どくしょ #T30*2 読書
どくしょう #T30 独唱
どくしん #T35*3 独身
どくじ #T05*27 独自
どくせい #T35*2 毒性
どくせん #T30*2 独占
どくせんじょう #T35 独擅場
どくそ #T35 毒素
どくそう #T30*2 独創 #T30 独奏 独走
どくたー #T35*1 ドクター
どくだん #T30*3 独断
どくとく #T05*3 独特
どくどくし #KY 毒々し
どくどくしゅう #KYU 毒々しゅう
どくは #T30*1 読破
どくはく #T30*1 独白
どくほん #T35*1 読本
どくぼう #T35 独房
どくやく #T35 毒薬
どくりつ #T00*44 独立
どくりょく #T35*2 独力
どけい #T35 時計
どけん #T35*1 土建
どこ #T35*137 どこ #T35*52 何処
どこか #T35*3 どこか
どころか #CJ*1 どころか
どごう #T30 怒号
どしー #JS*1 ℃ #T35*1 ℃
どしどし #F14*2 どしどし
どしゃ #T35 土砂
どじ #T00 ドジ
どじょう #T35*1 どじょう 土壌 鯲 鰌
どすう #T35*4 度数
どせい #KK*1 土星
どそう #T30 土葬
どそく #T35 土足
どぞう #T35 土蔵
どたんば #T35 土壇場
どだい #T35*3 土台
どちら #T35*96 どちら
どっか #T35*9 どっか #F06*3 どっか
どっかい #T30*1 読解
どっきんぐ #T30*1 ドッキング
どっきんほう #T35 独禁法
どっく #T35*1 ドック
どっぐ #T35 ドッグ
どっこい #F14 どっこい
どっこいしょ #CJ どっこいしょ
どっさり #F04 どっさり
どっしり #F04*1 どっしり
どっち #T35*21 どっち
どっと #T35*15 ドット #F14*2 どっと
どっとぶろっと #T35*1 ドットブロット
どて #T35*1 土手
どとう #T35*1 怒涛
どな #R5*1 どな 怒鳴
どなー #T35 ドナー
どなた #T35*20 どなた
どなべ #T35*1 土鍋
どなり #JCN 土成
どにち #T35*3 土日
どの #JNSUC*119 殿 #RT*85 どの #T35 殿
どのこと #T35 どのこと
どのみち #F14*1 どのみち
どのよう #T05*129 どのよう #T05*3 どの様
どのように #F12 どのように
どばし #JNS 土橋
どばと #T35 ドバト 土鳩
どひ #JNS 土肥
どひょう #T35*2 土俵
どびん #T35 土瓶
どべい #T35 土塀
どぼく #T35*1 土木
どぼるざーく #JN ドボルザーク
どま #T35*1 土間
どめいん #T35*177 ドメイン
ども #SUC*2 共 #R5r*1 吃 #SUC ども
どよう #T35*18 土曜 #T35*13 土用
どようび #T35*63 土曜日
どよめ #K5r どよめ
どらい #T05*4 ドライ
どらいば #T35*56 ドライバ
どらいばー #T35*2 ドライバー
どらいぶ #T30*23 ドライブ
どらっぎんぐ #T30*8 ドラッギング
どらっぐ #T30*13 ドラッグ
どらふと #T35*23 ドラフト
どらま #T35*3 ドラマ
どらむす #T35*1 ドラムス
どりあ #T35*1 ドリア
どりぶん #T05*4 ドリブン
どりょう #T35 度量
どりょうこう #T35 度量衝
どりょく #T30*34 努力
どる #JS*20 ドル #T35*4 ドル #T35*1 ＄
どるびー #KK*1 ドルビー
どれ #T35*54 どれ
どれい #T35*1 奴隷 #T35 土鈴
どれいん #T35*1 ドレイン
どれっしんぐ #T35*1 ドレッシング
どれほど #F14 どれほど
どろ #T35*7 泥
どろじあい #T35*1 泥試合
どろっぷ #T35*2 ドロップ
どろどろ #T30*1 泥々
どろなわ #T15 泥縄
どろぬま #T35*1 泥沼
どん #T30*9 鈍 #KJ*5 丼 #KJ*1 呑 曇 貪
どんか #T30 鈍化
どんかん #T05*1 鈍感
どんぐり #T35*1 ドングリ
どんこう #T35*1 鈍行
どんじゅう #T05 鈍重
どんちょう #T35 緞帳
どんつう #T35 鈍痛
どんてん #T35 曇天
どんと #F14 どんと
どんどん #F14*22 どんどん
どんな #RT*51 どんな #T05*48 どんな
どんぶり #T35 丼
どんま #T30 鈍磨
どんよく #T05*1 どん欲 貪欲
どんより #F04 どんより
どんらん #T05 貪婪
な #R5*255 な #KY*255 無 #T35*154 名 #KY*99 な #K5*82 泣 #R5*12 成 #T35*11 菜 #KJ*9 奈 #S5*8 な #K5*7 鳴 #R5r*5 鳴 #KJ*2 那 #S5*1 為 #KJ 茄 #S5 成 #G5r 凪 #KJ 納
ない #SUC*255 内 #T35*2 内
ないーぶ #T05*1 ナイーブ
ないあがら #CN ナイアガラ
ないいんせい #T35*1 内因性
ないえ #JCN 奈井江
ないか #T35*2 内科
ないかい #T35 内科医
ないかく #T35*2 内閣 #T35 内角
ないがい #T35*3 内外
ないがしろ #T35 蔑ろ
ないきょく #T35 内局
ないきん #T30 内勤
ないこう #T30 内向
ないこく #T35 内国
ないさいぼうかい #T35*1 内細胞塊
ないざい #T30*1 内在
ないし #F14 乃至
ないしきょう #T35 内視鏡
ないしは #CJ*1 ないしは
ないしゅっけつ #T30 内出血
ないしょ #T15*11 内緒
ないしょく #T30*1 内職
ないしん #T35*1 内心 #T30 内申 内診
ないしんのう #T35 内親王
ないじ #T30*2 内示
ないじつ #T35 内実
ないじゅ #T35*1 内需
ないじょう #T35 内情
ないせい #T30*1 内省 #T35*1 内政 #T35 内声
ないせき #T35*1 内積
ないせつ #T30*1 内接
ないせん #T35*28 内線 #T35*2 内戦
ないそう #T35*1 内層 内装
ないそく #T35*1 内側
ないぞう #T30*13 内蔵 #T35 内臓
ないだく #T30 内諾
ないち #T35 内地
ないてい #T30*3 内定
ないてき #T15 内的
ないと #T35*7 ナイト
ないとう #JCN 内藤
ないはつ #T30 内発
ないふ #T35*1 ナイフ
ないふん #T35 内紛
ないぶ #T35*144 内部
ないぶんぴつけい #T35*1 内分泌系
ないほう #T30 内包
ないみつ #T05 内密
ないめん #T35*1 内面
ないやく #T30 内約
ないゆう #T35 内憂
ないよう #T35*255 内容
ないらん #T35 内乱
ないりく #T35 内陸
ないりょく #T35 内力
ないろん #T35 ナイロン
なう #KY*1 ナウ
なえ #KS*1 萎え #T35*1 苗
なえば #JCN 苗場
なお #S5r*255 直 #R5r*168 直 #CJ*72 なお #R5r*53 治 #S5r*52 治 #KJ*10 尚 #S5r*8 なお #F14*2 猶 #R5r*1 なお #CJ*1 尚 #JNM 直 奈緒
なおい #JNM 直井
なおいり #JCN 直入
なおえ #JCN 直江
なおえつ #JCN 直江津
なおかわ #JCN 直川
なおき #JNM 尚紀 直樹
なおきち #JNM 直吉
なおこ #JNM 尚子 直子 奈保子
なおさら #F14*2 なおさら
なおしま #JCN 直島
なおじ #JNM 直治
なおた #JNS 直田
なおたけ #JNM 尚武
なおと #JNM 直人
なおはら #JNS 猶原
なおひさ #JNM 尚久
なおみ #JNM 尚美 直美
なおも #F14*1 なおも
なおや #JNM 直也
なおゆき #JNM 直行
なおよし #JNM 直義
なか #T35*209 中 #T35*18 仲 #T35*6 なか #JN ナカ #CNPRE 中 #JCN 中 那珂 那賀
なかい #JCN 中井 #T35 仲居
なかいず #CN 中伊豆
なかうおぬま #CN 中魚沼
なかお #JCN 中尾
なかかわね #JCN 中川根
なかかんばら #CN 中蒲原
なかがい #T30 仲買
なかがみ #JCN 中頭
なかがわ #JCN 中川 #CN 那賀川
なかきた #JNS 中来田
なかぎょう #CN 中京
なかくびき #JCN 中頚城
なかぐすく #JCN 中城
なかこま #CN 中巨摩
なかごう #CN 中郷
なかごめ #JNS 中込
なかごろ #T35*1 中ごろ
なかさつない #CN 中札内
なかさと #JCN 中里
なかざと #JCN 中里 仲里
なかざわ #JNS 中沢
なかしべつ #CN 中標津
なかしま #JNS 中島
なかじま #JCN 中島 #JNS 中嶋
なかじょう #JCN 中条
なかすじ #T35*4 中筋
なかせん #JCN 中仙
なかそね #JNS 中曽根
なかた #JCN 中田 #JNS 仲田
なかたがい #T30 仲たがい
なかたど #JCN 仲多度
なかたに #JNS 中谷 仲谷
なかたね #JNS 中種子
なかだ #JCN 中田 #JNS 仲田
なかだいら #JNS 中平
なかだち #T30 仲立ち #T35 媒
なかだるみ #T10 中だるみ
なかつ #JCN 中津
なかつえ #JCN 中津江
なかつがる #CN 中津軽
なかつがわ #JCN 中津川
なかつぎ #T30*1 中継ぎ
なかてん #T35*5 ナカテン
なかで #JNS 中出
なかとき #JNS 中時
なかとさ #CN 中土佐
なかとみ #JCN 中富
なかとんべつ #CN 中頓別
なかなおり #T30*1 仲直り
なかなか #F14*62 なかなか #T36 なかなか
なかに #F14 なかに
なかにいかわ #CN 中新川
なかにいだ #CN 中新田
なかにし #JNS 中西
なかには #F14 中には
なかにも #F14 中にも
なかにわ #T35*1 中庭
なかね #JCN 中根
なかの #JCN 中野
なかのくち #JCN 中之口
なかのしま #CN 中之島
なかのじょう #JCN 中之条
なかはら #JCN 中原
なかば #T35*3 半ば
なかばやし #JNS 中林
なかばる #JCN 中原
なかふらの #CN 中富良野
なかへじ #JCN 中辺路
なかほど #T35 中程
なかま #T35*13 仲間 #JCN 中間 #T35 中間
なかまち #JCN 中町
なかみ #T35*42 中身
なかみち #JCN 中道 #JNS 仲道
なかみなと #CN 那珂湊
なかむら #JCN 中村 #JNS 仲村
なかもと #JNS 中本
なかや #JCN 中谷
なかやま #JCN 中山
なかゆび #T35 中指
なかよ #KY*3 仲良
なかよう #KYU*1 仲良う
なかよし #T15*1 仲良し
なかんずく #CJ なかんずく
なかんづく #CJ なかんづく
なが #KY*255 長 #S5r*77 流 #KY*12 永 #KJ 永 長 #JCN 那賀 #JNS 名賀
ながあめ #T35 長雨
ながい #T30*9 長居 #JCN 永井 長井
ながいき #T10*1 長生き
ながいずみ #JCN 長泉
ながうた #T35 長唄
ながお #JCN 長尾
ながおか #JCN 長岡
ながおかきょう #CN 長岡京
ながくて #JCN 長久手
ながぐつ #T35 長靴
ながさか #JCN 長坂
ながさき #JCN 長崎
ながさわ #JCN 長沢
ながしこ #M5r*2 流し込
ながしま #JNS 永島 永嶋 #JCN 長島 #JNS 長嶋
ながす #JCN 長洲
ながすぎ #KS*9 長過ぎ
ながせ #JCN 永瀬 長瀬
ながそで #T35*1 長袖
ながた #JCN 永田 長田
ながたらし #KY 長たらし
ながたらしゅう #KYU 長たらしゅう
ながちょうば #T35 長丁場
ながつき #T35 長月
ながつだ #JCN 長津田
ながつづき #T10 長続き
ながと #JCN 長門
ながとろ #JCN 長瀞
ながなが #F06*3 長々
ながぬま #JCN 長沼
ながねん #T35*3 長年 #T35*1 永年
ながの #RT 永の #JNS 永野 #JCN 長野
ながのはら #JCN 長野原
ながはま #JCN 長浜
ながばなし #T30 長話
ながび #K5*3 長引
ながぶち #JNS 長淵 長渕
ながまつ #JNS 永松
ながめ #KSr*14 眺め
ながもち #T10*1 長持ち
ながや #T35*1 長屋
ながやま #JCN 永山
ながよ #JCN 長与
ながら #KJ*1 乍 #JCN 長柄
ながらく #F14*5 長らく
ながれ #KSr*115 流れ
ながれこ #M5r*1 流れ込
ながれだ #S5r*1 流れ出
ながれやま #JCN 流山
ながわ #JCN 奈川 名川
なき #T35*20 なき #T35*1 亡き
なきじん #CN 今帰仁
なぎ #T35*1 凪 薙 #JNS 奈義
なぎさ #T35*1 渚
なぎそ #CN 南木曾
なく #S5*11 無く #S5*6 なく #S5 亡く
なくな #R5*16 亡くな #R5*3 無くな
なぐ #R5*3 殴
なぐさ #M5r*1 慰
なぐさめ #KSr*2 慰め
なぐも #JNS 南雲
なぐり #JCN 名栗
なげ #KS*9 投げ #K5*2 嘆
なげいれ #KSr*1 投げ入れ
なげかけ #KS 投げかけ
なげかわし #KY 嘆かわし
なげかわしゅう #KYU 嘆かわしゅう
なげこ #M5r 投げ込
なげだ #S5*1 投げ出
なげつけ #KSr*1 投げつけ
なげやり #T05*2 投げやり
なこうど #T35*1 仲人
なご #M5*1 和 #CN 名護
なごう #KYU*3 長う #KYU*2 永う
なごや #JNS 名越 #JCN 名古屋
なごやか #T05*1 和やか
なごり #T35*2 名残
なごりおし #KY*1 名残惜し
なごりおしゅう #KYU*1 名残惜しゅう
なさ #L5*124 なさ
なさけ #T35 情け
なさけな #KY*4 情けな #KY*2 情な
なさけのう #KYU*1 情けのう 情のう
なさけぶか #KY 情け深
なさけぶこう #KYU 情け深う
なさそう #T05*8 なさそう #T05 無さそう
なざし #T30*3 名指し
なし #T35*128 無し #T35*44 なし #T35*1 梨
なしとげ #KS*3 成し遂げ
なしのつぶて #T35*2 なしのつぶて
なしょなる #KK*4 ナショナル
なじ #M5r*10 馴染 #M5r*2 なじ #R5 詰
なす #R5 なす #T35 なす #R5 擦 #JNS 那州 #JCN 那須
なすりつけ #KSr なすりつけ 擦り付け
なぜ #F14*42 なぜ #F14*36 何故 #JCN 名瀬
なぜか #F14*58 何故か
なぜならば #T35*1 ∵
なぜに #F14 なぜに
なぞ #T35*4 謎
なぞらえ #KS なぞらえ 準え
なたしょう #CN 名田庄
なだ #T35*1 灘 #CNSUC1 灘 #JCN 灘
なだか #KY 名高
なだこう #KYU 名高う
なださき #JCN 灘崎
なだち #JCN 名立
なだめ #KS なだめ 宥め
なだれ #T35*1 雪崩
なちかつうら #CN 那智勝浦
なちす #KK*1 ナチス
なっと #T35*4 ナット
なっとう #T35*1 納豆
なっとく #T30*29 納得
なっぱ #T35 菜っ葉
なつ #T35*33 夏 #KJ 捺
なついん #T30*1 捺印
なつかし #KY*15 懐かし #M5r*4 懐かし #KY*1 なつかし #M5r*1 なつかし
なつかしの #RT 懐かしの
なつかしゅう #KYU*2 懐かしゅう #KYU*1 なつかしゅう
なつき #JNM 奈月
なつこ #JNM 夏子
なつの #JNM なつの
なつば #T35 夏場
なつふく #T35 夏服
なつむき #T15 夏向き
なつめ #T35*1 棗
なつもの #T35 夏物
なつやすみ #T35*57 夏休み
なつやま #T35 夏山
なづけ #KSr*4 名付け
なで #KS*1 撫で
なでおろ #S5*1 なでおろ
なとり #JCN 名取 #T35 名取
なとりうむ #T35*3 ナトリウム
など #SUC*180 等 #T35*10 など
なな #NN*17 七
ななえ #JCN 七飯
ななお #JCN 七尾
ななかい #JCN 七会
ななせ #JNS 七瀬
ななそじ #T35 七十路
ななつ #T35*1 七つ
ななつか #JCN 七塚
ななめ #T05*6 斜め
ななやしろ #JNS 七社
ななやま #JCN 七山
なに #T35*255 何 #T35*19 なに
なにがし #KJ 某
なにげな #KY*2 何気な
なにげのう #KYU*1 何気のう
なにごと #T35*2 何事
なにしろ #F14*6 何しろ #F14*1 なにしろ
なにせ #F14*3 何せ
なにとぞ #F14*5 何卒
なにぶん #T35*10 何分
なにもの #T35*4 何者
なにゆえ #F14 何故
なにゆえに #F14 何故に
なにわ #JCN 難波 #JNS 浪花 #JCN 浪速
なの #R5r*4 名乗
なのか #T35*3 七日 #T35 7日 ７日
なは #JCN 那覇
なはり #JCN 奈半利
なばり #JCN 名張
なび #K5r なび
なふだ #T35*1 名札
なぶらちろわ #JN ナブラチロワ
なへん #T35 那辺
なべ #T35*20 鍋
なべぞこ #T35 鍋底
なべづる #T35 ナベヅル
なべもの #T35 鍋物
なほみ #JNM 奈保美
なま #T05*10 生 #R5r 鈍
なまいき #T05*1 生意気
なまえ #T35*255 名前
なまくび #T35 生首
なまぐさ #KYmi*1 生臭
なまぐそう #KYU*1 生臭う
なまけ #KS*1 怠け
なまこ #T35*1 ナマコ
なまじ #F05 なまじ
なまじっか #F05 なまじっか
なます #T35 膾
なまず #T35*1 鮠 鯰
なまなまし #KY*1 生々し
なまなましゅう #KYU*1 生々しゅう
なまはげ #T35 なまはげ
なまみ #T35 生身
なまり #T35*1 鉛
なまろく #T30*4 生録
なまわく #T35*1 生ワク
なみ #T35*16 〜 #T35*11 波 #T35*4 並 #T35*2 浪 #SUC*1 なみ #SUC 並
なみあい #JCN 浪合
なみあし #T30 並足
なみえ #JCN 浪江
なみおか #JCN 浪岡
なみかぜ #T35*1 波風
なみかた #JCN 波方
なみかわ #JCN 並河
なみき #T35*1 並木 #JCN 並木
なみだ #T30*11 涙 #T5r 波立
なみだぐ #M5 涙ぐ
なみだぐまし #KY 涙ぐまし
なみだぐましゅう #KYU 涙ぐましゅう
なみなみ #F02 並々
なみの #JCN 波野
なみはずれ #KS 並外れ
なみま #T35 波間
なむ #CJ*3 南無
なめ #KS*2 なめ #KS 嘗め #S5r 鞣
なめかた #JCN 行方
なめかわ #JCN 滑川
なめがた #JCN 行方
なめらか #T05*2 滑らか #T05*1 なめらか
なめりかわ #JCN 滑川
なや #M5r*14 悩
なやま #S5 悩ま
なよろ #JCN 名寄
なら #B5r*88 並 #W5r*6 習 #S5*5 鳴ら #T35*3 楢 #S5r*1 慣ら #S5 均 #B5 陳 #JCN 奈良 #B5 列
ならいごと #T35 習い事
ならお #JCN 奈良尾
ならかわ #JCN 楢川
ならき #JNS 奈良木
ならし #CN 奈良市
ならしの #JCN 習志野
ならでは #SUC*1 ならでは
ならは #JCN 楢葉
ならば #T35*1 ⇒
ならびた #T5 並び立
ならびに #CJ*2 ならびに
ならべ #KSr*1 並べ #KS 陳べ 列べ
ならわ #S5r 慣わ 習わ
なり #JS*4 也
なりきん #T35 成金
なりた #T5r*4 成り立 #T5r*1 なりた #JCN 成田
なりたち #T35 成立ち
なりひび #K5*1 鳴り響
なります #JCN 成増
なりやす #JNM 成康
なりゆき #T35*1 成行き
なりわ #JCN 成羽
なりわい #T35*1 生業
なる #JCN 奈留
なるこ #JCN 鳴子
なるさわ #JNS 成澤 #JCN 鳴沢
なるしすと #T35 ナルシスト
なるしずむ #T35 ナルシズム
なるせ #JCN 成瀬
なるたけ #F14*1 なるたけ
なると #JCN 鳴門
なるとう #JCN 成東
なるべく #F14*7 なるべく
なるほど #F14*7 なるほど #CJ*2 なるほど
なるみつ #JNM 成光
なるよし #JNM 成良
なれ #KSr*30 慣れ #KS 熟れ #KJ 馴 #KSr 馴れ #KS 狎れ
なれずし #T35 熟鮨
なれなれし #KY なれなれし
なれなれしゅう #KYU なれなれしゅう
なわ #T35*2 縄 #JCN 名和
なわて #T35 畷
なわばり #T35*1 縄張 #T30*1 縄張り
なん #NN*193 何 #T35*20 難 #ZX*13 難 #KJ*5 南 #KJ*1 男 軟 #KJ 楠
なんい #T35*2 難易 #T35 南緯
なんか #T30*19 軟化 #T30*12 南下
なんかい #T35*24 何回 #T05*1 難解 #T35 南海
なんかん #CN 南関 #T35 難関
なんがい #CN 南外
なんきょく #T35 難局
なんきん #CN 南京 #T30 軟禁
なんぎ #T00*1 難儀
なんぐん #T35 南軍
なんけん #T35*3 軟件
なんこう #T35*1 軟膏 #T30*1 難航 #CN 南光
なんこうふらく #T05 難攻不落
なんこく #CN 南国
なんこつ #T35 軟骨
なんごう #CN 南郷
なんごく #CN 南国
なんざん #JCN 南山
なんしき #T35 軟式
なんしつ #T05 軟質
なんしょ #T35*1 難所
なんしょく #T35*1 難色
なんじ #T35*32 何時 #T35*2 汝 #T35 爾 難事
なんじゃく #T05*4 軟弱
なんじゅう #T30*2 難渋
なんじょう #JCN 南条
なんすい #T35 軟水
なんせ #F14*2 なんせ 何せ
なんせい #T35*1 南西 軟性 #JCN 南勢
なんせんす #T05*1 ナンセンス
なんぞ #SUC*1 なんぞ
なんぞや #F06 何ぞや
なんたん #T35 南端
なんだい #T35*2 何台 #T35 難題
なんだか #F14*3 なんだか
なんだん #JCN 南淡
なんちゃくりく #T30 軟着陸
なんちょう #T35 難聴
なんて #F14*58 なんて
なんてつ #T35 軟鉄
なんてん #T35*2 難点
なんで #F14*23 なんで
なんでも #F14*2 なんでも
なんでもかんでも #F14 なんでもかんでも 何でもかんでも
なんと #F14*7 なんと
なんとう #JCN 南島 #T35 南東
なんとか #F12*38 何とか
なんとなく #F14*14 なんとなく
なんど #T35*31 何度 #T35 納戸
なんなく #F14 難なく
なんにち #T35*2 何日
なんにょ #T35 男女
なんにん #T35*10 何人
なんねん #T35*5 何年
なんのう #CN 南濃
なんば #JCN 難波
なんばー #T35*24 ナンバー #T35*1 ＃
なんぱ #T30*2 難破 #T30*1 軟派
なんびょう #T35 何秒 難病
なんぴと #T35*1 何人
なんぴょうよう #T35 南氷洋
なんぶ #T35*1 軟部 #JCN 南部
なんぷん #T35 何分
なんべい #CN 南米
なんぼく #T35*2 南北
なんぽう #T35*1 南方
なんぽろ #CN 南幌
なんみん #T35*1 難民
なんもく #CN 南牧
なんもん #T35*1 難問
なんや #JNS 南谷
なんよう #JCN 南陽
なんら #T35*38 何ら #T35*1 なんら #T35 何等
に #NN*255 二 #KS*160 似 #KS*16 煮 #KJ*9 仁 #KJ*4 尼 #T35*3 荷 #KJ*2 丹 #NN*1 弐 #KJ*1 邇 #KJ 似 児 爾 煮 迩
にーず #T35*2 ニーズ
にーもにっく #T35 ニーモニック
にあ #W5r*21 似合
にあげ #T30 荷揚げ
にあみす #T30*2 ニアミス
にありいこーる #T35*3 ≒
にい #KJ*2 新
にいかっぷ #CN 新冠
にいかわ #JNS 新川
にいがた #CN 新潟
にいさと #JCN 新里
にいさん #T35*1 兄さん
にいざ #CN 新座
にいざと #CN 新里
にいじま #JCN 新島
にいじまほんそん #CN 新島本村
にいつ #JCN 新津
にいつる #CN 新鶴
にいづま #T35 新妻
にいで #JNS 新出
にいはま #CN 新居浜
にいはり #JCN 新治
にいはる #CN 新治
にいほ #JCN 新穂
にいみ #CN 新見 #JNS 新実 新美
にう #JNS 丹生
にうけ #T30*1 荷受け
にえ #KS*1 煮え
にお #W5r*8 匂 #W5r*1 臭 #JCN 仁尾
におい #KJ 匂
におう #T35 仁王
におやか #T05 匂やか
におわ #S5 匂わ
にかい #T35*5 二階
にかほ #JCN 仁賀保
にかよ #W5 似通
にからぐあ #CN ニカラグア
にかわ #T35 膠
にが #KYmi*3 苦 #S5*1 逃 #S5 逃が
にがおえ #T35 似顔絵
にがて #T05*5 苦手
にがにがし #KY*1 苦々し
にがにがしゅう #KYU*1 苦々しゅう
にがわらい #T30 苦笑い
にき #JCN 仁木
にきび #T35 にきび
にぎ #R5r*6 握
にぎやか #T05*3 賑やか
にぎりつぶ #S5r*1 握りつぶ
にぎわ #W5r*3 賑わ #S5*2 賑わ
にく #T35*10 肉 #KYmi*9 憎 #M5*1 憎 #D2KY にく 憎 難 #KY 難
にくう #KYU*1 憎う
にくかい #T35 肉塊
にくが #T35*1 肉芽
にくがん #T35 肉眼
にくきゅう #T35 肉球
にくぎゅう #T35 肉牛
にくしみ #T35*1 憎しみ
にくしょく #T30 肉食
にくしん #T35 肉親
にくすい #T35*1 肉垂
にくせい #T35 肉声
にくたい #T35*3 肉体
にくづき #T35 肉付き
にくはく #T30 肉薄
にくらし #KY 憎らし
にくらしゅう #KYU 憎らしゅう
にぐるま #T35 荷車
にげ #KSr*11 逃げ
にげこ #M5*1 逃げ込
にげだ #S5*1 逃げ出
にげみち #T35 逃げ道
にげん #T35*1 二元
にこ #M5r*3 煮込
にこちん #T35 ニコチン
にこにこ #T30*2 ニコニコ
にこやか #T05 にこやか
にころが #S5r 煮ころが
にこん #KK ニコン
にご #R5r*1 濁 #S5*1 濁
にごう #KYU*1 苦う
にごら #S5 濁ら
にさんかたんそ #T35 二酸化炭素
にし #T35*179 西 #CNPRE*4 西 #CNSUC1 西 #JCN 西 #T35 螺
にしあいづ #CN 西会津
にしあざい #CN 西浅井
にしあざぶ #CN 西麻布
にしありえ #CN 西有家
にしありた #CN 西有田
にしあわくら #CN 西粟倉
にしいず #CN 西伊豆
にしいばらき #CN 西茨城
にしいややま #CN 西祖谷山
にしいわい #CN 西磐井
にしうすき #CN 西臼杵
にしうみ #JCN 西海
にしうわ #CN 西宇和
にしお #JCN 西尾
にしおおじ #CN 西大路
にしおか #JNS 西岡
にしおきたま #CN 西置賜
にしおこっぺ #CN 西興部
にしかいがん #T35*2 西海岸
にしかすがい #CN 西春日井
にしかぜ #T35 西風
にしかた #JCN 西方
にしかつら #CN 西桂
にしかも #CN 西加茂
にしかわ #JCN 西川
にしかんばら #CN 西蒲原
にしがいち #JNS 西垣内
にしがわ #T35*1 西側
にしき #T35*2 錦 #CN 錦 西紀 #JCN 西木
にしきごい #T35 錦鯉
にしきみ #JNS 錦見
にしきょう #CN 西京
にしくにさき #CN 西国東
にしくびき #CN 西頚城
にしぐち #T35*1 西口
にしごう #CN 西郷
にしごうし #CN 西合志
にしざき #JCN 西崎
にしざわ #JCN 西沢
にししらかわ #CN 西白河
にしじま #JNS 西島 西嶋
にしじん #CN 西陣
にしせんぼく #CN 西仙北
にしそのぎ #CN 西彼杵
にしたがわ #CN 西田川
にしたま #CN 西多摩
にしだ #JNS 西田
にしだて #JNS 西館
にしつがる #CN 西津軽
にしとさ #CN 西土佐
にしとなみ #CN 西砺波 西礪波
にしな #JCN 仁科
にしなり #JCN 西成
にしね #JCN 西根
にしの #JCN 西野
にしのおもて #CN 西之表
にしのしま #CN 西ノ島
にしのはら #JNS 西之原
にしのみや #JCN 西宮
にしはら #JCN 西原
にしはる #CN 西春
にしび #T35*1 西陽 #T35 西日
にしびわじま #CN 西枇杷島
にしまつうら #CN 西松浦
にしむら #JNS 西村
にしむらやま #CN 西村山
にしむろ #CN 西牟婁
にしめ #KSr 煮しめ #CN 西目
にしめや #CN 西目屋
にしめら #CN 西米良
にしもと #JNS 西本
にしもろかた #CN 西諸県
にしゃたくいつ #T30*2 二者択一
にしやつしろ #CN 西八代
にしやま #JCN 西山
にしよしの #CN 西吉野
にしよどがわ #CN 西淀川
にしわき #JCN 西脇
にしん #T05*2 二進 #T35*2 鰊 #T35*1 にしん 二審
にしんぎ #T35*1 二進木
にじ #T35*21 二次 #T35*3 虹 #M5r 滲
にじゅう #T35*94 二十 #T35*20 二重 #T35*1 廿
にじゅうまる #T35*5 ◎
にじょう #CN 二丈 #T30 二乗
にせ #T35*2 偽 #KS*1 似せ #T35 贋
にせい #T35 二世
にせこ #CN ニセコ
にせもの #T35*3 偽物
にせんしき #T35*25 二線式
にた #JCN 仁多
にたて #KS 煮立て
にたどり #JNS 似鳥
にたにた #F04 にたにた
にだい #T35 荷台
にち #JS*255 日 #T35*6 日
にちか #JNM 二千翔
にちげん #T35 日限
にちごう #JS 日号
にちじ #T35*32 日時
にちじょう #T35*7 日常
にちづけ #JS*2 日付
にちどう #KK*2 日動
にちどく #T35 日独
にちなん #CN 日南
にちはら #CN 日原
にちぶ #T35 日舞
にちぶり #JS 日ぶり
にちべい #CN 日米
にちぼつ #T30 日没
にちや #T35*2 日夜
にちよう #T35*26 日曜
にちようひん #T35*1 日用品
にちようび #T35*18 日曜日
にちれん #JNS 日蓮
にっか #T35*1 日課
にっかん #T35*3 日刊 #T35 日韓
にっき #T35*1 日記
にっきゅう #T35 日給
にっきょうそ #KK 日教組
にっきん #T30*1 日勤
にっけい #T35*25 日系 #KK*16 日経 #T35*4 日計
にっける #T35 ニッケル
にっこう #T35*4 日光 #KK*3 日興 #KK*1 日航 #CN 日光
にっこり #F04*1 にっこり
にっころが #S5r 煮っころが
にっさん #T30*1 日参 #T35*1 日産
にっし #T35 日誌
にっしゃ #T35 日射
にっしょう #KK*1 日商 #T35 日照
にっしょく #T35*1 日食
にっしん #CN 日進
にっしんげっぽ #T10*1 日進月歩
にっしんせいこう #KK*2 日新製鋼
にっすう #T35*3 日数
にっそ #CN 日ソ
にった #JCN 新田
にっちゅう #T35*2 日中
にっちょく #T35 日直
にってい #T35*82 日程
にっとう #KK*1 日東 #T35*1 日当
にっぽう #T35*2 日報
にっぽり #CN 日暮里
にっぽん #JCN*8 日本
にっぽんでんき #KK*32 日本電気
にっぽんばし #CN 日本橋
につかわし #KY 似つかわし
につかわしゅう #KYU 似つかわしゅう
につけ #KSr 煮つけ
につま #R5*1 煮詰ま
にづくり #T30*3 荷作り #T30 荷造り
にとろ #T35*2 ニトロ
にとろぐりせりん #T35*4 ニトログリセリン
にど #T35*5 二度
にな #W5*10 担
ににんしょう #T35 二人称
にぬし #T35 荷主
にのうで #T35 二の腕
にのく #T35 二の句
にのへ #JCN 二戸
にのみや #JCN 二宮
にびいろ #T15 鈍色
にふだ #T35 荷札
にぶ #KY*3 鈍 #R5r*2 鈍
にぶう #KYU*1 鈍う
にぶん #T30 二分
にぶんぎ #T35*1 二分木
にへい #JNS 二瓶
にほうせい #T35*1 二峰性
にほん #T35*13 二本 #JNS 日本
にほんいち #T35*2 日本一
にほんかい #KK 日本海
にほんこうかん #KK*1 日本鋼管
にほんご #KK*255 日本語
にほんごか #T30*32 日本語化
にほんしゅ #T35*3 日本酒
にほんしょき #KK*1 日本書紀
にほんじん #T35*18 日本人
にほんでんき #KK*21 日本電気
にほんばれ #T35 日本晴れ
にほんまつ #JCN 二本松
にぼし #T35 煮干
にま #CN 迩摩
にまいごし #T15 二枚腰
にまいじた #T15 二枚舌
にまめ #T35 煮豆
にみや #JNS 二宮
にもつ #T35*14 荷物
にもにっく #T35*1 ニモニック
にもの #T30 煮物
にや #S5 煮や
にやけ #KS にやけ
にやりと #F12 にやりと
にゅー #PRE*6 ニュー #T35*3 Ν #T35*1 ν
にゅーす #T35*232 ニュース
にゅーず #T35*13 ニューズ
にゅーとん #JN ニュートン
にゅーよーく #CN ニューヨーク
にゅーろん #T35*1 ニューロン
にゅあんす #T35*1 ニュアンス
にゅう #KJ*19 入 #KJ 柔 #JCN 丹生 #KJ 乳
にゅういん #T30*2 入院
にゅうえき #T35 乳液
にゅうえん #T30*1 入園
にゅうか #T30*3 入荷 #T30 乳化
にゅうかい #T30*2 入会
にゅうかいきん #T35*2 入会金
にゅうかく #T30 入閣
にゅうかわ #CN 丹生川
にゅうがく #T30*3 入学
にゅうがくきん #T35 入学金
にゅうがくしき #T35 入学式
にゅうきょ #T30 入居
にゅうきょう #T30*1 入鋏
にゅうきん #T30*17 入金
にゅうぎょう #T35 乳業
にゅうこ #T30 入庫
にゅうこう #T30*1 入構 #T30 入港
にゅうこく #T30 入国
にゅうこん #T30*1 入魂
にゅうさつ #T30*1 入札
にゅうさん #T35 乳酸
にゅうし #T35*4 入試 #T35 乳歯
にゅうしつ #T30 入室
にゅうしゃ #T30*9 入社
にゅうしゅ #T30*65 入手
にゅうしゅつりょく #T30*46 入出力
にゅうしょ #T30*1 入所
にゅうしょう #T30*1 入賞
にゅうじ #T35*1 乳児
にゅうじょう #T30*5 入場
にゅうせいひん #T35*1 乳製品
にゅうせき #T30*1 入籍
にゅうせん #T30*1 入選
にゅうぜん #JCN 入善
にゅうたい #T30 入隊
にゅうだん #T30 入団
にゅうてい #T30 入廷
にゅうでん #T30 入電
にゅうとう #T35*1 乳頭 #T30 入党
にゅうとうきん #T35*1 乳頭筋
にゅうねん #T05*1 入念
にゅうばい #T30 入梅
にゅうぶ #T30*2 入部
にゅうぼう #T35*1 乳房
にゅうもん #T30*30 入門
にゅうもんしょ #T35 入門書
にゅうよう #T05*2 入用
にゅうようじ #T35 乳幼児
にゅうよく #T30*2 入浴
にゅうりょう #T30*3 入寮
にゅうりょく #T30*255 入力
にゅうりょくせん #T35*33 入力線
にゅうわ #T05*1 柔和
にょ #KJ*3 如 #KJ*2 女
にょい #T35*1 如意
にょう #T35*4 尿
にょうかん #T35 尿管
にょうぼう #T35*3 女房
にょうまく #T35*1 尿膜
にょらい #T35*1 如来
にょろ #T35*1 〜
によど #CN 仁淀
にら #M5r*3 睨 #T35 韮
にらさき #JCN 韮崎
にらみあわせ #KS にらみ合わせ
にらやま #JCN 韮山
にりゅう #T35 二流
にりん #T35*1 二輪
にわ #T35*42 庭 #JNS 丹羽
にわか #T05*3 俄 #T15*1 にわか
にわかあめ #T35 にわか雨
にわき #T35 庭木
にわさき #T35 庭先
にわとり #T35*4 鶏
にん #JS*172 人 #T35*1 任 #KJ*1 忍 認 #SUC 人 #T35 人 #KJ 仁 #ZX 任 #KJ 妊
にんい #T05*37 任意
にんか #T30*2 認可
にんき #T35*6 人気 #T35*1 任期
にんきょう #T35 仁侠
にんぎょう #T35*19 人形
にんぐみ #JS 人組
にんげつ #JS*1 人月
にんげん #T35*127 人間
にんげんせい #T35 人間性
にんげんみ #T35 人間味
にんさんぷ #T35 妊産婦
にんしき #T30*70 認識
にんしょう #T30*2 認証 #T35*1 人称
にんしん #T30*1 妊娠
にんじ #T35 人時
にんじゃ #T35*3 忍者
にんじゅう #T30 忍従
にんじょう #T35*1 人情 #T35 刃傷
にんじん #T35*5 人参
にんずう #T35*34 人数
にんそう #T35 人相
にんそく #T35 人足
にんたい #T30*2 忍耐
にんち #T30*2 認知 #T35 任地
にんてい #T30*2 認定
にんにく #T35 ニンニク
にんぴ #T35 認否
にんぷ #T35 人夫 妊婦
にんまり #F04 にんまり
にんむ #T35*1 任務
にんめい #T30 任命
にんめん #T30 任免
ぬ #K5r*14 抜 #R5r*8 塗 #KJ*7 濡 #W5*3 縫 #G5*2 脱
ぬいぐるみ #T35*2 ぬいぐるみ
ぬいこ #M5r 縫い込
ぬいつけ #KSr 縫いつけ
ぬいめ #T35 縫目
ぬいもの #T30*2 縫物
ぬえ #T35 鵺
ぬか #S5*3 抜か #T35*1 糠 #R5r 抜か
ぬかた #JCN 額田
ぬき #SUC*7 抜き #T35*1 貫
ぬきさし #T30*1 抜き差し
ぬきと #R5r 抜き取
ぬきんで #KS*1 抜きん出
ぬく #KYmi 温 #M5r 温
ぬくう #KYU 温う
ぬくぬく #F06 ぬくぬく
ぬくも #R5r 温も
ぬくれおちど #T35*1 ヌクレオチド
ぬぐ #W5*10 拭 #W5*1 ぬぐ
ぬけ #KSr*70 抜け
ぬけだ #S5*4 抜け出
ぬけめ #T35 抜目
ぬし #T35*5 主
ぬす #M5r*2 盗
ぬすみ #T35*1 盗み
ぬすみだ #S5 盗み出
ぬたく #R5r ぬたく
ぬの #T35*2 布
ぬのきれ #T35 布切れ
ぬま #T35*22 沼
ぬまおか #JNS 沼岡
ぬまくま #CN 沼隈
ぬまた #JCN 沼田 #T35 沼田
ぬまち #T35 沼地
ぬまづ #CN 沼津
ぬら #S5 濡ら
ぬり #T35*1 塗
ぬりつけ #KSr 塗り付け
ぬりつぶ #S5r*14 塗りつぶ
ぬる #M5*1 ぬる #KY*1 温 #M5*1 温
ぬるう #KYU*1 温う
ぬるぬる #F04 ぬるぬる
ぬるまゆ #T35*1 ぬるま湯
ぬれ #KSr*2 濡れ
ね #KS*106 寝 #T35*9 根 #R5r*4 練 #T35*2 値 #KJ*2 禰 #T35 音 #KJ 子 祢
ねーみんぐ #T30*2 ネーミング
ねーむ #T35*8 ネーム
ねあがり #T30*1 値上がり #JCN 根上
ねあげ #T30*2 値上げ
ねい #R5r 寝入 #KJ 寧 #JCN 婦負
ねいき #T35 寝息
ねいしん #T35 佞臣
ねいてぃぶ #T05*1 ネイティブ
ねいろ #T35 音色
ねうち #T35*1 値打ち
ねえ #CJ*6 ねえ #T35*2 ねえ #T35*1 姉
ねえさん #T35 姉さん
ねお #JCN 根尾
ねおん #T35*1 ネオン
ねか #S5*4 寝か
ねが #W5r*50 願
ねがい #T35*2 願
ねがいさげ #KSr 願い下げ
ねぎ #T35*2 葱 #R5r*1 値切
ねぎし #JCN 根岸
ねぎら #W5r 労
ねくたい #T35*2 ネクタイ
ねぐりじぇ #T35*1 ネグリジェ
ねこ #T35*17 猫 #M5r*3 寝込
ねこじた #T05 猫舌
ねこじゅうじしゃ #KK*2 猫十字社
ねこぜ #T15 猫背
ねこま #CN 猫魔
ねごこち #T35 寝心地
ねごと #T35*1 寝言
ねさがり #T30*1 値下がり
ねさげ #T30*4 値下げ
ねざ #S5 根ざ
ねざめ #T35 寝覚め
ねじ #R5r*1 ねじ #R5r 捻 #T35 螺
ねじめ #JCN 根占
ねすと #T30*3 ネスト
ねず #JCN 根津
ねずみ #T35*1 鼠
ねた #T35*22 ネタ #M5r ねた 妬
ねだ #R5*1 ねだ
ねだん #T35*34 値段
ねちがえ #KS 寝違え
ねっ #S5 熱 #SX 熱
ねっき #T35*1 熱気
ねっきょう #T30*1 熱狂
ねっく #T35*3 ネック
ねっこ #T35 根っ子
ねっしゃびょう #T35 熱射病
ねっしょう #T30 熱唱
ねっしん #T05*8 熱心
ねっせん #T30 熱戦
ねったい #T35*3 熱帯
ねっちゅう #T30*2 熱中
ねっと #T35*45 ネット
ねっとう #T35 熱湯
ねっとり #F04 ねっとり
ねっとわーきんぐ #T30*3 ネットワーキング
ねっとわーく #T35*224 ネットワーク
ねっぱ #T35 熱波
ねっぷう #T35 熱風
ねつ #T35*20 熱 #K5r*1 寝つ
ねつい #T35*1 熱意
ねつえん #T30 熱演
ねつぞう #T30 捏造
ねつべん #T30*1 熱弁
ねつぼう #T30*1 熱望
ねつりょう #T35 熱量
ねつれつ #T05*1 熱烈
ねづ #K5*1 根付 #JCN 根津
ねづよ #KY*1 根強
ねづよう #KYU*1 根強う
ねどこ #T35*1 寝床
ねはば #T35 値幅
ねはん #T35*2 涅槃
ねば #R5r*1 粘 #CN 根羽
ねばりづよ #KY 粘り強
ねばりづよう #KYU 粘り強う
ねびえ #T30 寝冷え
ねびき #T30*1 値引き
ねふだ #T35 値札
ねぶか #KY 根深
ねぶこう #KYU 根深う
ねぶそく #T00*1 寝不足
ねまわし #T30*1 根回し
ねむ #R5r*34 眠 #KY*32 眠
ねむう #KYU*2 眠う
ねむけ #T35*1 眠気
ねむた #KY*1 眠た
ねむとう #KYU*1 眠とう
ねむら #S5 眠ら
ねむろ #CN 根室
ねもと #JNS 根本
ねや #KJ 閨
ねやがわ #JCN 寝屋川
ねら #W5r*11 狙 #W5r*4 ねら
ねり #T35*1 練り
ねりあげ #KSr*1 練り上げ
ねりあわせ #KSr 練り合わせ
ねりなお #S5r 練り直
ねりま #JCN 練馬
ねるそん #JN ネルソン
ねん #JS*255 年 #T35*225 年 #T35*34 念 #ZX*22 念 #KJ*2 稔 #KJ 然 捻 撚 燃 粘
ねんいり #T18 念入り
ねんかん #JS*13 年間 #T35*10 年間 #T35 年刊
ねんが #T35*1 年賀
ねんがく #T35 年額
ねんがじょう #T35*11 年賀状
ねんがっぴ #T35*14 年月日
ねんがん #T30*1 念願
ねんき #T35*1 年忌 #T35 年期 年季
ねんきん #T35*5 年金 #T35 粘菌
ねんぐ #T35 年貢
ねんげつ #T35*1 年月
ねんこう #T35 年功
ねんご #JS*1 年後
ねんごう #T35*1 年号
ねんさん #T35 年産
ねんざ #T30*2 捻挫
ねんし #T35*1 年始
ねんしゅう #T35*1 年収
ねんしゅつ #T30*1 捻出
ねんしょ #T35*1 念書
ねんしょう #T30*1 燃焼 #T35 年商 #T05 年少
ねんじ #T35*1 年次
ねんじゅう #T35*5 年中
ねんすう #T35*1 年数
ねんせい #JS*8 年生
ねんだい #JS*3 年代 #T35*2 年代
ねんちゃく #T30 粘着
ねんちょう #T05*1 年長
ねんてん #T30 捻転
ねんとう #T35*4 念頭 #T35*1 年頭
ねんど #JS*26 年度 #T35*22 年度 #T35 粘土
ねんどまつ #T35*2 年度末
ねんない #T35*1 年内
ねんねん #T35*2 年々
ねんぱい #T35*1 年配 #T35 年輩
ねんぴ #T35*2 燃費
ねんぴょう #T35*1 年表
ねんぶたーる #T35*1 ネンブタール
ねんぶつ #T30*1 念仏
ねんぶり #JS*5 年ぶり
ねんべつ #T35 年別
ねんぽう #T35 年俸 年報
ねんまく #T35 粘膜
ねんまつ #T35*5 年末
ねんらい #T35 年来
ねんり #T35 年利
ねんりき #T35 念力
ねんりょう #T35*1 燃料
ねんりん #T35 年輪
ねんれい #T35*7 年齢
の #T35*188 野 #R5*168 載 #M5r*163 飲 #R5*140 乗 #S5*24 の #K5*20 退 #R5*18 の #KJ*16 之 #M5*3 呑 #KJ*3 乃 #M5r*2 の #KJ 廼 埜
のー #CJ*3 ノー
のーと #T30*9 ノート
のーど #T35*102 ノード
のーまる #T05*3 ノーマル
のあ #JN ノア
のいず #T35*16 ノイズ
のいち #JCN 野市
のいまん #KK*3 ノイマン
のいろーぜ #T35*2 ノイローゼ
のう #T35*10 能 #KJ*9 納 #T35*7 脳 #KJ*3 農 #KYU*3 無う #KYU*2 のう #KJ*1 濃 #KJ 嚢 悩 #JCN 能生 #KJ 膿
のういっけつ #T35 脳溢血
のうえん #T35 脳炎 農園
のうか #T35*1 農家
のうかい #T35*4 納会
のうかすいたい #T35 脳下垂体
のうかん #T35*1 脳幹 #T30 納棺
のうかんき #T35 農閑期
のうがく #T35*1 農学 #T35 能楽
のうがた #JCN 直方
のうき #T35*8 納期
のうきぐ #T35 農機具
のうきょう #T35 農協
のうきん #T30 納金
のうぎょう #T35*4 農業
のうぐ #T35 農具
のうこう #T05*1 濃厚 #T30 農耕
のうこうそく #T35*1 脳硬塞
のうこつ #T30 納骨
のうさぎ #T35 野兎
のうさぎょう #T35 農作業
のうさつ #T30 悩殺
のうさん #T35*1 農産
のうし #T30 脳死
のうしゃ #T30*2 納車
のうしゅく #T30 濃縮
のうじょう #T35 農場
のうずい #T35 脳髄
のうぜい #T30 納税
のうそっちゅう #T35 脳卒中
のうそん #T35*1 農村
のうたん #T35*2 濃淡
のうち #T35*6 農地
のうてんき #T05 能天気
のうど #T35*1 濃度
のうどう #T35*1 能動
のうない #T35*2 脳内
のうにゅう #T30*12 納入
のうのうと #F14 のうのうと
のうは #T35 脳波
のうはう #T35*13 ノウハウ
のうはんき #T35 農繁期
のうひん #T30*18 納品
のうふ #T30*4 納付 #T35*1 農夫
のうまく #T35 脳膜
のうみ #JCN 能美
のうみつ #T05*1 濃密
のうみん #T35*1 農民
のうむ #T35*1 濃霧
のうめん #T35 能面
のうやく #T35 農薬
のうよう #T35*1 膿瘍
のうり #T35 脳裏
のうりつ #T35*1 能率
のうりょう #T35*2 納涼 #T35 脳梁
のうりょく #T35*28 能力
のうりん #T35*1 農林
のうりんすいさんしょう #KK 農林水産省
のおがた #JCN 直方
のかみ #JCN 野上
のが #S5*5 逃
のがた #JCN 野方
のがみ #JNS 野上
のがれ #KSr*3 逃れ
のき #T35*1 軒
のきさき #T35 軒先
のきなみ #T35*1 軒並
のぎ #T35 禾 #JCN 能義 #JNS 野義 #JCN 野木
のぐち #JCN 野口
のけ #KS*13 除け #KS 退け
のこ #R5r*117 残 #S5r*117 残
のこぎり #T35*1 鋸
のこのしま #JCN 能古島
のこらず #F14 残らず
のこり #SUC*16 残り #T35*1 残り
のさか #JCN 野栄
のざき #JCN 野崎
のざわ #JCN 野沢
のざわおんせん #CN 野沢温泉
のしろ #JCN 能代
のじま #JNS 野島 野嶋
のじゅく #T30*1 野宿
のじり #JCN 野尻
のせ #KS*55 載せ #KS*15 乗せ #JCN 能勢
のせがわ #JCN 野迫川
のぞ #K5*80 除 #M5r*17 望 #K5r*15 覗 #M5*1 臨
のぞえ #JNS 野副
のぞき #KJ 覗
のぞきこ #M5r*1 のぞき込
のぞまし #KY*36 望まし
のぞましゅう #KYU*2 望ましゅう
のぞみ #T35 望み
のたう #T5r のたう
のたま #W5*2 のたま #W5 宣
のだ #JCN 野田
のだがわ #JCN 野田川
のち #T35*45 後
のちのち #T35 後々
のっか #R5 乗っか
のっけ #T35 のっけ #R5 乗っけ
のっと #R5r*2 乗っ取 #R5*1 則
のっといこーる #T35*1 ≠
のつ #JCN 野津
のつけ #JCN 野付
のつはる #CN 野津原
のてん #T35*4 野天
のてんぶろ #T35*1 野天風呂
のと #JCN 能都
のとかわ #JCN 能登川
のとじま #JCN 能登島
のとばいおーと #T35*1 ノトバイオート
のど #T35*5 喉 #T35 咽
のどか #T05 のどか 長閑
のなか #T35*59 野中 #JNS 野中
ののいち #CN 野々市
ののし #R5r 罵
のはら #JNS 野原
のば #S5r*35 伸ば #S5r*7 延ば
のばなし #T00 野放し
のび #KSr*9 延び #KSr*4 伸び
のびなや #M5r 伸び悩
のびのび #F04*1 のびのび #T30 伸び伸び
のぶ #KJ*4 伸 #KJ*1 惟 延 宣 #KJ 允 信 暢
のぶあき #JNM 信明
のぶお #JNM 伸生 伸夫 信生 信男 信夫 信雄 陳雄
のぶおき #JNM 暢興
のぶこ #JNM 伸子 信子
のぶたか #JNM 信孝
のぶなが #JNM 信長
のぶひこ #JNM 信彦 宣彦
のぶひろ #JNM 伸浩
のぶまさ #JNM 宣政
のぶや #JNM 信也
のぶゆき #JNM 信幸 信行 信之
のぶよし #JNM 信義
のへじ #JCN 野辺地
のべ #KS*121 述べ #KSr*6 延べ #KS*1 伸べ #RT 延べ #JNS 野辺
のべおか #JCN 延岡
のほうず #T05 野放図
のほほんと #F12*2 のほほんと
のぼ #R5r*18 登 #R5r*2 のぼ #R5r*1 昇 上
のぼせ #KSr*1 のぼせ
のぼりつめ #KS 上り詰め
のぼりべつ #CN 登別
のぼる #JNM 昇 登
のま #KJ*1 々
のみ #JCN 能美 #T35 蚤
のみねーと #T30*1 ノミネート
のみほ #S5 飲み干
のみもの #T35*7 飲物
のみや #T35*4 飲み屋
のむら #JNS 野村
のめ #R5 のめ
のめりこ #M5r のめり込
のもざき #JCN 野母崎
のもと #JNS 野本
のやま #T35 野山
のら #T35*1 野良
のらくら #F04 のらくら
のらりくらり #F04 のらりくらり
のり #T35*4 糊 #T35*3 海苔 #KJ*2 規 則 #KJ*1 憲 朔 徳 範 #KJ 紀 矩 弼
のりあき #JNM 則明 範明
のりあげ #KS 乗り上げ
のりいれ #KSr*1 乗り入れ
のりうつ #R5r*1 乗り移
のりお #JNM 紀男 紀夫 則夫
のりおくれ #KSr*1 乗り遅れ
のりおり #T30*2 乗り降り
のりかえ #KSr*13 乗り換え #T35*1 乗換
のりかか #R5 乗りかか
のりき #R5*3 乗り切
のりく #M5r 乗り組
のりくみいん #T35 乗組員
のりくら #JCN 乗鞍
のりこ #M5r*2 乗り込 #JNM のり子 紀子 #S5r 乗り越 #JNM 典子 徳子 法子
のりこえ #KS*2 乗り越え
のりすて #KSr 乗り捨て
のりだ #S5*1 乗り出
のりつ #G5r*1 乗り継
のりつけ #KS 乗り付け
のりづけ #T30 糊付け
のりと #T35 祝詞
のりば #T35 乗り場
のりひさ #JNM 範久
のりひろ #JNM 典大
のりまわ #S5 乗り回
のりもの #T35*3 乗物
のりゆき #JNM 紀幸 紀之 矩之
のるうぇー #CN ノルウェー
のるむ #T35*2 ノルム
のろ #KY*2 のろ #W5r*2 呪 #KY*2 鈍 #JCN 野呂
のろう #KYU*1 のろう 鈍う
のろし #T35 狼煙
のろのろ #F04 のろのろ
のろわし #KY 呪わし
のろわしゅう #KYU 呪わしゅう
のん #PRE*3 ノン
のんき #T05 暢気
のんたーみなる #T05*1 ノンターミナル
のんびり #F04*6 のんびり
のんべんだらり #F06 のんべんだらり
は #T35*157 葉 #K5*100 吐 #R5r*85 張 #KJ*56 羽 #JNSUC*42 派 #W5*29 這 #K5*13 は #R5*12 貼 #T35*5 歯 #KJ*5 破 #KJ*3 覇 #KJ*2 端 播 波 #K5*2 履 #T35*1 刃 #KJ*1 派 芭 #G5*1 剥 #KJ*1 玻 #M5 喰 #KJ 頗 #K5 掃 #KJ 巴 把 杷 #SUC 波 派 #T35 派 #KJ 琶 #G5 矧 #KJ 簸 #K5 噴
はーと #T35*3 ハート
はーど #T05*52 ハード
はーどうぇあ #T35*91 ハードウェア
はーどうえあ #T35*4 ハードウエア
はーふ #PRE*2 ハーフ
はーぶ #T35*1 ハーブ
はあ #CJ*5 はあ
はあく #T30*26 把握
はい #R5r*255 入 #S5*82 排 #SX*82 排 #CJ*33 はい #KJ*13 配 #S5*5 廃 #SX*5 廃 #R5r*3 はい #S5*3 拝 #SX*3 拝 #KJ*3 背 #T35*2 肺 #S5*2 配 #SX*2 配 #T35*1 灰 #JS*1 敗 #KJ*1 牌 #PRE ハイ #KJ 埴 俳 敗 #JS 杯 #KJ 杯 盃 輩 這 稗
はいあん #T35 廃案
はいい #T35 配位
はいいろ #T35*2 灰色
はいいん #T35*1 敗因 #T30*1 配員
はいうぇい #T35*1 ハイウェイ
はいえん #T35 肺炎
はいえんど #T05*6 ハイエンド
はいか #T30 排架 #T35 配下 #T30 配架
はいかい #T35*1 俳諧 #T35 徘徊
はいかん #T30 廃刊 拝観 配管
はいがん #T35 肺癌
はいき #T30*10 排気 #T30*6 廃棄
はいきゅう #T30*2 廃休 #T30 配給
はいきょ #T35*1 廃虚
はいきん #T35 背筋
はいきんぐ #T35*2 ハイキング
はいぎょう #T30*1 廃業
はいく #T35*1 俳句
はいぐうし #T35*1 配偶子
はいぐうしゃ #T35*1 配偶者
はいけい #T35*33 背景 #T35*4 拝啓
はいけつしょう #T35*1 敗血症
はいけん #T30*5 拝見
はいこう #T35*1 廃鉱 #T35 廃校
はいご #T35*1 背後
はいごう #T30 配合
はいざら #T35*1 灰皿
はいし #T30*14 廃止
はいしゃ #T35*8 歯医者 #T35*5 敗者 #T30*2 配車 #T35*1 廃車
はいしゃく #T30*3 拝借
はいしゅつ #T30*3 排出
はいしょく #T30 配色
はいしん #T35 背信
はいじゃっく #T30 ハイジャック
はいじょ #T30*8 排除
はいじん #T35*1 俳人 #T35 廃人
はいすい #T30*2 排水 #T30*1 配水
はいすいのじん #T35 背水の陣
はいず #R5 這いず
はいせき #T30*1 排斥
はいせつ #T30*1 排泄
はいせん #T30*6 配線 #T30 敗戦
はいぜん #T30 配膳
はいそう #T30*23 配送
はいそく #T35*1 背側
はいぞく #T30*9 配属
はいた #T30*4 排他
はいたい #T30*1 敗退
はいたつ #T30*6 配達
はいち #T30*24 配置
はいちょう #T30*2 拝聴
はいつ #T35*4 ハイツ
はいてく #T05*1 ハイテク
はいでん #T30*1 配電
はいとう #T30*1 配当
はいにん #T30 背任
はいのう #T35 背嚢
はいはん #T35*1 排反 #T30 背反
はいばら #JCN 榛原
はいばん #T35*1 廃盤
はいばんほう #T35*1 胚盤胞
はいぱー #PRE*6 ハイパー #T05 ハイパー
はいひん #T35 廃品
はいび #T30 配備
はいふ #T30*152 配布 #T30*21 配付
はいふく #T35 拝復
はいふねーしょん #T30*1 ハイフネーション
はいふん #T35*8 ハイフン
はいぶつ #T35 廃物
はいぶん #T30*6 配分
はいぼく #T30*1 敗北
はいむ #T35*2 ハイム
はいめん #T30*1 背面
はいゆう #T35*2 俳優
はいよう #T35*1 胚葉
はいりこ #M5r*1 入り込
はいりほう #T35*2 背理法
はいりょ #T30*7 配慮
はいれつ #T30*86 配列 #T35 排列
はいれべる #T05*1 ハイレベル
はうす #T35*6 ハウス
はえ #KS*2 生え #KSr*1 映え #T35*1 蝿 #KSr 栄え #KS 這え
はえばる #CN 南風原
はお #R5 羽織
はおり #T35 羽織
はか #R5r*42 はか #R5r*13 計 #R5*6 図 #R5r*5 測 #T35*3 墓 #T30*1 破瓜 #R5 諮 謀 #R5r 量
はかい #T30*34 破壊 #T35 破戒
はかせ #T35*4 博士
はかた #JCN 伯方 博多
はかど #R5r*2 捗
はかな #KY*2 はかな #KY 儚
はかのう #KYU*1 はかのう #KYU 儚う
はかま #T35*1 袴
はかまいり #T30 墓参り
はから #W5r*1 計ら
はからずも #F14*1 はからずも
はかり #T35 秤
はかりごと #T35 謀
はが #S5r*1 はが #JNS 波賀 #S5r 剥 #JCN 芳賀
はがき #T35*5 葉書 #T35*4 はがき
はがね #T35 鋼
はき #T30*10 破棄 #T35*1 覇気 #JCN 杷木
はきだ #S5r*2 吐き出
はきもの #T35*4 履物
はきゅう #T30*2 波及
はきょく #T30 破局
はぎ #T35*13 萩 #T35 矧 #JCN 萩
はぎお #JNS 萩尾
はぎしり #T30 歯軋り
はぎの #JCN 萩野
はぎや #JNS 萩谷
はぎょう #T35 覇業
はぎれ #T35*2 歯切れ #T35 端切れ
はぎわら #JCN 萩原
はく #JS*24 泊 #T30*13 泊 #KJ*4 白 #KJ*1 伯 博 #S5*1 博 #SX*1 博 #T35*1 箔 #KJ*1 薄 #KJ 栢 狛 剥 #JS 拍 #KJ 拍 柏 粕 舶 迫
はくあ #T35 白亜
はくあい #T35 博愛
はくい #T35*1 白衣 #JCN 羽咋
はくがい #T30 迫害
はくがく #T05 博学
はくさい #T35 白菜
はくさん #JCN 白山
はくし #T35*10 白紙 #T35*8 博士
はくしき #T05 博識
はくしゃ #T35 拍車 薄謝
はくしゃく #T35*1 伯爵
はくしゅ #T30*3 拍手
はくしゅう #CN 白州
はくしょ #T35*2 白書
はくしょく #T05*3 白色
はくしん #T35*1 迫真
はくじ #T35 白磁
はくじつ #T35 白日
はくじゅ #T35 白寿
はくじょう #T05*1 薄情 #T30 白状
はくじん #T35*1 白人
はくすい #JCN 白水
はくせい #T35 剥製
はくせき #T35 白皙
はくせんきん #T35 白癬菌
はくせんしゃ #KK*2 白泉社
はくた #JCN 伯太
はくだく #T30 白濁
はくだつ #T30 剥奪
はくち #T35*1 白痴
はくちゅう #T35*1 白昼
はくちょう #T35*1 白鳥
はくどう #T30*1 拍動 #T35 白銅
はくねつ #T30*1 白熱
はくば #CN 白馬
はくばい #T35 白梅
はくび #T35 白眉
はくぶつ #T35*1 博物
はくぶつかん #T35*2 博物館
はくぶつし #T35 博物誌
はくほうどう #KK 博報堂
はくぼく #T35*2 白墨
はくまい #T35 白米
はくまく #T35*1 白膜
はくらい #T35 舶来
はくらく #T35 伯楽 #T30 剥落
はくらんかい #T35*1 博覧会
はくり #T30*1 剥離 #T35*1 白痢
はくりょく #T35*2 迫力
はくろ #T35*1 白露
はくろん #T35*1 博論
はぐく #M5 育
はぐり #JCN 葉栗
はぐるま #T35*1 歯車
はぐろ #JCN 羽黒
はけ #KS*1 はけ #T35 刷毛 #KS 捌け
はけい #T35*2 波形
はけん #T30*3 派遣
はげ #M5r*2 励 #T35*1 ハゲ #KS はげ #T35 禿 #KS 禿げ 矧げ #KSr 剥げ
はげし #KY*12 激し #KY はげし 烈し
はげしゅう #KYU*2 激しゅう #KYU はげしゅう 烈しゅう
はげま #S5r*2 励ま
はこ #T35*36 箱 #B5r*14 運 #T35 函
はこう #T30*2 跛行
はこさきぐう #JCN 筥崎宮
はこだて #CN 函館
はこね #JCN 箱根
はこねゆもと #CN 箱根湯本
はこびこ #M5r*1 運び込
はこびだ #S5r*1 運び出
はこぶね #T35*1 箱船 #T35 方船
はごいた #T35 羽子板
はさ #M5*5 はさ #M5*3 挟
はさい #T30 破砕
はさき #JCN 波崎
はさま #JCN 挟間 迫
はさみ #JCN 波佐見 #T35 鋏
はさみこ #M5r 挟み込
はさん #T30*1 破産
はざかいき #T35 端境期
はざま #T35*1 狭間 #T35 間 硲
はし #R5r*74 走 #T35*67 橋 #T35*27 端 #CNSUC1*16 橋 #T35*5 箸 #KJ*2 梯
はしうら #JNS 橋浦
はしかみ #JCN 階上
はしがき #T35*3 はしがき
はしぐち #JNS 橋口
はしご #T35 梯子
はしだ #JNS 橋田
はしづめ #T35*2 橋詰 #JNS 橋爪
はしま #JCN 羽島
はしもと #JCN 橋本
はしゃ #G5*10 はしゃ #T35*1 覇者
はしゅつ #T30 派出
はしゅつじょ #T35 派出所
はしら #T35*10 柱
はしりがき #T30 走り書き
はしりこ #M5r 走り込
はしりだ #S5r*2 走り出
はしりぬけ #KSr 走り抜け
はしりまわ #R5*1 走り回
はしわたし #T30*2 橋渡し
はじ #T35*8 恥 #K5*3 弾 #KS*3 恥じ #K5*1 はじ #JCN 土師
はじい #R5 恥じ入
はじま #R5r*83 始ま #R5r*2 はじま
はじまり #T35*1 はじまり
はじめ #KSr*184 始め #KSr*148 はじめ #T35*8 初め #T35*1 一 #JNM 一 元 創 肇 甫
はじめて #T35*81 初めて #F14*47 初めて
はす #T35*4 蓮
はすい #T30 破水
はすいけ #JCN 蓮池
はすう #T35*2 端数
はすだ #JCN 蓮田
はすぬま #JCN 蓮沼
はすは #JNS 泰葉
はすみ #JCN 羽須美 #JNS 蓮見
はず #T35*198 はず #T35*46 筈 #S5*41 外 #M5r*1 はず #M5r 弾 #JCN 幡豆
はずかし #KY*11 恥ずかし #KY*1 はずかし 恥し
はずかしめ #KSr 辱め
はずかしゅう #KYU*2 恥ずかしゅう #KYU*1 はずかしゅう 恥しゅう
はずれ #KSr*7 外れ #T35*3 はずれ
はせ #KS*1 馳せ #JCN 長谷
はせい #T30*2 派生 #JNS 長谷井
はせがわ #JNS 長谷川
はせべ #JNS 長谷部
はせん #T35*13 破線 #T35 波線
はぜ #T35*1 鯊 #KS はぜ #T35 櫨
はそん #T30*1 破損
はた #T35*255 畑 #S5*27 果た #T35*7 旗 端 #T35 機 #JNS 秦 波田 #KJ 幡 #JCN 幡多 #JN 畑 #T35 畠 #KJ 圃
はたいろ #T35 旗色
はたえ #T35 二十重
はたおり #T30 機織り
はたけ #T35*3 畑 #T35 畠
はたけやま #JNS 畠山
はたご #T35 はたご
はたして #F14 はたして 果して
はたしょう #CN 秦荘
はたち #T35*1 二十歳
はたと #F14*1 はたと
はたなか #JNS 畑中 畠中
はたの #JNS 波多野 幡野 #JCN 畑野
はたはた #T35*1 鰰
はたやま #JNS 畑山
はたら #K5r*33 働
はたらきかけ #KSr*4 働きかけ
はたん #T30*2 破綻
はだ #T35*6 肌 #T35*4 膚
はだあい #T35 肌合
はだいろ #T15 肌色
はだか #T35*4 裸
はだぎ #T35 肌着
はだざむ #KY 肌寒
はだざむう #KYU 肌寒う
はだざわり #T35 肌触り
はだし #N2T16*1 はだし #T35*1 裸足
はだの #JCN 秦野
はだみ #T35 肌身
はだん #T35 破談
はち #NN*30 八 #T35*4 鉢 #T35*1 ハチ 蜂
はちうえ #T30 鉢植え
はちおうじ #JCN 八王子
はちかい #CN 八開
はちく #T35*1 破竹
はちじょう #CN 八丈
はちのへ #CN 八戸
はちぶ #T35*8 八分
はちまき #T30 鉢巻
はちまん #JCN 八幡 #T35 八幡
はちみつ #T35*1 はちみつ 蜂蜜
はちめんろっぴ #T35 八面六膂
はちもり #JCN 八森
はちょう #T35*4 波長
はちりゅう #JCN 八竜
はちろう #JNM 八郎
はちろうがた #CN 八郎潟
はっ #NN*9 八 #S5*4 発 #SX*4 発
はっか #T30*2 発火 #T35 ハッカ 薄荷
はっかー #T35*2 ハッカー
はっかいさん #CN 八海山
はっかく #T30*1 発覚
はっかん #T30*1 発刊
はっき #T30*5 発揮
はっきゅう #T35 白球 薄給 #T30 発給
はっきょう #T30*2 発狂
はっきり #F04*43 はっきり
はっきん #T30 白金
はっく #T30*1 ハック
はっくつ #T30*1 発掘
はっけっきゅう #T35 白血球
はっけん #T30*64 発見 #T30*1 発券
はっこ #T30*13 発呼
はっこう #T30*118 発行 #T30*2 発効 #T30*1 発光
はっこうしゃ #T35 発行者
はっこつ #T35 白骨
はっさん #T30*1 発散
はっし #F03 発止
はっしゃ #T30*4 発射 #T30*1 発車
はっしゅ #T30*6 ハッシュ
はっしょう #T35*1 八章 #T30*1 発症 #T30 発祥
はっしん #T30*17 発信 #T30*3 発振 #T30*2 発進 #T35 発疹
はっしんしゃ #T35 発信者
はっせい #T30*130 発生 #T30 発声
はっそう #T30*14 発送 #T30*4 発想
はった #JCN 八田
はったつ #T30*6 発達
はっちゃく #T30*2 発着
はっちゅう #T30*19 発注
はってん #T30*15 発展
はっとう #CN 八東
はっとり #JCN 服部
はっぱ #T35*2 葉っぱ
はっぴー #T05*3 ハッピー
はっぴょう #T30*129 発表
はっぴょうかい #T35*16 発表会
はっぴょうしゃ #T35*5 発表者
はっぷ #T30 発布
はっぷん #T30 発奮
はっぽう #JCN 八方 #T30 発砲
はっぽうおね #CN 八方尾根
はつ #CNSUC1*29 発 #T35*13 初 #T35*1 発 #KJ*1 髪 #JNM はつ #SUC 初 #KJ 溌 #JS 発 #KJ 醗
はついく #T30 発育
はつえ #JNM 初江
はつえんとう #T35 発煙筒
はつお #JNM 初男
はつおん #T30*10 発音 #T35 撥音
はつか #T35*4 二十日
はつかいち #CN 廿日市
はつが #T30 発芽
はつがん #T30 発ガン
はつげん #T30*16 発言 #T30 発現
はつこい #T35*1 初恋
はつじょう #T30*1 発情
はつたろう #JNS 初太郎
はつでん #T30*10 発電
はつどう #T30 発動
はつに #T35 初荷
はつねつ #T30*4 発熱
はつばい #T30*14 発売
はつびょう #T30*1 発病
はつみみ #T15*1 初耳
はつめい #T30*3 発明
はつもうで #T30 初詣
はつゆき #T35 初雪
はつれい #T30 発令
はつろ #T30 発露
はつわ #T30*1 発話
はづき #T35 葉月
はて #KSr*10 果て #KSr*4 はて
はてんこう #T05 破天荒
はで #T05*9 派手
はと #T35 鳩
はとう #T35 波涛
はとがや #CN 鳩ヶ谷
はとどけい #T35 鳩時計
はとやま #JCN 鳩山
はとり #JCN 羽鳥
はどう #T35*3 波動
はどめ #T35*1 歯止め
はな #S5*131 話 #T35*68 花 #S5*58 離 #T35*8 鼻 #S5*7 放 #T35*6 華 #T5*2 放 #S5*1 はな #T35*1 はな #JN ハナ #KJ 塙
はない #JNS 花井
はないずみ #JNS 花泉
はなうた #T30 鼻歌
はなおか #JCN 花岡
はなくそ #T35 鼻糞
はなげ #T35*1 鼻毛
はなこ #JNM 花子 華子
はなざかり #T15 花盛り
はなざわ #JNS 花沢
はなし #T30*255 話 #T35*2 噺
はなしあ #W5r*15 話し合
はなしあい #T30*1 話合い
はなしか #T35 噺家
はなしこ #M5*1 話し込
はなぞの #T35*2 花園 #JCN 花園
はなたかだか #T06 鼻高々
はなたば #T35*2 花束
はなだ #JNS 花田
はなぢ #T35 鼻血
はなてん #CN 放出
はなどけい #T35 花時計
はなはだ #F14*1 甚だ
はなはだし #KY 甚だし
はなはだしゅう #KYU 甚だしゅう
はなばなし #KY*1 華々し
はなばなしゅう #KYU*1 華々しゅう
はなび #T35*4 花火
はなびら #T35*2 花びら
はなぶさ #KJ 英
はなまき #CN 花巻
はなみ #T30*5 花見
はなみず #T35 鼻水
はなむこ #T35*1 花婿
はなや #T35 花屋 #G5r 華や
はなやか #T05*3 華やか
はなやま #JCN 花山
はなれ #KSr*16 離れ #KS 放れ
はなわ #JCN 塙
はに #T35*3 埴
はにしな #JCN 埴科
はにゅう #JCN 羽生
はにゅうだ #JNS 羽生田
はにわ #T35*12 埴輪
はね #T35*9 羽 #KSr*4 はね #T35*1 羽根 #KSr*1 跳ね
はねかえ #S5r*2 跳ね返 #R5r*1 跳ね返
はねだ #JCN 羽田
はねむーん #T35*2 ハネムーン
はのうら #CN 羽ノ浦
はは #T35*32 母
ははおや #T35*32 母親
はば #T35*90 幅 #M5*1 阻 #T35 巾
はばか #R5r はばか 憚
はばた #K5r*1 羽ばた
はばつ #T35*1 派閥
はばひろ #KY*10 幅広
はばひろう #KYU*2 幅広う
はびきの #JCN 羽曳野
はびこ #R5*2 はびこ
はふ #R5r 葬
はぶ #K5*11 省
はぶり #T35*2 羽振り
はべ #R5 侍
はべらか #S5 侍らか
はぼろ #CN 羽幌
はま #T35*14 浜 #R5*2 はま #KJ*1 濱 #CNSUC1 浜 #JN 浜
はまおか #JCN 浜岡
はまき #T35*1 葉巻
はまきた #JCN 浜北
はまぐち #JNS 浜口
はまぐり #T35 蛤
はまさか #JCN 浜坂
はまさき #JCN 浜崎
はまざき #JNS 浜崎
はまじま #JCN 浜島
はまだ #JCN 浜田
はまとんべつ #CN 浜頓別
はまな #JCN 浜名
はまなか #JCN 浜中 #JNS 濱中
はまの #JCN 浜野
はまます #JCN 浜益
はままつ #JCN 浜松
はみがき #T30*1 歯磨き
はみがきこ #T35 歯磨き粉
はみだ #S5r*6 はみ出
はみで #KS はみ出
はみんぐ #KK*1 ハミング
はむ #T35 ハム
はむら #JCN 羽村
はめ #KS*3 はめ #T35*2 羽目 #KS*1 填め
はめこ #M5r はめ込
はめつ #T30 破滅
はめん #T35*1 波面
はも #T35*1 鱧 #JNS 羽茂
はもの #T35*1 刃物
はもん #T35 波紋 #T30 破門
はや #KY*255 早 #T35*173 早 #KY*117 速 #R5*22 はや #R5*1 逸 #S5*1 生や #R5r 流行
はやうま #T35 早馬
はやお #JNM 駿
はやおき #T10 早起き
はやかわ #JCN 早川
はやがてん #T10*1 早合点
はやきた #CN 早来
はやく #T35 端役
はやくち #T00 早口
はやさか #JCN 早坂
はやし #T35*25 林 #T35*3 囃子 #JNS 林
はやしだ #JNS 林田
はやしま #JCN 早島
はやしらいす #T35 ハヤシライス
はやせ #T35*1 早瀬
はやて #T35 疾風
はやと #T35*1 隼人 #JCN 隼人 #JNM 勇人
はやとちり #T10*1 早とちり
はやの #JNS 早野
はやばや #F06*2 早々
はやぶさ #T35*1 隼
はやべん #T30 早弁
はやま #R5r*2 早ま #R5r 速ま #JNS 端山 #JCN 葉山
はやみ #JNS 早見 #JCN 速見
はやみず #JNS 早水 速水
はやめ #KS*17 早め #KS*1 速め
はよう #F14*8 早う #KYU*3 早う 速う
はら #W5r*31 払 #T35*21 腹 #CNSUC1*12 原 #S5*8 晴ら #M5*1 はら #JCN 原 #T35 原 #M5r 孕
はらいこ #M5r*1 払い込
はらいさげ #T30 払い下げ
はらいどの #JNS 払殿
はらいもどし #T30*1 払い戻し
はらぐち #JNS 原口
はらぐろ #KY 腹黒 #T05 腹黒
はらぐろう #KYU 腹黒う
はらしま #JNS 原島 原嶋
はらじゅく #JCN 原宿
はらだ #JCN 原田
はらまち #CN 原町
はらわた #T35 腸
はらん #T35*1 波乱
はらんばんじょう #T05 波瀾万丈
はり #T35*3 針 #T35*1 玻璃 #KJ 貼 播 #T35 梁
はりあ #W5r*1 張り合
はりあい #T35 張合い
はりあわせ #KSr 張り合わせ
はりうっど #CN ハリウッド
はりがね #T35 針金
はりがみ #T30*1 張り紙 #T35 貼紙
はりき #R5*3 張り切
はりぐち #JNS 播口
はりこ #M5r*2 張り込 #T35 張子
はりだ #S5r 張り出
はりつけ #KSr*1 張り付け
はりま #JCN 播磨
はりまお #JN ハリマオ
はりまや #JNS 針間矢
はる #T35*32 春
はるえ #JCN 春江
はるお #JNM 治男 治夫 治雄 春男 春夫 春雄 晴夫 晴雄
はるおみ #JNM 晴臣
はるか #T06*12 はるか #T06*8 遥か
はるきち #JNM 春吉
はるこ #JNM 春子
はるし #JNM 治志
はるじ #JNM 春治
はるな #JCN 榛名
はるの #JCN 春野
はるばる #F06*1 はるばる
はるひ #JCN 春日
はるひこ #JNM 治彦 春彦 晴彦
はるひさ #JNM 晴久
はるみ #JNM はるみ 春美 #JCN 晴海
はるみだい #CN 晴美台
はるめ #K5 春め
はるやす #JNM 陽保
はるやすみ #T35*2 春休み
はるゆき #JNM 治之
はるよ #JNM 春代
はれ #KSr*92 晴れ #T35*6 晴 #KSr*1 腫れ
はれあが #R5r 晴れ上が
はれぎ #T35 晴れ着
はれすがた #T35 晴れ姿
はれつ #T30*1 破裂
はれわた #R5r 晴れ渡
はろう #T35 波浪
はわい #CN ハワイ #JCN 羽合
はん #JS*255 版 #T35*46 半 #T30*30 反 #KJ*6 汎 #KJ*3 般 #KJ*2 阪 板 #T35*2 版 #KJ*2 販 #T35*2 範 #KJ*1 幡 #T35*1 判 #ZX*1 判 #KJ*1 搬 飯 #KJ 坂 伴 #SUC 判 #PRE 半 反 #KJ 叛 帆 斑 氾 #SUC 版 #KJ 犯 #SUC 班 #T35 班 #KJ 畔 繁 #T35 藩 #KJ 釆 煩 頒 磐 蕃 凡
はんい #T35*120 範囲
はんえい #T30*79 反映 #T30*5 繁栄
はんえん #T35*1 半円
はんか #T05*1 繁華 #T35 頒価
はんかい #T30 半壊
はんかく #T35*29 半角
はんかつう #T15 半可通
はんかん #T35*4 反感
はんがく #T35*2 半額
はんき #T35*1 半期
はんきかん #T35 半規管
はんきゅう #KK*5 阪急 #T35 半球
はんきょう #T30*2 反響
はんぎゃく #T30 反逆
はんぐ #T30*11 ハング
はんぐりー #T05 ハングリー
はんけい #T35*2 半径
はんけつ #T30 判決
はんけん #T35 半券 版権
はんげき #T30 反撃
はんげん #T30*4 半減
はんこ #T35 判子
はんこう #T30*4 反抗 #T30 犯行
はんご #T35*2 反語
はんさ #T05 煩瑣
はんさき #JNS 飯崎
はんさよう #T30 反作用
はんざい #T35*2 犯罪
はんざつ #T05*3 繁雑 煩雑
はんざん #JCN 飯山
はんし #T35 半紙 藩士
はんした #T35 版下
はんしはんしょう #T15 半死半生
はんしゃ #T30*10 反射
はんしゅう #T30 半周
はんしゅつ #T30 搬出
はんしょう #T30 半焼
はんしょく #T30 繁殖
はんしょくち #T35 繁殖地
はんしん #CN 阪神 #T35 半身
はんしんはんぎ #T10 半信半疑
はんしんろん #T35 汎神論
はんじ #T35*1 判事
はんじゅく #T05 半熟
はんじょう #T30*1 繁盛
はんすう #T30*3 反芻 #T35 半数
はんせい #T30*25 反省
はんせいすうがた #T35*4 汎整数型
はんぜん #F01 判然
はんそ #T30 反訴
はんそう #T30*1 搬送
はんそく #T30*1 反則 #T35 販促
はんそで #T35*2 半袖
はんぞうもん #CN 半蔵門
はんたい #T30*21 反対
はんだ #JCN 半田
はんだい #KK*40 阪大
はんだくてん #T35*2 ゜
はんだん #T30*78 判断
はんちゅう #T35*3 範疇
はんちょう #T35*1 班長
はんちょくせん #T35*2 半直線
はんつき #T35*1 半月
はんてい #T30*41 判定
はんてん #T30*69 反転 #T35 斑点
はんとう #T35*2 半島
はんとうまく #T35*1 半透膜
はんとうめい #T05 半透明
はんとき #T35 半刻
はんとし #T35*6 半年
はんど #T35*7 ハンド
はんどう #T35 反動
はんどうたい #T35*9 半導体
はんどく #T30*1 判読
はんどしぇーく #T30*1 ハンドシェーク
はんどばっぐ #T35*1 ハンドバッグ
はんどぼーる #T35*1 ハンドボール
はんどら #T35*4 ハンドラ
はんどりんぐ #T30*8 ハンドリング
はんどる #T30*2 ハンドル
はんな #JCN 阪奈
はんなん #CN 阪南
はんにじゅう #T05*2 半二重
はんにち #T35*4 半日 #T35 反日
はんにゃ #T35 般若
はんにゃとう #T35 般若湯
はんにゅう #T30*1 搬入
はんにん #T35*4 犯人
はんね #T35*1 半値
はんのう #T30*28 反応 #JCN 飯能
はんはん #T15*2 半々
はんば #T35 飯場
はんばーが #T35 ハンバーガ
はんばーぐ #T35 ハンバーグ
はんばい #T30*34 販売
はんばいき #T35*1 販売機
はんばく #T30*1 反駁
はんぱ #T05*6 半端
はんぱつ #T30*1 反発
はんぴれい #T30 反比例
はんぶん #T35*34 半分
はんぷ #T30*1 頒布
はんぷく #T30*2 反復
はんべつ #T30*10 判別
はんぺい #JNM 汎平
はんぼう #T05 繁忙
はんまー #T30*3 ハンマー
はんみ #T35 半身
はんみょう #T35 ハンミョウ 斑猫
はんめい #T30*15 判明
はんめん #T35*4 半面 反面
はんも #T30 繁茂
はんもく #T30 反目
はんもと #T35 版元
はんもん #T30*1 反問
はんよう #T05*19 汎用
はんようき #T35*8 汎用機
はんらく #T30 反落
はんらん #T30*1 反乱 氾濫
はんりょ #T35*1 伴侶
はんりょう #T35 半量
はんれい #T35*3 判例 #T35*2 凡例 #T35*1 反例
はんろん #T30*5 反論
ば #T35*122 場 #R5*11 張 #JS*5 羽 #KJ*2 婆 馬 #R5*1 貼 #KJ 罵 芭
ばー #T35*38 バー
ばーくれー #CN バークレー
ばーくれい #CN バークレイ
ばーじょん #T35*255 バージョン
ばーすと #T35*1 バースト
ばーちゃる #T05*3 バーチャル
ばーちゃるさーきっと #T35*8 バーチャルサーキット
ばーべきゅー #T35*11 バーベキュー
ばあい #T35*255 場合
ばい #JS*100 倍 #S5*57 倍 #T30*57 倍 #KJ*2 売 #KJ*1 培 買 #KJ 唄 貝 媒 梅 楳 煤 狽 賠 陪 吠 焙
ばいあすろん #T35*1 バイアスロン
ばいう #T35*1 梅雨
ばいお #T35*4 バイオ #PRE*3 バイオ
ばいおりん #T35 バイオリン
ばいか #T30 倍加 #T35 売価
ばいかい #T30*2 媒介
ばいがく #T35 倍額
ばいきゃく #T30*1 売却
ばいきん #T35 黴菌
ばいきんぐ #T30 バイキング
ばいく #T35*3 バイク
ばいけつしょう #T35*1 敗血症
ばいしゃく #T30 媒酌
ばいしゅう #T30*1 買収
ばいしゅん #T30*2 買春 #T30 売春
ばいしょう #T30 賠償
ばいしょく #T30 陪食
ばいしん #T35 陪審
ばいじん #T35 煤塵
ばいすう #T35*1 倍数
ばいせい #T35*1 媒精
ばいせいど #T05*1 倍精度
ばいせん #T30 焙煎
ばいぞう #T30*1 倍増
ばいたい #T35*116 媒体
ばいち #T35*1 培地
ばいてん #T35*3 売店
ばいと #JS*244 バイト #T35*148 バイト
ばいなり #T35*171 バイナリ
ばいばい #T30*2 売買
ばいぶれーた #T35*1 バイブレータ
ばいめい #T35 売名
ばいやく #T35 売約
ばいよう #T30 培養
ばいようえき #T35*1 培養液
ばいりつ #T35*2 倍率
ばいんでぃんぐ #T30*35 バインディング
ばいんど #T30*46 バインド
ばか #T05*43 馬鹿 #S5r 化か
ばかげ #KS ばかげ
ばかず #T35 場数
ばかばかし #KY*1 ばかばかし バカバカし
ばかばかしゅう #KYU*1 ばかばかしゅう バカバカしゅう
ばかんす #T35 バカンス
ばく #S5*10 縛 #SX*10 縛 #KJ*4 幕 #KJ*3 爆 #KJ*2 曝 #KJ*1 漠 #F11 漠 #KJ 莫 駁 麦 暴
ばくげき #T30 爆撃
ばくさい #T35 博才
ばくしょう #T10 爆笑
ばくしん #T30 驀進
ばくぜん #F00*2 漠然
ばくだい #T05*2 莫大
ばくだん #T35*3 爆弾
ばくち #T35 博打 博奕
ばくてりあ #T35*1 バクテリア
ばくは #T30*2 爆破
ばくはつ #T30*9 爆発
ばくふ #T35 幕府 瀑布
ばくまつ #T35*1 幕末
ばくやく #T35 爆薬
ばくろ #T30*1 暴露
ばぐ #T35*255 バグ
ばけ #KSr*26 化け
ばけつ #T35*4 バケツ
ばこ #SUC*8 箱
ばし #CNSUC1*20 橋 #SUC 橋
ばしょ #T35*240 場所
ばしょう #JNM 芭蕉
ばしら #SUC*1 柱
ばす #T35*47 バス
ばすえ #T35 場末
ばすく #CN バスク
ばすけっと #T35*2 バスケット
ばすけっとぼーる #T35*1 バスケットボール
ばすてい #T35*1 バス停
ばすれふ #T35*1 バスレフ
ばせい #T35 罵声
ばたー #T35*1 バター
ばたけ #T35 畑
ばち #T35*1 罰 #T35 鉢
ばちがい #T05 場違い
ばっ #S5*1 罰 #SX*1 罰
ばっかり #SUC*18 ばっかり
ばっきょ #T30*1 抜去
ばっきん #T35*1 罰金
ばっく #T30*57 バック
ばっくあっぷ #T30*22 バックアップ
ばっくとらっく #T30*2 バックトラック
ばっくみらー #T35*2 バックミラー
ばっさり #F06 ばっさり
ばっし #T30 抜歯
ばっじ #T35*1 バッジ
ばっすい #T30*6 抜粋
ばっそく #T35*1 罰則
ばったり #F06*1 ばったり
ばっち #T35*15 バッチ
ばっちり #F06 ばっちり
ばってき #T30 抜擢
ばっと #T35*3 バット
ばっは #JN バッハ
ばっふぁ #T35*255 バッファ
ばっぽん #T35*1 抜本
ばつ #T35*255 × #T35*9 罰 #KJ*1 伐 抜 #T35*1 閥 #KJ 筏 末
ばつぐん #T05*1 抜群
ばとう #T30*1 罵倒 #JCN 馬頭
ばどみんとん #T35*1 バドミントン
ばなし #T35*7 話
ばなれ #N2T30*5 離れ
ばにら #T35*1 バニラ
ばば #T35*2 婆 #T35*1 馬場 #JCN 馬場
ばぶる #T35*2 バブル
ばめん #T35*5 場面
ばや #KY*3 早 #KY 速
ばやし #KJ 林
ばら #S5r*8 ばら #T35 バラ 薔薇
ばらつ #K5r ばらつ
ばらつき #T35 バラツキ
ばらばら #T05*2 バラバラ #T05*1 ばらばら
ばらま #K5r*2 ばらま
ばらんす #T35*5 バランス
ばり #SUC*2 針 #T30 罵詈
ばりえーしょん #T35*4 バリエーション
ばりき #JS*2 馬力 #T35*1 馬力
ばりとん #T35 バリトン
ばりばり #F04 ばりばり
ばりゅー #T35*2 バリュー
ばるーん #T35 バルーン
ばるく #T35*2 バルク
ばると #CN バルト
ばるとーく #JN バルトーク
ばるぶ #T35*4 バルブ
ばれーぼーる #T35*14 バレーボール
ばれいしょ #T35 馬鈴薯
ばれえ #T35*1 バレエ
ばれるす #JN バレルス
ばれんたいん #T35*1 バレンタイン #JN バレンタイン
ばろーす #KK*1 バロース
ばろめーた #T35*1 バロメータ
ばん #JS*181 番 #T35*178 版 #T35*116 番 #T35*53 晩 #KJ*52 万 #T35*16 盤 #T35*12 板 #T35*2 判 #KJ*2 蛮 #KJ*1 阪 伴 磐 #KJ 播 #SUC 版 #KJ 挽 蕃
ばんか #T35 挽歌 晩夏
ばんかい #T30 挽回
ばんかん #T35 万感
ばんがい #T35*1 番外
ばんきん #T35 板金
ばんく #T35*2 バンク
ばんぐみ #T35*24 番組
ばんこく #T35 万国
ばんこん #T15 晩婚
ばんごう #T35*255 番号
ばんさん #T35 晩餐
ばんざい #T30*2 万歳
ばんしゃく #T30 晩酌
ばんしゅう #T35 晩秋
ばんしゅん #T35 晩春
ばんしょう #T35*1 万障
ばんじ #T35 万事
ばんじゅく #T05 晩熟
ばんじょう #T35 盤上
ばんせい #T30 晩成 #T35 蛮声
ばんせん #JS 番線 #T35 番線
ばんぜん #T05*1 万全
ばんそう #T30*8 伴奏
ばんだい #JS 番台 #T35 番台 #CN 磐梯
ばんち #JS*9 番地 #T35*4 番地
ばんちゃ #T35 番茶
ばんて #JS 番手
ばんとう #T35 番頭
ばんど #T35*13 バンド
ばんどう #JCN 坂東
ばんにん #T35 番人 万人
ばんねん #T35*1 晩年
ばんのう #T05*2 万能
ばんばん #F14*3 ばんばん
ばんぱく #T35*1 万博
ばんぶつ #T35 万物
ばんぺい #T35*1 番兵
ばんみん #T35*1 万民
ばんゆう #T35 万有
ぱ #JS*12 波 #JS 羽
ぱーざ #T35*1 パーザ
ぱーす #T30*1 パース
ぱーせんてーじ #T35 パーセンテージ
ぱーせんと #T35*3 ％ #JS*2 パーセント #T35*2 パーセント
ぱーそなりてぃ #T35*1 パーソナリティ
ぱーそなる #T05*11 パーソナル
ぱーてぃ #T30*10 パーティ
ぱーてぃー #T35*17 パーティー
ぱーてぃしょん #T35*18 パーティション
ぱーと #T35*11 パート
ぱーとたいまー #T35*1 パートタイマー
ぱーとなー #T35*1 パートナー
ぱーふぇくと #T05*1 パーフェクト
ぱーみっしょん #T35*11 パーミッション
ぱーみる #T35*1 ‰
ぱーる #T35*1 パール
ぱい #T35*10 π #T35*3 パイ Π #JS*2 敗 #JS 杯
ぱいおにあ #KK*2 パイオニア
ぱいぷ #T30*20 パイプ
ぱいろっと #T35*1 パイロット
ぱきすたん #CN パキスタン
ぱく #JS*2 拍 #JS 泊
ぱくぱく #F04 ぱくぱく
ぱけっと #T35*167 パケット
ぱこ #JS*1 箱
ぱじゃま #T35*1 パジャマ
ぱす #T30*72 パス
ぱすかる #JN パスカル
ぱすた #T35*1 パスタ
ぱすてる #T35*2 パステル
ぱすわーど #T35*55 パスワード
ぱずる #T35*3 パズル
ぱそこん #T35*26 パソコン
ぱたーん #T35*35 パターン
ぱたごにあ #CN パタゴニア
ぱちぱち #F06 ぱちぱち パチパチ
ぱちんこ #T30*3 パチンコ
ぱっかーど #KK*2 パッカード
ぱっく #T30*30 パック
ぱっけーじ #T30*47 パッケージ
ぱっしんぐ #T30*2 パッシング
ぱっち #T30*96 パッチ
ぱっちり #F04 ぱっちり
ぱっと #T30*1 パット #F12 ぱっと
ぱっど #T35*1 パッド
ぱつ #JS*7 発
ぱとかー #T35 パトカー
ぱとろーる #T35 パトロール
ぱなま #CN パナマ
ぱにっく #T35*9 パニック
ぱねりすと #T35*1 パネリスト
ぱねる #T35*39 パネル
ぱのらま #T35*1 パノラマ
ぱぱ #T35*4 パパ
ぱふぉーまんす #T30*20 パフォーマンス
ぱぶりっく #T05*4 パブリック
ぱぶりっくどめいん #T05*6 パブリックドメイン
ぱらぐらふ #T35*2 パラグラフ #T35*1 ¶
ぱらだいむ #T35*2 パラダイム
ぱらどくす #T35 パラドクス
ぱらどっくす #T35 パラドックス
ぱらぱら #F04 ぱらぱら
ぱらふぃん #T35*1 パラフィン
ぱらめーた #T35*47 パラメータ
ぱらめた #T35*232 パラメタ
ぱらめとろん #T35*1 パラメトロン
ぱららいざー #T35*2 パラライザー
ぱられる #T05*2 パラレル
ぱり #CN パリ 巴里
ぱりてぃ #T35*2 パリティ
ぱるす #T35*4 パルス
ぱるぷ #T35 パルプ
ぱれす #T35*2 パレス
ぱれすちな #CN パレスチナ
ぱれっと #T35*1 パレット
ぱろでぃ #T35*1 パロディ
ぱわー #T35*13 パワー
ぱわふる #T05*3 パワフル
ぱん #JS*13 版 #T35*7 パン
ぱんく #T30*3 パンク
ぱんげあ #CN パンゲア
ぱんだ #T35*1 パンダ
ぱんつ #T35*1 パンツ
ぱんてぃ #T35*1 パンティ
ぱんふれっと #T35*6 パンフレット
ぱんぷす #T35*1 パンプス
ひ #T35*255 日 #PRE*148 非 #K5*129 引 #KS*80 干 #T35*70 非 #T35*37 火 #T30*31 比 #K5r*13 弾 #KJ*9 費 #T30*6 秘 #K5*4 ひ #KJ*4 肥 #KJ*3 否 #T35*3 陽 #K5*2 曳 #S5*2 秘 #K5*1 惹 退 #T35*1 灯 妃 #KJ*1 皮 飛 #KJ 干 轡 #K5 索 #KJ 匪 卑 庇 彼 悲 扉 批 披 斐 泌 疲 #T35 碑 #KJ 秘 緋 罷 被 #PRE 被 #KJ 誹 #SUC 費 #KJ 避 樋 簸 毘 桧 氷
ひーた #T35*4 ヒータ
ひーぷ #T35*6 ヒープ
ひーろー #T35*1 ヒーロー
ひあい #T35*1 悲哀
ひいては #F14*2 ひいては
ひいで #KS*1 秀で
ひいひい #F06 ひいひい
ひいまご #T35 曾孫
ひいらぎ #T35 柊
ひいろ #T35 緋色
ひうら #JNS 火浦 樋浦
ひうん #T35 非運
ひえ #KSr*4 冷え #T35*1 稗
ひえこ #M5r 冷え込
ひえだ #JCN 稗田
ひえづ #JCN 日吉津
ひえぬき #JCN 稗貫
ひおき #JCN 日置
ひか #R5*6 光 #T35 皮下
ひかえ #KSr*21 控え
ひかえしつ #T35 控室
ひかく #T30*109 比較 #T35 皮革
ひかげ #T35*1 日陰
ひかた #JCN 干潟
ひかみ #JCN 氷上
ひからび #KS*1 干からび
ひかり #T35*38 光 #CN 光
ひかる #JNM ひかる 光
ひかわ #JCN 斐川 簸川
ひかん #T30*4 悲観
ひが #M5r*1 ひが #M5r 僻
ひがい #T35*4 被害
ひがいしゃ #T35*4 被害者
ひがえり #T30*2 日帰り
ひがし #T35*43 東 #CNPRE*1 東 #CN 東 #CNSUC1 東
ひがしあざい #CN 東浅井
ひがしあわくら #CN 東粟倉
ひがしいけぶくろ #CN 東池袋
ひがしいず #CN 東伊豆
ひがしいずも #CN 東出雲
ひがしいちき #CN 東市来
ひがしいばらき #CN 東茨城
ひがしいややま #CN 東祖谷山
ひがしいわい #CN 東磐井
ひがしうすき #CN 東臼杵
ひがしうら #CN 東浦
ひがしうわ #CN 東宇和
ひがしおおさか #CN 東大阪
ひがしおおじ #CN 東大路
ひがしおきたま #CN 東置賜
ひがしかぐら #CN 東神楽
ひがしかつしか #CN 東葛飾
ひがしかも #CN 東加茂
ひがしかわ #JCN 東川
ひがしかんばら #CN 東蒲原
ひがしがわ #T35 東側
ひがしくしら #CN 東串良
ひがしくにさき #CN 東国東
ひがしくびき #CN 東頚城
ひがしくるめ #CN 東久留米
ひがしぐち #T35*1 東口
ひがししらかわ #CN 東白川
ひがしすみよし #CN 東住吉
ひがしそのぎ #CN 東彼杵
ひがしたがわ #CN 東田川
ひがしだ #JNS 東田
ひがしちくま #CN 東筑摩
ひがしちちぶ #CN 東秩父
ひがしつがる #CN 東津軽
ひがしつの #CN 東津野
ひがしとなみ #CN 東砺波 東礪波
ひがしどうり #CN 東通
ひがしなだ #CN 東灘
ひがしなり #JCN 東成
ひがしなるせ #CN 東成瀬
ひがしね #JCN 東根
ひがしの #JCN 東野
ひがしひろしま #CN 東広島
ひがしまつうら #CN 東松浦
ひがしまつやま #CN 東松山
ひがしむらやま #CN 東村山
ひがしむろ #CN 東牟婁
ひがしもろかた #CN 東諸県
ひがしやつしろ #CN 東八代
ひがしやま #JCN 東山
ひがしやまと #CN 東大和
ひがしやまなし #CN 東山梨
ひがしゆり #CN 東由利
ひがしよか #CN 東与賀
ひがしよしの #CN 東吉野
ひがしよどがわ #CN 東淀川
ひがしりしり #CN 東利尻
ひがた #T35 干潟
ひがん #T35*1 彼岸 悲願
ひき #JS*1 匹 #KJ*1 疋 #T35 引き #KJ 曳 挽 #JCN 比企 #KJ 匹
ひきあ #W5r*2 引き合
ひきあい #T35*1 引合 #T35 引合い
ひきあげ #KSr*2 引き上げ #T30*2 引き上げ #KSr*1 引き揚げ
ひきあて #T30 引当
ひきあわせ #KSr 引き合わせ
ひきい #KS*1 率い
ひきうけ #KSr*4 引き受け #T30*3 引き受け #T30 引受 引受け
ひきおと #S5r*1 引き落と
ひきおろ #S5r 引き下ろ
ひきかえ #KSr*3 ひきかえ #KSr*1 引き替え #T35*1 引換 #S5r 引き返
ひきがえる #T35 蟇蛙
ひきがね #T35*2 引金
ひきがわ #JCN 日置川
ひきこ #M5r*1 引き込
ひきさ #K5*1 引きさ 引き裂
ひきさが #R5*2 引き下が
ひきさげ #KS*2 引き下げ #T30 引下げ
ひきざん #T30*1 引き算 引算
ひきしお #T35 引き潮
ひきしま #R5r 引き締ま
ひきしめ #KSr 引き締め
ひきすう #T35*255 引数
ひきず #R5r*2 ひきず #R5r 引きず
ひきた #T5 引き立 #JNS 疋田
ひきたて #T35*1 引立て
ひきだ #S5*9 引き出
ひきだし #T30*1 引出し
ひきち #JNS 引地
ひきつ #G5r*30 引き継
ひきつけ #KSr*1 引き付け
ひきつづき #F14*8 引続き
ひきでもの #T35 引出物
ひきと #R5r*3 引き取
ひきとめ #KSr*2 引き留め
ひきぬ #K5r 引き抜
ひきのば #S5r*1 引き延ば 引き伸ば
ひきはな #S5*1 引き離
ひきみ #JCN 匹見
ひきもど #S5r 引き戻
ひきゃく #T35 飛脚
ひきょう #T05*1 卑怯 #T35*1 秘境 #T30 比況
ひきよせ #KS*1 引き寄せ
ひきわけ #KSr*2 引き分け
ひきわた #S5r*1 引き渡
ひきわたし #T30*1 引渡し
ひぎ #T35*2 秘技
ひぎょう #T35 罷業
ひく #KY*48 低
ひくう #KYU*2 低う
ひくつ #T05 卑屈
ひくめ #KS 低め
ひぐち #JNS 樋口
ひぐま #T35*1 ヒグマ
ひけた #JCN 引田
ひけっていし #T35*1 非決定子
ひけつ #T30*1 否決 #T35*1 秘訣
ひけらか #S5r ひけらか
ひけん #T30 比肩
ひけんしゃ #T35*1 被験者
ひげ #T30*2 卑下 #T35*1 鬚 #T35 髭
ひげき #T35*3 悲劇
ひこ #KJ*4 彦
ひこう #T30*2 飛行 #T35 非行
ひこうき #T35*16 飛行機
ひこうじょう #T35*1 飛行場
ひこく #T35*1 被告
ひこたろう #JNM 彦太郎
ひこね #JCN 彦根
ひご #T30*1 庇護 #JCN 肥後
ひごい #T35 緋鯉
ひごかん #T15 非互換
ひごと #T35 日毎
ひごろ #T35*9 日頃
ひさ #T35*6 久 #G5 ひさ
ひさい #JCN 久居 #T30 被災
ひさお #JNM 久男 久夫 久雄 寿男 尚生 尚夫
ひさきち #JNM 久吉
ひさこ #JNM 久子
ひさし #KY*4 久し #T35*1 久 #JNM 久 寿 尚 尚志 #T35 庇
ひさしぶり #T15*44 久しぶり
ひさしま #JNS 久島
ひさしゅう #KYU*1 久しゅう
ひさのり #JNM 久典
ひさひろ #JNM 久寛
ひさびさ #T15*7 久々
ひさみち #JNM 久道
ひさやま #JCN 久山
ひさん #T05*8 悲惨 #T30 飛散
ひざ #T35*1 ひざ #T35 膝
ひざし #T35*2 日差し #T35*1 日射し
ひし #T35*4 菱 #G5 ひし
ひしかり #JCN 菱刈
ひしかわ #JNS 菱川
ひしがた #T35*2 ◇ ◆ #T35 菱形
ひしひし #F06*1 ひしひし
ひしむろ #JCN 西牟婁
ひしゃ #T35 飛車
ひしゃく #T35 柄杓
ひしゃたい #T35*1 被写体
ひしゅ #T35*1 脾腫
ひしょ #T35*2 秘書 #T30 避暑
ひしょう #T30*2 飛翔
ひじ #T35*2 肘 #T35 肱 #CN 日出
ひじかわ #JCN 肱川
ひじゅう #T35*1 比重
ひじゅつ #T35 秘術
ひじゅん #T30 批准
ひじょう #T05*136 非常 #T05*3 非情
ひじょうぐち #T35*1 非常口
ひじょうしき #T05*1 非常識
ひじり #T35*1 聖
ひすてり #T35 ヒステリ
ひすてりしす #T35 ヒステリシス
ひすてりっく #T05 ヒステリック
ひすとり #T35*63 ヒストリ
ひすとん #T35*1 ヒストン
ひず #M5r*2 ひず #M5r*1 歪
ひせん #T05 卑賎
ひぜに #T35 日銭
ひぜん #CN 肥前
ひそ #M5*3 潜 #T35 砒素 #M5r 顰
ひそう #T05*1 皮相 #T05 悲壮
ひそか #T05*2 密か #T05 秘か
ひそめ #KS 潜め 顰め
ひぞう #T30 秘蔵 #T35 脾臓
ひぞく #T05 卑俗
ひた #R5*2 浸 #S5*1 浸 #JCN 日田
ひたい #T35*1 額
ひたすら #F14*4 ひたすら
ひたち #CN 常陸 #JCN 日立
ひたちおおた #CN 常陸太田
ひたちせいさくしょ #KK*2 日立製作所
ひたん #T30 悲嘆
ひだ #JCN 飛騨 #JNS 飛田 #T35 襞
ひだい #T30*3 肥大
ひだか #JCN 日高
ひだち #T35 肥立ち
ひだまり #T35 日溜り
ひだり #T35*110 左
ひだりうえ #T35*6 左上
ひだりがわ #T35*8 左側
ひだりした #T35*4 左下
ひだりて #T35*1 左手
ひだりや #T35*3 ←
ひちそう #CN 七宗
ひちゅう #T35 秘中
ひっか #T35*2 筆禍 #K5*1 引っか
ひっかか #R5r*10 引っかか
ひっかけ #KSr*2 引っかけ
ひっき #T30*1 筆記
ひっくりかえ #S5r*2 ひっくり返 #R5r*1 ひっくり返
ひっくるめ #KS ひっくるめ 引っくるめ
ひっけい #T35*1 必携
ひっけん #T35*1 必見
ひっこ #S5r*3 引っ越 #M5r*2 引っ込
ひっこう #T35 筆耕
ひっこし #T30*26 引越し
ひっこみ #T35 引込み
ひっさつ #T35*1 必殺
ひっし #T05*5 必死 #T35*2 必至
ひっしゃ #T35*7 筆者 #T30 筆写
ひっしゅう #T35*1 必修
ひっしょう #T35*6 必勝
ひっす #T05*20 必須
ひっせき #T35*1 筆跡
ひっそく #T30 逼塞
ひっそり #F04 ひっそり
ひっち #T35 筆致
ひっちゃく #T30*5 必着
ひっちゅう #T15 必中
ひっちょう #T35*1 必聴
ひってき #T30*4 匹敵
ひっと #T30*2 ヒット
ひっとう #T35*4 筆頭
ひっとらー #JN ヒットラー
ひっぱ #R5r*4 引っ張
ひっぱく #T30 逼迫
ひっぱつ #T30*1 必発
ひつ #KJ*9 筆 #KJ*2 必 #KJ 泌 匹 疋 弼 畢 逼
ひつあつ #T35 筆圧
ひつう #T05 悲痛
ひつぎ #T35*1 棺
ひつじ #T35*3 羊 #T35 未
ひつじゅ #T35*4 必需
ひつじゅん #T35 筆順
ひつじょう #T05 必定
ひつぜつ #T35*1 筆舌
ひつぜん #T35*4 必然
ひつだん #T30*1 筆談
ひつどく #T35 必読
ひつよう #T05*255 必要
ひづけ #T35*40 日付
ひづめ #T35 蹄
ひてい #T30*22 否定
ひてつ #T35 非鉄
ひで #KJ*3 英 #JNM 秀
ひであき #JNM 英明 秀明
ひでお #JNM 英男 英夫 英雄 秀男 秀夫 秀雄 #JNS 日出男 日出夫
ひでか #JNM 秀加
ひでき #JNM 英樹 秀樹
ひできち #JNM 秀吉
ひでこ #JNM 秀子
ひでと #JNM 英人
ひでとし #JNM 英俊
ひでのり #JNM 英紀
ひではる #JNM 英晴
ひでひこ #JNM 英彦
ひでみ #JNM 英実 秀実
ひでみつ #JNM 英光 秀光
ひでゆき #JNM 英之 秀幸 秀之
ひでよ #JNM 英世
ひでよし #JNM 秀吉
ひでり #T35 日照り
ひでん #T35*1 秘伝
ひと #T35*255 人 #NN*60 一 #T35*2 ヒト 他人 #KJ*1 一 #PRE ひと
ひとあき #JNM 仁明
ひとあし #T35 一足
ひといちばい #T35 人一倍
ひとう #T35 秘湯
ひとえ #T35*4 単 #T35*1 一重
ひとかげ #T35 人影
ひとかたならず #F14 ひとかたならず 一方ならず
ひとかたならぬ #F14 ひとかたならぬ 一方ならぬ
ひとがら #T35*1 人柄
ひとぎき #T35 人聞き
ひとく #T30 秘匿
ひとくち #T35*2 一口
ひとこと #T35*19 一言
ひところ #T35*1 ひところ
ひとごと #T35*2 人事 #T35 他人事
ひとごみ #T35*1 人混み #T35 人込み
ひとさしゆび #T35 人差指
ひとさらい #T30 人拐い
ひとさわがせ #T05 人騒がせ
ひとざと #T35 人里
ひとし #KY*32 等し #KY*1 ひとし 均し #JNM 均 斎 仁 仁志 等
ひとしなみ #T15 等し並み
ひとしゅう #KYU*2 等しゅう #KYU*1 ひとしゅう 均しゅう
ひとじち #T35*1 人質
ひとすじ #T05*1 一筋 #T05 ひとすじ
ひとすじなわ #T35 一筋縄
ひとたち #T35*10 人達
ひとたび #F14*2 ひとたび
ひとちがい #T30 人違い
ひとっこ #T35*1 ひとっこ
ひとつ #T35*87 一つ #T35*14 ひとつ #F14*6 ひとつ
ひとつばし #JCN 一ツ橋 一橋
ひとづま #T35*2 人妻
ひとで #T35*2 人手 人出
ひととおり #T10*3 一通り
ひととき #T35*1 一時 #T35 ひととき
ひとなみ #T05*1 人並
ひとはだ #T35 一肌 人肌
ひとばん #T35*2 一晩
ひとびと #T35*21 人々 #T35*2 人びと
ひとまえ #T35*1 人前
ひとまく #T35*1 一幕
ひとまず #F14*2 ひとまず
ひとまわり #T30*2 ひとまわり #T30*1 一回り
ひとみ #T35*4 瞳 #T35*1 ひとみ #JNM 一美 #JN 人見 #JNM 仁美
ひとよし #JCN 人吉
ひとらー #JN ヒトラー
ひとり #T35*112 一人 #T35*3 ひとり 独り
ひとりじめ #T30*2 一人占め #T30*1 ひとりじめ 独り占め
ひとりっこ #T35*1 一人っ子
ひとりぼっち #T15 一人ぼっち
ひど #KY*25 ひど #KY 酷
ひどう #KYU*2 ひどう #KYU 酷う
ひどうき #T05*6 非同期
ひどけい #T35 日時計
ひどり #T35*2 日取り
ひな #T35*8 雛
ひない #JCN 比内
ひなせ #JCN 日生
ひなた #T35*1 日向
ひなん #T30*8 避難 #T30*3 非難
ひにく #T05*3 皮肉
ひにち #T35*7 日日
ひにゅう #T35*1 泌乳
ひにん #T30*6 避妊 #T30 否認
ひね #R5r*1 捻 #KS ひね #R5r ひね
ひの #JCN 日野
ひのえ #T35 丙
ひのえまた #JCN 桧枝岐
ひのかげ #JCN 日之影
ひのき #T35 桧
ひのこ #T35 火の粉
ひので #T35*1 日の出 #CN 日の出
ひのはら #JNS 桧原
ひはん #T30*3 批判
ひば #CN 比婆
ひばく #T30*1 被曝
ひばしら #T35 火柱
ひばち #T35*1 火鉢
ひばな #T35*1 火花
ひばら #JCN 桧原
ひばり #T35 雲雀
ひばん #T35 非番
ひひょう #T30*2 批評
ひび #T35*13 日々 #K5r*12 響 #K5r*1 ひび
ひびか #S5 響か
ひびや #JCN 日比谷
ひびわれ #KSr ひび割れ
ひふ #T05*2 非負 #T35 皮膚
ひふく #T30*3 被覆 #T35*1 被服
ひふみ #JNM 一二三
ひふん #T30 悲憤
ひぶ #T35 日歩
ひほう #T35 悲報 秘宝 秘法
ひぼん #T05 非凡
ひま #T05*88 暇 #KJ 閑 隙
ひまく #T35*1 皮膜
ひましゆ #T35 ひまし油
ひまつ #T35 飛抹
ひまらや #CN ヒマラヤ
ひまわり #T35*1 向日葵
ひまん #T30*1 肥満
ひみ #JCN 氷見
ひみつ #T05*24 秘密
ひむろ #JCN 氷室
ひめ #T35*4 姫 #KS*1 秘め #KJ 媛
ひめい #T35*1 悲鳴
ひめかわ #JCN 姫川
ひめき #JCN 姫木
ひめきだいら #JCN 姫木平
ひめしま #JCN 姫島
ひめじ #JCN 姫路
ひめど #JCN 姫戸
ひめん #T30 罷免
ひも #T35*1 紐 #KJ 綬
ひもう #T35*1 被毛
ひもく #T35*1 費目
ひもと #T35 火元
ひもの #T35*1 干物
ひゃく #NN*11 百
ひゃくしょう #T35*1 百姓
ひゃくぶん #T35*1 百聞
ひゃくぶんりつ #T35*2 百分率
ひゃくまんべん #CN 百万遍
ひゃくめんそう #T30 百面相
ひゃっか #T35*1 百科
ひゃっかてん #T35 百貨店
ひゃっかりょうらん #T10 百花繚乱
ひや #S5r*2 冷や #T35 冷や
ひやあせ #T35 冷汗
ひやか #S5r*2 冷やか
ひやく #T30*2 飛躍
ひやま #JNS 桧山
ひやむぎ #T35 冷麦
ひややか #T05 冷やか
ひややっこ #T35*1 冷奴
ひやりと #F12 ひやりと
ひゅーすとん #CN ヒューストン
ひゅーまん #T35*3 ヒューマン
ひゅーりすてぃっく #T05*1 ヒューリスティック
ひゅーれっと #KK*3 ヒューレット
ひゅうが #JCN 日向
ひゆ #T35*1 比喩
ひゆうり #T05*1 非有理
ひょう #S5*255 表 #T30*255 表 #T35*74 票 #S5*6 評 #T30*6 評 #KJ*4 兵 #T35*1 標 #KJ 拍 #JS 俵 #KJ 俵 彪 #T35 氷 #KJ 漂 瓢 #JS 票 #SUC 票 表 #T35 豹 #KJ 平 #T35 雹
ひょうい #T35*1 表意 #T30 憑依
ひょうか #T30*185 評価
ひょうかん #T05 慓悍
ひょうが #T35*4 氷河
ひょうき #T30*18 表記 #T30*1 標記
ひょうきん #T05 ひょうきん 慓軽
ひょうぎ #T30 評議
ひょうけい #T35*1 表敬
ひょうけつ #T30 票決 表決
ひょうげん #T30*201 表現 #T35 評言
ひょうこう #T35*1 標高
ひょうご #T35*2 標語 #JCN 兵庫
ひょうさつ #T35 表札
ひょうざん #T35*1 氷山
ひょうし #T35*19 表紙 #T35*1 拍子
ひょうしき #T30*1 標識
ひょうしゅつ #T30*2 表出
ひょうしょう #T30*2 表彰 #T30*1 表象
ひょうじ #T30*255 表示
ひょうじゅん #T35*210 標準
ひょうじゅんか #T35 標準化
ひょうじょう #T35*4 表情 #T30 評定
ひょうせつ #T30 剽窃
ひょうそう #T35*2 表層
ひょうそく #T35 平仄
ひょうたん #T35 瓢箪
ひょうだい #T35*2 表題
ひょうちゃく #T30 漂着
ひょうてい #T30 評定
ひょうてき #T35*1 標的
ひょうてん #T35*1 氷点 #T35 評点
ひょうでん #T35 票田 評伝
ひょうど #T35 表土
ひょうどう #JNS 兵藤
ひょうはく #T30 漂泊 漂白
ひょうばん #T35*8 評判
ひょうへん #T30*1 豹変
ひょうほん #T35*2 標本
ひょうぼう #T30 標榜
ひょうめい #T30*76 表明
ひょうめん #T35*2 表面
ひょうり #T35*1 表裏
ひょうりゅう #T30 漂流
ひょうりょう #T35*1 秤量
ひょうろう #T35 兵糧
ひょうろん #T30*5 評論
ひょっこり #F06 ひょっこり
ひょっと #F12*14 ひょっと
ひょっとしたら #F14*1 ひょっとしたら
ひよう #T35*31 費用
ひよく #T05 肥沃
ひよし #JNS 日義 #JCN 日吉
ひよどり #T35 ヒヨドリ 鵯
ひより #T35*2 日和
ひよわ #T05 ひよわ ひ弱
ひら #K5r*58 開 #KJ*33 平 #PRE*16 平 #K5r*1 ひら #KJ 片
ひらあやまり #T30*1 平謝り
ひらい #JCN 平井
ひらいしん #T35*1 避雷針
ひらいずみ #JCN 平泉
ひらお #JCN 平生 平尾
ひらおか #JNS 平岡
ひらか #JCN 平賀 平鹿
ひらかた #JCN 枚方
ひらかわ #JCN 平河 平川
ひらが #JNS 平賀
ひらがな #T35*26 平仮名 #T35*18 ひらがな
ひらき #JNS 平木
ひらきなお #R5r*2 開きなお #R5r 開き直
ひらくら #JNS 平倉
ひらさわ #JCN 平沢
ひらぞこ #T15 平底
ひらた #KY*1 平た #JCN 平田
ひらつか #JCN 平塚
ひらとり #JCN 平取
ひらど #JCN 平戸
ひらない #JCN 平内
ひらの #JCN 平野
ひらはら #JNS 平原
ひらばやし #JNS 平林
ひらばる #JNS 平原
ひらひら #F04 ひらひら
ひらまつ #JNS 平松
ひらめ #K5r*2 ひらめ 閃 #T35*2 鮃
ひらや #JCN 平谷
ひらやま #JCN 平山
ひらら #JCN 平良
ひりき #T05 非力
ひりつ #T35*3 比率
ひりひり #F04 ひりひり
ひりょう #T35 肥料
ひりょく #T05 非力 #T35 膂力
ひる #T35*56 昼 #KJ*9 蒜 #M5r ひる 怯 #T35 蛭
ひるい #T30 比類
ひるかわ #JNS 比留川 #JCN 蛭川
ひるがえ #R5r 翻 #S5r 翻
ひるね #T30*3 昼寝
ひるべると #JN ヒルベルト
ひるま #T35*9 昼間
ひるめし #T35*4 昼飯
ひるやすみ #T35*5 昼休み
ひれ #T35 鰭
ひれい #T30*4 比例
ひれき #T30 披瀝
ひれつ #T05 卑劣
ひれふ #S5*1 ひれ伏
ひれん #T35*2 悲恋
ひろ #KY*87 広 #W5*32 拾 #KJ*19 広 #KJ*4 弘 浩 #KJ*3 博 #KJ*2 宏 紘 #KJ*1 洪 尋 祐 #KJ 寛 #JNM 比呂 #KJ 宥 #JNM 裕
ひろあき #JNM 宏明 広昭 広明 弘明 浩秋 博昭 博明 祐昭 洋晶
ひろう #T30*17 披露 #T30*3 疲労 #KYU*2 広う
ひろうえん #T35*18 披露宴
ひろえ #JNM 宏江
ひろお #JCN 広尾
ひろかず #JNM 弘一 博一
ひろかみ #JCN 広神
ひろかわ #JCN 広川
ひろが #R5r*1 広が
ひろがわ #JCN 広川
ひろき #JNM 宏樹
ひろきち #JNM 広吉
ひろげ #KS*16 広げ #KS*1 ひろげ
ひろこ #JNM 欧子 寛子 弘子 浩子 宙子 博子 裕子
ひろさき #JCN 弘前
ひろし #JNM 寛 啓之 宏 広 広司 広史 広志 弘 浩 浩志 博 博司 博史 裕 裕史 洋
ひろしげ #JNM 広重 浩重
ひろしま #JCN 広島
ひろじ #JNM 広治
ひろせ #JCN 広瀬
ひろた #JNS 広田
ひろつぐ #JNM 博嗣
ひろと #JNM 裕人
ひろとし #JNM 宏寿 浩俊
ひろなか #JNS 広中 #JN 弘仲
ひろの #JCN 広野
ひろのぶ #JNM 博信 裕信
ひろのみや #JNS 浩宮
ひろば #T35*7 広場
ひろひさ #JNM 浩寿 博久 裕久
ひろひと #JN 裕仁
ひろびろ #F00 広々
ひろふみ #JNM 啓文 博文 洋文
ひろま #R5r*4 広ま #T35 広間
ひろみ #JNM ひろみ ひろ美 宏美 #JCN 広見 #JNM 弘美 博美 比呂美 裕美
ひろみち #JNM 博道
ひろみつ #JNM 宏光 博満
ひろめ #KS*3 広め
ひろゆき #JNM 寛之 啓之 宏行 宏之 広幸 広行 弘恭 弘之 浩幸 浩之 博之 裕之
ひろよし #JNM 弘恭
ひわ #JCN 比和 #T35 秘話
ひわき #JCN 樋脇
ひわさ #JCN 日和佐
ひわたり #JNS 日渡
ひわり #T35 日割
ひわれ #T30 干割れ
ひん #JS*21 品 #T35*10 品 #S5*1 瀕 #T30*1 瀕 #S5*1 貧 #T30*1 貧 #SUC 品 #KJ 彬 斌 浜 賓 頻 牝
ひんい #T35*1 品位
ひんかく #T35 品格
ひんきゅう #T35*1 殯宮 #T30 貧窮
ひんけつ #T35 貧血
ひんこう #T35 品行
ひんこん #T05*1 貧困
ひんし #T35*218 品詞 #T35 瀕死
ひんしつ #T35*63 品質
ひんしゅ #T35*1 品種
ひんしゅく #T10*8 顰蹙
ひんじゃ #T35 貧者
ひんじゃく #T05*2 貧弱
ひんずー #KK*1 ヒンズー
ひんせい #T35 品性
ひんそう #T05 貧相
ひんと #T35*30 ヒント
ひんど #T35*255 頻度
ひんば #T35 牝馬
ひんぱつ #T30*1 頻発
ひんぱん #T05*19 頻繁
ひんぷ #T35 貧富
ひんみん #T35*2 貧民
ひんめい #T35*5 品名
ひんもく #JS*1 品目 #T35*1 品目
ひんやり #F04*1 ひんやり
び #T35*24 美 #KJ*4 尾 微 枇 #PRE*2 微 #KJ*2 媚 #KJ*1 備 毘 #SUC 火 日 #T35 日 #KJ 琵 眉 鼻
びーか #T35*1 ビーカ
びーかー #T35*1 ビーカー
びーとるず #KK*1 ビートルズ
びーなす #T35*1 ビーナス
びーなすらいん #CN ビーナスライン
びーぷ #T30*2 ビープ
びーる #T35*21 ビール
びえい #CN 美瑛
びえん #T35 鼻炎
びか #T30 美化
びかん #T35*2 美観
びがく #T35 美学
びき #JS*1 匹
びきょう #T35*1 鼻鏡
びく #T35 びく
びくう #T35*1 鼻腔
びこう #T35*39 備考 #T30 尾行
びさい #T05*1 微細 #JCN 尾西
びし #SUC*1 菱
びしゅ #T35*1 美酒
びしょう #T05*1 微小 #T30 微笑
びしょく #T30 美食
びしょびしょ #T18 びしょびしょ
びじー #T35*1 ビジー
びじねす #T35*14 ビジネス
びじゃく #T05 微弱
びじゅありぜーしょん #T30*2 ビジュアリゼーション
びじゅある #T05*3 ビジュアル
びじゅう #T35*1 鼻汁
びじゅつ #T35*6 美術
びじゅつかん #T35*7 美術館
びじょ #T35*1 美女
びじょん #T35*2 ビジョン
びじん #T35*3 美人
びすます #T35*1 ビスマス
びせい #JCN 美星
びせきぶん #T35*2 微積分
びぜん #CN 備前
びそく #T35*1 尾側 #T15 微速
びだん #T35 美談
びっくり #T30*56 びっくり
びっぐ #T05 ビッグ
びっしょり #F06 びっしょり
びっしり #F06 びっしり
びっちゅう #CN 備中
びっと #T35*156 ビット
びっとまっぷ #T35*52 ビットマップ
びてき #T05*1 美的
びでお #T35*41 ビデオ
びとく #T35 美徳
びなん #T35*1 美男
びにーる #T35*6 ビニール
びばい #CN 美唄
びひん #T35*11 備品
びび #F11*1 微々
びびし #KY 美々し
びびしゅう #KYU 美々しゅう
びふか #CN 美深
びふてき #T35 ビフテキ
びぶ #T35*1 尾部
びぶん #T30*2 微分
びほろ #CN 美幌
びぼう #T35 美貌
びぼうろく #T35 備忘録
びまん #T30 瀰満
びみょう #T05*13 微妙
びゃく #NN*1 百
びゃくや #T35 白夜
びゃっこ #KK 白虎
びやく #T35*1 媚薬
びゅー #T35*16 ビュー
びゅう #KJ 謬
びょう #T35*39 秒 #KJ*2 病 #JS*1 ″ #T35*1 ″ #KJ 猫 #JS 票 #KJ 廟 描 #SUC 病 #JS 秒 #KJ 苗 錨 #T35 鋲 #KJ 平
びょういん #T35*7 病院
びょうが #T30*48 描画
びょうき #T30*5 病気
びょうけつ #T30 病欠
びょうげん #T35*1 病原
びょうし #JS*4 拍子 #T30*1 病死
びょうしつ #T35 病室
びょうしゃ #T30*1 描写
びょうしょう #T35*1 病床
びょうしん #T35 病身 秒針
びょうじゃく #T05 病弱
びょうじょう #T35 病状
びょうそく #T35 秒速
びょうたい #T35*1 病態
びょうてき #T05*1 病的
びょうとう #T35 病棟
びょうどう #T05*2 平等
びょうにん #T35*2 病人
びょうぼつ #T30 病没
びょうま #T35 病魔
びょうめい #T35 病名
びょうりがく #T35*1 病理学
びょうれき #T35 病歴
びよう #T35*4 美容
びよく #T35*1 尾翼
びら #T35*1 ビラ
びらん #T30*1 糜爛
びりびり #F04 びりびり
びりやーど #T35*2 ビリヤード
びりゅう #T35*1 微粒
びりょう #T05 微量
びりょく #T05*1 微力
びる #JNSUC*84 ビル #JN ビル #T35 ビル
びろう #T05 尾篭
びわ #T35 琵琶
びわこ #CN 琵琶湖
びん #SUC*82 便 #T35*39 瓶 #KJ*1 敏 #T35*1 便 #KJ 貧 #T35 壜
びんかん #T05*3 敏感
びんご #T30 ビンゴ #CN 備後
びんしょう #T05*1 敏捷 #T30 憫笑
びんじょう #T30*4 便乗
びんせん #T35*1 便箋 #T35 処方箋
びんそく #T05 敏速
びんぞこ #T35 瓶底
びんぼう #T05*4 貧乏
びんらん #T35*1 便覧
びんわん #T05 敏腕
ぴーく #T35*1 ピーク
ぴーす #JS*6 ピース #T35*6 ピース
ぴあ #KK*4 ぴあ
ぴあにすと #T35*3 ピアニスト
ぴあの #T35*1 ピアノ
ぴかそ #JN ピカソ
ぴき #JS*2 匹
ぴくさー #KK*2 ピクサー
ぴくせる #JS*10 ピクセル #T35*10 ピクセル
ぴくぴく #F04 ぴくぴく
ぴっくあっぷ #T30 ピックアップ
ぴったり #F04*5 ぴったり #T05*2 ぴったり
ぴっち #T35*1 ピッチ
ぴっぷ #CN 比布
ぴぺっと #T35*1 ピペット
ぴゅあ #T05*1 ピュア
ぴょう #JS*6 票
ぴらにあ #T35 ピラニア
ぴらふ #T35*1 ピラフ
ぴらみっど #T35*1 ピラミッド
ぴりおど #T35*3 ピリオド
ぴる #T35 ピル
ぴるびんさん #T35*1 ピルビン酸
ぴん #JS*13 ピン #JS*1 品
ぴんく #T35*6 ピンク
ぴんせっと #T35*1 ピンセット
ぴんち #T15*1 ピンチ
ぴんと #T35*1 ピント
ふ #R5*255 降 #PRE*139 不 #M5*73 踏 #T35*72 負 #K5*58 吹 #R5r*27 振 #KJ*16 不 #S5*16 付 #T30*16 付 #T35*13 府 #K5*8 葺 #K5*6 拭 #T35*4 符 #KJ*2 夫 敷 普 訃 #KJ*1 富 冨 扶 浮 父 腐 芙 #T35*1 譜 #S5*1 賦 #T30*1 賦 #S5*1 伏 #K5*1 噴 #KJ*1 麸 #T35*1 麸 #S5 臥 #M5 践 #KJ 埠 婦 #SUC 婦 #KJ 布 怖 斧 膚 賦 赴 阜 #S5 附 #T30 附 #KJ 風 鮒 舗 鋪 歩 腑
ふーど #T35*1 フード
ふーりえ #JN フーリエ
ふーん #CJ*3 ふーん
ふぁーすと #T35*1 ファースト
ふぁーむ #KK ファーム
ふぁーむうぇあ #T35*4 ファームウェア
ふぁい #T35*23 Φ #T35*3 φ
ふぁいば #T35*2 ファイバ
ふぁいる #T30*255 ファイル
ふぁいんだ #T35*1 ファインダ
ふぁくしみり #T35*2 ファクシミリ
ふぁくと #T35*2 ファクト
ふぁくとり #T35*1 ファクトリ
ふぁしりてぃ #T35*1 ファシリティ
ふぁしりてい #T35*2 ファシリテイ
ふぁじー #T05 ファジー
ふぁじい #T05 ファジイ
ふぁっくす #T30*7 ファックス
ふぁっしょ #T35 ファッショ
ふぁっしょん #T35*1 ファッション
ふぁなてぃっく #T05*4 ファナティック
ふぁみこん #T35*1 ファミコン
ふぁみり #T35*4 ファミリ
ふぁん #T35*30 ファン
ふぁんくしょん #T35*37 ファンクション
ふぁんくた #T35*1 ファンクタ
ふぁんたじー #T35*3 ファンタジー
ふぁんたすてぃっく #T05 ファンタスティック
ふぁんでーしょん #T35 ファンデーション
ふあん #T05*24 不安
ふあんてい #T05*5 不安定
ふぃーだ #T35*1 フィーダ
ふぃーでぃんぐ #T30*1 フィーディング
ふぃーと #JS*1 フィート #T35*1 フィート
ふぃーどばっく #T30*2 フィードバック
ふぃーどふぉわーど #T30*1 フィードフォワード
ふぃーるど #T35*53 フィールド
ふぃくしょん #T35*1 フィクション
ふぃじかる #T05*2 フィジカル
ふぃっくす #T30*19 フィックス
ふぃっと #T30*1 フィット
ふぃらめんと #T35*1 フィラメント
ふぃりぴん #CN フィリピン
ふぃる #T30*11 フィル
ふぃるた #T35*27 フィルタ
ふぃるたー #T35*9 フィルター
ふぃるむ #T35*1 フィルム
ふい #T05*2 不意
ふいちょう #T30 吹聴
ふいっち #T35*2 不一致
ふう #CNSUC1*46 風 #T35*37 封 #T05*12 風 #T05*8 ふう #KJ 夫 #ZX 封 #KJ 楓 #SUC 風
ふういん #T30*1 封印
ふうう #T35*1 風雨
ふううん #T35 風雲
ふうか #T30 風化
ふうかく #T35 風格
ふうかん #T35*1 風乾
ふうが #T05*1 風雅
ふうがわり #T05*1 風変わり
ふうき #R5r*1 封切 #T15 富貴 #T35 風紀
ふうきょう #T35 風狂
ふうきり #T30 封切り
ふうぎ #R5r 封切
ふうけい #T35*5 風景
ふうこう #T35*1 風光 #T35 風向
ふうさ #T30 封鎖
ふうさい #T35 風采
ふうし #T30*1 風刺 #T30 諷刺
ふうしゃ #T35*1 風車
ふうしゅう #T35*2 風習
ふうしょ #T35 封書
ふうしん #T35*1 風疹
ふうじこ #M5*1 封じ込
ふうじこめ #KSr 封じ込め
ふうせつ #T35 風雪
ふうせん #T35 風船
ふうそく #T35 風速
ふうぞく #T35*1 風俗
ふうちょう #T35*2 風潮
ふうてい #T35 風体
ふうとう #T35*2 封筒
ふうど #T35*1 風土
ふうどう #T35 風洞
ふうにゅう #T30*1 封入
ふうび #T30 風靡
ふうふ #T35*7 夫婦
ふうぶつ #T35 風物
ふうぶつし #T35 風物詩
ふうぼう #T35 風貌 風防
ふうみ #T35*1 風味
ふうりゅう #T05 風流
ふうりょく #T35*1 風力
ふうりん #T35*1 風鈴
ふうれん #CN 風連
ふうん #T05 不運
ふぇーず #T35*2 フェーズ
ふぇあ #T05*9 フェア
ふぇいす #T35*3 フェイス
ふぇいず #T35*1 フェイズ
ふぇいる #T30*6 フェイル
ふぇざー #T35*2 フェザー
ふぇすてぃばる #T35*2 フェスティバル
ふぇっち #T30*1 フェッチ
ふぇみにずむ #T35*2 フェミニズム
ふぇらちお #T30*1 フェラチオ
ふぇりー #T35*2 フェリー
ふぇんす #T35*1 フェンス
ふえ #KSr*94 増え #KSr*1 ふえ #KSr 殖え #T35 笛
ふえい #T35 府営
ふえて #T15*1 不得手
ふえん #T30 敷衍
ふぉーかす #T35*4 フォーカス
ふぉーく #T35*6 フォーク
ふぉーじ #T35*1 フォージ
ふぉーとらん #T35*1 フォートラン
ふぉーど #KK*2 フォード
ふぉーまってぃんぐ #T30*1 フォーマッティング
ふぉーまっと #T30*93 フォーマット
ふぉーまる #T05*2 フォーマル
ふぉーむ #T35*10 フォーム
ふぉーらむ #T35*4 フォーラム
ふぉーると #T35*6 フォールト
ふぉーれ #JN フォーレ
ふぉあ #PRE*2 フォア #T35*2 フォア
ふぉるだ #T35*12 フォルダ
ふぉるて #T35 フォルテ
ふぉると #T30*2 フォルト
ふぉろー #T30*39 フォロー
ふぉわーど #T30*61 フォワード
ふぉん #KK*40 フォン
ふぉんと #T35*198 フォント
ふおん #T05*2 不穏
ふか #T35*123 負荷 #T35*77 不可 #T30*65 付加 #KYmi*61 深 #S5*1 更か #T35*1 鯊 鱶 #S5r 蒸か #KJ 深 #T35 府下
ふかい #T05*3 不快 #JNS 深井
ふかうら #JCN 深浦
ふかえ #JCN 深江
ふかお #JNS 深尾
ふかかい #T05*1 不可解
ふかがわ #JCN 深川
ふかぎゃく #T05*1 不可逆
ふかく #T05*5 不覚
ふかけつ #T05*9 不可欠
ふかさわ #JCN 深沢
ふかざわ #JCN 深沢
ふかしぎ #T15 不可思議
ふかしん #T15*1 不可侵
ふかせ #JNS 深瀬
ふかだ #JCN 深田
ふかち #T15 不可知
ふかつ #T35 賦活
ふかつか #T30*1 不活化
ふかの #JNS 深野
ふかのう #T05*55 不可能
ふかひ #T05*1 不可避
ふかぶかと #F14 深々と
ふかぶん #T05*2 不可分
ふかま #R5r*1 深ま
ふかみ #JNS 深海
ふかめ #KS*3 深め
ふかや #JCN 深谷
ふかやす #JCN 深安
ふかわ #JCN 布川
ふかん #T30 俯瞰
ふかんしょう #T35*1 不感症 #T35 不干渉
ふかんぜん #T05*5 不完全
ふがいな #KY*1 ふがいな
ふがいのう #KYU*1 ふがいのう
ふき #T30*2 付記 #T30*1 附記 #KJ 葺 #T35 蕗
ふきあげ #JCN 吹上
ふきあれ #KS*2 吹き荒れ
ふきかえ #S5r 吹き返 #T30 吹替え
ふきかけ #KSr 吹きかけ
ふきげん #T05*2 不機嫌
ふきこ #M5r*1 吹き込
ふきそ #T35 不起訴
ふきそく #T05*1 不規則
ふきだ #S5r 吹き出
ふきだまり #T35 吹き溜り
ふきつ #T05*1 不吉
ふきつけ #KSr 吹き付け
ふきでもの #T35 吹出物
ふきと #R5r 拭き取
ふきとば #S5*2 吹き飛ば
ふきなが #S5r 吹き流
ふきぬけ #T35 吹抜け
ふきゅう #T30*26 普及 #T35*1 不朽 #T35 不休
ふきょう #T05*3 不況 #T30*3 布教
ふきん #T35*10 付近 #T35*1 布巾
ふきんこう #T05*1 不均衡
ふきんしん #T05*1 不謹慎
ふぎ #T30*1 付議 #T35 不義
ふく #M5r*228 含 #T30*13 服 #KJ*11 副 #T35*11 福 #S5*3 服 #S5*2 復 #T30*2 復 #KJ*2 腹 #T30*1 伏 #PRE*1 複 #KJ*1 輻 #PRE 副 #KJ 幅 複 覆
ふくい #JCN 福井
ふくいん #T35*1 福音
ふくえ #JNS 福栄 #JCN 福江
ふくえき #T30 服役
ふくえん #T30 復縁
ふくおか #JCN 福岡
ふくかん #T35 副官
ふくがく #T30 復学
ふくぎ #T35*1 副木
ふくぎょう #T35 副業
ふくくう #T35*1 腹腔
ふくげん #T30*7 復元
ふくごう #T30*18 復号 #T30*17 複合
ふくさい #T35 副菜
ふくさよう #T35*14 副作用
ふくさんぶつ #T35*2 副産物
ふくざき #CN 福崎
ふくざつ #T05*58 複雑
ふくざわ #JNS 福沢
ふくし #T35*20 副詞 #T35*2 福祉
ふくしき #T35 腹式 複式
ふくしま #JCN 福島 #JNS 福嶋
ふくしまけん #CN 福島県
ふくしゃ #T30*22 複写
ふくしゅう #T30*17 復讐 #T30*3 復習
ふくしょう #T35*1 副賞 #T35 副将 #T30 復唱
ふくしょく #T35 副食 #T30 復職 #T35 服飾
ふくしん #T35 副審 腹心
ふくじ #T35*1 副次 #T35 服地 #JNM 福治
ふくじゅう #T30 服従
ふくすい #T35*2 覆水
ふくすう #T15*203 複数
ふくすうこ #T35*3 複数個
ふくせい #T30*5 複製
ふくせん #T35*2 伏線 #T35 複線
ふくそう #T35*3 服装 #T30*2 輻輳
ふくそく #T35*1 腹側
ふくぞく #T30 服属
ふくたろう #JNM 福太郎
ふくだ #JCN 福田
ふくだい #T35*4 副題
ふくち #JCN 福地
ふくちやま #CN 福知山
ふくちょう #T30*2 復調
ふくつ #T05 不屈
ふくつう #T35 腹痛
ふくで #JCN 福田
ふくとみ #JNS 福富
ふくとめ #JNS 福留
ふくどく #T35*1 副読 #T30 服毒
ふくどみ #JCN 福富
ふくなが #JNS 福永
ふくの #JCN 福野
ふくはら #JCN 福原
ふくびき #T35 福引
ふくぶ #T35*1 腹部
ふくへい #T35 伏兵
ふくべ #JCN 福部
ふくほん #T35 副本 複本
ふくま #JCN 福間
ふくまく #T35 腹膜
ふくまつ #JNS 福松
ふくみつ #JCN 福光
ふくむ #T30*8 服務
ふくめん #T30 覆面
ふくもと #JNS 福本
ふくやく #T30 服薬
ふくやま #JCN 福山
ふくよう #T30 服用
ふくようき #T35 複葉機
ふくら #M5r 膨ら
ふくり #T35 福利 複利
ふくれ #KSr*1 ふくれ #KSr 膨れ
ふくろ #T35*19 袋 #T35*2 復路
ふくろい #JCN 袋井
ふくろう #T35*1 ふくろう 梟
ふくろおび #T35 袋帯
ふくろこうじ #T35 袋小路
ふぐ #T35*1 河豚 鰒
ふぐあい #T05*8 不具合
ふぐう #T05 不遇
ふぐたいてん #T35*1 不倶載天
ふけ #KS*4 フケ #KS*1 更け #R5 耽 #KS 老け
ふけい #T35 婦警 父兄
ふけいき #T05 不景気
ふけいざい #T05*1 不経済
ふけこ #M5r 老け込
ふけつ #T05 不潔
ふけん #T35 府県
ふけんこう #T05 不健康
ふけんぜん #T05 不健全
ふげし #JCN 鳳至
ふげん #T30 付言
ふこう #T05*9 不幸 #KYU*2 深う #T05 不孝
ふこうへい #T05*3 不公平
ふこく #T30 布告
ふこころえ #T05*1 不心得
ふごう #T35*68 符号 #T30*4 符合 #T35 富豪
ふごうかく #T35*1 不合格
ふごうき #T35*4 符号器
ふごうり #T05*2 不合理
ふさ #T35*15 房 #KJ 絃 #G5r 塞 #KJ 惣
ふさい #T35*5 夫妻 #T35*1 負債
ふさいよう #T35 不採用
ふさが #R5r*1 塞が
ふさく #T35 不作
ふさこ #JNM 房子
ふさわし #KY*2 ふさわし #KY*1 相応し
ふさわしゅう #KYU*1 ふさわしゅう 相応しゅう
ふさんか #T35*6 不参加
ふさんせい #T35 不賛成
ふざい #T15*8 不在
ふざけ #KS*2 ふざけ
ふし #T35*1 節 #KJ 臥
ふしあわせ #T05 不幸せ
ふしぎ #T05*22 不思議
ふしぜん #T05*3 不自然
ふしだら #T05 ふしだら
ふしちょう #T35 不死鳥
ふしぶし #T35 節々
ふしまつ #T05 不始末
ふしみ #JCN 伏見
ふしめ #T35 節目
ふしゃ #T35 富者
ふしゅび #T05 不首尾
ふしょう #T15*1 不詳 #T30 負傷
ふしょうじ #T35*2 不祥事
ふしょく #T30 腐食
ふしん #T05*3 不審 #T35*1 不信 #T05*1 不振
ふしんせつ #T05*3 不親切
ふしんにん #T35 不信任
ふじ #T35*7 藤 #T35*4 不治 #KK*3 フジ #T35 不時 #JCN 富士
ふじい #JCN 藤井
ふじいでら #JCN 藤井寺
ふじうら #JNS 藤浦
ふじえだ #JCN 藤枝
ふじお #JNM 藤雄 富士夫 富士雄 冨士雄
ふじおか #JCN 藤岡
ふじかわ #JCN 藤川 富士川
ふじき #JNS 藤木
ふじくら #JNS 藤倉
ふじこ #JNM 藤子 冨士子
ふじさき #JCN 藤崎
ふじさわ #JCN 藤沢 #JNS 藤澤
ふじさん #CN 富士山
ふじしま #JCN 藤島 #JNS 藤嶋
ふじしろ #JNS 藤城 #JCN 藤代
ふじた #JNS 藤田
ふじたに #JNS 藤谷
ふじちゃく #T30 不時着
ふじつ #JCN 藤津
ふじつう #KK*35 富士通
ふじなみ #JNS 藤波
ふじの #JCN 藤野
ふじのみや #CN 富士宮
ふじはし #JCN 藤橋
ふじまき #JNS 藤巻
ふじみ #T05 不死身 #CN 富士見
ふじむら #JNS 藤村
ふじもと #JNS 藤本
ふじもり #JNS 藤森
ふじゅうぶん #T05*5 不十分
ふじゅさと #JNS 藤里
ふじゅん #T05*1 不純 不順
ふじゆう #T00*5 不自由
ふじょ #T35 婦女 #T30 扶助
ふじょう #T30*1 浮上 #T05 不浄
ふじょし #T35 婦女子
ふじよしだ #CN 富士吉田
ふじわら #JCN 藤原
ふじん #T35*3 夫人 #T35*1 婦人
ふすう #T35*2 負数
ふすま #T35 襖 #KJ 衾
ふず #T35 付図
ふずい #T30*5 付随 #T35 不随
ふせ #G5r*15 防 #T35*1 布施 #JCN 布施 #KS 伏せ #R5 伏せ
ふせい #T05*24 不正 #T05 不省 不整 不斉
ふせいりつ #T35 不成立
ふせき #T30*2 布石
ふせつ #T30*3 付設 #T30*1 敷設
ふせん #T35 不戦 付箋
ふせんしょう #T30 不戦勝
ふせんぱい #T30 不戦敗
ふぜい #T35*3 風情
ふぜん #T05 不全
ふそう #JCN 扶桑
ふそうおう #T05*1 不相応
ふそく #T30*33 不足 #T35*1 付則 #T35 附則
ふそくふり #T15 不即不離
ふそびょう #T35*1 腐蛆病
ふそん #T05*2 不尊 #T05 不遜
ふぞく #T30*83 付属 #T30*22 附属
ふぞくご #T35 付属語
ふぞろい #T05*1 不揃い
ふた #NN*6 二 #T30*3 蓋
ふたい #T30*1 付帯
ふたえ #T35 二重
ふたがみ #JCN 二上
ふたく #T30 付託 負託
ふたご #T35*3 双子
ふたござ #KK*1 双子座
ふたたび #F14*29 再び #F14*2 ふたたび
ふたつ #T35*88 二つ
ふたつぎ #JNS 二木
ふたば #T35*1 双葉 #JCN 双葉
ふたみ #JCN 双海 双三 二見
ふたり #T35*43 二人
ふたりして #F14 二人して
ふたん #T30*32 負担
ふだ #T35*10 札
ふだい #JCN 普代 #T35 譜代
ふだん #T35*16 普段 #T35*2 不断
ふだんぎ #T35 普段着
ふち #T30*13 付値 #T35*8 淵 #KJ*2 渕 #T35*1 縁 #T30 付置
ふちゃく #T30 付着
ふちゅう #JCN 婦中 府中
ふちゅうい #T05*2 不注意
ふちょう #T05*9 不調 #T35 婦長 府庁 符丁
ふっか #T30*1 フッ化
ふっかい #T35*1 復改
ふっかけ #R5 ふっ掛け
ふっかつ #T30*9 復活
ふっき #T30*9 復帰 #R5 ふっ切
ふっきゅう #T30*26 復旧
ふっきん #T35 腹筋
ふっくら #F04 ふっくら
ふっけん #KK*13 復建 #T35*2 復権 #KK*2 復研
ふっこ #T30 復古
ふっこう #T30 復興 復交
ふっこく #T30*1 復刻
ふっさ #JCN 福生
ふっしょく #T30 払拭
ふった #T35*2 フッタ
ふっつ #JCN 富津
ふってい #T30 払底
ふってん #T35 沸点
ふっと #B5 ふっ飛
ふっとう #T30 沸騰
ふっとぼーる #T35*1 フットボール
ふつ #JCN 布津 #KJ 弗 払 沸
ふつう #T05*149 普通 #T35*2 不通 #T05*1 ふつう
ふつか #T35*7 二日 #T35*1 2日 #T35 ２日
ふつごう #T05*10 不都合
ふつつか #RT*2 不束か #T05 ふつつか 不束
ふつふつ #F06 ふつふつ
ふづき #T35*2 文月
ふてい #T05*5 不定 #T05 不貞
ふてき #T05*1 不敵 不適
ふてきせつ #T05*3 不適切
ふてぎわ #T05*1 不手際
ふで #T35*4 筆
ふでき #T05 不出来
ふでぶしょう #T05 筆不精
ふでまめ #T05 筆まめ
ふと #KY*33 太 #R5*32 太 #F14*3 ふと #R5*1 ふと #KJ 太
ふとう #T05*10 不当 #KYU*2 太う #T05*2 不等 #T35*2 埠頭
ふとく #T05 不徳
ふとくい #T05*1 不得意
ふところ #T35*3 懐
ふところで #T30 懐手
ふとじ #T35*2 太字
ふとっぱら #T05 太っ腹 #T15 太っ腹
ふとん #T35*6 布団 #T35 蒲団
ふどう #T30*9 浮動 #T05*1 不動 #T05 不同
ふどうさん #T35*5 不動産
ふどうたい #T35 不導体
ふどうとく #T05 不道徳
ふどき #KK*1 風土記 #T35*1 風土記
ふな #PRE*1 舟 船 #T35*1 鮒
ふない #JCN 船井 #T35 府内
ふなおか #JCN 船岡
ふながた #JCN 舟形
ふなたび #T30 船旅
ふなで #T30 船出
ふなと #JNS 舟渡 船渡
ふなに #T35 船荷
ふなのり #T35*1 船乗り
ふなはし #JCN 舟橋
ふなばし #JCN 船橋
ふなびん #T35 船便
ふなほ #JNS 船穂
ふなよい #T30 船酔い
ふなれ #T05*1 不慣れ
ふにゅう #JNS 船生
ふにん #T35*1 不妊 #T30*1 赴任
ふぬけ #KS 腑抜け #T15 腑抜け
ふね #T35*6 船 #T35*4 舟
ふねひき #JCN 船引
ふねん #T35 不燃
ふの #JCN 布野
ふのう #T05*4 不能
ふはい #T35 不敗 #T30 腐敗
ふはつ #T35 不発
ふひつよう #T05*8 不必要
ふひょう #T05*1 不評 #T35 付表 附票 歩兵
ふび #T05*8 不備
ふびょうどう #T05*1 不平等
ふびん #T15 不憫
ふふく #T05 不服
ふぶ #K5*1 吹雪
ふぶき #T35*1 吹雪
ふへい #T05*4 不平
ふへん #T05*10 不変 #T35*5 普遍 #T35*1 父片
ふべん #T05*21 不便
ふほう #T35*2 訃報 #T05*1 不法
ふほんい #T05*1 不本意
ふぼ #T35*1 父母
ふまえ #KS*2 踏まえ #KS*1 ふまえ
ふまん #T05*16 不満
ふみ #T35*2 文 #JNM ふみ フミ
ふみあき #JNM 文昭
ふみえ #T35 踏み絵
ふみお #JNM 二三男 文男 文夫 文雄
ふみき #R5r*1 踏み切
ふみきり #T35*1 踏切
ふみこ #M5r*1 踏み込 #JNM ふみ子 富美子 冨美子 文子
ふみし #JNM 二三四
ふみたお #S5r 踏み倒
ふみたか #JNM 文孝 文隆
ふみだ #S5 踏み出
ふみだい #T35 踏台
ふみつけ #KSr 踏みつけ
ふみづき #T35 文月
ふみとどま #R5 踏みとどま
ふみにじ #R5*1 踏みにじ
ふみはず #S5r*1 踏み外
ふみひこ #JNM 文彦
ふみやす #JNM 文康
ふみよ #JNM 扶三代
ふみん #T35 不眠
ふむき #T05*1 不向き
ふめい #T05*69 不明
ふめいかく #T05*7 不明確
ふめいよ #T05 不名誉
ふめいりょう #T05*3 不明瞭
ふめつ #T05*1 不滅
ふめん #T35 譜面
ふもう #T05*2 不毛
ふもと #T35*5 麓
ふもん #T35 不問
ふや #S5*36 増や #S5*1 ふや #S5 殖や
ふやけ #KS ふやけ
ふゆ #T35*20 冬
ふゆう #T30*3 浮遊 #T05*1 富裕
ふゆかい #T05*2 不愉快
ふゆきとどき #T05 不行届き
ふゆば #T35*1 冬場
ふゆもの #T35 冬物
ふゆやすみ #T35*1 冬休み
ふゆやま #T35 冬山
ふよ #T30*1 付与
ふよう #T05*43 不要 #T30*3 扶養 #T05 不用
ふようい #T05*1 不用意
ふようど #T35 腐葉土
ふらい #T35*4 フライ
ふらいと #T35*4 フライト
ふらいぱん #T35*2 フライパン
ふらいんぐ #T30*1 フライング
ふらくたる #T05*2 フラクタル
ふらぐ #T35*128 フラグ
ふらぐめんてーしょん #T35*1 フラグメンテーション
ふらぐめんと #T35*3 フラグメント
ふらっぐ #T35*2 フラッグ
ふらっしゅ #T30*2 フラッシュ
ふらっと #T05*3 フラット #T35*1 ♭
ふらつ #K5r*2 ふらつ
ふらの #CN 富良野
ふらふら #F04 ふらふら
ふらみんご #T35 フラミンゴ
ふらめんこ #T35 フラメンコ
ふらり #F06 ふらり
ふらわー #T35*1 フラワー
ふらん #JS*1 フラン #T35 不乱 #T30 腐乱
ふらんく #T05*1 フランク
ふらんす #CN フランス
ふらんすご #KK*2 フランス語
ふらんそわーず #JN フランソワーズ
ふり #T30*5 ふり #T05*3 不利
ふりー #T05*43 フリー
ふりーく #T35*2 フリーク
ふりーだむ #T35*101 フリーダム
ふりえき #T35*2 不利益
ふりかえ #T35*2 振替 #KSr*1 振り替え #R5r*1 振り返
ふりかか #R5 降り掛か
ふりかけ #KSr 振りかけ
ふりかた #T35*1 振り方
ふりき #R5 振り切
ふりこ #M5r*6 振り込
ふりこみ #T30*3 振込
ふりすびー #T35*1 フリスビー
ふりそそ #G5 降り注
ふりそで #T35 振袖
ふりだ #S5*2 降り出 #S5r 振り出
ふりだし #T35 振出
ふりっぷふろっぷ #T35*1 フリップフロップ
ふりつ #T35*3 府立
ふりつけ #T30 振付け
ふりはら #W5 振り払
ふりま #K5 振りま
ふりまわ #S5*1 振り回
ふりみだ #S5 振り乱
ふりむ #K5*2 振り向 #K5*1 ふりむ
ふりょ #T35*1 不慮
ふりょう #T15*10 不良 #T35 不漁
ふりょく #T35 浮力
ふりわけ #KSr*3 振り分け
ふりん #T00*1 不倫
ふる #KY*138 古 #T35*81 古 #W5*12 震 #T35*7 フル #PRE*4 フル #W5*2 振る #W5*1 奮 #KY 旧
ふるーつ #T35*2 フルーツ
ふるいおこ #S5 奮い起こ
ふるいおと #S5r ふるい落と
ふるいた #T5 奮い立
ふるいわけ #KSr*1 ふるい分け
ふるう #KYU*3 古う #KYU 旧う #JCN 古宇
ふるえ #KSr 震え
ふるえあが #R5*1 震え上が
ふるおろぐらふぃー #T35*1 フルオログラフィー
ふるかわ #JCN 古川
ふるくさ #KY*1 古くさ
ふるさと #T35 古里
ふるさわ #JNS 古沢
ふるす #T35 古巣
ふるた #JNS 古田
ふるて #T35 古手
ふるどの #JCN 古殿
ふるの #JNS 古野
ふるはうす #T35*1 フルハウス
ふるはし #JNS 古橋
ふるはた #JNS 古畑
ふるび #KS*1 古び
ふるびら #JCN 古平
ふるほん #T35 古本
ふるま #W5r*14 振舞 #W5r*1 振る舞 #W5r ふるま
ふるまい #T35*4 振舞
ふるむら #JNS 古村
ふるめかし #KY 古めかし
ふるめかしゅう #KYU 古めかしゅう
ふるや #JNS 古屋 #T35 古屋 #JNS 古谷
ふるやま #JNS 古山
ふるわ #S5*2 震わ
ふれ #KS*39 触れ
ふれーく #T35*1 フレーク
ふれーま #T35*2 フレーマ
ふれーむ #T35*59 フレーム
ふれあ #W5r*1 触れ合
ふれあい #T35*4 ふれ合い
ふれこ #M5r 触れ込
ふれっくす #T15 フレックス
ふれんぞく #T05*2 不連続
ふれんち #T35*1 フレンチ
ふれんど #T35 フレンド
ふろ #T35*8 風呂
ふろー #T35*20 フロー
ふろーてぃんぐ #T35*1 フローティング
ふろーら #T35*1 フローラ
ふろあ #T35*8 フロア
ふろう #T35*1 不老 #T35 不労 #T30 浮浪
ふろく #T35*23 付録
ふろしき #T35*1 風呂敷
ふろっぴ #T35*4 フロッピ
ふろっぴー #T35*22 フロッピー
ふろっぴぃ #T35*5 フロッピィ
ふろんと #T35*30 フロント
ふわ #T05*1 不和 #JCN 不破
ふわく #T35 不惑
ふわたり #T35 不渡り
ふわふわ #F14*1 フワフワ
ふわらいどう #T30*1 付和雷同
ふわりと #F14 ふわりと
ふん #JS*74 分 #T35*30 分 #S5*8 扮 #T30*8 扮 #JS*1 ′ #T35*1 ′ #KJ*1 墳 #KJ 吻 噴 憤 #SX 憤 #KJ 焚 奮 粉 #T35 糞 #KJ 紛 雰
ふんいき #T35*24 雰囲気
ふんえん #T35 噴煙
ふんか #T30*1 噴火
ふんがい #T30*3 憤慨
ふんき #T35 噴気 #T30 奮起
ふんぎ #R5r ふん切
ふんさい #T30 粉砕
ふんしつ #T30*5 紛失
ふんしゃ #T30*1 噴射
ふんしゅつ #T30*2 噴出
ふんしょく #T30 粉飾
ふんすい #T35 噴水
ふんぜん #F00 憤然 奮然
ふんそう #T30*1 紛争 #T30 扮装
ふんそく #T35*1 吻側
ふんぞりかえ #R5 ふんぞり返
ふんだん #T05*2 ふんだん
ふんとう #T30*1 奮闘
ふんどう #T35 分銅
ふんどし #T35 褌
ふんにゅう #T35 粉乳
ふんぱつ #T30*1 奮発
ふんぷん #T00 紛々
ふんべつ #T30 分別
ふんまつ #T35 粉末
ふんまん #T35 噴懣
ふんむ #T30*1 噴霧
ふんわり #F04 ふんわり
ぶ #JS*255 部 #KJ*6 武 #T35*2 部 #KJ*1 不 #JS*1 分 #T35*1 分 #JS*1 歩 #KJ*1 無 #KJ 悔 #PRE 不 #KJ 侮 撫 舞 葡 蕪 #SUC 部 #KJ 歩
ぶーす #T35*6 ブース
ぶーすた #T35*3 ブースタ
ぶーつ #T35*3 ブーツ
ぶーと #T30*9 ブート
ぶーむ #T35*2 ブーム
ぶあい #T35 歩合
ぶあいそう #T05*2 無愛想
ぶあつ #KYmi*2 分厚
ぶあつう #KYU*1 分厚う
ぶいん #T35*1 部員
ぶえんりょ #T05 無遠慮
ぶか #KY*6 深 #T35*3 部下
ぶかい #T35*3 部会
ぶかん #CN 武漢
ぶがい #T35*3 部外
ぶき #T35*4 武器
ぶきみ #T05*3 不気味
ぶきょく #T35*2 部局
ぶきよう #T05*1 不器用
ぶきりょう #T05 不器量
ぶくみ #SUC 含み
ぶくろ #T35 袋
ぶこう #KYU*1 深う
ぶこく #T30 誣告
ぶさいく #T05 不細工
ぶさほう #T05 無作法
ぶざー #T35*1 ブザー
ぶざま #T05 不様
ぶし #JNSUC*4 節 #T35*1 武士
ぶしつ #T35 部室
ぶしつけ #T05 ぶしつけ
ぶしゅ #T35*123 部首
ぶしょ #T35*20 部署 #T35*15 部所
ぶしょう #T05*4 不精 #T35*4 武将
ぶじ #T05*43 無事
ぶじょく #T30 侮辱
ぶす #T35*2 ブス
ぶすい #T05 不粋
ぶすう #T35*8 部数
ぶせい #JS 部制
ぶぜん #JCN 豊前
ぶそう #T30*1 武装
ぶそく #T10*20 ぶそく #N2T16*7 不足 #T10*1 不足
ぶぞく #T35*1 部族
ぶた #SUC*3 ぶた #SUC*2 蓋 #T35*2 豚
ぶたい #T35*6 舞台 #T35*5 部隊
ぶたにく #T35 豚肉
ぶち #T35*13 斑
ぶちょう #T35*71 部長
ぶっか #T35 物価
ぶっかく #T35 仏閣
ぶっきょう #T35*1 仏教
ぶっきらぼう #T05 ぶっきらぼう
ぶっく #T35*24 ブック
ぶっくす #T35*1 ブックス
ぶっけん #T35*226 物件
ぶっし #T35*1 仏師 物資
ぶっしつ #T35*2 物質
ぶっしょう #T35 物証
ぶっしょく #T30*1 物色
ぶっせい #T35 物性
ぶっそう #T05 物騒
ぶったい #T35*13 物体
ぶったぎ #R5r ぶった切
ぶっだ #T35*1 仏陀
ぶっちゃけ #KS ぶっちゃけ
ぶっつけ #KSr ぶっつけ
ぶってき #T05 物的
ぶっと #B5 ぶっ飛
ぶっとお #S5r ぶっ通
ぶっぴん #T35*3 物品
ぶつ #SUC*45 物 #KJ*2 仏 #T35 物 #KJ 勿
ぶつか #R5r*4 ぶつか
ぶつぎ #T35*2 物議
ぶつけ #KS*2 ぶつけ
ぶつぞう #T35 仏像
ぶつだん #T35 仏壇
ぶつぶつ #F14*1 ぶつぶつ
ぶつめつ #T35*1 仏滅
ぶつり #T35*99 物理
ぶつりょう #T35*1 物療 #T35 物量
ぶてぃっく #T35*1 ブティック
ぶと #KY 太
ぶとう #KYU*1 太う
ぶどう #T35*2 武道 #T35*1 葡萄
ぶどまり #T35*1 歩留まり
ぶな #T35 樗
ぶない #T35*24 部内
ぶなん #T05*4 無難
ぶね #T35 船
ぶばいがわら #JCN 分倍河原
ぶひ #T35*8 部費
ぶひん #T35*42 部品
ぶふうりゅう #T05 無風流
ぶぶん #T35*255 部分
ぶもん #T35*39 部門
ぶよう #T35*1 舞踊
ぶようじん #T05 不用心
ぶらい #T05 無頼
ぶらいあん #JN ブライアン
ぶらいんど #T35*4 ブラインド
ぶらうざ #T35*5 ブラウザ
ぶらうす #T30*1 ブラウス
ぶらく #T35 部落
ぶらさが #R5r*2 ぶら下が
ぶらさげ #KSr*1 ぶら下げ
ぶらざー #JN ブラザー
ぶらじゃー #T35*1 ブラジャー
ぶらじる #CN ブラジル
ぶらっく #T05*10 ブラック
ぶらぶら #F04 ぶらぶら
ぶらり #F04 ぶらり
ぶらんく #T05*14 ブランク
ぶらんでー #T35*1 ブランデー
ぶらんど #T35*2 ブランド
ぶり #SUC*1 振り #SUC ぶり
ぶりかえ #S5r ぶり返
ぶりぢすとん #KK*1 ブリヂストン
ぶりっじ #T30*4 ブリッジ
ぶりょく #T35 武力
ぶるー #T05*3 ブルー
ぶるい #T35*3 部類
ぶるがりあ #CN ブルガリア
ぶるまい #N2T30*1 振舞い
ぶれ #KSr ぶれ
ぶれーか #T35*1 ブレーカ
ぶれーき #T35*4 ブレーキ
ぶれーく #T30*4 ブレーク
ぶれい #T05*10 無礼
ぶれいく #T30 ブレイク
ぶれいこう #T35 無礼講
ぶろ #T35*7 風呂
ぶろーか #T35 ブローカ
ぶろーどきゃすと #T30*22 ブロードキャスト
ぶろっく #T30*47 ブロック
ぶん #T35*255 文 #JS*158 分 #T35*4 分 #KJ*1 聞 #KJ 蚊
ぶんあつ #T35*1 分圧
ぶんい #T35*3 文意
ぶんいち #JNM 文一
ぶんえん #T30*4 分煙
ぶんか #T35*20 文化 #T35*3 分科 #T30*1 分化 #T35 文科
ぶんかい #T30*13 分解
ぶんかく #T35*2 文革 #T30*1 分画
ぶんかざい #T35 文化財
ぶんかつ #T30*74 分割 #JS*4 分割
ぶんかん #T35*2 分館
ぶんがく #T35*2 文学 #T35*1 文學
ぶんき #T30*4 分岐
ぶんきょう #T35*13 文教 #CN 文京
ぶんぎ #JS*6 分木
ぶんぎょう #T30*1 分業
ぶんぐ #T35 文具
ぶんけ #T30 分家
ぶんけい #T35*2 文型 文系
ぶんけん #T35*17 文献
ぶんげい #T35*1 文芸 文藝
ぶんげいしゅんじゅう #KK*1 文藝春秋
ぶんこ #T35*5 文庫
ぶんこう #T35*1 分校
ぶんこつ #T30 分骨
ぶんご #T35*1 文語 #CN 豊後
ぶんごう #T35*4 文豪
ぶんごたかだ #CN 豊後高田
ぶんさい #T35 文才
ぶんさつ #T30 分冊
ぶんさん #T30*104 分散
ぶんし #T35*2 分子 分詞
ぶんしつ #T35*3 分室
ぶんしゅう #T30*2 分周 #T35 文集
ぶんしょ #T35*196 文書
ぶんしょう #T35*77 文章
ぶんしん #T30*3 分身
ぶんじ #JNM 文治 文二
ぶんじょう #T30 分譲
ぶんすい #JCN 分水
ぶんすう #T35*1 分数
ぶんせき #T30*28 分析 #T35*1 文責
ぶんせつ #T35*255 文節
ぶんたい #T35*4 文体 #T35*1 分隊
ぶんたん #T30*27 分担
ぶんだん #T30*13 分断 #T35 文壇
ぶんちゅう #T30*1 分注 #T35 文中
ぶんちん #JNM 文珍
ぶんつう #T30*1 文通
ぶんど #R5r*1 ぶんど
ぶんのう #T30 分納
ぶんぱ #T30 分派
ぶんぱい #T30*2 分配
ぶんぴつ #T30 分泌 分筆 #T35 文筆
ぶんぶ #T35 文武
ぶんぶつ #T35 文物
ぶんぶん #F06 ぶんぶん
ぶんぷ #T30*7 分布
ぶんべつ #T30*4 分別
ぶんべん #T30 分娩
ぶんぺい #JNM 文平
ぶんぼ #T35*1 分母
ぶんぼうぐ #T35*5 文房具
ぶんぽう #T35*91 文法
ぶんまつ #T35*2 文末
ぶんみゃく #T35*3 文脈
ぶんめい #T35*1 文明
ぶんめん #T35*5 文面
ぶんや #T35*36 分野
ぶんよ #T30 分与
ぶんらく #T35*1 文楽
ぶんり #T30*7 分離
ぶんりつ #T30 分立
ぶんりょう #T35*3 分量
ぶんるい #T30*55 分類
ぶんれい #T35*2 文例
ぶんれつ #T30*1 分裂
ぷーどる #T35*1 プードル
ぷーる #T30*3 プール
ぷさい #T35*1 Ψ ψ
ぷっしゅ #T30*7 プッシュ
ぷらいおりてぃ #T35*6 プライオリティ
ぷらいばし #T35*1 プライバシ
ぷらいばしー #T35*3 プライバシー
ぷらいべーと #T05*4 プライベート
ぷらぐ #T35*35 プラグ
ぷらざ #T35*3 プラザ
ぷらす #T30*14 プラス #T35*3 ＋
ぷらすちっく #T35*3 プラスチック
ぷらすまいなす #T35*1 ±
ぷらすみど #T35*1 プラスミド
ぷらずま #T35*1 プラズマ
ぷらちな #T35 プラチナ
ぷらっと #T35*10 プラット
ぷらっとふぉーむ #T35*10 プラットフォーム
ぷらとん #JN プラトン
ぷらん #T35*4 プラン
ぷらんと #T35*1 プラント
ぷらんにんぐ #T30*4 プランニング
ぷり #PRE*22 プリ
ぷりえでぃっと #T30 プリエディット
ぷりずむ #T35*1 プリズム
ぷりみてぃぶ #T05*16 プリミティブ
ぷりんす #T35*6 プリンス
ぷりんせす #T35*2 プリンセス
ぷりんた #T35*97 プリンタ
ぷりんと #T30*54 プリント
ぷるだうん #T30*2 プルダウン
ぷれー #T30*20 プレー
ぷれーと #T30*5 プレート
ぷれーにんぐ #T30*1 プレーニング
ぷれーやー #T35*2 プレーヤー
ぷれいや #T35*1 プレイヤ
ぷれいやー #T35*1 プレイヤー
ぷれいん #T05*1 プレイン
ぷれいんきゅべーしょん #T30*1 プレインキュベーション
ぷれじでんと #T35*1 プレジデント
ぷれす #T30*3 プレス
ぷれぜんてーしょん #T30*6 プレゼンテーション
ぷれぜんと #T30*8 プレゼント
ぷれびゅーあ #T35*1 プレビューア
ぷれりゅーど #KK*1 プレリュード
ぷろ #T35*40 プロ
ぷろーぶ #T35*1 プローブ
ぷろぐらま #T35*27 プログラマ
ぷろぐらみんぐ #T30*82 プログラミング
ぷろぐらむ #T30*255 プログラム
ぷろしーじゃ #T35*23 プロシージャ
ぷろしじゃ #T35*2 プロシジャ
ぷろじぇくと #T35*111 プロジェクト
ぷろじぇすてろん #T35*1 プロジェステロン
ぷろすたぐらんでぃん #T35*1 プロスタグランディン
ぷろせしんぐ #T30*1 プロセシング
ぷろせす #T30*247 プロセス
ぷろせっさ #T35*73 プロセッサ
ぷろせっしんぐ #T35*7 プロセッシング
ぷろだくしょん #T35*4 プロダクション
ぷろだくと #T35*37 プロダクト
ぷろっと #T30 プロット
ぷろてくしょん #T30*1 プロテクション
ぷろてくと #T30*5 プロテクト
ぷろでゅーさ #T35*2 プロデューサ
ぷろでゅーす #T30*1 プロデュース
ぷろとこる #T35*82 プロトコル
ぷろとたいぴんぐ #T30*8 プロトタイピング
ぷろとたいぷ #T35*7 プロトタイプ
ぷろぱがんだ #T30 プロパガンダ
ぷろぱげーしょん #T30*6 プロパゲーション
ぷろぱてぃ #T35*2 プロパティ
ぷろぱん #T35*1 プロパン
ぷろふぁいら #T35*1 プロファイラ
ぷろふぃーる #T35*2 プロフィール
ぷろふぇっしょなる #T35*1 プロフェッショナル
ぷろぺら #T35*1 プロペラ
ぷろぽーざる #T35*4 プロポーザル
ぷろぽーず #T30*1 プロポーズ
ぷろもうと #T30*1 プロモウト
ぷろらくちん #T35*1 プロラクチン
ぷろんぷと #T35*20 プロンプト
ぷん #JS*54 分
へ #KS*54 経 #R5r*29 減 #KJ*1 辺
へい #KJ*19 平 #T35*13 兵 #KJ*10 閉 #KJ*2 併 #T35*1 丙 塀 #KJ*1 柄 蔽 #KJ 坪 瓶 幣 弊 並 #JCN 閉伊 #KJ 陛 箆 餅
へいあん #T05*1 平安
へいい #T05*1 平易
へいいち #JNM 平一
へいえい #T30*1 併映
へいえき #T35 兵役
へいおん #T05 平穏
へいか #T35*1 陛下 #T35 平価
へいかい #T30*1 閉会
へいかん #T30 閉館
へいがい #T35*3 弊害
へいき #T05*10 平気 #T35*1 兵器 #T30 併記
へいきち #JNM 平吉
へいきん #T30*21 平均
へいきんち #T35*2 平均値
へいきんてん #T35 平均点
へいけい #T30*1 閉経
へいげい #T30 睥睨
へいげん #T35*1 平原
へいこう #T30*18 並行 #T00*4 平行 #T30*2 平衡 #T30 併行 閉口 閉校
へいこうしへんけい #T35 平行四辺形
へいこうせん #T35 平行線
へいごう #T30*1 併合
へいさ #T30*1 閉鎖
へいざん #T30*1 閉山
へいし #T35*1 兵士 #T30*1 斃死
へいしゃ #T35*28 弊社 #T35 兵舎
へいしゅうごう #T35 閉集合
へいしょ #T35 閉所
へいしん #T30 平身 並進
へいじ #T35 平時
へいじつ #T35*11 平日
へいじょう #T05 平常 #T30 閉場
へいじょうきょう #KK*1 平城京
へいせい #KK*4 平成 #T05*1 平静
へいせかい #T35*1 閉世界
へいせつ #T30*1 併設
へいぜん #F00 平然
へいそ #T35*1 平素
へいそく #T30 閉塞
へいそん #T30 併存
へいたい #T35*1 兵隊
へいたん #T05 平坦
へいち #T35 平地
へいちょう #T30 閉庁
へいてい #T30 平定 閉廷
へいてん #T30*1 閉店 #T35 弊店
へいどん #T30 併呑
へいねつ #T35 平熱
へいねん #T35 平年
へいはちろう #JNM 平八郎
へいはつ #T30*2 併発
へいばい #T10 併売
へいふく #T35 平伏 平服
へいべい #JS 平米
へいほう #T35*3 閉包 #JS*2 平方 #T30 平方
へいぼん #T05*1 平凡
へいまく #T30*1 閉幕
へいみん #T35*1 平民
へいめい #T05 平明
へいめん #T35*3 平面
へいもん #T30 閉門
へいや #T35*2 平野
へいゆ #T30 平癒
へいゆう #T30 併有
へいよう #T30*3 併用
へいりつ #T30*1 並立
へいりょく #T35 兵力
へいれつ #T30*62 並列
へいろ #T35 閉路
へいわ #T05*7 平和 #CN 平和
へき #T35*1 壁 #JCN 日置 #T35 僻 癖 碧
へきえき #T30*1 へき易
へきかい #JCN 碧海
へきが #T35 壁画
へきち #T35 僻地
へきとう #T35 劈頭
へきなん #CN 碧南
へきめん #T35 壁面
へくたーる #T35*1 ヘクタール
へぐり #JCN 平群
へそ #T35 へそ
へそまがり #T15 へそ曲がり
へた #T00*12 下手
へたくそ #T05*1 へたくそ
へだ #CN 戸田
へだた #R5r*2 隔た
へだて #KSr 隔て
へったくれ #T35 へったくれ
へっだ #T35*120 ヘッダ
へっだー #T35*37 ヘッダー
へっちゃら #T06 へっちゃら
へっど #T35*18 ヘッド
へっどほん #T35*1 ヘッドホン
へてろじにあす #T05*1 ヘテロジニアス
へび #T35*2 蛇
へぼ #T05 へぼ
へぼん #JN ヘボン
へま #T35 へま
へもぐろびん #T35*1 ヘモグロビン
へや #T35*47 部屋
へら #S5r*39 減ら #T35*1 箆
へり #T35 縁
へりうむ #T35*1 ヘリウム
へりくだ #R5r 遜
へりっくす #T35*1 ヘリックス
へるつ #JS*1 ヘルツ
へるぷ #T35*17 ヘルプ
へん #T05*255 変 #ZX*247 変 #T35*183 辺 #S5*17 偏 #T30*17 偏 #T35*16 へん #KJ*5 返 #T35*2 編 #T35*1 片 #JNSUC*1 篇 #KJ*1 遍 #KJ 篇 #JNSUC 編 #JS 辺
へんあつき #T35*1 変圧器
へんい #T30*1 変移
へんか #T30*70 変化
へんかく #T35 偏角 #T15 変格 #T30 変革
へんかくかつよう #T30 変格活用
へんかん #T30*255 変換 #T30*16 返還
へんかんき #T35*7 変換器
へんきゃく #T30*27 返却
へんきゃくち #T35 返却値
へんきょう #T35*1 辺境
へんきょく #T30 編曲
へんきん #T30*3 返金
へんくつ #T05*1 偏屈
へんけい #T30*16 変形 #T30 変型
へんけん #T35*3 偏見
へんげ #T35*1 変化
へんこう #T30*255 変更 #T30*21 偏向 #T35 偏光
へんこうてん #T35 変更点
へんさ #T35*2 偏差
へんさい #T30*4 返済
へんさん #T30 編簒
へんざい #T30*13 偏在 #T30 遍在
へんし #T30 変死
へんしつ #T30*1 変質
へんしゅう #T30*150 編集 #T30 偏執 編修
へんしゅうきょう #T35 偏執狂
へんしょ #T35 返書
へんしょく #T30 偏食 変色
へんしん #T30*3 変身 #T30*2 返信
へんじ #T30*160 返事
へんじょう #T30*1 返上
へんじん #T35 変人
へんすう #T35*247 変数
へんせい #T30*5 編成 #T30*1 変性 #T30 変声 編制
へんせん #T30*2 変遷
へんそう #T30*9 返送 #T30*1 変奏 変装
へんそく #T05 変則 #T30 変速
へんぞう #T30 変造
へんたい #T05*3 変態 #T35*1 編隊
へんち #T35*3 辺地
へんちょう #T30*1 変調
へんちょうは #T35*1 変調波
へんてこ #T05 変てこ
へんてつ #T35 変哲
へんてん #T30 変転
へんとう #T30*10 返答 #T35*1 扁桃
へんどう #T30*7 変動
へんにゅう #T30*1 編入
へんのう #T30 返納
へんぴん #T30 返品
へんぷ #T30 返付
へんぼう #T30 変貌
へんぽん #T30 返本
へんよう #T30 変容
へんりん #T35 片鱗
へんれい #T30 返戻 返礼
へんれき #T30 遍歴
へんろ #T35 遍路
べ #KJ*44 部 #KJ*1 辺
べーこん #T35*1 ベーコン
べーしっく #T05*1 ベーシック
べーす #T35*188 ベース
べーた #T35*27 β #T35*19 ベータ #T35*2 Β
べーとーう゛ぇん #JN ベートーヴェン
べーとーべん #JN ベートーベン
べあ #T35*1 ベア
べいか #T35 米価
べいこく #CN 米国
べいさく #T35 米作
べいしきしゅうきゅう #T35 米式蹴球
べいしょく #T35 米食
べいじゅ #T35 米寿
べいちょう #JNM 米朝
べき #KJ*3 巾 #KJ*1 冪
べきじょう #T30*3 冪乗 #T30*2 巾乗
べくた #T35*4 ベクタ
べくたー #T35*2 ベクター
べくとる #T35*4 ベクトル
べしょべしょ #T10 べしょべしょ
べすと #T15*11 ベスト
べすとせらー #T35*1 ベストセラー
べそ #T35 べそ
べたべた #T10 べたべた
べっかく #T05 別格
べっかん #T35*6 別館 #T35 別巻
べっき #T30*1 別記
べっきょ #T30 別居
べっけん #T35*4 別件
べっこ #T05*1 別個
べっこう #T35*1 別項
べっさつ #T35*3 別冊
べっし #T35*22 別紙 #T30*2 蔑視
べっしつ #T35 別室
べっしやま #CN 別子山
べっしゅ #T35*1 別種
べっしょう #T35 別称
べっせい #T35*6 別姓
べっそう #T35*2 別荘 #T30 別送
べったく #T35 別宅
べったり #F04 べったり
べっちゅう #T30 別注
べっと #T35*19 別途
べっど #T35*3 ベッド
べっぴょう #T35 別表
べっぴん #T05 別嬪
べっぷ #JCN 別府
べっぽう #T35 別報
べつ #T05*255 別 #SUC*13 別 #KJ 捌 瞥 蔑
べつぎ #T35 別儀
べつくち #T35 別口
べつじょう #T35 別状
べつじん #T35*1 別人
べつだん #F14 別段
べつのう #T30 別納
べつばい #T30*1 別売
べつびん #T35*4 別便
べつべつ #T05*13 別々
べつむね #T35 別棟
べつめい #T35*19 別名
べつもの #T35*3 別物
べつり #T30 別離
べてらん #T35*2 ベテラン
べとなむ #CN ベトナム
べに #T35*2 紅
べびー #T35*3 ベビー
べや #SUC*3 部屋
べりふぁい #T30*1 ベリファイ
べりふぁいあ #T35*1 ベリファイア
べる #T35*10 ベル
べるぎー #CN ベルギー
べると #T35 ベルト
べるりん #CN ベルリン
べん #T35*27 弁 #ZX*25 弁 #T35*4 便 #ZX*2 便 #KJ 采 勉 娩 鞭
べんえき #T35 便益
べんかい #T30*1 弁解
べんがく #T30*1 勉学
べんきょう #T30*50 勉強
べんぎ #T05*3 便宜
べんぎじょう #F14*2 便宜上
べんけい #JN*1 弁慶
べんご #T30*1 弁護
べんごし #T35 弁護士
べんさい #T30 弁済
べんしょう #T30 弁償
べんじょ #T35 便所
べんたつ #T30 鞭撻
べんだ #T35*19 ベンダ
べんだー #T35*3 ベンダー
べんちまーく #T30*2 ベンチマーク
べんちゃ #T35*2 ベンチャ
べんちゃー #T35*1 ベンチャー
べんてん #JCN 弁天
べんとう #T35*12 弁当
べんぱつ #T35*3 弁髪
べんぴ #T10 便秘
べんべつ #T30 弁別
べんぽう #T35*1 便法
べんむかん #T35 弁務官
べんめい #T30*1 弁明
べんもう #T35*1 鞭毛
べんらん #T35 便覧
べんり #T05*62 便利 #T35*2 弁理
べんりし #T35 弁理士
べんれい #T30 勉励
べんろん #T30 弁論
ぺーじ #T35*174 ページ #JS*17 ページ #T35*3 頁 #JS 頁
ぺーじあうと #T30*3 ページアウト
ぺーじいん #T30*3 ページイン
ぺーじゃ #T35*2 ページャ
ぺーじんぐ #T30*1 ページング
ぺーす #T35*13 ペース
ぺーすと #T30*12 ペースト
ぺーぱー #T35*7 ペーパー
ぺあ #T35*10 ペア
ぺいじ #JS*2 ペイジ
ぺきん #CN 北京
ぺけ #T35*12 ×
ぺしみすと #T05*1 ペシミスト
ぺしみずむ #T35 ペシミズム
ぺってぃんぐ #T30*1 ペッティング
ぺっと #T35*6 ペット
ぺですたる #T35*1 ペデスタル
ぺとりさら #T35*1 ペトリ皿
ぺとりねっと #T35*2 ペトリネット
ぺなるてぃ #T35*1 ペナルティ
ぺにしりん #T35 ペニシリン
ぺにす #T35*1 ペニス
ぺぷちど #T35*1 ペプチド
ぺりー #JN ペリー
ぺりかん #T35 ペリカン
ぺん #JS*6 辺 #T35*3 ペン
ぺんき #T35*2 ペンキ
ぺんぎん #T35*1 ペンギン
ぺんしょん #T35*1 ペンション
ぺんとはうす #T35*1 ペントハウス
ぺんねーむ #T35*1 ペンネーム
ほ #JS*58 歩 #T30*52 補 #T30*47 保 #R5*38 ほ #S5*25 干 #R5*5 掘 #KJ*4 捕 穂 #R5r*1 彫 #T35*1 帆 #KJ*1 舗 #KJ 浦 蒲 畝 葡 鋪 圃 #T35 歩 #KJ 甫 輔
ほーみんぐ #T30*1 ホーミング
ほーむ #T35*57 ホーム
ほーむず #JN ホームズ
ほーる #T35*7 ホール
ほーん #KK*2 ホーン
ほあん #T35*2 保安
ほい #T30 補遺 #CN 宝飯
ほいく #T30*1 保育
ほう #T35*255 方 #T35*26 法 #T35*10 ほう #JS*10 報 #T35*10 報 #ZX*10 報 #KJ*4 包 #R5*4 放 #KJ*3 放 #KJ*2 奉 胞 #KJ*1 宝 峰 砲 縫 芳 訪 邦 鳳 #KJ 逢 鞄 蔀 鴇 保 倣 俸 呆 #ZX 奉 #KJ 峯 崩 #ZX 崩 #KJ 庖 抱 捧 朋 #SUC 法 #KJ 泡 烹 #SUC 砲 #KJ 萌 蓬 蜂 褒 豊 鋒 飽 鵬 棒 朴
ほうあん #T35*1 法案
ほうい #T35*2 方位 #T30 包囲
ほういがく #T35 法医学
ほうえい #T30*3 放映
ほうおう #T35 法王 鳳凰
ほうか #T30 放火 #T35 法科 邦貨
ほうかい #T30*2 崩壊
ほうかいしゃく #T35 法解釈
ほうかいせき #T35 方解石
ほうかご #T35 放課後
ほうかつ #T30*1 包括
ほうかん #T35 宝冠 幇間
ほうが #T35*16 邦画 #T30*4 奉賀 #T30 萌芽
ほうがい #T05*1 法外
ほうがく #T35*11 方角 #T35*2 法学
ほうがん #T30*4 包含 #T35 方丸 方眼
ほうがんし #T35 方眼紙
ほうき #T30*6 放棄 #T35*1 法規 #T35 芳紀 #T30 蜂起 #T35 箒
ほうきゅう #T35 俸給
ほうぎょ #T30*1 崩御
ほうけ #KS*1 惚け
ほうけい #T35*1 方形
ほうけん #T35 封建
ほうげき #T30 砲撃
ほうげん #T30*7 放言 #T35*4 方言
ほうこ #T35*1 宝庫
ほうこう #T35*101 方向 #T35*2 芳香 #T30*1 奉公 #T30 放校 彷徨
ほうこく #T30*202 報告
ほうごう #T30 縫合
ほうさく #T35*4 方策 #T15 豊作
ほうさん #T30 放散 #T35 硼酸
ほうし #T30*4 奉仕 #T35*2 法師 #T35 胞子 芳志
ほうしき #T35*221 方式
ほうしゃ #T30*3 放射
ほうしゃせい #T35*4 放射性
ほうしゃせん #T35*1 放射線
ほうしゃのう #T35*3 放射能
ほうしゅう #T35*3 報酬
ほうしゅつ #T30*3 放出
ほうしゅやま #CN 宝珠山
ほうしょ #T35 奉書
ほうしょう #T35 報償 #T30 報奨 #T35 報賞 褒賞
ほうしょく #T30*1 奉職 #T35 宝飾 #T30 飽食
ほうしょくひん #T35 宝飾品
ほうしん #T35*46 方針 #T30 放心
ほうじ #T35*1 法事 #KS 焙じ
ほうじゅう #T05 放縦
ほうじゅく #T30 豊熟
ほうじゅん #T05 芳醇
ほうじょ #T30 幇助
ほうじょう #JNS 方城 #T05 豊饒 #JCN 北条 #JNS 北條
ほうじょうき #KK 方丈記
ほうじん #T35*3 法人 #T35*2 邦人 #T35 方陣
ほうすい #T30 放水
ほうせい #T30*1 縫製 #T05 方正 #KK 法政
ほうせき #T35*1 宝石
ほうせつ #T30 包接
ほうせん #T35*3 法線
ほうせんか #T35 ホウセンカ 鳳仙花
ほうそ #T35 硼素
ほうそう #T30*10 放送 #T30*1 包装 #T35*1 疱瘡
ほうそうかい #T35*1 法曹界
ほうそうきょく #T35*3 放送局
ほうそうし #T35 包装紙
ほうそく #T35*3 法則
ほうたい #T35*1 方体 #T30 包帯
ほうたく #CN 飽託
ほうだい #T35*4 放題 #T35*3 邦題
ほうだん #T30*1 放談 #T35 砲弾
ほうち #T30*4 放置 #T35 法治
ほうちく #T30 放逐
ほうちゃく #T30 逢着
ほうちゅう #T30 訪中
ほうちょう #T35*2 包丁
ほうてい #T35*2 法定 #T35 法廷
ほうていしき #T35*3 方程式
ほうてき #T05*3 法的 #T30 放擲 抛擲
ほうでん #T30*2 放電
ほうとう #T35 ほうとう 宝刀 #T30 放蕩
ほうどう #T30*1 報道
ほうどうじん #T35 報道陣
ほうなん #JCN 方南
ほうにち #T30 訪日
ほうにん #T30 放任
ほうねん #T35 放念 豊年
ほうのう #T30*1 奉納
ほうはい #F02 澎湃
ほうはつ #T35 髣髪
ほうばい #T35 朋輩
ほうび #T35*4 褒美
ほうふ #T05*6 豊富 #T35*1 抱負 #CN 防府
ほうふく #T30*2 報復
ほうふつ #F00 彷彿
ほうぶつせん #T35 放物線
ほうぶん #T35 邦文
ほうへい #T35*1 砲兵
ほうべい #T30*1 訪米
ほうべん #T35 方便
ほうほう #T35*255 方法
ほうほく #CN 豊北
ほうぼう #T35 方々
ほうぼく #T30 放牧
ほうまん #T05 放漫 豊満
ほうむ #R5*1 葬 #T35 法務
ほうむしょう #KK*1 法務省
ほうめい #T35*1 芳名
ほうめん #T35*20 方面 #T30 放免
ほうもつ #T35 宝物
ほうもん #T30*8 訪問
ほうもんぎ #T35 訪問着
ほうや #JCN 保谷
ほうやく #T30*1 邦訳
ほうゆう #T35 朋友
ほうよう #T30*16 包容 #T30*1 抱擁 #T35 法要
ほうらい #JCN 鳳来
ほうらく #T30 崩落
ほうりつ #T35*8 法律
ほうりゅう #T30 放流
ほうりゅうじ #CN 法隆寺
ほうりん #JN 法林
ほうれい #T35*1 法令
ほうれん #CN 法蓮
ほうれんそう #T35*1 法蓮草
ほうろう #T30*1 放浪
ほうわ #T30*2 飽和
ほえ #KSr*1 吠え
ほお #T35*1 頬
ほおべに #T35 頬紅
ほおん #T30*1 保温
ほか #T35*255 他 #T35*16 ほか #T35 外
ほかく #T30*1 捕獲 #T35 保革
ほかならぬ #RT 他ならぬ
ほかん #T30*25 補間 #T30*16 補完 #T30*4 保管
ほがらか #T05 朗らか
ほき #T30 補記
ほきゅう #T30*1 補給
ほきょう #T30*2 補強
ほきん #T30 保菌
ほく #KJ*2 北
ほくい #T35 北緯
ほくおう #CN 北欧
ほくぐん #T35 北軍
ほくげん #T35 北限
ほくさい #JNM 北斎
ほくしん #KK*1 北辰
ほくじょう #T30*1 北上
ほくせい #CN 北勢 #T35 北西
ほくたん #T35 北端
ほくだい #KK*2 北大
ほくだん #CN 北淡
ほくと #T35*1 北斗
ほくとう #T35 北東
ほくぶ #T35*1 北部 #CN 北部
ほくべい #CN 北米
ほくぼう #CN 北房
ほくよう #T35 北洋
ほくりく #T35*1 北陸 #CN 北陸
ほくりゅう #CN 北龍
ほくろ #T35*1 ほくろ
ほけつ #T35*1 補欠
ほけん #T35*12 保険 #T35*3 保健
ほけんきん #T35 保険金
ほけんふ #T35*1 保健婦
ほげい #T35 捕鯨
ほこ #R5r*3 誇 #KJ*3 鉾 #KJ 矛
ほこう #T30*1 歩行
ほこうそ #T35*1 補酵素
ほこた #CN 鉾田
ほこらし #KY 誇らし
ほこらしゅう #KYU 誇らしゅう
ほこり #T35 埃
ほころ #B5r 綻
ほころび #KSr 綻び
ほご #T30*18 保護 #T35*1 補語
ほさ #T30*2 補佐
ほさか #JNS 保坂
ほさき #T35 穂先
ほさく #T30 補作
ほし #T35*255 ★ #KY*239 欲し #T35*202 ☆ #KY*37 ほし #T35*10 星 #T35*1 ＊ #JN 星
ほしあい #JNS 星合
ほしが #R5*1 欲しが
ほしがき #T35 干柿
ほしぞら #T35*1 星空
ほしな #JCN 保科
ほしの #JCN 星野
ほしば #JNS 保志場
ほしゃく #T30*1 保釈
ほしゅ #T30*61 保守
ほしゅう #KYU*3 欲しゅう #KYU*2 ほしゅう #T30*2 補修 #T30 補習
ほしゅは #T35 保守派
ほしょう #T30*62 保証 #T30*3 保障 補償 #T35 歩哨 堡礁
ほしょうきん #T35*1 補償金 #T35 保証金
ほしょうにん #T35*1 保証人
ほしよう #T35*2 補仕様
ほじ #T30*36 保持 #R5 ほじ
ほじく #R5 ほじく
ほじゅう #T30 補充
ほじょ #T30*20 補助
ほじょきん #T35*1 補助金
ほすう #T35*5 補数
ほすてす #T35*1 ホステス
ほすと #T35*255 ホスト
ほせい #T30*8 補正 #T30 補整
ほせん #T35 保線
ほぜん #T30*1 保全
ほそ #KYmi*6 細 #R5*4 細 #KJ 細
ほそい #JNS 細井
ほそいり #JCN 細入
ほそう #KYU*1 細う #T30*1 舗装
ほそえ #JCN 細江
ほそえちょう #CN 細江町
ほそかわ #JNS 細川
ほそく #T30*25 捕捉 #T30*11 補足 #T35 補則
ほそだ #JCN 細田
ほそなが #KY*1 細長
ほそなごう #KYU*1 細長う
ほその #JCN 細野
ほそぼそ #F14*1 細々
ほそめ #KS*1 細め
ほそや #JNS 細谷
ほぞ #T35 ほぞ
ほぞん #T30*110 保存
ほたい #T35*1 補体
ほたか #JCN 穂高
ほたる #T35*1 火垂る 蛍
ほだい #T35*2 補題
ほちゅう #T35 補注
ほちょう #T35*1 歩調
ほちょうき #T35 補聴器
ほっ #SX*6 欲
ほっかいどう #CN 北海道
ほっき #T30*1 発起
ほっけー #T35*1 ホッケー
ほっさ #T35*2 発作
ほっせき #T30*1 発赤
ほっそく #T30*3 発足
ほった #JCN 堀田
ほったん #T35*1 発端
ほっちきす #T35*3 ホッチキス
ほっと #T05*5 ホット #T30*4 ほっと #F14*3 ほっと
ほっぷ #T30*3 ホップ
ほっぽう #T35 北方
ほつ #KJ 発
ほづみ #JCN 穂積
ほて #R5r 火照
ほてい #T35*1 保定
ほてる #T35*43 ホテル
ほてん #T30 補填
ほとけ #T35*1 仏
ほととぎす #T35 ホトトギス 不如帰
ほとほと #F14*1 ほとほと
ほとんど #T35*123 ほとんど #F14*83 ほとんど #F14*25 殆んど #F14*1 殆ど #T35*1 殆ど #F14 殆
ほど #T35*103 程 #K5r*6 解 #T15 ほど #T35 ほど
ほどう #T35*2 舗道 #T35*1 歩道 #T30 補導
ほどうきょう #T35*2 歩道橋
ほどがや #CN 保土ヶ谷
ほどこ #S5*14 施
ほどほど #T05*1 ほどほど #T35*1 ほどほど
ほない #CN 保内
ほなみ #JCN 穂波
ほね #T35*7 骨
ほねお #R5r 骨折
ほねぐみ #T35*1 骨組み
ほねつぎ #T35 骨接ぎ
ほねぶと #T05 骨太
ほねみ #T35 骨身
ほねやすめ #T30 骨休め
ほのお #T35*1 炎 焔
ほのか #T05*1 ほのか
ほのぼの #F04 ほのぼの
ほのるる #CN ホノルル
ほばら #JCN 保原
ほひつ #T30 補弼 補筆
ほびー #T35*1 ホビー
ほふ #R5 屠
ほふまん #JN ホフマン
ほへい #T35*1 歩兵
ほべつ #CN 穂別
ほほ #T35*4 ほほ #T35 頬
ほほえ #M5r*2 微笑
ほほえまし #KY ほほえまし
ほほえましゅう #KYU ほほえましゅう
ほぼ #F14*38 ほぼ #T35 保母
ほまれ #T35 誉れ
ほむら #T35 焔
ほめ #KS*13 褒め #KS*3 誉め
ほもげないず #T35*1 ホモゲナイズ
ほもじぇねーと #T35*1 ホモジェネート
ほゆう #T30*2 保有
ほよう #T30*4 保養
ほら #CJ*8 ほら #T35 洞
ほらー #T35*1 ホラー
ほらど #CN 洞戸
ほり #T35*6 堀 #T35*1 濠 #T35 壕 #JCN 堀
ほりい #JNS 堀井
ほりうち #JCN 堀内
ほりえ #JCN 堀江
ほりかわ #JCN 堀川
ほりがね #JCN 堀金
ほりぐち #JNS 堀口
ほりこし #JCN 堀越
ほりだ #S5r*1 掘り出
ほりの #JNS 堀野
ほりのうち #JCN 堀之内
ほりゅう #T30*13 保留
ほりょ #T35*1 捕虜
ほるい #T35 堡塁
ほるまりん #T35*1 ホルマリン
ほれ #KS 惚れ
ほれこ #M5 惚れ込
ほろ #B5*1 滅 #T35 幌
ほろいずみ #CN 幌泉
ほろかない #CN 幌加内
ほろぐらふぃ #T35*2 ホログラフィ
ほろのべ #CN 幌延
ほろび #KSr*1 亡び #KSr 滅び
ほろぼ #S5 滅ぼ
ほわいと #T05*4 ホワイト
ほん #JS*255 本 #T35*215 本 #KJ 叛 奔 #PRE 本 #KJ 翻
ほんあん #T35 本案
ほんい #T35*10 本位 #T35*2 本意
ほんか #T35 本科
ほんかい #T35 本会
ほんかいぎ #T35*1 本会議
ほんかく #T35*17 本格
ほんかくは #T35*1 本格派
ほんかわね #CN 本川根
ほんかん #T35*3 本館
ほんがわ #CN 本川
ほんがん #T35*1 本願
ほんき #T05*7 本気
ほんきゅう #T35*1 本給
ほんきょ #T35 本拠
ほんきょく #T35*2 本局
ほんきょち #T35 本拠地
ほんぎまり #T35*1 本決まり
ほんぎょう #T35*4 本業
ほんぐう #JCN 本宮
ほんけ #T35*2 本家
ほんけん #T35*4 本件
ほんこう #T35*4 本稿 #T35 本校
ほんこん #CN 香港
ほんごう #JCN 本郷
ほんごく #T35 本国
ほんさい #T35 本妻
ほんざん #T35 本山
ほんし #T35*6 本誌 #T35*3 本紙 #T35 本旨
ほんしき #T05 本式
ほんしつ #T35*16 本質
ほんしゃ #T35*22 本社
ほんしゅう #CN 本州
ほんしょ #T35*9 本書 #T35 本署
ほんしょう #T35*4 本章 #T35 本省 本性
ほんしょく #T35 本職
ほんしん #T35 本心
ほんじつ #T35*101 本日
ほんじゅらす #CN ホンジュラス
ほんじょう #CN 本匠 #JCN 本庄 本城 本荘
ほんじん #T35 本陣
ほんすう #T35*2 本数
ほんすじ #T35*1 本筋
ほんせき #T35 本籍
ほんせん #T35 本線 本船 本選
ほんそう #T30 奔走
ほんそく #T35 本則
ほんたい #T35*48 本体
ほんたく #T35 本宅
ほんたて #T35*1 本立
ほんだ #KK*2 ホンダ #JNS 本多 本田
ほんだい #T35*8 本題 #T35 本代
ほんだな #T35*4 本棚
ほんちょう #T35*1 本庁
ほんてん #T35*2 本店
ほんとう #T05*179 本当 #T30 奔騰
ほんど #JCN 本渡 #T35 本土
ほんどう #T35*1 本道 #T35 本堂
ほんどおり #T35*1 本通り
ほんにん #T35*32 本人
ほんね #T35*5 本音
ほんねん #T35*5 本年
ほんの #RT*6 ほんの
ほんのう #T35*1 本能
ほんば #T35 奔馬 本場
ほんばこ #T35*1 本箱
ほんばん #T35*4 本番
ほんぶ #T35*121 本部
ほんぶん #T35*19 本文
ほんべつ #CN 本別
ほんぺん #T35*1 本編
ほんぽう #T35*1 本邦 #T05 奔放
ほんま #JNS 本間
ほんまち #CN 本町
ほんまつてんとう #T00*1 本末転倒
ほんみょう #T35*4 本名
ほんむ #T35 本務
ほんめい #T35*1 本命
ほんもう #T05 本望
ほんもと #T35 本元
ほんもの #T35*5 本物
ほんや #T35*8 本屋
ほんやく #T30*26 翻訳
ほんやばけい #CN 本耶馬渓
ほんらい #T35*38 本来 #F14*19 本来
ほんりゅう #T35 奔流 本流
ほんりょう #T35*2 本領
ほんろう #T30 翻弄
ぼ #SUC*30 簿 #KJ*2 母 菩 #KJ*1 墓 暮 簿 #KJ 姥 牡 募 慕 #T35 戊 #KJ 模
ぼーい #T35*2 ボーイ
ぼーかる #T35*1 ボーカル
ぼーっと #F12*1 ぼーっと
ぼーと #T35*2 ボート
ぼーど #T35*83 ボード
ぼーなす #T35*4 ボーナス
ぼーぶぉわーる #JN ボーヴォワール
ぼーりんぐ #T30*3 ボーリング
ぼーる #T35*18 ボール
ぼーれーと #T35*1 ボーレート
ぼーん #T35*8 ボーン
ぼいこっと #T30*1 ボイコット
ぼいす #T35*8 ボイス
ぼいん #T35*2 母音 #T35*1 拇印
ぼう #PRE*52 某 #KJ*42 望 #KJ*13 房 #T35*11 棒 #T35*5 某 #KJ*3 忘 #KJ*2 坊 冒 #KJ*1 虻 呆 傍 帽 忙 暴 謀 鉾 防 #KJ 卯 茅 乏 亡 剖 妨 紡 肪 膨 貌 貿 牟
ぼういん #T30 暴飲
ぼうえい #T30*4 防衛
ぼうえき #T30*2 貿易 #T35 防疫
ぼうえん #T10 望遠 #T30 防炎
ぼうえんきょう #T35 望遠鏡
ぼうおん #T30 防音
ぼうか #T35*1 膨化 #T30 防火
ぼうかん #T30*1 傍観 #T35 暴漢 #T30 防寒
ぼうが #T35 忘我
ぼうがい #T30*2 妨害 #T35*1 望外
ぼうきゃく #T30*2 忘却
ぼうきょ #T35*1 暴挙
ぼうきょう #T35 望郷
ぼうぎゃく #T05 暴虐
ぼうぎょ #T30*2 防御
ぼうくう #T35*1 防空
ぼうくうごう #T35 防空壕
ぼうけい #T35*1 傍系
ぼうけん #T30*10 冒険 #T30*1 剖検
ぼうげん #T30*1 暴言
ぼうこう #T30 暴行
ぼうご #T30*1 防護
ぼうさい #T30*2 防災
ぼうさつ #T30 忙殺 謀殺
ぼうし #T30*8 防止 #T35*5 帽子 #T35*4 某氏
ぼうしゃ #T35*1 某社
ぼうしゅ #T35*1 芒種
ぼうしゅう #T30 防臭
ぼうしゅうざい #T35 防臭剤
ぼうしょう #T30*1 傍証
ぼうしょく #T30 暴食
ぼうじ #T30*3 防磁
ぼうじゅ #T30*2 傍受
ぼうじょう #T35*1 棒状
ぼうすい #T30*1 防水
ぼうず #T35*2 坊主
ぼうぜん #F02*1 呆然
ぼうそう #T30*7 暴走 #CN 房総
ぼうだい #T05*13 膨大
ぼうだち #T35 棒立ち
ぼうだん #T30 防弾
ぼうちゅうざい #T35 防虫剤
ぼうちょう #T30*2 傍聴 膨張
ぼうとう #T35*1 冒頭 #T30 暴騰
ぼうどう #T35*1 暴動
ぼうねんかい #T35*6 忘年会
ぼうのつ #CN 坊津
ぼうはつ #T30 暴発
ぼうはてい #T35 防波堤
ぼうばく #F02 茫漠
ぼうび #T30*1 防備
ぼうふ #T30 防腐
ぼうふう #T35 暴風 防風
ぼうふうう #T35 暴風雨
ぼうへき #T35 防壁
ぼうめい #T30*2 亡命
ぼうや #T35*1 坊や
ぼうよう #F00*1 芒洋
ぼうらく #T30*1 暴落
ぼうり #T35 暴利
ぼうりゃく #T35 謀略
ぼうりょく #T35*1 暴力
ぼうる #T35*3 ボウル
ぼうれい #T35*1 亡霊
ぼうろん #T30*1 暴論
ぼか #S5r*2 ぼか #S5r*1 暈
ぼかん #T35 母艦
ぼき #T35*1 簿記
ぼきん #T30*1 募金
ぼく #T35*110 僕 #KJ*1 朴 牧 #T30 卜 #KJ 墨 撲 #JN 朴 #KJ 睦 穆 木
ぼくさつ #T30 撲殺
ぼくし #T35*1 牧師
ぼくしゅ #T30 墨守
ぼくしんぐ #T35*1 ボクシング
ぼくじゅう #T35 墨汁
ぼくじょう #T35 牧場
ぼくそう #T35 牧草
ぼくたち #T35*3 僕達
ぼくちく #T35*1 牧畜
ぼくとう #T35 木刀
ぼくめつ #T30 撲滅
ぼくら #T35*3 僕等 #T35 僕ら
ぼけ #KSr*3 惚け
ぼけつ #T35*1 墓穴
ぼこう #T35 母校 母港
ぼこく #T35*1 母国
ぼご #T35 母語
ぼさつ #T35*1 菩薩
ぼし #T35*2 母子
ぼしゅう #T30*23 募集
ぼしゅうだん #T35*1 母集団
ぼしょ #T35 墓所
ぼす #T35*1 ボス
ぼすとん #CN ボストン
ぼせい #T35 母性
ぼせん #T35 母船
ぼたい #T35*2 母体
ぼたもち #T35 牡丹餅
ぼたん #T35*110 ボタン #T35*8 牡丹 #T35 釦
ぼだい #T35 菩提
ぼち #T35*2 墓地
ぼちぼち #F04*3 ぼちぼち
ぼっ #S5*2 没 #T30*2 没
ぼっき #T30 勃起
ぼっくす #T35*22 ボックス
ぼっこう #T30 勃興
ぼっこん #T35 墨痕
ぼっしゅう #T30 没収
ぼっちゃん #T35*1 坊っちゃん
ぼっとう #T30 没頭
ぼっぱつ #T30 勃発
ぼつ #KJ*1 勃 #T35*1 没 #PRE 没
ぼつこうしょう #T05 没交渉
ぼつこせい #T05 没個性
ぼつにゅう #T30 没入
ぼつぼつ #F06 ぼつぼつ
ぼつらく #T30 没落
ぼでぃ #T35*3 ボディ
ぼとむ #T35*1 ボトム
ぼとむあっぷ #T05*3 ボトムアップ
ぼとる #T35*1 ボトル
ぼとるねっく #T35*2 ボトルネック
ぼにゅう #T35 母乳
ぼね #T35 骨
ぼば #T35 牡馬
ぼひ #T35 墓碑
ぼや #K5r*1 ぼや
ぼら #T35*1 鯔 鰡
ぼらんてぃあ #T35*4 ボランティア
ぼりゅーむ #T35*5 ボリューム
ぼるしち #T35 ボルシチ
ぼると #T35*2 ボルト
ぼれい #T35 牡蛎
ぼろ #KY ぼろ #T35 ぼろ #KY ボロ #T35 ボロ
ぼん #JS*11 本 #T35*6 盆 #KJ*1 凡 梵
ぼんおどり #T35 盆踊り
ぼんくれ #T35 盆暮れ
ぼんさい #T35 凡才 盆栽
ぼんじん #T35 凡人
ぼんぞく #T35*1 凡俗
ぼんち #T35*1 盆地
ぼんでぃんぐ #T30*1 ボンディング
ぼんど #T35*1 ボンド
ぼんのう #T35*1 煩悩
ぼんぷ #T35 凡夫
ぼんやり #F04 ぼんやり
ぼんよう #T05*1 凡庸
ぽ #JS*9 歩
ぽーかー #T35 ポーカー
ぽーず #T30*1 ポーズ
ぽーたびりてぃ #T35*7 ポータビリティ
ぽーたぶる #T15*4 ポータブル
ぽーと #T30*70 ポート
ぽーりんぐ #T30*10 ポーリング
ぽーる #T30*4 ポール
ぽい #F06*1 ぽい ポイ
ぽいんた #T35*183 ポインタ
ぽいんたー #T35*2 ポインター
ぽいんてぃんぐ #T30*3 ポインティング
ぽいんと #T30*34 ポイント
ぽう #JS*2 包 報
ぽけっと #T35*1 ポケット
ぽじしょん #T35*8 ポジション
ぽすたー #T35*2 ポスター
ぽすと #T30*95 ポスト
ぽたーじゅ #T35 ポタージュ
ぽっと #T35*4 ポット
ぽっぷ #T00*2 ポップ
ぽっぷあっぷ #T30*15 ポップアップ
ぽっぷす #T35*1 ポップス
ぽてと #T35*2 ポテト
ぽとふ #T35 ポトフ
ぽぴゅらー #T05*1 ポピュラー
ぽり #PRE*1 ポリ
ぽりごん #T35*1 ポリゴン
ぽりし #T35*2 ポリシ
ぽりしー #T35*3 ポリシー
ぽりぺぷちどさ #T35*1 ポリペプチド鎖
ぽりもるふぃずむ #T35*2 ポリモルフィズム
ぽるとがる #CN ポルトガル
ぽるの #T35 ポルノ
ぽん #JS*9 本
ぽんとちょう #CN 先斗町
ぽんど #T35*1 £ #JS*1 ポンド
ま #T5r*255 待 #T35*87 間 #K5*80 巻 #W5r*34 舞 #S5*33 増 #T35*17 真 #T35*12 ま #T35*6 魔 #K5*4 ま #T35*2 摩 #KJ*2 麻 #K5*1 撒 蒔 #K5 捲 #KJ 磨 #SX 磨
まーか #T35*1 マーカ
まーく #T30*38 マーク
まーけっと #T35 マーケット
まーけてぃんぐ #T35*1 マーケティング
まーじ #T30*31 マージ
まーじゃん #T35*2 マージャン
まーじん #T35*6 マージン
まーふぃー #JN マーフィー
まーらー #JN マーラー
まあ #CJ*87 まあ #F14*80 まあ #T35*80 まあ
まあい #T35 間合い
まあじゃん #T35*5 麻雀
まあたらし #KY*1 真新し
まあたらしゅう #KYU*1 真新しゅう
まい #JS*123 枚 #R5r*14 まい #R5r*5 参 #T35*5 舞 #KJ*5 米 #T35*3 毎 #R5r*1 詣 #PRE マイ #KJ 埋 妹 昧 #T35 枚 #NNPRE 毎
まいあさ #T35*4 毎朝
まいかー #T35 マイカー
まいかい #T35*7 毎回
まいき #T35*1 毎期
まいきょ #T30 枚挙
まいく #T35*3 マイク
まいくろ #T35*24 マイクロ
まいくろは #T35*1 マイクロ波
まいぐれーしょん #T35*13 マイグレーション
まいぐれーと #SX*8 マイグレート
まいける #JN マイケル
まいこ #M5*1 舞い込 #T35 舞子
まいこぷらずま #T35*1 マイコプラズマ
まいこん #T35*4 マイコン
まいご #T35*2 迷子 #T35*1 まいご
まいさか #JCN 舞阪
まいしゅう #T35*4 毎週
まいしん #T30*1 邁進
まいじ #T35*2 毎時
まいすう #T35*9 枚数
まいせつ #T30 埋設
まいそう #T30*1 埋葬
まいぞう #T30*1 埋蔵
まいつき #T35*6 毎月
まいづる #JCN 舞鶴
まいとし #T35*5 毎年
まいど #T35*14 毎度
まいなー #T05*9 マイナー
まいなす #T10*4 マイナス #T35*1 −
まいにち #T35*39 毎日
まいねん #T35*1 毎年
まいはら #JCN 米原
まいばら #JCN 米原
まいばん #T35*3 毎晩
まいぺーす #T15 マイペース
まいほーむ #T35*1 マイホーム
まいぼつ #T30*1 埋没
まいまい #T35 毎々
まいよ #T35 毎夜
まいる #T35*3 マイル #JS マイル 哩
まうえ #T35*1 真上
まうす #T35*93 マウス
まうんてん #T35*1 マウンテン
まうんと #T30*161 マウント
まえ #CNSUC1*255 前 #T35*37 前 #T35*1 まえ #JSSUC 前 #PRE 前 #SUC 前
まえあし #T35 前足
まえうり #T30*1 前売
まえおき #T30*1 前置き
まえかけ #T35 前掛け
まえかわ #JNS 前川
まえがき #T30*2 前書き
まえがし #T30 前貸し
まえがり #T30 前借り
まえがわ #T35 前側
まえきん #T35 前金
まえさわ #JCN 前沢
まえざわ #JNS 前沢
まえじま #JNS 前島 前嶋
まえだ #JNS 前田
まえつえ #JCN 前津江
まえの #JNS 前野
まえのめり #T35 前のめり
まえはら #JNS 前原
まえば #T35 前歯
まえばし #JCN 前橋
まえばらい #T30 前払い
まえばる #JCN 前原
まえぶれ #T30 前触れ
まえまえ #T35*2 前々
まえむき #T05*2 前向き
まえもって #F14*2 前もって
まえやま #JNS 前山
まか #S5*4 任 #R5*1 まか #T35*1 摩可 #S5 負か
まかい #T35 魔界
まかせ #KS*26 任せ #KS*2 まかせ #KS 委せ
まかな #W5r*1 賄
まかふしぎ #T18 摩訶不思議
まかべ #JCN 真壁
まが #R5r*5 曲が #W5r*1 紛 #R5r 曲
まがお #T15 真顔
まがじん #T35*4 マガジン
まがまがし #KY 凶々し #KYU 凶々しゅう
まがり #T30*6 間借り
まがりかど #T35 曲り角
まき #T35*1 真木 薪 #KK*1 馬来 #T35*1 牧 #JNS 巻 #KJ 蒔 #JNM 真紀 真樹 #JNS 牧 #KJ 槙
まきあげ #KSr 巻き上げ
まきえ #T35*1 蒔絵
まきおか #JCN 牧丘
まきかえし #T30*1 巻き返し
まきがみ #T35 巻紙
まきこ #M5r*3 巻き込 #JNM 満喜子
まきしま #JNS 槙島
まきじゃく #T35 巻尺
まきぞの #JCN 牧園
まきた #T35 真北 #JNS 牧田
まきつ #K5r 巻き付
まきつけ #KSr 巻き付け
まきの #JCN マキノ #JNS 牧野
まきのうち #JNS 牧之内
まきば #T35 牧場
まきもの #T35*1 巻物
まぎら #S5 紛ら
まぎらわし #KY*4 紛らわし
まぎらわしゅう #KYU*1 紛らわしゅう
まぎれ #KSr*2 紛れ
まぎれこ #M5*2 紛れ込
まぎわ #T35*2 間際
まく #T35*8 幕 #R5r まく 捲く #T35 膜
まくあけ #T35 幕開け
まくぎれ #T35 幕切れ
まくせる #KK*1 マクセル
まくどなるど #KK*3 マクドナルド
まくはり #CN 幕張
まくべつ #CN 幕別
まくら #T35*4 枕
まくらぎ #T35 枕木
まくらざき #JCN 枕崎
まくらのそうし #KK 枕草子
まくろ #T05*164 マクロ
まぐりっと #JN マグリット
まぐろ #T35*2 鮪
まぐろうひる #KK*1 マグロウヒル
まけ #KSr*25 負け #KSr*8 敗け
まけこ #S5r 負け越
まげ #KSr*3 曲げ #R5 枉げ
まこ #JNM 麻子
まこと #T35*15 誠 #T35*10 真 #T35*1 まこと #JNM 信 真 誠 眞
まことしやか #T05 まことしやか
まご #T35*4 孫
まごい #T35 真鯉
まごころ #T35 真心
まごつ #K5r まごつ
まごまご #SX*1 まごまご
まさ #KJ*6 雅 #KJ*5 正 #R5r*3 勝 #KJ*2 昌 #KJ*1 匡 政 #JNM まさ #JN マサ #KJ 柾 #R5r 優
まさあき #JNM 政昭 正昭 正明
まさいち #JNM 政一 正一
まさえ #JNM 雅江 昌枝
まさお #JNM 雅夫 雅雄 昌男 政男 政夫 政雄 正男 正夫 正雄
まさか #F14*5 まさか #T36 まさか
まさかず #JNM 正一 正和
まさかつ #JNM 政勝 正勝
まさかに #F14 まさかに
まさき #T35*7 正木 #JNM 雅樹 #JCN 松前 真幸 #JNM 正毅 正紀 正記 正樹 正木 眞幸
まさきち #JNM 政吉
まさぐ #R5r まさぐ
まさこ #JNM まさ子 雅子 昌子 晶子 真紗子 政子 正子 誠子
まさし #JNM 雅史 昌 昌嗣 政志 正 正司 正志
まさしく #F14*3 正しく
まさじ #JNM 政次 政治 正次 正治
まさたか #JNM 昌孝 正孝
まさちゅーせっつ #CN マサチューセッツ
まさつ #T30*1 摩擦
まさつぐ #JNM 雅紹 仁継
まさと #JNM 雅人 真人 政人 正人
まさとし #JNM 雅俊 雅敏 昌利 政利 正俊 正敏 正利
まさなお #JNS 正直
まさに #F14*13 まさに #F14*1 正に
まさのぶ #JNM 雅信 雅宣 昌信 政信 正信
まさのり #JNM 雅典 正紀 正則 正徳
まさはる #JNM 雅春 政春 正治 正春
まさひこ #JNM 雅彦 昌彦 正彦
まさひろ #JNM 雅博 雅裕 匡弘 昌宏 昌弘 政弘 正広 正弘 正浩 正博 正洋
まさふみ #JNM 将文 正文
まさみ #JNM 雅美 昌己 政美 正美 正巳
まさみち #JNM 正道
まさみつ #JNM 正光
まさめ #T35 正目
まさや #JNM 雅哉 雅也 昌也
まさやす #JNM 雅康 雅保 正康
まさゆき #JNM 雅之 昌幸 昌之 政幸 政行 正幸 正行 正之 誠之
まさゆめ #T35*2 正夢
まさよし #JNM 政義 正義 正元 正好
まさる #JNM 勝 優
まざ #R5*9 混ざ #R5*1 交ざ
まざまざ #F06 まざまざ
まし #T05*17 まし
ましき #JCN 益城
ましけ #JCN 増毛
ましこ #JCN 益子 #JNS 増子
ました #T35*2 真下 #JCN 益田
まして #F14*13 まして
ましてや #F14*2 ましてや
ましゃく #T35*1 間尺
ましょうめん #T35*1 真正面
ましん #T35*255 マシン #T35*1 麻疹
まじ #R5r*4 混じ #R5r*3 交じ
まじえ #KS*4 交え
まじっく #T35*8 マジック
まじま #JNS 間島
まじまじ #F06 まじまじ
まじめ #T05*18 真面目 #T05 まじめ
まじゅつ #T35 魔術
まじょ #T35*1 魔女
まじわ #R5r*2 交わ
ます #T35*5 鱒 #T35*4 ます #T35*3 マス #KJ*3 桝 #T35 益 升 #KJ 舛
ますい #JNS 増井 #T30 麻酔
ますお #JCN 増尾
ますく #T30*12 マスク
ますこ #JNM 益子 増子
ますこみ #T35*2 マスコミ
ますぞえ #JNS 舛添
ますた #T30*36 マスタ
ますたー #T30*4 マスター
ますたーべーしょん #T30*1 マスターベーション
ますたに #JNS 升谷
ますだ #JCN 益田 増田
ますぶち #JNS 増淵
ますほ #JCN 増穂
ますます #F14*5 ますます #F14*1 益々 #T36 ますます 益々
ますみ #JN 真澄 #JNM 満寿美
ますめ #T35*1 桝目
ますやま #JNS 増山
まず #F14*64 まず #KY*31 まず #F14*2 先ず
まずう #KYU*2 まずう
まずし #KY*2 貧し
まずしゅう #KYU*1 貧しゅう
まずまず #T15*1 まずまず
まぜ #KSr*8 混ぜ #JCN 馬瀬
まぜあわせ #KSr 混ぜ合わせ
まぜらん #JN マゼラン
また #CJ*84 また #T35*40 又 #T35*33 股 #G5r*10 跨 #KJ*3 俣 #KJ 亦
またが #R5*19 跨 #R5 またが
またぎ #T35 マタギ
またぎき #T30 又聞き
またしても #CJ またしても
またた #K5r*3 瞬
または #CJ*107 または
またま #JCN 真玉
またまた #F14*7 またまた
またも #F14*1 またも
またもや #F14*2 またもや
まだ #F14*149 まだ #F14*7 未だ
まだしも #F14*1 まだしも
まだまだ #F14*13 まだまだ
まだむ #T35*1 マダム
まだら #T15 斑
まだらお #JCN 斑尾
まだるっこし #KY まだるっこし
まだるっこしゅう #KYU まだるっこしゅう
まち #T35*94 町 #T35*42 街 #T35*6 待 #CNSUC1*5 町 #JNM 万智
まちあ #W5r 待ち合
まちあいしつ #T35*1 待合室
まちかど #T35*1 街角
まちが #W5r*213 間違
まちじゅう #T35*1 町中
まちだ #JCN 町田
まちぢゅう #T35 町中
まちでら #JNS 待寺
まちどおし #KY*2 待ち遠し
まちどおしゅう #KYU*1 待ち遠しゅう
まちなか #T35*4 町中
まちなみ #T35 町並み
まちのぞ #M5*1 待ち望
まちまち #T05*3 まちまち
まちや #JNS 町谷
まぢか #T05*1 まぢか #KY*1 間近 #T05*1 間近
まっか #T05*3 真っ赤
まっかーしー #JN マッカーシー
まっかり #CN 真狩
まっき #T35*1 末期
まっきんとっしゅ #KK*3 マッキントッシュ
まっく #KK マック
まっくす #JN マックス
まっくら #T05 真っ暗
まっくろ #KY*1 真っ黒 #T05*1 真っ黒
まっこう #T35 真向 抹香
まっさいちゅう #T05*2 真最中
まっさお #T05*1 真っ青
まっさき #T35*3 真っ先
まっさつ #T30*1 抹殺
まっしょう #T30*7 抹消 #T35 末梢
まっしろ #KY*2 真っ白 #T05*2 真っ白
まっすぐ #T05*5 まっすぐ #T05*1 真直
まっせ #T35*1 末世
まっせき #T35 末席
まったく #F14*176 全く #T18 全く
まったん #T35*3 末端
まっち #T30*16 マッチ
まっちゃ #T35*2 抹茶
まっちんぐ #T30*7 マッチング
まっとう #T30*4 全う #JCN 松任 #T05 真っ当
まっぴつ #T35 末筆
まっぴら #T15*1 まっぴら
まっぴるま #T35*2 まっぴるま #T35 真っ昼間
まっぴんぐ #T30*10 マッピング
まっぷ #T30*43 マップ
まっぷたつ #T05 まっぷたつ 真っ二つ
まつ #SUC*130 末 #T35*93 松 #R5r*13 祭 #R5*1 奉 #T35*1 末 #KJ 抹 沫
まつい #JNS 松井
まついだ #JCN 松井田
まつうら #JCN 松浦
まつえ #JCN 松江
まつえい #T35*2 末裔
まつお #JNM 松男 #JCN 松尾 #JNS 松夫 #JNM 松雄
まつおか #JCN 松岡
まつかた #JNS 松方
まつかわ #JCN 松川
まつき #JNS 松木
まつげ #T35*1 まつげ #T35 睫毛
まつご #T35 末期
まつさか #JNS 松坂 #JCN 松阪
まつさき #JNS 松崎
まつさわ #JNS 松沢
まつざか #JNS 松坂
まつざき #JCN 松崎
まつざわ #JCN 松沢
まつしげ #JNS 松重 #JCN 松茂
まつした #JNS 松下
まつしま #JCN 松島 #JNS 松嶋
まつしろ #JNS 松代
まつじつ #T35*2 末日
まつじめ #T35 末締め
まつたけ #T35 松茸
まつたろう #JNM 松太郎
まつだ #KK*4 マツダ #JCN 松田
まつだい #JCN 松代 #T35 末代
まつとう #JCN 松任
まつとうや #JNS 松任谷
まつど #JCN 松戸
まつなが #JCN 松永
まつの #JCN 松野
まつのやま #JCN 松之山
まつば #JCN 松葉
まつばせ #JCN 松橋
まつばづえ #T35*1 松葉杖
まつばら #T35*7 松原 #JCN 松原
まつび #T35*17 末尾
まつぶし #JCN 松伏
まつぼっくり #T35 松ぼっくり
まつまえ #JCN 松前
まつまる #JNS 松丸
まつむら #JNS 松村
まつもと #JCN 松元 松本
まつやま #T35*5 松山 #JCN 松山
まつり #T35*10 まつり #T35*2 祭
まつりごと #T35*1 政
まつろ #T35*1 末路
まつわ #R5*3 纏わ
まで #T35*17 まで #T35*15 迄
まと #T35*11 的 #W5r 纏
まとい #T35 纏
まとば #JCN 的場
まとま #R5r*14 まとま
まとめ #KSr*80 まとめ
まとも #T05*12 まとも #T05*4 マトモ
まとりくす #KK*2 マトリクス
まとりっくす #T35*1 マトリックス
まど #T35*17 窓 #W5r 惑
まどう #T35 魔道
まどお #T05 間遠
まどがわ #T35*1 窓側
まどぎわ #T35*1 窓際
まどぐち #T35*39 窓口
まどべ #T35 窓辺
まどり #T35 間取り
まどろ #M5r*1 まどろ
まどろっこし #KY まどろっこし
まどろっこしゅう #KYU まどろっこしゅう
まな #B5r*10 学
まなー #T35*5 マナー
まないた #T35 俎
まなこ #T35 眼
まなざし #T35 眼差し
まなつ #T35*1 真夏
まなつる #JCN 真鶴
まなでし #T35*1 愛弟子
まなびと #R5r 学び取
まなぶ #JNM 学
まにあ #W5*36 間に合 #T35*1 マニア
まにあっく #T05*2 マニアック
まにし #T35 真西
まにぴゅれーしょん #T30*1 マニピュレーション
まにゅある #T35*255 マニュアル
まにわ #JCN 真庭
まぬかれ #KS 免れ
まぬがれ #KS*1 免れ
まぬけ #T05*3 間抜け
まね #K5r*8 招 #KSr*5 真似 #T30*4 真似
まねーじゃ #T35*36 マネージャ
まねきん #T35*1 マネキン
まねじめんと #T30*4 マネジメント
まねじゃー #T35*1 マネジャー
まの #JNS 間野 #JCN 真野
まのび #T30*1 間延び
まはら #JNS 麻原
まばた #K5r*1 瞬
まばたき #T30*1 まばたき #T30 瞬き
まばら #T05 疎ら
まひ #T30*2 麻痺
まひがし #T35 真東
まひる #T35 真昼
まび #K5r*2 間引
まふゆ #T35*1 真冬
まぶ #S5 まぶ
まぶか #T05 目深
まぶし #KY*1 まぶし
まぶしゅう #KYU*1 まぶしゅう
まぶた #T35*1 まぶた
まほう #T35*9 魔法
まほうじん #T35 魔方陣
まぼろし #T35*2 幻
まま #T35*86 まま #T35*19 ママ #T35*1 侭 #F14 間々
ままあ #R5 ままあ
ままはは #T35 継母
まみ #JNM まみ
まみこ #JNM 麻巳子
まみず #T35 真水
まみなみ #T35 真南
まみれ #SUC*1 まみれ #KSr まみれ 塗れ
まむろがわ #JCN 真室川
まめ #T35*4 豆 #T05*2 まめ #T05*1 マメ
まめきゅう #T35 豆球
まめだ #JNS 豆田
まめつ #T30 摩滅
まめつぶ #T35 豆粒
まも #R5r*23 守 #R5r*1 まも 護
まもう #T30 摩耗
まもなく #CJ まもなく
まもの #T35 魔物
まもる #JNM 衛 守
まや #JCN 摩耶
まやく #T35 麻薬
まゆ #T35*1 眉 #T35 繭
まゆげ #T35*1 まゆげ 眉毛
まゆずみ #T35*6 黛
まゆみ #JCN 真弓 #JNM 真由美
まよ #W5r*14 迷 #JNM 麻世
まよなか #T35*1 真夜中
まらそん #T35*2 マラソン
まらるめ #JN マラルメ
まり #T35*3 掬 #JNM まり #T35 鞠 #JNM 真理 麻理 麻里
まりあ #JN マリア
まりお #JNM 真理雄
まりこ #JNM 真理子 麻理子 麻里子
まりな #JNM 満里奈
まりょく #T35*1 魔力
まる #T35*255 ○ #T35*191 ◎ #T35*110 丸 #KYmi*94 丸 #T35*31 ● #T35*2 ◯ マル
まるい #KK*2 丸井
まるいけ #JCN 丸池
まるいち #JNS 丸一
まるお #JCN 丸尾
まるおか #JCN 丸岡
まるがめ #JCN 丸亀
まるき #T35 丸木
まるきしずむ #T35 マルキシズム
まるく #T35*1 マルク
まるこ #JCN 丸子
まるこふ #JN マルコフ
まるごと #F14*2 まるごと
まるせっぷ #CN 丸瀬布
まるぜん #KK*1 丸善
まるぞこ #T15 丸底
まるた #T35*1 丸太
まるたに #JNS 丸谷
まるだし #T10 丸出し
まるち #PRE*57 マルチ
まるちきゃすと #T30*3 マルチキャスト
まるちぷれくさ #T35*1 マルチプレクサ
まるっきり #F14 まるっきり
まるで #F14*11 まるで
まるはだか #T05 丸裸
まるばやし #JNS 丸林
まるまる #T30*2 まるまる #T30*1 丸々
まるみ #JNS 円美
まるめ #KS*2 丸め
まるめこ #M5r 丸め込
まるもり #JCN 丸森
まるやま #JCN 円山 丸山
まるゆう #T35 マル優
まれ #T05*3 まれ 稀 #T05*1 稀れ #T05 希
まろ #T35*2 麿
まろうど #T35 客人
まろやか #T05 円やか
まわ #R5r*105 回 #S5r*94 回 #R5r*23 まわ #S5r*16 まわ #R5r*3 廻 #S5*3 廻
まわたり #JNS 馬渡
まわり #T35*19 周り #T35*7 廻り
まわりこ #M5r 回り込
まわりみち #T30*1 回り道
まん #NN*64 万 #KJ*59 満 #SUC*8 マン #NN*3 萬 #KJ*2 饅 #KJ*1 蔓 瞞 #KJ 鰻 幡 慢 漫
まんいち #F14*3 万一 #T35*2 万一
まんいん #T35*1 満員
まんえつ #T30 満悦
まんえん #T30*2 蔓延
まんかい #T30 満開
まんかんしょく #T15 満艦飾
まんが #T35*7 漫画
まんがいち #T35 万が一
まんがん #T35*1 満貫 #T35 マンガン 満願
まんがんぜんせき #T35 満漢全席
まんき #T35*1 満期
まんきつ #T30*1 満喫
まんげきょう #T35*1 万華鏡
まんげつ #T35*1 満月
まんさい #T30*1 満載
まんざい #T35*1 漫才
まんしつ #T35*2 満室
まんしゃ #T35 満車
まんしゅう #JCN 満州
まんしょん #T35*8 マンション
まんしん #T35*1 万進 #T30 慢心 #T35 満身
まんじ #T35*1 卍
まんじどもえ #T35 卍巴
まんじゅう #T35*3 まんじゅう #T35*2 饅頭
まんじょう #T35 満場
まんすい #T30 満水
まんせい #T35*1 慢性
まんせき #T15 満席
まんぜん #F00 漫然
まんぞく #T00*29 満足
まんだら #T35 マンダラ 曼荼羅
まんちょう #T35 満潮
まんてん #T35*1 満点
まんなか #T35*18 真中 #T35*2 真ん中
まんにん #T35*1 万人
まんねんひつ #T35 万年筆
まんのう #JCN 満濃
まんば #JCN 万場
まんめん #T35 満面
まんゆう #T30 漫遊
まんよう #T35*3 万葉
まんようしゅう #KK 万葉集
まんりょう #T30 満了
み #KS*255 見 #T35*138 未 #T35*72 身 #T5*70 満 #PRE*68 未 #KS*47 み #KJ*23 美 #T35*22 巳 #KS*10 観 #KJ*5 三 #T35*3 実 #T30*3 魅 #KJ*2 己 #KS*2 視 #KS*1 看 #KJ*1 御 #KS*1 診 #KJ*1 味 #KJ 見 深 壬 未 箕
みーてぃんぐ #T30*56 ミーティング
みーはー #T05*4 ミーハー
みーり #JN ミーリ
みあ #W5r*1 見合
みあげ #KS*1 見上げ
みあさ #JCN 美麻
みあた #R5*8 見当た
みあま #JNS 三甘
みあわせ #KSr 見合わせ
みい #R5*1 見入 #JCN 三井
みいけ #JCN 三池
みいだ #S5*6 見出
みいら #T35 ミイラ
みいらく #CN 三井楽
みうけ #KS*2 見受け
みうごき #T30*1 身動き
みうしな #W5*1 見失
みうち #T35*3 身内
みうら #JCN 三浦
みえ #KS*150 見え #KS*1 みえ #T35*1 見栄 #T35 見得 #JCN 三重
みえかくれ #T30*1 見えかくれ
みえこ #JNM 三枝子 美恵子 美枝子
みえりん #T35*1 ミエリン
みお #JCN 三尾
みおく #R5r*7 見送
みおさめ #KSr 見納め
みおと #S5r*3 見落と #S5r*1 見落
みおとり #T30*1 見劣り
みおぼえ #T35*3 見覚え
みおも #T05*1 身重
みおろ #S5 見下ろ
みか #JNS 実可 #JNM 美香
みかい #T05*3 未開
みかいけつ #T05*3 未解決
みかえ #R5r 見返 #S5 見返
みかえり #T35 見返り
みかぎ #R5*2 見限
みかく #T35*2 味覚
みかくてい #T15 未確定
みかくにん #T05*5 未確認
みかけ #KSr*7 見掛け #KSr*6 見かけ #T35*1 みかけ
みかさ #JCN 三笠
みかた #T35*6 見方 #T30*4 味方 #JCN 三方 美方
みかつき #CN 三日月
みかづき #T35*1 三日月 #CN 三日月
みかど #T35 帝
みかね #KS 見兼ね
みかみ #JCN 三上
みかめ #JCN 三瓶
みかも #JCN 三加茂 美甘
みかわ #S5 見交わ #JCN 三加和 三河 三川 美川
みかん #T15*2 未完 #T35*1 蜜柑 #T35 みかん 未刊
みかんせい #T05*1 未完成
みが #K5r*1 磨 #K5r 研
みがって #T05*1 身勝手
みがまえ #KSr*1 身構え
みがら #T35 身柄
みがる #T05 身軽
みがわり #T35 身代り
みき #T35*3 幹 #R5r*1 見切 #JNS 三喜 #JCN 三木 #JN 美樹
みきお #JNM 幹男 幹夫 幹雄
みきき #T30 見聞き
みきこ #JNM 実希子 美貴子 美樹子
みきひこ #JN 幹彦
みきや #JNS 御喜家
みきり #T35 見切り
みきわめ #KSr 見極め
みぎ #T30*105 右 #T35*47 右
みぎあし #T35 右足
みぎうえ #T35*2 右上
みぎうで #T35*1 右腕
みぎがわ #T35*20 右側
みぎした #T35*5 右下
みぎて #T35*1 右手
みぎはし #T35*23 右端
みぎめ #T35*1 右目
みぎや #T35*43 →
みぎわ #T35 汀
みくだ #S5r 見下
みくだりはん #T35 三下り半
みくに #JCN 三国
みくび #R5*10 見くび #R5 見縊
みくも #JCN 三雲
みくらしま #JCN 御蔵島
みくらべ #KSr*2 見比べ
みくろ #T05*1 ミクロ
みぐるし #KY*1 見苦し
みぐるしゅう #KYU*1 見苦しゅう
みけ #T35*3 三毛
みけつ #T15*3 未決
みけん #T35*2 眉間
みこ #M5r*7 見込 #S5*2 見越 #T35 御子
みこし #T35 御輿
みこと #T35 尊
みことのり #T35 詔 勅
みこん #T15 未婚
みごたえ #T35*10 見ごたえ 見応え
みごと #T05*8 見事
みごろし #T35 見殺し
みさ #T35 ミサ
みさい #T15 未済
みさいる #T35*2 ミサイル
みさお #JNM 操 #T35 操
みさか #JCN 御坂
みさかい #T35 見境
みさき #T35*1 岬 #JCN 三崎 #JNS 岬
みさくぼ #CN 水窪
みさげ #KS 見下げ
みささ #JCN 三朝
みさだめ #KSr 見定め
みさと #JCN 三郷 美郷 美里 箕郷
みさわ #JCN 三沢
みし #R5r*4 見知
みしがん #CN ミシガン
みしま #JCN 三島 #JNS 三嶋
みしゅう #T15 未収
みしゅく #JCN 三宿
みしょう #JCN 御荘
みしよう #T15 未使用
みしん #T35*5 ミシン
みじか #KY*30 短
みじこう #KYU*2 短う
みじたく #T30 身支度
みじめ #T05 みじめ 惨め
みじゅく #T05*2 未熟
みじん #T35 微塵
みす #T30*46 ミス #JNS 三栖 簾
みすい #T15 未遂
みすえ #KSr 見据え
みすぎ #JCN 美杉
みすご #S5r 見過ご
みすて #KSr*2 見捨て
みすてり #T35 ミステリ
みすてりあす #T05 ミステリアス
みすぼらし #KY みすぼらし
みすぼらしゅう #KYU みすぼらしゅう
みすみ #JCN 三角 三隅
みすみす #F14 みすみす
みず #T35*19 水 #T35 瑞
みずあび #T30 水浴び
みずあらい #T30 水洗い
みずいらず #T15 水入らず
みずいろ #T15*2 水色
みずうみ #T35*1 湖
みずかき #T35 水掻き
みずかさ #T35 水かさ
みずかみ #JCN 水上
みずかめざ #KK*1 水瓶座
みずから #T35*4 自ら
みずぎ #T35*2 水着
みずぎわ #T35 水際
みずぐち #JNS 水口
みずけ #T35 水気
みずこし #JCN 水越
みずさわ #JCN 水沢
みずしま #JNS 水島 水嶋
みずた #JCN 水田
みずたに #JNS 水谷
みずたま #T35 水玉
みずどけい #T35 水時計
みずどり #T35 水鳥
みずなみ #JCN 瑞浪
みずの #JNS 水野
みずはけ #T30 水捌け
みずひき #T35*1 水引
みずびたし #T15 水浸し
みずぶそく #T15 水不足
みずべ #T35 水辺
みずほ #T35*1 瑞穂 #JCN 瑞穂
みずま #JCN 三瀦
みずまき #JCN 水巻
みずまし #T30 水増し
みずむし #T35 水虫
みせ #SUC*42 店 #KS*38 見せ #KS*3 みせ #KS*1 魅せ #T35 店
みせいねん #T35 未成年
みせかけ #KSr*1 見せかけ
みせさき #T35 店先
みせじまい #T30 店仕舞い
みせす #T35 ミセス
みせつけ #KSr 見せつけ
みせばん #T30 店番
みせもの #T35*2 見せ物
みぜん #T35*2 未然
みぜんけい #T35 未然形
みそ #T35*3 味噌 #T35 晦
みそか #T35 晦日
みそぎ #T35 禊
みそこな #W5r 見損な
みそじ #T35 三十路
みその #JCN 御園
みそめ #KS 見初め
みそら #T35 身空
みぞ #T35*2 溝 #T35 渠
みぞう #T35 未曽有
みぞぐち #JCN 溝口
みぞべ #JCN 溝辺
みた #S5r*21 満た #JCN 三田 #S5 充た
みたか #JCN 三鷹
みたがわ #JCN 三田川
みたけ #T35*1 身丈 #JCN 御嵩 三岳
みたて #KSr 見立て
みたに #JNS 三谷
みたま #JCN 三珠
みだ #S5*2 乱 #S5r 見出
みだし #T35*16 見出し
みだしなみ #T35 身だしなみ
みだら #T05 淫ら
みだり #T05*1 みだり
みだりがまし #KY みだりがまし
みだりがましゅう #KYU みだりがましゅう
みだれ #KSr*11 乱れ
みち #T35*70 道 #T15*11 未知 #T35*3 路 #KS*1 満ち #KS 盈ち #T35 途
みちお #JNM 通夫 道男 道夫 道雄
みちがえ #KS 見違え
みちこ #JNM 道子 美千子 美智子 倫子
みちしお #T35 満ち潮
みちしるべ #T35*4 道標
みちじゅん #T35 道順
みちすじ #T35*2 道筋
みちづれ #T35*1 道連れ
みちのく #CN みちのく
みちのり #T35*1 道のり
みちはる #JNM 亨治 道春
みちばた #T35*2 道端
みちび #K5r*6 導
みちびきだ #S5r*1 導き出
みちゃく #T15*1 未着
みちよ #JNM 道世 美千代 美智代
みちる #JN ミチル
みぢか #T05*3 身近
みっか #T35*5 3日 #T35*3 三日 #T35 ３日
みっかい #T30 密会
みっかび #CN 三ヶ日
みっきょう #T35*1 密教
みっくす #T30*2 ミックス
みっこう #T30 密航
みっこく #T30 密告
みっし #T35 密使
みっしつ #T35 密室
みっしゅう #T30 密集
みっせつ #T00*4 密接
みっそう #T30 密葬
みっちゃく #T30*1 密着
みっちり #F06 みっちり
みっつ #T35*8 三つ
みってい #T35 密偵
みっともな #KY*2 みっともな
みっとものう #KYU*1 みっとものう
みっぷう #T30 密封
みっぺい #T30*1 密閉
みつ #T05*16 密 #KJ*6 三 #T35*2 蜜 #G5*1 貢 #JNM みつ ミツ #JCN 御津 #JNM 光
みつあき #JNM 光昭 光明
みつい #JNS 三井
みついし #JCN 三石
みつえ #JCN 御杖 #JNM 光枝
みつお #JNM 光男 光夫 光雄 三男 三夫 三雄 満夫 満雄
みつか #R5*55 見付か #R5*50 見つか #R5*4 みつか
みつかいどう #CN 水海道
みつぎ #CN 御調
みつくろ #W5r 見繕
みつぐ #JNM 貢
みつけ #KSr*55 見付け #KSr*44 見つけ #JCN 見附
みつけだ #S5r*3 見つけ出
みつこ #JNM 光子 美津子
みつこし #KK*5 三越
みつざわ #JNS 光澤
みつしま #JCN 美津島
みつせ #JCN 三瀬
みつぞう #T30 密造
みつぞろえ #T35 三つ揃え
みつだん #T30*1 密談
みつてる #JNS 充照
みつど #T35*3 密度
みつどもえ #T35 三つ巴
みつなり #JNM 光成
みつにゅうこく #T30 密入国
みつのぶ #JNM 光信
みつはし #JCN 三橋
みつばい #T30 密売
みつばち #T35*1 蜜蜂
みつひこ #JNM 光彦
みつびし #KK*11 三菱
みつびしそうけん #KK*2 三菱総研
みつまさ #JNM 光政 光正
みつめ #KS*2 見つめ
みつも #R5r*9 見積も
みつもり #T30*8 見積り #JNS 三森
みつもりしょ #T35*6 見積書
みつゆ #T30 密輸
みつよ #JNM 光代
みつよし #JNM 光義
みつりょう #T30 密漁 密猟
みつりん #T35 密林
みつる #JNM 充 満
みつわ #JNS 三輪
みてい #T15*23 未定
みていぎ #T05*23 未定義
みと #R5r*10 見取 #R5r*5 看取 #JCN 御津 水戸 美都 #JNS 弥刀
みとう #JCN 美東 #T15 未踏
みとお #S5r 見通
みとおし #T35*5 見通し
みとどけ #KSr*2 見届け
みとみ #JCN 三富
みとめ #KSr*29 認め #KSr*4 みとめ
みとや #JCN 三刀屋
みとよ #JCN 三豊
みとりず #T35 見取図
みどう #JCN 御堂 #T35 御堂
みどうすじ #CN 御堂筋
みどく #T05*8 未読
みどころ #T35*2 見所
みどり #T35*27 緑 #JNM みどり 翠 #JCN 美土里 #JNM 碧 #JCN 緑
みどりかわ #JCN 緑川
みどろ #SUC みどろ
みな #T35*54 皆 #S5r*30 みな #S5r*10 見な #S5r*1 見倣
みなえ #JNM 美奈江
みなお #S5r*15 見直
みなかた #JNS 南方
みなかみ #JCN 水上
みながわ #JNS 皆川
みなくち #JCN 水口
みなぐち #JNS 源口
みなこ #JNM 美奈子 美名子
みなさま #T35*48 皆様
みなさん #T35*139 皆さん #T35*1 みなさん
みなづき #T35 水無月
みなと #T35*18 港 #T35*1 湊 #CN 港
みなの #JCN 皆野
みなべ #JCN 南部
みなべがわ #CN 南部川
みなまた #JCN 水俣
みなみ #T35*27 南 #CNPRE*1 南 #JNS 皆実 #CNSUC1 南 #JCN 南 #JNS 美並
みなみあいき #CN 南相木
みなみあいづ #CN 南会津
みなみあきた #CN 南秋田
みなみあしがら #CN 南足柄
みなみあずみ #CN 南安曇
みなみあまべ #CN 南海部
みなみありま #CN 南有馬
みなみいず #CN 南伊豆
みなみうおぬま #CN 南魚沼
みなみうわ #CN 南宇和
みなみおぐに #CN 南小国
みなみかた #JCN 南方
みなみかやべ #CN 南茅部
みなみかわ #JNS 南河
みなみかわち #CN 南河内
みなみかんばら #CN 南蒲原
みなみがわ #T35*1 南側
みなみくしやま #CN 南串山
みなみこま #CN 南巨摩
みなみさいたま #CN 南埼玉
みなみさく #CN 南佐久
みなみしたら #CN 南設楽
みなみしなの #CN 南信濃
みなみじゅうじせい #T35 南十字星
みなみたかき #CN 南高来
みなみたかぎ #CN 南高来
みなみたね #CN 南種子
みなみたま #CN 南多摩
みなみだいとう #CN 南大東
みなみちた #CN 南知多
みなみつがる #CN 南津軽
みなみつる #CN 南都留
みなみなか #CN 南那珂
みなみなす #CN 南那須
みなみの #JNS 南野
みなみふらの #CN 南富良野
みなみまき #CN 南牧
みなみまつうら #CN 南松浦
みなみまわり #T30 南回り
みなみみのわ #CN 南箕輪
みなみむろ #CN 南牟婁
みなみもと #JNS 南本
みなみやましろ #CN 南山城
みなも #T35*2 水面
みなもと #T35*1 源
みなよ #JNM 美奈代
みなら #W5r*3 見習
みなり #T35 身なり
みなれ #KS*5 見慣れ #KS*1 見なれ
みに #PRE*13 ミニ
みにい #C5r*17 見に行
みにく #KY*3 見にく #KY*2 醜
みにくう #KYU*1 見にくう 醜う
みにこん #T35*2 ミニコン
みにまむ #T15*1 ミニマム
みぬ #K5*1 見抜
みね #KJ*1 峰 #T35*1 峰 嶺 #JCN 三根 #JN 三子 #JNS 美禰 #JCN 美祢 #JNS 峰 #KJ 峯
みねお #JNM 峰雄
みねぎし #JNS 峰岸 峯岸
みねの #JNS 峯野
みねはま #JCN 峰浜
みねやま #JCN 峰山
みの #T35*19 蓑 #R5r*2 実 #KJ*1 箕 #JCN 三野 美濃 #T35 箕 #R5 稔
みのう #T15*10 未納 #JCN 美嚢 美濃
みのうえ #T35 身の上
みのお #JCN 箕面
みのかも #CN 美濃加茂
みのが #S5r*4 見逃
みのしょう #JNS 美濃庄
みのち #JNS 水内
みのはら #JNS 蓑原
みのぶ #JCN 身延 #JNS 美乃夫
みのまわり #T35*1 身の回り
みのり #JCN 美野里 #KJ 稔
みのる #JNM 実 稔
みのわ #JCN 箕輪
みは #R5r*3 見張 #R5 瞠
みはいる #JN ミハイル
みはから #W5r 見計ら
みはし #JNS 三橋
みはな #S5 見放
みはま #JCN 御浜 美浜
みはら #S5r 見晴ら #JCN 三原 美原
みはらい #T15 未払い
みはる #JCN 三春 #JNS 美春
みばえ #T30*3 見栄え
みひつ #T35*1 未必
みひら #K5r*1 見開
みふね #JCN 御船
みぶ #JCN 壬生
みぶり #T35*1 見ぶり 身振り
みぶん #T35*2 身分
みほ #JCN 美浦 美保 #JNM 美穂
みほこ #JNM 美穂子
みほのせき #CN 美保関
みほん #T35*7 見本
みぼうじん #T35*1 未亡人
みま #W5r 見舞 #JCN 三間 美馬
みまい #T30*2 見舞
みまいきん #T35 見舞金
みまが #W5 見紛
みまがえ #KS 見紛え
みまさか #JCN 美作
みまた #JCN 三股
みまね #T30 見まね
みまも #R5*4 見守
みまわ #R5r*1 見回 #S5*1 見回
みまん #T15*2 未満
みみ #T35*20 耳
みみうち #T30*1 耳打ち
みみがくもん #T35 耳学問
みみざわり #T05*1 耳障り
みみず #T35*1 ミミズ
みみっち #KY みみっち
みみより #T05 耳寄り
みむ #K5r*1 見向
みむら #JNS 三村
みめい #T35 未明
みもだえ #T30 身悶え
みもと #T35 身元
みもり #JNS 未森
みもん #T15 未聞
みゃく #T35*2 脈
みゃくどう #T30 脈動
みゃくはく #T35 脈拍
みゃくらく #T35*1 脈絡
みや #T35*11 宮 #JCN 宮
みやうち #JCN 宮内
みやかわ #JCN 宮川
みやがわ #JCN 宮川
みやき #JCN 三養基
みやぎ #JCN 宮城
みやくぼ #JCN 宮窪
みやけ #JCN 三宅
みやげ #T35*10 土産 #T35*2 みやげ
みやげばなし #T35*1 土産話
みやこ #JCN 宮古 #CN 京都 #T35 都 #JNM 美也子
みやこじ #JCN 都路
みやこじま #JCN 都島
みやこのじょう #JCN 都城
みやさか #JCN 宮坂
みやざき #JCN 宮崎
みやざわ #JCN 宮沢
みやした #JNS 宮下
みやしろ #JCN 宮代
みやじ #JCN 宮地
みやじま #JCN 宮島 #JNS 宮嶋
みやぞの #JNS 宮苑
みやた #JNS 宮田
みやたけ #JNS 宮武
みやぢ #JNS 宮地
みやづ #JCN 宮津
みやの #JCN 宮野
みやのじょう #JCN 宮之城
みやはら #JCN 宮原
みやぶ #R5r*1 見破
みやべ #JNS 宮部
みやま #JCN 海山 #T35 深山 #JCN 美山
みやまえ #JCN 宮前
みやもと #JCN 宮本
みやもり #JCN 宮守
みやわき #JNS 宮脇
みゅー #T35*4 μ #T35*1 Μ
みゅーじしゃん #T35*3 ミュージシャン
みゅーたんと #T35*1 ミュータント
みゆき #JNM 幸 深雪 美幸 美雪 美由紀
みょう #T05*13 妙 #KJ*2 明 #KJ 冥 名 命
みょうあん #T35 妙案
みょうがた #CN 明方
みょうぎ #T35 妙技 #CN 妙義
みょうこう #CN 妙高
みょうごにち #T35 明後日
みょうざい #CN 名西
みょうしゅ #T35 妙手
みょうじ #T35*2 名字 #T35 苗字
みょうじょう #T35 明星
みょうだい #T35 名代
みょうちょう #T35*1 明朝
みょうどう #CN 名東
みょうにち #T35*2 明日
みょうねん #T35 明年
みょうばん #T35 明晩 明礬
みょうり #T35*1 冥利
みよ #JNM みよ ミヨ 美代
みよこ #JNM 美代子
みよし #JCN 三好 三次 #JNS 三善 三膳 #JCN 三芳
みよた #JCN 御代田
みより #T35 身寄り
みらい #T35*10 未来
みらさか #JCN 三良坂
みり #JS*10 ミリ
みりは #T35*1 ミリ波
みりめーとる #JS ミリメートル 粍
みりょう #T15 未了 #T30 魅了
みりょく #T35*10 魅力
みりん #T35 味醂
みるからに #F14 見るからに
みるく #T35*3 ミルク
みるみる #F14*1 みるみる
みれん #T05*2 未練
みろく #JNS 弥勒
みわ #JCN 三輪 #JNS 三和 #JNM 美和
みわく #T30*2 魅惑
みわけ #KSr*1 見分け
みわこ #JNS 美和子
みわた #S5r*1 見渡
みん #KJ*7 民 #KJ*5 明 #KJ*1 眠 閔 #JN ミン
みんい #T35 民意
みんえい #T35*1 民営
みんか #T35 民家
みんかん #T35*3 民間
みんけん #T35 民権
みんげい #T35*1 民芸
みんしゃとう #KK 民社党
みんしゅ #T35*2 民主
みんしゅう #T35*2 民衆
みんしゅく #T35 民宿
みんじ #T35 民事
みんすきー #JN ミンスキー
みんせい #T35*1 民生 #T35 民政
みんぞく #T35*3 民族 #T35*1 民俗
みんだ #JNS 民田
みんちょう #KK*2 明朝
みんちょうたい #KK*6 明朝体
みんてい #KK*1 明帝
みんと #T35*1 ミント
みんな #T35*62 みんな #T35*42 皆
みんぽう #T35*2 民法 #T35*1 民放
みんゆう #T35 民有
みんよう #T35*1 民謡
みんわ #T35 民話
む #T35*105 無 #K5r*104 向 #PRE*62 無 #KJ*4 務 #S5r*3 蒸 #KJ*2 夢 #KJ*1 武 牟 #K5 剥 #KJ 矛 霧 鵡 #T35 六
むーあ #JN ムーア
むーど #T35*1 ムード
むい #T05 無為
むいか #T35 6日 ６日 #CN 六日
むいかいち #CN 六日市
むいちもん #T15 無一文
むいみ #T05*14 無意味
むえき #T05*1 無益
むえん #T05*1 無縁 #T35 無鉛
むおん #T15 無音
むか #W5r*40 向か
むかい #JNS 向井
むかいあ #W5r 向かい合
むかいしま #CN 向島
むかいはら #JCN 向原
むかいやま #JNS 向山
むかえ #KSr*6 迎え
むかえいれ #KSr 迎え入れ
むかし #T35*77 昔 #T35*1 むかし
むかしばなし #T35*1 昔話
むかっぱら #T35 むかっ腹
むかわ #JCN 武川 鵡川
むかん #T35 無冠
むが #T35*1 無我
むがい #T35*1 無害
むがく #T05*1 無学
むき #T35*4 無機 #T35*3 無期
むきあ #W5r 向き合
むきず #T05 無傷
むきだ #S5r むき出
むきゅう #T35*1 無給 #T35 無休
むきん #T35*1 無菌
むぎ #T35*2 麦 #CN 武儀 牟岐
むぎたに #JNS 麦谷
むぎちゃ #T35 麦茶
むぎばたけ #T35 麦畑
むぎわら #T35*1 麦ワラ #T35 麦藁
むく #T05*2 無垢 #M5r むく #T35 椋
むくい #KSr 報い
むくだい #JNS 椋代
むくち #T05*2 無口
むくどり #T35 ムクドリ 椋鳥
むくむく #F04 むくむく
むくれ #KSr むくれ
むくろ #T35 骸 躯
むけ #KSr*27 向け
むけい #T15*1 無形
むけつ #T05 無欠
むげ #T15*1 無碍
むげい #T05 無芸
むげがわ #CN 武芸川
むげん #T05*16 無限
むげんえん #T35 無限遠
むげんだい #T35*2 ∞ #T05*2 無限大
むこ #T35*2 婿
むこう #T05*34 無効 #T35*25 向こう #T35*19 向う #T35*16 無向 #T35*2 むこう #CN 向日
むこうじま #JCN 向島
むこがわ #CN 武庫川
むこん #T15*1 無根
むご #KY*1 酷 #KY 惨
むごう #KYU*1 酷う #KYU 惨う
むごたらし #KY*1 むごたらし
むごたらしゅう #KYU*1 むごたらしゅう
むごん #T35*3 無言
むさし #T35*3 武蔵 #JCN 武蔵
むさしの #CN 武蔵野
むさしむらやま #CN 武蔵村山
むさぼ #R5r 貪
むさん #T35 無産
むざい #T35*1 無罪
むざむざ #F06 むざむざ
むざん #T05 無惨 無残
むし #T30*68 無視 #T35*42 虫 #KJ*1 蟲
むしあつ #KY*2 蒸し暑
むしあつう #KYU*1 蒸し暑う
むしけら #T35 虫けら
むしば #T35*1 虫歯 #M5 蝕
むしゃ #T35*1 武者
むしゃぶるい #T30 武者震い
むしゅう #T35 無臭
むしょう #T15*8 無償 #T35 無性 #T30 霧消
むしょうに #F14 むしょうに
むしょく #T35*1 無職 #T15 無色
むしろ #F14*16 むしろ #T35 筵 蓆
むしん #T00 無心
むしんろん #T35 無神論
むじ #T15*1 無地
むじつ #T35*1 無実
むじな #T35 貉
むじゃき #T05*1 無邪気
むじゅん #T10*22 矛盾
むじょう #T05*2 無情
むじょうけん #T15 無条件
むじん #T35*1 無人 #T05*1 無尽
むす #B5r*33 結
むすう #T05 無数
むすこ #T35*4 息子
むすびつ #K5r*6 結び付 #K5r*2 結びつ
むすびつけ #KSr*1 結びつけ
むすめ #T35*3 娘
むずかし #KY*73 難し #KY*1 むずかし
むずかしゅう #KYU*2 難しゅう #KYU*1 むずかしゅう
むせ #B5 咽
むせい #T35 無声
むせきにん #T05*4 無責任
むせっそう #T05*1 無節操
むせん #T35*17 無線 #T35 無銭
むせんき #T35*1 無線機
むぜい #T35 無税
むそう #T30*1 夢想
むそじ #T35 六十路
むぞうさ #T05*1 無造作
むた #JNS 牟田
むだ #T05*45 無駄 #T05*2 ムダ
むだん #T35*4 無断
むち #T05*16 無知 #T35*10 鞭
むちゃ #T00*2 むちゃ #T00*1 無茶
むちゃくちゃ #T05*3 むちゃくちゃ #T05*2 無茶苦茶
むちゅう #T05*1 夢中
むちん #T35 無賃
むっつ #T35*2 六つ
むつ #KJ*1 睦 #CN むつ #JCN 陸奥
むつう #T35 無痛
むつうら #JCN 六浦
むつお #JNM 睦夫
むつかし #KY*18 難し
むつかしゅう #KYU*2 難しゅう
むつき #T35 睦月
むつざわ #CN 睦沢
むつまじ #KY 睦まじ
むつまじゅう #KYU 睦まじゅう
むつみ #JNM 睦美
むてき #T05 無敵
むてじゅん #T35 無手順
むでん #T35 無電
むとう #JNS 武藤
むとんちゃく #T05 無頓着
むどく #T35 無毒
むなかた #JCN 宗像
むなげ #T35 胸毛
むなさわぎ #T30 胸騒ぎ
むなし #KY*2 虚し #KY 空し
むなしゅう #KYU*1 虚しゅう #KYU 空しゅう
むに #T35 無二
むにえる #T35*1 ムニエル
むね #T35*34 旨 #T35*8 胸 #KJ*2 宗 #JS 棟 #T35 棟
むねあげ #T30 棟上げ
むねお #JNM 宗男 宗雄
むねん #T05*2 無念
むのう #T05*2 無能
むはい #T35*1 無敗
むひ #T05 無比
むびょう #T05 無病
むふう #T35 無風
むへんかん #T30 無変換
むほう #T05*1 無法
むぼう #T05*2 無謀
むぼうび #T05 無防備
むま #T35 夢魔
むみ #T05*1 無味
むめい #T05*1 無名 #T35 無銘
むやみ #T05*2 無闇
むゆうびょう #T35*1 夢遊病
むよう #T05*4 無用
むよく #T05 無欲
むら #CNSUC1*72 村 #S5*5 蒸ら #KJ 群 #T35 村 #KJ 屯 邑
むらい #JNS 村井
むらおか #JCN 村岡
むらかみ #JCN 村上
むらが #R5 群が
むらき #JNS 村木
むらこし #JNS 村越
むらさき #T35*1 紫
むらせ #JNS 村瀬
むらた #JCN 村田
むらはちぶ #T35 村八分
むらまつ #JCN 村松
むらむら #F06*1 むらむら
むらやま #JCN 村山
むり #T05*80 無理
むりじい #T30 無理強い
むりやり #T05*13 無理矢理
むりょう #T15*10 無料 #T35 無量
むりょく #T05*2 無力
むるい #T05 無類
むれ #KS*2 群れ #T35 群 #KS 蒸れ #JCN 牟礼
むろ #T35*1 室 #JNS 牟婁
むろい #JNS 室井
むろう #JCN 室生
むろた #JNS 室田
むろと #CN 室戸
むろね #CN 室根
むろまち #T35 室町
むろらん #CN 室蘭
むろん #F14*13 無論 #F14*1 むろん
むろんの #RT むろんの 無論の
め #T35*255 目 #D2T35*16 目 #T35*11 芽 #S5*7 召 #KJ*3 女 #T35*2 眼 #K5r*1 め #SUC め #JSSUC 目
めーか #T35*20 メーカ
めーかー #T35*7 メーカー
めーくあっぷ #T30*1 メークアップ
めーた #T35*9 メータ
めーでー #T35 メーデー
めーとる #JS*3 メートル #T35*2 メートル
めーりんぐ #T30*49 メーリング
めーる #T30*255 メール
めあかし #T35 目明かし
めあたらし #KY*2 目新し
めあたらしゅう #KYU*1 目新しゅう
めあて #T35*2 目当て
めい #JS*255 名 #KJ*17 明 #T30*3 冥 #T35*2 名 #ZX*2 命 #KJ*2 鳴 #T35*1 命 #KJ*1 迷 #T35*1 銘 #ZX*1 銘 #R5*1 滅入 #KJ 謎 名 #PRE 名 #SUC 名 #KJ 盟 #PRE 銘 #T35 姪
めいあん #T35*2 名案 #T35*1 明暗
めいい #T35 名医
めいう #T5r 銘打
めいうん #T35 命運
めいおうせい #KK*1 冥王星
めいか #T35 名家 銘菓
めいかい #T00*1 明解 #T05 明快
めいかく #T05*31 明確
めいが #T35*1 名画
めいがら #T35*1 銘柄
めいき #T30*9 明記 #T30*2 銘記
めいきゅう #T35*1 迷宮
めいきょく #T35*1 名曲
めいぎ #T35*1 名義
めいく #T30*6 メイク
めいけん #T35 名剣 名犬
めいげつ #T35 名月
めいげん #T30*3 明言 #T35*1 名言
めいこう #T35*8 名工
めいさい #T05*16 明細
めいさく #T35*1 名作
めいさつ #T35 名刹
めいさま #JS 名様
めいさん #T35 名産
めいし #T35*50 名詞 #T35*9 名刺
めいしゅ #T35*1 名手 銘酒
めいしょ #T35*6 名所
めいしょう #T35*37 名称 #T35 名勝
めいしん #T35*4 迷信 #CN 名神
めいじ #T30*32 明示 #KK*4 明治
めいじつ #T35*2 名実
めいじん #T35*3 名人
めいすい #T35 名水
めいすう #T35 名数
めいせい #T35*1 名声
めいせき #T05*1 明晰
めいせん #T35 名泉
めいそう #T30*1 瞑想
めいだい #T35*13 命題 #KK*10 名大 #KK*1 明大
めいだいまえ #CN 明大前
めいち #T35 明知
めいちゅう #T30*1 命中
めいちょ #T35 名著
めいっぱい #T15 目一杯
めいとう #KK*2 名糖 #T35 名刀 #CN 名東 #T30 名答 #T35 名答
めいど #T35*1 明度 #T35 冥土
めいにち #T35 命日
めいはく #T05*2 明白
めいば #T35 名馬
めいび #T35*1 名媚 #T05*1 明媚
めいびん #T05 明敏
めいふ #T35 冥府
めいふく #T35 冥福
めいぶつ #T35*2 名物
めいぶん #T35*2 名文 #T35 名分
めいぶんか #T30*1 明文化
めいほう #T35 盟朋
めいぼ #T35*8 名簿
めいぼう #T35 名望
めいめい #T30*2 命名
めいめいはくはく #T05 明々白々
めいもく #T35*1 名目 #T30 冥黙
めいもん #T35 名門
めいよ #T05*3 名誉
めいりょう #T05*2 明瞭
めいりんぐ #T35*2 メイリング
めいりんぐりすと #T35*6 メイリングリスト
めいる #T30*25 メイル
めいれい #T30*117 命令
めいろ #T35*1 迷路
めいろう #T05 明朗
めいわ #CN 明和
めいわく #T00*18 迷惑
めいん #T05*24 メイン
めうえ #T35*1 目上
めうし #T35 牝牛
めうつり #T30 目移り
めおと #T35 夫婦
めか #T35*3 メカ
めかけ #T35 妾
めかた #T35 目方
めかにずむ #T35*13 メカニズム
めが #JS*7 メガ
めがしら #T35 目頭
めがね #T35 眼鏡
めがみ #T35*1 女神
めきしこ #CN メキシコ
めく #R5r めく
めくら #T35*1 めくら 盲
めくるめく #RT 目くるめく
めぐ #R5r*11 めぐ #M5r*4 恵 #R5r*1 巡
めぐすり #T35*1 目薬
めぐみ #JNM めぐみ #JN 恵
めぐりあ #W5r*4 巡り合
めぐろ #JCN 目黒
めげ #KS*2 めげ
めさき #T35*1 目先
めざ #S5*18 目指
めざと #KY*1 目ざと
めざとう #KYU*1 目ざとう
めざまし #KY*3 目覚し #T35*3 目覚し
めざましゅう #KYU*1 目覚しゅう
めざめ #KSr*2 目覚め
めざわり #T05*1 目障り
めし #T35*17 飯
めした #T35 目下
めしつかい #T35 召使
めじゃー #T05*9 メジャー
めじり #T35 目尻
めじるし #T30*1 目印
めじろ #T35*1 目白 #T35 メジロ 繍眼児
めす #T35*1 ♀ メス #T35 雌 牝
めずらし #KY*15 珍し
めずらしゅう #KYU*2 珍しゅう
めせん #T35*2 目線
めそっど #T35*33 メソッド
めそぽたみあ #CN メソポタミア
めぞねっと #T35*1 メゾネット
めた #PRE*22 メタ #T05*22 メタ
めたふぁ #T35*1 メタファ
めたもるふぉーぜ #T30*2 メタモルフォーゼ
めたる #T35*2 メタル
めたん #T35*1 メタン
めだ #T5*10 目立
めだま #T35*3 目玉
めだる #T35*1 メダル
めちゃくちゃ #T05*5 めちゃくちゃ #T05 滅茶苦茶
めちゃめちゃ #T05*1 めちゃめちゃ
めちる #T30*1 メチル
めっか #CN メッカ
めっき #T30 滅金
めっきゃく #T30 滅却
めっきり #F06 めっきり
めっきん #T30 滅菌
めっしつ #T35 滅失
めっしゅ #T35*2 メッシュ
めっせーじ #T35*161 メッセージ
めった #T05*2 滅多
めったに #F14*13 滅多に
めっぽう #T15 滅法
めつ #T35*4 滅
めつぼう #T30 滅亡
めでぃあ #T35*31 メディア
めでぃかる #T05 メディカル
めでた #KY*4 めでた
めでとう #KYU*1 めでとう
めとりっく #T35*8 メトリック
めど #T35*2 メド #T35*1 目処
めなし #CN 目梨
めにゅー #T35*201 メニュー
めぬま #JCN 妻沼
めのう #T35 碼碯
めはな #T35*1 目鼻
めばえ #KSr 芽生え
めぶ #K5r 芽吹
めぶんりょう #T15 目分量
めべり #T30 目減り
めぼし #KY めぼし #T35 目星
めまぐるし #KY*1 目まぐるし
めまぐるしゅう #KYU*1 目まぐるしゅう
めまんべつ #CN 女満別
めむろ #CN 芽室
めめし #KY*1 女々し
めめしゅう #KYU*1 女々しゅう
めも #T30*80 メモ
めもちょう #T35 メモ帳
めもと #T35 目元
めもり #T35*189 メモリ #T35*4 目盛 #T35*1 目盛り
めもりー #T35*4 メモリー
めやす #T35*3 目安
めやに #T35 目脂
めりけんこ #T35 メリケン粉
めりっと #T35*16 メリット
めろでぃー #T35*1 メロディー
めん #JS*68 面 #T35*29 免 #ZX*29 免 #T35*16 麺 #KJ 棉 #T35 綿 #KJ 緬 #SUC 面 #T35 面
めんえき #T35 免疫
めんかい #T30 面会
めんが #T35*1 面画
めんきょ #T30*4 免許
めんきょしょう #T35*1 免許証
めんざい #T30 免罪
めんし #T35 綿糸
めんしき #T35*5 面識
めんしょく #T30 免職
めんじょ #T30*1 免除
めんじょう #T35*2 免状
めんせき #T30*3 免責 #T35*2 面積
めんせつ #T30*11 面接
めんぜい #T30 免税
めんぜん #T35*1 面前
めんそ #T30 免租 免訴
めんそう #T35 面相
めんたいこ #T35*1 明太子
めんたる #T05*2 メンタル
めんだ #JCN 免田
めんだん #T30*4 面談
めんつ #T35*7 面子
めんてなんす #T30*137 メンテナンス
めんどう #T05*82 面倒 #T05*1 めんどう
めんどくさ #KY*26 めんどくさ
めんば #T35*90 メンバ
めんばー #T35*32 メンバー
めんぼく #T35*1 面目
めんみつ #T05 綿密
めんめん #T35*3 面々
めんもく #T35 面目
めんるい #T35 麺類
も #T5*255 持 #SX*45 模 #T5*19 も #M5*12 も #R5*5 漏 #R5r*3 盛 #M5*3 揉 #R5r*2 も #S5*2 燃 #T35*1 喪 藻 #R5 洩 #KJ 裳 摸 茂
もーしょん #T35 モーション
もーた #T35*2 モータ
もーたー #T35*1 モーター
もーつぁると #JN モーツァルト
もーど #T35*255 モード
もーるす #JN モールス
もう #F14*255 もう #S5*66 申 #KJ*22 猛 #KJ*4 網 #KJ*3 盲 #KJ*1 望 妄 #JS*1 毛 #KJ*1 魍 #KJ 亡 孟 毛 #SUC 網 #KJ 耗 蒙
もうい #T35 猛威
もうか #R5*2 儲か
もうきん #T35*1 猛禽
もうけ #KS*31 設け #KSr*5 儲け #KSr*1 もうけ #KJ 儲
もうげん #T35*1 妄言
もうしあげ #KS*16 申し上げ
もうしあわ #S5r 申し合わ
もうしいれ #KSr 申し入れ
もうしうけ #KSr 申し受け
もうしこ #M5r*65 申し込
もうしこみ #T35*3 申込
もうしこみしょ #T35*24 申込書
もうしたてしょ #T35*1 申立書
もうしつけ #KSr*1 申しつけ
もうしで #T35*2 申し出
もうしぶん #T35 申し分
もうしょ #T35 猛暑
もうしわけ #T30*125 申し訳
もうしわけな #KY*30 申し訳な
もうしわけのう #KYU*2 申し訳のう
もうしわた #S5r 申し渡
もうしん #T30 盲信
もうじゃ #T35 亡者
もうじゅう #T35*1 猛獣
もうじん #T35 盲人
もうせん #T35 毛氈
もうぜん #F02 猛然
もうそう #T30*1 妄想
もうたくとう #JNS 毛沢東
もうちょう #T35 盲腸
もうてん #T35 盲点
もうで #KSr 詣で
もうとう #F14*1 毛頭
もうどうけん #T35*1 盲導犬
もうどく #T15 猛毒
もうはつ #T35 毛髪
もうひつ #T35*1 毛筆
もうふ #T35*1 毛布
もうぼさんせん #T35 孟母三遷
もうまく #T35*1 網膜
もうもう #F00*1 もうもう
もうもく #T15 盲目
もうようたい #T35*1 毛様体
もうら #T30*2 網羅
もうり #JCN 毛利
もうれつ #T05*1 猛烈
もえ #KS*4 燃え #T35*1 萌 #KS 萌え
もおか #JCN 真岡
もがみ #JCN 最上
もぎ #T35 模擬 #JNS 茂木
もく #KJ*90 木 #KJ*4 目 #KJ 黙 #S5 黙 #SX 黙 #KJ 杢
もくあみ #T35 黙阿弥
もくげき #T30*1 目撃
もくさつ #T30*1 黙殺
もくさん #T30 目算
もくざい #T35 木材
もくし #T30*1 目指 目視 #T30 黙視
もくじ #T35*26 目次
もくせい #KK*1 木星
もくぜん #T35*2 目前
もくそく #T30 目測
もくぞう #T35 木造
もくたん #T35 木炭
もくてき #T30*121 目的
もくてきち #T35*1 目的地
もくとう #T30*1 黙祷
もくにん #T30 黙認
もくば #T35*1 木馬
もくひ #T30 黙秘
もくひょう #T35*48 目標
もくめ #T35*1 木目
もくもく #F00*2 黙々
もくよう #T35*5 木曜
もくようび #T35*27 木曜日
もくれい #T30 黙礼
もくろ #M5 目論
もくろく #T35*24 目録
もくろみ #T35 目論見
もぐ #R5r*1 潜
もぐりこ #M5r*1 もぐり込
もけい #T35 模型
もごもご #F04*1 もごもご
もさく #T30*2 模索
もし #F14*93 もし
もしか #T31*24 もしか
もしき #T35*2 模式
もしくは #CJ*16 もしくは
もしもし #CJ もしもし
もしゃ #T30 模写
もしや #CJ*1 もしや
もしゅ #T35 喪主
もしょう #T35 喪章
もじ #T35*255 文字 #R5r*6 もじ #JCN 門司
もじしゅ #T35 文字種
もじすう #T35 文字数
もじばけ #T30 文字化け
もじゅーる #T35*133 モジュール
もじれつ #T35 文字列
もすくわ #CN モスクワ
もず #T35*2 百舌 #T35*1 鵙
もずく #T35 もずく
もせうし #CN 妹背牛
もぞう #T30 模造
もぞもぞ #F04 もぞもぞ
もたげ #KS*1 もたげ
もたら #S5*25 もたら
もだえ #KSr 悶え
もだん #T05 モダン
もち #T35 持ち #KJ 勿 #T35 餅
もちーふ #T35 モチーフ
もちあげ #KSr*5 持ち上げ
もちある #K5r*1 持ち歩
もちあわ #S5r*1 持ち合わ
もちい #KS*161 用い
もちがせ #JCN 用瀬
もちこ #M5r*14 持ち込 #S5r*2 持ち越
もちごめ #T35 餅米
もちさ #R5 持ち去
もちだ #S5r*1 持ち出 #JNS 持田
もちづき #T35*1 望月 #JCN 望月
もちなお #S5r 持ち直
もちはこ #B5r 持ち運
もちはだ #T15 餅肌
もちまわ #R5r*1 持ち回
もちゅう #T35 喪中
もちよ #R5r*1 持ち寄
もちろん #F14*80 もちろん #F14*10 勿論
もちろんの #RT もちろんの 勿論の
もっこう #T35 木瓜 木工
もったい #T35*2 もったい #T35*1 勿体
もってのほか #T06 もってのほか
もっと #F14*136 もっと
もっとー #T35*1 モットー
もっとも #F14*35 最も #CJ*3 もっとも #T05*3 もっとも #CJ*1 尤も #T05*1 尤も
もっぱら #T06*4 専ら #T06*1 もっぱら
もつやく #T35 没薬
もつれ #KSr もつれ
もてあそ #B5 弄
もてあま #S5*1 持て余
もてぎ #JCN 茂木
もでぃふぁい #T30*1 モディファイ
もでむ #T35*77 モデム
もでら #T35*2 モデラ
もでりんぐ #T30*9 モデリング
もでる #T35*164 モデル
もと #T35*255 元 #T35*81 基 #T35*33 もと #T35*22 下 #KJ*19 本 #T35*4 素 #PRE*1 元 #M5 求 #SUC 元 #JNS 望都 #PRE 本
もとい #T35*1 基 #CJ もとい
もといけ #JNS 本池
もとうけ #T30 元請け
もとお #JNM 元夫
もとおか #JNS 元岡
もとき #T35 本木
もとこ #JNM 素子
もとじめ #T30*1 元締め
もとす #JCN 本巣
もとせん #T35 元栓
もとだ #JNS 元田
もとちょう #T35 元帳
もとづ #K5*55 基づ
もとで #T35 元手
もとどおり #T15*1 元通り
もとね #T35 元値
もとの #JCN 本埜
もとはし #JNS 本橋
もとひこ #JNM 元彦
もとぶ #JCN 本部
もとま #R5*5 求ま
もとみや #JCN 本宮
もとむら #JNS 本村
もとめ #KSr*69 求め
もともと #F14*19 もともと #T35*19 もともと #T35*9 元々
もとやま #JCN 本山
もとゆき #JNM 基之
もとよし #JNM 基良 元吉 #JCN 本吉
もとろーら #KK*2 モトローラ
もど #R5r*248 戻 #S5r*182 戻
もどき #SUC*2 もどき
もにた #T30*20 モニタ
もにたりんぐ #T30*4 モニタリング
もの #T35*255 物 #T35*160 もの #T35*54 者 #T35*8 モノ #D2T35*1 物
ものう #JCN 桃生
ものお #JCN 桃生
ものおき #T35*1 物置
ものおと #T35 物音
ものおもい #T30 物思い
ものかき #T35 物書き
ものがた #R5*8 物語
ものがたり #T35*2 物語
ものがなし #KY*1 もの悲し #KY 物悲し
ものがなしゅう #KYU*1 もの悲しゅう #KYU 物悲しゅう
ものくろ #T15*4 モノクロ
ものごい #T30 物乞い
ものごと #T35*5 物事
ものさし #T35*1 物差し
ものしずか #T06 物静か
ものしり #T15 物知り
ものすご #KYmi ものすご 物凄
ものすごう #KYU ものすごう 物凄う
ものずき #T05*1 物好き
ものたりな #KY*1 物足りな
ものたりのう #KYU*1 物足りのう
もののけ #T35*1 もののけ
もののべ #JNS 物部
ものべ #JCN 物部
ものほし #T35*1 物干し
ものめずらし #KY 物珍し
もはや #F14*13 もはや #F14*1 最早
もはん #T35*1 模範
もばら #JCN 茂原
もふく #T35*1 喪服
もほう #T30 模倣
もみ #KJ 籾
もみけ #S5r もみ消 揉み消
もみじ #T35*3 紅葉 #T35*2 椛
もめ #KS*5 もめ #KS*1 揉め
もめごと #S5r 揉め事
もめん #T35*1 木綿
もも #T35*7 桃 #KJ*2 百 #T35 股 腿
ももいし #JCN 百石
ももせ #JNS 百瀬
ももだに #JCN 桃谷
ももやま #JCN 桃山
ももんが #T35*1 モモンガ
もや #W5 もや #S5 燃や #T35 母屋 靄
もよう #T35*4 模様
もよお #S5r*3 催
もより #T35*2 最寄り
もら #W5*222 もら #W5r*31 貰 #S5r*1 漏ら
もらい #KJ 貰
もらとりあむ #T35 モラトリアム
もらる #T35*2 モラル
もり #T35*72 森 #T30*2 守 #JCN 森 #T35 杜
もりあが #R5r*8 盛り上が
もりあげ #KSr*2 盛り上げ
もりお #JNM 守男 盛夫 盛雄
もりおか #JNS 森岡 #JCN 盛岡
もりかえ #S5r*1 盛りかえ
もりかわ #JNS 森川
もりぐち #JCN 守口 #JNS 森口
もりこ #M5r*3 盛り込
もりさき #JNS 森崎
もりした #JCN 森下
もりしま #JNS 森島 森嶋
もりた #JCN 森田 #JNS 盛田
もりたか #JNS 森高
もりたて #KSr 盛り立て
もりとも #JNS 守友
もりなが #JNS 森永
もりの #JNS 森野
もりひこ #JNM 守彦
もりひろ #JNM 守弘
もりべ #JNS 森部
もりもと #JCN 森本
もりや #JNS 守屋 守谷 森谷
もりやま #JCN 守山 森山
もりよし #JCN 森吉
もるもっと #T35*1 モルモット
もれ #KSr*9 洩れ
もれしゃん #JN モレシャン
もれなく #F14 もれなく
もろ #T35*2 諸 #KY*1 脆 #T35 もろ
もろう #KYU*1 脆う
もろかた #JNS 諸県
もろずみ #JNS 諸角
もろつか #JCN 諸塚
もろどみ #JCN 諸富
もろぼし #JNS 諸星
もろみ #T35 もろみ
もろもろ #T35*2 諸々
もろやま #JCN 毛呂山
もん #T35*24 門 #JS*9 問 #KJ*6 文 #KJ*2 聞 #KJ*1 問 悶 #T35*1 紋
もんか #T35*1 門下
もんがいかん #T35*2 門外漢
もんきりがた #T05*1 紋切り型
もんく #T35*29 文句
もんげん #T35 門限
もんし #T30 悶死 #T35 門歯
もんしょう #T35*1 紋章
もんしん #T30 問診
もんじゅ #T35*1 文珠
もんすたー #T35*1 モンスター
もんぜん #JCN 門前 #T35 門前
もんたーじゅ #T30 モンタージュ
もんだい #T15*255 問題
もんだいてん #T35*77 問題点
もんちゃく #T30*1 悶着
もんつき #T35 紋付
もんてい #T35 門弟
もんてかるろ #CN モンテカルロ
もんとりおーる #CN モントリオール
もんど #JNS 門戸
もんどう #T30*1 問答
もんどりう #T5 もんどり打
もんばつ #T35 門閥
もんばん #T30 門番
もんぴょう #T35 門標
もんぶ #T35*2 文部
もんぶしょう #KK*3 文部省
もんべつ #CN 紋別 門別
もんめ #JS*1 匁 #T35*1 匁
もんよう #T35*1 文様
や #T35*252 → #M5*199 や #R5*195 や #SUC*139 屋 #K5r*31 焼 #T35*27 矢 #KJ*22 谷 #K5*10 妬 #KJ*8 哉 #R5*6 殺 #KJ*5 也 #KJ*3 夜 #KJ*2 家 八 #T35*2 野 #K5*1 や 灼 #M5*1 病 #KJ*1 耶 弥 #T35 屋 #R5 遣 #KJ 箭 埜 冶 爺
やーど #JS*1 ヤード
やあ #CJ*2 やあ #T35*2 やあ
やいた #JCN 矢板
やいづ #CN 焼津
やいのやいの #F12 やいのやいの
やいば #T35 刃
やえ #T35 八重
やえい #T30 野営
やえこ #JNM 八重子 彌生子
やえす #CN 八重洲
やえば #T35*1 八重歯
やえやま #JCN 八重山
やお #JCN 八尾
やおちょう #T30 八百長
やおつ #CN 八百津
やおもて #T35*1 矢面
やおや #T35 八百屋
やおら #F14*1 やおら
やかげ #JCN 矢掛
やかた #T35*1 屋形 舘 #T35 館
やかまし #KY やかまし 喧し
やかましゅう #KYU やかましゅう 喧しゅう
やから #T35*2 輩
やかん #T35*4 夜間
やがい #T35*1 野外
やがく #T35 夜学
やがて #F14*1 やがて
やがみ #JNS 八神 #JCN 矢上
やきつ #K5r*1 焼き付
やきつけ #KSr*1 やきつけ #T30 焼付け
やきにく #T35*2 焼肉
やきもの #T35 焼物
やきゅう #T35*9 野球
やきん #T30*1 夜勤 #T35 冶金
やぎ #T35*6 山羊 #JCN 八木
やぎざ #KK*1 山羊座
やぎぬま #JNS 八木沼
やく #T30*117 訳 #S5*116 訳 #NNPRE*101 約 #T30*63 約 #T35*18 薬 #T35*10 役 #T35*1 厄 #S5*1 約 #KJ*1 躍 #JCN 屋久 #SUC 役
やくいん #T35*1 役員
やくがい #T35 薬害
やくがく #T35 薬学
やくがら #T35*1 役柄
やくご #T35 訳語
やくざい #T35 薬剤
やくし #JCN 薬師
やくしまる #JNS 薬師丸
やくしゃ #T35*2 役者
やくしょ #T35*1 役所
やくしょく #T35*4 役職
やくしん #T30 躍進
やくじょ #T02 躍如
やくじょう #T30 約定
やくすう #T35 約数
やくそう #T35*1 薬草
やくそく #T30*12 約束
やくだ #T5r*5 役立
やくてい #T30 約定
やくとく #T35 役得
やくどう #T30*1 躍動
やくどく #T35*3 役得 #T30 訳読
やくどし #T35 厄年
やくにん #T35*1 役人
やくの #CN 夜久野
やくば #T35 役場
やくひん #T35 薬品
やくぶつ #T35 薬物
やくぶん #T35*1 訳文
やくまん #T35 役満
やくめ #T35*3 役目
やくも #JCN 八雲
やくよう #T35 薬用
やくり #T35*1 薬理
やくわり #T35*21 役割
やぐち #JCN 矢口
やぐら #T35 櫓
やけ #KSr*1 焼け #R5r 灼け
やけい #T35*2 夜景 #T35*1 野鶏 #T35 夜警
やけつ #K5r 焼け付
やけど #T30*1 火傷
やけぼっくい #T35 焼けぼっくい 焼け棒杭
やこう #T35*10 夜光 #T35*3 夜行
やごう #T35 屋号
やごと #JCN 八事
やさい #T35*5 野菜
やさか #JCN 八坂 弥栄
やさき #T35*2 矢先
やさく #JNS 矢作
やさし #KY*22 やさし #KY*9 優し #KY*1 易し
やさしゅう #KYU*2 やさしゅう #KYU*1 易しゅう 優しゅう
やさと #JCN 八郷
やざき #JNS 矢崎
やざわ #JNS 矢沢
やし #T35*1 香具師
やしお #JCN 八潮
やしき #T35*3 屋敷
やしな #W5r*1 養
やしま #JCN 矢島
やしょく #T35*1 夜食
やしろ #T35*1 社 #JCN 社
やしん #T35*1 野心
やじ #T35*1 野次
やじうま #T35 野次馬
やじま #JNS 矢島 矢嶋
やじゅう #T35 野獣
やじるし #T35*69 → #T30*23 矢印 #T35*14 ↓ #T35*12 ← #T35*8 ↑
やじん #T35 野人
やす #D2KY*226 やす #KY*101 安 #T35*83 安 #M5r*39 休 #KY*30 易 #KJ*7 靖 #KJ*3 康 #KJ*1 泰 #JCN 安 #D2KY 易 #KJ 保 #JCN 夜須 野洲
やすあがり #T05*1 安上がり
やすあき #JNM 靖昭
やすい #JCN 安井
やすう #KYU*3 安う #KYU*2 易う
やすうら #JCN 安浦
やすうり #T10*3 安売り
やすお #JNM 安男 安雄 康生 康男 康夫 康雄 泰男 泰夫 泰雄 保生 保男 保夫 保雄 靖夫
やすおか #JCN 泰阜
やすかわ #JNS 安川
やすぎ #JCN 安来
やすこ #JNM 安子 康子 泰子 靖子
やすし #JNM 安 泰 靖
やすじ #JNM 康司
やすだ #JCN 安田
やすっぽ #KY*1 安っぽ
やすづか #JCN 安塚
やすとみ #JCN 安富
やすとも #JNM 康友
やすね #T15 安値
やすのり #JNM 安則 泰憲
やすひこ #JNM 康彦 泰彦 靖彦
やすひろ #JNM 安弘 康宏 康広 康弘 康博 泰弘 保博
やすふみ #JNM 泰史
やすま #R5r 休ま
やすまさ #JNM 康正
やすみ #T35*15 休み
やすもの #T35*1 安物
やすゆき #JNM 恭之 康之 保之
やすよ #JNM 康代
やすよし #JNS 康善
やすら #G5r*1 安ら
やすらか #T05 安らか
やすろう #JNM 康朗
やすん #ZX*1 安ん
やず #JNS 八頭
やせ #KSr*3 痩せ
やせい #T35*1 野性 #T30 野生
やせおとろえ #KSr やせ衰え
やせほそ #R5r やせ細
やそう #T35 野草
やそうきょく #T35*1 夜想曲
やそじ #T35 八十路
やた #JNS 矢田
やたい #T35*2 屋台
やたべ #JNS 谷田部
やたら #F14*3 やたら #T05*3 やたら
やたろう #JNM 弥太郎
やだ #JNS 矢田
やちほ #JCN 八千穂
やちまた #JCN 八街
やちょう #T35 野鳥
やちよ #JCN 八千代 #T35 八千代
やちん #T35*2 家賃
やっか #T35*1 薬価 #M5r やっか
やっかい #T05*6 厄介
やっき #T35*1 やっき #T35 躍起
やっきょう #T35 薬莢
やっきょく #T35 薬局
やっこ #T35 奴
やっつ #T35*1 八つ
やっつけ #KS*2 やっつけ
やっと #F14*22 やっと #T36 やっと
やっとこ #T35 やっとこ
やっとこさ #F14 やっとこさ
やっぱり #F14*71 やっぱり
やつ #T35*83 奴 #T35*29 やつ #T35*11 八 #S5 やつ
やつお #JCN 八尾
やつか #JCN 八束
やつがだけ #CN 八ヶ岳
やつしろ #JCN 八代
やつまき #JNS 八巻
やつめうなぎ #T35*1 鱧
やと #W5*1 雇 #T35*1 野兎
やとい #T35 雇
やといぬし #T35 雇主
やとう #T35*1 野党 #T35 野盗
やとみ #JCN 弥富
やど #T35*12 宿 #R5*1 宿 #S5*1 宿
やどや #T35*1 宿屋
やな #T35 梁
やない #JCN 柳井
やないづ #JCN 柳津
やながわ #JCN 柳川 梁川
やなぎ #T35*2 柳 #JNS 柳
やなぎさわ #JNS 柳沢
やなぎだ #JCN 柳田
やなぎはら #JCN 柳原
やなぎもと #JCN 柳本
やなせ #JNS 柳瀬
やなだに #JCN 柳谷
やなはら #JNS 柵原
やに #T35 ヤニ 脂
やにさが #R5 やに下が
やぬし #T35 家主
やね #T35*2 屋根
やの #JCN 矢野
やはぎ #JCN 矢作
やはた #JNS 八幡
やはたにし #CN 八幡西
やはたひがし #CN 八幡東
やはば #JCN 矢巾
やはり #CJ*64 やはり
やはん #T35 夜半
やば #KY やば ヤバ
やばけい #CN 耶馬渓
やばん #T05*1 野蛮
やひこ #JCN 弥彦
やぶ #R5r*3 破 #T35 薮 #JCN 養父
やぶからぼう #T15 薮から棒
やぶき #JCN 矢吹
やぶさか #T05*2 やぶさか
やぶへび #T15 薮蛇
やぶれ #KS 敗れ
やぶれかぶれ #T05 破れかぶれ
やべ #JCN 矢部
やぼ #T05*2 野暮 #T05*1 ヤボ
やぼう #T35 野望
やぼった #KY やぼった
やま #T35*80 山 #CNSUC1*53 山 #JCN 耶麻
やまあい #T35 山間
やまい #T35*1 病 #JNS 山井
やまうち #JCN 山内
やまえ #JCN 山江
やまおか #JCN 山岡
やまおく #T35*1 山奥
やまかわ #JNS 山川 #T35 山川
やまが #JNS 山賀 #JCN 山香 山鹿
やまがた #JCN 山形 山県 山方
やまがみ #JNS 山上
やまがら #T35 ヤマガラ 山雀
やまがわ #JCN 山川 #T35 山側
やまきた #JCN 山北
やまぎし #JNS 山岸
やまくずれ #T30 山崩れ
やまくに #JCN 山国
やまぐち #JCN 山口
やまぐに #T35 山国
やまこし #JCN 山越 #JNS 山古志
やまごや #T35 山小屋
やまさき #JNS 山崎
やまざき #JCN 山崎
やまざくら #T35 山桜
やまざと #T35*1 山里
やまし #KY やまし #T35 山師 #KY 疚し
やました #JCN 山下
やましな #JCN 山科
やましゅう #KYU やましゅう 疚しゅう
やましろ #JCN 山城
やますそ #T35 山裾
やまぞえ #JCN 山添
やまだ #JCN 山田
やまつり #JCN 矢祭
やまづみ #T10*2 山積み
やまて #T35*3 山手 #JCN 山手
やまと #T35*7 大和 #JCN 山都 #JNS 山門 #JCN 大和
やまとこおりやま #CN 大和郡山
やまとたかだ #CN 大和高田
やまなか #JCN 山中
やまなかこ #CN 山中湖
やまなし #JCN 山梨
やまね #JCN 山根
やまの #JCN 山野
やまのうち #JCN 山ノ内 #JNS 山之内
やまのくち #JCN 山之口
やまのて #T35*2 山の手
やまのべ #JNS 山辺
やまのぼり #T30*2 山登り
やまは #KK*1 ヤマハ
やまはだ #T35 山肌
やまば #T35 山場
やまべ #JCN 山辺
やまむら #JNS 山村
やまもと #JCN 山元 山本
やまもり #JNS 山森 #T10 山盛り
やまやま #T35*4 山々
やまわけ #T10 山分け
やみ #T35*2 闇
やみうち #T30 闇討ち
やみつき #T35*1 病み付き
やむちゃ #T35*1 飲茶
やむな #KY やむな
やむのう #KYU やむのう
やむをえ #KS*2 やむを得
やめ #KS*29 止め #KS*4 辞め #JCN 八女
やもと #JCN 矢本
やや #F14*18 やや
ややこし #KY*4 ややこし
ややこしゅう #KYU*1 ややこしゅう
やゆ #T30 揶揄
やよい #T35*2 弥生 #JCN 弥生
やり #T35*3 槍 #T35 鑓
やりがい #T35*2 やりがい
やりがだけ #CN 槍ヶ岳
やりくり #T30*1 やりくり
やりすご #S5r やり過ご
やりだま #T35*1 やり玉
やりて #T35 やり手
やりとり #T30*20 やりとり #T30*5 やり取り
やりなお #S5r*8 やり直
やるせな #KY やるせな
やるせのう #KYU やるせのう
やろう #T35*6 野郎
やわ #T05 やわ
やわた #JCN 八幡
やわたはま #CN 八幡浜
やわはだ #T35 柔肌
やわはら #JNS 谷和原
やわら #JCN 谷和原 #G5r 和ら
やわらか #KY*4 軟らか #T05*2 軟らか #KYmi 柔らか #T05 柔らか
やわらこう #KYU*1 軟らこう #KYU 柔らこう
やんく #T30*1 ヤンク
やんぐ #T05*1 ヤング
やんわり #F04 やんわり
ゆ #T35*65 湯 #K5r*45 ゆ #K5r*22 行 #W5*13 結 #KJ*3 由 #KJ*1 愉 愈 油 輸 柚 #R5*1 揺 #KJ*1 喩 #K5 往 逝 #KJ 癒 諭
ゆーくりっど #JN ユークリッド
ゆーざ #T35*255 ユーザ
ゆーざー #T35*158 ユーザー
ゆーざーず #T35*5 ユーザーズ
ゆーてぃりてぃ #T35*91 ユーティリティ
ゆーとぴあ #T35 ユートピア
ゆーもあ #T35 ユーモア
ゆーもらす #T05*1 ユーモラス
ゆあさ #JCN 湯浅 #JNS 湯淺
ゆい #KJ*8 唯 #KJ 遺 #JNM 唯 #JCN 由比
ゆいいつ #T35*5 唯一
ゆいがどくそん #T15*1 唯我独尊
ゆいごん #T30*1 遺言
ゆいしょ #T35*1 由緒
ゆいのう #T35*3 結納
ゆいぶつ #T35 唯物
ゆう #S5*94 有 #T30*94 有 #T35*21 夕 #KJ*13 裕 #KJ*10 憂 #KJ*7 遊 #T35*3 優 #KJ*3 雄 #KJ*2 勇 融 #KJ*1 友 悠 由 祐 誘 #KJ 右 酉 佑 宥 幽 揖 柚 湧 涌 猶 猷 #JCN 由宇 #KJ 邑 郵
ゆうい #T05*7 優位 #T05*2 有意
ゆういぎ #T05*3 有意義
ゆういち #JNM 勇一 祐一 裕一 雄一
ゆういちろう #JNM 祐一郎
ゆういん #T30 誘引
ゆううつ #T05*1 憂鬱
ゆうえき #T05*6 有益
ゆうえつ #T30*1 優越
ゆうえん #JNS 結縁
ゆうえんち #T35*3 遊園地
ゆうか #T35*1 有価
ゆうかい #T05*1 有界 #T30*1 誘拐 #T30 融解
ゆうかん #T05*10 勇敢 #T35*1 夕刊
ゆうが #T05*1 優雅
ゆうがい #T05*1 有害
ゆうがく #T30 遊学
ゆうがた #T35*26 夕方
ゆうき #T35*4 勇気 #T35*2 有機 #T30*1 誘起 #JCN 結城 #JNM 優起 祐希
ゆうきち #JNM 勇吉
ゆうきゅう #T35*31 有給 #T05*1 悠久 #T35*1 有休 遊休
ゆうぐう #T30*1 優遇
ゆうぐれ #T35*1 夕暮れ
ゆうけい #T15 有形
ゆうけんしゃ #T35 有権者
ゆうげん #T05*12 有限
ゆうげんがいしゃ #T35 有限会社
ゆうこ #JNM 優子 由子 祐子 裕子
ゆうこう #T05*101 有効 #T35*2 友好 #T05*2 有向
ゆうこく #T35*1 夕刻
ゆうこん #T05 勇渾
ゆうごう #T30*4 融合
ゆうざい #T35*1 有罪
ゆうし #T35*6 有志 #T30*2 融資
ゆうしきしゃ #T35*2 有識者
ゆうしてっせん #T35 有刺鉄線
ゆうしゅう #T05*6 優秀
ゆうしゅつ #T30 湧出
ゆうしゅん #T35 優駿
ゆうしょう #T30*24 優勝 #T35*10 有償
ゆうしょく #T35*13 夕食
ゆうしんろん #T35 有神論
ゆうじ #JNM 勇次 勇治 勇二 祐司 祐二 裕司 裕史 裕治 裕二 雄二
ゆうじゅうふだん #T35*2 優柔不断
ゆうじょ #T30 宥恕 #T35 遊女
ゆうじょう #T35*2 友情
ゆうじろう #JNM 裕次郎 雄二郎
ゆうじん #T35*22 友人 #T35 有人
ゆうすう #T05*1 有数
ゆうすずみ #T30 夕涼み
ゆうずう #T30*3 融通
ゆうずうむげ #T05*2 融通無碍
ゆうせい #T35*8 郵政 #T35*2 有生 #T05*1 優勢 #KK*1 郵省 #T35*1 雄性 #T35 優生 遊星
ゆうせいしょう #KK*1 郵政省
ゆうせつ #T30 融雪
ゆうせん #T30*43 優先 #T35*1 有線
ゆうぜい #T35 有税 #T30 遊説
ゆうぜん #F00*1 悠然 #T35 友禅
ゆうそう #T30*4 郵送 #T05 雄壮
ゆうぞう #JNM 勇三 勇蔵 雄三
ゆうたい #T30 優待
ゆうだい #T05*1 雄大
ゆうだち #T35*2 夕立ち
ゆうだん #T30*1 勇断
ゆうち #T30*1 誘致
ゆうちょ #T35 郵貯
ゆうちょう #T05*1 悠長
ゆうづう #T30 融通
ゆうとう #T05 優等 #JCN 雄踏
ゆうどう #T30*2 誘導
ゆうどく #T05 有毒
ゆうのう #T05 有能
ゆうはつ #T30 誘発
ゆうはん #T35*3 夕飯
ゆうばり #CN 夕張
ゆうひ #T35*1 夕日 夕陽
ゆうひかく #KK*1 有斐閣
ゆうび #T05 優美
ゆうびん #T35*79 〒 #T35*20 郵便
ゆうびんきょく #T35*3 郵便局
ゆうびんぶつ #T35*1 郵便物
ゆうふく #T05*1 裕福
ゆうふつ #CN 勇払
ゆうへい #JNM 裕平
ゆうべ #T35*3 ゆうべ #T35*2 昨夜 #T35*1 夕べ
ゆうべつ #CN 湧別
ゆうべん #T05*1 雄弁
ゆうほう #T35 友朋
ゆうほどう #T35*1 遊歩道
ゆうぼう #T05*3 有望
ゆうぼく #T30 遊牧
ゆうめい #T05*18 有名
ゆうめし #T35*1 夕飯
ゆうや #JNM 雄也
ゆうやく #T30 勇躍 #T35 釉薬
ゆうやけ #T35*1 夕焼け
ゆうやみ #T35*1 夕闇
ゆうゆう #F00 悠々
ゆうゆうじてき #T30*1 悠々自適
ゆうよ #T30*1 猶予 #T35 有余
ゆうよう #T05*4 有用 #F02 悠揚
ゆうよく #T30 游弋
ゆうらくちょう #CN 有楽町
ゆうらん #T30*1 遊覧
ゆうり #T35*22 有理 #T05*11 有利 #T30*1 遊離
ゆうりすう #T35*1 有理数
ゆうりょ #T30*1 憂慮
ゆうりょう #T15*2 有料 #T05 優良
ゆうりょく #T05*4 有力
ゆうれい #T35*2 幽霊
ゆうれつ #T35*1 優劣
ゆうわ #JCN 雄和 #T30 融和
ゆうわく #T30*1 誘惑
ゆえ #T35*16 故 #T35*1 ゆえ #KS 結え
ゆえに #T35*1 ∴
ゆえん #JNM 結縁
ゆか #T35*3 床 #JNM 由佳 由香
ゆかい #T05*1 愉快
ゆかうえ #T35*2 床上
ゆかした #T35*3 床下
ゆかた #T35 浴衣
ゆかり #JNM ゆかり 由佳利
ゆかわ #JCN 湯川
ゆかん #T30 湯潅
ゆが #M5r*3 ゆが #M5r*1 歪 #K5r 湯が
ゆがわ #JCN 湯川
ゆがわら #CN 湯河原
ゆき #T35*40 雪 #T35*4 幸 #KJ*3 之 #CNSUC1*1 行 #JNM ユキ #KJ 倖 #JCN 湯来 油木 由岐 #JNM 由紀 由貴 由姫 裕紀
ゆきえ #JNM 雪絵 由紀江
ゆきお #JNM 幸男 幸夫 幸雄 行夫 行雄 由紀夫 由起夫
ゆきか #W5r 行き交
ゆきき #T30 行き来
ゆきぐに #T35*1 雪国
ゆきこ #JNM 幸子 雪子 由紀子
ゆきち #JNM 諭吉
ゆきとど #K5r ゆきとど 行き届
ゆきどけ #T35*1 雪解け
ゆきはる #JNM 幸春
ゆきひこ #JNM 幸彦
ゆきひと #JNM 幸仁
ゆきまさ #JNM 幸真
ゆきや #JNM 幸也
ゆくえ #T35*3 行方
ゆくはし #JCN 行橋
ゆくゆく #F14 ゆくゆく
ゆけつ #T30 輸血
ゆげ #JCN 弓削 #T35 湯気
ゆさ #JNS 遊佐
ゆさぶ #R5r*1 揺さぶ
ゆざまし #T35 湯冷まし
ゆざわ #JCN 湯沢
ゆし #T35 油脂
ゆしゅつ #T30*4 輸出
ゆしゅつにゅう #T35 輸出入
ゆす #R5r*1 ゆす #G5r 濯
ゆすはら #JCN 梼原
ゆず #R5*7 譲 #T35*3 柚 #KJ*1 梼
ゆずりあ #W5r 譲り合
ゆずる #JNM 譲
ゆせい #T35 油井 油性
ゆせいかん #T35*1 輸精管
ゆそう #T30*2 輸送
ゆたか #T05*6 豊か #JNM 泰 #JCN 豊 #JNM 裕 穰
ゆたんぽ #T35 湯たんぽ
ゆだ #JCN 湯田
ゆだね #KS*2 委ね
ゆだん #T30*1 油断
ゆちゃく #T30 癒着
ゆっくり #F04*14 ゆっくり
ゆったり #F04*2 ゆったり
ゆづかみ #CN 湯津上
ゆづき #JNS 弓月
ゆで #KSr*2 ゆで #KSr 茹で
ゆでん #T35 油田
ゆとり #T35*16 ゆとり
ゆに #PRE*1 ユニ #CN 由仁
ゆにーく #T05*19 ユニーク
ゆにしす #KK*4 ユニシス
ゆにっくす #KK*3 ユニックス
ゆにっと #T35*18 ユニット
ゆにばーさる #T05*2 ユニバーサル
ゆにばーす #T35*2 ユニバース
ゆにふぃけーしょん #T30*4 ユニフィケーション
ゆにふぉーむ #T35 ユニフォーム
ゆにゅう #T30*3 輸入
ゆのたに #CN 湯之谷
ゆのつ #JCN 温泉津
ゆのまえ #JCN 湯前
ゆのみ #T35*2 湯飲み
ゆはら #JNS 湯原
ゆば #JNS 弓場
ゆばら #JCN 湯原
ゆび #T35*14 指
ゆびさ #S5r*2 指さ
ゆびさき #T35*1 指先
ゆびわ #T35*4 指輪
ゆふいん #CN 湯布院 由布院
ゆぶいん #KK*1 湯布院
ゆぶね #T35 湯舟
ゆぷしろん #T35*2 υ #T35*1 Υ
ゆま #JNS 由真
ゆまく #T35*1 油膜
ゆみ #T35*4 弓 #JNM ゆみ 友美 有美 由実 由美
ゆみこ #JNM 弓子 由美子
ゆめ #T35*23 夢 #T35*10 ゆめ
ゆめさき #JCN 夢前
ゆめみ #KSr*1 夢見
ゆもと #JCN 湯本
ゆや #JCN 油谷
ゆよく #T35 湯浴
ゆら #G5r*1 揺ら #JCN 由良
ゆらい #T30*4 由来
ゆり #T35*2 百合 #JNM 由利
ゆりかご #T35*2 ゆりかご
ゆりこ #JNM 百合子 由利子 由里子
ゆる #S5r*68 許 #KY*5 緩 #M5r*4 緩 #G5r 揺る
ゆるう #KYU*1 緩う
ゆるめ #KS 緩め
ゆるやか #T05*2 緩やか
ゆれ #KSr*4 揺れ #KSr*2 ゆれ
ゆわえ #KS 結わえ
ゆわかし #T35*1 湯沸し
よ #B5*255 呼 #M5r*255 読 #KY*255 良 #KY*191 よ #T35*119 世 #R5*94 よ #S5*70 よ #W5r*68 酔 #R5r*64 寄 #NN*19 四 #T35*9 夜 #B5*8 よ #R5*4 依 #KJ*4 予 #KY*3 好 #M5*2 詠 #KY*2 善 #T35*2 代 余 #KJ*1 与 #R5 因 #KY 佳 #S5 止 #R5 選 #R5r 撚 #KJ 誉 輿 預
よーぐると #T35*1 ヨーグルト
よーどん #KK*1 ヨードン
よーろっぱ #CN ヨーロッパ
よあけ #T35*2 夜明け
よい #T35*33 宵
よいしょ #CJ よいしょ
よいた #JCN 与板
よいち #CN 余市
よいっぱり #T15 宵っ張り
よいん #T35*1 余韻
よう #T35*255 用 #T05*222 よう #S5*154 要 #T30*154 要 #T05*111 様 #T35*29 陽 #T35*8 洋 #D2T35*3 よう #KYU*3 よう 良う #KJ*2 妖 容 曜 養 #KYU*1 好う 善う #KJ*1 揚 揺 擁 #S5*1 擁 #SX*1 擁 #KJ*1 溶 熔 葉 蓉 瘍 #T35*1 癰 #KJ 厭 #KYU 佳う #KJ 腰 傭 幼 庸 楊 #SUC 用 #KJ 窯 羊 耀 謡 踊 遥
ようい #T30*255 用意 #T05*64 容易
よういおん #T35 陽イオン
よういく #T30*1 養育
よういち #JNM 洋一 要一 陽一
よういちろう #JNM 洋一郎 陽一郎
よういん #T35*12 要因 #T35*3 要員
ようえき #T35 溶液
ようおん #T35*1 拗音
ようか #T35 8日 ８日 #JCN 八鹿 #T35 八日
ようかい #T30 容喙 溶解
ようかいち #CN 八日市
ようかいちば #CN 八日市場
ようかん #T35 洋館 羊羹
ようが #T35*1 洋画 #JCN 用賀
ようがし #T35 洋菓子
ようき #T35*3 容器 #T05*3 陽気
ようきゅう #T30*232 要求
ようきょく #T35*2 陽極 #T35 謡曲
ようぎ #T35*1 容疑
ようぎょう #T35 窯業
ようぎん #T35 洋銀
ようぐ #T35*4 用具
ようけい #T35 養鶏
ようけつ #T30*1 溶血
ようけん #T35*4 用件 #T35*3 要件
ようげん #T35*16 用言
ようこ #JNM 容子 揚子 洋子 葉子 陽子
ようこう #T35*2 要項 陽光 #T30*1 洋行
ようこうろ #T35 溶鉱炉
ようご #T35*40 用語 #T30*10 養護 #T30*1 擁護
ようさい #T35*1 要塞 #T35 洋裁
ようさん #T35 養蚕
ようし #T35*39 用紙 #T35*3 陽子 #T35*1 容姿 要旨 #T35 養子
ようしき #T35*2 様式 #T35 洋式
ようしつ #T35*1 洋室
ようしゃ #T30*6 容赦
ようしゃな #KY 容赦な
ようしゅ #T35 洋酒
ようしゅん #T35 陽春
ようしょ #T35*3 洋書 #T35*1 要所
ようしょう #T05 幼少 #T35 要衝
ようしょく #T35*4 洋食 #T30*1 養殖 #T35 容色 要職
ようじ #T35*23 用事 #T35*10 幼児 #JNM 洋司 洋次 洋二 陽治
ようじゃく #T35*1 幼若
ようじゅつ #T35*1 妖術
ようじょ #T35*4 幼女 #T35 養女
ようじょう #T35 洋上 #T30 養生
ようじん #T30*2 用心 #T35*1 要人
ようす #T35*28 様子
ようすい #T35*1 用水 #JNM 陽水
ようせい #T30*8 要請 #T30*3 養成 #T35*1 妖精 #T35 陽性 #T30 夭逝
ようせき #T35 容積
ようせつ #T30*1 夭折 #T30 溶接
ようせん #T35 用箋
ようそ #T35*76 要素
ようそう #T35*2 様相
ようだい #T35 容体
ようち #T05*1 幼稚 #T35 用地
ようちえん #T35*2 幼稚園
ようちゅう #T35*2 幼虫
ようちょう #F02 羊腸
ようつい #T05*4 要対
ようてい #T35 要諦
ようてん #T35*4 要点
ようでんし #T35*1 陽電子
ようと #T35*9 用途 #F13 杳と
ようとん #T35 養豚
ようど #T35 用度
ようにん #T30*3 容認
ようねん #T35 幼年
ようひし #T35 羊皮紙
ようひん #T35*3 洋品
ようび #T35*6 曜日
ようふ #T35*1 養父
ようふう #T05 洋風
ようふく #T35*3 洋服
ようふぼ #T35*1 養父母
ようぶん #T35*1 養分
ようほう #T35*6 用法
ようぼう #T30*26 要望 #T35 容貌
ようま #T35 洋間
ようまく #T35*1 羊膜
ようむ #T35 用務
ようめい #T35 用命
ようもう #T35 羊毛
ようもの #T35 洋物
ようやく #F14*18 ようやく #T30*2 要約
ようゆう #T30*2 熔融
ようらん #T35*1 要覧
ようらんき #T35 揺籃期
ようりょう #T35*78 容量 #T35*10 要領
ようりょくそ #T35 葉緑素
ようりょくたい #T35 葉緑体
ようれい #T35 用例
ようれき #T35 陽暦
ようろう #CN 養老 #T35 養老
ようろく #T35 要録
よか #T35*1 余暇
よかく #T35*52 与格
よかわ #JCN 吉川
よかん #T30*1 予感
よき #T30*3 予期
よきょう #T35*1 余興
よきん #T30*9 預金
よぎな #KY*1 余儀な
よく #T35*13 欲 #F14*5 良く #KJ*3 翌 #KJ*1 翼 #F14 よく #T35 慾 #KJ 抑 沃 #S5 浴 #T30 浴
よくあつ #T30 抑圧
よくげつ #T35*1 翌月
よくし #T30*1 抑止
よくしつ #T35 浴室
よくしゅう #T35 翌週
よくじつ #T35*9 翌日
よくじょう #T35 浴場
よくせい #T30*8 抑制
よくそう #T35 浴槽
よくちょう #T35*1 翌朝
よくとく #T35 欲得
よくとし #T35*1 翌年
よくど #T35 沃土
よくねん #T35*1 翌年
よくば #R5r 欲張
よくぼう #T35*2 欲望
よくよう #T35 抑揚 浴用
よくよく #PRE*4 翌々 #F14*2 よくよく #T36 よくよく
よくよくじつ #T35 翌々日
よくりゅう #T30*1 抑留
よけ #KS*1 避け
よけい #T05*16 余計
よげん #T30*1 予言 預言 #T35 余弦
よこ #T35*44 横
よこい #JNS 横井
よこう #T35*13 予行 #T35*2 予稿
よこお #JNS 横尾
よこかわ #KK*1 横河 #JCN 横川
よこがお #T35 横顔
よこがわでんき #KK*2 横河電機
よこぎ #R5r 横切
よこく #T30*13 予告
よこけい #T35*1 ─ ━
よこごし #JCN 横越
よこざま #T05*1 横様
よこしば #JCN 横芝
よこしま #T05*1 よこしま #JCN 横島 #T05 邪
よこじま #T35 横縞
よこすか #JCN 横須賀
よこせ #JCN 横瀬
よこた #JCN 横田
よこたわ #R5*1 横たわ
よこづな #T35*2 横綱
よこて #T35*2 横手 #JCN 横手
よこどり #T30*2 横取り
よこなが #T35*1 横長
よこはま #JCN 横浜
よこばい #T10 横ばい
よこひら #JNS 横平
よこみぞ #JNS 横溝
よこみち #T35 横道
よこめ #T35*1 横目
よこやま #JCN 横山
よこれんぼ #T30*1 横恋慕
よご #S5*1 汚 #JCN 余呉
よごれ #KSr*1 汚れ
よさ #JCN 与謝
よさそう #T05*2 よさそう 良さそう
よさぶろう #JNM 与三郎
よさん #T35*22 予算
よざ #JNS 与謝
よし #T35*23 由 #KJ*6 佳 #T35*5 好 #KJ*4 義 吉 #KJ*1 善 禎 芳 #JNM よし ヨシ #T35 葦 #JNM 好 #KJ 美 #JNM 良
よしあき #JNM 義昭 義明 吉明 美明 芳明 良昭 良明
よしあし #T35*3 善し悪し
よしい #JCN 吉井 #JNS 芳井
よしいち #JNM 義一
よしうみ #JCN 吉海
よしお #JNM 嘉男 義男 義夫 義雄 吉男 吉夫 吉雄 好男 好夫 好雄 善雄 芳男 芳夫 芳雄 由夫 良男 良夫 良雄
よしおか #JCN 吉岡
よしかず #JNM 英一 義一 義和 好一 良和
よしかつ #JNM 義勝 好克
よしかわ #JCN 吉川
よしき #JCN 吉城 吉敷 #JNM 美樹 芳樹
よしこ #JNM よし子 ヨシ子 佳子 嘉子 好子 美子 芳子 良子
よしざき #JCN 吉崎
よしざわ #JNS 吉沢
よしじ #JNM 良治
よしずみ #JNS 吉住
よしたか #JNM 義孝 義隆
よしたろう #JNM 芳太郎 由太郎
よしだ #JCN 吉田
よしつぐ #JNM 善次
よしと #JNM 義人
よしとみ #JCN 吉富
よしなが #JCN 吉永
よしなり #JNM 善成
よしの #JCN 吉野
よしのだに #JCN 吉野谷
よしのぶ #JNM 栄伸 義信
よしのり #JNM 佳典 義則
よしはる #JNM 喜晴 義治
よしひこ #JNM 義彦 吉彦 芳彦 良彦
よしひさ #JNM 義久 芳久
よしひと #JNM 芳人
よしひろ #JNM 喜弘 義弘 美博
よしほ #JNM 美穂
よしまつ #JCN 吉松 #JNS 由松
よしみ #JNM 義美 #JCN 吉見 #JNM 好美 芳美 由美
よしみつ #JNM 義光
よしむら #JNS 吉村
よしもと #JNS 吉本
よしや #JNM 義哉
よしゅう #T30 予習
よしゆき #JNM 嘉之 義行 義之 善之 良之
よしろう #JNM 義郎 芳郎 与四郎
よしわ #JCN 吉和
よしわら #JCN 吉原
よしん #T35 余震
よしんば #CJ よしんば
よじょう #T05 余剰 #T35 余情
よじん #T35 余人
よせ #KSr*10 寄せ #T35*1 寄席
よせい #T35 余生
よせがき #T30*1 寄書き
よせん #T35*16 予選 #T35*2 予餞
よそ #T35*11 よそ
よそう #T30*40 予想
よそうがい #T05*1 予想外
よそお #W5r 装
よそく #T30*22 予測
よそじ #T35 四十路
よぞら #T35 夜空
よたく #T30 預託
よだ #JNS 依田
よだれ #T35 よだれ
よだん #T35*8 余談 #T30 予断
よち #T35*10 余地 #T30*2 予知
よちょう #T35 予兆
よちょきん #T30 預貯金
よっか #T35*1 四日 #T35 4日 ４日
よっかいち #CN 四日市
よっきゅう #T30*2 欲求
よっつ #T35*2 四つ
よっぱら #W5r*7 酔っ払
よっぽど #T05*3 よっぽど
よつかいどう #CN 四街道
よつかど #T35 四つ角
よつぎ #T35 世継
よつや #CN 四ツ谷 #JCN 四谷
よてい #T30*255 予定
よとう #T35*1 与党
よど #M5r*1 淀
よどえ #JCN 淀江
よどがわ #JCN 淀川
よどやばし #CN 淀屋橋
よなか #T35*5 夜中
よなぐすく #CN 与那城
よなぐに #CN 与那国
よなご #CN 米子
よなばる #CN 与那原
よね #KJ*2 米 #JNM よね
よねきち #JNM 米吉
よねこ #JNS 米子
よねざき #JNS 米崎
よねざわ #JCN 米沢 #JNS 米澤
よねだ #JNS 米田
よねつ #T35 余熱
よねづ #JNS 米津
よねやま #JCN 米山
よねん #T35 余念
よの #JCN 与野
よのうづ #JCN 米水津
よのなか #T35*18 世の中
よは #T35*1 余波
よはく #T35*2 余白
よばい #T30*1 夜這い
よばわり #N2T30*2 呼ばわり
よび #T35*30 予備
よびかけ #KSr*2 呼びかけ #T30*1 呼掛け
よびぐん #T35 予備軍
よびこ #M5r 呼び込
よびこう #T35 予備校
よびごえ #T35 呼び声
よびだ #S5r*139 呼び出
よびだし #T35*19 呼出 #T30*6 呼出し
よびもど #S5r*1 呼び戻
よびよせ #KSr*1 呼び寄せ
よふかし #T10 夜更し
よふけ #T35 夜更け
よぶこ #JCN 呼子
よぶん #T05*7 余分
よほう #T30*3 予報
よほど #F14*3 よほど #T36 よほど
よぼう #T30*2 予防
よまいごと #T35 世迷い言
よみあげ #KSr*1 読み上げ
よみうり #KK*2 読売
よみかき #T30*18 読み書き
よみかた #T35*6 読み方
よみがえ #R5r*2 甦 #R5r 蘇
よみがな #T35*13 読みがな
よみこ #M5r*153 読み込
よみたに #JCN 読谷
よめ #T35*7 嫁
よもぎ #T35 蓬
よもぎた #JCN 蓬田
よやく #T30*87 予約
よゆう #T35*21 余裕
よよぎ #JCN 代々木
より #F14*54 より
よりあ #W5r 寄り合
よりい #JCN 寄居
よりかか #R5r*1 寄りかか 寄り掛か
よりこ #JNM 依子 頼子
よりしま #JCN 寄島
よりそ #W5r 寄り添
よりどころ #T35*2 拠り所
よりみち #T30*1 寄り道
よりょく #T35*1 余力
よる #T35*114 夜
よろい #T35*1 鎧
よろく #T35 余録
よろこ #B5r*22 喜 #B5r*1 よろこ #B5r 悦 歓 慶
よろこばし #KY*1 喜ばし
よろこばしゅう #KYU*1 喜ばしゅう
よろし #KY*255 よろし
よろしく #F14*255 宜しく #F14*37 よろしく
よろしゅう #KYU*3 よろしゅう
よろず #T35 万
よろん #T35 世論 #JCN 与論
よわ #KYmi*17 弱 #R5*11 弱
よわい #T35 齢
よわき #T05*2 弱気
よわごし #T15 弱腰
よわね #T35 弱音
よわま #R5r 弱ま
よわみ #T35 弱味
よわめ #KS*1 弱め
よわよわし #KY*1 弱々し
よん #NN*37 四
よんどころな #KY 拠ん所な
よんどころのう #KYU 拠ん所のう
ら #JNSUC*191 ら #KJ*5 螺 #KJ*2 羅 裸 #JNSUC 等 #JN 羅
らーめん #T35*3 ラーメン
らーゆ #T35 辣油
らい #SUC*58 来 #KJ*4 磊 #KJ*1 来 雷 #KJ 莱 頼
らいう #T35 雷雨
らいえん #T30 来演
らいおん #T35*4 ライオン
らいか #T35 来夏
らいかい #T30 来会
らいかん #T30 来館
らいき #T35*1 来季 #T35 来期
らいきゃく #T35 来客
らいげつ #T35*11 来月
らいこう #T35 来光 #T30 来校 来航
らいごう #T30 来降
らいさん #T30*1 礼賛 #T30 礼讃
らいしゃ #T30 来社
らいしゅう #T35*85 来週 #T35 来秋 #T30 来襲
らいしゅん #T35*1 来春
らいしょ #T30 来所
らいしん #T30 来信 来診
らいじょう #T30*1 来場
らいす #T35*2 ライス
らいせ #T35*1 来世
らいせんす #T30*12 ライセンス
らいた #T35*8 ライタ
らいたー #T35*7 ライター
らいたく #T30 来宅
らいちょう #T35*1 雷鳥 #T30 来聴
らいてぃんぐ #T30*1 ライティング
らいてん #T30 来店
らいと #T35*18 ライト
らいにち #T30*2 来日
らいねん #T35*31 来年
らいねんど #T35*4 来年度
らいはい #T30*1 礼拝
らいばる #T35*1 ライバル
らいひん #T35 来賓
らいふ #T35*5 ライフ
らいぶ #T35*12 ライブ
らいぶらり #T35*255 ライブラリ
らいぶらりあん #T35*1 ライブラリアン
らいほう #T30*1 来訪
らいめい #T35 雷鳴
らいらく #T05 磊落
らいらっく #T35 ライラック
らいん #T35*68 ライン
らうす #CN 羅臼
らうんど #T35*1 ラウンド
らうんどろびん #T35*1 ラウンドロビン
らがん #T35 裸眼
らく #T05*49 楽 #T00*27 楽 #KJ*1 洛 絡 #KJ 落 酪
らくえん #T35*1 楽園
らくがき #T30*1 落書 #T30 落書き
らくがん #T35 落雁
らくご #T35*1 落語 #T30 落後
らくさ #T35*1 落差
らくさつ #T30 落札
らくしゅ #T30 落手
らくしょう #T10*1 楽勝
らくじつ #T35 落日
らくせい #T30 落成
らくせき #T30 落石
らくせん #T30 落選
らくたん #T30*1 落胆
らくだ #T35 駱駝
らくだい #T30*1 落第
らくちゃく #T30*1 落着
らくちょう #T35 落丁
らくてん #T35*1 楽天
らくのう #T35 酪農
らくはく #T30 落魄
らくば #T30 落馬
らくばん #T30 落磐
らくもいど #T35*1 ラクモイド
らくよう #T30*1 落葉 #JNS 洛陽
らくらい #T30*1 落雷
らくらく #F14 らくらく 楽々
らくるい #T30 落涙
らぐびー #T35*1 ラグビー
らけっと #T35*10 ラケット
らしゅつ #T30 裸出
らしんばん #T35 羅針盤
らじうむ #T35 ラジウム
らじお #T35*4 ラジオ
らじおしてぃ #T35*4 ラジオシティ
らじかせ #T30 ラジカセ
らじかる #T05 ラジカル
らすた #T35*7 ラスタ
らすと #T35*3 ラスト
らせん #T35 螺旋
らち #T35 埓 #T30 拉致
らっか #T30*1 落下
らっかん #T30*3 楽観
らっきー #T05*12 ラッキー
らっきょ #T35 らっきょ
らっきょう #T35 らっきょう
らっこ #T35*1 ラッコ
らっしゃ #L5*1 らっしゃ
らっしゅ #T35*1 ラッシュ
らっち #T30*1 ラッチ
らっと #T35*1 ラット
らっぱ #T35*1 ラッパ
らつ #KJ*1 辣
らつわん #T05 辣腕
らでぃかる #T05 ラディカル
らでん #T35 螺鈿
らびおり #T35*1 ラビオリ
らびりんす #T35*1 ラビリンス
らふぉーれ #KK*1 ラフォーレ
らぶ #T35*7 ラブ
らべる #T35*19 ラベル #JN ラベル
らぼ #T35*62 ラボ
らむだ #T35*13 λ #T35*1 Λ
らりー #T30*2 ラリー
られつ #T30*1 羅列
らん #T35*57 欄 #T35*4 ラン #T35*2 乱 #T35*1 蘭 覧 #KJ*1 爛 鸞 #T35 卵 #KJ 嵐 濫 藍
らんおう #T35 卵黄
らんおうのう #T35*1 卵黄嚢
らんかく #T30 乱獲 #T35 卵殻
らんかつ #T35*1 卵割
らんかん #T35*1 卵管 欄干
らんかんさい #T35*1 卵管采
らんがい #T35*1 欄外
らんきゅう #T35*1 卵丘
らんきりゅう #T35 乱気流
らんきんぐ #T30 ランキング
らんぎょう #T35 乱行
らんく #T30 ランク
らんげーじ #T35*1 ランゲージ
らんこう #T30 乱交
らんこうげ #T30 乱高下
らんこし #CN 蘭越
らんこん #T30 乱婚
らんざつ #T05 乱雑
らんざん #JCN 嵐山
らんし #T35 乱視 卵子
らんしゃ #T30 乱射
らんじゅく #T30*1 爛熟
らんすう #T35*2 乱数
らんせん #T35 乱戦
らんそう #T35 卵巣
らんたいむ #T35*1 ランタイム
らんだむ #T05*10 ランダム
らんちょう #T35 乱丁
らんちょん #T35*1 ランチョン
らんとう #T30 乱闘
らんど #SUC*10 ランド
らんどく #T30 乱読
らんなー #T35*1 ランナー
らんにゅう #T30 乱入
らんばつ #T30 乱伐
らんぱく #T35 卵白
らんぱつ #T30*1 乱発
らんぴ #T30 乱費
らんぶ #T30*2 乱舞
らんぷ #T35*5 ランプ
らんぼう #T00*1 乱暴
らんぽ #JNM 乱歩
らんぽう #T35*1 卵胞
らんま #T35 欄間
らんまん #T05*1 爛漫
らんみゃく #T05 乱脈
らんよう #T30*1 乱用 濫用
らんらん #F02 爛々
らんりつ #T30*1 乱立
らんれんぐす #KK*1 ランレングス
り #T35*69 理 #JS*22 里 #S5*11 利 #T30*11 利 #KJ*7 履 #KJ*3 哩 李 #T35*3 里 #KJ*1 吏 #T35*1 裏 #KJ*1 離 #KJ 浬 #SUC 吏 #KJ 梨 璃 痢 裡
りーぐ #T35*2 リーグ
りーじょん #T35*7 リージョン
りーす #T30*2 リース
りーずなぶる #T05*1 リーズナブル
りーだ #T35*16 リーダ
りーど #T30*5 リード
りありずむ #T35*3 リアリズム
りある #T05*6 リアル
りあるたいむ #T05*10 リアルタイム
りいち #JNM 利一
りえ #JNM 利恵 理恵
りえき #T30*7 利益
りえこ #JNM 理恵子
りか #T35*6 理科 #JNM 利佳 里歌
りかーしぶ #T05 リカーシブ
りかーじょん #T35*1 リカージョン
りかい #T30*99 理解
りかばー #T30*2 リカバー
りかん #T30*1 罹患
りがい #T35*1 利害
りがく #T35*2 理学
りがんど #T35*1 リガンド
りき #SUC*4 力 #M5r 力 #T35 力
りきえん #T30 力演
りきお #JNM 力男 力雄
りきがく #T35*1 力学
りきさく #T35*2 力作
りきし #T35 力士
りきせつ #T30 力説
りきせん #T30 力戦
りきち #JNM 利吉
りきとう #T30 力闘
りきゅう #T35 離宮
りきりょう #T35*1 力量
りく #T35*3 陸
りくあげ #T30 陸揚げ
りくうん #T35 陸運
りくえすと #T30*39 リクエスト
りくじょう #T35*17 陸上
りくぜん #CN 陸前
りくぜんたかた #CN 陸前高田
りくち #T35 陸地
りくつ #T35*3 理屈
りくとう #T35*1 陸稲
りくべつ #CN 陸別
りくるーと #T30*2 リクルート
りくろ #T35 陸路
りけい #T35 理系
りけん #T35*1 利権
りこ #T35*1 利己
りこー #KK*5 リコー
りこう #T35*16 理工 #T30*8 履行
りこん #T30*1 離婚
りごう #T30*1 離合
りさーち #T30*3 リサーチ
りさいず #T30*2 リサイズ
りさぶろう #JNM 利三郎
りさん #T30*4 離散
りざーぶ #T30*3 リザーブ
りざると #T35*1 リザルト
りし #T35 利子
りしゅう #T30*1 履修
りしょく #T30*1 離職
りしり #CN 利尻
りしんろん #T35 理神論
りじ #T35*2 理事
りじぇくと #T30*3 リジェクト
りじゅん #T35 利潤
りす #T35*35 リス
りすく #T35*5 リスク
りすたーと #T30*1 リスタート
りすと #T30*219 リスト
りすとあ #T30*2 リストア
りすとあっぷ #T30*2 リストアップ
りすとら #T30 リストラ
りすぷ #KK*4 リスプ
りずむ #T35*1 リズム
りせい #T35*2 理性
りせっと #T30*9 リセット
りせぷた #T35*1 リセプタ
りそーす #T35*61 リソース
りそう #T35*7 理想
りそく #T35*1 利息
りそん #T30 離村
りぞっと #T35*1 リゾット
りた #T35*1 利他
りたーん #T30*53 リターン
りたいあ #T30*3 リタイア
りだいれくしょん #T30*2 リダイレクション
りだいれくと #T30*2 リダイレクト
りだくしょん #T30*4 リダクション
りだつ #T30 離脱
りち #T35*1 理知
りちうむ #T35*1 リチウム
りちぎ #T05*1 律義 #T05 律儀
りちゃくりく #T30 離着陸
りっ #S5 律 #SX 律
りっか #T35 立夏
りっきゃく #T30*1 立脚
りっきょう #T35 陸橋 #KK 立教
りっけん #T35 立憲
りっこうほ #T30*1 立候補
りっし #T30 立志
りっしゅう #T35 立秋
りっしゅん #T35*1 立春
りっしょう #T30*2 立証
りっしょく #T30*2 立食
りっしん #T30*1 立身
りっそく #T35*2 律速
りったい #T35*1 立体
りっち #T05*1 リッチ
りっとう #CN 栗東 #T35 立冬
りっとる #JS*9 リットル #T35*2 リットル
りっぱ #T05*9 立派
りっぷく #T30 立腹
りっぷる #T35*1 リップル
りっぽう #JS*1 立方 #T30*1 立法
りっぽうたい #T35*2 立方体
りつ #T35*10 率 #KJ*2 律 #SUC*2 立 #KJ*1 慄 #T35 律 #SUC 率 #KJ 立 #T35 葎
りつあん #T30*1 立案
りつきょう #KK*1 立教
りつこ #JNM 利津子 理津子
りつぜん #F00 慄然
りつぞう #T35 立像
りつどう #T30 律動
りつめいかん #KK*2 立命館
りつろん #T30 立論
りてらる #T05*42 リテラル
りてん #T35*32 利点
りとあにあ #CN リトアニア
りとう #T30 離島
りとく #T35 利得
りとる #T05*1 リトル
りにあ #T05*1 リニア
りにゅう #T30 離乳
りねん #T35*6 理念
りのう #T30 離農
りはっちゃく #T30 離発着
りはつ #T05 利発 #T30 理髪
りはびり #T30 リハビリ
りばーす #T30*17 リバース
りびじょん #T35*19 リビジョン
りぴーた #T35*22 リピータ
りふ #CN 利府
りふぁれんす #T30*38 リファレンス
りふじん #T05*1 理不尽
りふと #T35*6 リフト
りふれいん #T30 リフレイン
りぶーと #T30*11 リブート
りぷらい #T30*8 リプライ
りぷりけーしょん #T30*6 リプリケーション
りぷりけーと #T30*1 リプリケート
りぷる #T35*1 リプル
りへい #JNM 利平
りべつ #T30*1 離別
りべん #T35 利便
りぼぞーむ #T35*1 リボゾーム
りぼん #T35*2 りぼん リボン
りぽーた #T35*1 リポータ
りぽーと #T30*3 リポート
りまわり #T35 利回り
りみった #T35*1 リミッタ
りめん #T35*1 裏面
りもーと #T35*47 リモート
りゃく #T35*186 略 #S5*20 略 #KJ 掠
りゃくかい #T30*1 略解
りゃくぎ #T35 略儀
りゃくご #T35*1 略語
りゃくしき #T05*1 略式
りゃくしょう #T30*13 略称
りゃくじ #T35 略字
りゃくず #T35 略図
りゃくそう #T30 略装
りゃくだつ #T30*1 略奪
りゃくひつ #T30 略筆
りゃくれき #T35 略歴
りゃっかい #T35*1 略解
りゃっき #T30*1 略記
りゅう #T35*16 竜 #JNSUC*7 流 #T35*5 龍 #KJ*2 留 #KJ*1 柳 劉 硫 粒 隆 #T35 流 #KJ 溜 琉 瑠
りゅうい #T30*7 留意
りゅういき #T35 流域
りゅういち #JNM 隆一 竜一 龍一
りゅういん #T35 溜飲
りゅうおう #CN 竜王
りゅうか #T30 硫化
りゅうかい #T30 流会
りゅうかん #T35 流感
りゅうがく #T30*6 留学
りゅうがくせい #T35*2 留学生
りゅうがさき #CN 竜ヶ崎
りゅうがん #T35*1 竜顔
りゅうき #T30*1 隆起
りゅうぎ #T35*2 流儀
りゅうけつ #T30 流血
りゅうげん #T35 流言
りゅうげんひご #T35 流言飛語 流言蜚語
りゅうこう #T30*2 流行
りゅうご #JNM 隆吾 龍五
りゅうさん #T35 硫酸
りゅうざん #T30*1 流産
りゅうし #T35*3 粒子
りゅうしつ #T30 流失
りゅうしゅつ #T30*2 流出
りゅうじ #JNM 隆司 隆二
りゅうじん #JCN 龍神
りゅうすい #T35*4 流水
りゅうすけ #JNM 隆介
りゅうず #T35 竜頭
りゅうせい #T05*1 隆盛
りゅうそく #T35 流速
りゅうぞう #JNM 隆三 竜三
りゅうた #JNM 竜太
りゅうたい #T35*3 流体
りゅうち #T30 留置
りゅうちょう #T05*2 流暢
りゅうつう #T30*8 流通
りゅうとうだび #T15 竜頭蛇尾 龍頭蛇尾
りゅうどう #T30*4 流動
りゅうにゅう #T30*1 流入
りゅうにん #T30 留任
りゅうねん #T30*1 留年
りゅうは #T35 流派
りゅうひょう #T35*1 流氷
りゅうへい #JNM 龍平
りゅうほ #T30 留保
りゅうほく #CN 竜北
りゅうよう #T30*18 流用 #CN 竜洋
りゅうりょう #T35*2 流量
りゆう #T35*130 理由
りょ #KJ*8 慮 #KJ*1 呂 #KJ 侶 旅 虜
りょう #T35*201 量 #T30*18 領 #T35*14 良 #JS*12 両 #T35*12 寮 #T35*6 料 #KJ*3 梁 #T35*2 了 #KJ*2 陵 #T35*1 漁 #KJ*1 椋 凌 涼 稜 魎 #KJ 綾 菱 #JNM 亮 #KJ 僚 #PRE 両 #T35 両 #SUC 寮 料 #T35 猟 #KJ 療 瞭 糧 #JNM 良 #T30 諒 #KJ 遼 #SUC 量 #KJ 令 嶺 怜 霊
りょうあし #T35 両足
りょうあんじ #KK*1 竜安寺
りょういき #T35*223 領域
りょういち #JNM 亮一 良一
りょういん #T35 両院
りょうえん #T35 良縁
りょうかい #T30*36 了解 #T30 諒解 #T35 領海
りょうかみ #CN 両神
りょうかん #T35 量感
りょうが #T30*1 凌駕
りょうがえ #T30 両替
りょうがわ #T35*2 両側
りょうき #T35 猟奇
りょうきち #JNM 良吉
りょうきょく #T35*1 両極
りょうきん #T35*26 料金
りょうくう #T35 領空
りょうけ #T35 両家
りょうけん #T35 了見 両県 料簡 猟犬
りょうこ #JNM 量子
りょうこう #T05*4 良好
りょうこく #T35*1 両国
りょうごく #CN 両国
りょうさいけんぼ #T35*2 良妻賢母
りょうさく #T35 良策
りょうさん #T30*1 量産
りょうし #T35*6 量子 #T35*2 両氏 #T35 漁師 両市 猟師
りょうしき #T35*1 良識
りょうしつ #T05 良質
りょうしゃ #T35*18 両者 #T35*2 両社
りょうしゅ #T35 領主
りょうしゅう #T30*3 領収 #T35 領袖
りょうしょう #T30*4 了承 #T30 諒承
りょうしょく #T35 糧食
りょうしん #T35*5 両親 #T35*2 良心
りょうじ #T30 療治 #JNM 良治 良二 #T35 領事
りょうじつ #T35*4 両日
りょうじゅう #T35 猟銃
りょうじん #T35 良人
りょうせい #T35 両性 #T05 良性
りょうせん #T35*2 稜線
りょうぜん #CN 霊山
りょうそく #T35*1 両側
りょうぞう #JNM 良三
りょうた #JNM 良太
りょうたん #T35*4 両端
りょうだん #T30 両断
りょうち #T35*1 領地
りょうつ #JCN 両津
りょうて #T35*3 両手
りょうてい #T35*1 料亭
りょうてき #T05*1 量的
りょうとく #T35 両得
りょうど #T35 領土
りょうどう #T35*1 両道 糧道
りょうない #T35 領内
りょうなん #CN 綾南
りょうにん #T35*1 両人
りょうねん #T35 両年
りょうは #T35*2 両派
りょうはし #T35*1 両端
りょうひ #T35 良否
りょうぶん #T35*1 領分 #T30 両分
りょうへい #JNM 良平
りょうほう #T35*105 両方 #T35*1 療法
りょうぼ #T35 寮母 陵墓
りょうみ #T35 涼味
りょうめ #T35*1 両目
りょうめん #T35*2 両面
りょうゆう #T30 領有
りょうよう #T30*3 療養 #T30*2 両用 #T35 両様
りょうよく #T35 両翼
りょうり #T30*28 料理
りょうりつ #T30*1 両立
りょうろん #T30*1 両論
りょかく #T35 旅客
りょかん #T35*1 旅館
りょがい #T35 慮外
りょく #SUC*56 力 #KJ 力 緑
りょくしょく #T35*1 緑色
りょくち #T35 緑地
りょくちゃ #T35*4 緑茶
りょけん #T35 旅券
りょこう #T30*48 旅行
りょじょう #T35 旅情
りょだん #T35*1 旅団
りょっか #T30 緑化
りょてい #T35 旅程
りょひ #T35*3 旅費
りよう #T30*215 利用 #T35 理容
りようけん #T35*3 利用券
りようしゃ #T35 利用者
りらきしん #T35*1 リラキシン
りりーす #T30*203 リリース
りりく #T30*1 離陸
りりし #KY*1 凛々し
りりしゅう #KYU*1 凛々しゅう
りりつ #T35*1 利率
りれー #T30*11 リレー
りれーしょん #T35*16 リレーション
りれき #T35*45 履歴
りれきしょ #T35*5 履歴書
りろん #T35*24 理論
りん #KJ*5 林 #KJ*2 臨 #T35*1 燐 #KJ*1 隣 鱗 稟 #KJ 倫 #JS 厘 #SUC 林 #KJ 淋 琳 #JS 輪 #KJ 輪 麟 #T35 鈴
りんか #T35*1 リンカ #T35 隣家
りんかい #T35*1 臨海 #T30*1 臨界
りんかく #T35*3 輪郭
りんかん #T35 林間
りんきおうへん #T00 臨機応変
りんぎ #T30*2 稟議
りんぎょう #T35 林業
りんく #T30*190 リンク
りんくう #T35*1 臨空
りんくす #KK*6 リンクス
りんぐ #T35*20 リング
りんけーじ #T35*1 リンケージ
りんげつ #T35 臨月
りんげん #T35 綸言
りんこう #T30*13 輪講
りんご #T35*4 りんご
りんごく #T35*1 隣国
りんさん #T35 林産
りんしょう #T35 臨床 #T30 輪唱
りんしょく #T00 吝嗇
りんじ #T15*2 臨時
りんじゅう #T30 臨終
りんじょう #T30 臨場
りんじん #T35 隣人
りんず #T35 綸子
りんせき #T30 臨席
りんせつ #T30*10 隣接
りんてん #T30*2 輪転
りんと #F06 凛と
りんどう #T35*1 竜胆 #T35 林道
りんね #T30 輪廻
りんぱ #T35*1 リンパ
りんや #T35 林野
りんり #T35*1 倫理
りんりつ #T35 林立
る #KJ*72 琉 #KJ*16 留 #KJ*1 婁 #KJ 屡 瑠
るーじゅ #T35*1 ルージュ
るーず #T05 ルーズ
るーた #T35*9 ルータ
るーちん #T35*51 ルーチン
るーつ #T35 ルーツ
るーてぃんぐ #T30*38 ルーティング
るーと #T35*22 ルート #T35*6 √
るーぶる #T35*1 ルーブル
るーぷ #T30*34 ループ
るーぺ #T35 ルーペ
るーまにあ #CN ルーマニア
るーむ #T35*6 ルーム
るーる #T35*50 ルール
るい #T30*20 類 #KJ*1 涙 #JS 塁 #T35 塁 累 #JS 類 #SUC 類
るいけい #T30*4 累計
るいご #T35*1 類語
るいさん #T30*1 累算
るいしょ #T35 類書
るいしょう #T35*1 類症 #T30 類焼
るいしん #T35 塁審 #T30 累進
るいじ #T10*12 類似 #T35 類字
るいじょう #T30 累乗
るいすい #T30*3 類推
るいせき #T30*3 累積
るいせん #T35 涙腺
るいどう #T30 類同
るいべつ #T30*1 類別
るいるい #F02 累々
るえん #T35*1 流涎
るくす #T35 ルクス
るす #T10*5 留守
るすつ #CN 留寿都
るすばん #T30*6 留守番
るっくす #T35*1 ルックス
るてん #T30 流転
るのわーる #JN ルノワール
るび #T35*2 ルビ
るふ #T30*1 流布
るぽるたーじゅ #T35*1 ルポルタージュ
るみこ #JNM 留美子
るもい #CN 留萌
るり #T35*1 瑠璃
るろう #T30*1 流浪
れーがん #JN レーガン
れーざ #T35*13 レーザ
れーざー #T35*6 レーザー
れーす #T35*4 レース
れーだ #T35*1 レーダ
れーと #T35*3 レート
れーべる #T35*2 レーベル
れーる #T35*2 レール
れい #T35*255 例 #T35*19 霊 #T35*14 礼 #T35*4 レイ #KJ*3 冷 励 #SUC*2 令 #KJ*1 令 嶺 怜 玲 鈴 麗 黎 #KJ 蛎 砺 戻 伶 #SUC 例 #KJ 苓 隷 #T35 零 #KJ 齢
れいあうと #T30*24 レイアウト
れいえん #T35 霊園
れいか #T35 冷夏 零下
れいかい #T35*2 例会
れいかん #T35 霊感
れいがい #T15*32 例外 #T35 冷害
れいき #T35 冷気
れいきゃく #T30*2 冷却
れいきん #T35*1 礼金
れいぎ #T35*1 礼儀
れいぐう #T30 冷遇
れいけつ #T05 冷血
れいけつかん #T35*1 冷血漢
れいけん #T35*2 霊験
れいげつ #T35 例月
れいこ #JNM 令子 怜子 玲子 礼子
れいこう #T30*2 励行
れいこく #T05 冷酷
れいこん #T35 霊魂
れいさい #T05 零細
れいしょ #T35 隷書
れいしょう #T30*1 例証 #T30 冷笑
れいじ #T30*1 例示 #JNM 玲二 #T35 零時
れいじょう #T35*2 礼状 #T35 令嬢 令状 霊場
れいすい #T35 冷水
れいせい #T05*3 冷静
れいせん #T35 冷戦
れいぜん #T35 霊前
れいそく #T35 令息
れいぞうこ #T35*3 冷蔵庫
れいたん #T05 冷淡
れいだい #T35*21 例題
れいとう #T30*1 冷凍
れいど #T35 零度
れいにゅう #T30 戻入
れいねつ #T35 冷熱
れいねん #T35*2 例年
れいばい #T35 冷媒 霊媒
れいふう #T35 冷風
れいふく #T35*1 礼服
れいぶん #T35*10 例文
れいほう #T35 礼法 礼砲
れいほく #CN 苓北
れいぼう #T35*1 冷房
れいめい #T35 黎明
れいや #T35*23 レイヤ
れいやー #T35*3 レイヤー
れいれいし #KY 麗々し
れいれいしゅう #KYU 麗々しゅう
れき #T35*13 歴 #T35*1 暦 #KJ*1 礫
れきし #T35*17 歴史 #T30 轢死
れきぜん #F00*1 歴然
れきだい #T35*1 歴代
れきほう #T30 歴訪
れきれき #T35 歴々
れぎゅらー #T05*1 レギュラー
れくいえむ #T35*1 レクイエム
れくちゃー #T30*2 レクチャー
れこーだ #T35*1 レコーダ
れこーでぃんぐ #T30*2 レコーディング
れこーど #T35*67 レコード
れしーと #T35*1 レシート
れしーば #T35 レシーバ
れじ #T35*2 レジ
れじーむ #T35*19 レジーム
れじすた #T35*42 レジスタ
れじゃー #T35*1 レジャー
れじゅめ #T35*1 レジュメ
れす #SUC*32 レス
れすとらん #T35*6 レストラン
れすぽんす #T30*11 レスポンス
れすりんぐ #T35*1 レスリング
れせぷしょん #T35*1 レセプション
れぞりゅーしょん #T30*4 レゾリューション
れたー #T35*9 レター
れっか #T30*2 劣化 #T35 烈火
れっき #T30*2 列記
れっきと #T12 れっきと 歴と
れっきょ #T30*6 列挙
れっきょう #T35 列強
れっこく #T35 列国
れっしゃ #T35*11 列車
れっしょう #T35 裂傷
れっせい #T05 劣勢 劣性
れっせき #T30 列席
れっとう #T35*1 列島 #T05 劣等
れっど #T05*2 レッド
れっぱく #T35 烈帛
れつ #JS*255 列 #T35*148 列 #KJ*3 劣 #KJ*1 烈 #SUC 列 #KJ 裂
れつあく #T05*2 劣悪
れつでん #T35*1 列伝
れとると #T35*4 レトルト
ればー #T35*3 レバー
れびゅー #T30*6 レビュー
れふ #T35 レフ
れぶぇる #T35*1 レヴェル
れぶん #CN 礼文
れべる #T35*216 レベル
れぽーと #T30*31 レポート
れもねーど #T35*1 レモネード
れもん #T35*2 レモン
れん #T35*153 連 #KJ*1 鎌 廉 煉 蓮 錬 #KJ 恋 憐 漣 簾 練 聯
れんあい #T30*1 恋愛
れんか #T05*1 廉価
れんかん #T30*1 連関
れんが #T35*1 煉瓦
れんき #T30*1 連記
れんきゅう #T35*25 連休
れんきんじゅつ #T35*1 錬金術
れんぐす #T35*1 レングス
れんけい #T30*4 連係 #T30*2 連携
れんけつ #T10*13 連結
れんけつし #T35*1 連結子
れんげん #T35*1 連言
れんげんし #T35*6 連言肢
れんこ #T30*2 連呼
れんこう #T30 連行
れんこん #T35*1 蓮根
れんご #T35 連語
れんごう #T30*3 連合
れんさ #T30*2 連鎖
れんさい #T30*10 連載
れんさく #T30 連作
れんざ #T30 連座
れんざん #T35 連山
れんしゅう #T30*28 練習
れんしょ #T30 連署
れんしょう #T30*2 連勝
れんじ #T35*6 レンジ
れんじつ #T35 連日
れんじゅ #T35 連珠
れんせい #T30 連成 錬成
れんせつ #T30*2 連接
れんせつし #T35*1 連接子
れんそう #T30*6 連想
れんぞく #T10*32 連続
れんたい #T35*8 連体 #T30*3 連帯 #T35*1 連隊
れんたかー #T35 レンタカー
れんたつ #T30*1 練達
れんたる #T30*3 レンタル
れんだ #T30*3 連打
れんだく #T30 連濁
れんだりんぐ #T30*8 レンダリング
れんだん #T30 連弾
れんちゅう #T35*7 連中
れんとう #T30 連騰
れんとげん #T35 レントゲン
れんどう #T30*5 連動
れんねん #T35 連年
れんぱ #T30 連覇 連破
れんぱい #T30*1 連敗
れんぱつ #T30*2 連発
れんびん #T35 憐憫
れんぶんせつ #T35 連文節
れんぽう #T35*1 連邦 #T35 連峰
れんま #T30 錬磨
れんめい #T35*5 連名 #T35*2 連盟
れんめん #F03 連綿
れんや #T35*1 連夜
れんよう #T30*6 連用
れんようけい #T35 連用形
れんらく #T30*255 連絡
れんりつ #T30*2 連立
ろ #SUC*63 路 #T35*3 炉 #KJ*2 呂 露 #KJ*1 魯 路 #KJ 鷺 櫨 蕗 #T35 櫓 #KJ 賂
ろー #PRE*10 ロー #T35*3 Ρ #T35*1 ρ
ろーかる #T05*115 ローカル
ろーず #T35*1 ローズ
ろーど #T30*79 ロード
ろーま #CN ローマ
ろーまじ #T35*180 ローマ字
ろーれしあ #CN ローレシア
ろーれべる #T05*1 ローレベル
ろーん #T35*2 ローン
ろあく #T30 露悪
ろう #KJ*7 郎 #T30*6 労 #SUC*3 楼 #T35*2 朗 #KJ*1 浪 牢 #T35*1 老 #KJ*1 蝋 籠 #KJ 滝 糧 婁 廊 #S5 弄 #T30 弄 #KJ 楼 榔 漏 狼 篭 聾
ろうあ #T35 ろうあ 聾唖
ろうえい #T30*2 漏洩
ろうか #T35*8 廊下 #T30*1 老化
ろうかい #T05 老獪
ろうがん #T35 老眼
ろうきほう #KK 労基法
ろうきゅう #T30*1 老朽
ろうきょく #T35*1 浪曲
ろうく #T30 労苦
ろうご #T35 老後
ろうごく #T35*1 牢獄
ろうさい #T35 労災 老妻
ろうさく #T30*2 労作
ろうし #T35 労使 浪士
ろうしゅつ #T30 漏出
ろうしょう #T35 労相
ろうじゃく #T05 老弱
ろうじゅう #T35 老中
ろうじゅく #T30 老熟
ろうじょう #T35 老嬢 #T30 籠城
ろうじん #T35*3 老人
ろうすい #T30 漏水 老衰
ろうせき #T35 蝋石
ろうぜき #T35 狼籍
ろうそ #T35 労組
ろうそく #T35 ロウソク 蝋燭
ろうたい #T35 老体
ろうちん #T35 労賃
ろうでん #T30*1 漏電
ろうと #T35 漏斗
ろうとう #T35 郎等
ろうどう #T30*4 労働
ろうどうしょう #KK 労働省
ろうどく #T30*3 朗読
ろうにゃく #T35 老若
ろうにん #T30*3 浪人
ろうはい #T30 老廃
ろうば #T35*1 老婆
ろうばい #T30 狼狽
ろうひ #T30*4 浪費
ろうふ #T35 老夫 老父
ろうほう #T35 朗報
ろうぼ #T35 老母
ろうぼく #T35 老木
ろうむ #T35 労務
ろうりょく #T35*3 労力
ろうれい #T05 老齢
ろうれつ #T05 陋劣
ろうれん #T05 老練
ろかた #T35 路肩
ろぎん #T35 路銀
ろく #NN*25 六 #SUC*6 録 #T05*5 ろく #T30*1 録 #KJ 漉 鹿 緑 麓 禄 肋
ろくおん #T30*14 録音
ろくが #T30*6 録画
ろくがどり #T30*1 録画撮り
ろくごう #CN 六郷
ろくすっぽ #F14 ろくすっぽ
ろくせい #CN 鹿西
ろくのへ #CN 六戸
ろくぶんぎ #T35 六分儀
ろくぼく #T35 肋木
ろくまく #T35 肋膜
ろくよう #T35 六曜
ろくろ #T35*1 ろくろ
ろくろう #JNM 六郎
ろくろく #F06 ろくろく
ろぐ #T30*25 ログ
ろぐいん #T30*116 ログイン
ろけーしょん #T35 ロケーション
ろけーた #T35*1 ロケータ
ろけーる #T35 ロケール
ろけっと #T35*3 ロケット
ろけん #T30*1 露見
ろこう #T30 露光
ろこつ #T05*1 露骨
ろご #T35*4 ロゴ
ろさんぜるす #CN ロサンゼルス
ろし #T35*1 ろ紙 濾紙
ろしあ #CN ロシア
ろしあご #KK*2 ロシア語
ろしゅつ #T30 露出
ろしん #T35*1 炉心
ろじ #T35*2 路地 #T35 露地
ろじっく #T35*14 ロジック
ろじょう #T35 路上
ろせん #T35*3 路線
ろだん #JN ロダン
ろちょう #T35 路長 顱頂
ろっ #NN*3 六
ろっかー #T35*8 ロッカー
ろっきーど #KK ロッキード
ろっく #T30*86 ロック
ろっこう #CN 六甲
ろっこうさん #CN 六甲山
ろっこつ #T35 肋骨
ろって #KK ロッテ
ろっぽう #T35*4 六法
ろっぽんぎ #JCN 六本木
ろてい #T30 露呈
ろてん #T35*1 露天 #T35 露店 露点
ろばん #T35 路盤
ろびんそん #JN ロビンソン
ろぼ #SUC*3 ロボ
ろぼっと #T35*4 ロボット
ろまんてぃしずむ #T35*1 ロマンティシズム
ろむ #T35*1 ロム
ろめん #T35*1 路面
ろりこん #T15*10 ロリコン
ろん #T35*52 論 #ZX*52 論 #SUC*5 論
ろんがい #T05*4 論外
ろんきゅう #T30 論及 論究
ろんきょ #T35 論拠
ろんぎ #T30*2 論議
ろんぐ #T05*5 ロング
ろんこく #T30 論告
ろんご #T35 論語
ろんし #T35 論旨
ろんしゃ #T35 論者
ろんしょう #T30 論証
ろんじゃ #T35 論者
ろんじゅつ #T30 論述
ろんそう #T30*1 論争
ろんだい #T35 論題
ろんちょう #T35 論調
ろんてん #T35*1 論点
ろんどん #CN ロンドン 倫敦
ろんぴょう #T30 論評
ろんぶん #T35*91 論文
ろんぽう #T35*1 論法
ろんり #T35*123 論理
わ #S5*140 和 #T30*140 和 #T35*17 輪 #R5r*14 割 #K5*11 湧 #KJ*2 我 #KJ*1 吾 #K5*1 沸 涌 #KJ*1 倭 話 #JS 羽 #T35 環 #JS 把 #SUC 話
わーく #T35*144 ワーク
わーど #JS*64 ワード #T35*16 ワード
わーどぷろせっさ #T35*2 ワードプロセッサ
わーぷ #T30*3 ワープ
わーぷろ #T35*29 ワープロ
わーるど #T35*1 ワールド
わい #KJ*2 隈 #KJ 歪 賄
わいあ #T35*2 ワイア
わいきょく #T30 歪曲
わいしゃつ #T35 ワイシャツ
わいしょう #T05 矮小
わいせい #T35 矮星
わいせつ #T05 猥褻
わいだん #T30 猥談
わいど #T05*69 ワイド
わいぱー #T35*3 ワイパー
わいや #T35*1 ワイヤ
わいやー #T35*2 ワイヤー
わいやーど #T35*1 ワイヤード
わいるど #T05*2 ワイルド
わいろ #T35 賄賂
わいん #T35*6 ワイン
わか #R5r*255 分か #R5r*199 わか #KY*34 若 #R5r*31 判 #R5r*15 解 #R5r*10 分 #S5*1 沸か #T35*1 和歌 #PRE 若 #T35 若
わかい #T30*4 和解 #JNS 若井
わかがえ #R5r*1 若返
わかくさ #CN 若草 #T35 若草
わかくわ #JNS 若桑
わかこ #JNM 和佳子 和歌子
わかさ #JCN 若桜
わかち #T35*2 わかち
わかちがき #T35*1 わかち書き
わかつき #JNS 若月
わかて #T35*4 若手
わかば #T35*2 若葉 #JCN 若葉
わかばやし #JCN 若林
わかまつ #JCN 若松 #T35 若松
わかみ #CN 若美
わかみや #JCN 若宮
わかめ #T35*1 若布
わかもの #T35*6 若者
わかやなぎ #JCN 若柳
わかやま #JNS 若山 #JCN 和歌山
わかれ #KSr*22 別れ #KSr*10 分かれ
わかわかし #KY*1 若々し
わかわかしゅう #KYU*1 若々しゅう
わかん #T30 和姦
わが #RT*5 我が #T35*4 我 #RT*1 わが #JCN 和賀
わがくに #T35*2 わが国
わがし #T35*1 和菓子
わがはい #T35 我輩
わがまま #T00*5 わがまま 我儘 #T00*2 我がまま #T00*1 我侭
わがや #T35*3 我が家
わき #T35*5 脇 #JCN 脇
わきあいあい #T00*2 和気合い合い
わきげ #T35*1 脇毛
わきた #T5r 沸き立
わきのさわ #CN 脇野沢
わきばら #T35 脇腹
わきまえ #KSr 弁え
わきみ #T30 わき見
わきみち #T35 脇道
わぎ #T30 和議 #CN 和木
わぎゅう #T35 和牛
わぎり #T30*1 輪切り
わく #T35*47 枠 #KJ 惑
わくがい #T35*1 枠外
わくぐみ #T35*10 枠組
わくせい #T35*4 惑星
わくちん #T35 ワクチン
わくない #T35*1 枠内
わくや #JCN 涌谷
わけ #T35*234 わけ #KSr*160 分け #T35*91 訳 #KSr*23 わけ #JCN 和気
わけい #R5 分け入
わけまえ #T35 分け前
わけめ #T35 分け目
わこーる #KK ワコール
わこう #KYU*2 若う #JCN 和光
わこうど #T35*1 若人
わごう #T30 和合
わごん #T35*1 ワゴン
わさい #T35*2 和裁
わさび #T35 山葵
わざ #T35*31 技 #T35*2 わざ #T35*1 業 #KJ 術
わざと #T06*3 わざと
わざわい #T35 禍 #T30 災い
わざわざ #F14*18 わざわざ
わし #T35*3 鷲 #T35 和紙
わしお #JNS 鷲尾
わしき #T35 和式
わしつ #T35*2 和室
わしま #CN 和島
わしみや #CN 鷲宮
わしゃ #T35*1 話者
わしょく #T35*3 和食
わしんとん #CN ワシントン
わじき #CN 鷲敷
わじま #JCN 輪島 #JNS 和島
わすれ #KSr*188 忘れ
わすれさ #R5r*1 忘れ去
わずか #T05*3 わずか #T05*2 僅か
わずら #W5r*1 煩 #W5r 患
わずらわ #S5*5 煩わ
わずらわし #KY*5 煩わし
わずらわしゅう #KYU*1 煩わしゅう
わせ #T35*4 早稲 #T35*1 早生
わせい #T35 和声
わせだ #JCN 早稲田
わた #S5r*255 渡 #R5r*233 渡 #T35*38 棉 #T35*12 綿 #R5r*5 わた #R5*1 航 #R5 亙
わたくし #T35*55 私
わたくしごと #T35*2 私事
わたくしたち #T35 私達
わたくしりつ #T35 私立
わたし #T35*255 私 #T35*76 わたし #T35 渡し
わたしたち #T35*7 私達
わたしども #T35*1 私供
わたしぶね #T35 渡し船
わたなべ #JNS 渡部 渡辺
わたぬき #JNS 綿貫
わたべ #JNS 渡部
わたらい #JCN 度会
わたり #T35 渡り #JCN 亘理
わたりどり #T35 渡り鳥
わたる #JNM 渉 #KJ 亙 #JNM 亘
わだ #JCN 和田
わだい #T35*52 話題
わだかま #R5r*1 わだかま
わだち #T35 轍
わだやま #CN 和田山
わち #JCN 和知
わっか #T35 輪っか
わっかない #JCN 稚内
わっさむ #CN 和寒
わっしょい #CJ わっしょい
わっと #JS*1 ワット #T35*1 ワット
わづか #CN 和束
わとじ #T35 和綴じ
わとそん #JN ワトソン
わどまり #CN 和泊
わな #T35*9 罠
わに #T35 鰐
わのうち #CN 輪之内
わび #KS 詫び
わびじょう #T35 詫状
わふう #T35*1 和風
わふく #T35*1 和服
わぶん #T35*1 和文
わへい #T30 和平
わほう #T35*1 話法
わぼく #T30 和睦
わやく #T30*1 和訳
わよう #T35 和洋
わら #W5r*38 笑 #T35*2 稿 #CN 和良 #T35 藁
わらび #T35*3 蕨 #CN 蕨
わらべ #T35 童
わり #JS*79 割 #T35*62 割 #SUC 割
わりあい #T35*8 割合
わりあて #KS*149 割り当て #T30*16 割当
わりいん #T35 割印
わりかん #T35*1 わりかん
わりき #R5r*4 割り切
わりこ #M5r*26 割り込
わりざん #T35 割算
わりだか #T05*1 割高
わりつけ #KSr*10 割り付け
わりばし #T35*1 割箸
わりび #K5r*16 割り引
わりびき #T35*4 割引
わりふ #R5r*6 割り振 #R5r*2 割りふ
わりまし #T35*1 割増し
わりもど #S5r 割り戻
わりもどし #T35 割戻し
わりやす #T05*2 割安
わる #KY*209 悪 #T35*87 悪 #T35*1 ÷ #KY*1 わる
わるあがき #T30 悪あがき
わるう #KYU*3 悪う #KYU*1 わるう
わるぎ #T35*1 悪気
わるくち #T35*1 悪口
わるだくみ #T30 悪巧み
わるぢえ #T35 悪知恵
わるびれ #KS 悪びれ
わるもの #T35*2 悪者
われ #KSr*5 割れ #T35*2 我
われめ #T35 割れ目
われら #T35*1 われら #T35 我ら
われわれ #T35*48 我々
わん #T35*16 椀 #CNSUC1*3 湾 #T35*3 湾 #KJ 碗 腕
わんがん #T35 湾岸
わんきょく #T30 湾曲
わんさか #F06 わんさか
わんさと #F14 わんさと
わんしょう #T35 腕章
わんたん #T35*1 ワンタン
わんない #T35 湾内
わんぱく #T05 腕白
わんまん #T05 ワンマン
わんりょく #T35*1 腕力
